{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
unit AdifEng;

interface

var
  AdifStatus : String;
  AdifRecords : longint;

function ConvertToAdif(InFile, OutFile : String) : Boolean;


implementation

uses RecTypes;
type

  Date = word;
  Time = longint;
  DayType = (Sunday, Monday, Tuesday, Wednesday, Thursday, Friday, Saturday);
  DateString = string[40];

  VersionType    = (V1_1_6, V2_0_0);

  ModeType       = (RTTYMODE, ASCII, AMTOR, Packet, Other);

  BandType       = (B80, B40, B20, B15, B10, TOT);

  Log_Rec        = record
                     ToolBoxNumber  : Integer;
                     VersionControl : VersionType;
                     Contest        : ContestType;
                     Date           : Date;
                     Ts             : Time;
                     Band           : BandType;
                     Mode           : ModeType;
                     Call           : String[12];
                     His_Country    : String[5];
                     His_RST        : Byte;
                     His_Qth        : String[3]; {grp# in waedc}
                     His_Zone       : Byte; {age in jarts, qtcs in waedc}
                     His_Serno      : Word;
                     His_Time       : Time;
                     My_Rst         : Byte;
                     My_Qth         : String[3];
                     My_Zone        : Byte;
                     My_Serno       : Word;
                     My_Time        : Time;
                     Point          : Byte;
                     New_Zone       : Boolean;
                     New_Country    : Boolean;
                     New_Qth        : Boolean; {in wae, F=not sent, T=sent}
                     New_Continent  : Boolean;
                     Dupe           : Boolean;
                     Comment        : String[25];
                   end;





var
  Inf : file of log_rec;
  RecordNumber : LongInt;
  OutF : text;
  Scrap : log_rec;

const
  DateLen = 40;
  First2Months = 58;         {1900 was not a leap year}
  FirstDayOfWeek = Monday;   {01/01/1900 was a Monday}
  MinYear  = 1900;
  MaxYear  = 2078;
  MinDate  = $0000;        {= 01/01/1900}
  MaxDate  = $FF62;        {= 12/31/2078}
  Date1900 = $0000;        {= 01/01/1900}
  Date1980 = $7223;        {= 01/01/1980}
  Date2000 = $8EAC;        {= 01/01/2000}
  BadDate  = $FFFF;
  MonthOnly = 'm';           {these are for date/time pictures, and allow}
  DayOnly = 'd';             {  numbers and spaces only}
  YearOnly = 'y';
  MonthOnlyU = 'M';
  DayOnlyU = 'D';
  DateSlash = '/';
  SlashChar : Char = '/';
  NameOnly = 'n';
  NameOnlyU = 'N';
  WeekDayOnly = 'w';
  WeekDayOnlyU = 'W';
  Threshold2000 : Integer = 1900;
    MinTime = 0;               {= 00:00:00 am}
  MaxTime = 86399;           {= 23:59:59 pm}
  BadTime = $FFFFFFFF;

  SecondsInDay = 86400;      {number of seconds in a day}
  SecondsInHour = 3600;      {number of seconds in an hour}
  SecondsInMinute = 60;      {number of seconds in a minute}
  HoursInDay = 24;           {number of hours in a day}
  MinutesInHour = 60;        {number of minutes in an hour}
  TimeOnly = 't';            {generates 'p', 'P', 'a', or 'A'}
  EmOnly = 'e';              {optional--generates 'm' or 'M'}
  TimeColon = ':';
  ColonChar : Char = ':';
  UpCaseTime : Boolean = False; {if true, 't' and 'e' force upper case}
  HourOnly = 'h';
  MinOnly = 'm';
  SecOnly = 's';
  HourOnlyU = 'H';
  MinOnlyU = 'M';
  SecOnlyU = 'S';


const
  MonthString : array[1..12] of string[12] = (
    'January', 'February', 'March', 'April', 'May', 'June', 'July',
    'August', 'September', 'October', 'November', 'December');
  DayString : array[DayType] of string[15] = (
    'Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday');


  Modes          : array[ModeType] of String[6] = (
             'RTTY', 'ASCII', 'AMTOR', 'PACKET', 'OTHER');

  BandS          : array[BandType] of String[3] = (
             '80', '40', '20', '15', '10', 'Tot');

  function Long2Str(L : LongInt) : string;
    {-Convert a long/word/integer/byte/shortint to a string}
  var
    S : string;
  begin
    Str(L, S);
    Long2Str := S;
  end;
  function Trim(S : string) : string;
    {-Return a string with leading and trailing white space removed}
  var
    I : Word;
    SLen : Byte absolute S;
  begin
    while (SLen > 0) and (S[SLen] <= ' ') do
      Dec(SLen);

    I := 1;
    while (I <= SLen) and (S[I] <= ' ') do
      Inc(I);
    Dec(I);
    if I > 0 then
      Delete(S, 1, I);

    Trim := S;
  end;

  procedure MergeIntoPicture(var Picture : DateString; Ch : Char; I : Integer);
    {-Merge I into location in Picture indicated by format character Ch}
  var
    Tmp : DateString;
    TLen : Byte absolute Tmp;
    PLen : Byte absolute Picture;
    J, K : Integer;
    UCh, CPJ, CTI : Char;
  begin
    {find the start of the subfield}
    J := Pos(Ch, Picture);
    UCh := Upcase(Ch);
    if J = 0 then begin
      J := Pos(UCh, Picture);
      if J = 0 then
        Exit;
    end;

    {find the end of the subfield}
    K := J;
    while (J < PLen) and (Upcase(Picture[J+1]) = UCh) do
      Inc(J);

    if (UCh = WeekDayOnlyU) or (UCh = NameOnlyU) then begin
      if UCh = WeekDayOnlyU then
        case I of
          Ord(Sunday)..Ord(Saturday) :
            Tmp := DayString[DayType(I)];
          else
            Tmp := '';
        end
      else
        case I of
          1..12 :
            Tmp := MonthString[I];
          else
            Tmp := '';
        end;
      K := Succ(J-K);
      if K > TLen then
        FillChar(Tmp[TLen+1], K-TLen, ' ');
      TLen := K;
    end
    else
      {convert I to a string}
      Str(I:DateLen, Tmp);

    {now merge}
    I := TLen;
    CPJ := Picture[J];
    while (Upcase(CPJ) = UCh) and (J > 0) and (I > 0) do begin
      CTI := Tmp[I];
      if (UCh = NameOnlyU) or (UCh = WeekDayOnlyU) then begin
        case CPJ of
          NameOnlyU, WeekDayOnlyU :
            CTI := Upcase(CTI);
        end;
      end
      {change spaces to 0's if desired}
      else if (CPJ >= 'a') and (CTI = ' ') then
        CTI := '0';
      Picture[J] := CTI;
      Dec(J);
      Dec(I);
      CPJ := Picture[J];
    end;
  end;



    function DayOfWeek(Julian : Date) : DayType;
    {-Return the day of the week for the date. Returns DayType(7) if Julian =
      BadDate.}
  var
    B : Byte;
  begin
    if Julian = BadDate then begin
      B := 7;
      DayOfWeek := DayType(B);
    end
    else
      DayOfWeek := DayType( (Julian+Ord(FirstDayOfWeek)) mod 7 );
  end;

  function IsLeapYear(Year : Integer) : Boolean;
    {-Return True if Year is a leap year}
  begin
    IsLeapYear := (Year mod 4 = 0) and (Year mod 4000 <> 0) and
      ((Year mod 100 <> 0) or (Year mod 400 = 0));
  end;



  function DaysInMonth(Month, Year : Integer) : Integer;
    {-Return the number of days in the specified month of a given year}
  begin
    if Word(Year) < 100 then begin
      Inc(Year, 1900);
      if Year < Threshold2000 then
        Inc(Year, 100);
    end;

    case Month of
      1, 3, 5, 7, 8, 10, 12 :
        DaysInMonth := 31;
      4, 6, 9, 11 :
        DaysInMonth := 30;
      2 :
        DaysInMonth := 28+Ord(IsLeapYear(Year));
    else
      DaysInMonth := 0;
    end;
  end;

  function ValidDate(Day, Month, Year : Integer) : Boolean;
  {-Verify that day, month, year is a valid date}
  begin
    if Word(Year) < 100 then begin
      Inc(Year, 1900);
      if Year < Threshold2000 then
        Inc(Year, 100);
    end;

    if (Day < 1) or (Year < MinYear) or (Year > MaxYear) then
      ValidDate := False
    else case Month of
      1..12 :
        ValidDate := Day <= DaysInMonth(Month, Year);
    else
      ValidDate := False;
    end
  end;

  procedure SubstChar(var Picture : DateString; OldCh, NewCh : Char);
    {-Replace all instances of OldCh in Picture with NewCh}
  var
    I : Byte;
    UpCh : Char;
  begin
    UpCh := Upcase(OldCh);
    if (Pos(OldCh, Picture) <> 0) or (Pos(UpCh, Picture) <> 0) then
      for I := 1 to Length(Picture) do
        if Upcase(Picture[I]) = UpCh then
          Picture[I] := NewCh;
  end;



  function DMYtoDate(Day, Month, Year : Integer) : Date;
    {-Convert from day, month, year to a julian date}
  begin
    if Word(Year) < 100 then begin
      Inc(Year, 1900);
      if Year < Threshold2000 then
        Inc(Year, 100);
    end;

    if not ValidDate(Day, Month, Year) then
      DMYtoDate := BadDate
    else if (Year = MinYear) and (Month < 3) then
      if Month = 1 then
        DMYtoDate := Pred(Day)
      else
        DMYtoDate := Day+30
    else begin
      if Month > 2 then
        Dec(Month, 3)
      else begin
        Inc(Month, 9);
        Dec(Year);
      end;
      Dec(Year, MinYear);
      DMYtoDate :=
        {$IFDEF FourByteDates}
          ((LongInt(Year div 100)*146097) div 4)+
          ((LongInt(Year mod 100)*1461) div 4)+
        {$ELSE}
          ((LongInt(Year)*1461) div 4)+
        {$ENDIF}
          (((153*Month)+2) div 5)+Day+First2Months;
    end;
  end;

  function DayOfWeekDMY(Day, Month, Year : Integer) : DayType;
    {-Return the day of the week for the day, month, year}
  begin
    DayOfWeekDMY := DayOfWeek( DMYtoDate(Day, Month, Year) );
  end;

  function DMYtoDateString(Picture : DateString; Day, Month, Year : Integer) : DateString;
    {-Merge the month, day, and year into the picture}
  var
    DOW : Integer;
  begin
    if Word(Year) < 100 then begin
      Inc(Year, 1900);
      if Year < Threshold2000 then
        Inc(Year, 100);
    end;

    DOW := Integer( DayOfWeekDMY(Day, Month, Year) );
    MergeIntoPicture(Picture, MonthOnly,   Month);
    MergeIntoPicture(Picture, DayOnly,     Day);
    MergeIntoPicture(Picture, YearOnly,    Year);
    MergeIntoPicture(Picture, NameOnly,    Month);
    MergeIntoPicture(Picture, WeekDayOnly, DOW);

    {map slashes}
    SubstChar(Picture, DateSlash, SlashChar);

    DMYtoDateString := Picture;
  end;
  procedure DateToDMY(Julian : Date; var Day, Month, Year : Integer);
    {-Convert from a julian date to month, day, year}
  var
    I, J : LongInt;
  begin
    if Julian = BadDate then begin
      Day := 0;
      Month := 0;
      Year := 0;
    end
    else if Julian <= First2Months then begin
      Year := MinYear;
      if Julian <= 30 then begin
        Month := 1;
        Day := Succ(Julian);
      end
      else begin
        Month := 2;
        Day := Julian-30;
      end;
    end
    else begin
      I := (4*LongInt(Julian-First2Months))-1;
      {$IFDEF FourByteDates}
      J := (4*((I mod 146097) div 4))+3;
      Year := (100*(I div 146097))+(J div 1461);
      I := (5*(((J mod 1461)+4) div 4))-3;
      {$ELSE}
      Year := I div 1461;
      I := (5*((I mod 1461) div 4)) + 2;
      {$ENDIF}
      Month := I div 153;
      Day := ((I mod 153)+5) div 5;
      if Month < 10 then
        Inc(Month, 3)
      else begin
        Dec(Month, 9);
        Inc(Year);
      end;
      Inc(Year, MinYear);
    end;
  end;

  function DateToDateString(Picture : DateString; Julian : Date) : DateString;
    {-Convert Julian to a string of the form indicated by Picture}
  var
    Month, Day, Year : Integer;
  begin
    if Julian = BadDate then begin
      {map picture characters to spaces}
      SubstChar(Picture, MonthOnly,   ' ');
      SubstChar(Picture, NameOnly,    ' ');
      SubstChar(Picture, DayOnly,     ' ');
      SubstChar(Picture, YearOnly,    ' ');
      SubstChar(Picture, WeekDayOnly, ' ');

      {map slashes}
      SubstChar(Picture, DateSlash, SlashChar);

      DateToDateString := Picture;
    end
    else begin
      {convert Julian to day/month/year}
      DateToDMY(Julian, Day, Month, Year);

      {merge the month, day, and year into the picture}
      DateToDateString := DMYtoDateString(Picture, Day, Month, Year);
    end;
  end;

    procedure TimeToHMS(T : Time; var Hours, Minutes, Seconds : Byte);
    {-Convert a Time variable to Hours, Minutes, Seconds}
  begin
    if T = BadTime then begin
      Hours := 0;
      Minutes := 0;
      Seconds := 0;
    end
    else begin
      Hours := T div SecondsInHour;
      Dec(T, LongInt(Hours)*SecondsInHour);
      Minutes := T div SecondsInMinute;
      Dec(T, LongInt(Minutes)*SecondsInMinute);
      Seconds := T;
    end;
  end;



  function TimeToTimeString(Picture : DateString; T : Time) : DateString;
    {-Convert T to a string of the form indicated by Picture}
  const
    AmPm : array[Boolean] of Char = ('a', 'p');
  var
    Hours, Minutes, Seconds : Byte;
    TPos, EPos : Byte;
    Ch1, Ch2 : Char;
  begin
    {merge the hours, minutes, and seconds into the picture}
    TimeToHMS(T, Hours, Minutes, Seconds);

    {check for TimeOnly}
    TPos := Pos(TimeOnly, Picture);
    if TPos <> 0 then begin
      Ch1 := AmPm[Hours >= 12];
      Ch2 := 'm';
      if UpCaseTime then begin
        Ch1 := Upcase(Ch1);
        Ch2 := 'M';
      end;

      {plug in the 'p' or 'a'}
      Picture[TPos] := Ch1;

      {get position of EmOnly}
      EPos := Pos(EmOnly, Picture);

      {adjust hours}
      case Hours of
        0 : Hours := 12;
        13..23 : Dec(Hours, 12);
      end;
    end
    else
      EPos := 0;

    if T = BadTime then begin
      {map picture characters to spaces}
      SubstChar(Picture, HourOnly, ' ');
      SubstChar(Picture, MinOnly, ' ');
      SubstChar(Picture, SecOnly, ' ');
    end
    else begin
      {merge the numbers into the picture}
      MergeIntoPicture(Picture, HourOnly, Hours);
      MergeIntoPicture(Picture, MinOnly, Minutes);
      MergeIntoPicture(Picture, SecOnly, Seconds);
    end;

    {map colons}
    SubstChar(Picture, TimeColon, ColonChar);

    {plug in the em now--if we do it earlier it looks like MinOnly}
    if EPos <> 0 then
      Picture[EPos] := Ch2;

    TimeToTimeString := Picture;
  end;

  function WritelnF(s : String) : Boolean;
  begin
    WritelnF := True;
  {$I-}
    WriteLn(OutF, s);
  {$I+}
    if IoResult > 0 then begin
      AdifStatus := 'Error writing to output file';
      WritelnF := False;
    end;
  end;

  function WriteAdifHeader : Boolean;
  begin
    if not WritelnF('ADIF Export for RTTY by WF1B v 1.2 Copyright 1996-2000 by Wyvern Technology, Inc.') then begin
      WriteAdifHeader := False;
      Exit;
    end;
    if not WritelnF('') then begin
      WriteAdifHeader := False;
      Exit;
    end;
    if not WritelnF('Wyvern Technology, Inc.') then begin
      WriteAdifHeader := False;
      Exit;
    end;
    if not WritelnF('35 Colvintown Road') then begin
      WriteAdifHeader := False;
      Exit;
    end;
    if not WritelnF('Coventry, RI 02816-8509') then begin
      WriteAdifHeader := False;
      Exit;
    end;
    if not WritelnF('') then begin
      WriteAdifHeader := False;
      Exit;
    end;
    if not WritelnF('401-823-7889 office') then begin
      WriteAdifHeader := False;
      Exit;
    end;
    if not WritelnF('401-822-0554 fax') then begin
      WriteAdifHeader := False;
      Exit;
    end;
    if not WritelnF('') then begin
      WriteAdifHeader := False;
      Exit;
    end;
    if not WritelnF('wf1b@wf1b.com') then begin
      WriteAdifHeader := False;
      Exit;
    end;
    if not WritelnF('http://www.wf1b.com') then begin
      WriteAdifHeader := False;
      Exit;
    end;
    if not WritelnF('') then begin
      WriteAdifHeader := False;
      Exit;
    end;
    if not WritelnF('<eoh>') then begin
      WriteAdifHeader := False;
      Exit;
    end;
    WriteAdifHeader := True;
  end;

  function WriteAdifItem(tag : String; len : Byte; desc : String; value : String) : Boolean;
  begin
    WriteAdifItem := True;
    if (tag <> 'eor') and (Len = 0) then Exit;
  {$I-}
    if tag = 'eor' then
      WriteLn(OutF, '<eor>')
    else
      if desc = '' then
        Write(OutF, '<'+tag+':'+long2str(len)+'>'+value+' ')
    else
      Write(OutF, '<'+tag+':'+long2str(len)+':'+trim(desc)+'>'+value+' ');
  {$I+}
    if IoResult > 0 then begin
      AdifStatus := ' Error writing to Output file ';
      WriteAdifItem := False;
    end;
  end;


  function WriteAdifRecord(A : log_rec) : Boolean;
  begin
    WriteAdifRecord := True;
    if not WriteAdifItem('call', Length(A.call), '', A.call) then begin
      WriteAdifRecord := False;
      Exit;
    end;
    if not WriteAdifItem('qso_date', 8, ' d', DateToDateString('yyyymmdd', A.Date)) then begin
      WriteAdifRecord := False;
      Exit;
    end;
    if not WriteAdifItem('time_on', 6, '', TimeToTimeString('hhmmss', A.Ts)) then begin
      WriteAdifRecord := False;
      Exit;
    end;
    if not WriteAdifItem('time_off', 6, '', TimeToTimeString('hhmmss', A.Ts)) then begin
      WriteAdifRecord := False;
      Exit;
    end;
    if not WriteAdifItem('rst_sent', 3, '', long2str(A.his_rst)+'9') then begin
      WriteAdifRecord := False;
      Exit;
    end;
    if not WriteAdifItem('rst_rcvd', 3, '', long2str(A.my_rst)+'9') then begin
      WriteAdifRecord := False;
      Exit;
    end;
    if not WriteAdifItem('mode', Length(Modes[A.Mode]), '', Modes[A.Mode]) then begin
      WriteAdifRecord := False;
      Exit;
    end;
    if not WriteAdifItem('band', Length(trim(BandS[A.Band])+'M'), '', trim(BandS[A.Band])+'M') then begin
      WriteAdifRecord := False;
      Exit;
    end;
    if not WriteAdifItem('contest_id', Length(trim(ContestName[A.Contest])), '', trim(ContestName[A.Contest])) then begin
      WriteAdifRecord := False;
      Exit;
    end;
    if not WriteAdifItem('comment', Length(Trim(A.comment)), '', Trim(A.comment)) then begin
      WriteAdifRecord := False;
      Exit;
    end;
    { contest specific data here - near end of record }
    case A.Contest of
      ARRL, CQWW : if (A.his_Qth <> '') and (A.His_Country = 'W') then
                     if not WriteAdifItem('state', 2, '', A.His_Qth) then begin
                       WriteAdifRecord := False;
                       Exit;
                     end;
    end;
    if not WriteAdifItem('eor', 0, '', '') then begin
      WriteAdifRecord := False;
      Exit;
    end;

  end;

  function ConvertToAdif(InFile, OutFile : String) : Boolean;
  var
    NumRead : integer;
    NumRecs : integer;
    i : integer;
    SizeOfScrap : integer;
    l : integer;
    hioresult : integer;
  begin
    Assign(Inf, InFile);
    if IoResult <> 0 then
    begin
      ConvertToAdif := False;
      AdifStatus := 'Error opening input file 1';
      Exit;
    end;

    Reset(Inf);
    if IoResult <> 0 then
    begin
      ConvertToAdif := False;
      AdifStatus := 'Error opening input file 2';
      Exit;
    end;

    Assign(OutF, OutFile);
    if IoResult <> 0 then
    begin
      ConvertToAdif := False;
      AdifStatus := 'Error opening output file 1';
      Exit;
    end;

    Rewrite(OutF);
    if IoResult <> 0 then
    begin
      ConvertToAdif := False;
      AdifStatus := 'Error opening output file 2';
      Exit;
    end;

    if not WriteAdifHeader then begin
      ConvertToAdif := False;
      AdifStatus := 'Could not write header to output file';
      Exit;
    end;
    NumRecs := FileSize(Inf)-1;
    RecordNumber := 0;
    SizeOfScrap := Sizeof(Scrap);
    Read(InF, Scrap);
    for i := 1 to numrecs do begin
      Read(InF, Scrap);
      inc(RecordNumber);
      hioresult := ioresult;
      case (hIoResult = 0) of
        True : AdifStatus := 'Processing record # '+long2str(RecordNumber);
        False : begin
                  AdifStatus := 'Get data failed at # '+long2str(RecordNumber)+' Error # '+long2str(hioresult);
                  ConvertToAdif := False;
                  Exit;
                end;
      end;
      if Scrap.ToolBoxNumber = 0 then begin
        if not WriteAdifRecord(Scrap) then begin
          ConvertToAdif := False;
          Exit;
        end;
        inc(AdifRecords);
      end;
    end;
    Close(InF);
    Close(Outf);
 end;


begin
  AdifStatus := ' ADIF Engine Initialized.. . ';
  AdifRecords := 0;
end.
