{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
program AEAUtil;
uses
  TpCrt,
  ApMisc,
  ApPort,
  ApUart,
  OoCom,
  OpString,
  IniTools;
var
  UP, UP2 : UartPortPtr;
  C : Char;
  FoundIt,Finished : Boolean;
  GS : string[255];
  P : PortRecPtr;
  ThisComName  : ComNameType;
  ThisBaud     : LongInt;
  ThisDataBits : DataBitType;
  ThisStopBits : StopBitType;
  ThisParity   : ParityType;
  TestComName  : ComNameType;
  TestBaud     : LongInt;
  TestDataBits : DataBitType;
  TestStopBits : StopBitType;
  TestParity   : ParityType;
  i : integer;
  tmp_i : byte;
  tmp_b : word;
const
  TestBauds : array[1..5] of longint = (19200,9600,4800,2400,1200);

  procedure Async_Init;
    { initialize variables }
  var
    S : String[4];
    T : String[7];
    Irq : Byte;
    Vector : Byte;
    BaseAddr : Word;
    Com : ComNameType;
    Ini : TIni;
  begin
    Ini.Init('RTTY.INI', True);
    ps2detectmode := ps2ignore;
    ActivateApUart; {REQUIRED!!! .., in order to use upper interupts}
    for Com := Com1 to Com8 do begin
      S := ComNameString(Com);
      Irq := Ini.GetProfileByte('IrqNumber', S, 0);
      case Irq of
       $05 : Vector := $0D;
       $06 : Vector := $0E;
       $07 : Vector := $0F;
       $08 : Vector := $70;
       $09 : Vector := $71;
       $0A : Vector := $72;
       $0B : Vector := $73;
       $0C : Vector := $74;
       $0D : Vector := $75;
       $0E : Vector := $76;
       $0F : Vector := $77;
      else
             Vector := $00;
      end;
      BaseAddr := Ini.GetProfileWord('BaseAddr', S, 0);
      if BaseAddr = 0 then {Try for both spellings}
        BaseAddr := Ini.GetProfileWord('BaseAddress', S, 0); {v2.5a}
      SetUart(Com, BaseAddr, Irq, Vector);
      if TestComname = Com then begin
        if irq > 0 then tmp_i := irq;
        if BaseAddr > 0 then tmp_b := BaseAddr;
      end;
    end;
    Ini.Done;
  end;                            { Async_Init }



procedure MyColors;
begin
  TextColor(Green);
  TextBackground(Black);
end;

procedure DataColors;
begin
  TextColor(LightRed);
end;


procedure ShowMsg(const Msg : string; const ErrorNum : word);
begin
  if (ParamStr(1) = '/msg') or (ParamStr(2) = '/msg')  or (ParamStr(3) = '/msg') then
    begin
      WriteLn;
      MyColors;
      Write(Msg+':');
      DataColors;
      Writeln(StatusStr(ErrorNum));
    end;
end;

procedure ShowData(const Data : string);
begin
  if (ParamStr(1) = '/data') or (ParamStr(2) = '/data')  or (ParamStr(3) = '/data') then
    begin
      WriteLn;
      MyColors;
      Write('Data Recd:');
      DataColors;
      Writeln(Data);
    end;
end;

function WaitnSee : boolean;
begin
  UP^.PutString(^A'OHON'^W);
  if AsyncStatus <> ecOk then begin
    ShowMsg('TX Line error ', AsyncStatus);
    UP^.FlushInBuffer;
    end;
  UP^.GetStringTimeout(GS, 15, [], 5);
  if AsyncStatus <> ecOk then begin
    ShowMsg('RX Line error ', AsyncStatus);
    UP^.FlushInBuffer;
    end;
  WaitNSee := Pos('HOST',GS) > 0;
  ShowData(GS);
  UP^.PutString(^C);
  if AsyncStatus <> ecOk then begin
    ShowMsg('TX Line error ', AsyncStatus);
    UP^.FlushInBuffer;
    end;
  UP^.GetStringTimeout(GS, 15, [], 5);
  if AsyncStatus <> ecOk then begin
    ShowMsg('RX Line error ', AsyncStatus);
    UP^.FlushInBuffer;
    end;
  ShowData(GS);
  WaitNSee := Pos('cmd:',GS) > 0;
end;

function TryThis(NewComName  : ComNameType;
                 NewBaud     : LongInt;
                 NewDataBits : DataBitType;
                 NewStopBits : StopBitType;
                 NewParity   : ParityType) : boolean;
begin
  TryThis := False;
  if (NewDataBits = 8) and (NewStopBits = 2) then Exit; {invalid combo!}
  MyColors;
  write('Trying...');
  DataColors;
  writeln(ComNameString(NewComName)+','+
          long2str(NewBaud)+','+
          long2str(NewDataBits)+','+
          long2str(NewStopBits)+','+
          ParityString[NewParity]);
  UP := New(UartPortPtr, InitCustom(NewComName,
                NewBaud,
                NewParity,
                NewDataBits,
                NewStopBits, 50, 50, DefPortOptions));
  if AsyncStatus <> ecOk then begin
    MyColors;
    Write('Failed to open comport: ');
    DataColors;
    Writeln(StatusStr(AsyncStatus));
    Halt;
    end;
  if UP^.CheckLineError then begin
    UP^.FlushInBuffer; { flush the input buffer }
    if UP^.CheckFifoError then begin
      SetFifoBuffering(UP^.PR^.BaseAddr,False,0); { flush the fifo buffer }
      SetFifoBuffering(UP^.PR^.BaseAddr,True,14);
    end;
    UP^.GetLineError;  { resets error indicators }
  end;
  if WaitNSee then begin
    TryThis := True;
    ThisComName  := NewComName;
    ThisBaud     := NewBaud;
    ThisDataBits := NewDataBits;
    ThisStopBits := NewStopBits;
    ThisParity   := NewParity;
  end;
  Dispose(Up,Done);
end;

procedure Main;
begin
  FoundIt := True;
  for TestBaud := 1 to 5 do
    for TestDataBits := 8 downto 7 do
      for TestStopBits := 2 downto 1 do
        for TestParity   := NoParity to EvenParity do begin
          if TryThis(TestComName,TestBauds[TestBaud],TestDataBits,TestStopBits,TestParity) then Exit;
          if Keypressed then begin
            FoundIt := False;
            if ReadKeyWord <> $FFFF then Exit;
          end;
        end;
  FoundIt := False;
end;


begin
  clrscr;
  MyColors;
  writeln('AEA Utility to identify serial parameters.');
  writeln('By Ray Ortgiesen, WF1B');
  writeln;
  if ParamCount = 0 then begin
    writeln('Tries to place the AEA unit into the command mode.');
    writeln('Use /1 to use COM1 ... /8=Com8');
    writeln('Use /msg to see error messages');
    writeln('Use /data to see incoming data stream');
    writeln('Both /msg and /data must be in lower case if used');
    writeln;
    writeln('examples:     AeaUtil /1');
    writeln('              AeaUtil /2 /msg');
    writeln('              AeaUtil /1 /msg /data');
    writeln;
    halt;
    end;
  if (ParamStr(1) = '/1') or (ParamCount = 0) then
    TestComName := Com1;
  if (ParamStr(1) = '/2') then
    TestComName := Com2;
  if (ParamStr(1) = '/3') then
    TestComName := Com3;
  if (ParamStr(1) = '/4') then
    TestComName := Com4;
  if (ParamStr(1) = '/5') then
    TestComName := Com5;
  if (ParamStr(1) = '/6') then
    TestComName := Com6;
  if (ParamStr(1) = '/7') then
    TestComName := Com7;
  if (ParamStr(1) = '/8') then
    TestComName := Com8;
  Async_Init;
  Writeln('Comport   '+ComNameString(TestComname));
  if tmp_b > 0 then writeln('BaseAddr  $'+hexw(tmp_b))
               else writeln('BaseAddr  <default>');
  if tmp_i > 0 then writeln('Irq       '+long2str(tmp_i))
               else writeln('Irq       <default>');
  Writeln('Does your AEA unit have just 1 LED lit?(Upper right corner)');
  DataColors;
  readln(gs);
  MyColors;
  if upcase(gs[1]) = 'Y' then begin
    UP2 := New(UartPortPtr, InitCustom(TestComName,9600,NoParity,8,1,50,50,DefPortOptions));
    if UP2^.CheckLineError then begin
      UP2^.FlushInBuffer; { flush the input buffer }
      if UP2^.CheckFifoError then begin
        SetFifoBuffering(UP2^.PR^.BaseAddr,False,0); { flush the fifo buffer }
        SetFifoBuffering(UP2^.PR^.BaseAddr,True,14);
      end;
      UP2^.GetLineError;  { resets error indicators }
    end;
    UP2^.PutChar('*'); Delay(400);
    UP2^.PutChar('*'); Delay(400);
    UP2^.PutChar('*'); Delay(400);
    UP2^.PutChar(^M);
    for i := 1 to 10 do
      UP2^.GetString(GS, 15, []);
    Dispose(UP2,Done);
  end;

  main;
  if not FoundIt then begin
    writeln;
    writeln('Unable to put AEA unit into command mode...');
    writeln('Are you sure that was the right serial port?');
    halt;
    end;
  UP2 := New(UartPortPtr, InitCustom(
                ThisComName,
                ThisBaud,
                ThisParity,
                ThisDataBits,
                ThisStopBits, 500, 500, DefPortOptions));
  if UP2^.CheckLineError then begin
    UP2^.FlushInBuffer; { flush the input buffer }
    if UP2^.CheckFifoError then begin
      SetFifoBuffering(UP2^.PR^.BaseAddr,False,0); { flush the fifo buffer }
      SetFifoBuffering(UP2^.PR^.BaseAddr,True,14);
    end;
    UP2^.GetLineError;  { resets error indicators }
  end;
  UP2^.PutChar(^M);
  UP2^.FlushInBuffer;
  UP2^.PutChar(^M);
  UP2^.GetStringTimeout(GS, 15, [], 10);
  UP2^.PutChar(^M);
  UP2^.GetStringTimeout(GS, 15, [], 10);
  UP2^.PutChar(^M);
  UP2^.GetStringTimeout(GS, 15, [], 10);
  if Pos('cmd:',GS) > 0 then begin
    MyColors;
    writeln('AEA Unit now in Command Mode!');
    writeln('Settings:');
    write('  Port     : ');
    DataColors;
    writeln(ComNameString(ThisComName));
    MyColors;
    write('  Baud Rate: ');
    DataColors;
    Writeln(long2str(ThisBaud));
    MyColors;
    write('  Data Bits: ');
    DataColors;
    writeln(long2str(ThisDataBits));
    MyColors;
    write('  Stop Bits: ');
    DataColors;
    writeln(long2str(ThisStopBits));
    MyColors;
    write('  Parity   : ');
    DataColors;
    writeln(ParityString[ThisParity]);
    writeln;
    end;
  MyColors;
  UP2^.PutString('PERM'^M);
  if (ThisBaud <> 9600) or
     (ThisDataBits <> 8) or
     (ThisStopBits <> 1) or
     (ThisParity <> NoParity) then begin
    writeln('Would you like to change to 9600,8,1,N? (y/n)');
    DataColors;
    readln(gs);
    if upcase(gs[1]) = 'Y' then begin
      UP2^.PutString('TB 9600'^M);
      UP2^.PutString('AWLEN 8'^M);
      UP2^.PutString('PARITY 0'^M);
      UP2^.PutString('8BITCONV ON'^M);
      Delay(1000);
      UP2^.PutString('PERM'^M);
      Delay(1000);
      for i := 1 to 30 do
        UP2^.GetString(GS, 15, []);
      Delay(1000);
      UP2^.PutString('RESTART'^M);
      Delay(2000);
    end;
  end
  else
    Writeln('AEA Unit already in optimal settings.');
  Dispose(UP2,Done);
  MyColors;
  Writeln('Done!');
  NormVideo;
end.
