{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit ARFDPrec;

interface

uses OpRoot;

type
  PPrecData = ^TPrecData;
  TPrecData = object(SingleListNode)
                Callsign : string[12];
                Precidence : string[3];
                constructor Init(const aCall : string; const aPrecidence : string);
                Destructor Done; virtual;
                function GetPrecidence : string;
                procedure SetPrecidence(const aPrecidence : string);
                function GetCall : string;
                procedure SetCall(const aCall : string);
              end;

  PPrecList = ^TPrecList;
  TPrecList = object(SingleList)
                CurrNode : PPrecData;
                constructor Init;
                destructor Done; virtual;
                function Locate(const aCall : string) : boolean;
                function GetPrecidence(const aCall : string) : string;
                procedure SetPrecidence(const aCall : string; const aPrecidence : string);
              end;

var
  PrecList : PPrecList;

implementation

uses
  DebugRtns,
  OpString;

constructor TPrecData.Init(const aCall : string; const aPrecidence : string);
begin
  if not inherited Init then Fail;
  Precidence := Trim(aPrecidence);
  Callsign := Trim(aCall);
end;

Destructor TPrecData.Done;
begin
  inherited Done;
end;

function TPrecData.GetPrecidence : string;
begin
  GetPrecidence := Precidence;
end;

procedure TPrecData.SetPrecidence(const aPrecidence : string);
begin
  Precidence := Trim(aPrecidence);
end;

function TPrecData.GetCall : string;
begin
  GetCall := Callsign;
end;

procedure TPrecData.SetCall(const aCall : string);
begin
  Callsign := Trim(aCall);
end;


constructor TPrecList.Init;
begin
  inherited Init;
    {$IFDEF DEBUG}
    AppendDebugMessage('PrecList created');
    {$ENDIF}
end;

destructor TPrecList.Done;
begin
  inherited Done;
    {$IFDEF DEBUG}
    AppendDebugMessage('PrecList destroyed');
    {$ENDIF}
end;

function TPrecList.Locate(const aCall : string) : boolean;
var
  Found : Boolean;
begin
  Found := False;
  CurrNode := PPrecData(Head);
  while (CurrNode <> nil) and (not Found) do
    if CurrNode^.GetCall = Trim(aCall) then Found := True
                                       else CurrNode := PPrecData(Next(CurrNode));
  Locate := Found;
end;

function TPrecList.GetPrecidence(const aCall : string) : string;
begin
  if Locate(aCall) then
    GetPrecidence := CurrNode^.GetPrecidence
  else
    GetPrecidence := '';
end;

procedure TPrecList.SetPrecidence(const aCall : string; const aPrecidence : string);
var
  bPrecidence : string[10];
begin
  if Locate(aCall) then
    {nothing to do, already in the list}
    CurrNode^.Precidence := aPrecidence
  else begin
    if Length(aPrecidence) > sizeof(bPrecidence) then bPrecidence := Copy(aPrecidence,1,sizeof(bPrecidence))
                                     else bPrecidence := aPrecidence;
    CurrNode := New(PPrecData, Init(aCall, aPrecidence));
    Append(CurrNode);
  end;
end;

end.