{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$F+,O+}
{$E+} { numeric coprocessor emulation enabled}
UNIT Bearings;

INTERFACE

uses OpRoot;

type

  PBearingNode = ^TBearingNode;
  TBearingNode =
    object(SingleListNode)
      private
      Code    : String[7];
      Heading : Real;
      public
      constructor Init(const NewCode : String; const NewHeading : real);
      function GetCode : string;
      function GetHeading: real;
      destructor Done; virtual;
    end;

  PBearingList = ^TBearingList;
  TBearingList =
    object(SingleList)
      BearingPtr : PBearingNode;
      constructor Init;
      function    Add(const NewCode : String; const NewBearing : real) : Boolean;
      function    Find(const Thiscode : String) : Boolean;
      destructor  Done; virtual;
    end;

PBearing = ^TBearing;
TBearing = OBJECT
             MyLat,
             MyLong : real;
             BearingList : PBearingList;
             FUNCTION BearingTo(his_lat, his_long : real) : real;
             FUNCTION BearingToCode(code : string) : real;
             PROCEDURE LoadDataFile;
             CONSTRUCTOR Init;
             DESTRUCTOR  Done;
           END;

IMPLEMENTATION

USES
  cRttyWin,
  TpString,
  TpDos,
  DebugRtns,
  IniFile,
  IniTools;

CONST
  to_kilometers   : real = 111.11;
  to_radians      : real = 57.29577951308238;
  arc_to_nautical : real = 60;
  arc_to_statute  : real = 69.047;

  constructor TBearingNode.Init(const NewCode : String; const NewHeading : real);
  begin
    inherited Init;
    Code := NewCode;
    Heading := NewHeading;
  end;

  function TBearingNode.GetCode : string;
  begin
    GetCode := Code;
  end;

  function TBearingNode.GetHeading : real;
  begin
    GetHeading := Heading;
  end;

  destructor TBearingNode.Done;
  begin
    inherited Done;
  end;

  constructor TBearingList.Init;
  begin
    inherited Init;
    {$IFDEF DEBUG}
    AppendDebugMessage('BearingList created');
    {$ENDIF}
  end;

  destructor TBearingList.Done;
  begin
    inherited Done;
    BearingPtr := PBearingNode(Head);
    while BearingPtr <> nil do begin
      Delete(BearingPtr);
      BearingPtr := PBearingNode(Head);
    end;
    {$IFDEF DEBUG}
    AppendDebugMessage('BearingList destroyed');
    {$ENDIF}
  end;

  function TBearingList.Find(const ThisCode : String) : Boolean;
  begin
    Find := False;
    {Get the Head, then traverse the entire List}
    BearingPtr := PBearingNode(Head);
    while BearingPtr <> nil do begin
      if BearingPtr^.GetCode = ThisCode then begin
        Find := True;
        Exit;
      end;
      if BearingPtr^.GetCode > ThisCode then begin
        Find := False;
        Exit;
      end;
      BearingPtr := PBearingNode(Next(BearingPtr));
    end;
  end;

  function TBearingList.Add(const NewCode : String; const NewBearing : real) : Boolean;
  var
    NewBearingPtr : PBearingNode;
  begin
    if Trim(NewCode) = '' then begin
      Add := False;
      Exit;
    end;
    if not Find(NewCode) then begin
      {add node}
      Add := True;
      if BearingPtr = nil then begin
        New(BearingPtr, Init(NewCode,NewBearing)); {make a new list node}
        Append(BearingPtr);        {put it at the end of the list}
      end
      else begin
        New(NewBearingPtr, Init(NewCode,NewBearing)); {make a new list node}
        PlaceBefore(NewBearingPtr, BearingPtr); {put it at the end of the list}
      end;
    end
    else begin
      {node exists}
      Add := False;
    end;
  end;


CONSTRUCTOR TBearing.Init;
VAR
  s : STRING[7];
BEGIN
  { defaults are to WF1B qth }
  S := RttyIni^.GetProfileString('Latitude','PROFILE','41.15');
  IF NOT Str2real(S,MyLat) THEN MyLat := 41.15;
  S := RttyIni^.GetProfileString('Longitude','PROFILE','71.34');
  IF NOT Str2real(S,MyLong) THEN MyLong := 71.34;
  MyLat := MyLat / to_radians;
  BearingList := New(PBearingList, Init);
  LoadDataFile;
END;

DESTRUCTOR TBearing.Done;
BEGIN
  Dispose(BearingList, Done);
END;

PROCEDURE TBearing.LoadDataFile;
VAR
  f : text;
  data : string[59];
  code : string[7];
  lat  : string[9];
  long : string[9];
  rlat, rlong : real;
BEGIN
  if not existfile('BEARINGS.DAT') then Exit;
  rttywin^.putstring(^M+'Calculating all beam headings...');
  assign(f, 'BEARINGS.DAT');
  reset(f);
  while not Eof(F) do begin
    ReadLn(F, Data);
    Data := Trim(Data);
    code := trim(copy(data,1,7));
    Delete(Data,1,7);
    Data := Trim(Data);
    lat  := trim(copy(data,1,9));
    Delete(Data,1,9);
    Data := Trim(Data);
    long  := trim(copy(data,1,9));
    IF NOT Str2real(lat,rlat) THEN rlat    := 0;
    IF NOT Str2real(long,rlong) THEN rlong := 0;
    BearingList^.Add(Code,BearingTo(rlat,rlong));
  end;
  close(f);
  rttywin^.putstring('done');
END;

FUNCTION TBearing.BearingToCode(code : string) : real;
BEGIN
  case BearingList^.Find(code) of
    False : BearingToCode := 0;
    True  : BearingToCode := BearingList^.BearingPtr^.GetHeading;
  end;
END;

FUNCTION TBearing.BearingTo(his_lat, his_long : real) : real;
VAR
  degrees_of_arc  : real;
  bearing_angle : real;
  int_value_e : real;
  diFf_in_long : real;
BEGIN
  his_lat := (his_lat + 0.0000005) / to_radians;
  diff_in_long := (MyLong - his_long + 0.0000005) / to_radians;
  int_value_e := Sin(MyLat) * Sin(his_lat) + Cos(MyLat) * Cos(his_lat) * Cos(diff_in_long);
  degrees_of_arc := -ArcTan(int_value_e / Sqrt(1 - Sqr(int_value_e)) ) + 1.57079;
  bearing_angle := (Sin(his_lat) - Sin(MyLat) * int_value_e) / (Cos(MyLat) * Sin(degrees_of_arc));
  IF bearing_angle >= 1 THEN
    bearing_angle := 0
  ELSE
  IF bearing_angle <= -1 THEN
    bearing_angle := 180 / to_radians
  ELSE
    bearing_angle := -ArcTan(bearing_angle /Sqrt(1 - Sqr(bearing_angle))) + 1.57079;
  bearing_angle := bearing_angle * to_radians;
  IF Sin(diff_in_long) < 0 THEN bearing_angle := 360 - bearing_angle;
{  writeln('Bearing:     ',bearing_angle:3:1);
   writeln('Kilometers: ',to_kilometers * degrees_of_arc* to_radians:5:1);
   writeln('Nautical:   ',arc_to_nautical * degrees_of_arc * to_radians:5:1);
   writeln('Statute:    ',arc_to_nautical * degrees_of_arc * to_radians:5:1); }
  BearingTo := bearing_angle;
END;

END.

