{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit CAeaCommand;
interface
uses
  RecTypes,
  tncobject,
  ApPort,
  TpCrt;
type
  PAeaCommand          = ^TAeaCommand;
  TAeaCommand          = object(TTnc)
                     constructor Init(const Port : ComNameType);
                     procedure Transmit(const Xmit : Boolean); virtual;
                     procedure SetUsos(const USOS : Boolean); virtual;
                     procedure SetBaudRate(const Rate : Byte); virtual;
                     procedure SetMode(const Mode : String); virtual;
                     procedure Letters; virtual;
                     procedure ClearXmitBuffer; virtual;
                     procedure StopEverything; virtual;
                     procedure SetSense(const Sense : Boolean); virtual;
                     destructor Done; virtual;
                   end;

implementation
uses
  TpString,
  cRttyWin,
  DebugRtns,
  IniFile,
  LowLvl;

  procedure TAeaCommand.Transmit(const Xmit : Boolean);
  begin
    if Xmit then SendString(^C+'X'+^M)
    else SendChar(^D);
  end;

  procedure TAeaCommand.SetUsos(const USOS : Boolean);
  begin
    inherited SetUsos(Usos);
    case Usos of
      True: SendString('USOS ON'^M);
      False: SendString('USOS OFF'^M);
    end;
  end;

  procedure TAeaCommand.Letters;
  begin
    SendString(^C+'L'+^M);
  end;

  procedure TAeaCommand.ClearXmitBuffer;
  begin
    SendString(^C+'R'+^M+'TCLEAR'+^M);
  end;

  procedure TAeaCommand.SetMode(const Mode : String);
  begin
    inherited SetMode(Mode);
    if Mode = 'RTTY' then SendString(^C+'BA'+^M);
  end;

  procedure TAeaCommand.SetSense(const Sense : Boolean);
  begin
    case Sense of
      True : SendString('RXREV OFF'^M); { normal }
      False: SendString('RXREV ON'^M);  { reverse }
    end;
    inherited SetSense(Sense);
  end;

  procedure TAeaCommand.SetBaudRate(const Rate : Byte);
  begin
    AppendDebugMessage(long2str(rate));
    inherited SetBaudRate(Rate);
    case Rate of
      74 : SendString(^C+'RB 75'+^M);
    else
      SendString(^C+'RB '+long2str(Rate)+^M);
    end;
  end;

  procedure TAeaCommand.StopEverything;
  begin
    SendString(^C+'TCLEAR'+^M);
    SendString(^C+'R'+^M);
  end;

  constructor TAeaCommand.Init(const Port : ComNameType);
  var
    c : char;
    i : integer;
    s : string[255];
    EntireString : Boolean;

    function GetToCmd : Boolean;  {3.3.05}
      function HaveCommand : Boolean;
      var
        S : String;
      begin
        WaitForString(255, S, 1, EntireString);
        HaveCommand := (Pos('PK-232', S) > 0) or (Pos('cmd:', S) > 0);
      end;

    var
      B : Byte;
      S : String;
    begin
      GetToCmd := True;

      { try pressing ENTER first }
      UP^.PutString(^M);
      if HaveCommand then Exit;
      if HaveCommand then Exit;
      if HaveCommand then Exit;
      { it wasnt in command mode }

      { try HOST MODE now }
      UP^.PutString(^A+'OHON'+^W);
      if HaveCommand then Exit;
      if HaveCommand then Exit;
      if HaveCommand then Exit;

      { try AUTOBAUD detection }
      for B := 1 to 10 do begin
        UP^.PutString('*');
        delay(100);
      end;
      if HaveCommand then Exit;
      if HaveCommand then Exit;
      if HaveCommand then Exit;
      UP^.PutString(^M);
      if HaveCommand then Exit;
      if HaveCommand then Exit;
      if HaveCommand then Exit;
      UP^.PutString(^M);
      if HaveCommand then Exit;
      if HaveCommand then Exit;
      if HaveCommand then Exit;

      { try CTL-C & ENTER detection }
      UP^.PutString(^C);
      if HaveCommand then Exit;
      if HaveCommand then Exit;
      if HaveCommand then Exit;
      UP^.PutString(^C);
      if HaveCommand then Exit;
      if HaveCommand then Exit;
      if HaveCommand then Exit;
      UP^.PutString(^C);
      if HaveCommand then Exit;
      if HaveCommand then Exit;
      if HaveCommand then Exit;
      UP^.PutString(^M);
      if HaveCommand then Exit;
      if HaveCommand then Exit;
      if HaveCommand then Exit;
      GetToCmd := False;
    end;


  begin
    inherited InitCom(Port);
    TncNum := bAeaCommand;
    RttyWin^.PutString(^M'Initializing AEA Command Mode'); {v2.5a}
    if NOT GetToCmd then begin {3.3.05 try new way, if it fails try old }
      for i := 1 to 5 do begin
        SendString('*');
        RttyWin^.PutString('.');
        delay(500);
      end;
      UP^.FlushInBuffer;
      SendString(^C+^C+^C);
      RttyWin^.PutString('.');
      Delay(500);
      s := '';
      c := #$255;
      while (C <> cNUL) and (Length(S) < 253) do begin
        Receive(c);
        if C <> cNUL then s := s + c;
      end;
      if pos('cmd:',s) = 0 then begin
        SendString(^A'OHON'^W);
        Delay(200);
        s := '';
        c := #$255;
        while (C <> cNUL) and (Length(S) < 253) do begin
          Receive(c);
          if C <> cNUL then s := s + c;
        end;
      end;
      if pos('cmd:',s) > 0 then RttyWin^.PutString('OK'^M);
    end;
    SendString(^M);
    SendString('XFLOW'^M);
    Delay(200);
    UP^.GetString(S,0,[cCr]);
    UP^.GetString(S,0,[cCr]);
    Delay(2000);
    SendString('EXPERT ON'^M);
    Delay(200);
    SetMode(GetMode);
    Delay(200);
    SendString('RBAUD 45'^M);
    Delay(200);
    SetSense(GetSense);
    Delay(200);
    SendString('TXREV OFF'^M);
    Delay(200);
    SendString('ECHO ON'^M);
    Delay(200);
    SendString('EAS ON'^M);
    Delay(200);
    SetUsos(GetUsos);
    Delay(200);
    case RttyIni^.GetProfileBool('Diddles', 'TNC1', True) of
      true: SendString('DIDDLE ON'^M);
      false: SendString('DIDDLE OFF'^M);
    end;
    Delay(200);
    UP^.FlushInBuffer;
    SendString(^M);
    {$IFDEF DEBUG}
    AppendDebugMessage('AeaCommand created');
    {$ENDIF}
  end;

  destructor TAeaCommand.Done;
  begin
    {$IFDEF DEBUG}
    AppendDebugMessage('AeaCommand destroyed');
    {$ENDIF}
    inherited Done;
  end;

end.
