{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit cAutoCq;

interface

uses
  TpDate,
  Diskio;

type
  PAutoCq = ^TAutoCq;
  TAutoCq = object
            private
              AutoCqFlag      : boolean;
              AutoCqBufferNum : integer;
              AutoCqSeconds   : integer;
              AutoCqLastTime  : Time;
            public
              constructor Init;
              destructor  Done;
              procedure   CheckAutoRepeat(const R : log_rec);
              procedure   SetupAutoRepeat(const Command : String);
              procedure   Stop;
              procedure   Start;
            end;

var
  AutoCq : PAutoCq;


implementation

uses
  TpEntry,
  TpString,
  cRttyWin,
  cMsgWin,
  Routines,
  DebugRtns;

  constructor TAutoCq.Init;
  begin
    AutoCqFlag := False;
    AutoCqLastTime := CurrentTime;
    AutoCqBufferNum := 1;
    AutoCqSeconds := 15;
    {$ifdef debug}
    AppendDebugMessage('AutoCq created.');
    {$endif}
  end;

  destructor  TAutoCq.Done;
  begin
    {$ifdef debug}
    AppendDebugMessage('AutoCq destroyed.');
    {$endif}
  end;

  procedure  TAutoCq.CheckAutoRepeat(const R : log_rec);
  var
    Hours, Minutes, Seconds : Byte;
    Temp_Rec : Log_Rec;

  function FNumToCmdNum(const FNum : Byte) : esType;
  begin
    case FNum of
      1 : FNumToCmdNum := EsMy13;
      2 : FNumToCmdNum := Esmy14;
      3 : FNumToCmdNum := Esmy15;
      4 : FNumToCmdNum := Esmy16;
      5 : FNumToCmdNum := ESmy17;
      6 : FNumToCmdNum := ESmy18;
      7 : FNumToCmdNum := Esmy19;
      8 : FNumToCmdNum := Esmy50;
      9 : FNumToCmdNum := ESmy51;
      10: FNumToCmdNum := Esmy52;
      11: FNumToCmdNum := EsMy60;
      12: FNumToCmdNum := EsMy61;
      13: FNumToCmdNum := EsMy62;
      14: FNumToCmdNum := EsMy63;
      15: FNumToCmdNum := EsMy64;
      16: FNumToCmdNum := EsMy65;
      17: FNumToCmdNum := EsMy66;
      18: FNumToCmdNum := EsMy67;
      19: FNumToCmdNum := EsMy68;
      20: FNumToCmdNum := EsMy69;
    else
      FNumToCmdNum := EsMy13;
    end;
    {$ifdef debug}
    AppendDebugMessage('CAUTOCQ.FNumToCmdNum= '+long2str(FNum));
    {$endif}
  end;

  begin
    if AutoCqFlag then begin
      TimeDiff(CurrentTime, AutoCqLastTime, Hours, Minutes, Seconds);
      {$ifdef debug}
      AppendDebugMessage('CAUTOCQ.CheckAutoRepeat:HMS: '+long2str(Hours)+','+
      +long2str(Minutes)+','+long2str(Seconds));
      {$endif}
      if (Hours > 0) or
      (Minutes > 0) or
      (Seconds >= AutoCqSeconds) then begin
        AutoCqLastTime := CurrentTime;
        {$ifdef debug}
        AppendDebugMessage('CAUTOCQ.CheckAutoRepeat: AutoCqLastTimeSet');
        {$endif}
        Move(R, Temp_Rec, SizeOf(Temp_Rec));
        {$ifdef debug}
        AppendDebugMessage('CAUTOCQ.CheckAutoRepeat: R copied to Temp, success.');
        {$endif}
        RttyWin^.SendKeys(FNumToCmdNum(AutoCqBufferNum), Temp_Rec);
        {$ifdef debug}
        AppendDebugMessage('CAUTOCQ.CheckAutoRepeat: SendKeys complete');
        {$endif}
      end;
    end;
  end;

  procedure TAutoCq.SetupAutoRepeat(const Command : String);
  var
    BufferNumber : Integer;
    NumberOfSeconds : Integer;
    Error : Integer;
    S : String[6];
    b : Byte;
    Cmd : string[12];
  begin
    AutoCqFlag := False;
    Cmd := Command;
    while Pos('_', cmd) > 0 do
      cmd[Pos(' ', S)] := ' ';
    Delete(Cmd, 1, 4);
    Cmd := Trim(Cmd);
    b := Pos(' ', Cmd);
    if b = 0 then begin { only 1 parm passed, assume it is the function key#}
      Val(Cmd, BufferNumber, Error);
      if Error > 0 then begin
        Msgwin^.Display('AUTOCQ: Invalid buffer number, use 1-20');
        Exit;
      end;
      NumberOfSeconds := AutoCqSeconds;
    end else begin {2 parms}
      S := Copy(Cmd, 1, b-1);
      Val(S, BufferNumber, Error);
      if Error > 0 then begin
        Msgwin^.Display('AUTOCQ: Invalid buffer number, use 1-20');
        Exit;
      end;
      Delete(Cmd, 1, b);
      Cmd := Trim(Cmd);
      Val(Cmd, NumberOfSeconds, Error);
      if Error > 0 then begin
        Msgwin^.Display('AUTOCQ: Invalid seconds value, use 1-59');
        Exit;
      end;
    end;
    if (BufferNumber > 20) or (BufferNumber = 0) then begin
      Msgwin^.Display('AUTOCQ: Invalid buffer number, use 1-20');
      Exit;
    end;
    if (NumberOfSeconds > 59) or (NumberOfSeconds = 0) then begin
      Msgwin^.Display('AUTOCQ: Invalid seconds value, use 1-59');
      Exit;
    end;
    AutoCqFlag := True;
    AutoCqBufferNum := BufferNumber;
    AutoCqSeconds := NumberOfSeconds;
    {$ifdef debug}
    AppendDebugMessage('CAUTOCQ.SetupAutoRepeat: AutoCqFlag=True');
    AppendDebugMessage('CAUTOCQ.SetupAutoRepeat: AutoCqBufferNum='+long2str(AutoCqBufferNum));
    AppendDebugMessage('CAUTOCQ.SetupAutoRepeat: AutoCqSeconds='+long2str(AutoCqseconds));
    {$endif}
  end;

  procedure   TAutoCq.Stop;
  begin
    AutoCqFlag := FALSE;
    {$ifdef debug}
    AppendDebugMessage('CAUTOCQ.SetupAutoRepeat: AutoCqFlag=False');
    {$endif}
  end;

  procedure   TAutoCq.Start;
  begin
    AutoCqFlag := TRUE;
    {$ifdef debug}
    AppendDebugMessage('CAUTOCQ.SetupAutoRepeat: AutoCqFlag=True');
    {$endif}
  end;


end.