{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit CBaudotDrv;
interface
uses RecTypes,
  TpString,
  TpCrt,
  Keys,
  ApPort,
  TncObject;
type
  ShiftType = (LTRS, FIGS);
  PBaudotDrv = ^TBaudotDrv;
  TBaudotDrv = object(Ttnc)
                CharDelay : Integer;
                constructor Init(Port : ComNameType);
                procedure SendByte(const Data : Byte); virtual;
                procedure SendString(const Data : String); virtual;
                procedure Transmit(const Xmit : Boolean); virtual;
                procedure Receive(var Ch : Char); virtual;
                procedure Letters; virtual;
                procedure Figures; virtual;
                procedure SetBaudRate(const Rate : Byte); virtual;
                procedure StopEverything; virtual;
                procedure ComPut(B : Byte);
                procedure ComPutCh(Ch : Char);
                destructor Done; virtual;
                procedure PortReady;
                procedure SetSlowBaud(Rate : Byte);
              end;
const
  TXTABLE : array[1..128] of Byte
  = (0, 0, 0, 0, 0, 0, 0, 133, 0, 0, 2, 0, 0, 8, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    4, 141, 145, 148, 137, 0, 154, 139, 143, 146, 0, 0, 140, 131, 156, 157,
    150, 151, 147, 129, 138, 144, 149, 135, 134, 152, 142, 158, 0, 0, 0, 153,
    0, 3, 25, 14, 9, 1, 13, 26, 20, 6, 11, 15, 18, 28, 12, 24,
    22, 23, 10, 5, 16, 7, 30, 19, 29, 21, 17, 31, 133, 27, 0, 0,
    0, 3, 25, 14, 9, 1, 13, 26, 20, 6, 11, 15, 18, 28, 12, 24,
    22, 23, 10, 5, 16, 7, 30, 19, 29, 21, 17, 0, 0, 0, 0, 0);

  RXTABLE : array[0..31, ShiftType] of Char
  = ((#0, #0), ('E', '3'), (#10, #10), ('A', '-'),
    (' ', ' '), ('S', #39), ('I', '8'), ('U', '7'),
    (#13, #13), ('D', '$'), ('R', '4'), ('J', #39),
    ('N', ','), ('F', '!'), ('C', ':'), ('K', '('),
    ('T', '5'), ('Z', '"'), ('L', ')'), ('W', '2'),
    ('H', '#'), ('Y', '6'), ('P', '0'), ('Q', '1'),
    ('O', '9'), ('B', '?'), ('G', '&'), ('f', 'f'),
    ('M', '.'), ('X', '/'), ('V', ';'), ('~', '~'));

var
  Shift, RXshift : ShiftType;
  Ch : Char;
  Murray : Byte;


implementation

uses
  cRttyWin,
  IniTools,
  DebugRtns,
  LowLvl,
  IniFile;

  procedure TBaudotDrv.SetSlowBaud(Rate : Byte);
  const
    DivisorLatchBit = $80;
    UartTHR = $00;
    UartRBR = $00;
    UartIER = $01;
    UartIIR = $02;
    UartLCR = $03;
    UartMCR = $04;
    UartLsr = $05;

  var
    a, B : Byte;
    speed : Integer;
    divided : Real;
    base : word;
  begin
    base := UP^.PR^.BaseAddr;
    divided := 115200.0/Rate;
    speed := Trunc(divided);
    if Port[Base+UartLCR] < DivisorLatchBit then
      Port[Base+UartLCR] := Port[Base+UartLCR]+DivisorLatchBit;
    Port[Base+UartTHR] := Lo(speed);
    Port[Base+UartIER] := Hi(speed);
    Port[Base+UartLCR] := Port[Base+UartLCR]-DivisorLatchBit;
  end;

  { Port_Ready & ComPut are methods private to this object }
  procedure TBaudotDrv.PortReady;
    { this routine waits for the last character to be sent before it allows  }
    { another character to overwrite the previous one                        }
  begin
    while not UP^.TransReady  do ;
  end;

  procedure TBaudotDrv.ComPut(B : Byte);
  begin
    PortReady;
    UP^.PutString(chr(b));
  end;

  procedure TBaudotDrv.ComPutCh(Ch : Char);
  begin
    PortReady;
    UP^.PutString(ch);
  end;

  { end of private methods }

  procedure TBaudotDrv.SendByte(const Data : Byte);
    { transmit a character }
    { this routine takes a character, converts it to a byte     }
    { then does a lookup in TXtable for the baudot value. If the murray      }
    { value is greater than 127 then it is a FIGS character.                 }
  var
    C : Char;
  begin
    C := Chr(Data);
    if RttyIni^.GetProfileBool('LocalEcho', 'Profile', True) then RttyWin^.PutChar(C); { put it on screen }
    PortReady;
    case C of
      ^_ : begin                  {chr(31)=letters}
             ComPutCh(^_);
             Shift := LTRS;
           end;
      ^M : begin                  {chr(13)=cr/lf/ltrs}
             ComPutCh(^H); PortReady;
             ComPutCh(^B); PortReady;
             ComPutCh(^_);
             Shift := LTRS;
           end;
      ^[ : begin
             Shift := FIGS;
             ComPutCh(^[);
           end;
      ' ' : begin
              ComPutCh(^D);       {space}
              if GetUsos then
                begin
                  PortReady;
                  ComPutCh(^_);
                  Shift := LTRS;
                end;
            end;
      ^@ : {null} ;
      ^T : Transmit(True);
      ^R : Transmit(False);
    else
      begin
        Murray := TXTABLE[Ord(C)+1];
        if Murray <> 0 then
          begin
            if (Murray < 128) and (Shift = FIGS) then
              begin
                Shift := LTRS;
                ComPutCh(^_);
              end;
            if (Murray > 127) and (Shift = LTRS) then
              begin
                Shift := FIGS;
                ComPutCh(^[);
                Murray := Murray-128;
              end;
            PortReady;
            ComPut(Murray);
          end;
      end;
    end;
  end;

  procedure TBaudotDrv.SendString(const Data : String);
  var
    B : Byte;
    UserPressedEsc : Boolean;
  begin
    for B := 1 to Length(Data) do begin
      SendByte(Ord(Data[B]));
      if KeyPressed then begin
        UserPressedEsc := (ReadKeyWord = kbEsc);
        if UserPressedEsc then begin
          UP^.FlushOutBuffer;
          while KeyPressed do ReadKeyWord;
          B := Length(Data);
        end;
      end
      else
        MyDelayTicks(CharDelay);
    end;
  end;

  procedure TBaudotDrv.Transmit(const Xmit : Boolean);
  var
    UserPressedEsc : Boolean;
  begin
    UserPressedEsc := False;
    if not Xmit then begin
      while (UP^.OutBuffUsed > 0) and (not UserPressedEsc) do
        if KeyPressed then UserPressedEsc := (ReadKeyWord = kbEsc);
      if UserPressedEsc then begin
        StopEverything;
        while KeyPressed do ReadKeyWord;
      end;
    end;
    UP^.SetDtr(xmit);
    UP^.SetRts(xmit);
  end;

  procedure TBaudotDrv.Receive(var Ch : Char);
    procedure BaudotToAscii(var C : Char);
      { this routine takes the value in C and coverts it to an ascii }
      { value to be returned.      }
    begin
      if C < Chr(32) then begin
        C := RXTABLE[Ord(C), RXshift];
        case C of
          ' ' : if GetUsos then RXshift := LTRS;
          ^J  : begin
                  RXshift := LTRS;
                  C := ^J;
                end;
          'f' : begin
                  RXshift := FIGS;
                  C := #00;
                end;
          '~' : begin
                  RXshift := LTRS;
                  C := #00;
                end;
        else
          C := C; { no special action }
        end;
      end
      else C := #00; { not a valid baudot char }
    end;

  begin
    inherited receive(Ch);
    if Ch > #$00 then BaudotToAscii(Ch);
  end;

  procedure TBaudotDrv.Letters;
  begin
    SendByte(Ord(^L));
  end;

  procedure TBaudotDrv.Figures;
  begin
    SendByte(Ord(^N));
  end;

  procedure TBaudotDrv.StopEverything;
  begin
    UP^.FlushOutBuffer;
    UP^.SetDtr(False);
    UP^.SetRts(False);

  end;

  procedure TBaudotDrv.SetBaudRate(const Rate : Byte);
  begin
    SetSlowBaud(Rate);
  end;

  procedure SetBaud(Base : Word; Rate : Byte);
  const
    DivisorLatchBit = $80;
    UartTHR = $00;
    UartRBR = $00;
    UartIER = $01;
    UartIIR = $02;
    UartLCR = $03;
    UartMCR = $04;
    UartLsr = $05;

  var
    a, B : Byte;
    speed : Integer;
    divided : Real;
  begin
    divided := 115200.0/Rate;
    speed := Trunc(divided);
    if Port[Base+UartLCR] < DivisorLatchBit then
      Port[Base+UartLCR] := Port[Base+UartLCR]+DivisorLatchBit;
    Port[Base+UartTHR] := Lo(speed);
    Port[Base+UartIER] := Hi(speed);
    Port[Base+UartLCR] := Port[Base+UartLCR]-DivisorLatchBit;
  end;

  constructor TBaudotDrv.Init(Port : ComNameType);
  var
    iStopBits : StopBitType;
  begin
    inherited InitCom(Port);
    TncNum := bBaudotDrv;
    UP^.SetDtr(False);
    UP^.SetRts(False);
    CharDelay := RttyIni^.GetProfileWord('CharDelay', 'TNC1', 3);
    iStopBits := RttyIni^.GetProfileByte('StopBits', 'TNC1', 2);
    if CharDelay = 0 then Inc(CharDelay);
    with UP^ do begin
      ChangeBaud(110);
      ChangeDataBits(5);
      ChangeStopBits(iStopBits);
    end;
    SetSlowBaud(45);
    Transmit(False);
    Shift := LTRS;                {start in letters}
    RXshift := LTRS;
    {$IFDEF DEBUG}
    AppendDebugMessage('BaudotDrv created');
    {$ENDIF}
  end;

  destructor TBaudotDrv.Done;
  begin
    inherited Done;
    {$IFDEF DEBUG}
    AppendDebugMessage('BaudotDrv destroyed');
    {$ENDIF}
  end;

end.

