{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
unit cBotWin;

interface

uses
  TpWindow;

type

PBotWin = ^TBotWin;
TBotWin = object
             Win : WindowPtr;
             xlow,
             ylow,
             xhigh,
             yhigh : Byte; { tpcrt standard }
             constructor Init;
             destructor  Done; virtual;
             procedure   Update(Who : String; qth : string); virtual;
           end;

var
  BotWin : PBotWin;

implementation

uses
  IniFile,
  LowLvl,
  DebugRtns,
  RecTypes,
  cDupeTbl,
  cCountry,
  cSection,
  TpString,
  TpMouse,
  TpDate,
  Diskio,
  cProvince,
  cState,
  cDist_W,
  cDist_VE,
  cDist_VK,
  cDist_JA,
  cDist_ZL,
  cYearTbl,
  cColors,
  CEditWin,
  TpCrt;


function BearingToCode(Cd : string) : string;
begin
  If Assigned(Bearing) then BearingToCode := Real2Str(Bearing^.BearingToCode(Cd),3,0)+''
  else BearingToCode := '';
end;

constructor TBotWin.Init;
const
  NoFrame = False;
  ClearWindow = True;
  SaveWindow = True;
begin
  xlow  := MW_xhi+1; { just to right of main window }
  ylow  := MW_Ylo+7;
  xhigh := ScreenWidth;
  yhigh := ylow+5;
  with Colors^ do
  if not MakeWindow(Win, xlow, ylow, xhigh, yhigh,
      NoFrame, ClearWindow, SaveWindow,
      WindowAttr, FrameAttr, HeaderAttr, '') then
    AppendDebugMessage('TBotWin.Init, MakeWindow(Win) failed');
  DisplayWindow(Win);
  {$IFDEF DEBUG}
  AppendDebugMessage('BotWin created');
  {$ENDIF}
end;

destructor  TBotWin.Done;
begin
  KillWindow(Win);
  {$IFDEF DEBUG}
  AppendDebugMessage('BotWin destroyed');
  {$ENDIF}
end;

procedure   TBotWin.Update(Who : String; qth : string);
var
  Band : BandType;
  Info : array[BandType] of Boolean;
  Slot : Word;
  Part2Id : String[19];
  Dist : Byte;
  ix : integer;
  a : log_rec;

  procedure SetCountryInfo;
  var
    Band : BandType;
  begin
    with Countrylist^ do begin
      if CtyNum = NoCountry then Exit;
      for Band := B80 to B10 do
        Info[Band] := GetCountry(CtyNum,Band);
      Part2Id := Ct.Code+' '+BearingToCode(ct.code);
    end;
  end;

  procedure SetCountryARRL;
  var
    Band : BandType;
  begin
    with Countrylist^ do begin
      if CtyNum = NoCountry then Exit;
      if GetCountry(CtyNum,TOT) then begin
        for Band := B80 to B10 do
          Info[Band] := True;
      end
      else begin
        for Band := B80 to B10 do
           Info[Band] := False;
      end;
      Part2Id := Ct.Code+' '+BearingToCode(ct.code);
    end;
  end;

  procedure SetJADistricts;
  var
    Band : BandType;
  begin
    Dist := JADistList^.GetDistNo(Who);
    if Dist = 99 then Exit;
    for Band := B80 to B10 do
      Info[Band] := JADistList^.GetDist(Dist,Band);
    Part2Id := 'JA'+long2str(Dist)+' '+BearingToCode('JA-JA'+long2str(Dist));
  end;

  procedure SetWDistricts;
  var
    Band : BandType;
  begin
    Dist := WDistList^.GetDistNo(Who);
    if Dist = 99 then Exit;
    for Band := B80 to B10 do
      Info[Band] := WDistList^.GetDist(Dist,Band);
    Part2Id := 'W'+long2str(Dist)+' '+BearingToCode('W-W'+long2str(Dist));
  end;

  procedure SetYear;
  var
    Band : BandType;
    Yr : word;
  begin
    Yr := EditW^.EditRecs[1].His_Serno;
    if Yr < 1900 then Exit;
    for Band := B80 to B10 do
      Info[Band] := YearList^.GetYear(Yr,Band);
    Part2Id := Long2str(Yr);
    with Countrylist^ do begin
      if CtyNum = NoCountry then Exit;
      Part2Id := Part2Id+' '+BearingToCode(ct.code);
    end;
  end;


  procedure SetVEDistricts;
  var
    Band : BandType;
  begin
    Dist := VEDistList^.GetDistNo(Who);
    if Dist = 99 then Exit;
    for Band := B80 to B10 do
      Info[Band] := VEDistList^.GetDist(Dist,Band);
    Part2Id := 'VE'+long2str(Dist)+' '+BearingToCode('VE-VE'+long2str(Dist));
  end;

  procedure SetVKDistricts;
  var
    Band : BandType;
  begin
    Dist := VKDistList^.GetDistNo(Who);
    if Dist = 99 then Exit;
    for Band := B80 to B10 do
      Info[Band] := VKDistList^.GetDist(Dist,Band);
    Part2Id := 'VK'+long2str(Dist)+' '+BearingToCode('VK-VK'+long2str(Dist));
  end;

  procedure SetZLDistricts;
  var
    Band : BandType;
  begin
    Dist := ZLDistList^.GetDistNo(Who);
    if Dist = 99 then Exit;
    for Band := B80 to B10 do
      Info[Band] := ZLDistList^.GetDist(Dist,Band);
    Part2Id := 'ZL'+long2str(Dist)+' '+BearingToCode('ZL-ZL'+long2str(Dist));
  end;

  procedure SetProvince;
  var
    Band : BandType;
  begin
    ix := ProvList^.GetProvNo(qth);
    if ix = 0 then Exit;
    for Band := B80 to B10 do
      Info[Band] := ProvList^.GetProv(ix,Band);
    Part2Id := qth+' '+BearingToCode('VE-'+qth);
  end;

  procedure SetProvinceARRL;
  var
    Band : BandType;
  begin
    ix := ProvList^.GetProvNo(qth);
    if ix = 0 then Exit;
    Part2Id := qth+' '+BearingToCode('VE-'+qth);
    if ProvList^.GetProv(ix,TOT) then
      for Band := B80 to B10 do
        Info[Band] := True;
  end;

  procedure SetState;
  var
    Band : BandType;
  begin
    ix := StateList^.GetStateNo(Qth);
    if ix = 0 then Exit;
    Part2Id := qth+' '+BearingToCode('W-'+qth);
    for Band := B80 to B10 do
      Info[Band] := StateList^.GetState(ix,Band);
  end;

  procedure SetStateARRL;
  var
    Band : BandType;
  begin
    ix := StateList^.GetStateNo(Qth);
    if ix = 0 then Exit;
    Part2Id := qth+' '+BearingToCode('W-'+qth);
    if StateList^.GetState(ix,TOT) then
      for Band := B80 to B10 do
        Info[Band] := True;
  end;

  procedure SetSection;
  var
    Band : BandType;
  begin
    ix := SectionList^.GetSectionNo(Qth);
    if ix = 0 then Exit;
    Part2Id := qth;
    for Band := B80 to B10 do
      Info[Band] := SectionList^.GetSection(ix,Band);
  end;





begin
  if Trim(Who) = '' then Exit;
  for Band := B80 to B10 do
    Info[Band] := False;
  LogFile^.BlankLogRec(a);
  a.Call := Who;
  a.his_qth := qth;
  CountryList^.Identify(a);
  Part2Id := '';
  with CountryList^ do begin
    case Contest of
      VOLT : begin
               if Ct.Code = 'ZL' then SetZLDistricts else
               if Ct.Code = 'JA' then SetJADistricts else
               if Ct.Code = 'W' then SetWDistricts else
               if Ct.Code = 'VE' then SetVEDistricts else
               if Ct.Code = 'VK' then SetVKDistricts else
               SetCountryInfo;
             end;
      NAQP:  begin
               if ct.code = 'W' then SetState else
               if ct.code = 'VE' then SetProvince;
               if (Part2ID = '') then SetCountryInfo;
             end;
      CQWW:  begin
               if ct.code = 'W' then SetState else
               if ct.code = 'VE' then SetProvince;
               if (Part2ID = '') then SetCountryInfo;
             end;
      ARFD:  begin
               if (ct.code = 'W') or
                  (ct.code = 'VE') then SetSection;
               if (Part2ID = '') then SetCountryInfo;
             end;
      HSSC,
      SART,
      BART,
      BSPR,
      ANAR,
      JART : begin
               if Ct.Code = 'JA' then SetJADistricts else
               if Ct.Code = 'W' then SetWDistricts else
               if Ct.Code = 'VE' then SetVEDistricts else
               if Ct.Code = 'VK' then SetVKDistricts else
               SetCountryInfo;
             end;
      ARRL,
      TARA : begin
               if ct.code = 'W' then SetStateArrl else
               if ct.code = 'VE' then SetProvinceArrl;
               if (Part2ID = '') then SetCountryArrl;
             end;
      SCCC:  begin
               SetYear;
               if (Part2ID = '') then SetCountryInfo;
             end;
      else
        SetCountryInfo;
    end;
  end;
  Part2Id := Copy(Pad(Part2ID,11),1,11);
  HideMouse;
  with Colors^ do begin
   if Trim(Part2Id) <> '' then begin
     FastWriteStack(Win, Pad(Part2Id, 11),           1, 2, HiLiteAttr);
     FastWriteStack(Win, '80 - '+NeedStr(Info[B80]), 2, 2, NeedAttr(Info[B80]));
     FastWriteStack(Win, '40 - '+NeedStr(Info[B40]), 3, 2, NeedAttr(Info[B40]));
     FastWriteStack(Win, '20 - '+NeedStr(Info[B20]), 4, 2, NeedAttr(Info[B20]));
     FastWriteStack(Win, '15 - '+NeedStr(Info[B15]), 5, 2, NeedAttr(Info[B15]));
     FastWriteStack(Win, '10 - '+NeedStr(Info[B10]), 6, 2, NeedAttr(Info[B10]));
   end
   else begin
     FastWriteStack(Win, 'UNKNOWN!   ',          1, 2, HiLiteAttr);
     FastWriteStack(Win, '80 -'+CharStr(' ', 7), 2, 2, WindowAttr);
     FastWriteStack(Win, '40 -'+CharStr(' ', 7), 3, 2, WindowAttr);
     FastWriteStack(Win, '20 -'+CharStr(' ', 7), 4, 2, WindowAttr);
     FastWriteStack(Win, '15 -'+CharStr(' ', 7), 5, 2, WindowAttr);
     FastWriteStack(Win, '10 -'+CharStr(' ', 7), 6, 2, WindowAttr);
   end;
  end;
  ShowMouse;
end;

end.