{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
unit cclock;

interface

uses
  TpDate;

type

pclock = ^Tclock;
TClock = object
           private
             DateMask : string[6];
             HrsDiff  : integer;
           public
             constructor Init;
             destructor  Done;
             function    TimeSLoc : string;
             function    TimeSUtc : string;
             function    DateSLoc : string;
             function    DateSUtc : string;
             function    TimeLoc : Time;
             function    TimeUtc : Time;
             function    DateLoc : Date;
             function    DateUtc : Date;
             procedure   DateAndTimeSUtc(var UtcDate : String; var UtcTime : String);
             procedure   DateAndTimeUtc(var UtcDate : Date; var UtcTime : Time);
             procedure   Adjust;
             procedure   ConvertToUtc(var D : Date; var T : Time);
             function    Date2DateString(const D : Date) : string;
             function    HoursFromUtc : integer;
             procedure   SetHoursFromUtc(const Hrs : integer);
           private
             procedure   UpdateTimeSample;
         end;

var
  Clock : PClock;

implementation

uses
  Dos,
  Keys,
  TpCrt,
  TpEntry,
  TpMouse,
  TpWindow,
  TpString,
  RecTypes,
  cColors,
  OneTimers,
  DebugRtns,
  IniFile;



constructor TClock.Init;
begin
  DateMask := StLoCase(RttyIni^.GetProfileString('DateMask','PROFILE','yymmdd'));
  if (DateMask <> 'yymmdd') and (DateMask <> 'mmddyy') and (DateMask <> 'ddmmyy')
    then DateMask := 'yymmdd';
  HrsDiff := RttyIni^.GetProfileInt('HoursFromUtc','PROFILE',0);
  if (HrsDiff < -23) or (HrsDiff>23) then HrsDiff := 0;
    {$IFDEF DEBUG}
    AppendDebugMessage('Clock created');
    {$ENDIF}
end;

destructor  TClock.Done;
begin
    {$IFDEF DEBUG}
    AppendDebugMessage('Clock destroyed');
    {$ENDIF}
end;

function    TClock.TimeSLoc : string;
begin
  TimeSLoc := CurrentTimeString('hh:mm');
end;

function    TClock.TimeSUtc : string;
var
  UtcTime : Time;
  UtcDate : Date;
begin
  UtcDate := Today;
  UtcTime := CurrentTime;
  ConvertToUtc(UtcDate, UtcTime);
  TimeSUtc := TimeToTimeString('hh:mm',UtcTime);
end;

function    TClock.Date2DateString(const D : Date) : string;
begin
  Date2DateString := DateToDateString(Datemask,D);
end;


function    TClock.TimeLoc : Time;
begin
  TimeLoc := CurrentTime;
end;

function    TClock.DateLoc : Date;
begin
  DateLoc := Today;
end;

procedure TClock.ConvertToUtc(var D : Date; var T : Time);
var
  H, M, S : Integer;
  H1, M1, S1 : Byte;
begin
  if HrsDiff = 0 then Exit;
  TimeToHMS(T, H1, M1, S1);
  H := H1;
  M := M1;
  S := S1;
  H := H+HrsDiff;
  if (HrsDiff > 0) and (H > 23) then begin
    H := H-24;
    D := IncDate(D, 1, 0, 0);
  end;
  if (HrsDiff < 0) and (H < 0) then begin
    H := H+24;
    D := IncDate(D, -1, 0, 0);
  end;
  T := HMSToTime(H, M, S);
end;

function    TClock.DateSLoc : string;
begin
  DateSLoc := TodayString(DateMask);
end;

function    TClock.DateSUtc : string;
var
  UtcTime : Time;
  UtcDate : Date;
begin
  UtcDate := Today;
  UtcTime := CurrentTime;
  ConvertToUtc(UtcDate, UtcTime);
  DateSUtc := Date2DateString(UtcDate);
end;

function    TClock.TimeUtc : Time;
var
  UtcTime : Time;
  UtcDate : Date;
begin
  UtcDate := Today;
  UtcTime := CurrentTime;
  ConvertToUtc(UtcDate, UtcTime);
  TimeUtc := UtcTime;
end;

function    TClock.DateUtc : Date;
var
  UtcTime : Time;
  UtcDate : Date;
begin
  UtcDate := Today;
  UtcTime := CurrentTime;
  ConvertToUtc(UtcDate, UtcTime);
  DateUtc := UtcDate;
end;

procedure   TClock.DateAndTimeSUtc(var UtcDate : String; var UtcTime : String);
var
  UT : Time;
  UD : Date;
begin
  UD := Today;
  UT := CurrentTime;
  ConvertToUtc(UD, UT);
  UtcDate := Date2DateString(UD);
  UtcTime := TimeToTimeString('hh:mm',UT);
end;

procedure   TClock.DateAndTimeUtc(var UtcDate : Date; var UtcTime : Time);
begin
  UtcDate := Today;
  UtcTime := CurrentTime;
  ConvertToUtc(UtcDate, UtcTime);
end;




  procedure IncChoice(var Value; FieldID : Byte;
    Factor : Integer;
    var St : String); far;
  var
    Hrs : Integer absolute Value;
  begin
    case FieldID of
      2 : begin
              case Factor of
                1 : if Hrs < 23 then Inc(Hrs)
                    else Hrs := -23;
                -1 : if Hrs > -23 then Dec(Hrs)
                     else Hrs := 23;
              end;
              St := Long2str(hrs);
            end;
     end;
  end;

function MyGetKey : Word; far;
begin
  while not KeyPressed do begin
    Clock^.UpdateTimeSample;
  end;
  MyGetKey := ReadKeyorButton;
end;

var
  SampTime : Time;
  SampDate : Date;

procedure TClock.UpdateTimeSample;
begin
  SampDate := DateLoc;
  SampTime := TimeLoc;
  ConvertToUtc(SampDate, SampTime);
  FastWrite('UTC Date-Time: '+Date2DateString(SampDate), 11,  4, Colors^.WindowAttr);
  FastWrite(TimeToTimeString('hhmm',SampTime),           11, 27, Colors^.WindowAttr);
end;

procedure TClock.Adjust;
var
  T : Time;
  D : Date;
  W : WindowPtr;
  E : EsRecord;
  ExitCommand : Word;
  hh,mm,ss,hs : byte;
  mo, da, yy : integer;
  holdptr : pointer;
  temp_hrs : integer;
begin
  temp_hrs := HrsDiff;
  holdptr := entrykeyptr;
  entrykeyptr := @mygetkey;
  with Colors^ do begin
    MakeWindow(W, 3, 3, 40, 13, True, True, False, WindowAttr,
        FrameAttr, HeaderAttr, 'Set Date & Time');
    HideMouse;
    DisplayWindow(W);
    InitEsRecord(E);
    T := CurrentTime;
    D := Today;
    AddDateField(E,  'System Date(yyyymmdd):',     5, 4,
                     'yyyymmdd',                   5, 26, 0,
                     MinDate, Today, D);
    AddTimeField(E,  'System Time(hhmmss):',       7, 4,
                     'hhmmss',                     7, 26, 0,
                     MinTime, MaxTime, T);
    AddChoiceField(E,'Hours from UTC:',            9, 4,
                     'XXX',                        9, 26, 0,
                     1, @IncChoice, HrsDiff);
    FastWrite('Ctrl-Enter to continue', 13, 4, WindowAttr);
  end;
  ShowMouse;
  AddCmd(ESEnd, kbEnd);
  AddCmd(ESHome,kbHome);
  AddCmd(EsQuit,kbEsc);
  repeat
    ExitCommand := EditScreen(E, 0, False);
  until ExitCommand in [EsQuit, EsDone];
  AddCmd(ESmy25, kbEnd);
  AddCmd(ESmy23, kbHome);
  AddCmd(Esmy49, kbEsc);
  if ExitCommand = EsDone then
    begin { hit ctl-enter }
      TimeToHMS(T, hh, mm, ss);
      hs := 0;
      DateToDMY(D, da, mo, yy);
      SetTime(hh,mm,ss,hs);
      SetDate(yy,mo,da);
      RttyIni^.SetOrAddProfileString('HoursFromUtc','PROFILE',long2str(HrsDiff));
    end
    else
    begin { hit escape }
      HrsDiff := Temp_Hrs;
    end;
  KillWindow(W);
  DisposeEditScreen(E);
  entrykeyptr := holdptr;
end;

function    TClock.HoursFromUtc : integer;
begin
  HoursFromUtc := HrsDiff;
end;

procedure  TClock.SetHoursFromUtc(const Hrs : integer);
begin
  RttyIni^.SetOrAddProfileString('HoursFromUtc','PROFILE',long2str(Hrs));
  HrsDiff := Hrs;
end;


end.