{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$F+,O+}
unit cContinent;
interface

  {* FUNCTIONS AVAILABLE FROM THIS UNIT *}

uses
  TpRarray,
  Rectypes;

const
  Num_Contients = 7;
type
  ContinentType  = (NA, SA, EU, AS, AF, OC, AN);
const
  Cont_First = NA;
  Cont_Last  = AN;
type

  PContList = ^TContList;
  TContList = object
                 private
                   Continent_Tbl : array[ContinentType] of Mult_Rec;
                 public
                   constructor Init;
                   Destructor  Done;
                   function    GetContinent(const ct : ContinentType; const Band : BandType) : boolean;
                   function    GetContinentByAbbr(const ct : String; const Band : BandType) : boolean;
                   procedure   SetContinent(const ct : ContinentType; const Band : BandType);
                   procedure   SetContinentByAbbr(const ct : String; const Band : BandType);
                   procedure   ReSetContinent(const ct : ContinentType; const Band : BandType);
                   function    HowMany(const Band : BandType) : longint;
                   procedure   Display;
                   procedure   Print;
                   function    BandString(const ct : ContinentType) : string;
                   function    GetContNo(const Ct : String) : ContinentType;
                   function    ContAbbrev(const Ct : ContinentType) : string;
                   function    ContName(const Ct : ContinentType) : string;
                   function    Load(const CT : string; const Band : BandType) : Boolean;
                 end;

  {* NOW DEFINE THE FUNCTIONS *}

var
  ContList : PContList;

procedure MakeContList;
procedure KillContList;

implementation

uses
  DebugRtns,
  TpString,
  TpCrt,
  cColors,
  TpMouse,
  TpSort,
  TpWindow,
  Keys,
  cEditWin,
  cMsgWin,
  c_Contest,
  Routines,
  LowLvl,
  IniFile,
  OneTimers;

  const
  Continent_Names : array[ContinentType] of String[10] = (
    'N. America', 'S. America', 'Europe    ', 'Asia      ',
    'Africa    ', 'Oceania   ', 'Antarctica');

  Continent_Abbr : array[ContinentType] of String[2] = (
    'NA', 'SA', 'EU', 'AS', 'AF', 'OC', 'AN');

  constructor TContList.Init;
  var
    C : ContinentType;
    Band : BandType;
  begin
    for C := Cont_First to Cont_Last do
      for Band := B80 to TOT do
        Continent_Tbl[C].Band[Band] := False;
    {$IFDEF DEBUG}
    AppendDebugMessage('ContList created');
    {$ENDIF}
  end;

  Destructor  TContList.Done;
  begin
    {$IFDEF DEBUG}
    AppendDebugMessage('ContList destroyed');
    {$ENDIF}
  end;

  function    TContList.GetContinent(const ct : ContinentType; const Band : BandType) : boolean;
  begin
    {$IFDEF DEBUG}
    if (ct < Cont_First) or (ct > Cont_Last) then
      AppendDebugMessage('CContinent.GetContinent, invalid ord(Continent#)='+long2str(ord(ct)));
    {$ENDIF}
    if (ct < Cont_First) or (ct > Cont_Last) then begin
      GetContinent := False;
      Exit;
    end;
    GetContinent := Continent_Tbl[ct].Band[Band]
  end;

  function    TContList.GetContinentByAbbr(const ct : String; const Band : BandType) : boolean;
  begin
    GetContinent(GetContNo(ct),Band);
  end;

  procedure   TContList.SetContinent(const ct : ContinentType; const Band : BandType);
  begin
    {$IFDEF DEBUG}
    if (ct < Cont_First) or (ct > Cont_Last) then
      AppendDebugMessage('CContinent.GetContinent, invalid ord(Continent#)='+long2str(ord(ct)));
    {$ENDIF}
    if (ct < Cont_First) or (ct > Cont_Last) then Exit;
    Continent_Tbl[ct].Band[Band] := True;
    Continent_Tbl[ct].Band[TOT]  := True;
  end;

  procedure   TContList.SetContinentByAbbr(const ct : String; const Band : BandType);
  begin
    SetContinent(GetContNo(ct),Band);
  end;

  procedure   TContList.ReSetContinent(const ct : ContinentType; const Band : BandType);
  var
    b : bandType;
  begin
    {$IFDEF DEBUG}
    if (ct < Cont_First) or (ct > Cont_Last) then
      AppendDebugMessage('CContinent.GetContinent, invalid ord(Continent#)='+long2str(ord(ct)));
    {$ENDIF}
    if (ct < Cont_First) or (ct > Cont_Last) then Exit;
    Continent_Tbl[ct].Band[Band] := False;
    Continent_Tbl[ct].Band[TOT] := False;
    for B := B80 to B10 do begin
      if GetContinent(ct,B) then
        Continent_Tbl[ct].Band[TOT]  := True;
    end;
  end;

  function    TContList.HowMany(const Band : BandType) : longint;
  var
    L       : longint;
    ct      : ContinentType;
  begin
    L := 0;
    for ct := Cont_First to Cont_Last do
      if GetContinent(ct,Band) then Inc(L);
    HowMany := L;
    {$IFDEF DEBUG}
    AppendDebugMessage('TContList.HowMany = '+Bands[Band]+'M '+long2str(l));
    {$ENDIF}
  end;

  procedure TContList.Display;
  var
    W              : Word;
    col, Row, ix   : Byte;
    Win            : WindowPtr;
    S              : String[5];
    Band           : BandType;
    Cont           : ContinentType;
  begin
    with Colors^ do begin
     if not MakeWindow(Win, 1, 2, 80, 14, True, True, True,
                       WindowAttr, FrameAttr, HeaderAttr,
                       'Continent Multiplier Check Sheet') then Exit;
     HideMouse;
     DisplayWindow(Win);
     for Row := 1 to 6 do begin
       Cont := ContinentType(Row-1);
       FastWriteWindow(ContName(Cont)+' |'+BandString(Cont)+'|', Row+1, 2, WindowAttr);
     end;
     FastWrite('Press Any Key to continue', 14, 2, WindowAttr);
    end;
    ShowMouse;
    ReadKeyorButton;
    KillWindow(Win);
  end;

  procedure TContList.Print;
  var
    Str            : String[80];
    Continent      : ContinentType;
    PrintFile      : Text;
  begin
    Assign(PrintFile, FileName('CON')); {v2.5a}
    {$I-} Rewrite(PrintFile); {$I+}
    MsgWin^.Display('Preparing: '+FileName('CON')); {v2.5a}
    Str := 'Continent Multiplier sheet for '+RttyIni^.GetProfileString('Call','Profile','');
    Str := Pad(Str, 70)+'Page 1';
    Writeln(PrintFile,Str);
    Writeln(PrintFile,'');
    for Continent := Cont_First to Cont_Last do begin
      Str := Continent_Names[Continent]+' |'+
             BandString(Continent)+
             '|';
      Writeln(PrintFile,Str);
    end;
    Write(PrintFile,^L);
    Close(PrintFile);
  end;

  function  TContList.BandString(const ct : ContinentType) : string;
  var
    s : string[5];
    Band : BandType;
  begin
    S := '.....';
    for Band := B80 to B10 do
      if GetContinent(ct,Band) then S[Ord(Band)+1] := '*';
    BandString := S;
  end;

  function TContList.GetContNo(const Ct : String) : ContinentType;
  var
    ix : ContinentType;
  begin
    for ix := Cont_First to Cont_Last do
      if Ct = Continent_Abbr[ix] then begin
        GetContNo := ix;
        Exit;
      end;
    GetContNo := Cont_First; { if not found, use Cont_First?! }
  end;

  function    TContList.Load(const CT : string; const Band : BandType) : Boolean;
  var
    c : ContinentType;
  begin
    C := GetContNo(Ct);
    Load := NOT Continent_Tbl[c].Band[TOT];
    Continent_Tbl[c].Band[Band] := True;
    Continent_Tbl[c].Band[TOT]  := True;
  end;


  function    TContList.ContAbbrev(const Ct : ContinentType) : string;
  begin
    ContAbbrev := Continent_Abbr[Ct];
  end;

  function    TContList.ContName(const Ct : ContinentType) : string;
  begin
    ContName := Continent_Names[Ct];
  end;

  procedure MakeContList;
  begin
    ContList := New(PContList, Init);
    if not assigned(ContList) then begin
      MessageBox('Severe Error!',
                 'Unable to create Continent list',
                 'Contact Wyvern Technology',
                 'Consider reinstalling from diskette');
      CloseDownScreen;
      Halt;
    end;
    {$IFDEF DEBUG}
    AppendDebugMessage('ContinentList created');
    {$ENDIF}
  end;

  procedure KillContList;
  begin
    if Assigned(ContList) then begin
      Dispose(ContList, Done);
      ContList := NIL;
      {$IFDEF DEBUG}
      AppendDebugMessage('ContinentList destroyed');
      {$ENDIF}
    end;
  end;



end.
