{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$V-,F+,O+}
unit CCountry;
interface

  {* FUNCTIONS AVAILABLE FROM THIS UNIT *}

uses
  TpRarray,
  DiskIo,
  Rectypes;

const
  NoCountry    : Word = 999;

type
  Ct_Rec_Type    = record
                     Name           : String[20];
                     Code           : String[5];
                     CQzone         : Byte;
                     Continent      : String[2];
                     Info           : String[10];
                   end;

  Pf_Rec_Type    = record
                     Prefix         : String[8];
                     Cty_No         : Integer;
                   end;

  PCountryList = ^TCountryList;
  TCountryList = object
                 public
                   CtyNum         : Word;
                   CT             : Ct_Rec_Type;
                 private
                   Ct_Num         : word;
                   Pf_Num         : word;
                   PF_Max         : word;
                   Ct_Max         : word;
                   MaxPrefix      : Word;
                   MaxCountry     : Word;
                   PF             : Pf_Rec_Type;
                   Country_Tbl    : array[0..350] of Mult_Rec;
                   Ct_Array       : TParray;
                   Pf_Array       : TParray;
                   LastCallChecked: String[12];
                 public
                   constructor Init(const Cname, PName : String);
                   Destructor  Done;
                   function    GetPrefixToId(Prefix : String) : String;
                   procedure   ParseCall(Call :  String; var Prefix : string; var Suffix : string);
                   function    Identify(const R : Diskio.log_rec) : Word; { CORE ROUTINE }
                   function    IdentifyCountry(Call : String; R : log_rec) : Word;
                   function    GetCountry(const Country : Word; const Band : BandType) : boolean;
                   function    GetAnyBand(const Country : Word) : boolean;
                   procedure   SetCountry(const Country : Word; const Band : BandType);
                   procedure   ReSetCountry(const Country : Word; const Band : BandType);
                   function    HowMany(const Band : BandType) : longint;
                   function    BandString(const Country : Word) : string;
                   procedure   GetCtRec(const Country : Word);
                   procedure   BlankCTRec;
                   procedure   DisplayMults;
                   procedure   DisplayAllCodes;
                   procedure   DisplayDXCCStatus;
                   procedure   DisplayInfoWin;
                   procedure   Print;
                   function    NumCountries : word;
                   function    UnknownCountry : Boolean;
                   function    NumberOfQuadMults : Integer;
                   function    IsQuadMult(const Country : Word) : Boolean;
                   function    CompletesQuadMult(const Country : Word; const Band : BandType) : Boolean;
                   procedure   PrintInterContinentMults;
                   function    IsUSorCanada(const R : log_rec) : Boolean;
                   function    Load(const R : log_rec;const Band : BandType) : Boolean;
                   procedure   List(const FName : string);
                 end;

  {* NOW DEFINE THE FUNCTIONS *}

var
  CountryList : PCountryList;

procedure MakeCountryList(const Cname, PName : String);
procedure KillCountryList;

implementation

uses
  DebugRtns,
  TpString,
  c_Contest,
  TpCrt,
  LowLvl,
  TpMouse,
  TpSort,
  TpWindow,
  Keys,
  cEditWin,
  cColors,
  cMsgWin,
  Routines,
  cContinent,
  IniFile,
  OneTimers;

  procedure TCountryList.ParseCall(Call :  String; var Prefix : string; var Suffix : string);
  var
    i : byte;
    EndPrefixPos : byte;
  begin
    Call := GetPrefixToId(Call);
    For i := 1 to Length(Call) do
      if Call[i] in ['0'..'9'] then EndPrefixPos := i;
    Prefix := Copy(Call,1,EndPrefixPos);
    Suffix := Copy(Call,EndPrefixPos+1,2);
  end;





  {* This routine checks to see if the callsign uses a portable designator. *}
  {* If PREFIX does contain a '/' then the prefix to identify the callsign  *}
  {* will be considered the shorter of the two halves.  If the shorter side *}
  {* is just 1 letter and it is a digit, the it will replace the first      *}
  {* number after the second digit.                                         *}
  {* Examples:  WF1B/KH6=KH6   NL7XX/4=NL4=W   W6/G0AZT = W                 *}
  {* KC1G/6 = KC6 = INVALID so... recommend KC1G/W6 instead                 *}

  function TCountryList.GetPrefixToId(Prefix : String) : String;
  var
    X, N, L, R     : Integer;
    Digit          : Char;
    Call, Orig     : String[12];
  label 999;
  begin
    Call := Prefix;
    Orig := Trim(Call);
    if Pos('/', Call) = 0 then goto 999;
    if (Pos('/MM', Call) > 0) or  {dont process maritime mobiles }
       (Pos('/M ', Call) > 0) or  {dont process mobiles }
       (Pos('/P ', Call) > 0) or  {dont process portables }
       (Pos('/AE ', Call) > 0) or {dont process extra }
       (Pos('/AG ', Call) > 0) or {dont process general }
       (Pos('/AA ', Call) > 0) or {dont process advanced }
       (Pos('/KT ', Call) > 0) or {dont process techs }
       (Pos('/2K ', Call) > 0) or {dont process techs }
       (Pos('/QRP', Call) > 0) then begin {dont process qrpers }
      GetPrefixToId := Copy(Prefix, 1, Pos('/', Prefix)-1);
      Exit;
      END;
    Digit := #0;
    Call := Orig;
    N := 0;
    for X := 1 to Length(Call) do
      if Call[X] = '/' then Inc(N);
    if N > 1 then Call := Pad(Copy(Call, 1, Pos('/', Call)), 8);
    if N > 1 then goto 999;
    L := Pos('/', Call)-1;        { length of left half }
    R := Length(Call)-Pos('/', Call); { length of right half }
    if L < R then
      if L > 8 then Call := Copy(Call, 1, 8)
      else Call := Copy(Call, 1, L); {use left portion}
    if R < L then
      if R > 8 then Call := Copy(Call, Pos('/', Call), 8)
      else Call := Copy(Call, Pos('/', Call)+1, R); {use right}
    if (L = 1) or (R = 1) then
    begin
      if (L = 1) then
      begin
        Digit := Orig[1];
        Call := Copy(Orig, 2, Length(Orig)-2);
      end;
      if (R = 1) then
      begin
        Digit := Call[Length(Orig)];
        Call := Copy(Orig, 1, Length(Orig)-2);
      end;
      if Digit in ['0'..'9'] then
      begin
        N := 1;
        repeat
          Inc(N);
        until Call[N] in ['0'..'9'];
        Call[N] := Digit;
      end
      else begin
        Call := Digit; { handles G/ON4NN or ON4GG/F }
      end;
    end;
999: GetPrefixToId := Pad(Call, 12);
  end;


  function TCountryList.IdentifyCountry(Call : String; R : log_rec) : Word;
  var
    Hi, Lo, Mid,
    RecCount, HoldMid : Word;
    Found, Collided,
    Stop           : Boolean;
  begin
    if Length(Trim(Call)) > 2 then
    begin
      if (R.His_Qth <> '') and (Copy(Call, 1, 3) = 'KC6') then Call := 'K';
      if (R.His_Qth <> '') and (Copy(Call, 1, 3) = 'KC4') then Call := 'K';
    end;
    Hi := PF_Max;
    Lo := 0;
    Found := False;
    Collided := False;
    Stop := False;
    Mid := (Lo+Hi) div 2;
    {* This portion finds a prefix with the same first letter as call *}
    repeat
      RetA(Pf_Array, Mid, 0, Pf);
      if (Lo = Mid) or (Hi = Mid) then Collided := True
      else
        if Call[1] = Pf.Prefix[1] then Stop := True
      else
      begin
        if Call > Pf.Prefix then Lo := Mid
        else Hi := Mid;
        Mid := (Lo+Hi) div 2;
      end;
    until Stop or Collided;
    Stop := False;

    repeat                        { find last matching first letter }
      Inc(Mid, 5);                {gets there 5x faster most of time}
      RetA(Pf_Array, Mid, 0, Pf);
    until (Call[1] <> Pf.Prefix[1]) or (Mid > MaxPrefix);

    repeat                        { now backup to last matching lead letter }
      Dec(Mid, 1);
      RetA(Pf_Array, Mid, 0, Pf);
    until (Call[1] = Pf.Prefix[1]) or (Mid = 0);

    repeat
      if Copy(Call, 1, Length(Pf.Prefix)) = Pf.Prefix then
      begin
        HoldMid := Mid;
        Found := True;
      end
      else
      begin
        if Mid > 0 then Dec(Mid, 1);
        RetA(Pf_Array, Mid, 0, Pf);
      end;
    until (Call[1] <> Pf.Prefix[1]) or (Found) or (Mid = 0);

    if Found and not Collided then
    begin
      RetA(Pf_Array, HoldMid, 0, Pf);
      IdentifyCountry := Pf.Cty_No;
    end
    else IdentifyCountry := NoCountry;
  end;

  {* This routine tries to open CNAME,PNAME parameters as TPRARRY arrays.  *}
  {* It also determines the maximum sizes of the arrays PF_MAX, and CT_MAX *}

  constructor TCountryList.Init(const Cname, PName : String);
  var
    W    : Word;
    Mult : Mult_Rec;
    Band : BandType;
  begin
    MaxPrefix   := 2000;
    MaxCountry  := 350;
    LastCallChecked := '~~~~~~~~~~~~';
    for Band := B80 to TOT do
      Mult.Band[Band] := False;
    for W := 0 to MaxCountry do
      Country_Tbl[W] := Mult;
    LoadA(Ct_Array, Cname);
    case ErrorA of
      0 : begin
            Ct_Max := 0;
            for W := 0 to MaxCountry-1 do begin
              RetA(Ct_Array, Ct_Max, 0, Ct);
              if Ct.Name <> '' then inc(Ct_Max);
            end;
          end;
      7 : NotEnoughMemory(True); { halt program }
    else
      Terminate(1);
    end;
    LoadA(Pf_Array, PName);
    Case ErrorA of
      0 : begin
            PF_Max := 0;
            for W := 0 to MaxPrefix-1 do begin
              RetA(Pf_Array, PF_Max, 0, Pf);
              if Pf.Prefix <> '' then PF_Max := PF_Max+1;
            end;
          end;
      7 : NotEnoughMemory(True); { halt program }
    else
      Terminate(2);
    end;
    {$IFDEF DEBUG}
    AppendDebugMessage('CountryList created');
    {$ENDIF}
  end;

  {* This is the core to the CTIDENT unit. Pass this routine the callsign  *}
  {* you want to have identified.  It will return a pointer to the country *}
  {* code appropriate for the callsign. A NoCountry is returned if the function  *}
  {* is not successful                                                     *}

  function TCountryList.Identify(const R : log_rec) : Word;
  begin
    If Trim(R.Call) = '' then Exit;
    if (LastCallChecked = R.Call) and (CtyNum <> NoCountry) then begin { same as the one we just checked ! }
      Identify := CtyNum;
      Exit;
    end;
    CtyNum := IdentifyCountry(GetPrefixToId(Pad(R.Call, 12)),R);
    if Not UnknownCountry then GetCtRec(CtyNum);
    LastCallChecked := Trim(R.Call);
    Identify := CtyNum;
  end;


  Destructor TCountryList.Done;
  begin
    DisposeA(Ct_Array);
    DisposeA(Pf_Array);
    {$IFDEF DEBUG}
    AppendDebugMessage('CountryList destroyed');
    {$ENDIF}
  end;

  function    TCountryList.GetCountry(const Country : Word; const Band : BandType) : boolean;
  begin
    {$IFDEF DEBUG}
    if Country > MaxCountry then
      AppendDebugMessage('TCountryList.GeTCountry, invalid country#');
    {$ENDIF}
    if Country > MaxCountry then begin
      GeTCountry := False;
      Exit;
    End;
    {$IFDEF DEBUG}
    AppendDebugMessage('Country='+long2str(Country)+' Band='+Bands[Band]);
    {$ENDIF}
    GetCountry := Country_Tbl[Country].Band[Band];
  end;

  procedure   TCountryList.SetCountry(const Country : Word; const Band : BandType);
  begin
    if Country > MaxCountry then Exit;
    {$IFDEF DEBUG}
    if Country > MaxCountry then
      AppendDebugMessage('TCountryList.SetCountry, invalid country#');
    {$ENDIF}
    if RttyIni^.GetProfileBool('CCOUNTRY.SETCOUNTRY', 'DEBUG', False) then
      AppendDebugMessage('CCountry.SetCountry Before '+long2str(Country)+','+long2str(ord(Band))+','+
        long2str(ord(Country_Tbl[Country].Band[Band])));
    Country_Tbl[Country].Band[Band] := True;
    Country_Tbl[Country].Band[TOT]  := True;
    if RttyIni^.GetProfileBool('CCOUNTRY.SETCOUNTRY', 'DEBUG', False) then
      AppendDebugMessage('CCountry.SetCountry  After '+long2str(Country)+','+long2str(ord(Band))+','+
        long2str(ord(Country_Tbl[Country].Band[Band])));
  end;

  procedure   TCountryList.ReSetCountry(const Country : Word; const Band : BandType);
  var
    b : bandType;
  begin
    if Country > MaxCountry then Exit;
    {$IFDEF DEBUG}
    if Country > MaxCountry then
      AppendDebugMessage('TCountryList.SetCountry, invalid country#');
    {$ENDIF}
    Country_Tbl[Country].Band[Band] := False;
    Country_Tbl[Country].Band[TOT] := False;
    for B := B80 to B10 do begin
      if GetCountry(Country,B) then
        Country_Tbl[Country].Band[TOT]  := True;
    end;
  end;

  function    TCountryList.GetAnyBand(const Country : Word) : boolean;
  begin
    if Country > MaxCountry then begin
      GetAnyBand := False;
      Exit;
    end;
    GetAnyBand := (Country_Tbl[Country].Band[B80]) or
                  (Country_Tbl[Country].Band[B40]) or
                  (Country_Tbl[Country].Band[B20]) or
                  (Country_Tbl[Country].Band[B15]) or
                  (Country_Tbl[Country].Band[B10]);
  end;


  procedure TCountryList.GetCtRec(const Country : Word);
  begin
    if Country <= MaxCountry then RetA(Ct_Array, Country, 0, Ct);
    { if same country, no need to reset callsign}
    if Country <> CtyNum then LastCallChecked := ''; { this is to prevent a serious bug! }
    { identify(R);  example code!
      GetCtRec(1);
      identify(R);
      withoyut this line, the seconf identify would think that the ct
      record is still OK, that is the lastcallchecked would be = r.call
      but the ct record would really be for country# 1
    }
  end;

  procedure TCountryList.BlankCTRec;
  begin
    CtyNum := NoCountry;
    Ct.Name := '';
    Ct.Code := '';
    Ct.CQzone := 0;
    Ct.Continent := '';
    Ct.Info := '';
  end;

  function TCountryList.HowMany(const Band : BandType) : longint;
  var
    l       : longint;
    Country : Word;
  begin
    l := 0;
    for Country := 0 to Ct_Max-1 do
      if Country_Tbl[Country].Band[Band] then Inc(l);
    HowMany := L;
    {$IFDEF DEBUG}
    AppendDebugMessage('TCountryList.HowMany '+Bands[Band]+' '+long2str(l));
    {$ENDIF}
  end;

  function  TCountryList.BandString(const Country : Word) : string;
  var
    s : string[5];
    Band : BandType;
  begin
    S := '.....';
    if Country > MaxCountry then begin
      BandString := S;
      Exit;
    end;
    for Band := B80 to B10 do
      if Country_Tbl[Country].Band[Band] then S[Ord(Band)+1] := '*';
    BandString := S;
  end;




  procedure TCountryList.DisplayMults;
  const
    ArrSz = 90;
  var
    NumCtys        : Word;
    W              : Word;
    ContArray      : TParray;
    C              : Cont_Rec_Type;
    Cont           : ContinentType;
    KeyStroke,
    col, Row       : Byte;
    ArrayOK        : Boolean;
    ContWin        : WindowPtr;

    function BuildContinentArray : Boolean;
    var
      Band           : BandType;
      Num            : Word;
      Countries      : Boolean;
    begin
      Countries := False;
      NumCtys := 0;
      for Num := 0 to NumCountries-1 do begin
        GetCtRec(Num);
        if Ct.Continent = ContList^.ContAbbrev(Cont) then begin
          C.Code := Pad(Ct.Code, 5);
          C.Info := BandString(Num);
          Seta(ContArray, NumCtys, 0, C);
          Countries := True;
          Inc(NumCtys);
          if NumCtys = ArrSz then Num := NumCountries-1; {force loop to end}
        end;
      end;
      if NumCtys > 0 then Dec(NumCtys); {* includes the zero entry *}
      BuildContinentArray := Countries;
    end;


  begin
    Cont := Cont_First;
    KeyStroke := kbNoKey;
    with Colors^ do begin
    if not MakeWindow(ContWin, 1, 1, 80, 25, True, True, True,
       WindowAttr, FrameAttr, HeaderAttr,'Multiplier Check Sheet') then begin
         NotEnoughMemory(False);
         Exit;
    end;
    MakeA(ContArray, ArrSz, 1, SizeOf(Cont_Rec_Type));
    if ErrorA <> 0 then begin
      KillWindow(ContWin);
      NotEnoughMemory(False);
      Exit;
    end;
    HideMouse;
    DisplayWindow(ContWin);
    FastWriteWindow('Press M or <space> for next Continent', 24, 21, HeaderAttr);
    repeat
      FastWriteWindow(ContList^.ContName(Cont), 2, 3, WindowAttr);
      C.Code := '     ';
      C.Info := '.....';
      ClearA(ContArray, C, ExactInit);
      BuildContinentArray;
      W := 0;
      Row := 4;
      col := 3;
      for W := 0 to ArrSz-1 do begin
        RetA(ContArray, W, 0, C);
        if Trim(C.Code) <> '' then FastWriteWindow(C.Code+' |'+C.Info+'|', Row, col, WindowAttr)
        else FastWriteWindow(CharStr(' ', 13), Row, col, WindowAttr);
        Inc(Row);
        if Row = 22 then begin
          Row := 4;
          Inc(col, 15);
        end;
      end;
      ShowMouse;
      KeyStroke := ReadKeyorButton;
      if Cont = Cont_Last then Cont := Cont_First
      else Inc(Cont);
    until not(Lo(KeyStroke) in [$4D, $6D, $20]); {'M'&'m'& ' '}
    end;
    DisposeA(ContArray);
    KillWindow(ContWin);
  end;


  { ------- PRINT MULTS ------------------------------------------------- }

  var
    ContArray      : TParray;
    C_Rec          : Cont_Rec_Type;


  procedure CInpRecs; far;
  var
    w : word;
  begin
    for W := 0 to CountryList^.Ct_Max-1 do begin
      RetA(ContArray, W, 0, C_Rec);
      PutElement(C_Rec);
    end;
  end;

  procedure COutpRecs; far;
  var
    w : word;
  begin
    for W := 0 to CountryList^.Ct_Max-1 do begin
      GetElement(C_Rec);
      Seta(ContArray, W, 0, C_Rec);
    end;
  end;

  function CLessRec(var R1, R2)    : Boolean; far;
  var
    X1             : Cont_Rec_Type absolute R1;
    X2             : Cont_Rec_Type absolute R2;
  begin
    CLessRec := X1.Code < X2.Code;
  end;



  procedure TCountryList.Print;
  var
    Row, col       : Word;
    Num         : Word;
    PageCT         : Byte;
    SortStat       : SortStatus;
    PrintFile      : Text;
    Page           : TParray;
    T              : String[100];

  procedure PrintF(Text : String);
  begin
    {$I-}
    if Text = ^L then Write(PrintFile, ^L)
    else WriteLn(PrintFile, Text);
    {$I+}
  end;

  procedure ClearAndFillContinentArray;
  var
    Num : Word;
  begin
    C_Rec.Code := '';
    C_Rec.Info := '.....';
    ClearA(ContArray, C_Rec, ExactInit);
    ClearA(Page, C_Rec, ExactInit);
    for Num := 0 to NumCountries-1 do begin
      GetCtRec(Num);
      C_Rec.Code := Pad(Ct.Code, 5);
      C_Rec.Info := BandString(Num);
      Seta(ContArray, Num, 0, C_Rec);
    end;
  end;

  procedure PrintMultPage(PageCT : Byte);
  var
    Row, col       : Word;
  begin
    Inc(PageCT);
    PrintF('');
    T := 'Country Multiplier sheet for '+RttyIni^.GetProfileString('Call','Profile','');
    T := Pad(T, 70)+'Page '+Long2Str(PageCT);
    Inc(PageCT);
    PrintF(T);
    PrintF('');
    for Row := 0 to 49 do begin
      T := ' ';
      for col := 0 to 4 do begin
        RetA(Page, Row, col, C_Rec);
        if Trim(C_Rec.Code) <> '' then T := T+C_Rec.Code+' |'+C_Rec.Info+'|'+'   ';
      end;
      if Length(T) > 79 then T := Copy(T, 1, 79);
      PrintF(T);
    end;
    if contest = VOLT then
      PrintF('W,VE,JA,VK,ZL countries shown for intercontinent credit, not country credit');
    PrintF(^L);
    C_Rec.Code := '';
    C_Rec.Info := '.....';
    for Row := 0 to 49 do
      for col := 0 to 4 do
        Seta(Page, Row, col, C_Rec);
  end;

  begin
    case Contest of
      WPXC,
      SPRI : Exit;
    end;
    PageCT := 0;
    Assign(PrintFile, FileName('MUL'));
    {$I-} Rewrite(PrintFile); {$I+}
    MsgWin^.Display('Preparing: '+FileName('MUL'));
    MakeA(ContArray, Ct_Max, 1, SizeOf(Cont_Rec_Type));
    MakeA(Page, 50, 5, SizeOf(Cont_Rec_Type));
    ClearAndFillContinentArray;
    SortStat := Sort(ct_max, SizeOf(Cont_Rec_Type), @CInpRecs, @CLessRec, @COutpRecs);
    case Sortstat of
      sortoutofmemory : AppendDebugMessage('TCountryList.Sort out of memory');
      sorttoomanyelements : AppendDebugMessage('TCountryList.Sort too many elements');
    end;
    if SortStat <> SortSuccess then begin
      DisposeA(Page);
      DisposeA(ContArray);
      Close(PrintFile);
      NotEnoughMemory(False);
      MsgWin^.Display('Unable to sort, not enough memory');
      Exit;
    end;
    Row := 0;
    col := 0;
    for Num := 0 to NumCountries-1 do begin
      RetA(ContArray, Num, 0, C_Rec); { sorted calls }
      if C_Rec.Code <> '' then begin
        Seta(Page, Row, col, C_Rec);
        Inc(Row);
        if Row = 50 then begin
          Inc(col);
          Row := 0;
        end;
        if col = 5 then begin
          PrintMultPage(PageCT);
          Inc(PageCT);
          col := 0;
          Row := 0;
        end;
      end;
    end;
    PrintMultPage(PageCT);
    DisposeA(Page);
    DisposeA(ContArray);
    Close(PrintFile);
  end;

  function    TCountryList.NumCountries : word;
  begin
    NumCountries := CT_Max;
  end;

  function    TCountryList.UnknownCountry : Boolean;
  begin
    UnknownCountry := CtyNum = NoCountry;
  end;

  function TCountryList.NumberOfQuadMults : Integer;
  var
    Country : Integer;
    Number : Integer;
  begin
    Number := 0;
    for Country := 0 to NumCountries-1 do
      if IsQuadMult(Country) then Inc(Number);
    NumberOfQuadMults := Number;
  end;

  function TCountryList.IsQuadMult(const Country : Word) : Boolean;
  var
    Band : BandType;
    NumBandsWorked : Byte;
  begin
    if Country = NoCountry then begin {v2.5c}
      IsQuadMult := False;
      Exit;
    end;
    GetCtRec(Country);
    if MyContinent = CT.Continent then
      IsQuadMult := False
    else begin
      NumBandsWorked := 0;
      for Band := B80 to B10 do
        if GetCountry(Country,Band) then Inc(NumBandsWorked);
      IsQuadMult := NumBandsWorked >= 4;
    end;
  end;

  procedure TCountryList.PrintInterContinentMults;
  var
    Band  : BandType;
    Cnt   : Word;
    PrintFile : Text;
  begin
    Assign(PrintFile,FileName('ICD'));
    {$I-} Rewrite(PrintFile); {$I+}
    MsgWin^.Display('Preparing: '+FileName('ICD'));
    Writeln(PrintFile,'Intercontinental DX Multiplier sheet for '+RttyIni^.GetProfileString('Call','Profile',''));
    Writeln(PrintFile,'');
    for Cnt := 0 to NumCountries-1 do
      if IsQuadMult(Cnt) then begin
        GetCtRec(Cnt);
        Writeln(PrintFile,pad(CT.Code, 5));
      end;
    Write(PrintFile,^L);
    Close(PrintFile);
  end;

  function TCountryList.CompletesQuadMult(const Country : Word; const Band : BandType) : Boolean;
  var
    T : Byte;
    b : BandType;
  begin
    T := 0;
    for b := B80 to B10 do
      if GetCountry(Country,b) then Inc(T);
    case T of
      0..2,
      4..5 : CompletesQuadMult := False;
      3 : if not GetCountry(Country,Band) then begin
            Inc(T);               { if it is needed then add to temp }
            GetCtRec(Country);
            if (T = 4) and (MyContinent <> CT.Continent)
            then CompletesQuadMult := True {if it's 4 now then its a mult!}
            else CompletesQuadMult := False;
          end
          else CompletesQuadMult := False;
    end;
  end;

  function TCountryList.IsUSorCanada(const R : log_rec) : Boolean;
  begin
    IsUSorCanada := (R.His_Country = 'W') or (R.His_Country = 'VE');
  end;

  procedure TCountryList.DisplayAllCodes;
  var
    AllWin      : WindowPtr;
    col, Row    : Byte;
    W           : Word;
  begin
    with Colors^ do begin
    if not MakeWindow(AllWin, 1, 1, 80, 25, True, True, True,
                      WindowAttr, FrameAttr, HeaderAttr,
                      'All Country Codes') then Exit;
    HideMouse;
    DisplayWindow(AllWin) ;
    Row := 1;
    col := 1;
    for W := 0 to NumCountries-1 do begin
      GetCtRec(W);
      FastWriteWindow(Ct.Code, Row, col, WindowAttr);
      Inc(Row);
      if Row = 24 then begin
        Row := 1;
        col := col+5;
      end;
    end;
    FastWrite('Press Any Key to continue', 25, 2, WindowAttr);
    end;
    ShowMouse;
    ReadKeyorButton;
    KillWindow(AllWin);
  end;

  procedure TCountryList.DisplayDXCCStatus;
  var
    DXCCWin        : WindowPtr;
    Cnt,
    col, Row       : Byte;
    Country : word;
  begin
    with Colors^ do begin
    if not MakeWindow(DXCCWin, 1, 1, 80, 25, True, True, True,
                      WindowAttr, FrameAttr, HeaderAttr, 'DXCC Status') then Exit;
    HideMouse;
    DisplayWindow(DXCCWin) ;
    Cnt := 0;
    Row := 1;
    col := 1;
    for Country := 0 to CountryList^.NumCountries-1 do begin
        if GetAnyBand(Country) then begin
          GetCtRec(Country);
          FastWriteWindow(Ct.Code, Row, col, WindowAttr);
          Inc(Cnt);
          Inc(Row);
          if Row = 24 then begin
            Row := 1;
            col := col+5;
          end;
        end;
    end;
    FastWriteWindow('('+Long2Str(Cnt)+')', Row, col, HiLiteAttr);
    FastWrite('Press Any Key to continue', 25, 2, WindowAttr);
    end;
    ShowMouse;
    ReadKeyorButton;
    KillWindow(DXCCWin);
  end;

  procedure TCountryList.DisplayInfoWin;
  var
    CtyWin         : WindowPtr;
    WinL           : Byte;
    Info           : String[49];
  begin
    if Trim(EditW^.EditRecs[1].Call) = '' then Exit;
    Ct_Num := Identify(EditW^.EditRecs[1]);
    if Ct_Num = NoCountry then begin
      MsgWin^.Display('Country is unknown');
      Exit;
    end;
    Info := Ct.Code+': '+Ct.Name+' '+Ct.Info+ ' |'+BandString(Ct_Num)+'|';
    WinL := Length(Info)+3;
    if WinL < 20 then WinL := 20;
    with Colors^ do begin
      if not MakeWindow(CtyWin, 80-Winl, 3, 80, 7, True, True, True, WindowAttr,
                        FrameAttr, HeaderAttr, 'Country Check: '+Trim(EditW^.EditRecs[1].Call)) then begin
        MsgWin^.Display('Not enough memory');
        Exit;
      end;
      HideMouse;
      DisplayWindow(CtyWin) ;
      FastWriteStack(CtyWin, Info, 2, 2, WindowAttr);
    end;
    ShowMouse;
    ReadKeyorButton;
    KillWindow(CtyWin);
  end;

  function TCountryList.Load(const R : log_rec;const Band : BandType) : Boolean;
  var
    Info : Boolean;
    tcn : word;     { temp ct.num, it is easier to type in }
  begin
    tcn := CountryList^.Identify(R);
    if tcn = NoCountry then begin
      Load := False;
      Exit;
    end;
    Load := not CountryList^.GetCountry(tcn,Band);
    SetCountry(tcn,Band);
  end;

  procedure TCountryList.List(const FName : string);
  var
    s     : string[255];
    firstone  : boolean;
    numc,nump  : word;
    outf  : text;
  begin
    assign(outf,FName);
    rewrite(outf);
    for Numc := 0 to MaxCountry-1 do begin
      firstone := true;
      RetA(Ct_Array, Numc, 0, CT);
      if CT.Code <> '' then begin
        s := pad(CT.Name+':',26)+pad(long2str(CT.cqzone)+':',5)+pad(CT.Code + ':',7);
        for Nump := 0 to MaxPrefix-1 do begin
          RetA(Pf_Array, Nump, 0, PF);
          if (Numc = PF.Cty_No) and (trim(pf.prefix)<>'') then begin
            pf.prefix := trim(pf.Prefix);
            if length(s) + length(pf.prefix) > 73 then begin
              writeln(outf,s+',');
              s := '&'+charstr(' ',37);
              firstone := true;
              end;
            if firstone then begin
              s := s + pf.prefix;
              firstone := false;
              end
            else
              s := s + ',' + pf.prefix;
            end;
          end;
          WriteLn(outf,s+';(',ct.continent,')');
        end;
      end;
    write(outf,^Z);
    close(outf);
  end;





{-------------------------------------------------------------}
  procedure MakeCountryList(const Cname, PName : String);
  begin
    CountryList := New(PCountryList, Init(CName, PName));
    if not assigned(CountryList) then begin
      MessageBox('Severe Error!',
                 'Unable to create country list',
                 'Verify .CTY & .PFX file are OK',
                 'Consider reinstalling from diskette');
      CloseDownScreen;
      Halt;
    end;
    {$IFDEF DEBUG}
    AppendDebugMessage('CountryList created');
    {$ENDIF}
  end;

  procedure KillCountryList;
  begin
    if Assigned(CountryList) then begin
      Dispose(CountryList, Done);
      CountryList := NIL;
      {$IFDEF DEBUG}
      AppendDebugMessage('CountryList destroyed');
      {$ENDIF}
    end;
  end;



end.
