{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$F+,O+}
unit cDist;
interface

  {* FUNCTIONS AVAILABLE FROM THIS UNIT *}

uses
  TpRarray,
  Rectypes;

type

  PDistList = ^TDistList;
  TDistList = object
                 private
                   Dist_Tbl    : array[0..9] of Mult_Rec;
                   Code        : string[5];
                 public
                   constructor Init(const Cty : string);
                   Destructor  Done; virtual;
                   function    GetDist(const D : Byte; const Band : BandType) : boolean;
                   procedure   SetDist(const D : Byte; const Band : BandType);
                   procedure   SetDistByCall(const Call : String; const Band : BandType);
                   procedure   ReSetDist(const D : Byte; const Band : BandType);
                   procedure   ResetDistByCall(const Call : String; const Band : BandType);
                   function    HowMany(const Band : BandType) : longint;
                   function    BandString(const D : Byte) : string;
                   function    GetDistNo(const Call : String) : Byte;
                   function    DistPrefix : string;
                   procedure   Print; virtual;
                   procedure   Display;
                   function    Load(const Cl : string; const Band : BandType) : Boolean;
                 end;

  {* NOW DEFINE THE FUNCTIONS *}


implementation

uses
  cDist_W,
  cDist_VE,
  cDist_VK,
  cDist_JA,
  cDist_ZL,
  cColors,
  DebugRtns,
  TpMouse,
  TpString,
  TpWindow,
  TpCrt,
  cMsgWin,
  cCountry,
  c_Contest,
  Routines,
  IniFile,
  LowLvl,
  OneTimers;


  constructor TDistList.Init;
  var
    I : Integer;
    Mult : Mult_Rec;
    Band : BandType;
  begin
    for Band := B80 to TOT do
      Mult.Band[Band] := False;
    for I := 0 to 9 do
      Dist_Tbl[I] := Mult;
    {$IFDEF DEBUG}
    AppendDebugMessage('DistList created');
    {$ENDIF}
  end;

  Destructor  TDistList.Done;
  begin
    {$IFDEF DEBUG}
    AppendDebugMessage('DistList destroyed');
    {$ENDIF}
  end;

  function    TDistList.GetDist(const D : Byte; const Band : BandType) : boolean;
  begin
    {$IFDEF DEBUG}
    if D > 9 then
      AppendDebugMessage('CTDistList.GetDist, invalid Dist# :'+long2str(D));
    {$ENDIF}
    if D > 9 then begin
      GetDist := False;
      Exit;
    end;
    GetDist := Dist_Tbl[D].Band[Band]
  end;

  procedure   TDistList.SetDist(const D : Byte; const Band : BandType);
  begin
    {$IFDEF DEBUG}
    if D > 9 then
      AppendDebugMessage('CTDistList.SetDist, invalid Dist# :'+long2str(D));
    {$ENDIF}
    if D > 9 then Exit;
    Dist_Tbl[D].Band[Band] := True;
    Dist_Tbl[D].Band[TOT]  := True;
  end;

  procedure   TDistList.SetDistByCall(const Call : String; const Band : BandType);
  begin
    SetDist(GetDistNo(Call),Band);
  end;

  procedure   TDistList.ReSetDist(const D : Byte; const Band : BandType);
  var
    b : bandType;
  begin
    {$IFDEF DEBUG}
    if D > 9 then
      AppendDebugMessage('CTDistList.ReSetDist, invalid Dist# :'+long2str(D));
    {$ENDIF}
    if D > 9 then Exit;
    Dist_Tbl[D].Band[Band] := False;
    Dist_Tbl[D].Band[TOT] := False;
    for B := B80 to B10 do begin
      if GetDist(D,B) then
        Dist_Tbl[D].Band[TOT]  := True;
    end;
  end;

  procedure   TDistList.ResetDistByCall(const Call : String; const Band : BandType);
  begin
    ResetDist(GetDistNo(Call),Band);
  end;

  function    TDistList.HowMany(const Band : BandType) : longint;
  var
    L       : longint;
    D      : Word;
  begin
    L := 0;
    for D := 0 to 9 do
      if Dist_Tbl[D].Band[Band] then Inc(L);
    HowMany := L;
    {$IFDEF DEBUG}
    AppendDebugMessage('TDistList.HowMany = '+Bands[Band]+'M '+long2str(l));
    {$ENDIF}
  end;

  function  TDistList.BandString(const D : Byte) : string;
  var
    s : string[5];
    Band : BandType;
  begin
    S := '.....';
    for Band := B80 to B10 do
      if GetDist(D,Band) then S[Ord(Band)+1] := '*';
    BandString := S;
  end;

  function TDistList.GetDistNo(const Call : String) : Byte;
  var
    I, District : Byte;
    Temp : String[12];
  begin
    District := 99;
    GetDistNo := 99;
    if Trim(Call) = '' then Exit;
    Temp := Trim(CountryList^.GetPrefixToId(Call));
    for I := 1 to Length(Temp) do
      if Temp[I] in ['0'..'9'] then District := Ord(Temp[I])-48;
    if District = 99 then
      for I := 1 to Length(Call) do
        if Call[I] in ['0'..'9'] then District := Ord(Call[I])-48;
    GetDistNo := District;
  end;

  function    TDistList.DistPrefix : string;
  begin
    DistPrefix := Trim(Code);
  end;

  procedure TDistList.Print;
  var
    Str            : String[80];
    Row            : Byte;
    PrintFile      : Text;
  begin
    Assign(PrintFile, FileName('DIS'));
    {$I-} Rewrite(PrintFile); {$I+}
    MsgWin^.Display('Preparing: '+FileName('DIS'));
    Str := 'Districts Multiplier sheet for '+RttyIni^.GetProfileString('Call','Profile','');
    Str := Pad(Str, 70)+'Page 1';
    Writeln(PrintFile,Str);
    Writeln(PrintFile,'');
    case Contest of
      HSSC,
      ANAR,
      BART,
      BSPR,
      SART,
      JART : begin
               Writeln(PrintFile,'    USA       CAN       AUS       JAP');
               Writeln(PrintFile,'    ---       ---       ---       ---');
               for Row := 0 to 9 do begin
                 Str := ' '+Long2Str(Row)+'|';
                 Str := Str+WDistList^.BandString(Row);
                 Str := Str+'|  '+Long2Str(Row)+'|';
                 Str := Str+VEDistList^.BandString(Row);
                 Str := Str+'|  '+Long2Str(Row)+'|';
                 Str := Str+VKDistList^.BandString(Row);
                 Str := Str+'|  '+Long2Str(Row)+'|';
                 Str := Str+JADistList^.BandString(Row);
                 Str := Str+'|';
                 Writeln(PrintFile,Str);
               end;
             end;
      VOLT : begin
               Writeln(PrintFile,'    USA       CAN       AUS       JAP       ZL ');
               Writeln(PrintFile,'    ---       ---       ---       ---       ---');
               for Row := 0 to 9 do begin
                 Str := ' '+Long2Str(Row)+'|';
                 Str := Str+WDistList^.BandString(Row);
                 Str := Str+'|  '+Long2Str(Row)+'|';
                 Str := Str+VEDistList^.BandString(Row);
                 Str := Str+'|  '+Long2Str(Row)+'|';
                 Str := Str+VKDistList^.BandString(Row);
                 Str := Str+'|  '+Long2Str(Row)+'|';
                 Str := Str+JADistList^.BandString(Row);
                 if (Row > 0) and (Row < 5) then begin
                   Str := Str+'|  '+Long2Str(Row)+'|';
                   Str := Str+ZLDistList^.BandString(Row);
                 end;
                 Str := Str+'|';
                 Writeln(PrintFile,Str);
               end;
             end;
    end;
    Writeln(PrintFile,^L);
    Close(PrintFile);
  end;

  procedure TDistList.Display;

  procedure DispDistVolta;
  var
    DistWin        : WindowPtr;
    Str            : String[55];
    Row            : Byte;
    Band : BandType;
  begin
    with Colors^ do begin
    if not MakeWindow(DistWin, 3, 3, 55, 17, True, True, True,
                      WindowAttr, FrameAttr, HeaderAttr,
                      'Districts') then Exit;
    HideMouse;
    DisplayWindow(DistWin);
    FastWriteWindow('USA', 2, 5, WindowAttr);
    FastWriteWindow('CAN', 2, 15, WindowAttr);
    FastWriteWindow('AUS', 2, 25, WindowAttr);
    FastWriteWindow('JAP', 2, 35, WindowAttr);
    FastWriteWindow('ZL ', 2, 45, WindowAttr);
    for Row := 0 to 9 do
    begin
      FastWriteWindow(Long2Str(Row)+'|', Row+3, 2, WindowAttr);
      Str := WDistList^.BandString(Row);
      Str := Str+'|  '+Long2Str(Row)+'|';
      Str := Str+VEDistList^.BandString(Row);
      Str := Str+'|  '+Long2Str(Row)+'|';
      Str := Str+VKDistList^.BandString(Row);
      Str := Str+'|  '+Long2Str(Row)+'|';
      Str := Str+JADistList^.BandString(Row);
      if (Row > 0) and (Row < 5) then begin
        Str := Str+'|  '+Long2Str(Row)+'|';
        Str := Str+ZLDistList^.BandString(Row);
      end;
      FastWriteWindow(Str+'|', Row+3, 4, WindowAttr);
    end;
    FastWrite('Press Any Key to continue', 17, 4, WindowAttr);
    end;
    ShowMouse;
    ReadKeyorButton;
    KillWindow(DistWin);
  end;

  procedure DispDistSartAndJartAndAnarAndBart; {v2.5a}
  var
    DistWin        : WindowPtr;
    Str            : String[35];
    Row            : Byte;
    Band           : BandType;
  begin
    with Colors^ do begin
    if not MakeWindow(DistWin, 3, 3, 45, 17, True, True, True,
                      WindowAttr, FrameAttr, HeaderAttr,
                      'Districts') then Exit;
    HideMouse;
    DisplayWindow(DistWin);
    FastWriteWindow('USA', 2, 5, WindowAttr);
    FastWriteWindow('CAN', 2, 15, WindowAttr);
    FastWriteWindow('AUS', 2, 25, WindowAttr);
    FastWriteWindow('JAP', 2, 35, WindowAttr);
    for Row := 0 to 9 do
    begin
      FastWriteWindow(Long2Str(Row)+'|', Row+3, 2, WindowAttr);
      Str := WDistList^.BandString(Row);
      Str := Str+'|  '+Long2Str(Row)+'|';
      Str := Str+VEDistList^.BandString(Row);
      Str := Str+'|  '+Long2Str(Row)+'|';
      Str := Str+VKDistList^.BandString(Row);
      Str := Str+'|  '+Long2Str(Row)+'|';
      Str := Str+JADistList^.BandString(Row);
      FastWriteWindow(Str+'|', Row+3, 4, WindowAttr);
    end;
    FastWrite('Press Any Key to continue', 17, 4, WindowAttr);
    end;
    ShowMouse;
    ReadKeyorButton;
    KillWindow(DistWin);
  end;

  begin
    case contest of {disp districts}
      VOLT : DispDistVolta;
      HSSC, SART, BART, BSPR, JART, ANAR : DispDistSartAndJartAndAnarAndBart;
    end;
  end;


  function TDistList.Load(const Cl : string; const Band : BandType) : Boolean;
  var
    Dist : byte;
  begin
    Dist := GetDistNo(Cl);
    if Dist > 9 then begin
      Load := False;
      Exit;
    end;
    Load := not GetDist(Dist,Band);
    SetDist(Dist,Band);
    SetDist(Dist,TOT);
  end;




end.
