{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit cDSP232;
interface
uses
  tncobject,
  TpCrt,
  TpString,
  TpEntry,
  TpWindow,
  TpMouse,
  DebugRtns,
  oproot,
  ApTimer,
  ApPort;
type
  PDSP232 = ^TDSP232;
  TDSP232 = object(TTnc)
             CHSwitch : Byte;
             Threshold : Byte;
             InChar_Queue : StaticQueue;
             UtChar_Queue : StaticQueue;
             UtTimer : EventTimer;
             UtTimerTics : Integer;
             constructor Init(const Port : ComNameType);
             procedure Setup;
             procedure SendByte(const Data : Byte); virtual;
             procedure SendChar(const Data : Char); virtual;
             procedure SendString(const Data : String); virtual;
             procedure SendCommand(const Command : String); virtual;
             procedure Transmit(const Xmit : Boolean); virtual;
             procedure Receive(var Ch : Char); virtual;
             procedure SetUsos(const USOS : Boolean); virtual;
             procedure SetBaudRate(const Rate : Byte); virtual;
             procedure SetSquelchLevel(const SquelchLevel : Byte); virtual;
             procedure SetMode(const Mode : String); virtual;
             procedure Letters; virtual;
             procedure ClearXmitBuffer; virtual;
             procedure StopEverything; virtual;
             procedure SetSense(const Sense : Boolean); virtual;
             procedure SetTxSense(const Sense : Boolean); virtual;
             procedure SetModem(const Modem : Byte); virtual;
             procedure SetDiddles(const Diddles : Boolean); virtual;
             destructor Done; virtual;
             procedure ToggleTncTalk; virtual;
           end;

implementation
uses
  OneTimers,
  Keys,
  Lowlvl,
  IniFile,
  RecTypes,
  cRttyWin,
  cMsgWin,
  cColors,
  Routines;

const
  cCOM = #$03;

  procedure IncChoice(var Value; FieldID : Byte;
    Factor : Integer;
    var St : String); far;
  var
    Switch   : Boolean absolute Value;
    Polarity : Boolean absolute Value;
  begin
    case FieldID of
      000, 004 : begin
                   if Factor <> 0 then Switch := not Switch;
                   St := OnOff[Switch];
                 end;
      001,002 : begin
              if Factor <> 0 then Polarity := not Polarity;
              St := PolarityChoices[Polarity];
            end;
    end;
  end;

  procedure TDSP232.Setup;
  var
    Esr : EsRecord;
    SetupPtr : WindowPtr;
    ExitCommand : EStype;
    aUsos : Boolean;
    aLevel : Boolean;
    aTxLevel : Boolean;
    aSpeed : Byte;
    aDiddles : Boolean;
    aModem : Byte;
    aSquelch : byte;
  begin
    aUsos := GetUsos;
    aLevel := GetSense;
    aTxLevel := GetTxSense;
    aSpeed := GetBaudRate;
    aDiddles := GetDiddles;
    aModem := GetModem;
    aSquelch := GetSquelchLevel;
    with Colors^ do
    if not MakeWindow(SetupPtr, 2, 2, 40, 12, True, True, False, WindowAttr,
      FrameAttr, HeaderAttr, 'DSP-232 Setup Menu') then Exit;
    HideMouse;
    DisplayWindow(SetupPtr);
    InitEsRecord(Esr);
    Colors^.SetEntryAttributes;
    SetWrapMode(Esr, WrapAtEdges);
    AddChoiceField(Esr, 'UnShift On Space', 4, 3, 'XXX',
      4, 24, 0, 1, @IncChoice, aUsos);
    AddChoiceField(Esr, 'RX Polarity', 5, 3, 'XXXXXXX',
      5, 24, 0, 1, @IncChoice, aLevel);
    AddChoiceField(Esr, 'TX Polarity', 6, 3, 'XXXXXXX',
      6, 24, 0, 1, @IncChoice, aTxLevel);
    AddByteField(Esr, 'Speed',  7, 3, 'XXX',
      7 , 24, 0, 45, 100, aSpeed);
    AddChoiceField(Esr, 'Diddles', 8, 3, 'XXX',
      8, 24, 0, 1, @IncChoice, aDiddles);
    AddByteField(Esr,   'Modem', 9, 3, '99',
      9, 24, 0, 1, 21, aModem);
    AddByteField(Esr,   'Threshold', 10, 3, '999',
      10, 24, 0, 1, 100, aSquelch);
    CtrlEnter(12, 4);
    AddCmd(ESEnd, kbEnd);
    AddCmd(ESHome, kbHome);
    AddCmd(EsQuit, kbEsc);
    repeat
      ExitCommand := EditScreen(Esr, 0, False);
    until ExitCommand in [EsQuit, EsDone];
    AddCmd(ESmy25, kbEnd);
    AddCmd(ESmy23, kbHome);
    AddCmd(Esmy49, kbEsc);
    KillWindow(SetupPtr);
    DisposeEditScreen(Esr);
    if ExitCOmmand = EsDone then begin
      {set new values}
      SetUsos(aUsos);
      SetSense(aLevel);
      SetTxSense(aTxLevel);
      SetBaudRate(aSpeed);
      SetDiddles(aDiddles);
      SetModem(aModem);
      SetSquelchLevel(aSquelch);
    end;
  end;

  procedure TDSP232.SetSense(const Sense : Boolean);
  begin
    inherited SetSense(Sense);
    case Sense of
      True : SendCommand('RXN');
      False : SendCommand('RXY');
    end;
  end;


  procedure TDSP232.SetTxSense(const Sense : Boolean);
  begin
    inherited SetTxSense(Sense);
    case Sense of
      True  : SendCommand('TXN');
      False : SendCommand('TXY');
    end;
  end;

  procedure TDSP232.SetModem(const Modem : Byte);
  begin
    inherited SetModem(Modem);
    SendCommand('IW'+Chr(Modem));
  end;

  procedure TDSP232.SetSquelchLevel(const SquelchLevel : Byte);
  begin
    inherited SetSquelchLevel(SquelchLevel);
  end;

  procedure TDSP232.SendByte(const Data : Byte);
  begin
    SendString(Chr(Data));
  end;

  procedure TDSP232.SendChar(const Data : Char);
  begin
    SendString(Data);
  end;

  procedure TDSP232.SendString(const Data : String);
  var
    i : integer;
    Ch : Char;
  begin
    for I := 1 to Length(Data) do begin
      Ch := Data[i];
      UtChar_Queue.PushTail(Ch);
    end;
  end;

  procedure TDSP232.SendCommand(const Command : String);
  begin
    inherited SendString(cSOH+'O'+Command+cETB);
  end;

  procedure TDSP232.Transmit(const Xmit : Boolean);
  var
   Ch : char;
  begin
    case Xmit of
      True : begin
               SendCommand('TC');
               Delay(100);
               SendCommand('XM');
             end;
      False : begin
               SendChar(^D);
             end;

    end;
  end;

  procedure TDSP232.Receive(var Ch : Char);

    procedure SendStringToQueue(const S : String);
    var
      B : Byte;
      C : Char;
    begin
      if (S[1] <> ^A) or (S[Length(S)] <> ^W) then Exit;
      for B := 3 to Length(S)-1 do begin
        C := S[B];
        InChar_Queue.PushTail(C);
      end;
    end;

  var
    B : Byte;
    S : String[255];
    s1 : string[255];
    EntireBlock : Boolean;
    C : Char;
    RecdLen : word;
  begin

    { see if there is any TXDATA to be sent, if so, send it!}
    if TimerExpired(UtTimer) then begin
      if UtChar_Queue.Elements > 0 then begin
        UtChar_Queue.PopHead(C);     {if queue empty, ch will be $00}
        if UtChar_Queue.GetStatus = 0 then begin
          inherited SendString(cSOH+' '+C+cETB);
        end;
      end;
      NewTimer(UtTimer,UtTimerTics);
    end;

    if UP^.BlockReady(4, [^W]) then begin
      FillChar(S, SizeOf(S), ' ');
      S[0] := #00;  { Set length byte }

      WaitForBlock(S, 1, EntireBlock); {wait just .1 for a char}
      {Up^.GetBlock(S, 4, RecdLen, [^W]);}

      s1 := '';
      for b := 1 to length(S) do
        s1 := s1 + Hexb(ord(s[B])) + ' ' ;
      {AppendDebugMessage('Block=>'+S+'<=  Hex=>'+S1+'<=  RecdLen='+long2str(recdlen)); }

      case Ord(S[2]) of
        $30 : begin
                C := ^z;
                InChar_Queue.PushTail(C);
                SendStringToQueue(S);
              end;
        $2F : begin
                C := ^y;
                InChar_Queue.PushTail(C);
                SendStringToQueue(S);
              end;
{      else begin
        AppendDebugMessage('Type='+Hexb(ord(S[2]))+' Data='+S1);
        AppendDebugMessage(S);
        end; }
      end;
    end;
    InChar_Queue.PopHead(Ch);     {if queue empty, ch will be $00}
  end;

  procedure TDSP232.SetUsos(const USOS : Boolean);
  begin
    inherited SetUsos(USOS);
    case USOS of
      True : SendCommand('USY');
      False : SendCommand('USN');
    end;
  end;

  procedure TDSP232.SetDiddles(const Diddles : Boolean);
  begin
    inherited SetDiddles(Diddles);
    case diddles of
      True  : SendCOmmand('DDY');
      False : SendCOmmand('DDN');
    end;
  end;

  procedure TDSP232.Letters;
  begin
    SendCommand('LO');
  end;

  procedure TDSP232.ClearXmitBuffer;
  begin
    UtChar_Queue.Clear;
    SendCommand('TC');
    Delay(200);
  end;

  procedure TDSP232.SetMode(const Mode : String);
  begin
    inherited SetMode(Mode);
    if Mode = 'RTTY' then SendCommand('BA');
  end;

  procedure TDSP232.SetBaudRate(const Rate : Byte);
  begin
    inherited SetBaudRate(Rate);
    case Rate of
      74 : SendCommand('RB75');
    else
      SendCommand('RB'+trim(long2str(Rate)));
    end;
  end;

  procedure TDSP232.StopEverything;
  begin
    ClearXmitBuffer;
    SendCommand('RC');
  end;


  constructor TDSP232.Init(const Port : ComNameType);
  var
    Ch : Char;
    S : String;
    B : Byte;
    EntireString : Boolean;
    EntireBlock : Boolean;
  const
    CmdMode = 1;
    Unknown = 0;

    function GetToCmd : Boolean;  {3.3.05}
      var
        S : String;
    begin
      WaitForString(255, S, 3, EntireString);
      {$IFDEF DEBUG}
      AppendDebugMessage('GetToCmd1 S='+S);
      {$ENDIF}
      WaitForString(255, S, 3, EntireString);
      {$IFDEF DEBUG}
      AppendDebugMessage('GetToCmd2 S='+S);
      {$ENDIF}
      WaitForString(255, S, 3, EntireString);
      {$IFDEF DEBUG}
      AppendDebugMessage('GetToCmd3 S='+S);
      {$ENDIF}
      WaitForString(255, S, 3, EntireString);
      {$IFDEF DEBUG}
      AppendDebugMessage('GetToCmd4 S='+S);
      {$ENDIF}
      WaitForString(255, S, 3, EntireString);
      {$IFDEF DEBUG}
      AppendDebugMessage('GetToCmd5 S='+S);
      {$ENDIF}
      UP^.PutString('*');
      WaitForString(255, S, 3, EntireString);
      {$IFDEF DEBUG}
      AppendDebugMessage('GetToCmd6 S='+S);
      {$ENDIF}
      WaitForString(255, S, 3, EntireString);
      {$IFDEF DEBUG}
      AppendDebugMessage('GetToCmd7 S='+S);
      {$ENDIF}
      WaitForString(255, S, 3, EntireString);
      {$IFDEF DEBUG}
      AppendDebugMessage('GetToCmd8 S='+S);
      {$ENDIF}
      WaitForString(255, S, 3, EntireString);
      {$IFDEF DEBUG}
      AppendDebugMessage('GetToCmd9 S='+S);
      {$ENDIF}
      WaitForString(255, S, 3, EntireString);
      {$IFDEF DEBUG}
      AppendDebugMessage('GetToCmd10 S='+S);
      {$ENDIF}
    end;

    procedure EnterHostMode;
    begin
      {$IFDEF DEBUG}
      AppendDebugMessage('Going to enter host mode');
      {$ENDIF}
      UP^.PutString(^X);
      UP^.PutString(^C);
      UP^.PutString(^M);
      UP^.PutString('EXPERT ON'^M);
      UP^.PutString('AWLEN 8'^M);
      UP^.PutString('PARITY 0'^M);
      UP^.Putstring('RESTART'^M);
      Delay(300);
      UP^.PutString('CASEDISP 2'^M);
      UP^.PutString('FLOW OFF'^M);
      UP^.PutString('8BITCONV ON'^M);
      UP^.PutString('THRESHLD 5'^M);
      Delay(300);
      WaitForString(255, S, 4, EntireString);
      while S <> '' do begin
        RttyWin^.PutString('.');
        {$IFDEF DEBUG}
        AppendDebugMessage('EnterHostMode S='+S);
        {$ENDIF}
        WaitForString(255, S, 4, EntireString);
      end;
      UP^.PutString('HOST ON'^M);

      { now make the software match the changes above!}
      UP^.ChangeDataBits(8);
      UP^.ChangeParity(NoParity);
    end;

  const
    QueueSize = 1024;
  var
    TestResult : Integer;
    I : Integer;
  begin
    inherited InitCom(Port);
    TncNum := bDSP232;
    RttyWin^.PutString(^M'Initializing DSP-232');
    UtTimerTics := RttyIni^.GetProfileInt('TxDelayTics', 'TNC1', 5);
    GetToCmd;
    EnterHostMode;
    RttyWin^.PutString('OK!');
    if not InChar_Queue.Init(QueueSize, SizeOf(Char), True) then begin
      WriteLn('Unable to allocate input queue, program halted.');
      Halt;
    end;
    if not UtChar_Queue.Init(QueueSize, SizeOf(Char), True) then begin
      WriteLn('Unable to allocate output queue, program halted.');
      Halt;
    end;
    UP^.PutString(cSOH+'OHPN'+cETB);  {hpoll off}
    SetMode(GetMode);
    UP^.PutString(cSOH+'ORB45'+cETB); {set baudot 45 }
    UP^.PutString(cSOH+'OTC'+cETB); {tclear the buffer}
    UP^.PutString(cSOH+'OEAY'+cETB); {echo as sent ON}
    UP^.PutString(cSOH+'OECY'+cETB); {echo ON}
    UP^.PutString(cSOH+'OCRY'+cETB); {make eol cr/cr/lf}
    UP^.PutString(cSOH+'ORC'+cETB); {force receive}

    SetUsos(GetUsos);
    SetSense(GetSense);
    SetTxSense(GetTxSense);
    SetBaudRate(GetBaudRate);
    SetDiddles(GetDiddles);
    SetModem(GetModem);

    if not FastStart then Setup;

    for I := 1 to 20 do           {read each block for above commands}
      begin
      WaitForBlock(S, 0, EntireBlock);
      end;
    NewTimer(UtTimer,UtTimerTics);
    {$IFDEF DEBUG}
    AppendDebugMessage('DSP232 created');
    {$ENDIF}
  end;

  destructor TDSP232.Done;
  var
    s : string;
    EntireString : Boolean;
  begin
    UP^.PutString(cSOH+'OHON'+cETB);
    delay(250);
    UP^.PutString('AWLEN 7'^M);
    UP^.PutString('PARITY 3'^M);
    UP^.Putstring('RESTART'^M);
    WaitForString(255, S, 3, EntireString);
    WaitForString(255, S, 3, EntireString);
    WaitForString(255, S, 3, EntireString);
    WaitForString(255, S, 3, EntireString);
    WaitForString(255, S, 3, EntireString);
    WaitForString(255, S, 3, EntireString);
    InChar_Queue.Done;
    UtChar_Queue.Done;
    inherited Done;
    {$IFDEF DEBUG}
    AppendDebugMessage('DSP232 destroyed');
    {$ENDIF}
  end;

  procedure TDSP232.ToggleTncTalk;
  begin
    Setup;
  end;

end.
