{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit cDupeTbl;

interface

uses
  Diskio,
  TpDate,
  TpRArray,
  TpWindow,
  Rectypes;

type
  TDupe_Rec = record
                Call : String[12];
                QTH : String[3];
                Point : Byte;
                qtime : array[BandType] of Time;
              end;

  PDupeTbl = ^TDupeTbl;
  TDupeTbl = object
             private
               Dupe_Tbl : TParray;
               DR : TDupe_Rec;
             public
               constructor Init;
               destructor Done;
               procedure ClearAll;
               procedure PrintDupeReport;
               procedure PartialCallSearch;
               procedure SuperPartialCallSearch;
               procedure ResetDupeByCall(const Cl : String; const Bd : BandType);
               procedure ResetDupe(const Sl : Word; const Bd : BandType);
               function GetDupeByCall(const Cl : String; var Slot : Word) : Boolean;
               function GetCall : String;
               function GetQth : String;
               function GetYear : Word;
               function GetPoint : Byte;
               function GetTime(const Bd : BandType) : Time;
               function GetMax : Word;
               function Load(const R : log_rec) : Boolean;
               function IsDupe(const Cl : String; const Bd : BandType) : Boolean;
               function BandString(const Slot : Word) : String;
               procedure DisplayCallSignInfo;
               function IsItGoodCall(const Search : String) : Boolean;
             private
               procedure GetDupe(const RowNumber : Word; var aDR : TDupe_Rec);
               procedure SetDupe(const RowNumber : Word; const aDR : TDupe_Rec);
             end;

var
  DupeTbl : PDupeTbl;

procedure MakeDupeTbl;
procedure KillDupeTbl;

implementation

uses
  IniFile,
  TpDos,
  TpPick,
  TpCrt,
  TpSort,
  TpString,
  Routines,
  LowLvl,
  cEditWin,
  TpMouse,
  cMaster,
  cMsgWin,
  cColors,
  DebugRtns,
  c_Contest,
  cCountry,
  OneTimers;

  function Hash(const Call : String) : Word;
  var
    C : String[12];
    Len : Integer;
    Temp : LongInt;
  begin
    { 61 83 43 }
    { 171 833 437 }
    C := Trim(Call);
    Len := Length(C);
    if Len < 4 then Hash := 0
    else begin
      Temp := (Ord(C[Len])*261);
      Temp := Temp+(Ord(C[Len-1])*183);
      Temp := Temp+(Ord(C[Len-2])*43);
      while Temp >= DupeTbl^.GetMax do
        Temp := Temp-DupeTbl^.GetMax+1;
      Hash := Temp;
    end;
  end;

  constructor TDupeTbl.Init;
  begin
    MakeA(Dupe_Tbl, GetMax, 1, SizeOf(TDupe_Rec));
    if ErrorA > 0 then Terminate(103);
    ClearAll;
    {$IFDEF DEBUG}
    AppendDebugMessage('DupeTbl created');
    {$ENDIF}
  end;

  destructor TDupeTbl.Done;
  begin
    DisposeA(Dupe_Tbl);
  {$IFDEF DEBUG}
  AppendDebugMessage('DupeTbl destroyed');
  {$ENDIF}
  end;

  procedure TDupeTbl.ClearAll;
  var
    Band : BandType;
    Slot : Word;
  begin
    DR.Call := '';
    DR.QTH := '';
    DR.Point := 0;
    for Band := B80 to B10 do DR.qtime[Band] := BadTime;
    for Slot := 0 to GetMax-1 do SetDupe(Slot, DR);
  end;

  procedure DupeInpRecs; far;
  var
    X : Word;
    DR : TDupe_Rec;
  begin
    for X := 0 to DupeTbl^.GetMax-1 do begin
      DupeTbl^.GetDupe(X, DR);
      PutElement(DR)
    end;
    DupeTbl^.ClearAll;
  end;

  function DupeLessRec(var R1, R2) : Boolean; far;
  var
    X1 : TDupe_Rec absolute R1;
    X2 : TDupe_Rec absolute R2;
  begin
    DupeLessRec := X1.Call < X2.Call;
  end;

  procedure DupeOutPrecs; far;
  var
    X : Word;
    DR : TDupe_Rec;
  begin
    X := 0;
    while GetElement(DR) do begin
      DupeTbl^.SetDupe(X, DR);
      Inc(X);
    end;
  end;

  procedure TDupeTbl.PrintDupeReport;
  var
    Row, Col : Word;
    PageCt : Byte;
    SortStat : SortStatus;
    Band : BandType;
    Call : String[12];
    Page : TParray;
    X : Word;
    PrntDupeRec : TDupe_Rec;
    PrintFile : Text;

    procedure Print_Page(PageCt : Byte);
    var
      TotThisPage : Word;
      Row, Col : Word;
      LineOut : String[100];
    begin
      TotThisPage := 0;
      LineOut := ' '+BandS[Band]+' Meter DUPE SHEET';
      LineOut := Pad(LineOut, 70)+'Page '+Long2Str(PageCt);
      WriteLn(PrintFile, LineOut);
      WriteLn(PrintFile, '');
      for Row := 0 to 49 do begin
        LineOut := ' ';
        for Col := 0 to 5 do begin
          RetA(Page, Row, Col, Call);
          if Trim(Call) <> '' then begin
            Inc(TotThisPage);
            LineOut := LineOut+Pad(Call, 13);
          end;
        end;
        WriteLn(PrintFile, LineOut);
      end;
      WriteLn(PrintFile, ' Total this page '+Long2Str(TotThisPage));
      Write(PrintFile, ^L);
      Call := '';
      ClearA(Page, Call, ExactInit);
    end;

  begin
    if Contest = SPRI then Exit;
    Assign(PrintFile, FileName('DUP'));
    {$I-} Rewrite(PrintFile); {$I+}
    MsgWin^.Display('Preparing: '+FileName('DUP'));
    MakeA(Page, 50, 6, SizeOf(Call));
    Call := '';
    ClearA(Page, Call, ExactInit);
    StoreA(Dupe_Tbl, 'hold.$$$');
    SortStat := TpSort.Sort(GetMax, SizeOf(TDupe_Rec), @DupeInpRecs, @DupeLessRec, @DupeOutPrecs);
    case SortStat of
      sortoutofmemory : AppendDebugMessage('PrintDupes.Sort: out of memory');
      sorttoomanyelements : AppendDebugMessage('PrintDupes.Sort: too many elements');
    end;
    if SortStat <> SortSuccess then begin
      Close(PrintFile);
      DisposeA(Page);
      DisposeA(Dupe_Tbl);
      LoadA(Dupe_Tbl, 'hold.$$$');
      DeleteFile('hold.$$$');
      MsgWin^.Display('Unable to sort, not enough memory');
      Exit;
    end;
    PageCt := 0;
    for Band := B80 to B10 do begin
      Row := 0;
      Col := 0;
      for X := 0 to GetMax-1 do begin
        GetDupe(X, DR);           { sorted calls }
        if (DR.Call <> '') and (DR.qtime[Band] <> BadTime) then begin
          Seta(Page, Row, Col, DR.Call); { page }
          Inc(Row);
          if Row = 50 then begin
            Inc(Col);
            Row := 0;
          end;
          if Col = 6 then begin
            Inc(PageCt);
            Col := 0;
            Row := 0;
            Print_Page(PageCt);
            Call := '';
            ClearA(Page, Call, ExactInit);
          end;
        end;
      end;
      Inc(PageCt);
      Print_Page(PageCt);
    end;
    DisposeA(Page);
    DisposeA(Dupe_Tbl);
    LoadA(Dupe_Tbl, 'hold.$$$');
    DeleteFile('hold.$$$');
    Close(PrintFile);
  end;

var
  PartialArray : array[1..102] of String[12];

  function PartialCallsignAt(No : Word) : String; far;
  begin
    PartialCallsignAt := PartialArray[No];
    if not DupeTbl^.IsDupe(PartialArray[No], EditW^.EditRecs[1].Band) then AltPickAttr := True;
  end;

  procedure TDupeTbl.PartialCallSearch;
  var
    Choice : Word;
    HiNum : Word;
    HOrientType : PickOrientType;
    D : TDupe_Rec;
  begin
    if Trim(EditW^.EditRecs[1].Call) = '' then Exit;
    HOrientType := PickOrientation;
    for Choice := 1 to 102 do
      PartialArray[Choice] := '';
    Choice := 1;
    HiNum := 0;
    while (HiNum < 102) and (Choice < GetMax) do begin
      GetDupe(Choice, D);
      if (Pos(EditW^.EditRecs[1].Call, D.Call) > 0) then begin
        Inc(HiNum);
        PartialArray[HiNum] := D.Call;
      end;
      Inc(Choice);
    end;
    if HiNum = 0 then Exit;
    Choice := 1;
    Pickmatrix := 6;
    PickWindow(@PartialCallsignAt, HiNum, 1, 1, 80, 15,
      True, Colors^.PickColors, 'Similar callsigns this contest', Choice);
    if PickCmdNum = PKSSelect then EditW^.EditRecs[1].Call := PartialArray[Choice];
  end;

  procedure TDupeTbl.SuperPartialCallSearch;
  var
    Choice : Word;
    HiNum : Word;
    HOrientType : PickOrientType;
    D : TDupe_Rec;
    Search : String[12];
    S : String[12];
    NumHits : Word;
  begin
    Search := Trim(EditW^.EditRecs[1].Call);
    if Search = '' then Exit;
    HOrientType := PickOrientation;
    for Choice := 1 to 102 do
      PartialArray[Choice] := '';
    { get ones from THIS contest first !}
    Choice := 1;
    HiNum := 0;
    while (HiNum < 102) and (Choice < GetMax) do begin
      GetDupe(Choice, D);
      if (Pos(EditW^.EditRecs[1].Call, D.Call) > 0) then begin
        Inc(HiNum);
        PartialArray[HiNum] := '*'+Copy(D.Call, 1, 11);
      end;
      Inc(Choice);
    end;
    NumHits := 0;
    MastList^.RefreshHitList(Search);
    while (HiNum < 102) and (NumHits < MastList^.HitCount) do begin
      Inc(NumHits);
      Inc(HiNum);
      PartialArray[HiNum] := ' '+Copy(MastList^.GetHit(NumHits), 1, 11);
    end;
    if HiNum = 0 then Exit;
    Choice := 1;
    Pickmatrix := 6;
    PickWindow(@PartialCallsignAt, HiNum, 1, 1, 80, 15,
      True, Colors^.PickColors, 'Super Partial Check   ''*'' = worked this contest', Choice);
    if PickCmdNum = PKSSelect then begin
      EditW^.EditRecs[1].Call := PartialArray[Choice];
      if EditW^.EditRecs[1].Call[1] = '*' then EditW^.EditRecs[1].Call := Copy(EditW^.EditRecs[1].Call, 2, 12);
      EditW^.EditRecs[1].Call := Trim(EditW^.EditRecs[1].Call);
    end;
  end;

  function TDupeTbl.IsItGoodCall(const Search : String) : Boolean;
  var
    Slot : Word;
  begin
    if Search = '' then Exit;
    if GetDupeByCall(Search, Slot) then begin
      IsItGoodCall := True;
      Exit;
    end;
    if Assigned(MastList) then IsItGoodCall := MastList^.Exists(Search);
  end;

  procedure TDupeTbl.ResetDupeByCall(const Cl : String; const Bd : BandType);
  var
    Slot : Word;
  begin
    if GetDupeByCall(Cl, Slot) then begin
      GetDupe(Slot, DR);
      DR.qtime[Bd] := BadTime;
      SetDupe(Slot, DR);
    end;
  end;

  procedure TDupeTbl.ResetDupe(const Sl : Word; const Bd : BandType);
  begin
    GetDupe(Sl, DR);
    DR.qtime[Bd] := BadTime;
    SetDupe(Sl, DR);
  end;

  procedure TDupeTbl.GetDupe(const RowNumber : Word; var aDR : TDupe_Rec);
  begin
    RetA(Dupe_Tbl, RowNumber, 0, aDR);
    aDR.Call := Trim(aDR.Call);
  end;

  procedure TDupeTbl.SetDupe(const RowNumber : Word; const aDR : TDupe_Rec);
  var
    Temp : TDupe_Rec;
  begin
    Temp := aDR;
    Seta(Dupe_Tbl, RowNumber, 0, Temp);
  end;

  function TDupeTbl.GetDupeByCall(const Cl : String; var Slot : Word) : Boolean;
    { locates CALL in array and returns the slot number }
    { T= found call, F=not found call}
  begin
    Slot := Hash(Cl);
    GetDupe(Slot, DR);
    while ((not SameCall(DR.Call, Cl)) and (DR.Call <> '')) do begin
      Inc(Slot);
      if Slot >= GetMax then Slot := 1;
      GetDupe(Slot, DR);
    end;
    GetDupeByCall := DR.Call <> '';
  end;

  function TDupeTbl.GetCall : String;
  begin
    GetCall := DR.Call;
  end;

  function TDupeTbl.GetQth : String;
  begin
    GetQth := DR.QTH;
  end;

  function TDupeTbl.GetYear : Word;
  var
    Yr : word;
  begin
    if Str2Word(DR.QTH,Yr) then GetYear := Yr + 1900;
  end;

  function TDupeTbl.GetPoint : Byte;
  begin
    GetPoint := DR.Point;
  end;

  function TDupeTbl.GetTime(const Bd : BandType) : Time;
  begin
    GetTime := DR.qtime[Bd];
  end;

  function TDupeTbl.GetMax : Word;
  begin
    GetMax := RttyIni^.GetProfileWord('MaxQsos', 'PROFILE', 1500)
  end;


  function TDupeTbl.Load(const R : log_rec) : Boolean;
  var
    Slot : Word;
    Dupe : Boolean;
    HoldBand : BandType;
    Band : BandType;
    A : log_rec;
  begin
    if GetDupeByCall(R.Call, Slot) then
      Dupe := DR.qtime[R.Band] <> BadTime { if exists, is it dupe? }
    else begin
      Dupe := False;              { doesnt exists...}
      for Band := B80 to B10 do DR.qtime[Band] := BadTime;
    end;
    if Dupe then begin            { it was already in dupe table, no need to continue}
      Load := True;
      Exit;
    end;
    DR.qtime[R.Band] := R.ts;
    DR.Call := Trim(R.Call);
    case Contest of
      SCCC : DR.QTH := long2str(R.His_Serno-1900);
    else
      DR.QTH := R.His_Qth;
    end;
    case Contest of
      WPXC,
      VOLT : begin                { these 2 contests score diff on diff bands, so I use}
               A := R;
               HoldBand := A.Band; {20m to get a good point value}
               A.Band := B20;
               DR.Point := ThisContest^.PointValue(A);
               A.Band := HoldBand;
             end;
    else
      DR.Point := ThisContest^.PointValue(R);
    end;
    SetDupe(Slot, DR);
    Load := False;
  end;

  function TDupeTbl.IsDupe(const Cl : String; const Bd : BandType) : Boolean;
    { returns true if CALL was worked on BAND before and does nothing else}
  var
    Slot : Word;
  begin
    if not GetDupeByCall(Cl, Slot) then
      IsDupe := False
    else
      IsDupe := DR.qtime[Bd] <> BadTime; {was found, send back result if wrked on band}
  end;

  function TDupeTbl.BandString(const Slot : Word) : String;
  var
    S : String[5];
    Band : BandType;
  begin
    S := '.....';
    GetDupe(Slot, DR);
    for Band := B80 to B10 do
      if DR.qtime[Band] <> BadTime then S[Ord(Band)+1] := '*';
    BandString := S;
  end;

  procedure TDupeTbl.DisplayCallSignInfo;
  var
    CallWin : WindowPtr;
    WinL : Byte;
    Info : String[38];
    C : String[7];
    Band : BandType;
    Slot : Word;
  begin
    if Trim(EditW^.EditRecs[1].Call) = '' then Exit;
    Info := '';
    with CountryList^ do
      if CountryList^.Identify(EditW^.EditRecs[1]) <> NoCountry then
        Info := Ct.Code+': '+Ct.Name+' ';
    if GetDupeByCall(EditW^.EditRecs[1].Call, Slot) then
      C := BandString(Slot)
    else
      C := '.....';
    Info := Info+'|'+C+'|';
    WinL := Length(Info)+3;
    if WinL < 17 + length(EditW^.EditRecs[1].Call) then
      WinL := 17 + length(EditW^.EditRecs[1].Call);
    with Colors^ do
      if not MakeWindow(CallWin, 45, 3, 45+WinL, 7, True, True, True, WindowAttr,
        FrameAttr, HeaderAttr, 'Callsign Check: '+Trim(EditW^.EditRecs[1].Call)) then begin
        MsgWin^.Display('Not enough memory');
        Exit;
      end;
    HideMouse;
    DisplayWindow(CallWin);
    FastWriteWindow(Info, 2, 2, Colors^.WindowAttr);
    ShowMouse;
    ReadKeyorButton;
    KillWindow(CallWin);
  end;



  {-------------------------------------------------------------------------}
  procedure MakeDupeTbl;
  begin
    DupeTbl := nil;
    DupeTbl := New(PDupeTbl, Init);
  {$IFDEF DEBUG}
  if Assigned(DupeTbl) then AppendDebugMessage('DupeTbl allocated');
  {$endif}
  end;

  procedure KillDupeTbl;
  begin
    if Assigned(DupeTbl) then begin
    {$IFDEF DEBUG}
    AppendDebugMessage('DupeTbl freed');
    {$endif}
      Dispose(DupeTbl, Done);
    end;
    DupeTbl := nil;
  end;

end.
