{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit cDxp38;
interface
uses ApTimer,
  TncObject,
  RecTypes,
  ApPort;
const
  ControllerProgram = 'DXP38.S28';
  DspProgram = 'DXP38.LOD';
  FilterWidthString : array[0..2] of String[6] = ('Narrow', 'Medium', 'Wide');
  FilterWidthDefault = 0;


type
  PDxp38 = ^TDxp38;
  TDxp38 = object(TTnc)
               DisplayString1 : String[20];
               DisplayString2 : String[20];
               DisplayAttr1   : Byte;
               DisplayAttr2   : Byte;
               MarkCt         : array[1..255] of Integer;
               SpaceCt        : array[1..255] of Integer;
               MarkLoCt       : array[1..255] of Integer;
               SpaceHiCt      : array[1..255] of Integer;
               FreqAttr       : Byte;
               ZeroTuneRev    : Boolean;
               CollectNumbers : Boolean;
               MarkLo         : Byte;
               Mark           : Byte;
               Space          : Byte;
               SpaceHi        : Byte;
               Scale          : Byte;
               ProgramPath    : String[39];
               ExtraDelay     : LongInt;
               constructor Init(const Port : ComNameType);
               function InitializeBox : Boolean;
               destructor Done; virtual;
               function InputFifoFull : Boolean;
               function InputFifoEmpty : Boolean;
               function InputFifoHalf : Boolean;
               function InputFifoChCount : Word;

               function OutputFifoHasData : Boolean;
               procedure WaitOutputFifoData;

               function ReadOutputPort : Word;
               function ReadOutputPortWait : Word;
               function IsData(const w : Word) : Boolean;
               function DownloadDspProgram : Boolean;
               function DownloadControllerProgram : Boolean;
               function RoomInFifo : Boolean;
               procedure WaitRoomInFifo;
               procedure SendCommand(const w : Word);
               procedure DrainOutFifo(const Silent : Boolean);
               procedure Setup;
               procedure SendByte(const Data : Byte); virtual;
               procedure SendChar(const Data : Char); virtual;
               procedure Transmit(const Xmit : Boolean); virtual;
               procedure SendString(const  Data : String); virtual;
               procedure Receive(var Ch : Char); virtual;
               procedure SetUsos(const usos : Boolean); virtual;
               procedure SetDiddles(const diddles : Boolean); virtual;
               procedure SetBaudRate(const Rate : Byte); virtual;
               procedure SetMode(const Mode : String);virtual;
               procedure SetFilterWidth(const FilterWidth : Byte); virtual;
               procedure Letters; virtual;
               procedure Figures; virtual;
               procedure SetSense(const Sense : Boolean); virtual;
               procedure SetFskPolarity(const FskPolarity : boolean); virtual;
               procedure ClearXmitBuffer; virtual;
               procedure SetSquelchLevel(const SquelchLevel : Byte); virtual;
               procedure StopEverything; virtual;
               procedure SetDisplay(const Display : Byte); virtual;
               procedure SetDisplayScale(const DisplayScale : Byte); virtual;
               procedure SetMarkFreq(const MarkFreq : word); virtual;
               procedure SetSpaceFreq(const SpaceFreq : word); virtual;
               procedure SetBothFreqs;
               procedure UpdateDisplay;
               procedure UpdateFreqDelta;
               procedure GetDisplayData;
               procedure OneBarDisplay;
               procedure TwoBarDisplay;
               procedure FourBarDisplay;
               procedure ZeroCenterDisplay;
               procedure TwoBarTouchDisplay;
               function  ZeroCenterString : string;
               procedure ToggleTncTalk; virtual;
               procedure GetZero(var NumForZero : shortint; var DispAttr : byte); virtual;
               procedure UpdateFilterStrings(var S1, S2 : string; var A1, A2 : Byte); virtual;
             end;

implementation
uses
  TpCrt,
  TpDos,
  TpString,
  TpEntry,
  TpWindow,
  TpMouse,
  Routines,
  OneTimers,
  Keys,
  LowLvl,
  cRttyWin,
  OpInline,
  DebugRtns,
  cColors,
  IniTools,
  IniFile;


  procedure IncChoice(var Value; FieldID : Byte;
    Factor : Integer;
    var St : String); far;
  var
    Code : Boolean absolute Value;
    Switch : Boolean absolute Value;
    Polarity : Boolean absolute Value;
    Speed : Byte absolute Value;
    FilterWidth : Byte absolute Value;
    Display : Byte absolute Value;
    EOL : Byte absolute Value;
  begin
    case FieldID of
      001,
      002,
      005 : begin
              if Factor <> 0 then Switch := not Switch;
              St := OnOff[Switch];
            end;
      000,
      007 : begin
              if Factor <> 0 then Polarity := not Polarity;
              St := PolarityChoices[Polarity];
            end;
      003 : begin
              case Factor of
                 1 : if EOL < 3 then Inc(EOL) else EOL := 0;
                -1 : if EOL > 0 then Dec(EOL) else EOL := 3;
              end;
              St := EndOfLineChoices[EOL];
            end;
      008 : begin
              case Factor of
                 1 : if FilterWidth < 2 then Inc(FilterWidth) else FilterWidth := 0;
                -1 : if FilterWidth > 0 then Dec(FilterWidth) else FilterWidth := 2;
              end;
              St := FilterWidthString[FilterWidth];
            end;
      009 : begin
              case Factor of
                 1 : if Display < 5 then Inc(Display) else Display := 0;
                -1 : if Display > 0 then Dec(Display) else Display := 5;
              end;
              St := DisplayS[Display];
            end;
    end;
  end;

  procedure TDxp38.Setup;
  var
    Esr : EsRecord;
    SetupPtr : WindowPtr;
    ExitCommand : EStype;
    aPolarity : Boolean;
    aUSOS : Boolean;
    aDiddles : Boolean;
    aEndOfLine : byte;
    aPttDelayOn : word;
    aPttDelayOff : word;
    aSpeed : byte;
    aSquelchEnabled : boolean;
    aSquelchLevel : byte;
    aFskPolarity : boolean;
    aFilterWidth : byte;
    aDisplay : byte;
    aDisplayScale : byte;
  begin
    { Get current values }
    aPolarity          := GetSense;
    aUSOS           := GetUsos;
    aDiddles        := GetDiddles;
    aEndOfLine      := GetEol;
    aSpeed          := GetBaudRate;
    aSquelchEnabled := GetSquelchEnabled;
    aSquelchLevel   := GetSquelchLevel;
    aFskPolarity    := GetFskPolarity;
    aFilterWidth    := GetFilterWidth;
    aDisplay        := GetDisplay;
    aDisplayScale   := GetDisplayScale;
    with Colors^ do
      if not MakeWindow(SetupPtr, 2, 2, 40, 18, True, True, False, WindowAttr,
        FrameAttr, HeaderAttr, 'DXP38 Setup Menu') then Exit;
    HideMouse;
    DisplayWindow(SetupPtr);
    InitEsRecord(Esr);
    Colors^.SetEntryAttributes;
    SetWrapMode(Esr, WrapAtEdges);
{0} AddChoiceField(Esr, 'Polarity', 5, 3, 'XXXXXXX',
      5, 24, 0, 1, @IncChoice, aPolarity);
{1} AddChoiceField(Esr, 'UnShift On Space', 6, 3, 'XXX',
      6, 24, 0, 1, @IncChoice, aUSOS);
{2} AddChoiceField(Esr, 'Sync Idle', 7, 3, 'XXX',
      7, 24, 0, 1, @IncChoice, aDiddles);
{3} AddChoiceField(Esr, 'End of line', 8, 3, 'XXXXXXXXXX',
      8, 24, 0, 1, @IncChoice, aEndOfLine);
{4} AddByteField(Esr, 'Speed',  9, 3, 'XXX',
      9 , 24, 0, 45, 100, aSpeed);
{5} AddChoiceField(Esr, 'Print Squelch', 10, 3, 'XXX',
      10, 24, 0, 1, @IncChoice, aSquelchEnabled);
{6} AddByteField(Esr,   '        Level', 11, 3, '999',
      11, 24, 0, 0, 127, aSquelchLevel);
{7} AddChoiceField(Esr, 'FSK Polarity', 12, 3, 'XXXXXXX',
      12, 24, 0, 1, @IncChoice, aFSKPolarity);
{8} AddChoiceField(Esr, 'Filter Width', 13, 3, 'XXXXXX',
      13, 24, 0, 1, @IncChoice, aFilterWidth);
{9} AddChoiceField(Esr, 'Tuning Display', 14, 3, 'XXXXXXXXXXXXXX',
      14, 24, 0, 1, @IncChoice, adisplay);
{10}AddByteField(Esr, 'Display Scale', 15, 3, '999',
      15, 24, 0, 1, 200, aDisplayScale);
    CtrlEnter(18, 4);
    AddCmd(ESEnd, kbEnd);
    AddCmd(ESHome, kbHome);
    AddCmd(EsQuit, kbEsc);
    repeat
      ExitCommand := EditScreen(Esr, 0, False);
    until ExitCommand in [EsQuit, EsDone];
    AddCmd(ESmy25, kbEnd);
    AddCmd(ESmy23, kbHome);
    AddCmd(Esmy49, kbEsc);
    KillWindow(SetupPtr);
    DisposeEditScreen(Esr);
    if ExitCOmmand = EsDone then begin
      {set new values}
        SetSense(aPolarity);
        SetUsos(aUsos);
        SetDiddles(aDiddles);
        SetEol(aEndOfLine);
        SetBaudRate(aSpeed);
        SetSquelchEnabled(aSquelchEnabled);
        SetSquelchLevel(aSquelchLevel);
        SetFskPolarity(aFskPolarity);
        SetFilterWidth(aFilterWidth);
        SetDisplay(aDisplay);
        SetDisplayScale(aDisplayScale);
    end;
  end;

  procedure TDxp38.SendCommand(const w : Word);
  begin
    SendPortWord(w);
  end;


  function TDxp38.ReadOutputPortWait : Word;
  var
   et : EventTimer;
  begin
    { added the timer portion of this at hc8}
    newtimer(Et, 18);
    repeat until (TimerExpired(Et)) or (UP^.BlockReady(2,[]));
    ReadOutputPortWait := GetPortWord and $80FF
  end;

  function TDxp38.ReadOutputPort : Word;
  begin
    if UP^.BlockReady(2,[]) then
      ReadOutputPort := GetPortWord and $80FF
    else
      ReadOutputPort := $0000;
  end;

  function TDxp38.InputFifoChCount : Word;
  var
    W1, W2 : Word;
  begin
    SendCommand($807C);
    W1 := ReadOutputPortWait and $00FF;
    W2 := ReadOutputPortWait and $00FF;
    InputFifoChCount := (W1*256)+W2;
  end;

  function TDxp38.InputFifoEmpty : Boolean;
  begin
    InputFifoEmpty := InputFifoChCount = 0;
  end;

  function TDxp38.InputFifoHalf : Boolean;
  begin
    InputFifoHalf := (InputFifoChCount > 200) and (InputFifoChCount < 300);
  end;

  function TDxp38.InputFifoFull : Boolean;
  begin
    InputFifoFull := InputFifoChCount > 500;
  end;

  function TDxp38.RoomInFifo : Boolean;
  begin
    RoomInFifo := not InputFifoFull;
  end;

  procedure TDxp38.WaitRoomInFifo;
  var
    et : eventtimer;
  begin
    newtimer(et, 36);             {wait max 2 seconds}
    repeat until RoomInFifo or TimerExpired(et);
  end;

  function TDxp38.OutputFifoHasData : Boolean;
  begin
    OutputFifoHasData := UP^.BlockReady(2,[]);
  end;

  procedure TDxp38.WaitOutputFifoData;
  var
    et : eventtimer;
  begin
    newtimer(et, 18);
    repeat until OutputFifoHasData or TimerExpired(et);
  end;

  function TDxp38.IsData(const w : Word) : Boolean;
  begin
    IsData := not FlagIsSet(w, $8000);
  end;


  function TDxp38.DownloadDspProgram : Boolean;
  var
    F : file;
    w : Word;
    I : Integer;
    NumRead : Word;
    Buf : array[1..4096] of Byte;
  begin
    DownloadDspProgram := False;
    Assign(F, ProgramPath+DspProgram); {.lod}
    Reset(F, 1);
    SendCommand($8000);           {begin dsp program load}
    if ReadOutputPortWait <> $8000 then Exit;
    w := $0000;
    repeat
      BlockRead(F, Buf, SizeOf(Buf), NumRead);
      for I := 1 to NumRead do begin
        w := Buf[I];
        WaitRoomInFifo;
        SendCommand(w);
      end;
    until NumRead < SizeOf(Buf);
    Close(F);
    if ReadOutputPortWait <> $8002 then Exit;
    if ReadOutputPortWait = $800A then DownloadDspProgram := True;
  end;

  function TDxp38.DownloadControllerProgram : Boolean;
  var
    F : file;
    w : Word;
    I : Integer;
    NumRead : Word;
    Buf : array[1..4096] of Byte;
  begin
    DownloadControllerProgram := False;
    Assign(F, ProgramPath+ControllerProgram);
    Reset(F, 1);
    SendCommand($8001);
    if ReadOutputPortWait <> $8001 then Exit;
    w := $0000;
    repeat
      BlockRead(F, Buf, SizeOf(Buf), NumRead);
      for I := 1 to NumRead do begin
        w := Buf[I];
        WaitRoomInFifo;
        SendCommand(w);
      end;
    until NumRead < SizeOf(Buf);
    Close(F);
    if ReadOutputPortWait <> $8002 then Exit;
    if ReadOutputPortWait = $800A then DownloadControllerProgram := True;
  end;


  procedure TDxp38.DrainOutFifo(const Silent : Boolean);
  begin
    while OutputFifoHasData do
      case Silent of
        True : ReadOutputPort;
        False : RttyWin^.PutString('draining...'+hexw(ReadOutputPort));
      end;
  end;


  procedure TDxp38.SendByte(const Data : Byte);
  begin
    case Data of
      $0D : begin
              inherited SendByte($0D);
              inherited SendByte($0A);
            end;
    else
      inherited SendByte(Data);
    end;
  end;

  procedure TDxp38.SendChar(const Data : Char);
  begin
    case ord(Data) of
      $0D : begin
              inherited SendByte($0D);
              inherited SendByte($0A);
            end;
    else
      inherited SendChar(Data);
    end;
  end;


  procedure TDxp38.SendString(const Data : String);
  var
    I : Integer;
  begin
    if Length(Data) = 0 then Exit;
    for I := 1 to Length(Data) do
      SendChar(Data[I]);
  end;


  procedure TDxp38.Transmit(const Xmit : Boolean);
  begin
    case Xmit of
      True : SendCommand($800D);
      False : SendCommand($800E);
    end;
  end;

  procedure TDxp38.Receive(var Ch : Char);
  var
    w : word;
  begin
    Ch := #00;
    if Not  OutputFifoHasData then Exit;
    inherited Receive(Ch);
    if not(Ch in [#$80, #$81]) then Exit; {all set, it was data}
    inherited Receive(Ch);  {second part of command}
    case Ord(Ch) of
      $80 : ;                     {discard this, it was data that was a hex 80, useless in rtty}
      $81 : ;                     {discard this, it was data that was a hex 81, useless in rtty}
      $30 : RttyWin^.PutString(^Z);
      $31 : RttyWin^.PutString(^Y);
      $70 : begin
              GetDisplayData;     {get data}
              UpdateDisplay;      {update strings}
            end;
      $7E : begin
              UpdateFreqDelta;    {get data}
            end;
      $7A : case ReadOutputPortWait of
              $8007 : RttyWin^.PutString(^Z); {now rx}
              $8006 : RttyWin^.PutString(^Y); {now tx}
            end;
      $0D : RttyWin^.PutString(^Y);
      $0E : RttyWin^.PutString(^Z);
      $09 : InitializeBox;
      {$IFDEF DEBUG}
      $E8 : RttyWin^.PutString(^M'SQUELCH LEVEL SET');
      $EC : RttyWin^.PutString(^M'MARK/SPACE SET');
      $C3 : RttyWin^.PutString(^M'USOS OFF');
      $D3 : RttyWin^.PutString(^M'USOS ON');
      $C0 : RttyWin^.PutString(^M'CCITT CODE');
      $D0 : RttyWin^.PutString(^M'US CODE');
      $C1 : RttyWin^.PutString(^M'DIDDLES OFF');
      $D1 : RttyWin^.PutString(^M'DIDDLES ON');
      $8A : RttyWin^.PutString(^M'FORCED FIGS');
      $89 : RttyWin^.PutString(^M'FORCED LTRS');
      $E4 : RttyWin^.PutString(^M'SPEED SET');
      $EB : RttyWin^.PutString(^M'FILTER BW SET');
      $5B : RttyWin^.PutString(^M'NORMAL TONES');
      $4B : RttyWin^.PutString(^M'REVERSE TONES');
      $59 : RttyWin^.PutString(^M'ECHO AS SENT');
      $E3 : RttyWin^.PutString(^M'PTT on delay set to 200ms');
      $ED : RttyWin^.PutString(^M'PTT off delay set to 100ms');
      $06 : RttyWin^.PutString(^M'IMMEDIATE ABORT ACK');
      $DB : RttyWin^.PutString(^M'Normal FSK');
      $CB : RttyWin^.PutString(^M'Inverted FSK');
      else begin
        ch := #$00;
{        RttyWin^.PutString('Unknown Status: $'+hexb(ord(ch))+' '^G);}
      end;
      {$ENDIF}
    end;
    Ch := cNUL;
  end;

  procedure TDxp38.SetUSOS(const usos : Boolean);
  begin
    inherited SetUsos(usos);
    case usos of
      True : SendCommand($80D3);
      False : SendCommand($80C3);
    end;
  end;

  procedure TDxp38.Letters;
  begin
    SendCommand($8089);
  end;

  procedure TDxp38.Figures;
  begin
    SendCommand($808A);
  end;

  procedure TDxp38.SetDisplay(const Display : byte);
  begin
    inherited SetDisplay(Display);
  end;

  procedure TDxp38.SetSense(const Sense : boolean);
  begin
    inherited SetSense(Sense);
    case Sense of
      True: SendCommand($805B);
      False: SendCommand($804B);
    end;
  end;


  procedure TDxp38.SetFskPolarity(const FskPolarity : boolean);
  begin
    inherited SetFskPolarity(FskPolarity);
    case FskPolarity of
      True  : SendCommand($80DB); {fsk norm command}
      False : SendCommand($80CB); {fsk rev command}
    end;
  end;

  procedure TDxp38.ClearXmitBuffer;
  begin
    UP^.FlushOutBuffer;
  end;

  procedure TDxp38.SetMode(const Mode : string);
  var
    et : EventTimer;
    Feedback : Word;
    ok : boolean;
  begin
    inherited Setmode(mode);
    if Mode = 'RTTY' then begin
      SendCommand($8085);           {rtty}
      { added the timer portion of this at hc8}
      newtimer(Et, 23);
      repeat until (ReadOutputPortWait = $8085) OR (TimerExpired(Et));
      ok := true;
      if ReadOutputPortWait <> $807A then OK := false;
      if ReadOutputPortWait <> $8007 then OK := False;
      if ReadOutputPortWait <> $8030 then ok := false;
      if OK then RttyWin^.PutString(^M'RTTY mode enabled.')
            else AppendDebugMessage('Warning: No echo of RTTY mode for DXP38');
    end;
  end;

  procedure TDxp38.SetBaudRate(const Rate : Byte);
  begin
    inherited SetBaudRate(Rate);
    SendCommand($80E4);
    SendCommand($8000+SpeedToByte(Long2str(Rate)));
  end;

  procedure TDxp38.SetFilterWidth(const FilterWidth : Byte);
  begin
    inherited SetFilterWidth(FilterWidth);
    SendCommand($80EB);
    SendCommand($8000+FilterWidth);
  end;


  procedure TDxp38.SetDiddles(const diddles : Boolean);
  begin
    inherited SetDiddles(Diddles);
    case diddles of
      True  : SendCommand($80D1);
      False : SendCommand($80C1);
    end;
  end;


  procedure TDxp38.SetSquelchLevel(const SquelchLevel : Byte);
  begin
    inherited SetSquelchLevel(SquelchLevel);
    if GetSquelchEnabled then begin
      SendCommand($80E8); {set squelch}
      SendCommand($8000+SquelchLevel);   {to a some value}
    end
    else begin
      SendCommand($80E8); {set squelch}
      SendCommand($8000);                   {to no level at all}
    end;
  end;


  procedure TDxp38.SetDisplayScale(const DisplayScale : Byte);
  begin
    inherited SetDisplayScale(DisplayScale);
    Scale := DisplayScale;
  end;


  procedure  TDxp38.SetBothFreqs;
  var
    SpaceFreq : word;
    MarkFreq  : word;
    SpaceFLo, SpaceFHi : byte;
    MarkFLo, MarkFHi  : byte;
  begin
    SpaceFreq := GetSpaceFreq;
    MarkFreq := GetMarkFreq;
    SpaceFHi := Hi(SpaceFreq AND $FF00);
    SpaceFLo := Lo(SpaceFreq AND $00FF);
    MarkFHi := Hi(MarkFreq AND $FF00);
    MarkFLo := Lo(MarkFreq AND $00FF);
    SendCommand($80EC);
    SendCommand($8000+MarkFHi);
    SendCommand($8000+MarkFLo);
    SendCommand($8000+SpaceFHi);
    SendCommand($8000+SpaceFLo);
  end;

  procedure  TDxp38.SetMarkFreq(const MarkFreq : word);
  begin
    inherited SetMarkFreq(MarkFreq);
    SetBothFreqs;
  end;

  procedure  TDxp38.SetSpaceFreq(const SpaceFreq : word);
  begin
    inherited SetSpaceFreq(SpaceFreq);
    SetBothFreqs;
  end;



  procedure TDxp38.GetDisplayData;

    procedure OverMaxIntAlarm;
    begin
      Sound(500); delay(250);
      Sound(800); delay(250);
      Sound(500); delay(250);
      Sound(800); delay(250);
    end;

  begin
    MarkLo := Lo(ReadOutputPortWait);
    ReadOutputPortWait;
    Mark := Lo(ReadOutputPortWait);
    ReadOutputPortWait;
    Space := Lo(ReadOutputPortWait);
    ReadOutputPortWait;
    SpaceHi := Lo(ReadOutputPortWait);
    ReadOutputPortWait;

    if CollectNumbers then begin
      if MarkLoCt[MarkLo] < MaxInt then Inc(MarkLoCt[MarkLo])
      else OverMaxIntAlarm;
      if MarkCt[Mark] < MaxInt then Inc(MarkCt[Mark])
      else OverMaxIntAlarm;
      if SpaceCt[Space] < MaxInt then Inc(SpaceCt[Space])
      else OverMaxIntAlarm;
      if SpaceHiCt[SpaceHi] < MaxInt then Inc(SpaceHiCt[SpaceHi])
      else OverMaxIntAlarm;
    end;

    if MarkLo > Scale then Dec(MarkLo, Scale)
    else MarkLo := 0;
    if Mark > Scale then Dec(Mark, Scale)
    else Mark := 0;
    if Space > Scale then Dec(Space, Scale)
    else Space := 0;
    if SpaceHi > Scale then Dec(SpaceHi, Scale)
    else SpaceHi := 0;

  end;

  procedure TDxp38.FourBarDisplay;
  const
    SpCh = 223;
    MkCh = 220;
    BtCh = 219;
    DisplayIncr = 3;
  var
    Diff : Byte;
    s : String[3];
    MarkLoLeds : Byte;
    MarkLeds : Byte;
    SpaceLeds : Byte;
    SpaceHiLeds : Byte;
  begin

    MarkLoLeds := MarkLo div DisplayIncr;
    MarkLeds := Mark div DisplayIncr;
    SpaceLeds := Space div DisplayIncr;
    SpaceHiLeds := SpaceHi div DisplayIncr;

    Diff := Abs(MarkLeds-MarkLoLeds);

    if Diff > 19 then Diff := 19;

    if MarkLoLeds > MarkLeds then
      DisplayString1 := CharStr('', MarkLeds)+CharStr('', Diff)
    else
      DisplayString1 := CharStr('', MarkLoLeds)+CharStr('', Diff);

    Diff := Abs(SpaceHiLeds-SpaceLeds);

    if Diff > 19 then Diff := 19;

    if SpaceLeds > SpaceHiLeds then
      DisplayString2 := CharStr('', SpaceHiLeds)+CharStr('', Diff)
    else
      DisplayString2 := CharStr('', SpaceLeds)+CharStr('', Diff);

    DisplayAttr1 := Colors^.PromptAttr;
    DisplayAttr2 := Colors^.PromptAttr;
    DisplayString1 := Pad(DisplayString1, SizeOf(DisplayString1));
    DisplayString2 := Pad(DisplayString2, SizeOf(DisplayString2));
  end;




  procedure TDxp38.TwoBarDisplay;
  const
    DisplayIncr = 3;
  var
    MarkLeds : Integer;
    SpaceLeds : Integer;
  begin
    MarkLeds := Mark div DisplayIncr;
    SpaceLeds := Space div DisplayIncr;

    if MarkLeds > 39 then MarkLeds := 39;
    if SpaceLeds > 39 then SpaceLeds := 39;

    if Odd(MarkLeds) then {its an odd number if true}
      DisplayString1 := CharStr('', MarkLeds div 2)+''
    else
      DisplayString1 := CharStr('', MarkLeds div 2);

    if Odd(SpaceLeds) then {its an odd number if true}
      DisplayString2 := CharStr('', SpaceLeds div 2)+''
    else
      DisplayString2 := CharStr('', SpaceLeds div 2);

    DisplayString1 := Pad(DisplayString1, SizeOf(DisplayString1));
    DisplayString2 := Pad(DisplayString2, SizeOf(DisplayString2));
    DisplayAttr1 := Colors^.PromptAttr;
    DisplayAttr2 := Colors^.PromptAttr;
  end;

  procedure TDxp38.TwoBarTouchDisplay;
  const
    DisplayIncr = 3;
  var
    MarkLeds   : integer;
    SpaceLeds  : integer;
  begin
    MarkLeds  := Mark  div DisplayIncr;
    SpaceLeds := Space div DisplayIncr;

    if MarkLeds  > 39 then MarkLeds  := 39;
    if SpaceLeds > 39 then SpaceLeds := 39;

    if Odd(MarkLeds) then  {its an odd number if true}
      DisplayString1 := CharStr('',MarkLeds div 2)+''
    else
      DisplayString1 := CharStr('',MarkLeds div 2);

    if Odd(SpaceLeds) then {its an odd number if true}
      DisplayString2 := ''+CharStr('',SpaceLeds div 2)
    else
      DisplayString2 := CharStr('',SpaceLeds div 2);

    DisplayString2 := Pad(DisplayString1,10) + LeftPad(DisplayString2,10);
    DisplayString1 := ZeroCenterString;
    DisplayAttr1 :=FreqAttr;
    DisplayAttr2 := Colors^.PromptAttr;
  end;

  procedure TDxp38.OneBarDisplay;
  const
    DisplayIncr = 3;
  var
    Leds   : integer;
  begin
    Leds  := (Mark + Space) div DisplayIncr;

    if Leds  > 40 then Leds  := 40;

    if Odd(Leds) then  {its an odd number if true}
      DisplayString2 := CharStr('',Leds div 2)+''
    else
      DisplayString2 := CharStr('',Leds div 2);
    DisplayString1 := ZeroCenterString;
    DisplayString2 := Pad(DisplayString2,Sizeof(DisplayString2));
    DisplayAttr1 := FreqAttr;
    DisplayAttr2 := Colors^.PromptAttr;
  end;

  function TDxp38.ZeroCenterString : string;
  var
    Led : integer;
    LedCh : char;
  begin
    {limiter circuit}
    If FreqDelta < -80 then FreqDelta := -80;
    If FreqDelta > 80  then FreqDelta := 80;

    {divider circuit}
    Led := 21+(FreqDelta div 4);

    if Odd(Led) then {leds number 0 - 39}
      LedCh := ''
    else
      LedCh := '';

    ZeroCenterString := Pad(LeftPad(LedCh,(Led) div 2),SizeOf(DisplayString1));
  end;

  procedure TDxp38.ZeroCenterDisplay;
  const
    DisplayIncr = 3;
  var
    MarkLeds,
    SpaceLeds,
    Diff : byte;
  begin
    DisplayString1 := ZeroCenterString;

    MarkLeds   := Mark div DisplayIncr;
    SpaceLeds  := Space div DisplayIncr;
    Diff := abs(SpaceLeds - MarkLeds);

    if Diff > 19 then Diff := 19;

    if MarkLeds > SpaceLeds then
      DisplayString2 := CharStr('',SpaceLeds)+CharStr('',Diff)
      else
      DisplayString2 := CharStr('',MarkLeds)+CharStr('',Diff);
    DisplayString2 := Pad(DisplayString2,Sizeof(DisplayString2));
    DisplayAttr1 := FreqAttr;
    DisplayAttr2 := Colors^.PromptAttr;
  end;

  procedure TDxp38.UpdateDisplay;
  begin
    case GetDisplay of
      0 : Exit;
      1 : ZeroCenterDisplay;
      2 : OneBarDisplay;
      3 : TwoBarDisplay;
      4 : TwoBarTouchDisplay;
      5 : FourBarDisplay;
    end;
  end;

  procedure TDxp38.UpdateFreqDelta;
  var
    si : shortint;
    i : integer;
    w : word;
  begin
    w := ReadOutputPortWait;
    i := Lo(w);
    if i > 127 then i := -255 + i;
    si := i;
    FreqDelta := si;

    if ZeroTuneRev then FreqDelta := -1*FreqDelta;
    case FreqDelta of
      -127.. -30 : FreqAttr := Hi(Colors^.WindowAttr)+Red;
      -29.. -6 : FreqAttr := Hi(Colors^.WindowAttr)+Yellow;
      -5.. +5 : FreqAttr := Hi(Colors^.WindowAttr)+Green;
      +6.. +30 : FreqAttr := Hi(Colors^.WindowAttr)+Yellow;
      +31.. +127 : FreqAttr := Hi(Colors^.WindowAttr)+Red;
    end;
  end;

  procedure TDxp38.StopEverything;
  begin
    ClearXmitBuffer;
    UP^.SetDtr(False); {tell remote to stop everything}
    Delay(600);        { for this long...}
    UP^.SetDtr(True);  { ok to begin again}
    Transmit(False);
  end;

  function TDxp38.InitializeBox : Boolean;
  var
    et, Et2 : eventtimer;
    w, W1, W2 : Word;
    Ch : Char;


  procedure GetVersionInfo;
  var
    i : integer;
  begin
    SendCommand($8076);
    i :=2999;
    repeat
     w := ReadOutputPortWait;
     dec(i);
    until (W = $8076) or (I=0);
    W1 := ReadOutputPortWait and $00FF;
    W2 := ReadOutputPortWait and $00FF;
    RttyWin^.PutString(^M+Pad(DspProgram+' Version', 20)+long2str(W1)+'.'+long2str(W2));

    SendCommand($8077);
    i :=2999;
    repeat
     w := ReadOutputPortWait;
     dec(i);
    until (W = $8077) or (I=0);
{    repeat until ReadOutputPortWait = $8077; }
    W1 := ReadOutputPortWait and $00FF;
    W2 := ReadOutputPortWait and $00FF;
    RttyWin^.PutString(^M+Pad(ControllerProgram+' Version', 20)+long2str(W1)+'.'+long2str(W2));

    SendCommand($8078);
    i :=2999;
    repeat
     w := ReadOutputPortWait;
     dec(i);
    until (W = $8078) or (I=0);
{    repeat until ReadOutputPortWait = $8078;}
    W1 := ReadOutputPortWait and $00FF;
    W2 := ReadOutputPortWait and $00FF;
    RttyWin^.PutString(^M+Pad('Boot ROM Version', 20)+long2str(W1)+'.'+long2str(W2));
  end;

  function Get8002 : Boolean;
  begin
    NewTimerSecs(et, 30);
    UP^.FlushInBuffer;
    UP^.FlushOutBuffer;
    repeat
      SendCommand($8002);
      newtimer(Et2, 6);
      repeat until (TimerExpired(Et2)) or (UP^.BlockReady(2,[]));
      if TimerExpired(Et2) then RttyWin^.PutString('.');
      w := GetPortWord;
      if (w = $0280) or (w = $0A80) then
        repeat
          Receive(Ch);
        until ch <> cNUL;
      if (w <> $8002) then w := GetPortWord; {2 reads per write}
    until TimerExpired(et) or (w = $8002);
    Get8002 := w = $8002;
  end;

  procedure ReloadFirmware;
  begin
    RttyWin^.PutString(^M'DXP38 Checksum failed, reloading firmware...');
    newtimer(et, 15);
    SendCommand($800F);
    repeat until TimerExpired(et) or (GetPortWord = $800F);
    if (not ExistFile(ProgramPath+ControllerProgram)) or
      (not ExistFile(ProgramPath+DspProgram)) then begin
      RttyWin^.PutString(^M'DXP-38 Support files not found in the '+Trim(ProgramPath)+' directory');
      RttyWin^.PutString(^M'Edit the RTTY.INI file to identify the proper location of the files');
      Exit;
    end;
    if not DownloadDspProgram then begin
      WriteLn('Unable to load '+ProgramPath+DspProgram);
      Halt;
      end
    else
      RttyWin^.PutString(^M+ProgramPath+DspProgram+' loaded.');
    if not DownloadControllerProgram then begin
      WriteLn('Unable to load '+ProgramPath+ControllerProgram);
      Halt;
      end
    else
      RttyWin^.PutString(^M+ProgramPath+ControllerProgram+' loaded.');
  end;

  begin
    InitializeBox := False;       {assume failure!}
    RttyWin^.PutString(^M'Initializing DXP38');

    if not Get8002 then begin
      RttyWin^.PutString(^M'Difficulty establishing communication with DXP38');
      RttyWin^.PutString(^M'Power Off & On, then press <enter> to try again');
      repeat until keypressed;
      ReadKeyWord;
      if not Get8002 then Exit;
    end;

    repeat until UP^.BlockReady(2,[]);
    w := GetPortWord;

    UP^.FlushInBuffer;
    UP^.FlushOutBuffer;

    case w of
      $800A : RttyWin^.PutString(^M'Software download OK');
      $800B : ReloadFirmware;
    end;

    GetVersionInfo;

    Mark := 1;
    Space := 1;
    SendCommand($8084);           {enable fsk modes}
    repeat until ReadOutputPortWait = $8084;
    SendCommand($8059);
    repeat until ReadOutputPortWait = $8059;
    SendCommand($80E3); { PTT On Delay }
    SendCommand($8000); { 0 ms }
    repeat until ReadOutputPortWait = $80E3;
    SendCommand($80ED); { PTT Off Delay }
    SendCommand($8000); { 0 ms }
    repeat until ReadOutputPortWait = $80ED;
    InitializeBox := True;
  end;


  constructor TDxp38.Init(const Port : ComNameType);
  var
    w : Word;
    s : String[19];
  begin
    inherited InitCom(Port);
    TncNum := bDxp38;
    DisplayString1 := '';
    DisplayString2 := '';
    FreqDelta := 0;
    ProgramPath := RttyIni^.GetProfileString('Path', 'TNC1', 'C:\DXP38');
    ExtraDelay := RttyIni^.GetProfileLong('ExtraDelay', 'TNC1', 2);
    CollectNumbers := RttyIni^.GetProfileBool('CollectNumbers', 'TNC1', False);
    ZeroTuneRev := RttyIni^.GetProfileBool('ZeroTuneRev', 'TNC1', False);
    ProgramPath := AddBackSlash(ProgramPath);
    ChangeHandshake(UP, RtsCts);

    SendCommand($8009);
    { Delay(100); }

    if InitializeBox then begin
      RttyWin^.PutString(^M'DXP38 initialization complete.');
      end
    else begin
      RttyWin^.PutString(^M'Unable to initialize DXP38.');
      Fail;
    end;

    {set initial commands}
    SetMode('RTTY');
    SetMarkFreq(GetMarkFreq);
    SetSpaceFreq(GetSpaceFreq);
    SetSense(GetSense);
    SetUsos(GetUsos);
    SetDiddles(GetDiddles);
    SetEol(GetEOL);
    SetBaudRate(GetBaudRate);
    SetSquelchEnabled(GetSquelchEnabled);
    SetSquelchLevel(GetSquelchLevel);
    SetFskPolarity(GetFskPolarity);
    SetFilterWidth(GetFilterWidth);
    SetDisplay(GetDisplay);
    SetDisplayScale(GetDisplayScale);
    SendCommand($805D);  {enable FSK Tuning Status}
    SendCommand($8051);  {enable Auto Channel Statistics Reports}
    if not FastStart then Setup;

    if CollectNumbers then
      for w := 0 to 255 do begin
        MarkLoCt[w] := 0;
        MarkCt[w] := 0;
        SpaceCt[w] := 0;
        SpaceHiCt[w] := 0;
      end;
    {$IFDEF DEBUG}
    AppendDebugMessage('Dxp38 created');
    {$ENDIF}
  end;

  procedure TDxp38.ToggleTncTalk;
  begin
    Setup;
  end;

  procedure TDxp38.GetZero(var NumForZero : shortint; var DispAttr : byte);
  begin
    NumForZero := FreqDelta;
    DispAttr := FreqAttr;
  end;

  procedure tDxp38.UpdateFilterStrings(var S1, S2 : string; var A1, A2 : Byte);
  begin
    S1 := DisplayString1;
    S2 := DisplayString2;
    A1 := DisplayAttr1;
    A2 := DisplayAttr2;
  end;

  destructor TDxp38.Done;
  var
    F : Text;
    I : Integer;
  begin
    SendCommand($804D);           {disable FSK Tuning Status}
    SendCommand($8041);           {disable Auto Channel Statistics Reports}
    Delay(500);
    if CollectNumbers then begin
      Assign(F, 'DXP38.RPT');
      Rewrite(F);
      WriteLn(F, 'Value', Chr(9), 'Mark Lo Ct', Chr(9), 'Mark Ct', Chr(9), 'Space Ct', Chr(9), 'Space Hi Ct');
      for I := 1 to 255 do
        WriteLn(F, I, Chr(9), MarkLoCt[I], Chr(9), MarkCt[I], Chr(9), SpaceCt[I], Chr(9), SpaceHiCt[I]);
      Close(F);
    end;

    inherited Done;

    {$IFDEF DEBUG}
    AppendDebugMessage('Dxp38 destroyed');
    {$ENDIF}

  end; {destructor}

end.
