{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$F+,O+}
unit cEaTbl;
interface

  {* FUNCTIONS AVAILABLE FROM THIS UNIT *}

uses
  TpRarray,
  Rectypes;

const
  Num_EA_Provs = 52;

type

  PEaList = ^TEaList;
  TEaList = object
                 private
                   EA_Tbl    : array[1..Num_EA_Provs] of Mult_Rec;
                 public
                   constructor Init;
                   Destructor  Done;
                   function    GetProv(const Pv : Byte; const Band : BandType) : boolean;
                   procedure   SetProv(const Pv : Byte; const Band : BandType);
                   procedure   ReSetProv(const Pv : Byte; const Band : BandType);
                   function    HowManyProvs(const Band : BandType) : longint;
                   procedure   Display;
                   procedure   Print;
                   function    BandString(const Pv : Byte) : string;
                   function    GetProvNo(const Pr : String) : Byte;
                   function    Load(const Qth : string; const Band : BandType) : Boolean;
                 end;

  {* NOW DEFINE THE FUNCTIONS *}

var
  EaList : PEaList;

procedure MakeEaList;
procedure KillEaList;

implementation

uses
  DebugRtns,
  TpString,
  IniFile,
  TpCrt,
  TpMouse,
  cColors,
  TpSort,
  TpWindow,
  LowLvl,
  Keys,
  cEditWin,
  cMsgWin,
  c_Contest,
  Routines,
  OneTimers;

  const
  EA_Provs  : array[1..Num_EA_Provs] of String[2] = (
             'A' , 'AB', 'AL', 'AV',
             'B' , 'BA', 'BI', 'BU',
             'C' , 'CA', 'CC', 'CE',
             'CO', 'CR', 'CS', 'CU',
             'GC', 'GI', 'GR', 'GU',
             'H' , 'HU', 'J' , 'L' ,
             'LE', 'LO', 'LU', 'M' ,
             'MA', 'ML', 'MU', 'NA',
             'O' , 'OR', 'P' , 'PM',
             'PO', 'S' , 'SA', 'SE',
             'SG', 'SO', 'SS', 'T' ,
             'TE', 'TF', 'TO', 'V' ,
             'VA', 'VI', 'Z' , 'ZA' );

  constructor TEaList.Init;
  var
    I : Integer;
    Mult : Mult_Rec;
    Band : BandType;
  begin
    for Band := B80 to TOT do
      Mult.Band[Band] := False;
    for I := 1 to Num_EA_Provs do
      EA_Tbl[I] := Mult;
    {$IFDEF DEBUG}
    AppendDebugMessage('EATbl created');
    {$ENDIF}
  end;

  Destructor  TEaList.Done;
  begin
    {$IFDEF DEBUG}
    AppendDebugMessage('EATbl destroyed');
    {$ENDIF}
  end;

  function    TEaList.GetProv(const Pv : Byte; const Band : BandType) : boolean;
  begin
    {$IFDEF DEBUG}
    if (Pv = 0) or (Pv > Num_Ea_Provs) then
      AppendDebugMessage('CEaProv.GetEaProv, invalid EaProv# :'+long2str(Pv));
    {$ENDIF}
    if (Pv = 0) or (Pv > Num_Ea_Provs) then begin
      GetProv := False;
      Exit;
    end;
    GetProv := Ea_Tbl[Pv].Band[Band]
  end;

  procedure   TEaList.SetProv(const Pv : Byte; const Band : BandType);
  begin
    {$IFDEF DEBUG}
    if (Pv = 0) or (Pv > Num_Ea_Provs) then
      AppendDebugMessage('CEaProv.SetProv, invalid EaProv# :'+long2str(Pv));
    {$ENDIF}
    if (Pv = 0) or (Pv > Num_Ea_Provs) then Exit;
    Ea_Tbl[Pv].Band[Band] := True;
    Ea_Tbl[Pv].Band[TOT]  := True;
  end;

  procedure   TEaList.ReSetProv(const Pv : Byte; const Band : BandType);
  var
    b : bandType;
  begin
    {$IFDEF DEBUG}
    if (Pv = 0) or (Pv > Num_Ea_Provs) then
      AppendDebugMessage('CEaProv.ResetProv, invalid EaProv# :'+long2str(Pv));
    {$ENDIF}
    if (Pv = 0) or (Pv > Num_Ea_Provs) then Exit;
    Ea_Tbl[Pv].Band[Band] := False;
    Ea_Tbl[Pv].Band[TOT] := False;
    for B := B80 to B10 do begin
      if GetProv(Pv,B) then
        Ea_Tbl[Pv].Band[TOT]  := True;
    end;
  end;

  function    TEaList.HowManyProvs(const Band : BandType) : longint;
  var
    L       : longint;
    Pv      : Word;
  begin
    L := 0;
    for Pv := 1 to Num_Ea_Provs do
      if GetProv(Pv,Band) then Inc(L);
    HowManyProvs := L;
    {$IFDEF DEBUG}
    AppendDebugMessage('TEaList.HowManyEaProvs = '+Bands[Band]+'M '+long2str(l));
    {$ENDIF}
  end;

  procedure TEaList.Display;
  var
    W              : Word;
    col, Row, ix   : Byte;
    Win            : WindowPtr;
    S              : String[5];
    Band           : BandType;
  begin
    with Colors^ do begin
      if not MakeWindow(Win, 1, 2, 80, 14, True, True, True,
                        WindowAttr, FrameAttr, HeaderAttr,
                        'EA Provinces Multiplier Check Sheet') then Exit;
      HideMouse;
      DisplayWindow(Win);
      for col := 0 to 5 do
        for Row := 1 to 9 do
        begin
          ix := col*9+row;
          if ix <= Num_EA_Provs then begin
           FastWriteWindow(Pad(Trim(EA_Provs[ix]), 3)+' |'+BandString(ix)+'|', Row+1, col*13+2, WindowAttr);
          end;
        end;
      FastWrite('Press Any Key to continue', 14, 2, WindowAttr);
    end;
    ShowMouse;
    ReadKeyorButton;
    KillWindow(Win);
  end;

  procedure TEaList.Print;
  var
    Str  : String[80];
    ix   : Byte;
    PrintFile : Text;
  begin
    Assign(PrintFile, FileName('EA'));
    {$I-} Rewrite(PrintFile); {$I+}
    MsgWin^.Display('Preparing: '+FileName('EA'));
    Writeln(PrintFile,'EA Province Multiplier sheet for '+RttyIni^.GetProfileString('Call','Profile',''));
    Writeln(PrintFile,'');
    for ix := 1 to Num_EA_Provs do begin
      Str := ' '+Pad(EA_Provs[ix], 4)+' |'+BandString(ix)+'|';
      Writeln(PrintFile,Str);
    end;
    Write(PrintFile,^L);
    Close(PrintFile);
  end;

  function  TEaList.BandString(const Pv : Byte) : string;
  var
    s : string[5];
    Band : BandType;
  begin
    S := '.....';
    for Band := B80 to B10 do
      if GetProv(Pv,Band) then S[Ord(Band)+1] := '*';
    BandString := S;
  end;

  function TEaList.GetProvNo(const Pr : String) : Byte;
  var
    P : String[2];
    ix : Byte;
  begin
    P := Trim(Copy(Pr, 1, 2));
    for ix := 1 to Num_Ea_Provs do
      if P = EA_Provs[ix] then begin
        GetProvNo := ix;
        Exit;
      end;
    GetProvNo := 0;
  end;

  function TEaList.Load(const Qth : string; const Band : BandType) : Boolean;
  var
    T  : Boolean;
    ix : Byte;
  begin
    T := False;
    ix := GetProvNo(Qth);
    if ix > 0 then T := not GetProv(ix,Band);
    SetProv(ix,Band);
    SetProv(ix,TOT);
    Load := T;
  end;





  procedure MakeEaList;
  begin
    EaList := New(PEaList, Init);
    if not assigned(EaList) then begin
      MessageBox('Severe Error!',
                 'Unable to create EaProv list',
                 'Contact Wyvern Technology',
                 'Consider reinstalling from diskette');
      CloseDownScreen;
      Halt;
    end;
  end;

  procedure KillEaList;
  begin
    if Assigned(EaList) then Dispose(EaList, Done);
    EaList := NIL;
  end;



end.
