{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$F+,O+}
unit CEditWin;

interface

uses
  Diskio,
  RecTypes;

type


  PEditWin = ^TEditWin;
  TEditWin = object
               EditRecs       : Array[1..5] of Log_Rec;
               HoldRecs       : Array[1..5] of Log_Rec;
               TARecNums      : Array[1..5] of LongInt;
               Mults          : Array[1..5] of String[5];
               Ix             : Array[1..5] of String[6];
               constructor Init;
               destructor  Done; virtual;
               procedure   NewQso(const R : log_rec; Loading : Boolean);
               procedure   NewQsoFromNet(const R : log_rec);
               procedure   Scroll;
               function    ScreenFieldNum : byte;
               function    ScreenRecordNum : byte;
               procedure   SaveToDisk;
               procedure   RefreshFromDisk;
               procedure   MoveFromTo(const FromNum, ToNum : byte);
               procedure   BlankArrays;
             end;
var
  EditW : PEditWin;

implementation

uses
  Dos,
  TpCrt,
  TpDos,
  TpEntry,
  TpString,
  TpPick,
  TAHigh,
  TAccess,
  Keys,
  c_Radio,
  c_Contest,
  cCountry,
  cDupeTbl,
  Routines,
  DebugRtns,
  IniFile,
  Network,
  LowLvl;

  constructor TEditWin.Init;
  begin
    BlankArrays;
  {$IFDEF DEBUG}
  AppendDebugMessage('EditWin created');
  {$ENDIF}
  end;

  destructor TEditWin.Done;
  begin
  {$IFDEF DEBUG}
  AppendDebugMessage('EditWin destroyed');
  {$ENDIF}
  end;

  procedure TEditWin.BlankArrays;
  var
    I : Integer;
  begin
    for I := 1 to 5 do begin
      LogFile^.BlankLogRec(EditRecs[i]);
      EditRecs[i].Band := B20;
      LogFile^.BlankLogRec(HoldRecs[i]);
      HoldRecs[i].Band := B20;
      TARecNums[i] := 0;
      ix[i] := '';
      Mults[i] := '';
    end;
    if Assigned(RttyIni) and Assigned(EditW) then begin
      case RttyIni^.GetProfileInt('Band', 'CURRENT', 20) of
        80 : EditW^.EditRecs[1].Band := b80;
        40 : EditW^.EditRecs[1].Band := b40;
        20 : EditW^.EditRecs[1].Band := b20;
        15 : EditW^.EditRecs[1].Band := b15;
        10 : EditW^.EditRecs[1].Band := b10;
      end;
    end;
  end;

  procedure  TEditWin.MoveFromTo(const FromNum, ToNum : byte);
  begin
    Move(EditRecs[FromNum],EditRecs[ToNum],SizeOf(EditRecs[ToNum])); { src, dest, len }
    Move(HoldRecs[FromNum],HoldRecs[ToNum],SizeOf(HoldRecs[ToNum])); { src, dest, len }
    TARecNums[ToNum] := TARecNums[FromNum];
    Ix[ToNum] := ix[FromNum];
    Mults[ToNum] := Mults[FromNum];
  end;

  procedure TEditWin.Scroll;
  var
    b : byte;
  begin
    for b := 5 downto 3 do MoveFromTo(b-1,b);
  end;

  procedure TEditWin.NewQso(const R : log_rec; Loading : Boolean);
  begin
    if not Loading then SaveToDisk;
    Scroll;
    EditRecs[2]  := R;
    HoldRecs[2]  := R;
    TaRecNums[2] := TaRecNum;
    ix[2] := LogFile^.MakeIx(EditRecs[2]);
    mults[2] := ThisContest^.MakeMultStr(EditRecs[2]);
    LogFile^.BlankLogRec(EditRecs[1]); { current QSO area }
    if gInMainLoop then DrawEditScreen(EntryEsr);
  end;

  procedure TEditWin.NewQsoFromNet(const R : log_rec);

    procedure ScrollLast3Only;
    var
      b : byte;
    begin
      for b := 5 downto 3 do MoveFromTo(b-1,b);
    end;

  begin
    ScrollLast3Only;
    Move(R,EditRecs[2],SizeOf(EditRecs[2])); { src, dest, len }
    Move(R,HoldRecs[2],SizeOf(HoldRecs[2])); { src, dest, len }
    TaRecNums[2] := TaRecNum;
    ix[2] := LogFile^.MakeIx(EditRecs[2]);
    mults[2] := ThisContest^.MakeMultStr(EditRecs[2]);
    {if gInMainLoop then DrawEditScreen(EntryEsr);}
  end;

  function TeditWin.ScreenFieldNum : byte;
  { returns the field number (across) on the screen }
  begin
    ScreenFieldNum := (EntryEsr.CurrentId mod ThisContest^.FieldsPerRecord);
  end;

  function TEditWin.ScreenRecordNum : byte;
  { returns 1-6 depending which column is current, I build the
    record number }
  begin
    ScreenRecordNum := (EntryEsr.CurrentId div ThisContest^.FieldsPerRecord) + 1;
  end;

  procedure TEditWin.SaveToDisk;
  var
    i : integer;
    max : integer;
    AnyChanges : Boolean;
  begin
    max := 1+LogFile^.NumRecs;
    if max > 5 then max := 5;
    AnyChanges := False;
    for i := max downto 2 do
      if TaRecNums[i] <> 0 then
        if CompStruct(HoldRecs[i],EditRecs[i],SizeOf(EditRecs[i])) <> EQUAL then AnyChanges := True;
    if not AnyChanges then Exit;
    {$IFDEF DEBUG}
    AppendDebugMessage('TEditWin.SaveToDisk: AnyChanges=Yes');
    {$ENDIF}

    { USE HOLD record to remove old stuff }
    { first, remove all mults & dupes }
    for i := max downto 2 do
      if TaRecNums[i] <> 0 then begin
        ThisContest^.RemoveOldMults(HoldRecs[i]);
        if NOT HoldRecs[i].Dupe then
          DupeTbl^.ResetDupeByCall(HoldRecs[i].Call,HoldRecs[i].Band);
      end;

    { then, re-add any mults, dupetbl, etc... }
    for i := max downto 2 do
      if TaRecNums[i] <> 0 then begin
        if CountryList^.Identify(EditRecs[i]) <> NoCountry then
          EditRecs[i].His_Country := CountryList^.Ct.Code;
        ThisContest^.AddNewMults(EditRecs[i]);
        EditRecs[i].Dupe := DupeTbl^.Load(EditRecs[i]);
        if EditRecs[i].Dupe then EditRecs[i].Point := 0 else
          EditRecs[i].Point := ThisContest^.PointValue(EditRecs[i]);
        mults[i] := ThisContest^.MakeMultStr(EditRecs[i]);
        Ix[i]    := LogFile^.MakeIx(EditRecs[i]);
        LogFile^.FindThenUpdate(TaRecNums[i],EditRecs[i]);
      end;

    { then, if anything changed, save it, then send it to the network }
    for i := 2 to max do
      if CompStruct(HoldRecs[i],EditRecs[i],SizeOf(EditRecs[i])) <> EQUAL then begin
        Move(EditRecs[i],HoldRecs[i],SizeOf(EditRecs[i])); { save it }
        if Assigned(Net) then begin
          Net^.SendLogData(False, EditRecs[i],Ix[i]);
          {$IFDEF DEBUG}
          AppendDebugMessage('TEditWin.SaveToDisk: New='+EditRecs[i].Call+' Old='+holdrecs[i].Call);
          {$ENDIF}
        end;
      end;
    { finally, send it everyone on the network }
    DrawEditScreen(EntryEsr);
 end;

  procedure TEditWin.RefreshFromDisk;
  var
    i,j : integer;
    a : log_rec;

  procedure Moverec(num : byte);
  begin
    Move(a,HoldRecs[num],SizeOf(a));
    Move(a,EditRecs[num],SizeOf(a));
    mults[num] := ThisContest^.MakeMultStr(EditRecs[num]);
    ix[num]        := LogFile^.Index;    { from diskio unit }
    TaRecNums[num] := TaRecnum; { from diskio unit }
  end;

  begin
    BlankArrays;
    i := LogFile^.NumRecs;
    if i = 0 then Exit;
    if i > 4 then i := 4;
    LogFile^.ResetLog;
    for j := 2 to I+1 do begin
      LogFile^.Get_Prev(a);
      if OK then MoveRec(j);
    end;
  end;

end.
