{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$F+,O+}
unit cITbl;
interface

  {* FUNCTIONS AVAILABLE FROM THIS UNIT *}

uses
  TpRarray,
  Rectypes;

const
  Num_I_Provs = 103;

type

  PIList = ^TIList;
  TIList = object
                 private
                   I_Tbl    : array[1..Num_I_Provs] of Mult_Rec;
                 public
                   constructor Init;
                   Destructor  Done;
                   function    GetProv(const Pv : Byte; const Band : BandType) : boolean;
                   procedure   SetProv(const Pv : Byte; const Band : BandType);
                   procedure   ReSetProv(const Pv : Byte; const Band : BandType);
                   function    HowManyProvs(const Band : BandType) : longint;
                   procedure   Display;
                   procedure   Print;
                   function    BandString(const Pv : Byte) : string;
                   function    GetProvNo(const Pr : String) : Byte;
                   function    Load(const Qth : string; const Band : BandType) : Boolean;
                 end;

  {* NOW DEFINE THE FUNCTIONS *}

var
  IList : PIList;

procedure MakeIList;
procedure KillIList;

implementation

uses
  DebugRtns,
  TpString,
  LowLvl,
  TpCrt,
  cColors,
  c_Contest,
  IniFile,
  TpMouse,
  TpSort,
  TpWindow,
  Keys,
  cEditWin,
  cMsgWin,
  Routines,
  OneTimers;

  const
  I_Provs  : array[1..Num_I_Provs] of String[2] = (
  'AG','AL','AN','AO','AP','AQ','AR','AT','AV','BA',
  'BG','BI','BL','BN','BO','BR','BS','BZ','CA','CB',
  'CE','CH','CL','CN','CO','CR','CS','CT','CZ','EN',
  'FE','FG','FI','FO','FR','GE','GO','GR','IM','IS',
  'KR','LC','LE','LI','LO','LT','LU','MC','ME','MI',
  'MN','MO','MS','MT','NA','NO','NU','OR','PA','PC',
  'PD','PE','PG','PI','PN','PO','PR','PS','PT','PV',
  'PZ','RA','RC','RE','RG','RI','RM','RN','RO','SA',
  'SI','SO','SP','SR','SS','SV','TA','TE','TN','TO',
  'TP','TR','TS','TV','UD','VA','VB','VC','VE','VI',
  'VR','VT','VV');

  constructor TIList.Init;
  var
    I : Integer;
    Mult : Mult_Rec;
    Band : BandType;
  begin
    for Band := B80 to TOT do
      Mult.Band[Band] := False;
    for I := 1 to Num_I_Provs do
      I_Tbl[I] := Mult;
    {$IFDEF DEBUG}
    AppendDebugMessage('ITbl created');
    {$ENDIF}
  end;

  Destructor  TIList.Done;
  begin
    {$IFDEF DEBUG}
    AppendDebugMessage('ITbl destroyed');
    {$ENDIF}
  end;

  function    TIList.GetProv(const Pv : Byte; const Band : BandType) : boolean;
  begin
    {$IFDEF DEBUG}
    if (Pv = 0) or (Pv > Num_I_Provs) then
      AppendDebugMessage('CTIList.GetProv, invalid I Prov# :'+long2str(Pv));
    {$ENDIF}
    if (Pv = 0) or (Pv > Num_I_Provs) then begin
      GetProv := False;
      Exit;
    end;
    GetProv := I_Tbl[Pv].Band[Band]
  end;

  procedure   TIList.SetProv(const Pv : Byte; const Band : BandType);
  begin
    {$IFDEF DEBUG}
    if (Pv = 0) or (Pv > Num_I_Provs) then
      AppendDebugMessage('CTIList.SetProv, invalid I Prov# :'+long2str(Pv));
    {$ENDIF}
    if (Pv = 0) or (Pv > Num_I_Provs) then Exit;
    I_Tbl[Pv].Band[Band] := True;
    I_Tbl[Pv].Band[TOT]  := True;
  end;

  procedure   TIList.ReSetProv(const Pv : Byte; const Band : BandType);
  var
    b : bandType;
  begin
    {$IFDEF DEBUG}
    if (Pv = 0) or (Pv > Num_I_Provs) then
      AppendDebugMessage('CTIList.ReSetProv, invalid I Prov# :'+long2str(Pv));
    {$ENDIF}
    if (Pv = 0) or (Pv > Num_I_Provs) then Exit;
    I_Tbl[Pv].Band[Band] := False;
    I_Tbl[Pv].Band[TOT] := False;
    for B := B80 to B10 do begin
      if GetProv(Pv,B) then
        I_Tbl[Pv].Band[TOT]  := True;
    end;
  end;

  function    TIList.HowManyProvs(const Band : BandType) : longint;
  var
    L       : longint;
    Pv      : Word;
  begin
    L := 0;
    for Pv := 1 to Num_I_Provs do
      if GetProv(Pv,Band) then Inc(L);
    HowManyProvs := L;
    {$IFDEF DEBUG}
    AppendDebugMessage('TIList.HowManyProvs = '+Bands[Band]+'M '+long2str(l));
    {$ENDIF}
  end;

  procedure TIList.Display;
  var
    W              : Word;
    col, Row, ix   : Byte;
    Win            : WindowPtr;
    S              : String[5];
    Band           : BandType;
  begin
    with Colors^ do begin
     if not MakeWindow(Win, 1, 2, 80, 22, True, True, True,
                       WindowAttr, FrameAttr, HeaderAttr,
                       'I Provinces Multiplier Check Sheet') then Exit;
     HideMouse;
     DisplayWindow(Win);
     for col := 0 to 5 do
       for Row := 1 to 18 do
       begin
         ix := col*18+row;
         if ix <= Num_I_Provs then
           FastWriteWindow(Pad(Trim(I_Provs[ix]), 3)+' |'+BandString(ix)+'|', Row+1, col*13+2, WindowAttr);
       end;
     FastWrite('Press Any Key to continue', 22, 2, WindowAttr);
    end;
    ShowMouse;
    ReadKeyorButton;
    KillWindow(Win);
  end;

  procedure TIList.Print;
  var
    Str  : String[80];
    ix   : Byte;
    PrintFile : Text;
  begin
    Assign(PrintFile, FileName('I'));
    {$I-} Rewrite(PrintFile); {$I+}
    MsgWin^.Display('Preparing: '+FileName('I'));
    Writeln(PrintFile,'I Province Multiplier sheet for '+RttyIni^.GetProfileString('Call','Profile',''));
    Writeln(PrintFile,'');
    for ix := 1 to Num_I_Provs do begin
      Str := ' '+Pad(I_Provs[ix], 4)+' |'+BandString(ix)+'|';
      Writeln(PrintFile,Str);
    end;
    Write(PrintFile,^L);
    Close(PrintFile);
  end;

  function  TIList.BandString(const Pv : Byte) : string;
  var
    s : string[5];
    Band : BandType;
  begin
    S := '.....';
    for Band := B80 to B10 do
      if GetProv(Pv,Band) then S[Ord(Band)+1] := '*';
    BandString := S;
  end;

  function TIList.GetProvNo(const Pr : String) : Byte;
  var
    P : String[2];
    ix : Byte;
  begin
    P := Trim(Copy(Pr, 1, 2));
    for ix := 1 to Num_I_Provs do
      if P = I_Provs[ix] then begin
        GetProvNo := ix;
        Exit;
      end;
    GetProvNo := 0;
  end;

  function TIList.Load(const Qth : string; const Band : BandType) : Boolean;
  var
    T  : Boolean;
    ix : Byte;
  begin
    T := False;
    ix := GetProvNo(Qth);
    if ix > 0 then T := not GetProv(ix,Band);
    SetProv(ix,Band);
    SetProv(ix,TOT);
    Load := T;
  end;





  procedure MakeIList;
  begin
    IList := New(PIList, Init);
    if not assigned(IList) then begin
      MessageBox('Severe Error!',
                 'Unable to create I Prov list',
                 'Contact Wyvern Technology',
                 'Consider reinstalling from diskette');
      CloseDownScreen;
      Halt;
    end;
    {$IFDEF DEBUG}
    AppendDebugMessage('IList created');
    {$ENDIF}
  end;

  procedure KillIList;
  begin
    if Assigned(IList) then begin
      Dispose(IList, Done);
      IList := NIL;
      {$IFDEF DEBUG}
      AppendDebugMessage('IList destroyed');
      {$ENDIF}
    end;
  end;



end.
