{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit cK6STI;
{
  The K6STI software modem uses the Sound Blaster 16 card to handle the
  data processing.  We worked out an API using the Interupt 14 and COM8
  to allow our programs to work cooperatively.
}
interface
uses
  tncobject,
  OoCom;
type
  TPipRec = record
              DispStr: String[10];
              Bright : Boolean;
              Value  : Byte;
            end;

  PK6sti = ^TK6sti;
  TK6sti = object(TTnc)
                 Pip : array[1..2] of TPipRec;
                 constructor Init;
                 procedure   Transmit(const Xmit : Boolean);virtual;
                 procedure   Receive(var Ch : Char);virtual;
                 procedure   SetBaudRate(const Rate : Byte); virtual;
                 procedure   Letters; virtual;
                 procedure   Figures; virtual;
                 procedure   StopEverything; virtual;
                 procedure   SetSense(const Sense : boolean); virtual;
                 destructor  Done; virtual;
                 procedure   ToggleTncTalk; virtual;
                 procedure   GetPip(const WhichOne : byte; var Display : string; var DispAttr : byte); virtual;
               end;



implementation


  uses
    ApPort,
    RecTypes,
    DebugRtns,
    TpCrt,
    LowLvl,
    cRttyWin,
    cColors,
    TpString;

  procedure TK6sti.Transmit(const Xmit : Boolean);
  begin
    case Xmit of
      True  : SendChar(^H); {this unit auto xmits upon receiving data}
      False : SendChar(^R);
    end;
  end;

  procedure TK6sti.Receive(var Ch : Char);

  procedure FormatPip(const PipNo : Byte; Data : Char);
  begin
    with Pip[Pipno] do begin
      Bright := ord(Data) >= 128;
      if Bright then Value := ord(Data) - 128
                else Value := ord(Data);
      if Pipno = 2 then Value := Value - 20;  {means we are doing "right" pip}
      DispStr := '          ';
      if Value > 0 then
        DispStr[(((Value-1) div 2) + 1)] := Chr(222 - (Value and $01));
    end;
  end;


  procedure FormatDisplay(var S : string);
  begin
    S := Pad('',20);
    if Pip[1].Value > 0 then begin
      S[(((Pip[1].Value-1) div 2) + 1)] := Chr(222 - (Pip[1].Value and $01));
    end;
    if Pip[2].Value > 0 then begin
      S[(((Pip[2].Value-1) div 2) + 1)] := Chr(222 - (Pip[2].Value and $01));
    end;
  end;

  begin
    if UP^.CharReady then
      UP^.GetChar(Ch)
    else
      Ch := cNUL;
    case Ch of
      ^B : begin
             repeat until UP^.CharReady;
             UP^.GetChar(Ch);
             FormatPip(1, Ch);
             repeat until UP^.CharReady;
             UP^.GetChar(Ch);
             FormatPip(2, Ch);
             Ch := cNUL;
           end;
      ^Y : begin
             Ch := ^X;  { ^Y from K6STI means use RI color code}
           end;
    end;
  end;

  procedure TK6sti.Letters;
  begin
    SendChar(^L);
  end;

  procedure TK6sti.Figures;
  begin
    SendChar(^F);
  end;

  procedure TK6sti.SetBaudRate(const Rate : Byte);
  begin
    SendChar(^S);
    case Rate of
      45 : SendByte(0);
      50 : SendByte(1);
      57 : SendByte(2);
      74 : SendByte(3);
      100: SendByte(4);
    else
      SendByte(0);
    end;
  end;

  procedure TK6sti.SetSense(const Sense : boolean);
  begin
    case Sense of
      False: SendChar(^V);
      True:  SendChar(^N);
    end;
    inherited SetSense(Sense);
  end;

  procedure TK6sti.StopEverything;
  begin
    SendChar(^[); {escape - force rx}
  end;


  constructor TK6sti.Init;
  begin
    inherited Init;
    TncNum := bK6Sti;
    UP := New(Int14PortPtr,  InitCustom(Com8,
    9600, NoParity, 8, 1, 128, 128, DefPortOptions));  { this is the right one }
    if UP = nil then begin
      RttyWin^.PutString(^M'Couldn''t initialize the RITTY modem!');
      RttyWin^.PutString(^M'Did you load it as a TSR? (eg RITTY I)');
      halt;
    end;
    StopEverything;
    SetSense(getSense);
    Pip[1].Bright := False;
    Pip[1].Value := 0;
    Pip[1].DispStr := '          ';
    Pip[2].Bright := False;
    Pip[2].Value := 0;
    Pip[2].DispStr := '          ';
    {$IFDEF DEBUG}
    AppendDebugMessage('K6STI created');
    {$ENDIF}
  end;

  destructor TK6sti.Done;
  begin
    inherited Done;
    {$IFDEF DEBUG}
    AppendDebugMessage('K6STI destroyed');
    {$ENDIF}
  end;

  procedure TK6sti.ToggleTncTalk;
  begin
   { override parent, no action desired for this tnc! }
  end;

  procedure TK6sti.GetPip(const WhichOne : byte; var Display : string; var DispAttr : byte);
  begin
    Display := Pip[WhichOne].DispStr;
    case Pip[WhichOne].Bright of
      True : DispAttr := Colors^.HiLiteAttr;
      False: DispAttr := Colors^.FrameAttr;
    end;
  end;

end.
