{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$F+,O+}
unit ClstrCom;

interface

uses
  OoCom,
  ApPort,
  TpWIndow,
  ClusterTerm,
  ApTimer;

type


  PComClusterTerminal = ^TComClusterTerminal;
  TComClusterTerminal = object(TClusterTerminal)
                       ComPort : ComNameType;
                       UP : UartPortPtr;
                       Msg : string;
                       constructor Init(const Port : ComNameType);
                       destructor Done; virtual;
                       procedure Idle; virtual;
                       procedure SendMessageToCluster(const AMsg : string);virtual;
                     end;


implementation

uses
  Dos,
  TpDos,
  TpString,
  TpCrt,
  NetWork,
  CCountry,
  RecTypes,
  ClusterMon,
  lowlvl,
  ApUart,
  ApMisc,
  DebugRtns,
  TpEdit,
  IniFile;

  constructor TComClusterTerminal.Init(const Port : ComNameType);
  var
    iBaudRate : Longint;
    iParity   : string[7];
    iDataBits : Longint;
    iStopBits : Longint;
    Parity : ParityType;
    ThisUP : UartPortPtr;
    iHandShake : HandShakeType;
    S      : string;
  begin
    inherited Init;
    Msg := '';
    ComPort := Port;
    iBaudRate := RttyIni^.GetProfileLong('BaudRate', ComNameString(ComPort), 9600);
    iParity   := RttyIni^.GetProfileString('Parity', ComNameString(ComPort), 'N');
    iDataBits := RttyIni^.GetProfileLong('DataBits', ComNameString(ComPort), 8);
    iStopBits := RttyIni^.GetProfileLong('StopBits', ComNameString(ComPort), 1);
    S         := StUpCase(RttyIni^.GetProfileString('HandShake', ComNameString(ComPort), 'None'));
    iHandShake:= HandShakeStringToHandShake(S);
    Parity := ParityStringToParity(iParity);
    UP := New(UartPortPtr, InitCustom(ComPort,
                  LongInt(iBaudRate),
                  Parity,
                  DataBitType(iDatabits),
                  StopBitType(iStopBits),
                  1024,
                  1024,
                  DefPortOptions));
    {$IFDEF DEBUG}
    if AsyncStatus <> ecOk then AppendDebugMessage('ComClusterTerminal.Init.New:'+
      StatusStr(AsyncStatus)+':'+long2str(AsyncStatus));
    {$ENDIF}
    ChangeHandShake(UP, iHandShake);
    {$IFDEF DEBUG}
    if AsyncStatus <> ecOk then AppendDebugMessage('ComClusterTerminal.Init.ChangeHandShake:'+
      StatusStr(AsyncStatus)+':'+long2str(AsyncStatus));
    {$ENDIF}
    UP^.FlushInBuffer; { flush the input buffer }
    UP^.FlushOutBuffer; { flush the output buffer }
    if UP^.CheckFifoError then begin
      SetFifoBuffering(UP^.PR^.BaseAddr,False,0); { flush the fifo buffer }
      SetFifoBuffering(UP^.PR^.BaseAddr,True,14);
    end;
    if RttyIni^.GetProfileBool('SendEnter', 'CLUSTER', True) then
      UP^.PutChar(^M);
    {$IFDEF DEBUG}
    AppendDebugMessage('ComClusterTerminal created');
    {$ENDIF}
  end;

  destructor TComClusterTerminal.Done;
  begin
    Dispose(UP, Done);
    UP := nil;
    inherited Done;
    {$IFDEF DEBUG}
    AppendDebugMessage('ComClusterTerminal destroyed');
    {$ENDIF}
  end;

  procedure TComClusterTerminal.SendMessageToCluster(const AMsg : string);
  begin
    {$IFDEF DEBUG}
    AppendDebugMessage('TComClusterTerminal.SendMessageToCluster=>'+aMsg);
    {$ENDIF}
    UP^.PutString(AMsg+^M);
    {$IFDEF DEBUG}
    if AsyncStatus <> ecOK then
      AppendDebugMessage('TComClusterTerminal.SendMessageToCluster.PutString:'+
        StatusStr(AsyncStatus)+':'+long2str(AsyncStatus));
    {$ENDIF}
  end;

  procedure TComClusterTerminal.Idle;
  var
    c : char;
    b : byte;
    InBuffSz : word;
  begin
    UP^.GetChar(c);
    Case AsyncStatus mod 10000 of
      ecOK : begin { everything OK }
               case C of
                 #$07 : C := ' ';  {remove bells}
                 #$09 : C := ' '; {replace tabs w/ space}
               end;
               with ClusterMonitor^ do begin
                 AppendChToLine(C); { pass everything along, $0D, and $0A too}
                 Idle(True);
               end;
               if (C <> #$0D) and (C <> #$0A) then LineIn := LineIn + C;
               if (C = #$0D) or (Length(LineIn) = 78) then begin
                 ScrollInLines;
                 LinesIn[20] := LineIn;
                 LineIn := '';
                 NeedRefresh := True;
               end;
             end;
      ecBufferIsEmpty    : Exit;
      ecStringIncomplete : Exit;
    else
      AppendDebugMessage('ComCluster.Idle.GetChar:'+StatusStr(AsyncStatus)+':'+long2str(AsyncStatus));
    end;
  end;

end.
