{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$F+,O+}
unit ClstrFil;

interface

uses OoCom,
     ApPort,
     ApTimer,
     ClusterTerm;

type


  PFileClusterTerminal = ^TFileClusterTerminal;
  TFileClusterTerminal = object(TClusterTerminal)
                          ET : EventTimer;
                          FileName   : string[39];
                          UtFileName : string[39];
                          ReadFreq   : integer;
                          F : Text;
                       constructor Init;
                       destructor Done; virtual;
                       procedure Idle; virtual;
                       procedure SendMessageToCluster(const AMsg : string);virtual;
                     end;



implementation

uses
  Dos,
  TpDos,
  TpString,
  TpCrt,
  NetWork,
  cCountry,
  RecTypes,
  ClusterMon,
  lowlvl,
  TpEdit,
  DebugRtns,
  IniFile;


  constructor TFileClusterTerminal.Init;
  begin
    inherited Init;
    FileName := Rttyini^.GetProfileString('InputFile','CLUSTER','SPOTS.TXT');
    UtFileName := Rttyini^.GetProfileString('CommandFile','CLUSTER','COMMANDS.TXT');
    ReadFreq := RttyIni^.GetProfileInt('FileSeconds','CLUSTER',30);
    NewTimerSecs(ET,1); {start right away}
    {$IFDEF DEBUG}
    AppendDebugMessage('FileClusterTerminal created');
    {$ENDIF}
  end;

  destructor TFileClusterTerminal.Done;
  begin
    {$IFDEF DEBUG}
    AppendDebugMessage('FileClusterTerminal destroyed');
    {$ENDIF}
    inherited Done;
  end;

  procedure TFileClusterTerminal.Idle;
  var
    b : byte;
  begin
    if not TimerExpired(ET) then Exit;
    if GetFileSize(FileName) = 0 then exit;
    Assign(F,FileName);
    Reset(F);
    while not eof(f) do begin
      readln(F, LineIn);
      for b := 1 to length(LineIn) do begin
        if Linein[b] = #$07 then Linein[b] := ' ';
        if Linein[b] = #$09 then Linein[b] := ' ';
      end;
      {$IFDEF DEBUG}
      AppendDebugMessage('TFileClusterTerminal.LineIn=>'+LineIn+'<=');
      {$ENDIF}
      with ClusterMonitor^ do begin
        AppendStrToLine(LineIn); { pass everything along, $0D, and $0A too}
        Idle(True); {call it's idle}
      end;
      ScrollInLines;
      LinesIn[20] := LineIn;
      LineIn := '';
      NeedRefresh := True;
    end;
    close(f);
    DeleteFile('SPOTS.TXT');
    NewTimerSecs(ET,ReadFreq);
  end;

  procedure TFileClusterTerminal.SendMessageToCluster(const AMsg : string);
  const
    FileOK = $0;
    FileNotFound = $2;
  var
    IOStatus : word;
  begin
    Assign(F, UtFileName);
    {$I-} Append(F); {$I+}
    IOStatus := IoResult;
    if (IOStatus <> FileOK) and (IOStatus <> FileNotFound) then Exit; {FATAL ERROR-Get the hell out!}
    if IOStatus = FileNotFound then Rewrite(F);
    WriteLn(F, AMsg);
    Close(F);
  end;


end.
