{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$F+,O+}
unit ClusterMon;

interface

uses
  DiskIo,
  RecTypes;

type

  SpotRec = record
              Time : String[4];
              FMHz : String[8];
              Who : String[12];
            end;

  PClusterMonitor = ^TClusterMonitor;
  TClusterMonitor = object
                      msg : String[59];
                      DataLine : String;
                      DataDone : Boolean;
                      Spotter : String[10];
                      Freq : String[8];
                      MHz : String[2];
                      What : String[12];
                      Band : BandType;
                      Comment : String[30];
                      When : String[4];
                      FName : String[79];
                      Spot : array[1..20] of SpotRec; {1234 14090 CALLSIGN}
                      refresh : Boolean;
                      a : log_rec;
                      TTYOffset : Integer;
                      NumSpots : Byte;
                      constructor Init;
                      destructor Done; virtual;
                      procedure AppendChToLine(ch : Char);
                      procedure AppendStrToLine(S : String);
                      procedure HandleAnnounce;
                      procedure HandleWWV;
                      procedure HandleNewMail;
                      procedure HandleDxSpot;
                      procedure HandleShowDxSpot;
                      procedure HandleDisConnect;
                      procedure HandleTeletypeSpot;
                      procedure AppendDataLineToFile;
                      procedure PickSpot(var R : log_rec);
                      function Message : String;
                      function SpotString(const Num : Byte) : String;
                      procedure SendSpotToCluster;
                      function NeedRefresh : Boolean;
                      procedure ProcessDataLine;
                      procedure Idle(const FromComport : Boolean);
                      procedure AddToSpots(tm, fq, wh : String);
                      procedure RemoveFromSpots(wh : String; bd : bandtype);
                      procedure ClearSpots;
                      procedure ShiftSpotsDown1;
                    end;

var
  ClusterMonitor : PClusterMonitor;



implementation

uses
  Dos,
  TpCrt,
  TpDate,
  TpDos,
  TpEntry,
  TpMouse,
  TpString,
  TpPick,
  TpWindow,
  Keys,
  c_Radio,
  CCountry,
  cRttyWin,
  Routines,
  IniFile,
  cEditWin,
  cMsgWin,
  OneTimers,
  ClusterTerm,
  Network,
  cDupeTbl,
  cColors,
  DebugRtns,
  ApPort,
  LowLvl;

  constructor TClusterMonitor.Init;
  begin
    DataLine := '';
    msg := 'Packet monitor initialized';
    Spotter := '';
    Freq := '';
    MHz := '';
    What := '';
    Comment := '';
    When := '';
    DataDone := False;
    refresh := False;
    ClearSpots;
    FName := RttyIni^.GetProfileString('FileName', 'CLUSTER', 'Cluster.txt');
    NumSpots := RttyIni^.GetProfileByte('NumSpots', 'CLUSTER', 10);
    if NumSpots = 0 then NumSpots := 5;
    if NumSpots > 20 then NumSpots := 20;
    TTYOffset := RttyIni^.GetProfileInt('TTYOffset', 'CLUSTER', 0);
    msg := 'Packet monitor initialized';
    {$IFDEF DEMO}
    Band := B80;
    AddToSpots('0135', '3587.3',  'K1AM');
    Band := B40;
    AddToSpots('0140', '7087.7',  'W6/G0AZT');
    Band := B20;
    AddToSpots('0214', '14081.4', 'K3MM');
    Band := B15;
    AddToSpots('0321', '21082.1', 'N1RCT');
    Band := B10;
    AddToSpots('0428', '28082.8', 'KO1X');
    msg := '<03> :   SFI= 72, A= 14, K= 2, SA=VL,GMF Quiet-Active';
    {$ENDIF}
    {$IFDEF DEBUG}
    AppendDebugMessage('ClusterMonitor created');
    {$ENDIF}
  end;

  destructor TClusterMonitor.Done;
  begin
    {$IFDEF DEBUG}
    AppendDebugMessage('ClusterMonitor destroyed');
    {$ENDIF}
  end;


  procedure TCLusterMonitor.ClearSpots;
  var
    b : byte;
  begin
    for b := 1 to 20 do
    begin
      Spot[b].Time := '';
      Spot[b].FMHz := '';
      Spot[b].Who := '';
    end;
  end;

  procedure TCLusterMonitor.ShiftSpotsDown1;
  var
    b : byte;
  begin
    for b := NumSpots downto 2 do begin
      Spot[b].Time := Spot[b-1].Time;
      Spot[b].FMHz := LeftPad(Trim(Spot[b-1].FMHz), 8);
      Spot[b].Who := Spot[b-1].Who;
    end;
    Spot[1].Time := '';
    Spot[1].FMHz := '';
    Spot[1].Who  := '';
  end;

  procedure ClusterBeep;
  var
    BeepFreq : Word;
    BeepLen : Word;
  begin
    if RttyIni^.GetProfileBool('Beep-OK', 'CLUSTER', True) then begin
      BeepFreq := RttyIni^.GetProfileWord('Beep-Frequency', 'CLUSTER', 1000);
      BeepLen := RttyIni^.GetProfileWord('Beep-Duration', 'CLUSTER', 250);
      Sound(BeepFreq);
      Delay(BeepLen);
      NoSound;
    end;
  end;

  function TClusterMonitor.Message : String;
  begin
    Message := msg;
    msg := '';
  end;

  function SpotFunc(Choice : Word) : String; far;
  begin
    SpotFunc := ClusterMonitor^.SpotString(Lo(Choice));
  end;

  procedure TClusterMonitor.PickSpot(var R : log_rec);
  var
    Choice  : Word;
    TopLine : byte;
    n : byte;
    s : string[11];
  begin
    Choice := 1;
    PickMatrix := 1;
    if NumSpots > 20 then
      TopLine := 1
    else
      TopLine := 22-NumSpots-1;
    PickWindow(@SpotFunc, NumSpots, 50, TopLine, 77, 22, True, Colors^.PickColors, 'Packet', Choice);
    if PickCmdNum = PKSSelect then begin
      S := Trim(Spot[Choice].FMHz);
      while Pos('.', S) > 0 do
        Delete(S, Pos('.', S), 1);
      S := Trim(S)+'00';
      if Assigned(Radio) then Radio^.ChangeFrequencyHz(S,TTYOffset);
      {$IFDEF DEBUG}
      AppendDebugMessage('Packet selected frequency is: '+Spot[Choice].FMHz);
      {$ENDIF}
      R.Call := Spot[Choice].Who;
    end;
  end;

  procedure TClusterMonitor.AppendChToLine(ch : Char);
  begin
    case ch of
      #10 : Exit;
      #13 : begin
              DataDone := True;
              Exit;
            end;
      #09 : ch := ' ';
    end;
    if Length(DataLine) < 254 then begin
      DataLine[Length(DataLine)+1] := ch;
      Inc(DataLine[0]);
    end;
  end;

  procedure TClusterMonitor.AppendStrToLine(S : String);
  begin
    DataLine := DataLine+S;
    DataDone := True;
    {$IFDEF DEBUG}
    if DataDone then
      AppendDebugMessage('TClusterMonitor.AppendStrToLine, DataDone=TRUE, DataLine='+DataLine)
    else
      AppendDebugMessage('TClusterMonitor.AppendStrToLine, DataDone=FALSE, DataLine='+DataLine)
    {$ENDIF}
  end;

  procedure TClusterMonitor.AppendDataLineToFile;
  var
    f : Text;
  begin
    if not RttyIni^.GetProfileBool('SaveToDisk', 'CLUSTER', False) then Exit;
    if not existfile(FName) then begin
      Assign(f, FName);
      Rewrite(f);
    end
    else begin
      Assign(f, FName);
      Append(f);
    end;
    Writeln(f, DataLine);
    Close(f);
  end;

  procedure TClusterMonitor.AddToSpots(tm, fq, wh : String);
  var
    b : Byte;
    ShowCurr : Boolean;
    ShowAll  : Boolean;
    ThisBand : Boolean;
  begin
    ShowCurr := RttyIni^.GetProfileBool('CurrentBand','CLUSTER',True);
    ShowAll  := RttyIni^.GetProfileBool('OtherBands','CLUSTER',True);
    ThisBand := Editw^.EditRecs[1].Band = Band;
    if (ShowCurr AND ThisBand)
       OR
       (ShowAll  AND NOT ThisBand)
    then begin
      ShiftSpotsDown1;
      Spot[1].Time := tm;
      Spot[1].FMHz := LeftPad(Trim(fq), 8);
      Spot[1].Who := wh;
    end;
  end;

  procedure TClusterMonitor.RemoveFromSpots(wh : String; bd : BandType);
  var
    i,b : Byte;
    oneToDel : Byte;

  procedure DeleteRec;
  var
    A : Byte;
  begin
    for a := oneToDel to 19 do begin
      Spot[a].Time := Spot[a+1].Time;
      Spot[a].FMHz := Spot[a+1].FMHz;
      Spot[a].Who  := Spot[a+1].Who;
    end;
    Spot[20].Time := '';
    Spot[20].FMHz := '';
    Spot[20].Who := '';
  end;

  begin
    for i := 1 to 20 do begin
      oneToDel := 0;
      for b := 1 to NumSpots do
        if (Trim(Spot[b].Who) = Trim(wh)) and
           (FreqToBand(Spot[b].Fmhz) = Bd) then begin
             oneToDel := b;
             DeleteRec;
        end;
      if oneToDel = 0 then Exit;
    end;
  end;

    procedure TClusterMonitor.HandleTeletypeSpot;
    var
      CtyW : Word;
      slot : Word;
    begin
      CtyW := 999;
      LogFile^.BlankLogRec(a);
      a.Call := What;
      CtyW := CountryList^.Identify(a);
      Band := TOT;
      if Trim(MHz) = '3' then Band := B80;
      if Trim(MHz) = '7' then Band := B40;
      if Trim(MHz) = '14' then Band := B20;
      if Trim(MHz) = '21' then Band := B15;
      if Trim(MHz) = '28' then Band := B10;
      if Band = TOT then Exit;


      {$IFDEF DEBUG}
      msg := When+Freq+' '+What+' '+Comment;
      AddToSpots(When,Freq,What);
      ClusterBeep;
      MsgWin^.Display(Msg);
      Refresh := True;
      {$ELSE}
      if (not CountryList^.GetCountry(CtyW,Band)) or
      { we have a needed country mult here !!}
      (not DupeTbl^.IsDupe(What, Band))
      { we have a needed qso here !!}
      then begin
        if (
            RttyIni^.GetProfileBool('CurrentBand','CLUSTER',True) and
            (Editw^.EditRecs[1].Band = Band)
           )
           or
           (
            RttyIni^.GetProfileBool('OtherBands','CLUSTER',True)
           ) then begin
          msg := When+' '+Freq+' '+What+' '+Comment;
          AddToSpots(When, Freq, What);
          ClusterBeep;
          MsgWin^.Display(msg);
          refresh := True;
        end;
      end;
      {$ENDIF}
    end;

  function IsItRtty(Comment,Freq : string) : boolean;
  var
    Freq2 : string[5];
  begin
    IsItRtty := True;
    {$IFDEF DEBUG}
    Exit;
    {$ENDIF}
    if RttyIni^.GetProfileBool('AllowAnySpot', 'CLUSTER', true) then Exit;
    if (Pos('TTY', Comment) > 0) or
       (Pos('TEST', Comment) > 0) or
       (Pos('FSK', Comment) > 0) then Exit;
    Freq2 := Trim(Freq);
    if RttyIni^.GetProfileBool('Loose', 'CLUSTER', True) then begin
      if (Freq2 > '7025') and  (Freq2 < '7095')  then Exit;
      if (Freq2 > '14070') and (Freq2 < '14105') then Exit;
      if (Freq2 > '21070') and (Freq2 < '21105') then Exit;
      if (Freq2 > '28070') and (Freq2 < '28105') then Exit;
    end;
    IsItRtty := False;
  end;

  procedure TClusterMonitor.HandleDxSpot;
  begin
    {----+----1----+----2----+----3----+----4----+----5----+----6----+----7----+----8}
    {DX de WA6ALQ:    14201.2  S52CD        tont,slovinia 5-9+             2203Z   }
    DataLine := StUpCase(DataLine);
    Spotter := Copy(DataLine, 7, Pos(':', DataLine)-7);
    Freq := Copy(DataLine, 17, 9);
    MHz := Copy(DataLine, 18, 2);
    What := Copy(DataLine, 27, 12);
    Comment := Copy(DataLine, 40, 30);
    When := Copy(DataLine, 71, 4);
    {$IFDEF DEBUG}
    AppendDebugMessage('ClusterMonitor: Spotter '
     +Spotter+', Freq '+
     +Freq+', MHz '+
     +Mhz+', What '+
     +What+', Comment'+
     +Comment+', When '+
     +When);
    {$ENDIF}
    if IsItRtty(Comment,Freq) then HandleTeletypeSpot;
  end;

  procedure TClusterMonitor.HandleShowDxSpot;
  begin
{
....+....1....+....2....+....3....+....4....+....5....+....6....+....7
  1831.6  OZ7C        10-Sep-1997 0232Z                                <K1VW>
  3805.0  W0CXX       10-Sep-1997 0229Z  HUMONGUS SIGNAL!             <WE2DX>
 14180.0  V63KU       10-Sep-1997 0228Z                              <W8LR-@>
  7008.7  9Y4/PA3EWP  10-Sep-1997 0227Z                                <KP3X>
 14087.3  3D2RW       10-Sep-1997 0214Z  RTTY fsk                      <K2YG>
}
    DataLine := StUpCase(DataLine);
    Spotter := Trim(Copy(DataLine, 67, 10));
    if Spotter[1] = '<' then Spotter[1] := ' ';
    if Spotter[Length(Spotter)] = '>' then Spotter[Length(Spotter)] := ' ';
    Spotter := Trim(Spotter);
    Freq := Copy(DataLine, 2, 7);
    MHz := Copy(DataLine,  2, 2);
    What := Copy(DataLine, 11, 12);
    Comment := Copy(DataLine, 42, 25);
    When := Copy(DataLine, 35, 4);
    {$IFDEF DEBUG}
    AppendDebugMessage('ClusterMonitor: Spotter '
     +Spotter+', Freq '+
     +Freq+', MHz '+
     +Mhz+', What '+
     +What+', Comment'+
     +Comment+', When '+
     +When);
    {$ENDIF}
    if IsItRtty(Comment,Freq) then HandleTeletypeSpot;
  end;

  procedure TClusterMonitor.HandleAnnounce;
{
To ALL de K2KI <0031Z> : many lids chasing vr6mw 0n 14.021.4
To ALL de K2KI <0032Z> : he sent AS several times and they still
}
  begin
    msg := Trim(Copy(DataLine, Pos(':', DataLine)-8, 59));
    MsgWin^.Display(msg);
    Alarm;
  end;

  procedure TClusterMonitor.HandleWWV;
  begin
    msg := Trim(Copy(DataLine, Pos(':', DataLine)-5, 59));
    MsgWin^.Display(msg);
    Alarm;
  end;

  procedure TClusterMonitor.HandleNewMail;
  begin
    msg := Trim(Copy(DataLine, 1, 50));
    MsgWin^.Display(msg);
    Alarm;
  end;

  procedure TClusterMonitor.HandleDisConnect;
  begin
    msg := 'Disconnected from packet';
    MsgWin^.Display(msg);
    Alarm;
  end;

  procedure TClusterMonitor.ProcessDataLine;
  const
    NotAMsg = 0;
    DxSpot = 1;
    Prompt = 2;
    Disconnected = 3;
    Announce = 4;
    WWV = 5;
    ShowDxSpot = 6;
    NewMail = 7;
  var
    MsgType : Integer;
    f : Text;

{ show dx command & results
sh/dx
....+....1....+....2....+....3....+....4....+....5....+....6....+....7....+.
  1831.6  OZ7C        10-Sep-1997 0232Z                                <K1VW>
  3805.0  W0CXX       10-Sep-1997 0229Z  HUMONGUS SIGNAL!             <WE2DX>
 14180.0  V63KU       10-Sep-1997 0228Z                              <W8LR-@>
  7008.7  9Y4/PA3EWP  10-Sep-1997 0227Z                                <KP3X>
 14087.3  3D2RW       10-Sep-1997 0214Z  RTTY fsk                      <K2YG>
WF1B de NE1R   10-Sep 0234Z >
}


  begin
    MsgType := NotAMsg;
    if (DataLine[1] = ' ') and
       (DataLine[2] in [' ','1'..'2']) and    { 10's of mhz}
       (DataLine[3] in ['1','3','7','4','8']) and { 1's of mhz}
       (DataLine[4] in ['0'..'9']) and
       (DataLine[5] in ['0'..'9']) and
       (DataLine[7] = '.') and
       (DataLine[8] in ['0'..'9']) and
       (DataLine[9] = ' ') then MsgType := ShowDxSpot;
    if Pos('DX ', DataLine) in [1, 2, 3] then MsgType := DxSpot;
    {  To ALL de K1IED <2348Z> : ANBODY KNOW IF PARTS FOR AN ALLIANCE HD73 ROTATOR CA }
    if Pos('To ALL', DataLine) in [1, 2, 3] then MsgType := Announce;
    {  WWV de VE6JO (03) : SOLAR FLARE blah blah }
    if Pos('WWV ', DataLine) in [1, 2, 3] then MsgType := WWV;
    if Pos('*** DISCONNECT', DataLine) > 0 then MsgType := Disconnected;
    if Pos('You have new mail', DataLine) > 0 then MsgType := NewMail;
    case MsgType of
      DxSpot : HandleDxSpot;
      Announce : HandleAnnounce;
      WWV : HandleWWV;
      Disconnected : HandleDisConnect;
      ShowDxSpot : HandleShowDxSpot;
      NewMail : HandleNewMail;
    end;
    AppendDataLineToFile;
    DataLine := '';
    DataDone := False;
  end;

  procedure TClusterMonitor.Idle(const FromComport : Boolean);
  begin
    if DataDone then begin
      if Assigned(Net) then begin
      { and it if data just came from a comport then send it,
        else do not send it }
        if FromComport then Net^.SendClusterRxData(DataLine);
      end;
      ProcessDataLine;
    end;
  end;

  function TClusterMonitor.NeedRefresh : Boolean;
  begin
    NeedRefresh := refresh;
    refresh := False;
  end;

  function TClusterMonitor.SpotString(const Num : Byte) : String;
  begin
    with ClusterMonitor^.Spot[Num] do
      SpotString := Time+' '+FMHz+' '+Who;
  end;

  procedure TClusterMonitor.SendSpotToCluster;
  var
    DxMsg : String;
    f : LongInt;
    Freqs : String[9];
    Calls : String[13];
    b : BandType;
    TTYMsg : String[19];

    procedure SetCallsign;
    begin
      Calls := '';
      if EditW^.EditRecs[2].Call <> '' then Calls := EditW^.EditRecs[2].Call;
      if EditW^.EditRecs[1].Call <> '' then Calls := EditW^.EditRecs[1].Call;
    end;

    procedure SetFrequency;
    begin
      if not Assigned(Radio) then begin
        if EditW^.EditRecs[2].Call <> '' then b := EditW^.EditRecs[2].Band;
        if EditW^.EditRecs[1].Call <> '' then b := EditW^.EditRecs[1].Band;
        case b of
          B80 : Freqs := '3500';
          B40 : Freqs := '7000';
          B20 : Freqs := '14000';
          B15 : Freqs := '21000';
          B10 : Freqs := '28000';
        else
          Freqs := '';
        end;
      end
      else begin
        f := Radio^.GetDisplayFrequency;
        Freqs := Trim(Long2str(f+TTYOffset));
        Freqs := Copy(Freqs,1,Length(Freqs)-2);
        Insert('.',Freqs,Length(Freqs));
      end;
    end;

    procedure EditMessage(var EdtMessage : String);
    var
      Esr : EsRecord;
      ExitCommand : esType;
      WinPtr : WindowPtr;
    begin
      with Colors^ do
      MakeWindow(WinPtr, 3, 3, 75, 7, True, True, False, WindowAttr,
        FrameAttr, HeaderAttr, 'DX Announcement');
      HideMouse;
      DisplayWindow(WinPtr);
      InitEsRecord(Esr);
      Colors^.SetEntryAttributes;
      SetWrapMode(Esr, WrapAtEdges);
      AddStringField(Esr, 'Message', 5, 5, '', 5, 14, 59, 0, nil, EdtMessage);
      FastWriteStack(WinPtr,'<ESC> to cancel, <ENTER> to send',4,2,Colors^.HeaderAttr);
      ShowMouse;
      AddCmd(ESEnd,kbEnd);
      AddCmd(ESHome,kbHome);
      AddCmd(EsQuit,kbEsc);
      AddCmd(EsMy98,kbEnter);
      repeat
        ExitCommand := EditScreen(Esr, 0, False);
      until ExitCommand in [EsQuit, EsDone, EsMy49, EsMy98];
      KillWindow(WinPtr);
      AddCmd(ESmy25, kbEnd);
      AddCmd(ESmy23, kbHome);
      AddCmd(Esmy49, kbEsc);
      AddCmd(EsMy98, $0000);
      DisposeEditScreen(Esr);
      if ExitCommand in [EsQuit,EsMy49] then EdtMessage := '';
    end;

  begin
    SetCallsign;
    SetFrequency;
    TTYMsg := RttyIni^.GetProfileString('TTYMsg', 'CLUSTER', 'RTTY');
    DxMsg := 'DX '+Calls+' '+Freqs+' '+TTYMsg;
    EditMessage(DxMsg);
    if DxMsg <> '' then ClusterTerminal^.SendMessageToCluster(DxMsg);
  end;


end.