{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$F+,O+}
unit ClusterTerm;

interface

uses
  OoCom,
  ApPort,
  ApTimer,
  TpWIndow;

type

  PClusterTerminal = ^TClusterTerminal;
  TClusterTerminal = object
                       LinesIn : array[1..20] of String[78];
                       LineIn  : string[79];
                       LineOut : String[79];
                       NeedRefresh : Boolean;
                       TermInWin  : WindowPtr;
                       TermOutWin : WindowPtr;
                       constructor Init;
                       destructor Done; virtual;
                       procedure RunTerminal;
                       procedure Idle; virtual;
                       procedure ScrollInLines;
                       procedure RefreshWindow;
                       procedure SendMessageToCluster(const AMsg : string);virtual;
                       procedure AcceptDataFromNetCluster(const AData : string); virtual;
                     end;

var
  ClusterTerminal : PClusterTerminal;
  procedure InitCluster;


implementation

uses
  Dos,
  TpDos,
  cRttyWin,
  TpString,
  TpCrt,
  NetWork,
  cCountry,
  RecTypes,
  ClusterMon,
  ClstrNet,
  ClstrCom,
  ClstrFil,
  cColors,
  lowlvl,
  cMsgWin,
  TpEdit,
  DebugRtns,
  IniFile;


  constructor TClusterTerminal.Init;
  var
    Line : Byte;
  begin
    LineIn := '';
    TermInWin  := NIL;
    TermOutWin := NIL;
    NeedRefresh := False;
    for Line := 1 to 20 do
      LinesIn[Line] := '';
    LineOut := '';
    ClusterMonitor := New(PCLusterMonitor, Init);
    if not Assigned(ClusterMonitor) then
      AppendDebugMessage('Error! Cluster Monitor not created!')
    else
      AppendDebugMessage('Cluster Monitor created');
  {$IFDEF DEBUG}
  AppendDebugMessage('ClusterTerminal created');
  {$ENDIF}
  end;

  destructor TClusterTerminal.Done;
  begin
    Dispose(ClusterMonitor, Done);
    ClusterMonitor := nil;
  {$IFDEF DEBUG}
  AppendDebugMessage('ClusterTerminal destroyed');
  {$ENDIF}
  end;

  procedure TClusterTerminal.SendMessageToCluster(const AMsg : string);
  begin
    AppendDebugMessage('TCLusterTerminal.SendMessageToCluster - Abstract');
    {$IFDEF ASSERT}
    writeln('TClusterTerminal.SendMessageToCluster - Abstract');
    halt;
    {$ENDIF}
  end;

  procedure TClusterTerminal.ScrollInLines;
  var
    Line : Byte;
  begin
    for Line := 2 to 20 do
      LinesIn[Line-1] := LinesIn[Line];
    LinesIn[20] := '';
    {$IFDEF XDEBUG}
    AppendDebugMessage('TClusterTerminal.ScrollInLines');
    {$ENDIF}
  end;

  procedure TClusterTerminal.Idle;
  begin
    {$IFDEF DEBUG}
    AppendDebugMessage('TClusterTerminal.Idle - ERROR SHOULDNT BE HERE');
    {$ENDIF}
    {$IFDEF ASSERT}
    writeln('TClusterTerminal.Idle - Abstract');
    halt;
    {$ENDIF}
  end;


  procedure TClusterTerminal.AcceptDataFromNetCluster(const aData : string);
  begin
    {$IFDEF ASSERT}
    writeln('TClusterTerminal.AcceptDataFromNetCluster - Abstract');
    halt;
    {$ENDIF}
  end;

    procedure TClusterTerminal.RefreshWindow;
    var
      Line : Byte;
    begin
      for Line := 1 to 20 do
        FastWriteStack(TermInWin, Pad(LinesIn[Line], ScreenWidth-2), Line, 1, Colors^.WindowAttr);
      NeedRefresh := False;
    end;

  function IdleGetKey : Word; far;
  begin
    with ClusterTerminal^ do begin
      while not keypressed do begin
        Idle;
        if Assigned(Net) then Net^.Idle;
        if NeedRefresh   then RefreshWindow;
      end;
      IdleGetKey := ReadKeyWord;
      {$IFDEF XDEBUG}
      AppendDebugMessage('IdleGetKey');
      {$ENDIF}
    end;
  end;

  procedure TClusterTerminal.RunTerminal;
  var
    Escaped : Boolean;
    HoldPtr : Pointer;
  begin
    with Colors^ do begin
    if not MakeWindow(TermInWin, 1, 1, ScreenWidth, ScreenHeight-3, True, True, True,
      WindowAttr, FrameAttr, HeaderAttr, 'Packet Window') then Exit;
    if not DisplayWindow(TermInWin) then Exit;
    if not MakeWindow(TermOutWin, 1, ScreenHeight-2, ScreenWidth, ScreenHeight, True, True, True,
      WindowAttr, HeaderAttr, HeaderAttr, '') then Exit;
    if not DisplayWindow(TermOutWin) then Exit;
    LineOut := '';
    HoldPtr := EditKeyPtr;
    EditKeyPtr := @IdleGetKey;
    NeedRefresh := True;
    repeat
      ActivateWindow(TermOutWin);
      ReadString('', 1, 1, 77, PromptAttr, StringAttr, CtrlAttr, Escaped, LineOut);
      if RSCommand = rsEnter then begin
        SendMessageToCluster(LineOut);
        LineOut := '';
      end;
    until (RSCommand = RSQuit) or (Escaped);
    end;
    EditKeyPtr := HoldPtr;
    KillWindow(TermInWin);
    KillWindow(TermOutWin);
  end;


procedure InitCluster;
var
  P : Byte;
  S : string[9];
begin
  ClusterTerminal := nil;
  S := StUpCase(Trim(RttyIni^.GetProfileString('ComPort', 'CLUSTER', '')));
  if S = '' then Exit;
  if S[1] in ['1'..'8'] then begin
    p := ord(s[1])-ord('0');
    ClusterTerminal := New(PComClusterTerminal, Init(ComNameType(P-1)));
    if not Assigned(ClusterTerminal) then
      AppendDebugMessage('Error! Cluster Terminal not created for COM'+ long2str(P))
    else begin
      AppendDebugMessage('Cluster Terminal created for COM'+ long2str(P));
      RttyWin^.PutString(^M+'Packet: Port is COM'+ long2str(P));
    end;
  end;
  if Copy(S,1,3) = 'NET' then begin
    ClusterTerminal := New(PNetClusterTerminal, Init);
    if not Assigned(ClusterTerminal) then
      AppendDebugMessage('Error! NetCluster Terminal not created')
    else begin
      AppendDebugMessage('NetCluster Terminal created');
      RttyWin^.PutString(^M+'Packet: Port is NET');
    end;
  end;
  if Copy(S,1,3) = 'FIL' then begin
    ClusterTerminal := New(PFileClusterTerminal, Init);
    if not Assigned(ClusterTerminal) then
      AppendDebugMessage('Error! File Cluster Terminal not created')
    else begin
      AppendDebugMessage('File Cluster Terminal created');
      RttyWin^.PutString(^M+'Packet: Port is File - '+PFileClusterTerminal(ClusterTerminal)^.FileName);
    end;
  end;

end;

end.
