{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit cMaster;

interface
uses
  OpRoot;

type
  PMastCall = ^TMastCall;
  TMastCall = object(SingleListNode)
                Callsign : string[12];
                constructor Init(const Cl : string);
              end;

  PMastList = ^TMastList;
  TMastlist = object
                CallList : SingleList;
                ListCreated : Boolean;
                SearchP : PMastCall;
                HitList : Array[1..100] of string[12];
                HitCount: Integer;
                FName   : string[29];
                constructor Init(const FileName : string);
                destructor  Done;
                procedure   Show;
                function    Exists(const Cl : string) : boolean;
                procedure   Top;
                function    Search(const Cl : string; var Close : string) : boolean;
                procedure   RefreshHitList(const Cl : string);
                procedure   ClearHitList;
                function    GetHit(const Num : integer) : string;
              end;

var
  Mastlist : PMastList;

implementation

uses
  CRttyWin,
  IniFile,
  DebugRtns,
  TpDos,
  TpCrt,
  TpString;

  constructor TMastCall.Init(const Cl : string);
  begin
    Callsign := Trim(Cl);
  end;


  constructor TMastList.Init(const FileName : string);
  var
    F : Text;
    S : string[25];
    P : PMastCall;
  begin
    FName := FileName;
    ClearHitList;
    CallList.Init; { make the link list }
    if (ExistFile(FName)) AND (RttyIni^.GetProfileBool('UseMasterFile','PERFORMANCE',True)) then
    begin
      {$I-}
      Assign(F, FName);
      Reset(F);
      while not Eof(f) do begin
        ReadLn(F, S);
        s := trim(StUpCase(s));
        if S <> '' then begin
          P := New(PMastCall, Init(s));
          CallList.Append(P);
        end;
      end;
      Close(F);
      {$I+}
    end
    else
      ListCreated := False;
    RttyWin^.PutString(^M'MASTER.CAL loaded: '+long2str(CallList.Size)+ ' callsigns');
    AppendDebugMessage('Master List, callsigns loaded: '+long2str(CallList.Size));
  end;

  destructor  TMastList.Done;
  var
    P : PMastCall;
  begin
    P := PMastCall(CallList.Head);
    while P <> NIL do begin
      CallList.Delete(P);
      P := PMastCall(CallList.Head);
    end;
    {$IFDEF DEBUG}
    AppendDebugMessage('Master List destroyed');
    {$ENDIF}
  end;

  procedure TMastList.ClearHitList;
  var
    I : Integer;
  begin
    for i := 1 to 100 do
      HitList[i] := '';
    HitCount := 0;
  end;


  function    TMastList.Exists(const Cl : string) : boolean;
  var
    P : PMastCall;
    S : string[12];
  begin
    if not ListCreated then begin
      Exists := False;
      Exit;
    end;
    S := Trim(Cl);
    P := PMastCall(CallList.Head);
    while P <> NIL do begin
      if PMastCall(P)^.Callsign = S then begin
        Exists := True;
        Exit;
      end;
      P := PMastCall(CallList.Next(P));
    end;
    Exists := False;
  end;

  procedure  TMastList.Top;
  begin
    SearchP := PMastCall(CallList.Head);
  end;

  function    TMastList.Search(const Cl : string; var Close : string) : boolean;
  var
    S : string[12];
  begin
    if not ListCreated then begin
      Search := False;
      Close := '';
      Exit;
    end;
    S := Trim(Cl);
    while SearchP <> NIL do begin
      if (Pos(S,PMastCall(SearchP)^.Callsign ) > 0) then begin
        Close := PMastCall(SearchP)^.Callsign;
        Search := True;
        if SearchP = PMastCall(CallList.Tail) then
          Top  { reset to begining for next search }
        else
          SearchP := PMastCall(CallList.Next(SearchP)); { point to NEXT one for next search }
        Exit;
      end;
      SearchP := PMastCall(CallList.Next(SearchP));
    end;
    Search := False;
    Close := '';
  end;

  procedure TMastList.Show;
  var
    I : integer;
    P : PMastCall;
  begin
    I := 0;
    P := PMastCall(CallList.Head);
    while P <> NIL do begin
      inc(i);
      writeln(I,' ',PMastCall(P)^.Callsign);
      P := PMastCall(CallList.Next(P));
    end;
  end;

  procedure   TMastList.RefreshHitList(const Cl : string);
  var
    S : string[12];
    P : PMastCall;
    w : word;
  begin
    ClearHitList;
    S := Trim(Cl);
    HitCount := 0;
    P := PMastCall(CallList.Head);
    for w := 1 to Calllist.size do begin
      if (Pos(S,PMastCall(P)^.Callsign ) > 0) then begin
        if HitCount < 100 then begin
          inc(HitCount);
          HitList[HitCount] := PMastCall(P)^.Callsign;
          if HitCount = 100 then Exit;
        end;
      end;
      P := PMastCall(CallList.Next(P));
    end;
  end;

  function    TMastList.GetHit(const Num : integer) : string;
  begin
    if Num <= HitCount then
      GetHit := HItList[Num]
    else GetHit := 'ERROR';
  end;

end.