{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{ Compiler Directives ===================================================== }
{$O+}                             {Overlays enabled}
{$F+}                             {Use far calls}
{$R-}                             {Range checking off}
{$B-}                             {Boolean complete evaluation off}
{$S-}                             {Stack checking off}
{$I-}                             {I/O checking off}
{$V-}                             {Variable range checking off}

{ Name ==================================================================== }

unit CmdLnObj;
{
	The purpose of CmdLn is to provide routines for parsing and handling
	command line parameters and parameter strings passed to a program.
}

  { Interface =============================================================== }

interface

uses OpString;

  { Declarations ============================================================ }
  { Constants --------------------------------------------------------------- }

type
  PCmdLnObj      = ^TCmdLnObj;
  TCmdLnObj      = object
                     ParameterCount : Byte; { number of params }
                     ParameterStr   : String; { store params }
                     ParameterStrArray : array[1..25] of Byte; { index to params }

                     SwitchCount    : Byte; { number of switches }
                     SwitchStr      : String; { store switches }
                     SwitchStrArray : array[1..25] of Byte; { index to switches }

                     constructor Init; { set up ParameterStr }
                     function Parameter(Index : Byte) : String; { return string }
                     function PosParameter(S : String) : Byte; { index pos of param }
                     procedure DeleteIndex(Index : Byte); { delete param by index }
                     procedure DeleteParameter(S : String); { zap all occurrences of S }

                     function Switch(Index : Byte) : String; { return switch }
                     function ExistSwitch(S : String) : Boolean; { does Switch exist }
                     function ExistSwitchCombined(S : String) : Boolean;
                     destructor Done;
                   end;

var
  CmdLn          : PCmdLnObj;


  { ========================================================================= }
  { IMPLEMENTATION ========================================================== }

implementation

uses DebugRtns;

  { ========================================================================= }
  { CmdLnOb.Init ============================================================ }

  constructor TCmdLnObj.Init;
  var
    Loop           : Byte;
    Index          : Byte;
    S              : String;

  begin
{
	First null out the ParameterStr variables and the SwitchStr variables.
}
    ParameterCount := 0;          { null out counter }
    ParameterStr := '';           { null out string }
    FillChar
    (ParameterStrArray, 0, SizeOf(ParameterStrArray)); { null out array }
    SwitchCount := 0;             { null out counter }
    SwitchStr := '';              { null out string }
    FillChar
    (SwitchStrArray, 0, SizeOf(SwitchStrArray)); { null out array }

{
	Now, loop through command line parameters.
}
    for Index := 1 to ParamCount do begin { loop through params }
      S := Trim(ParamStr(Index)); { get S }
      if                          { if }
      (S[1] <> '/') and           { S doesn't lead with '/' }
      (S[1] <> '-')               { or '-' }
      then begin                  { then it's a parameter }
{
	If the ParamStr doesn't lead with a switch char,
	then it's a parameter, not a switch.  Stash it.

	But if it's got a '/' embedded in it, then it has
	an appended switch or set of switches.
}
        Inc(ParameterCount);      { inc parameter counter }
        ParameterStrArray[ParameterCount] :=
          Succ(Length(ParameterStr)); { store parameter index }

        Loop := Pos('/', S);      { get pos of '/' }
        if Loop = 0 then          { if not found then }
          ParameterStr := ParameterStr+S+' ' { stash param }
        else begin                { else there's a switch }
          Dec(Loop);              { dec index }
          ParameterStr := ParameterStr+
                          Copy(S, 1, Loop)+' '; { stash param, leave switch }
          Delete(S, 1, Loop);     { delete param from S }
        end;
      end;                        { if S [1] <> '/' }

{
	Now, stash -s-e or other switches/p/d/q.
}
      while                       { while }
      (S[1] = '/') or             { S leads with '/' }
      (S[1] = '-')                { or '-' }
      do begin                    { do: }
        Delete(S, 1, 1);          { delete '/' lead }
        if S = '' then S := '@';  { trap nul string }
        Inc(SwitchCount);         { add to count }
        SwitchStrArray[SwitchCount] :=
          Succ(Length(SwitchStr)); { store index }

        Loop := Pos('/', S);      { get pos of mid '/' }
        if Loop = 0 then          { if no pos then }
          Loop := Pos('-', S);    { get pos of mid '-' }
        if Loop = 0 then
          SwitchStr := SwitchStr+S { stash switch }
        else begin
          Dec(Loop);              { dec index }
          SwitchStr := SwitchStr+
                       Copy(S, 1, Loop); { stash 1st switch only }
          Delete(S, 1, Loop);     { delete 1st switch }
        end;
      end;                        { While S[1] = '/', etc. }
    end;                          { for Index := 1 to ParamCount }

    ParameterStrArray[Succ(ParameterCount)] :=
      Succ(Length(ParameterStr)); { store param index }
    SwitchStrArray[Succ(SwitchCount)] :=
      Succ(Length(SwitchStr));    { store switch index }
    SwitchStr := StUpCase(SwitchStr); { upcase SwitchStr }
    {$IFDEF DEBUG}
    AppendDebugMessage('CmdLnObj created');
    {$ENDIF}
  end;

  { TCmdLnObj.Parameter ======================================================= }

  function TCmdLnObj.Parameter(Index : Byte) : String;
    { returns parameter string }

  begin
    Parameter := Copy(ParameterStr,
                      ParameterStrArray[Index],
                      ParameterStrArray[Succ(Index)]-
                      Succ(ParameterStrArray[Index]));
  end;

  { TCmdLnObj.PosParameter ==================================================== }

  function TCmdLnObj.PosParameter(S : String) : Byte; { does param exist? }
  var
    Flag           : Boolean;
    Loop           : Byte;

  begin
    PosParameter := 0;
    Flag := False;
    Loop := 0;

    repeat
      Inc(Loop);
      Flag := (CompUCString(S, Parameter(Loop)) = Equal);
    until
    Flag or (Loop = ParameterCount);

    if Flag then
      PosParameter := Loop;
  end;

  { TCmdLnObj.DeleteIndex ===================================================== }

  procedure TCmdLnObj.DeleteIndex(Index : Byte);
  var
    Loop,
    Len            : Byte;
  begin
    if Index > ParameterCount then Exit;

    Len := ParameterStrArray[Succ(Index)]-
           ParameterStrArray[Index]; { get length of word }
    Delete
    (ParameterStr, ParameterStrArray[Index], Len); { delete word }
    for Loop := Index to Succ(ParameterCount) do
      ParameterStrArray[Loop] :=
        ParameterStrArray[Succ(Loop)]-Len; { adjust array }
    Dec(ParameterCount);          { dec param count }
  end;

  { TCmdLnObj.DeleteParameter ================================================= }

  procedure TCmdLnObj.DeleteParameter(S : String);
  var
    Loop           : Byte;
  begin
    repeat
      Loop := PosParameter(S);
      if Loop > 0 then
        DeleteIndex(Loop);
    until
    Loop = 0;
  end;

  { TCmdLnObj.Switch ========================================================== }

  function TCmdLnObj.Switch(Index : Byte) : String;
    { returns switch }

  begin
    Switch := Copy(SwitchStr,
                   SwitchStrArray[Index],
                   SwitchStrArray[Succ(Index)]-
                   SwitchStrArray[Index]);
  end;

  { TCmdLnObj.ExistSwitch ===================================================== }

  function TCmdLnObj.ExistSwitch(S : String) : Boolean;
    { returns true if switch exists, then deletes it from the switch array }

  var
    Len,
    Loop           : Byte;
    Flag           : Boolean;
  begin
    Flag := False;                { nul out flag }
    S := StUpCase(S);
    Loop := 1;
    if SwitchStr = '' then begin
      ExistSwitch := False;
      Exit;  {!! 10/24/95 exit if no switches at all }
    end;

    { is switch in array? }
    repeat                        { loop through array }
      Inc(Loop);
      Flag := (S = Switch(Loop)); { looking for switch }
    until                         { until }
    Flag or                       { found }
    (Loop = SwitchCount);         { or end of array }
    ExistSwitch := Flag;
    if not Flag then Exit;        { no switch, return }

    { delete switch from array }
    Len := SwitchStrArray[Succ(Loop)]-
           SwitchStrArray[Loop];  { get length of switch }
    Delete(SwitchStr, SwitchStrArray[Loop], Len); { delete switch }
    for Loop := Loop to Succ(SwitchCount) do
      SwitchStrArray[Loop] :=
        SwitchStrArray[Succ(Loop)]-Len; { adjust array }
    Dec(SwitchCount);             { dec param count }
  end;

  { TCmdLnObj.ExistSwitchCombined ============================================= }

  function TCmdLnObj.ExistSwitchCombined(S : String) : Boolean;
    { returns true if switch exists, then deletes it from the switch array }

  var
    Flag           : Boolean;
    Loop           : Byte;
    I              : Byte;
    Len            : Byte absolute S;

  begin
    Flag := ExistSwitch(S);       { single switch? }
    if Flag then begin            { if yes then }
      ExistSwitchCombined := Flag; { return value }
      Exit;                       { and exit }
    end;

    S := StUpCase(S);             { upcase S }
    Loop := Pos(S, SwitchStr);    { get pos }
    Flag := Loop > 0;             { flag = switch exists }
    ExistSwitchCombined := Flag;  { return value }

    if Flag then                  { if switch exists then }
    repeat                        { delete all occurrences }
      if Loop > 0 then begin      { if switch exists }
        Delete(SwitchStr, Loop, Len); { delete from SwitchStr }
        for I := 1 to SwitchCount do { loop through array }
          if SwitchStrArray[I] > Loop then { if array member > Loop }
            Dec(SwitchStrArray[I]); { then dec array member }
      end;
      Loop := Pos(S, SwitchStr);  { does switch still exist? }
    until
    Loop = 0;                     { until no more occurrences }
  end;

  destructor TCmdLnObj.Done;
  begin
    {$IFDEF DEBUG}
    AppendDebugMessage('CmdLnObj destroyed');
    {$ENDIF}
  end;

  { ========================================================================= }
  { Initialization ========================================================== }

end.
