{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
unit cMsgWin;

interface

uses
  ApTimer,
  TpWindow;

type

PMsgWin = ^TMsgWin;
TMsgWin = object
             Win : WindowPtr;
             xlow,
             ylow,
             xhigh,
             yhigh : Byte; { tpcrt standard }
             Attr  : Byte;
             ET    : EventTimer;
             constructor Init;
             destructor  Done; virtual;
             procedure   Display(const Msg : string); virtual;
             procedure   Clear;
             function    AskYesOrNo(const Prompt : string) : boolean;
             function    AskString(const Prompt : string; const DefaultVal : String) : String;
             procedure   Idle;
           end;

var
  MsgWin : PMsgWin;

implementation

uses
  IniFile,
  DebugRtns,
  RecTypes,
  TpDate,
  TpEdit,
  TpString,
  TpMouse,
  CColors,
  TpCrt;


  constructor TMsgWin.Init;
  const
    NoFrame = False;
    ClearWindow = True;
    SaveWindow = True;
  begin
    xlow  := 1; { the bottom line of the screen }
    ylow  := ScreenHeight;
    xhigh := 56;
    yhigh := ScreenHeight;
    Attr  := Colors^.MessageAttr;
    with Colors^ do
      if not MakeWindow(Win, xlow, ylow, xhigh, yhigh,
          NoFrame, ClearWindow, SaveWindow,
          WindowAttr, FrameAttr, HeaderAttr, '') then
        AppendDebugMessage('TMsgWin.Init, MakeWindow(Win) failed');
    DisplayWindow(Win);
  {$IFDEF DEBUG}
  AppendDebugMessage('MsgWin created');
  {$ENDIF}
  end;

  destructor  TMsgWin.Done;
  begin
    KillWindow(Win);
  {$IFDEF DEBUG}
  AppendDebugMessage('MsgWin destroyed');
  {$ENDIF}
  end;

  procedure TMsgWin.Display(Const Msg : String);
  begin
    HideMouse;
    FastWriteStack(Win, Copy(Pad(Msg, xhigh-1),1,xhigh-1), 1, 1, Attr);
    ShowMouse;
    NewTimer(ET,Secs2Tics(RttyIni^.GetProfileWord('MsgLife','PROFILE',600)));
  end;

  procedure TMsgWin.Clear;
  begin
    HideMouse;
    FastWriteStack(Win, CharStr(' ', xhigh), 1, 1, Attr);
    ShowMouse;
  end;

  function TMsgWin.AskYesOrNo(const Prompt : string) : boolean;
  var
    HoldWinPtr : WindowPtr;
  begin
    HoldWinPtr := CurrentWindow;
    ActivateWindow(Win);
    Clear;
    HideMouse;
    AskYesOrNo := YesOrNo(Prompt, 1, 1, Attr, #0);
    ShowMouse;
    Clear;
    ActivateWindow(HoldWinPtr);
  end;

  function TMsgWin.AskString(const Prompt : string; const DefaultVal : String) : String;
  var
    HoldWinPtr : WindowPtr;
    S : string[39];
    HitEsc : Boolean;
  begin
    HoldWinPtr := CurrentWindow;
    ActivateWindow(Win);
    Clear;
    S := DefaultVal;
    HideMouse;
    ReadString(Prompt, 1, 1, 39, Attr, Attr, Attr, HitEsc, S);
    ShowMouse;
    Clear;
    ActivateWindow(HoldWinPtr);
    if HitEsc then S := '';
    AskString := S;
  end;

  procedure TMsgWin.Idle;
  begin
    if TimerExpired(ET) then begin
      Clear;
      NewTimer(ET, 9999);
    end;
  end;


end.