{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit cMyWin;

interface

uses
  TpWindow,
  TpCrt;

type

  TCursorLoc = record
                 row, col : Byte;
               end;

  PMyWin = ^TMyWin;
  TMyWin = object
           private
             Win : WindowPtr;
             DrawFrame : Boolean;
             WindowAttr : Byte;
             FrameAttr : Byte;
             HeaderAttr : Byte;
             PromptAttr : Byte;
             StringAttr : Byte;
             CtrlAttr : Byte;
             HiLiteAttr : Byte;
             Cursor : TCursorLoc;
             xhi, xlo, yhi, ylo : Byte;
             VirtScreen : VScreen;
             SaveMouseCursor : Boolean;
           public
             constructor Init(const aXlo, aYlo, aXhi, aYhi : Byte;
                              const Frame : Boolean; const aTitle : String);
             destructor Done; virtual;
             procedure Activate;
             procedure Display;
             procedure Hide;
             procedure wWrite(const S : String);
             procedure wWriteLn(const S : String);
             procedure WriteAt(const S : String;
                               const Down, Across : Byte;
                               Attr : Byte);
             procedure Move(const XDelta, YDelta : Integer);
             procedure ReSize(const XDelta, YDelta : Integer);
             procedure Scroll(const Up : Boolean;
                              const Lines : Byte);
             procedure ToFront;
             function  IsActive : Boolean;
             procedure Append(const S : String;
                              Attr : Byte);
             procedure SizeOf(var Rows, Cols : Byte);
             procedure LocationOf(var aXlo, aYlo, aXhi, aYhi : Byte);
             function  IsIn(const row, col : Byte) : Boolean;
             function  IsInTitleBar(const row, col : Byte) : Boolean;
           PRIVATE
             procedure MouseOff;
             procedure MouseOn;
           end;

implementation

uses
  DebugRtns,
  OpString,
  TpMouse,
  IniFile;

var
  T : WindowPtr;
  r, c : Byte;
  st : String;

  constructor TMyWin.Init(const aXlo, aYlo, aXhi, aYhi : Byte;
    const Frame : Boolean; const aTitle : String);
  const
    ClearWindow = True;
    SaveWindow = True;
  begin
    with RttyIni^ do begin
      WindowAttr := GetProfileByte('WindowAttr', 'COLORS', 2);
      FrameAttr := GetProfileByte('FrameAttr', 'COLORS', 4);
      HeaderAttr := GetProfileByte('HeaderAttr', 'COLORS', 10);
      PromptAttr := GetProfileByte('PromptAttr', 'COLORS', 3);
      StringAttr := GetProfileByte('StringAttr', 'COLORS', 56);
      CtrlAttr := GetProfileByte('CtrlAttr', 'COLORS', 59);
      HiLiteAttr := GetProfileByte('HiLiteAttr', 'COLORS', 7);
    end;
    xlo := aXlo;
    xhi := aXhi;
    ylo := aYlo;
    yhi := aYhi;
    DrawFrame := Frame;
    if not MakeWindow(Win, xlo, ylo, xhi, yhi, DrawFrame, ClearWindow,
      SaveWindow, WindowAttr, FrameAttr, HeaderAttr,
      aTitle) then Exit;
    with Cursor do begin
      row := 1;
      col := 1;
    end;
    if not MakeVScreen(VirtScreen, yhi-ylo, xhi-xlo) then Fail;
    ClearVScreen(VirtScreen, WindowAttr, ' ');
    {$IFDEF DEBUG}
    AppendDebugMessage('MyWin created');
    {$ENDIF}
  end;

  destructor TMyWin.Done;
  begin
    DisposeVScreen(VirtScreen);
    KillWindow(Win);
    {$IFDEF DEBUG}
    AppendDebugMessage('MyWin destroyed');
    {$ENDIF}
  end;

  procedure TMyWin.SizeOf(var Rows, Cols : Byte);
    {rows and cols "inside" the window}
  begin
    Case DrawFrame of
      TRUE : begin
               Rows := yhi-ylo-1;
               Cols := xhi-xlo-1;
             end;
      FALSE: begin
               Rows := yhi-ylo+1;
               Cols := xhi-xlo+1;
             end;
    end;
  end;

  procedure TMyWin.LocationOf(var aXlo, aYlo, aXhi, aYhi : Byte);
  { returns location of the window eg upper left col xlo=(1) ,
                                      upper left row ylo=(1)
                                      lower right col xhi=(80)
                                      lower right row yhi=(25)
  }
  begin
    aXlo := xlo;
    aXhi := xhi;
    aYlo := ylo;
    aYhi := yhi;
  end;

  function TMyWin.IsIn(const row, col : Byte) : Boolean;
  begin
    IsIn := (row > ylo) and
      (row < yhi) and
      (col > xlo) and
      (col < xhi);
  end;

  function TMyWin.IsInTitleBar(const row, col : Byte) : Boolean;
  begin
    IsInTitleBar := (row = ylo) and
      (col > xlo) and
      (col < xhi);
  end;

  procedure TMyWin.Activate;
  begin
    MouseOff;
    ActivateWindow(Win);
    MouseOn;
  end;

  procedure TMyWin.Display;
  begin
    if IsActive then
      ToFront
    else begin
      MouseOff;
      DisplayWindow(Win);
      MouseOn;
    end;
  end;

  procedure TMyWin.Hide;
  begin
    ToFront;
    MouseOff;
    EraseTopWindow;
    MouseOn;
  end;

  procedure TMyWin.wWrite(const S : String);
  begin
    Activate;
    MouseOff;
    Write(S);
    MouseOn;
  end;

  procedure TMyWin.wWriteLn(const S : String);
  begin
    Activate;
    MouseOff;
    WriteLn(S);
    MouseOn;
  end;

  procedure TMyWin.WriteAt(const S : String;
    const Down, Across : Byte;
    Attr : Byte);
  begin
    if Attr = 0 then Attr := WindowAttr;
    MouseOff;
    FastWriteStack(Win, S, Down, Across, Attr);
    MouseOn;
  end;

  procedure TMyWin.Move(const XDelta, YDelta : Integer);
  begin
    Activate;
    MoveWindow(XDelta, YDelta);
  end;

  procedure TMyWin.ReSize(const XDelta, YDelta : Integer);
  begin
    Activate;
    ReSizeWindow(XDelta, YDelta, ' ');
  end;

  procedure TMyWin.Scroll(const Up : Boolean;
    const Lines : Byte);
  begin
    Activate;
    MouseOff;
    ScrollWindow(Up, Lines);
    MouseOn;
  end;

  procedure TMyWin.ToFront;
  begin
    MouseOff;
    SetTopWindow(Win);
    MouseOn;
  end;

  function TMyWin.IsActive : Boolean;
  begin
    IsActive := WindowIsActive(Win);
  end;

  procedure TMyWin.Append(const S : String;
    Attr : Byte);
  var
    x : Byte;
  begin
    if Attr = 0 then Attr := WindowAttr;
    if Length(S)+Cursor.col < xhi-xlo then
      begin
        MouseOff;
        FastWriteStack(Win, S, Cursor.row, Cursor.col, Attr);
        MouseOn;
        Inc(Cursor.col, Length(S));
        Exit;
      end;
    for x := 1 to Length(S) do begin
      if Cursor.col = xhi-xlo then
        begin
          if Cursor.row = yhi-ylo-1 then
            begin
              T := CurrentWindow;
              ActivateVScreen(VirtScreen);
              MouseOff;
              ActivateWindow(Win);
              MoveWindowToVScreen(VirtScreen, 1, 1);
              MoveVScreenToWindow(VirtScreen, 2, 1);
              ActivateWindow(T);
              MouseOn;
              DeactivateVScreen;
            end
          else
            Inc(Cursor.row);
          Cursor.col := 1;
        end;
      MouseOff;
      FastWriteStack(Win, S[x], Cursor.row, Cursor.col, Attr);
      MouseOn;
      Inc(Cursor.col);
    end;
  end;

  procedure TMyWin.MouseOff;
  begin
    SaveMouseCursor := MouseInstalled and MouseCursorOn;
    if SaveMouseCursor then HideMouse;
  end;

  procedure TMyWin.MouseOn;
  begin
    if SaveMouseCursor then ShowMouse;
  end;

end.
