{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit cp38;

interface

uses
  TncObject,
  ApTimer;

const
  FIFO_INPUT_OFFSET = $00;
  FIFO_OUTPUT_OFFSET = $02;
  FIFO_STATUS_OFFSET = $04;
  IMMEDIATE_ABORT_OFFSET = $06;
  HARDWARE_RESET_OFFSET = $08;

  ControllerProgram = 'P38FSK.S28';
  DspProgram        = 'P38.LOD';

  Start_dsp_program_download = $8000;
  Start_controller_program_download = $8001;
  End_program_download = $8002;
  Software_download_successful = $800A;
  Software_download_failed = $800B;
  Software_WrongVersion = $800C;
  ForceRttyTx = $800D;
  ForceRttyRx = $800E;

  Immediate_abort = $8006;
  Select_baudot = $8085;
  force_ltrs_shift = $8089;
  force_figs_shift = $808A;
  received_first_character_on_line = $808E;

  ccitt_on = $80C0;
  us_baudot_on = $80D0;

  disable_diddles = $80C1;
  enable_diddles = $80D1;

  enable_usos = $80D3;
  disable_usos = $80C3;

  set_control_delay = $80E0;
  set_transmit_delay = $80E1;
  set_audio_delay = $80E2;
  ptt_on_delay = $80E3;
  set_baudot_speed = $80E4;
  set_print_squelch_level = $80E8;
  set_filter_bw = $80EB;
  ptt_off_delay = $80ED;

  set_baudot_speed_45 = $00;
  set_baudot_speed_50 = $01;
  set_baudot_speed_57 = $02;
  set_baudot_speed_75 = $03;
  set_baudot_speed_100 = $04;

  set_filter_bw_narrow = $00;
  set_filter_bw_medium = $01;
  set_filter_bw_wide = $02;

  get_dsp_software_version = $8076;
  get_controller_software_version = $8077;
  get_eprom_software_version = $8078;
  get_eprom_checksum = $8079;

  status = $807A;
  fsk_tx = $8006;
  fxk_rx = $8007;
  command_error = $807F;

  enable_echo_as_sent = $8059;
  disable_echo_as_sent = $8049;

  use_normal_tones = $805B;
  use_inverted_tones = $804B;

  switch_to_receive_data_chars = $8030;
  switch_to_transmit_data_chars = $8031;

  hardware_reset_ack = $8009;


  FilterWidthString : array[0..2] of String[6] = ('Narrow','Medium', 'Wide');
  FilterWidthDefault = set_filter_bw_wide;


type
  PP38 = ^TP38;
  TP38 = object(Ttnc)
               StatusWord : Word;
               Input_Port : Word; {+$00}
               Output_Port : Word; {+$02}
               Status_Port : Word; {+$04}
               ImmediateAbort_Port : Word; {+$06}
               HardwareReset_Port : Word; {+$08}
	       DisplayString1 : string[20];
	       DisplayString2 : string[20];
               DisplayAttr1 : byte;
               DisplayAttr2 : byte;
               MarkCt    : array[1..255] of integer;
               SpaceCt   : array[1..255] of integer;
               MarkLoCt  : array[1..255] of integer;
               SpaceHiCt : array[1..255] of integer;
               FreqAttr  : byte;
               ZeroTuneRev : Boolean;
               CollectNumbers : boolean;
               MarkLo : byte;
	       Mark : byte;
	       Space : Byte;
               SpaceHi : byte;
               Scale : Byte;
               ProgramPath : string[39];
               ExtraDelay : LongInt;
               ControllerProgram : string[12];
               DspProgram        : string[12];
	       constructor Init;
               destructor Done; virtual;
               procedure ReadStatusPort;
               function InputFifoFull : boolean;
               function InputFifoEmpty : Boolean;
               function InputFifoHalf : Boolean;

               function OutputFifoFull : boolean;
               function OutputFifoEmpty : Boolean;
               function OutputFifoHalf : Boolean;
	       function OutputFifoHasData : Boolean;
               procedure WaitOutputFifoData;

	       function ReadOutputPort : Word;
	       function ReadOutputPortWait : Word;
               function IsData(const w : Word) : Boolean;
               function DownloadDspProgram : Boolean;
               function DownloadControllerProgram : Boolean;
               function RoomInFifo : Boolean;
               procedure WaitRoomInFifo;
	       procedure SendCommand(const w : Word);
               procedure DrainOutFifo(const Silent : Boolean);
               procedure Setup;
               procedure SendByte(const Data : Byte); virtual;
               procedure SendChar(const Data : Char); virtual;
               procedure Transmit(const Xmit : Boolean); virtual;
               procedure SendString(const Data : String); virtual;
               procedure Receive(var Ch : Char); virtual;

               procedure Setusos(const usos : Boolean); virtual;
               procedure SetDiddles(const diddles : Boolean); virtual;
               procedure SetBaudRate(const Rate : Byte); virtual;
               procedure SetMode(const Mode : String); virtual;
               procedure SetFilterWidth(const FilterWidth : Byte); virtual;
               procedure SetFskPolarity(const FskPolarity : boolean); virtual;
               procedure SetSquelchLevel(const SquelchLevel : Byte); virtual;
               procedure SetDisplay(const Display : byte); virtual;
               procedure SetDisplayScale(const DisplayScale : Byte); virtual;
               procedure Letters; virtual;
               procedure Figures; virtual;
               procedure SetSense(Const Sense : Boolean); virtual;
               procedure ClearXmitBuffer; virtual;
               procedure StopEverything; virtual;
               procedure SetMarkFreq(const MarkFreq : word); virtual;
               procedure SetSpaceFreq(const SpaceFreq : word); virtual;
               procedure SetBothFreqs;

               procedure UpdateDisplay;
               procedure UpdateFreqDelta;
               procedure GetDisplayData;
               procedure OneBarDisplay;
               procedure TwoBarDisplay;
               procedure FourBarDisplay;
               procedure ZeroCenterDisplay;
               procedure TwoBarTouchDisplay;
               function  ZeroCenterString : string;
               procedure ToggleTncTalk; virtual;
               procedure GetZero(var NumForZero : shortint; var DispAttr : byte); virtual;
               procedure UpdateFilterStrings(var S1, S2 : string; var A1, A2 : Byte); virtual;
             end;

implementation
uses
  TpCrt,
  cRttyWin,
  TpDos,
  TpString,
  TpEntry,
  TpWindow,
  TpMouse,
  Routines,
  RecTypes,
  Keys,
  LowLvl,
  OpInline,
  IniTools,
  ApPort,
  OneTimers,
  DebugRtns,
  cColors,
  IniFile;




  procedure IncChoice(var Value; FieldID : Byte;
    Factor : Integer;
    var St : String); far;
  var
    Code : Boolean absolute Value;
    Switch : Boolean absolute Value;
    Polarity : Boolean absolute Value;
    Speed : Byte absolute Value;
    FilterWidth : Byte absolute Value;
    Display : Byte absolute Value;
    EOL : Byte absolute Value;
  begin
    case FieldID of
      001,
      002,
      005 : begin
              if Factor <> 0 then Switch := not Switch;
              St := OnOff[Switch];
            end;
      003 : begin
              case Factor of
                 1 : if EOL < 3 then Inc(EOL) else EOL := 0;
                -1 : if EOL > 0 then Dec(EOL) else EOL := 3;
              end;
              St := EndOfLineChoices[EOL];
            end;
      000,
      007 : begin
              if Factor <> 0 then Polarity := not Polarity;
              St := PolarityChoices[Polarity];
            end;
      008 : begin
              case Factor of
                 1 : if FilterWidth < 2 then Inc(FilterWidth) else FilterWidth := 0;
                -1 : if FilterWidth > 0 then Dec(FilterWidth) else FilterWidth := 2;
              end;
              St := FilterWidthString[FilterWidth];
            end;
      009 : begin
              case Factor of
                 1 : if Display < 5 then Inc(Display) else Display := 0;
                -1 : if Display > 0 then Dec(Display) else Display := 5;
              end;
              St := DisplayS[Display];
            end;
    end;
  end;

  procedure TP38.Setup;
  var
    Esr : EsRecord;
    SetupPtr : WindowPtr;
    ExitCommand : EStype;
    aPolarity : Boolean;
    aUSOS : Boolean;
    aDiddles : Boolean;
    aEndOfLine : byte;
    aPttDelayOn : word;
    aPttDelayOff : word;
    aSpeed : byte;
    aSquelchEnabled : boolean;
    aSquelchLevel : byte;
    aFskPolarity : boolean;
    aFilterWidth : byte;
    aDisplay : byte;
    aDisplayScale : byte;
  begin
    { Get current values }
    aPolarity          := GetSense;
    aUSOS           := GetUsos;
    aDiddles        := GetDiddles;
    aEndOfLine      := GetEol;
    aSpeed          := GetBaudRate;
    aSquelchEnabled := GetSquelchEnabled;
    aSquelchLevel   := GetSquelchLevel;
    aFskPolarity       := GetFskPolarity;
    aFilterWidth    := GetFilterWidth;
    aDisplay        := GetDisplay;
    aDisplayScale   := GetDisplayScale;
    with Colors^ do
      if not MakeWindow(SetupPtr, 2, 2, 40, 18, True, True, False, WindowAttr,
        FrameAttr, HeaderAttr, 'P38 Setup Menu') then Exit;
    HideMouse;
    DisplayWindow(SetupPtr);
    InitEsRecord(Esr);
    Colors^.SetEntryAttributes;
    SetWrapMode(Esr, WrapAtEdges);
{0} AddChoiceField(Esr, 'Polarity', 5, 3, 'XXXXXXX',
      5, 24, 0, 1, @IncChoice, aPolarity);
{1} AddChoiceField(Esr, 'UnShift On Space', 6, 3, 'XXX',
      6, 24, 0, 1, @IncChoice, aUSOS);
{2} AddChoiceField(Esr, 'Sync Idle', 7, 3, 'XXX',
      7, 24, 0, 1, @IncChoice, aDiddles);
{3} AddChoiceField(Esr, 'End of line', 8, 3, 'XXXXXXXXXX',
      8, 24, 0, 1, @IncChoice, aEndOfLine);
{4} AddByteField(Esr, 'Speed',  9, 3, 'XXX',
      9 , 24, 0, 45, 100, aSpeed);
{5} AddChoiceField(Esr, 'Print Squelch', 10, 3, 'XXX',
      10, 24, 0, 1, @IncChoice, aSquelchEnabled);
{6} AddByteField(Esr,   '        Level', 11, 3, '999',
      11, 24, 0, 0, 127, aSquelchLevel);
{7} AddChoiceField(Esr, 'FSK Polarity', 12, 3, 'XXXXXXX',
      12, 24, 0, 1, @IncChoice, aFSKPolarity);
{8} AddChoiceField(Esr, 'Filter Width', 13, 3, 'XXXXXX',
      13, 24, 0, 1, @IncChoice, aFilterWidth);
{9} AddChoiceField(Esr, 'Tuning Display', 14, 3, 'XXXXXXXXXXXXXX',
      14, 24, 0, 1, @IncChoice, adisplay);
{10}AddByteField(Esr, 'Display Scale', 15, 3, '999',
      15, 24, 0, 1, 200, aDisplayScale);
    CtrlEnter(18, 4);
    AddCmd(ESEnd, kbEnd);
    AddCmd(ESHome, kbHome);
    AddCmd(EsQuit, kbEsc);
    repeat
      ExitCommand := EditScreen(Esr, 0, False);
    until ExitCommand in [EsQuit, EsDone];
    AddCmd(ESmy25, kbEnd);
    AddCmd(ESmy23, kbHome);
    AddCmd(Esmy49, kbEsc);
    KillWindow(SetupPtr);
    DisposeEditScreen(Esr);
    if ExitCOmmand = EsDone then begin
      {set new values}
        SetSense(aPolarity);
        SetUsos(aUsos);
        SetDiddles(aDiddles);
        SetEol(aEndOfLine);
        SetBaudRate(aSpeed);
        SetSquelchEnabled(aSquelchEnabled);
        SetSquelchLevel(aSquelchLevel);
        SetFskPolarity(aFskPolarity);
        SetFilterWidth(aFilterWidth);
        SetDisplay(aDisplay);
        SetDisplayScale(aDisplayScale);
        If aDisplay = 0 then begin
          SendCommand($804D);  {disable FSK Tuning Status}
          SendCommand($8041);  {disable Auto Channel Statistics Reports}
          end
        else begin
          SendCommand($805D);  {enable FSK Tuning Status}
          SendCommand($8051);  {enable Auto Channel Statistics Reports}
          end;
    end;
  end;




  procedure TP38.SendCommand(const w : Word);
  begin
    MyDelayTicks(ExtraDelay);
    WaitRoomInFifo;
    Portw[Input_Port] := w;
  end;

  destructor TP38.Done;
  var
   F : text;
   I : Integer;
  begin
    SendCommand($804D);           {disable FSK Tuning Status}
    SendCommand($8041);           {disable Auto Channel Statistics Reports}
    Delay(500);
    if CollectNumbers then begin
      Assign(F, 'P38.RPT');
      Rewrite(F);
      writeln(F,'Value',chr(9),'Mark Lo Ct',chr(9),'Mark Ct',chr(9),'Space Ct',chr(9),'Space Hi Ct');
      for I := 1 to 255 do
        writeln(F,i,chr(9),MarkLoct[i],chr(9),Markct[i],chr(9),Spacect[i],chr(9),SpaceHiCt[i]);
      Close(F);
    end;
    {$IFDEF DEBUG}
    AppendDebugMessage('P38 destroyed');
    {$ENDIF}
  end;

  function TP38.ReadOutputPortWait : Word;
  begin
    WaitOutputFifoData;
    ReadOutputPortWait := Portw[Output_Port] and $80FF
  end;

  function TP38.ReadOutputPort : Word;
  begin
    if OutputFifoHasData then
      ReadOutputPort := Portw[Output_Port] and $80FF
    else
      ReadOutputPort := $0000;
  end;

  function TP38.InputFifoEmpty : Boolean;
  begin
    ReadStatusPort;
    InputFifoEmpty := not FlagIsSet(StatusWord, $0001);
  end;

  function TP38.InputFifoHalf : Boolean;
  begin
    ReadStatusPort;
    InputFifoHalf := not FlagIsSet(StatusWord, $0002);
  end;

  function TP38.InputFifoFull : boolean;
  begin
    ReadStatusPort;
    InputFifoFull := not FlagIsSet(StatusWord, $0004);
  end;

  function TP38.RoomInFifo : Boolean;
  begin
    RoomInFifo := NOT InputFifoFull;
  end;

  procedure TP38.WaitRoomInFifo;
  var
    et : eventtimer;
  begin
    newtimer(et,36); {wait max 2 seconds}
    repeat until RoomInFifo or TimerExpired(et);
  end;

  procedure TP38.ReadStatusPort;
  begin
    StatusWord := Portw[Status_Port] and $0707;
  end;

  function TP38.OutputFifoEmpty : Boolean;
  begin
    ReadStatusPort;
    OutputFifoEmpty := not FlagIsSet(StatusWord, $0100);
  end;

  function TP38.OutputFifoHalf : Boolean;
  begin
    ReadStatusPort;
    OutputFifoHalf := not FlagIsSet(StatusWord, $0200);
  end;

  function TP38.OutputFifoFull : boolean;
  begin
    ReadStatusPort;
    OutputFifoFull := not FlagIsSet(StatusWord, $0400);
  end;

  function TP38.OutputFifoHasData : Boolean;
  begin
    OutputFifoHasData := not OutputFifoEmpty;
  end;

  procedure TP38.WaitOutputFifoData;
  var
    et : eventtimer;
  begin
    newtimer(et,18);
    repeat until OutputFifoHasData or timerexpired(et);
  end;

  function TP38.IsData(const w : Word) : Boolean;
  begin
    IsData := not FlagIsSet(w, $8000);
  end;


  function TP38.DownloadDspProgram : Boolean;
  var
    f : file;
    w : Word;
    I : integer;
    NumRead : Word;
    Buf : array[1..4096] of Byte;
  begin
    DownloadDspProgram := False;
    Assign(f, ProgramPath+DspProgram); {.lod}
    Reset(f, 1);
    SendCommand(Start_dsp_program_download); {begin dsp program load}
    if ReadOutputPortWait <> Start_dsp_program_download then Exit;
    w := $0000;
    repeat
      BlockRead(f, Buf, SizeOf(Buf), NumRead);
      for i := 1 to NumRead do begin
        w := Buf[i];
        WaitRoomInFifo;
        Portw[Input_Port] := w;
      end;
    until NumRead < SizeOf(Buf);
    Close(f);
    WaitRoomInFifo;
    SendCommand(End_program_download); {end dsp program load}
    if ReadOutputPortWait <> End_program_download then Exit;
    if ReadOutputPortWait = Software_download_successful then DownloadDspProgram := True;
  end;

  function TP38.DownloadControllerProgram : Boolean;
  var
    f : file;
    w : Word;
    i : integer;
    NumRead : Word;
    Buf : array[1..4096] of Byte;
  begin
    DownloadControllerProgram := False;
    Assign(f, ProgramPath+ControllerProgram);
    Reset(f, 1);
    SendCommand(Start_controller_program_download);
    if ReadOutputPortWait <> Start_controller_program_download then Exit;
    w := $0000;
    repeat
      BlockRead(f, Buf, SizeOf(Buf), NumRead);
      for i := 1 to NumRead do begin
        w := Buf[i];
        WaitRoomInFifo;
        Portw[Input_Port] := w;
      end;
    until NumRead < SizeOf(Buf);
    Close(f);
    SendCommand(End_program_download); {end dsp program load}
    if ReadOutputPortWait <> End_program_download then Exit;
    if ReadOutputPortWait = Software_download_successful then DownloadControllerProgram := True;
  end;


  procedure TP38.DrainOutFifo(const Silent : Boolean);
  begin
    while OutputFifoHasData do
      case Silent of
        True : ReadOutputPort;
        False : RttyWin^.PutString('draining...'+hexw(ReadOutputPort));
      end;
  end;


  procedure TP38.SendByte(const Data : Byte);
  begin
    Case Data of
    $0D : begin
            WaitRoomInFifo;
            Portw[Input_Port] := word($0D);
            WaitRoomInFifo;
            Portw[Input_Port] := word($0A);
          end;
    else
      WaitRoomInFifo;
      Portw[Input_Port] := word(Data);
    end;
  end;

  procedure TP38.SendChar(const Data : Char);
  begin
    SendByte(Ord(Data));
  end;



  procedure TP38.SendString(const Data : String);
  var
    i : Integer;
  begin
    if Length(Data) = 0 then Exit;
    for i := 1 to Length(Data) do
      SendChar(Data[i]);
  end;


  procedure TP38.Transmit(const Xmit : Boolean);
  begin
    case Xmit of
      True : SendCommand(ForceRttyTx);
      False : SendCommand(ForceRttyRx);
    end;
  end;

  procedure TP38.Receive(var Ch : Char);
  var
    w : Word;
  begin
    Ch := #00;
    w := ReadOutputPort;
    case IsData(w) of
      False :                     {command or status}
        case w of
          $8030 : RttyWin^.PutString(^Z);
          $8031 : RttyWin^.PutString(^Y);
          $8070 : begin
                   GetDisplayData;
                   UpdateDisplay;
                  end;
          $807E : UpdateFreqDelta;
        {$IFDEF DEBUG}
          $807A : if ReadOutputPort = $8007 then RttyWin^.PutString(^Z){now rx}
                  else RttyWin^.PutString(^Y);{now tx}
          $800D : RttyWin^.PutString(^Y);
          $800E : RttyWin^.PutString(^Z);
          $80E8 : RttyWin^.PutString(^M'SQUELCH LEVEL SET');
          $80EC : RttyWin^.PutString(^M'MARK/SPACE SET');
          $80C3 : RttyWin^.PutString(^M'USOS OFF');
          $80D3 : RttyWin^.PutString(^M'USOS ON');
          ccitt_on : RttyWin^.PutString(^M'CCITT CODE');
          us_baudot_on : RttyWin^.PutString(^M'US CODE');
          disable_diddles : RttyWin^.PutString(^M'DIDDLES OFF');
          enable_diddles : RttyWin^.PutString(^M'DIDDLES ON');
          force_figs_shift : RttyWin^.PutString(^M'FORCED FIGS');
          force_ltrs_shift : RttyWin^.PutString(^M'FORCED LTRS');
          set_baudot_speed : RttyWin^.PutString(^M'SPEED SET');
          set_filter_bw : RttyWin^.PutString(^M'FILTER BW SET');
          use_normal_tones : RttyWin^.PutString(^M'NORMAL TONES');
          use_inverted_tones : RttyWin^.PutString(^M'REVERSE TONES');
          enable_echo_as_sent : RttyWin^.PutString(^M'ECHO AS SENT');
          ptt_on_delay : RttyWin^.PutString(^M'PTT on delay set to 200ms');
          ptt_off_delay : RttyWin^.PutString(^M'PTT off delay set to 100ms');
          immediate_abort : RttyWin^.PutString(^M'IMMEDIATE ABORT ACK');
          $80DB : RttyWin^.PutString(^M'Normal FSK');
          $80CB : RttyWin^.PutString(^M'Inverted FSK');
	else begin
          RttyWin^.PutString('status:'+hexw(w)+':'^G);
          end;
        {$ENDIF}
        end;
      True : begin  {data}
	  Ch := Chr(Lo(w));
        end;
    end;
  end;


  procedure TP38.SetMode(const Mode : string);
  var
    Feedback : Word;
  begin
    inherited Setmode(mode);
    if Mode = 'RTTY' then begin
      SendCommand(Select_baudot);   {rtty}
      Feedback := ReadOutputPort;
      {$IFDEF DEBUG}
      if Feedback <> Select_baudot then AppendDebugMessage('WARNING: Starting rtty no ack');
      {$ENDIF}
      if ReadOutputPort = $807A then
        if ReadOutputPort = $8007 then
          if ReadOutputPort = $8030 then RttyWin^.PutString(^M'RTTY mode enabled.');
    end;
  end;


  procedure TP38.SetBaudRate(const Rate : Byte);
  begin
    inherited SetBaudRate(Rate);
    SendCommand($80E4);
    SendCommand($8000+SpeedToByte(Long2str(Rate)));
  end;

  procedure  Tp38.SetBothFreqs;
  var
    SpaceFreq : word;
    MarkFreq  : word;
    SpaceFLo, SpaceFHi : byte;
    MarkFLo, MarkFHi  : byte;
  begin
    SpaceFreq := GetSpaceFreq;
    MarkFreq := GetMarkFreq;
    SpaceFHi := Hi(SpaceFreq AND $FF00);
    SpaceFLo := Lo(SpaceFreq AND $00FF);
    MarkFHi := Hi(MarkFreq AND $FF00);
    MarkFLo := Lo(MarkFreq AND $00FF);
    SendCommand($80EC);
    SendCommand($8000+MarkFHi);
    SendCommand($8000+MarkFLo);
    SendCommand($8000+SpaceFHi);
    SendCommand($8000+SpaceFLo);
  end;

  procedure  Tp38.SetMarkFreq(const MarkFreq : word);
  begin
    inherited SetMarkFreq(MarkFreq);
    SetBothFreqs;
  end;

  procedure  Tp38.SetSpaceFreq(const SpaceFreq : word);
  begin
    inherited SetSpaceFreq(SpaceFreq);
    SetBothFreqs;
  end;



  procedure TP38.SetFskPolarity(const FskPolarity : boolean);
  begin
    inherited SetFskPolarity(FskPolarity);
    case FskPolarity of
      True  : SendCommand($80DB); {fsk norm command}
      False : SendCommand($80CB); {fsk rev command}
    end;
  end;

  procedure TP38.SetFilterWidth(const FilterWidth : Byte);
  begin
    inherited SetFilterWidth(FilterWidth);
    SendCommand(set_filter_bw);
    SendCommand($8000+FilterWidth);
  end;

  procedure TP38.SetDiddles(const diddles : Boolean);
  begin
    inherited SetDiddles(Diddles);
    case diddles of
      True : SendCommand(enable_diddles);
      False : SendCommand(disable_diddles);
    end;
  end;

  procedure TP38.SetSquelchLevel(const SquelchLevel : Byte);
  begin
    inherited SetSquelchLevel(SquelchLEvel);
    if GetSquelchEnabled then begin
      SendCommand(set_print_squelch_level); {set squelch}
      SendCommand($8000+SquelchLevel);   {to a some value}
    end
    else begin
      SendCommand(set_print_squelch_level); {set squelch}
      SendCommand($8000);                   {to no level at all}
    end;
  end;

  procedure TP38.SetDisplay(const Display : byte);
  begin
    inherited SetDisplay(Display);
    Case Display = 0 of {get auto results from board}
      FALSE: begin
              SendCommand($805D); {enable FSK Tuning Status}
              SendCommand($8051); {enable Auto Channel Statistics Reports}
             end;
      TRUE:  begin
               SendCommand($804D);  {disable FSK Tuning Status}
               SendCommand($8041);  {disable Auto Channel Statistics Reports}
             end;
    end;
  end;

  procedure TP38.SetDisplayScale(const DisplayScale : Byte);
  begin
    inherited SetDisplayScale(DisplayScale);
    Scale := DisplayScale;
  end;






  procedure TP38.SetUsos(const usos : Boolean);
  begin
    inherited SetUsos(Usos);
    case usos of
      True : SendCommand(enable_usos);
      False : SendCommand(disable_usos);
    end;
  end;

  procedure TP38.Letters;
  begin
    SendCommand(force_ltrs_shift);
  end;

  procedure TP38.Figures;
  begin
    SendCommand(force_figs_shift);
  end;

  procedure TP38.SetSense(const Sense : boolean);
  begin
    case Sense of
      True: SendCommand(use_normal_tones);
      False:SendCommand(use_inverted_tones);
    end;
    inherited SetSense(Sense);
  end;

  procedure TP38.ClearXmitBuffer;
  begin
    if Portw[ImmediateAbort_Port] = 0 then {};
  end;


  procedure TP38.GetDisplayData;

  procedure OverMaxIntAlarm;
  begin
    Sound(500);delay(250);
    Sound(800);delay(250);
    Sound(500);delay(250);
    Sound(800);delay(250);
  end;

  begin
    MarkLo := lo(ReadOutputPortWait);
    ReadOutputPortWait;
    Mark  := lo(ReadOutputPortWait);
    ReadOutputPortWait;
    Space := lo(ReadOutputPortWait);
    ReadOutputPortWait;
    SpaceHi := lo(ReadOutputPortWait);
    ReadOutputPortWait;

    if CollectNumbers then begin
      if MarkLoCt[MarkLo] < maxint then inc(MarkLoCt[Marklo])
        else OverMaxIntAlarm;
      if MarkCt[Mark] < maxint then inc(MarkCt[Mark])
        else OverMaxIntAlarm;
      if SpaceCt[Space] < maxint then inc(SpaceCt[space])
        else OverMaxIntAlarm;
      if SpaceHict[SpaceHi] < maxint then inc(SpaceHiCt[spacehi])
        else OverMaxIntAlarm;
    end;

    if MarkLo  > Scale then Dec(MarkLo,Scale)
                              else MarkLo := 0;
    if Mark    > Scale then Dec(Mark,Scale)
                              else Mark := 0;
    if Space   > Scale then Dec(Space,Scale)
                              else Space := 0;
    if SpaceHi > Scale then Dec(SpaceHi,Scale)
                              else SpaceHi := 0;


  end;

  procedure TP38.FourBarDisplay;
  const
    SpCh = 223;
    MkCh = 220;
    BtCh = 219;
    DisplayIncr = 3;
  var
    Diff : byte;
    s : string[3];
    MarkLoLeds : byte;
    MarkLeds   : byte;
    SpaceLeds  : byte;
    SpaceHiLeds: byte;
  begin
    MarkLoLeds := (MarkLo div DisplayIncr) div 2;
    MarkLeds   := Mark div DisplayIncr;
    SpaceLeds  := Space div DisplayIncr;
    SpaceHiLeds:= (SpaceHi div DisplayIncr) div 2;

    Diff := abs(MarkLeds - MarkLoLeds);

    if Diff > 19 then Diff := 19;

    if MarkLoLeds > MarkLeds then
      DisplayString1 := CharStr('',MarkLeds)+CharStr('',Diff)
      else
      DisplayString1 := CharStr('',MarkLoLeds)+CharStr('',Diff);

    Diff := abs(SpaceHiLeds - SpaceLeds);

    if Diff > 19 then Diff := 19;

    if SpaceLeds > SpaceHiLeds then
      DisplayString2 := CharStr('',SpaceHiLeds)+CharStr('',Diff)
      else
      DisplayString2 := CharStr('',SpaceLeds)+CharStr('',Diff);

    DisplayAttr1 := Colors^.PromptAttr;
    DisplayAttr2 := Colors^.PromptAttr;
    DisplayString1 := Pad(DisplayString1,Sizeof(DisplayString1));
    DisplayString2 := Pad(DisplayString2,SizeOf(DisplayString2));
  end;




  procedure TP38.TwoBarDisplay;
  const
    DisplayIncr = 3;
  var
    MarkLeds   : integer;
    SpaceLeds  : integer;
  begin
    MarkLeds  := Mark  div DisplayIncr;
    SpaceLeds := Space div DisplayIncr;

    if MarkLeds  > 39 then MarkLeds  := 39;
    if SpaceLeds > 39 then SpaceLeds := 39;

    if Odd(MarkLeds) then  {its an odd number if true}
      DisplayString1 := CharStr('',MarkLeds div 2)+''
    else
      DisplayString1 := CharStr('',MarkLeds div 2);

    if Odd(SpaceLeds) then {its an odd number if true}
      DisplayString2 := CharStr('',SpaceLeds div 2)+''
    else
      DisplayString2 := CharStr('',SpaceLeds div 2);

    DisplayString1 := Pad(DisplayString1,Sizeof(DisplayString1));
    DisplayString2 := Pad(DisplayString2,Sizeof(DisplayString2));
    DisplayAttr1 := Colors^.PromptAttr;
    DisplayAttr2 := Colors^.PromptAttr;
  end;

  procedure TP38.TwoBarTouchDisplay;
  const
    DisplayIncr = 3;
  var
    MarkLeds   : integer;
    SpaceLeds  : integer;
  begin
    MarkLeds  := Mark  div DisplayIncr;
    SpaceLeds := Space div DisplayIncr;

    if MarkLeds  > 39 then MarkLeds  := 39;
    if SpaceLeds > 39 then SpaceLeds := 39;

    if Odd(MarkLeds) then  {its an odd number if true}
      DisplayString1 := CharStr('',MarkLeds div 2)+''
    else
      DisplayString1 := CharStr('',MarkLeds div 2);

    if Odd(SpaceLeds) then {its an odd number if true}
      DisplayString2 := ''+CharStr('',SpaceLeds div 2)
    else
      DisplayString2 := CharStr('',SpaceLeds div 2);

    DisplayString2 := Pad(DisplayString1,10) + LeftPad(DisplayString2,10);
    DisplayString1 := ZeroCenterString;
    DisplayAttr1 :=FreqAttr;
    DisplayAttr2 := Colors^.PromptAttr;
  end;

  procedure TP38.OneBarDisplay;
  const
    DisplayIncr = 3;
  var
    Leds   : integer;
  begin
    Leds  := (Mark + Space) div DisplayIncr;

    if Leds  > 40 then Leds  := 40;

    if Odd(Leds) then  {its an odd number if true}
      DisplayString2 := CharStr('',Leds div 2)+''
    else
      DisplayString2 := CharStr('',Leds div 2);
    DisplayString1 := ZeroCenterString;
    DisplayString2 := Pad(DisplayString2,Sizeof(DisplayString2));
    DisplayAttr1 := FreqAttr;
    DisplayAttr2 := Colors^.PromptAttr;
  end;

  function TP38.ZeroCenterString : string;
  var
    Led : integer;
    LedCh : char;
  begin
    {limiter circuit}
    If FreqDelta < -80 then FreqDelta := -80;
    If FreqDelta > 80  then FreqDelta := 80;

    {divider circuit}
    Led := 21+(FreqDelta div 4);

    if Odd(Led) then {leds number 0 - 39}
      LedCh := ''
    else
      LedCh := '';

    ZeroCenterString := Pad(LeftPad(LedCh,(Led) div 2),SizeOf(DisplayString1));
  end;


  procedure TP38.ZeroCenterDisplay;
  const
    DisplayIncr = 3;
  var
    MarkLeds,
    SpaceLeds,
    Diff : byte;
  begin
    DisplayString1 := ZeroCenterString;

    MarkLeds   := Mark div DisplayIncr;
    SpaceLeds  := Space div DisplayIncr;
    Diff := abs(SpaceLeds - MarkLeds);

    if Diff > 19 then Diff := 19;

    if MarkLeds > SpaceLeds then
      DisplayString2 := CharStr('',SpaceLeds)+CharStr('',Diff)
      else
      DisplayString2 := CharStr('',MarkLeds)+CharStr('',Diff);
    DisplayString2 := Pad(DisplayString2,Sizeof(DisplayString2));
    DisplayAttr1 := FreqAttr;
    DisplayAttr2 := Colors^.PromptAttr;
  end;

  procedure TP38.UpdateDisplay;
  begin
    case GetDisplay of
      0 : Exit;
      1 : ZeroCenterDisplay;
      2 : OneBarDisplay;
      3 : TwoBarDisplay;
      4 : TwoBarTouchDisplay;
      5 : FourBarDisplay;
    end;
  end;

  procedure TP38.UpdateFreqDelta;
  begin
    FreqDelta := lo(ReadOutputPortWait) and $00FF;
    if ZeroTuneRev then FreqDelta := -1 * FreqDelta;
    case FreqDelta of
      -127..-30 : FreqAttr := Hi(Colors^.WindowAttr)+Red;
      -29..-6   : FreqAttr := Hi(Colors^.WindowAttr)+Yellow;
      -5..+5    : FreqAttr := Hi(Colors^.WindowAttr)+Green;
      +6..+30   : FreqAttr := Hi(Colors^.WindowAttr)+Yellow;
      +31..+127 : FreqAttr := Hi(Colors^.WindowAttr)+Red;
    end;
  end;

  procedure TP38.StopEverything;
  begin
    ClearXmitBuffer;
    Transmit(False);
    MyDelayTicks(2);
    SendCommand($8006);
  end;

  constructor TP38.Init;
  const
    DefBaseAddr = $360;
    NumAddresses = 19;
    ValidAddresses : array[1..NumAddresses] of Word =
                                            ($210, $240, $250, $260, $280,
                                             $290, $2A0, $2B0, $2C0, $2D0,
                                             $300, $310, $330, $340, $350,
                                             $360, $380, $390, $3A0);
  var
    BaseAddr : Word;
    FileBaseAddr : Word;
    BaseCt : Integer;
    w,w1,w2 : Word;
    S : string[19];
  begin
    inherited Init;
    TncNum := bP38;
    DisplayString1 := '';
    DisplayString2 := '';
    FreqDelta := 0;
    ProgramPath := RttyIni^.GetProfileString('Path','TNC1','C:\P38');
    FileBaseAddr:= RttyIni^.GetProfileWord('BaseAddress','TNC1',DefBaseAddr);
    ExtraDelay  := RttyIni^.GetProfileLong('ExtraDelay','TNC1',2);
    CollectNumbers := RttyIni^.GetProfileBool('CollectNumbers','TNC1',False);
    ZeroTuneRev := RttyIni^.GetProfileBool('ZeroTuneRev','TNC1',False);
    ProgramPath := AddBackSlash(ProgramPath);
    Input_Port := FileBaseAddr ;
    Output_Port := FileBaseAddr + $02;
    Status_Port := FileBaseAddr + $04;
    ImmediateAbort_Port := FileBaseAddr +$06;
    HardwareReset_Port := FileBaseAddr +$08;
    w := Portw[HardwareReset_Port];
    MyDelayTicks(9+ExtraDelay);
    ReadStatusPort;
    MyDelayTicks(9+ExtraDelay);
    w := ReadOutputPort;
    if W <> hardware_reset_ack then begin { try again ?! }
      w := Portw[HardwareReset_Port];
      MyDelayTicks(9+ExtraDelay);
      ReadStatusPort;
      MyDelayTicks(9+ExtraDelay);
      w := ReadOutputPort;
    end;
    if w = hardware_reset_ack then RttyWin^.PutString(^M'P38 installed at addresss $'+hexw(FileBaseAddr))
    else begin
      Input_Port := 0;
      for BaseCt := 1 to 19 do
        w := Portw[ValidAddresses[BaseCt]+HARDWARE_RESET_OFFSET];
      MyDelayTicks(18+ExtraDelay);
      for BaseCt := 1 to 19 do begin
        Input_Port := ValidAddresses[BaseCt]+FIFO_INPUT_OFFSET;
        Output_Port := ValidAddresses[BaseCt]+FIFO_OUTPUT_OFFSET;
        Status_Port := ValidAddresses[BaseCt]+FIFO_STATUS_OFFSET;
        ImmediateAbort_Port := ValidAddresses[BaseCt]+IMMEDIATE_ABORT_OFFSET;
        HardwareReset_Port := ValidAddresses[BaseCt]+HARDWARE_RESET_OFFSET;
        if (Portw[Status_Port] and $0100) = $0100 then
          w := Portw[Output_Port] and $80FF;
        MyDelayTicks(9+ExtraDelay);
        if (w = hardware_reset_ack) then Break;
      end;
      if w = hardware_reset_ack then begin
        RttyWin^.PutString(^M'P38 installation found at address: $'+hexw(Input_Port));
        RttyIni^.SetOrAddProfileString('BaseAddress','TNC1','$'+hexw(Input_Port));
        RttyWin^.PutString(^M'P38 base address saved in RTTY.INI');
        end
      else begin
        RttyWin^.PutString(^M'Searched for P38 but did not find it!');
        Halt;
      end;
    end;

    StatusWord := $00;

    ControllerProgram := 'P38FSK.S28';
    if ExistFile(ProgramPath+'P38FSK.LOD') then DspProgram := 'P38FSK.LOD'
      else DspProgram := 'P38.LOD';
    if (not ExistFile(ProgramPath+ControllerProgram)) or
       (not ExistFile(ProgramPath+DspProgram)) then begin
       RttyWin^.PutString(^M'P38 Support files not found in the '+Trim(ProgramPath)+' directory');
       RttyWin^.PutString(^M'Edit the RTTY.INI file to identify the proper location of the files');
       fail;
       end;


    if not DownloadDspProgram then begin
      WriteLn('Unable to load '+ ProgramPath+DspProgram);
      Halt;
    end
    else
      RttyWin^.PutString(^M+ProgramPath+DspProgram+' loaded.');

    if not DownloadControllerProgram then begin
      WriteLn('Unable to load '+ ProgramPath+ControllerProgram);
      Halt;
    end
    else
      RttyWin^.PutString(^M+ProgramPath+ControllerProgram+' loaded.');

    SendCommand($8076);
    while
     ReadOutPutPort <> $8076
    do {};
    W1 := ReadOutPutPortWait and $00FF;
    W2 := ReadOutPutPortWait and $00FF;
    RttyWin^.PutString(^M+Pad(DspProgram+' Version',20)+long2str(W1)+'.'+long2str(w2));

    SendCommand($8077);
    while
     ReadOutPutPort <> $8077
    do {};
    W1 := ReadOutPutPortWait and $00FF;
    W2 := ReadOutPutPortWait and $00FF;
    if W1 > 8 then W1 := W1 - 8; {bit set mean s/n < 600}
    RttyWin^.PutString(^M+Pad(ControllerProgram+' Version',20)+long2str(W1)+'.'+long2str(w2));

    SendCommand($8078);
    while
     ReadOutPutPort <> $8078
    do {};
    W1 := ReadOutPutPortWait and $00FF;
    W2 := ReadOutPutPortWait and $00FF;
    if W1 > 8 then W1 := W1 - 8; {bit set mean s/n < 600}
    RttyWin^.PutString(^M+Pad('Card ROM Version',20)+long2str(W1)+'.'+long2str(w2));

    Mark := 1;
    Space := 1;

    {set initial commands}
    SetMode('RTTY');
    SetMarkFreq(GetMarkFreq);
    SetSpaceFreq(GetSpaceFreq);
    SetSense(GetSense);
    SetUsos(GetUsos);
    SetDiddles(GetDiddles);
    SetEol(GetEOL);
    SetBaudRate(GetBaudRate);
    SetSquelchEnabled(GetSquelchEnabled);
    SetSquelchLevel(GetSquelchLevel);
    SetFskPolarity(GetFskPolarity);
    SetFilterWidth(GetFilterWidth);
    SetDisplay(GetDisplay);
    SetDisplayScale(GetDisplayScale);
    SendCommand(enable_echo_as_sent);
    SendCommand(ptt_on_delay);
    SendCommand($8000); {0 ms}
    SendCommand(ptt_off_delay);
    SendCommand($8000); {0 ms}
    If GetDisplay = 0 then begin
      SendCommand($804D);  {disable FSK Tuning Status}
      SendCommand($8041);  {disable Auto Channel Statistics Reports}
      end
    else begin
      SendCommand($805D);  {enable FSK Tuning Status}
      SendCommand($8051);  {enable Auto Channel Statistics Reports}
      end;

    if not FastStart then Setup;

    if CollectNumbers then
      for w := 0 to 255 do begin
        MarkLoCt[w] := 0;
        MarkCt[w] := 0;
        SpaceCt[w] := 0;
        SpaceHiCt[w] := 0;
      end;
    {$IFDEF DEBUG}
    AppendDebugMessage('P38 created');
    {$ENDIF}
  end;

  procedure TP38.ToggleTncTalk;
  begin
    Setup;
    TncTalk := False;
    KeyBoardLive := False;
  end;

  procedure TP38.GetZero(var NumForZero : shortint; var DispAttr : byte);
  begin
    NumForZero := FreqDelta;
    DispAttr := FreqAttr;
  end;

  procedure TP38.UpdateFilterStrings(var S1, S2 : string; var A1, A2 : Byte);
  begin
    S1 := DisplayString1;
    S2 := DisplayString2;
    A1 := DisplayAttr1;
    A2 := DisplayAttr2;
  end;

end.
