{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit CPCI3000;

interface

Uses
  TncObject;

type
  SpeedType = (B45, B50, B57, B74, B100);

const
  OffOnString     : array[boolean] of string[3] = ('Off','On ');
  LevelTypeString : array[Boolean] of String[7] = ('Reverse','Normal ');
  SpeedTypeString : array[SpeedType] of String[3] = (' 45', ' 50', ' 57', ' 74', '100');
  SpeedTypeByte : array[SpeedType] of Byte = (45, 50, 57, 74, 100);


type
  PPci3000 = ^TPci3000;
  TPci3000 = object(TTnc)
             BaseAddress : Integer;
             pcin : Integer;
             pcout : Integer;
             pcdata : Integer;
             Reset : Integer;
             Speed : SpeedType;
             modecode : Byte;
             CommandDelay : Integer;
             CharDelay : Integer;
             ExtraDelay : LongInt;
             constructor Init;
             procedure Setup;
             function  DataReady : boolean;
             function  Busy : boolean;
             procedure SendByte(const Data : Byte); virtual;
             procedure SendChar(const Data : Char); virtual;
             procedure SendControl(const Data : Byte);
             procedure Transmit(const Xmit : Boolean); virtual;
             procedure SendString(const Data : String); virtual;
             procedure Receive(var Ch : Char); virtual;
             procedure SetUsos(const USOS : boolean); virtual;
             procedure SetDiddles(const diddles : Boolean); virtual;
             procedure SetBaudRate(const Rate : Byte); virtual;
             procedure SetMode(const Mode : String); virtual;
             procedure SetSquelchLevel(const SquelchLevel : Byte); virtual;
             procedure Letters; virtual;
             procedure Figures; virtual;
             procedure SetSense(const Sense : boolean); virtual;
             procedure ClearXmitBuffer; virtual;
             procedure StopEverything; virtual;
             destructor Done; virtual;
             procedure  ToggleTncTalk; virtual;
           end;

implementation
uses
  TpCrt,
  TpDos,
  TpString,
  TpEntry,
  TpWindow,
  TpMouse,
  IniFile,
  Routines,
  RecTypes,
  Keys,
  LowLvl,
  cRttyWin,
  IniTools,
  ApPort,
  cAutoCq,
  cColors,
  DebugRtns,
  OneTimers,
  ApTimer;


  procedure IncChoice(var Value; FieldID : Byte;
    Factor : Integer;
    var St : String); far;
  var
    OffOn : Boolean absolute Value;
    Level : Boolean absolute Value;
    Speed : SpeedType absolute Value;
  begin
    case FieldID of
      000,
      003,
      005 : begin
              if Factor <> 0 then OffOn := Not OffOn;
              St := OffOnString[OffOn];
            end;
      001 : begin
              if Factor <> 0 then Level := Not Level;
              St := LevelTypeString[Level];
            end;
      002 : begin
              case Factor of
                1 : if Speed < B100 then Inc(Speed);
                -1 : if Speed > B45 then Dec(Speed);
              end;
              St := SpeedTypeString[Speed];
            end;
    end;
  end;

  procedure TPci3000.Setup;
  var
    Esr : EsRecord;
    SetupPtr : WindowPtr;
    ExitCommand : EStype;
    aUsos : Boolean;
    aLevel : Boolean;
    aDiddles : Boolean;
    aSquelch : boolean;
    aSquelchLevel : byte;
  begin
    aUsos := GetUsos;
    aLevel := GetSense;
    aDiddles := GetDiddles;
    aSquelch := GetSquelchEnabled;
    aSquelchLevel := GetSquelchLevel;
    with Colors^ do
      if not MakeWindow(SetupPtr, 2, 2, 40, 13, True, True, False, WindowAttr,
        FrameAttr, HeaderAttr, 'PCI-3000 Setup Menu') then Exit;
    HideMouse;
    DisplayWindow(SetupPtr);
    InitEsRecord(Esr);
    Colors^.SetEntryAttributes;
    SetWrapMode(Esr, WrapAtEdges);
    AddChoiceField(Esr, 'UnShift On Space', 4, 3, 'XXX',
      4, 24, 0, 1, @IncChoice, aUSOS);
    AddChoiceField(Esr, 'Levels', 5, 3, 'XXXXXXX',
      5, 24, 0, 1, @IncChoice, aLevel);
    AddChoiceField(Esr, 'Speed', 6, 3, 'XXX',
      6, 24, 0, 1, @IncChoice, Speed);
    AddChoiceField(Esr, 'Print Squelch', 7, 3, 'XXX',
      7, 24, 0, 1, @IncChoice, aSquelch);
    AddByteField(Esr, 'Print Squelch Level', 8, 3, '999',
      8, 24, 0, 0, 127, aSquelchLevel);
    AddChoiceField(Esr, 'Diddles', 9, 3, 'XXX',
      9, 24, 0, 1, @IncChoice, aDiddles);
    AddIntField(Esr, 'Character Delay (ms)', 10, 3, '999',
      10, 24, 0, 1, 999, CharDelay);
    AddIntField(Esr, 'Command Delay (ms)', 11, 3, '999',
      11, 24, 0, 1, 999, CommandDelay);
    CtrlEnter(13, 4);
    AddCmd(ESEnd, kbEnd);
    AddCmd(ESHome, kbHome);
    AddCmd(EsQuit, kbEsc);
    repeat
      ExitCommand := EditScreen(Esr, 0, False);
    until ExitCommand in [EsQuit, EsDone];
    AddCmd(ESmy25, kbEnd);
    AddCmd(ESmy23, kbHome);
    AddCmd(Esmy49, kbEsc);
    KillWindow(SetupPtr);
    DisposeEditScreen(Esr);
    if ExitCOmmand = EsDone then begin
      {set new values}
      SetUsos(aUSOS);
      SetSense(aLevel);
      SetBaudRate(SpeedTypeByte[Speed]);
      SetSquelchLevel(aSquelchLevel);
      SetSquelchEnabled(aSquelch);
      SetDiddles(aDiddles);
    end;
  end;


  function TPci3000.DataReady : boolean;
  var
    Status : byte;
  begin
    Status := Port[PcData];
    Status := Status AND $01;
    DataReady := Status = $01;
  end;

  function TPci3000.Busy : boolean;
  var
    Status : byte;
  begin
    Status := Port[PcData];
    Status := Status AND $02;
    Busy   := Status = $02;
  end;

  procedure TPci3000.SendControl(const Data : Byte);
  begin
    Delay(CommandDelay);
    Repeat
      delay(5);
    until not busy;
    Port[pcout] := Data;
  end;

  procedure TPci3000.SendByte(const Data : Byte);
  const
    NumTries = 255;
  var
    Ch : Char;
    Ct : Integer;
    Data1 : Byte;
  begin
    Delay(CharDelay);
    Data1 := Ord(Upcase(Chr(Data)));
    if Data1 = $0D then begin
      SendControl($0D);
      RttyWin^.PutChar(^M);
      Delay(CharDelay);  {2.20h}
    end;
    if Data1 in [$3C, $3E] then begin
      SendControl(Data1);
      Delay(CharDelay); {2.20h}
      end;
    if Chr(Data1) in ['A'..'Z', '0'..'9', '-', '?', ':', '(', ')', '.', ',',
    #39, ' ', '#', '/', '+', '!', '&', '#', ';', '"'] then begin
      Repeat
        delay(5);
      until not busy;
      Port[pcout] := Data1;
      Ct := NumTries;
      repeat
        Delay(22*5);
        Receive(Ch);
        dec(ct);
      until (ct = 0) or (Ord(Ch) = Data1);
      if Ch <> cNUL then RttyWin^.PutChar(Ch);
    end;
  end;

  procedure TPci3000.SendChar(const Data : Char);
  begin
    SendByte(Ord(Data));
  end;

  procedure TPci3000.SendString(const Data : String);
  var
    B : Byte;
  begin
    for B := 1 to Length(Data) do begin
      SendChar(Data[B]);
      if KeyPressed then
        if ReadKeyWord = kbEsc then begin
          B := Length(Data);
          AutoCq^.Stop;
        end;
    end;
    while KeyPressed do
      ReadKeyWord;
  end;


  procedure TPci3000.Transmit(const Xmit : Boolean);
  begin
    if Xmit then begin
      SendControl($8A);
    end
    else begin
      MyDelayTicks(1+ExtraDelay);
      SendControl($8B);
      SendControl($85);
    end;
  end;

  procedure TPci3000.Receive(var Ch : Char);
  begin
    if DataReady then
      Ch := Chr(Port[pcin])
    else
      Ch := #00;
  end;

  procedure TPci3000.SetUsos(const USOS : boolean);
  begin
    inherited SetUsos(Usos);
    if usos then SendControl($96)
    else SendControl($97);
  end;

  procedure TPci3000.Letters;
  begin
    SendControl($8C);
  end;

  procedure TPci3000.Figures;
  begin
    SendControl($8D);
  end;

  procedure TPci3000.SetSense(const Sense : Boolean);
  begin
    case Sense of
      True:  SendControl($98);
      False: SendControl($99);
    end;
    inherited SetSense(Sense);
  end;

  procedure TPci3000.ClearXmitBuffer;
  begin
  end;

  procedure TPci3000.SetMode(const Mode : String);
  begin
    inherited SetMode(Mode);
    if Mode = 'RTTY' then SendControl($85); {RTTY}
  end;

  procedure TPci3000.SetBaudRate(const Rate : Byte);
  begin
    TPci3000.SendControl($AA);      {baud rate change coming}
    case Rate of
      45 : SendControl(0);        {send appropriate code}
      50 : SendControl(1);
      57 : SendControl(2);
      74 : SendControl(3);
      100 : SendControl(4);
      110 : SendControl(5);
    end;
  end;

  procedure TPci3000.SetDiddles(const diddles : Boolean);
  begin
    inherited SetDiddles(Diddles);
    case diddles of
      True:  SendControl($90);
      False: SendControl($91);
    end;
  end;

  procedure TPci3000.SetSquelchLevel(const SquelchLevel : Byte);
  begin
    inherited SetSquelchLevel(SquelchLEvel);
    if GetSquelchEnabled then begin
      SendControl($B1);
      SendControl($B0);
      SendControl(SquelchLevel);
    end
    else begin
      SendControl($B2);
      SendControl($B0);  {set squelch}
      SendControl($00);  {to no level at all}
    end;
  end;

  procedure TPci3000.StopEverything;
  begin
    ClearXmitBuffer;
    Transmit(False);
  end;

  constructor TPci3000.Init;
  var
    FileBaseAddr : integer;
    LoopCt : integer;
    ET : EventTimer;

    function ThisOne(const aBaseAddr : integer) : boolean;
    const
      NumTries = 50;  {2.20h}
    var
      Data : Byte;
      i    : integer; {2.20h}
    begin
      Data := Port[aBaseAddr+4]; {supposed reset!}
      MyDelayTicks(9+ExtraDelay);
      i := NumTries;         {2.20h}
      Data := 0;             {2.20h}
      while (I > 0) and (data <> $DF) do begin {2.20h}
        Data := Port[aBaseAddr+0];             {2.20h}
        dec(i);                                {2.20h}
      end;
      ThisOne := (Data = $DF);
    end;

    procedure DeterminePCIBaseAddress;
    const
      ValidPCIAddress : array[0..42] of Integer = (
        $210, $218, $240, $248, $250, $258, $260, $268, $270, $280, $288,
        $290, $298, $2a0, $2a8, $2b0, $2b8, $2c0, $2c8, $2d0, $2d8,
        $2e0, $2e8, $300, $308, $310, $318, $330, $338, $340, $348,
        $350, $358, $360, $368, $370, $380, $388, $390, $398, $3a0,
        $3a8, $3e8);
    var
      TestIndex : Byte;
    begin
      RttyWin^.Putstring(^M'Searching');
      for TestIndex := 0 to 42 do begin
        if (TestIndex mod 2 = 0) then RttyWin^.Putstring('.');
        if ThisOne(ValidPCIAddress[TestIndex]) then begin
          BaseAddress := ValidPCIAddress[TestIndex];
          Exit;
        end;
      end;
    end;

  begin
    inherited Init;
    TncNum := bPCI3000;
    BaseAddress := 0;
    {try default installation first}
    FileBaseAddr:= RttyIni^.GetProfileInt('BaseAddress','TNC1',$260); {2.20h}
    ExtraDelay  := RttyIni^.GetProfileLong('ExtraDelay','TNC1',2);
    RttyWin^.Putstring(^M'Initializing PCI-3000');
    If FileBaseAddr > 0 then {they told me where it is... lets see if they right}
      for LoopCt := 1 to 5 do begin
        If ThisOne(FileBaseAddr) then begin
          BaseAddress := FileBaseAddr;
          Break;
        end;
        RttyWin^.Putstring('.');
      end;
    If BaseAddress = 0 then begin {no addr yet, try default a couple of times}
      for Loopct := 1 to 5 do begin
        If ThisOne($260) then begin
          BaseAddress := $260; {try 1st at default}
          Break;
        end;
        RttyWin^.Putstring('.');
      end;
    end;
    if BaseAddress = 0 then DeterminePCIBaseAddress; {cant find it, now go searching for it}
    if BaseAddress <> 0 then begin
      RttyWin^.PutString(^M'PCI-3000 installation found at address: $'+hexw(BaseAddress));
      RttyIni^.SetOrAddProfileString('BaseAddress','TNC1','$'+hexw(BaseAddress));
      end
    else begin
      RttyWin^.PutString(^M'Searched for PCI-3000 but did not find it!');
      Halt;
    end;
    pcin := BaseAddress;          { from pci to pc }
    pcout := BaseAddress+1;       { from pc to pci }
    pcdata := BaseAddress+2;      { bit0=1, data ready ,bit1=1, data busy }
    Reset := BaseAddress+4;       { resets PCI-3000}
    Speed := B45;
    CharDelay := RttyIni^.GetProfileWord('CharDelay', 'TNC1', 2);
    CommandDelay := RttyIni^.GetProfileWord('CommandDelay', 'TNC1', 4);
    SetMode(GetMode);
    SetUsos(GetUsos);
    SetSense(GetSense);
    SetBaudRate(SpeedTypeByte[Speed]);
    SetSquelchEnabled(GetSquelchEnabled);
    SetSquelchLevel(GetSquelchLevel);
    if not FastStart then Setup;
    for LoopCt := 1 to 3 do begin
      MyDelayTicks(9+ExtraDelay);
      StopEverything; {reset this damn board!!!!}
    end;
    {$IFDEF DEBUG}
    AppendDebugMessage('PCI3000 created');
    {$ENDIF}
  end;

  destructor TPci3000.Done;
  var
    Data : Byte;
  begin
    Data := Port[Reset]; {supposed reset!}
    Delay(500); {2.20h}
    {$IFDEF DEBUG}
    AppendDebugMessage('PCI3000 destroyed');
    {$ENDIF}
  end;

  procedure TPci3000.ToggleTncTalk;
  begin
    Setup;
    TncTalk := False;
  end;

end.
