{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit cPK900;
interface
uses
  TncObject,
  TpCrt,
  TpString,
  TpEntry,
  TpWindow,
  TpMouse,
  DebugRtns,
  OpRoot,
  ApTimer,
  ApPort;
type
  PPK900 = ^TPk900;
  TPk900 = object(TTnc)
                 CHSwitch : Byte;
                 diddles : Boolean; {t=on}
                 TxLevel : Boolean; {t=normal,f=reverse}
                 Speed : Byte;
                 InChar_Queue : StaticQueue;
                 constructor Init(const Port : ComNameType);
                 procedure Setup;
                 procedure SendByte(const Data : Byte); virtual;
                 procedure SendChar(const Data : Char); virtual;
                 procedure SendString(const Data : String); virtual;
                 procedure SendCommand(const Command : String); virtual;
                 procedure Transmit(const Xmit : Boolean); virtual;
                 procedure Receive(var Ch : Char);    virtual;
                 procedure SetUsos(const USOS : Boolean); virtual;
                 procedure SetBaudRate(const Rate : Byte); virtual;
                 procedure SetMode(const Mode : String); virtual;
                 procedure Letters; virtual;
                 procedure ClearXmitBuffer; virtual;
                 procedure StopEverything; virtual;
                 procedure SetSense(const Sense : boolean); virtual;
                 destructor Done; virtual;
                 procedure ToggleTncTalk;virtual;
               end;

implementation
uses
  Lowlvl,
  IniFile,
  RecTypes,
  cRttyWin,
  cColors,
  Routines;

const
  cCOM = #$03;
  OffOnString : array[Boolean] of String[3] = ('Off', 'On');
  LevelString : array[Boolean] of String[7] = ('Reverse', 'Normal');
  SpeedTypeString : array[0..4] of String[3] = (' 45', ' 50', ' 57', ' 74', '100');
  SpeedByte : array[0..4] of byte = (45,50,57,74,100);
  FilterWidthString : array[0..2] of String[6] = ('Medium', 'Wide', 'Narrow');
  FilterWidthDefault = $02;

  procedure IncChoice(var Value; FieldID : Byte;
    Factor : Integer;
    var St : String); far;
  var
    Switch : Boolean absolute Value;
    Level : Boolean absolute Value;
    Speed : Byte absolute Value;
  begin
    case FieldID of
      000, 003 : begin
                   if Factor <> 0 then Switch := not Switch;
                   St := OffOnString[Switch];
                 end;
      001, 004 : begin
                   if Factor <> 0 then Level := not Level;
                   St := LevelString[Level];
                 end;
      002 : begin
              case Factor of
                1 : if Speed < 4 then Inc(Speed);
                -1 : if Speed > 0 then Dec(Speed);
              end;
              St := SpeedTypeString[Speed];
            end;
    end;
  end;

  procedure TPk900.Setup;
  var
    Esr : EsRecord;
    SetupPtr : WindowPtr;
    ExitCommand : EStype;
    aUsos : Boolean;
    aLevel : Boolean;
  begin
    aUSOS := GetUsos;
    aLevel := GetSense;
    with Colors^ do
    if not MakeWindow(SetupPtr, 2, 2, 40, 10, True, True, False, WindowAttr,
      FrameAttr, HeaderAttr, 'PK-900 Setup Menu') then Exit;
    HideMouse;
    DisplayWindow(SetupPtr);
    InitEsRecord(Esr);
    Colors^.SetEntryAttributes;
    SetWrapMode(Esr, WrapAtEdges);
    AddChoiceField(Esr, 'UnShift On Space', 4, 3, 'XXX',
      4, 24, 0, 1, @IncChoice, aUSOS);
    AddChoiceField(Esr, 'Levels', 5, 3, 'XXXXXXX',
      5, 24, 0, 1, @IncChoice, aLevel);
    AddChoiceField(Esr, 'Speed', 6, 3, 'XXX',
      6, 24, 0, 1, @IncChoice, Speed);
    AddChoiceField(Esr, 'Diddles', 7, 3, 'XXX',
      7, 24, 0, 1, @IncChoice, diddles);
    AddChoiceField(Esr, 'TX Levels', 8, 3, 'XXXXXXX',
      8, 24, 0, 1, @IncChoice, TxLevel);
    CtrlEnter(10, 4);
    repeat
      ExitCommand := EditScreen(Esr, 0, False);
    until ExitCommand in [EsQuit, EsDone];
    KillWindow(SetupPtr);
    DisposeEditScreen(Esr);
    SetUsos(aUSOS);
    SetSense(aLevel);
    SetBaudRate(SpeedByte[Speed]);
    if diddles then SendCommand('DDY')
    else SendCommand('DDN');
    case TxLevel of
      True : begin                {normal tones}
               SendCommand('TXN');
             end;
      False : begin               {reverse tones}
                SendCommand('TXY');
              end;
    end;

  end;

  procedure TPk900.SetSense(const Sense : boolean);
  begin
    case Sense of
      True : SendCommand('RXN');
      False: SendCommand('RXY');
    end;
    inherited SetSense(Sense);
  end;

  procedure TPk900.SendByte(const Data : Byte);
  begin
    SendString(cSOH+' '+Chr(Data)+cETB);
  end;

  procedure TPk900.SendChar(const Data : Char);
  begin
    SendString(cSOH+' '+Data+cETB);
  end;

  procedure TPk900.SendString(const Data : String);
  begin
    inherited SendString(cSOH+' '+Data+cETB);
  end;

  procedure TPk900.SendCommand(const Command : String);
  begin
    inherited SendString(cSOH+'O'+Command+cETB);
  end;

  procedure TPk900.Transmit(const Xmit : Boolean);
  begin
    case Xmit of
      True : begin
               SendCommand('TC');
               Delay(100);
               SendCommand('XM');
             end;
      False : SendChar(^D);
    end;
  end;

  procedure TPk900.Receive(var Ch : Char);

    procedure SendStringToQueue(const S : String);
    var
      B : Byte;
      C : Char;
    begin
      if (S[1] <> ^A) or (S[Length(S)] <> ^W) then Exit;
      for B := 3 to Length(S)-1 do begin
        C := S[B];
        InChar_Queue.PushTail(C);
      end;
    end;


  var
    B : Byte;
    S : String[255];
    EntireBlock : Boolean;
    C : Char;
  begin
    if UP^.BlockReady(4,[^W]) then begin
      S := '';
      WaitForBlock(S, 3, EntireBlock);
      case Ord(S[2]) of
        {   $2F : PutString(^M'$2F='+copy(s,3,10)); { i use theses to spy on}
        {   $30 : PutString(^M'$30='+copy(s,3,10));   {the 900 during testing}
        {   $31 : PutString(^M'$31='+copy(s,3,10));
        {   $3F : PutString(^M'$3F='+copy(s,3,10));
        {   $40 : PutString(^M'$40='+copy(s,3,10));
        {   $41 : PutString(^M'$41='+copy(s,3,10));
        {   $5F : PutString(^M'$5F='+copy(s,3,10)); }
        $3F : begin
                  {C := ^z;
                  InChar_Queue.PushTail(C);}
                SendStringToQueue(S);
              end;
        $2F : begin
                  {C := ^y;
                  InChar_Queue.PushTail(C);}
                SendStringToQueue(S);
              end;
        $4F :                     {just a command echo ?}
      end;
    end;
    InChar_Queue.PopHead(Ch);     {if queue empty, ch will be $00}
  end;

  procedure TPk900.SetUSOS(const USOS : Boolean);
  begin
    inherited SetUsos(USOS);
    case USOS of
      True : SendCommand('USY');
      False : SendCommand('USN');
    end;
  end;

  procedure TPk900.Letters;
  begin
    SendCommand('LO');
  end;

  procedure TPk900.ClearXmitBuffer;
  begin
    SendCommand('TC');
  end;

  procedure TPk900.SetMode(const Mode : String);
  begin
    inherited Setmode(mode);
    if Mode = 'RTTY' then SendCommand('BA');
  end;

  procedure TPk900.SetBaudRate(const Rate : Byte);
  begin
    inherited SetBaudRate(Rate);
    case Rate of
      74 : SendCommand('RB75');
    else
      SendCommand('RB'+trim(long2str(Rate)));
    end;
  end;


  procedure TPk900.StopEverything;
  begin
    ClearXmitBuffer;
    SendCommand('RC');
  end;


  constructor TPk900.Init(const Port : ComNameType);
  var
    Ch : Char;
    S : String;
    B : Byte;
    EntireString : Boolean;
    EntireBlock : Boolean;

    function HaveCommand : Boolean;
    var
      S : String;
    begin
      WaitForString(255, S, 1, EntireString);
      S := StUpCase(S);
      HaveCommand := (
        (Pos('PK-', S) > 0) or
        (Pos('OPYRIGHT', S) > 0) or
        (Pos('DVANCED', S) > 0) or
        (Pos('ELEASE', S) > 0) or
        (Pos('HECKSUM', S) > 0) or
        (Pos('HAT?', S) > 0) or
        (Pos('CMD:', S) > 0)
        );
    end;

    function GetToCmd : Boolean;  {3.3.05}
    var
      B : Byte;
      S : String;
    begin
      GetToCmd := True;

      { try pressing ENTER first }
      UP^.PutString(^M);
      if HaveCommand then Exit;
      if HaveCommand then Exit;
      if HaveCommand then Exit;
      { it wasnt in command mode }


      { try HOST MODE now }
      UP^.PutString(^A+'OHON'+^W);
      if HaveCommand then Exit;
      if HaveCommand then Exit;
      if HaveCommand then Exit;

      { try AUTOBAUD detection }
      for B := 1 to 10 do begin
        UP^.PutString('*');
        delay(100);
      end;
      if HaveCommand then Exit;
      if HaveCommand then Exit;
      if HaveCommand then Exit;
      UP^.PutString(^M);
      if HaveCommand then Exit;
      if HaveCommand then Exit;
      if HaveCommand then Exit;
      UP^.PutString(^M);
      if HaveCommand then Exit;
      if HaveCommand then Exit;
      if HaveCommand then Exit;

      { try CTL-C & ENTER detection }
      UP^.PutString(^C);
      if HaveCommand then Exit;
      if HaveCommand then Exit;
      if HaveCommand then Exit;
      UP^.PutString(^C);
      if HaveCommand then Exit;
      if HaveCommand then Exit;
      if HaveCommand then Exit;
      UP^.PutString(^C);
      if HaveCommand then Exit;
      if HaveCommand then Exit;
      if HaveCommand then Exit;
      UP^.PutString(^M);
      if HaveCommand then Exit;
      if HaveCommand then Exit;
      if HaveCommand then Exit;
      GetToCmd := False;
    end;

    procedure EnterHostMode;
    begin
      UP^.PutString('EXPERT ON'^M);
      UP^.PutString('AWLEN 8'^M);
      UP^.PutString('PARITY 0'^M);
      UP^.PutString('8BITCONV ON'^M);
      UP^.PutString('CONM CONVERSE'^M);
      UP^.PutString(^Q^X^C'HOST Y'^M);
    end;

  var
    I : Integer;
  const
    QueueSize = 1024;
  begin
    inherited InitCom(Port);
    TncNum := bPK900;
    if not InChar_Queue.Init(QueueSize, SizeOf(Char), True) then begin
      WriteLn('Unable to allocate input queue, program halted.');
      Halt;
    end;
    if GetToCmd then begin        {3.3.05}
      EnterHostMode;
      RttyWin^.PutString('PK-900 Initialized for host mode')
    end
    else
      if GetToCmd then begin        {3.3.05}
        EnterHostMode;
        RttyWin^.PutString('PK-900 Initialized for host mode(2)')
      end;
    UP^.PutString(cSOH+'OHPN'+cETB);
    SetMode(GetMode);
    UP^.PutString(cSOH+'ORB45'+cETB); {set baudot 45 }
    Speed := 0;
    SetSense(GetSense);
    TxLevel := RttyIni^.GetProfileBool('FskNormalShift', 'TNC1', True);
    case TxLevel of
      True: UP^.PutString(cSOH+'OTXN'+cETB); {normal tx }
      False:UP^.PutString(cSOH+'OTXY'+cETB); {reverse tx }
    end;
    UP^.PutString(cSOH+'OMQ1'+cETB); {modem 1 = 170hz }
    UP^.PutString(cSOH+'OTC'+cETB); {tclear the buffer}
    UP^.PutString(cSOH+'OEAY'+cETB); {echo as sent ON}
    UP^.PutString(cSOH+'OECY'+cETB); {echo ON}
    Diddles := RttyIni^.GetProfileBool('Diddles', 'TNC1', True);
    case Diddles of
    True: UP^.PutString(cSOH+'ODDY'+cETB); {diddles ON}
    False: UP^.PutString(cSOH+'ODDN'+cETB); {diddles ON}
    end;
    UP^.PutString(cSOH+'OCRY'+cETB); {make eol cr/cr/lf}
    UP^.PutString(cSOH+'ORC'+cETB); {force receive}
    for I := 1 to 12 do begin     {read each block for above commands}
      WaitForBlock(S, 0, EntireBlock);
    end;
    {$IFDEF DEBUG}
    AppendDebugMessage('PK900 created');
    {$ENDIF}
  end;

  destructor TPk900.Done;
  begin
    UP^.PutString(cSOH+'OHON'+cETB);
    Delay(250);
    InChar_Queue.Done;
    inherited Done;
    {$IFDEF DEBUG}
    AppendDebugMessage('PK900 destroyed');
    {$ENDIF}
  end;

  procedure TPk900.ToggleTncTalk;
  begin
    Setup;
  end;

end.
