{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit cPTC1;
interface
uses
  RecTypes,
  tncobject,
  ApPort;
type
  PPTC1   = ^TPTC1;
  TPTC1  = object(TTnc)
               constructor Init(const Port : ComNameType);
               procedure  Transmit(const Xmit : Boolean); virtual;
               procedure  SetBaudRate(const Rate : Byte); virtual;
               procedure  Letters; virtual;
               procedure  SetSense(const Sense : Boolean); virtual;
               procedure  ClearXmitBuffer; virtual;
               procedure  StopEverything; virtual;
               destructor Done; virtual;
             end;

implementation

uses
  TpCrt,
  LowLvl,
  cRttyWin,
  DebugRtns,
  tpstring;

const
  Escape = #$1B;
  Enter  = cCr;

  procedure TPTC1.Transmit(const Xmit : Boolean); {updated for PTC1}
  begin
    case Xmit of
      TRUE : SendString(^Y);
      FALSE: SendString(^D); {when other data sent, this will go to rx}
    end;
  end;

  procedure TPTC1.Letters; {updated for PTC1}
  begin
    SendString(^B);
  end;

  procedure TPTC1.SetSense(const Sense : boolean); {updated for PTC1}
  var
    TxSense : boolean;
  begin
    inherited SetSense(Sense);
    TxSense := GetTxSense;
    if     TxSense AND     Sense then SendString(Escape + 'TR 0' + Enter); {tx normal/rx normal}
    if     TxSense AND NOT Sense then SendString(Escape + 'TR 1' + Enter); {tx normal/rx reverse}
    if NOT TxSense AND     Sense then SendString(Escape + 'TR 2' + Enter); {tx reverse/rx normal}
    if NOT TxSense AND NOT Sense then SendString(Escape + 'TR 3' + Enter); {tx reverse/rx reverse}
  end;

  procedure TPTC1.ClearXmitBuffer; {updated for PTC1}
  begin
    SendString(Escape+'CLR'+Enter);
  end;

  procedure TPTC1.SetBaudRate(const Rate : Byte);
  begin
    inherited SetBaudRate(Rate);
    SendString(Escape + 'BAU '+long2str(Rate)+Enter);
  end;

  procedure TPTC1.StopEverything; {updated for PTC1}
  var
    EntireString : Boolean;
    S : String[255];
  begin
    SendString(Escape + 'DD' + Enter);
    WaitForString(255, S, 9, EntireString);
    WaitForString(255, S, 9, EntireString);
    SendString(Escape + 'BAU' + Enter);
    WaitForString(255, S, 9, EntireString);
    WaitForString(255, S, 9, EntireString);
    SendString(Escape + 'CLR' + Enter);
    WaitForString(255, S, 9, EntireString);
  end;

  constructor TPTC1.Init(const Port : ComNameType);
  begin
    inherited InitCom(Port);
    TncNum := bPTC1;
    RttyWin^.PutString('Initializing PTC-I.');
    SendString(Escape + 'DD' + Enter); { get into standby mode }
    Delay(400);
    RttyWin^.PutString('.');
    SendString(Escape + 'PT' + Enter);
    Delay(400);
    RttyWin^.PutString('.');
    SendString('TERM 1' + Enter);
    Delay(400);
    RttyWin^.PutString('.');
    SendString('AM' + Enter);
    Delay(400);
    RttyWin^.PutString('.');
    SendString(Escape + 'QRTCH 04' + Enter);
    Delay(400);
    RttyWin^.PutString('.');
    SendString(Escape + 'BAU' +Enter);
    Delay(400);
    RttyWin^.PutString('.');
    SetSense(GetSense);
    {$IFDEF DEBUG}
    AppendDebugMessage('PTC1 created');
    {$ENDIF}
  end;

  destructor TPTC1.Done;
  begin
    SendString(Escape + 'DD' + Enter);
    Delay(400);
    SendString(Escape + 'PT' + Enter);
    Delay(500);
    UP^.FlushInBuffer;
    inherited Done;
    {$IFDEF DEBUG}
    AppendDebugMessage('PTC1 destroyed');
    {$ENDIF}
  end;

end.
