{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit cPTC2;
interface
uses
  RecTypes,
  tncobject,
  ApPort;
type
  PPTC2   = ^TPTC2;
  TPTC2  = object(TTnc)
               constructor Init(const Port : ComNameType);
               procedure  Transmit(const Xmit : Boolean); virtual;
               procedure  SetBaudRate(const Rate : Byte); virtual;
               procedure  Letters; virtual;
               procedure  SetSense(const Sense : Boolean); virtual;
               procedure  SetSquelchLevel(const SquelchLevel : Byte); virtual;
               procedure  SetMarkFreq(const MarkFreq : word); virtual;
               procedure  SetSpaceFreq(const SpaceFreq : word); virtual;
               procedure  ClearXmitBuffer; virtual;
               procedure  StopEverything; virtual;
               destructor Done; virtual;
             end;

implementation

uses
  TpCrt,
  LowLvl,
  cRttyWin,
  DebugRtns,
  tpstring;

const
  Escape = #$1B;
  Enter  = cCr;

  procedure TPTC2.Transmit(const Xmit : Boolean); {updated for PTC2}
  begin
    Case Xmit of
      True : SendString(^Y);
      False : SendString(^D); {when other data sent, this will go to rx}
    end;
  end;

  procedure TPTC2.Letters; {updated for PTC2}
  begin
    SendString(^B);
  end;

  procedure TPTC2.SetSense(const Sense : boolean); {updated for PTC2}
  var
    TxSense : boolean;
  begin
    inherited SetSense(Sense);
    TxSense := GetTxSense;
    if     TxSense AND     Sense then SendString(Escape + 'TR 0' + Enter); {tx normal/rx normal}
    if     TxSense AND NOT Sense then SendString(Escape + 'TR 1' + Enter); {tx normal/rx reverse}
    if NOT TxSense AND     Sense then SendString(Escape + 'TR 2' + Enter); {tx reverse/rx normal}
    if NOT TxSense AND NOT Sense then SendString(Escape + 'TR 3' + Enter); {tx reverse/rx reverse}
  end;

  procedure TPTC2.SetSquelchLevel(const SquelchLevel : Byte);
  begin
    inherited SetSquelchLevel(SquelchLevel);
    SendString(Escape + 'SQUELCH '+long2str(SquelchLevel));
  end;

  procedure  TPTC2.SetMarkFreq(const MarkFreq : word);
  begin
    inherited SetMarkFreq(MarkFreq);
    SendString(Escape + 'MARK '+long2str(MarkFreq));
  end;

  procedure  TPTC2.SetSpaceFreq(const SpaceFreq : word);
  begin
    inherited SetSpaceFreq(SpaceFreq);
    SendString(Escape + 'SPACE '+long2str(SpaceFreq));
  end;

  procedure TPTC2.ClearXmitBuffer; {updated for PTC2}
  begin
    SendString(Escape+'CLR'+Enter);
  end;

  procedure TPTC2.SetBaudRate(const Rate : Byte);
  begin
    inherited SetBaudRate(Rate);
    SendString(Escape + 'BAU '+long2str(Rate)+Enter);
  end;

  procedure TPTC2.StopEverything; {updated for PTC2}
  var
    EntireString : Boolean;
    S : String[255];
    B : Byte;
  begin
    SendString(Escape + 'DD' + Enter);
    SendString(Escape + 'BAU' + Enter);
    for b := 1 to 7 do
      WaitForString(255, S, 3, EntireString);
  end;

  constructor TPTC2.Init(const Port : ComNameType);
  begin
    inherited InitCom(Port);
    TncNum := bPTC2;
    RttyWin^.PutString('Initializing PTC-II.');
    SendString(^M);
    Delay(100);
    SendString(Escape + 'DD' + Enter); { get into standby mode }
    RttyWin^.PutString('.');
    SendString('TERM 1' + Enter);
    RttyWin^.PutString('.');
    SendString('AM' + Enter);
    RttyWin^.PutString('.');
    SendString('QRTCH 04' + Enter);
    SendString('TONES 2' + Enter);
    RttyWin^.PutString('.');
    SetSense(GetSense);
    SetSquelchLevel(GetSquelchLevel);
    SetMarkFreq(GetMarkFreq);
    SetSpaceFreq(GetSpaceFreq);
    SendString(Escape + 'BAU' +Enter);
    RttyWin^.PutString('.OK');
    {$IFDEF DEBUG}
    AppendDebugMessage('PTC2 created');
    {$ENDIF}
  end;

  destructor TPTC2.Done;
  begin
    SendString(Escape + 'DD' + Enter);
    Delay(400);
    SendString(Escape + 'PT' + Enter);
    Delay(500);
    UP^.FlushInBuffer;
    inherited Done;
    {$IFDEF DEBUG}
    AppendDebugMessage('PTC2 destroyed');
    {$ENDIF}
  end;

end.
