{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
unit cRateWin;

interface

uses
  EventRate,
  TpDate,
  TpWindow;

type

PRateWin = ^TRateWin;
TRateWin = object(TEventRate)
             Win : WindowPtr;
             xlow,
             ylow,
             xhigh,
             yhigh : Byte; { tpcrt standard }
             Delta : byte;
             Delta2 : byte;
             Format : byte;
             Realtime : boolean;
             hh, mm, ss : Byte;
             T : Time;
             D : Date;
             last_ss : Byte;
             constructor Init;
             destructor  Done; virtual;
             procedure   Update; virtual;
             procedure   UpdateNow; virtual;
             procedure   Draw; virtual;
           end;

var
  RateWin : PRateWin;

implementation

uses
  IniFile,
  DebugRtns,
  RecTypes,
  Diskio,
  cColors,
  cClock,
  TpString,
  TpMouse,
  TpCrt;


constructor TRateWin.Init;
const
  NoFrame = False;
  ClearWindow = True;
  SaveWindow = True;
begin
  inherited Init;
  Realtime := RttyIni^.GetProfileBool('RealTimeRate', 'PERFORMANCE', TRUE);
  Format := RttyIni^.GetProfileByte('TYPE', 'RATE', 1);
  Delta := RttyIni^.GetProfileByte('DELTA', 'RATE', 10);
  if (Delta < 5) then Delta := 5;
  if (Delta > 100) then Delta := 100;
  RttyIni^.SetOrAddProfileString('DELTA','RATE',long2str(Delta));
  Delta2 := RttyIni^.GetProfileByte('DELTA2', 'RATE', 100);
  if (Delta2 < 5) then Delta2 := 5;
  if (Delta2 > 100) then Delta2 := 100;
  RttyIni^.SetOrAddProfileString('DELTA2','RATE',long2str(Delta2));
  xlow  := MW_xhi+1; { just to right of main window }
  ylow  := MW_Yhi-1; { same as bottom row of main win minus 1 }
  xhigh := ScreenWidth;
  yhigh := MW_Yhi-1;
  last_ss := 0;
  with Colors^ do
  if not MakeWindow(Win, xlow, ylow, xhigh, yhigh,
      NoFrame, ClearWindow, SaveWindow,
      WindowAttr, FrameAttr, HeaderAttr, '') then
    AppendDebugMessage('TRateWin.Init, MakeWindow(Win) failed');
  DisplayWindow(Win);
  {$IFDEF DEBUG}
  AppendDebugMessage('RateWin created');
  {$ENDIF}
end;

destructor  TRateWin.Done;
begin
  KillWindow(Win);
  inherited Done;
  {$IFDEF DEBUG}
  AppendDebugMessage('RateWin destroyed');
  {$ENDIF}
end;

procedure   TRateWin.Draw;
begin
  HideMouse;
  Case Format of
    1 : FastWriteStack(Win,
          TimeToTimeString('h:mm:ss', HMSToTime(hh, mm, ss))+
          LeftPad(long2str(RatePerHourLast(Delta)),5),
          1, 1, Colors^.WindowAttr);
    2 : FastWriteStack(Win,
          LeftPad(long2str(RatePerHourLast(Delta)),5)+' / '+
          long2str(RatePerHourLast(Delta2)),
          1, 1, Colors^.WindowAttr);
  end;
  ShowMouse;
end;

procedure   TRateWin.Update;
begin
  if not Realtime then Exit;
  Clock^.DateAndTimeUtc(D, T);
  TimeDiff(T, LogFile^.LastQSOTime, hh, mm, ss);
  if last_ss = ss then Exit;
  last_ss := ss;
  Draw;
end;

procedure   TRateWin.UpdateNow;
begin
  Clock^.DateAndTimeUtc(D, T);
  TimeDiff(T, LogFile^.LastQSOTime, hh, mm, ss);
  last_ss := ss;
  Draw;
end;

end.