{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$F+,O+}
unit cRTbl;
interface

  {* FUNCTIONS AVAILABLE FROM THIS UNIT *}

uses
  TpRarray,
  Rectypes;

const
  Num_R_Provs = 88;

type

  PRList = ^TRList;
  TRList = object
                 private
                   R_Tbl    : array[1..Num_R_Provs] of Mult_Rec;
                 public
                   constructor Init;
                   Destructor  Done;
                   function    GetProv(const Pv : Byte; const Band : BandType) : boolean;
                   procedure   SetProv(const Pv : Byte; const Band : BandType);
                   procedure   ReSetProv(const Pv : Byte; const Band : BandType);
                   function    HowManyProvs(const Band : BandType) : longint;
                   procedure   Display;
                   procedure   Print;
                   function    BandString(const Pv : Byte) : string;
                   function    GetProvNo(const Pr : String) : Byte;
                   function    Load(const Qth : string; const Band : BandType) : Boolean;
                 end;

  {* NOW DEFINE THE FUNCTIONS *}

var
  RList : PRList;

procedure MakeRList;
procedure KillRList;

implementation

uses
  DebugRtns,
  TpString,
  TpCrt,
  cColors,
  TpMouse,
  TpSort,
  c_Contest,
  LowLvl,
  TpWindow,
  Keys,
  IniFile,
  cEditWin,
  cMsgWin,
  Routines,
  OneTimers;

  const
  R_Provs : array[1..Num_R_Provs] of String[2] = (
  'AB','AD','AL','AM','AO','AR','BA','BO','BR','BU'
 ,'CB','CK','CN','CT','CU','DA','EA','EW','GA','HA'
 ,'HK','HM','IR','IV','JA','JN','KA','KB','KC','KE'
 ,'KG','KI','KJ','KK','KL','KM','KN','KO','KP','KR'
 ,'KS','KT','KU','LO','LP','MA','MD','MG','MO','MR'
 ,'MU','NN','NO','NS','NV','OB','OM','OR','PE','PK'
 ,'PM','PS','RA','RO','SA','SL','SM','SO','SP','SR'
 ,'ST','SV','TA','TB','TL','TM','TN','TO','TU','TV'
 ,'UD','UL','UO','VG','VH','VL','VO','YA');


  constructor TRList.Init;
  var
    I : Integer;
    Mult : Mult_Rec;
    Band : BandType;
  begin
    for Band := B80 to TOT do
      Mult.Band[Band] := False;
    for I := 1 to Num_R_Provs do
      R_Tbl[I] := Mult;
    {$IFDEF DEBUG}
    AppendDebugMessage('RList created');
    {$ENDIF}
  end;

  Destructor  TRList.Done;
  begin
    {$IFDEF DEBUG}
    AppendDebugMessage('RList destroyed');
    {$ENDIF}
  end;

  function    TRList.GetProv(const Pv : Byte; const Band : BandType) : boolean;
  begin
    {$IFDEF DEBUG}
    if (Pv = 0) or (Pv > Num_R_Provs) then
      AppendDebugMessage('CTRList.GetProv, invalid Oblast# :'+long2str(Pv));
    {$ENDIF}
    if (Pv = 0) or (Pv > Num_R_Provs) then begin
      GetProv := False;
      Exit;
    end;
    GetProv := R_Tbl[Pv].Band[Band]
  end;

  procedure   TRList.SetProv(const Pv : Byte; const Band : BandType);
  begin
    {$IFDEF DEBUG}
    if (Pv = 0) or (Pv > Num_R_Provs) then
      AppendDebugMessage('CTRList.SetProv, invalid Oblast# :'+long2str(Pv));
    {$ENDIF}
    if (Pv = 0) or (Pv > Num_R_Provs) then Exit;
    R_Tbl[Pv].Band[Band] := True;
    R_Tbl[Pv].Band[TOT]  := True;
  end;

  procedure   TRList.ReSetProv(const Pv : Byte; const Band : BandType);
  var
    b : bandType;
  begin
    {$IFDEF DEBUG}
    if (Pv = 0) or (Pv > Num_R_Provs) then
      AppendDebugMessage('CTRList.ReSetProv, invalid Oblast# :'+long2str(Pv));
    {$ENDIF}
    if (Pv = 0) or (Pv > Num_R_Provs) then Exit;
    R_Tbl[Pv].Band[Band] := False;
    R_Tbl[Pv].Band[TOT] := False;
    for B := B80 to B10 do begin
      if GetProv(Pv,B) then
        R_Tbl[Pv].Band[TOT]  := True;
    end;
  end;

  function    TRList.HowManyProvs(const Band : BandType) : longint;
  var
    L       : longint;
    Pv      : Word;
  begin
    L := 0;
    for Pv := 1 to Num_R_Provs do
      if GetProv(Pv,Band) then Inc(L);
    HowManyProvs := L;
    {$IFDEF DEBUG}
    AppendDebugMessage('TRList.HowManyProvs = '+Bands[Band]+'M '+long2str(l));
    {$ENDIF}
  end;

  procedure TRList.Display;
  var
    W              : Word;
    col, Row, ix   : Byte;
    Win            : WindowPtr;
    S              : String[5];
    Band           : BandType;
  begin
    with Colors^ do begin
     if not MakeWindow(Win, 1, 2, 80, 22, True, True, True,
                       WindowAttr, FrameAttr, HeaderAttr,
                       'Russian Oblasts Multiplier Check Sheet') then Exit;
     HideMouse;
     DisplayWindow(Win);
     for col := 0 to 5 do
       for Row := 1 to 18 do
       begin
         ix := col*18+row;
         if ix <= Num_R_Provs then
           FastWriteWindow(Pad(Trim(R_Provs[ix]), 3)+' |'+BandString(ix)+'|', Row+1, col*13+2, WindowAttr);
       end;
     FastWrite('Press Any Key to continue', 22, 2, WindowAttr);
    end;
    ShowMouse;
    ReadKeyorButton;
    KillWindow(Win);
  end;

  procedure TRList.Print;
  var
    Str  : String[80];
    ix   : Byte;
    PrintFile : Text;
    Band : BandType;
  begin
    Assign(PrintFile, FileName('OBL'));
    {$I-} Rewrite(PrintFile); {$I+}
    MsgWin^.Display('Preparing: '+FileName('OBL')); {v3.0}
    Writeln(PrintFile,'Russian Oblast Multiplier sheet for '+RttyIni^.GetProfileString('Call','Profile',''));
    Writeln(PrintFile,'');
    for ix := 1 to 50 do begin
      Str := ' '+Pad(R_Provs[ix], 4)+' |';
      for Band := B80 to B10 do
        if R_Tbl[ix].Band[Band] then Str := Str+'*'
          else Str := Str+'.';
      Str := Str+'|  ';
      if 50+ix <= Num_R_Provs then begin
        Str := Str + ' '+Pad(R_Provs[50+ix], 4)+' |';
        for Band := B80 to B10 do
          if R_Tbl[50+ix].Band[Band] then Str := Str+'*'
            else Str := Str+'.';
        Str := Str+'|';
      end;
      Writeln(PrintFile,Str);
    end;
    Write(PrintFile,^L);
    Close(PrintFile);
  end;

  function  TRList.BandString(const Pv : Byte) : string;
  var
    s : string[5];
    Band : BandType;
  begin
    S := '.....';
    for Band := B80 to B10 do
      if GetProv(Pv,Band) then S[Ord(Band)+1] := '*';
    BandString := S;
  end;

  function TRList.GetProvNo(const Pr : String) : Byte;
  var
    P : String[2];
    ix : Byte;
  begin
    P := Trim(Copy(Pr, 1, 2));
    for ix := 1 to Num_R_Provs do
      if P = R_Provs[ix] then begin
        GetProvNo := ix;
        Exit;
      end;
    GetProvNo := 0;
  end;

  function TRList.Load(const Qth : string; const Band : BandType) : Boolean;
  var
    T  : Boolean;
    ix : Byte;
  begin
    T := False;
    ix := GetProvNo(Qth);
    if ix > 0 then T := not GetProv(ix,Band);
    SetProv(ix,Band);
    SetProv(ix,TOT);
    Load := T;
  end;





  procedure MakeRList;
  begin
    RList := New(PRList, Init);
    if not assigned(RList) then begin
      MessageBox('Severe Error!',
                 'Unable to create Oblast list',
                 'Contact Wyvern Technology',
                 'Consider reinstalling from diskette');
      CloseDownScreen;
      Halt;
    end;
    {$IFDEF DEBUG}
    AppendDebugMessage('RList created');
    {$ENDIF}
  end;

  procedure KillRList;
  begin
    if Assigned(RList) then begin
      Dispose(RList, Done);
      RList := NIL;
      {$IFDEF DEBUG}
      AppendDebugMessage('RList destroyed');
      {$ENDIF}
    end;
  end;



end.
