{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
unit cRttyWin;

interface

uses
  Diskio,
  TpEntry,
  TpWindow;

type

  PRttyWin = ^TRttyWin;
  TRttyWin = object
               Win : WindowPtr;
               xlow,
               ylow,
               xhigh,
               yhigh : Byte;      { tpcrt standard }
               MainRow,
               MainCol : Byte;
               RxBuffer : VScreen;
               ScrollWinPtr : WindowPtr;
               Scrolling : Boolean;
               RxCurLine : Byte;
             private
               InBuff : String[19];
               SmBuff : String[19];
             public
               constructor Init;
               destructor Done; virtual;
               procedure Activate;
               procedure Clear;
               procedure HighLightCall(const Cl : String; const Attr : Byte);
               procedure PutChar(const Ch : Char);
               procedure PutString(Text : String);
               procedure HilightOn;
               procedure HilightOff;
               procedure MakeTop;
               procedure RxScrolling(ScrollUp : Boolean);
               procedure RxScrollingStop;
               procedure ProcessIncomingBuffer(Inchar : Char; var R : log_rec);
               procedure SendKeys(Command : esType; var R : log_rec);
               procedure ChangeBand(WhichWay : Char; var R : log_rec);
               procedure ReinitAfterShell;
             private
               procedure PutCharW(Ch : Char);
               procedure Scroll;
               procedure AddToRxBuffer(const newline : String);
             end;

var
  RttyWin : PRttyWin;

implementation

uses
  TncControl,
  LowLvl,
  IniFile,
  DebugRtns,
  RecTypes,
  cClock,
  c_Contest,
  c_Radio,
  cCallQue,
  cCountry,
  TpDate,
  TpEdit,
  TpString,
  cEditWin,
  cTopWin,
  cBotWin,
  cDupeTbl,
  Routines,
  cSerNum,
  Network,
  TpMouse,
  cColors,
  cMsgWin,
  TpCrt;


  constructor TRttyWin.Init;
  const
    Frame = True;
    ClearWindow = True;
    SaveWindow = True;
  var
    I : Integer;
  begin
    xlow := MW_xlo;               { just to left of main window }
    ylow := MW_Ylo;               { same as bottom row of main win }
    xhigh := MW_xhi;
    yhigh := MW_Yhi;
    MainCol := 1;
    MainRow := 1;
    Scrolling := False;
    InBuff := '';
    SmBuff := '';
    with Colors^ do begin
      if not MakeWindow(Win, xlow, ylow, xhigh, yhigh, Frame, ClearWindow, SaveWindow,
        WindowAttr, FrameAttr, HeaderAttr, rttybywf1b+version+copyright) then begin
        AppendDebugMessage('TRttyWin.Init, MakeWindow(Win) failed');
        NotEnoughMemory(True);
      end;
      DisplayWindow(Win);
      for I := 1 to 5 do FastWriteWindow('', I, MW_xhi-1, FrameAttr);
      for I := 7 to 9 do FastWriteWindow('', I, MW_xhi-1, FrameAttr);
      for I := 11 to 15 do FastWriteWindow('', I, MW_xhi-1, FrameAttr);
    end;
    RxBufferSize := RttyIni^.GetProfileByte('Lines', 'RX-SCROLLING', 50);
    if RxBufferSize < 25 then RxBufferSize := 25;
    RxScrollIncr := RttyIni^.GetProfileByte('Increment', 'RX-SCROLLING', 5);
    if RxScrollIncr > 15 then RxBufferSize := 15;
    if not MakeVScreen(RxBuffer, RxBufferSize, MW_xhi-2) {width of mainwin}
    then NotEnoughMemory(True);
    ClearVScreen(RxBuffer, Colors^.WindowAttr, ' ');
  {$IFDEF DEBUG}
  AppendDebugMessage('RttyWin created');
  {$ENDIF}
  end;

  destructor TRttyWin.Done;
  begin
    DisposeVScreen(RxBuffer);
    KillWindow(Win);
  {$IFDEF DEBUG}
  AppendDebugMessage('RttyWin destroyed');
  {$ENDIF}
  end;

  procedure TRttyWin.ReinitAfterShell;
  var
    I : Integer;
  begin
    KillWindow(Win);
    with Colors^ do begin
      if not MakeWindow(Win, xlow, ylow, xhigh, yhigh, True, True, True,
        WindowAttr, FrameAttr, HeaderAttr, rttybywf1b+version+copyright) then begin
        AppendDebugMessage('TRttyWin.ReinitAfterShell, MakeWindow(Win) failed');
        NotEnoughMemory(True);
      end;
      DisplayWindow(Win);
      for I := 1 to 5 do FastWriteWindow('', I, MW_xhi-1, FrameAttr);
      for I := 7 to 9 do FastWriteWindow('', I, MW_xhi-1, FrameAttr);
      for I := 11 to 15 do FastWriteWindow('', I, MW_xhi-1, FrameAttr);
    end;
  end;

  procedure TRttyWin.Activate;
  begin
    ActivateWindow(Win);
  end;

  procedure TRttyWin.Clear;
  begin
    ActivateWindow(Win);
    ScrollWindow(True, yhigh);    { num lines to scroll }
    MainCol := 1;
    MainRow := 1;
  end;

  procedure TRttyWin.HighLightCall(const Cl : String; const Attr : Byte);
  var
    b, I : Byte;
    Temp : String[12];
  begin
    ActivateWindow(Win);
    for b := MainCol downto 1 do begin
      FastRead(Length(Cl), MainRow+1, b, Temp); {len, row, col, string }
      if Temp = Cl then begin
        ChangeAttributeWindow(Length(Cl), MainRow, b-1, Attr);
        Exit;
      end;
    end;
    for b := xhigh-Length(Cl) downto 1 do begin
      FastRead(Length(Cl), MainRow, b, Temp); {len, row, col, string }
      if Temp = Cl then begin
        ChangeAttributeWindow(Length(Cl), MainRow-1, b-1, Attr);
        Exit;
      end;
    end;
  end;

  procedure TRttyWin.AddToRxBuffer(const newline : String);
  var
    b : Byte;
    s : String;
  begin
    for b := 2 to 50 do begin
      ReadVScreen(RxBuffer, MW_xhi, b, 1, s); {read line}
      WriteVScreen(RxBuffer, CharStr(' ', MW_xhi), b-1, 1, TextAttr);
      WriteVScreen(RxBuffer, s, b-1, 1, TextAttr); {move it to previous line}
    end;
    WriteVScreen(RxBuffer, newline, 50, 1, TextAttr);
  end;

  procedure TRttyWin.Scroll;
  var
    Temp : String;
    MouseX, MouseY : Byte;
  begin
    MakeTop;
    MainCol := 1;
    if MainRow < MW_Yhi-2 then begin
      HideMouse;                  { speed test }
      FastRead(MW_xhi-2, MainRow, 2, Temp); {len, row, col, string }
      AddToRxBuffer(Temp);
      Inc(MainRow);
      if MainRow = MW_Yhi then Dec(MainRow);
      ShowMouse;                  { speed test }
    end
    else begin
      HideMouse;                  { speed test }
      FastRead(MW_xhi-2, MW_Yhi-1, 2, Temp); {len, row, col, string }
      AddToRxBuffer(Temp);
      ScrollWindow(True, 1);
      if RttyIni^.GetProfileBool('AutoScroll', 'Mouse', False) then begin
        MouseX := MouseWhereX;
        MouseY := MouseWhereY;
        if (MouseY > MW_Ylo+1) and (MouseY < MW_Yhi) and
        (MouseX > 1) and (MouseX < MW_xhi) then begin
          Dec(MouseY);
          MouseGotoXY(MouseX, MouseY);
        end;
      end;
      ShowMouse;                  { speed test }
    end
  end;


  procedure TRttyWin.PutCharW(Ch : Char);

    procedure AppendLine(const Msg : String);
    const
      FileOK = $0;
      FileNotFound = $2;
    var
      AirFile : Text;
      IOStatus : Integer;
    begin
      Assign(AirFile, FileName('AIR'));
      {$I-} Append(AirFile); {$I+}
      IOStatus := IoResult;
      if (IOStatus <> FileOK) and (IOStatus <> FileNotFound) then
        Exit;                     {FATAL ERROR-Get the hell out!}
      if IOStatus = FileNotFound then Rewrite(AirFile);
      Write(AirFile, Msg);
      Close(AirFile);
    end;


  begin
    fastwritestack(Win, ' ', MainRow, MainCol, TextAttr);
    if RttyIni^.GetProfileBool('RecordData', 'PROFILE', False) then begin
      DataLine := DataLine+Ch;
      if Length(DataLine) >= DataCount then begin
        AppendLine(DataLine);
        DataLine := '';
      end;
    end;
    case Ch of
      ' ' : if MainCol >= RttyIni^.GetProfileByte('AutoCr', 'PROFILE', Lo(ScreenWidth)-17)
            then Scroll
            else begin
              fastwritestack(Win, ' ', MainRow, MainCol, TextAttr);
              Inc(MainCol);
            end;
      ^G : Beep;                  {Beep}
      ^H : if MainCol > 1 then Dec(MainCol); {backspace}
      ^J : {} ;                   { ignore line feeds}
      ^M : Scroll;                {carriage return}
      ^X : TextAttr := Colors^.RIAttr;
      ^Y : TextAttr := Colors^.HiLiteAttr; {my hilite code}
      ^Z : TextAttr := Colors^.WindowAttr; {my return to normal code}
      ^A..^F, ^I, ^L, ^N..^W : if RttyTerm1^.TncTalk then begin
                                 if MainCol < MW_xhi-4 then begin
                                   fastwritestack(Win, '^'+Chr(Ord(Ch)+64), MainRow, MainCol, TextAttr);
                                   Inc(MainCol, 2);
                                 end;
                               end;
      #128..#255 :                {ignore for everyone except k6sti}
        if RttyTerm1^.TncNum = bK6Sti then begin
          fastwritestack(Win, Ch, MainRow, MainCol, TextAttr);
          Inc(MainCol);
        end;
    else                          {normal character}
      begin
        fastwritestack(Win, Ch, MainRow, MainCol, TextAttr);
        Inc(MainCol);
      end;
    end;
    if Assigned(RttyTerm1) then
      case (RttyTerm1^.KeyBoardLive or RttyTerm1^.TncTalk) of
        True : begin
                 if MainCol >= MW_xhi-2 then Scroll; {reached end of line}
                 if Ch = ^H then fastwritestack(Win, '  ', MainRow, MainCol, TextAttr);
                 if MW_xhi-1 > MainCol then fastwritestack(Win, '_', MainRow, MainCol, TextAttr);
               end;
        False : if MainCol >= MW_xhi-1 then
                  Scroll;         {reached end of line}
      end
    else if MainCol >= MW_xhi-1 then Scroll; {reached end of line}
  end;

  procedure TRttyWin.PutChar(const Ch : Char);
  begin
    HideMouse;
    PutCharW(Ch);
    ShowMouse;
  end;

  procedure TRttyWin.PutString(Text : String);
  var
    b : Byte;
  begin
    if Length(Text) = 0 then Exit;
    HideMouse;
    for b := 1 to Length(Text) do PutCharW(Text[b]);
    ShowMouse;
  end;

  procedure TRttyWin.HilightOn;
  begin
    PutChar(^Y);                  {hi-lite}
  end;

  procedure TRttyWin.HilightOff;
  begin
    PutChar(^Z);                  {normal color}
  end;

  procedure TRttyWin.MakeTop;
  begin
    SetTopWindow(Win);
  end;

  procedure TRttyWin.RxScrolling(ScrollUp : Boolean);
  begin
    {I use makewindow as a simple mechanism to save the contents of the screen
      before I screw them up with scorlling}
    if not Scrolling then begin
      RxCurLine := RxBufferSize-10;
      with Colors^ do
        if not MakeWindow(ScrollWinPtr, MW_xlo+1, MW_Ylo+1, MW_xhi-1, MW_Yhi-1, False, True, True,
          WindowAttr, FrameAttr, HeaderAttr, '') then begin
          NotEnoughMemory(False); { not need to stop }
          Exit;
        end;
      DisplayWindow(ScrollWinPtr);
    end;
    case ScrollUp of
      True : if RxCurLine > RxScrollIncr then Dec(RxCurLine, RxScrollIncr)
             else RxCurLine := 1;
      False : if RxCurLine < RxBufferSize-15 then Inc(RxCurLine, RxScrollIncr)
                {else RxCurLine := RxBufferSize-15 }
              else begin
                RxScrollingStop;
                Exit;
              end;
    end;
    ScrollWindow(True, 15);       { "clear window" }
    MoveVScreenToWindow(RxBuffer, RxCurLine, 1);
    Scrolling := True;
  end;

  procedure TRttyWin.RxScrollingStop;
  begin
    if Scrolling then begin
      KillWindow(ScrollWinPtr);   { restore contents of the screen}
      Scrolling := False;
    end;
  end;

  procedure TRttyWin.ProcessIncomingBuffer(Inchar : Char; var R : log_rec);
  var
    Offset : Integer;
    Attr : Byte;
    hCall : String[12];
    SameAsMine : Boolean;
  const
    GoodCallChars : set of Char = ['A'..'Z', '0'..'9', '/'];

    procedure SetAttr(const Cl : String);
    begin
      case DupeTbl^.IsDupe(Cl, R.band) of
        True : Attr := Colors^.CatchDupeAttr;
        False : Attr := Colors^.CatchGoodAttr;
      end;
    end;

    procedure PutCharAtEndOfBuffer(var C : Char);
    begin
      C := Upcase(C);
      case C of
        ^M : Offset := -1;
        ' ' : if MainCol < RttyIni^.GetProfileByte('AutoCr', 'PROFILE', Lo(ScreenWidth)-17) then Offset := 0
              else Offset := -1;
      else
        Offset := 0;
      end;
      if not(C in GoodCallChars) then C := ' ';
      if (C = ' ') and (InBuff[Length(InBuff)] = ' ') then
        C := #$00                 { ignore double ' ' and remove it from being processsed }
      else
        InBuff := InBuff+C;
      if Length(InBuff) > 15 then Delete(InBuff, 1, 1);
      InBuff := TrimLead(InBuff);
    end;

    procedure PutCharAtEndOfSmBuffer(var C : Char);
    begin
      C := Upcase(C);
      {      if C = ^M then Offset := -1 else Offset := 0; }
      case C of
        ^M : Offset := -1;
        ' ' : if MainCol < RttyIni^.GetProfileByte('AutoCr', 'PROFILE', Lo(ScreenWidth)-17) then Offset := 0
              else Offset := -1;
      else
        Offset := 0;
      end;
      if not(C in GoodCallChars) then C := ' ';
      if (C = ' ') and (SmBuff[Length(SmBuff)] = ' ') then
        C := #$00                 { ignore double ' ' and remove it from being processsed }
      else
        SmBuff := SmBuff+C;
      if Length(SmBuff) > 8 then Delete(SmBuff, 1, 1);
      SmBuff := TrimLead(SmBuff);
    end;

    procedure NotifyUser;
    var
      a : log_rec;
      Msg : String[9];
      Slot : Word;
    begin
      SetAttr(CCall);
      if ThisContest^.ContestId = 'SPRI' then Msg := '-Not yet '
      else Msg := '-DUPE ';
      LogFile^.BlankLogRec(a);
      a.Call := CCall;
      a.band := EditW^.EditRecs[1].band;
      if ThisContest^.QsoIsGood(a) then Callqueue^.Add(CCall)
      else if RttyIni^.GetProfileBool('PutMsgs', 'PROFILE', False) then PutString(^Y^G+Msg+^Z);
      HighLightCall(CCall, Attr);
      if RttyIni^.GetProfileBool('QuoteCall', 'PROFILE', False) then PutString(^Y' "'+Trim(CCall)+'" '^Z^G); {v2.5a}
      if RttyIni^.GetProfileBool('PutMsgs', 'PROFILE', False) then ThisContest^.IdAnyMults(a);
      TopWin^.Update(a.Call);     {b/c idanymults ignores dupes}
      if ThisContest^.ContestId = 'ARFD' then begin
        if DupeTbl^.GetDupeByCall(a.Call, Slot) then
          BotWin^.Update(a.Call,DupeTbl^.GetQth);
      end
      else BotWin^.Update(a.Call, a.his_qth); {b/c idanymults ignores dupes}
      CountryList^.BlankCTRec;
    end;

    function HasLetterAndNumber(const s : String) : Boolean;
    var
      I : Byte;
      Num, Let : Boolean;
    begin
      Num := False;
      Let := False;
      for I := 1 to Length(s) do begin
        if s[I] in ['0'..'9'] then Num := True;
        if s[I] in ['A'..'Z'] then Let := True;
      end;                        { for }
      HasLetterAndNumber := Num and Let;
    end;

  var
    I : Byte;
    OK : Boolean;
    Hold : log_rec;

    procedure OriginalDetection;
    begin
      PutCharAtEndOfBuffer(Inchar);
      if Inchar = #$00 then Exit;
      case WordCount(InBuff, [' ']) of
        0 : Exit;                 { no data , get out }
        1 : Exit;                 { only 1 word , get out }
        2 : if (ExtractWord(1, InBuff, [' ']) = 'DE') and
            (Length(InBuff) > 3) and
            (Inchar = ' ') then begin
              CCall := ExtractWord(2, InBuff, [' ']);
              Delete(InBuff, 1, 3+Length(CCall)+1); { delete this >> "DE KA1JGB "}
              SameAsMine := CCall = RttyIni^.GetProfileString('Call', 'PROFILE', '');
              if SameAsMine then begin
                CCall := hCall;
                Exit;
              end;                { if }
              if not HasLetterAndNumber(CCall) then begin
                CCall := hCall;
                Exit;
              end;                { if }
              NotifyUser;
            end;                  {if }
      else
        Delete(InBuff, 1, Length(ExtractWord(1, InBuff, [' ']))+1); { delete the first word }
      end;                        { case }
    end;

    procedure DetectionOnSpace2;
    begin
      PutCharAtEndOfBuffer(Inchar);
      if Inchar = #$00 then Exit;
      case WordCount(InBuff, [' ']) of
        0 : Exit;                 { no data , get out }
        1 : if (Inchar = ' ') and
            (ExtractWord(1, InBuff, [' ']) <> 'DE') then begin
              CCall := ExtractWord(1, InBuff, [' ']);
              Delete(InBuff, 1, Length(CCall)+1); { delete "KA1JGB " }
              if Length(CCall) <= 3 then begin
                CCall := hCall;
                Exit;
              end;
              SameAsMine := CCall = RttyIni^.GetProfileString('Call', 'PROFILE', '');
              if SameAsMine then begin
                CCall := hCall;
                Exit;
              end;                { if }
              if not HasLetterAndNumber(CCall) then begin
                CCall := hCall;
                Exit;
              end;                { if }
              SetAttr(CCall);
              if DupeTbl^.IsItGoodCall(CCall) then NotifyUser; {using delimited method}
            end;
        2 : if (ExtractWord(1, InBuff, [' ']) = 'DE') and
            (Length(InBuff) > 3) and
            (Inchar = ' ') then begin
              CCall := ExtractWord(2, InBuff, [' ']);
              Delete(InBuff, 1, 3+Length(CCall)+1); { delete this >> "DE KA1JGB "}
              SameAsMine := CCall = RttyIni^.GetProfileString('Call', 'PROFILE', '');
              if SameAsMine then begin
                CCall := hCall;
                Exit;
              end;                { if }
              if not HasLetterAndNumber(CCall) then begin
                CCall := hCall;
                Exit;
              end;                { if }
              NotifyUser;
            end;                  {if }
      else
        Delete(InBuff, 1, Length(ExtractWord(1, InBuff, [' ']))+1); { delete the first word }
      end;                        { case }
    end;

    procedure DetectionOnSpace;

      function AnyDE : Boolean;
      var
        X : Integer;
      begin
        AnyDE := False;
        for X := 1 to WordCount(InBuff, [' ']) do
          if ExtractWord(X, InBuff, [' ']) = 'DE' then AnyDE := True;
      end;

      function PosDE : Integer;
      var
        X : Integer;
      begin
        for X := 1 to WordCount(InBuff, [' '])-1 do
          if ExtractWord(X, InBuff, [' ']) = 'DE' then begin
            PosDE := X;
            Exit;
          end;
      end;

      procedure TryEachWord;
      begin
        while WordCount(InBuff, [' ']) > 0 do begin
          CCall := ExtractWord(1, InBuff, [' ']);
          Delete(InBuff, 1, Length(CCall)+1);
          if (Length(CCall) >= 4) and
          (CCall <> RttyIni^.GetProfileString('Call', 'PROFILE', '')) and
          (HasLetterAndNumber(CCall)) then begin
            if DupeTbl^.IsItGoodCall(CCall) then begin
              SetAttr(CCall);
              NotifyUser;         {using delimited method}
            end
            else
              CCall := hCall;
          end
          else
            CCall := hCall;
        end;
      end;

      procedure HandleDE;
      begin
        if (WordCount(InBuff, [' ']) < 2) or
        (PosDE = WordCount(InBuff, [' '])) then Exit;
        if ExtractWord(PosDE, InBuff, [' ']) = 'DE' then begin
          CCall := ExtractWord(PosDE+1, InBuff, [' ']);
          Delete(InBuff, 1, Pos(CCall, InBuff)+Length(CCall)+1);
          if (Length(CCall) >= 4) and
          (CCall <> RttyIni^.GetProfileString('Call', 'PROFILE', '')) and
          (HasLetterAndNumber(CCall)) then begin
            SetAttr(CCall);
            NotifyUser;
          end
          else
            CCall := hCall;
        end;
      end;

    begin
      PutCharAtEndOfBuffer(Inchar);
      if Inchar = #$00 then Exit;
      if Inchar <> ' ' then Exit;
      if InBuff = 'DE ' then Exit;
      if not AnyDE then
        TryEachWord
      else
        HandleDE;
    end;

    procedure DetectAll;
    var
      l : Byte;
      b : Byte;
      s : Byte;                   { size of smbuffer }

    begin
      PutCharAtEndOfSmBuffer(Inchar);
      if Pos(' ', SmBuff) < 4 then
        Delete(SmBuff, 1, Pos(' ', SmBuff));
      if Pos(' ', SmBuff) < 4 then
        Delete(SmBuff, 1, Pos(' ', SmBuff));
      if Length(SmBuff) < 4 then Exit;
      l := 0;
      b := 4;
      s := Pos(' ', SmBuff);
      repeat
        if DupeTbl^.IsItGoodCall(Copy(SmBuff, 1, b))
        then l := b
        else Inc(b);
      until (l <> 0) or (b >= s);
      if l = 0 then Exit;
      CCall := Copy(SmBuff, 1, l);
      if CCall = RttyIni^.GetProfileString('Call', 'PROFILE', '') then begin
        CCall := hCall;
        Exit;
      end;
      if not HasLetterAndNumber(CCall) then begin
        CCall := hCall;
        Exit;
      end;                        { if }
      NotifyUser;
      Delete(SmBuff, 1, Length(CCall)+1);
    end;

  begin
    hCall := CCall;
    case RttyIni^.GetProfileByte('Detector', 'PROFILE', 2) of
      0 : { no detection requested } ;
      1 : OriginalDetection;
      2 : DetectionOnSpace;
      3 : DetectAll;
    else OriginalDetection;
    end;
  end;


  function GetBreakText(var Note : String) : Boolean;
  var
    BreakWinPtr : WindowPtr;
    Escaped : Boolean;
  begin
    with Colors^ do
      if not MakeWindow(BreakWinPtr, 3, 3, 68, 5, True, True, False,
        WindowAttr, FrameAttr, HeaderAttr, '') then begin
        NotEnoughMemory(False);
        Exit;
      end;
    DisplayWindow(BreakWinPtr);
    repeat
      with Colors^ do
        ReadString('Break Text:', 1, 1, 50, PromptAttr, StringAttr, CtrlAttr, Escaped, Note);
    until (RSCommand = RSQuit) or (RSCommand = RSEnter) or (Escaped);
    Note := StUpCase(Trim(Note));
    DisposeWindow(EraseTopWindow);
    GetBreakText := Note <> '';
  end;

  procedure TRttyWin.SendKeys(Command : esType; var R : log_rec);
  var
    Msg : String[150];
    TDate : Date;
    BreakText : String[50];
  begin
    with RttyIni^ do
      case Command of
        EsMy13 : Msg := GetProfileString('F1', 'FKEYS', '');
        EsMy14 : Msg := GetProfileString('F2', 'FKEYS', '');
        EsMy15 : Msg := GetProfileString('F3', 'FKEYS', '');
        EsMy16 : Msg := GetProfileString('F4', 'FKEYS', '');
        EsMy17 : Msg := GetProfileString('F5', 'FKEYS', '');
        EsMy18 : Msg := GetProfileString('F6', 'FKEYS', '');
        EsMy19 : Msg := GetProfileString('F7', 'FKEYS', '');
        EsMy44 : Msg := GetProfileString('B4', 'FKEYS', '');
        EsMy50 : Msg := GetProfileString('F8', 'FKEYS', '');
        EsMy51 : Msg := GetProfileString('F9', 'FKEYS', '');
        EsMy52 : Msg := GetProfileString('F10', 'FKEYS', '');
        EsMy60 : Msg := GetProfileString('SF1', 'FKEYS', '');
        EsMy61 : Msg := GetProfileString('SF2', 'FKEYS', '');
        EsMy62 : Msg := GetProfileString('SF3', 'FKEYS', '');
        EsMy63 : Msg := GetProfileString('SF4', 'FKEYS', '');
        EsMy64 : Msg := GetProfileString('SF5', 'FKEYS', '');
        EsMy65 : Msg := GetProfileString('SF6', 'FKEYS', '');
        EsMy66 : Msg := GetProfileString('SF7', 'FKEYS', '');
        EsMy67 : Msg := GetProfileString('SF8', 'FKEYS', '');
        EsMy68 : Msg := GetProfileString('SF9', 'FKEYS', '');
        EsMy69 : Msg := GetProfileString('SF10', 'FKEYS', '');
      end;
    if (Pos('<13>', Msg) > 0) and (not QSOTimeSet) then begin
      R.My_Time := Clock^.TimeUtc;
      QSOTimeSet := True;
    end;
    if (Pos('<14>', Msg) > 0) and (R.My_Serno = 0) then begin
      R.My_Serno := SerNum^.NextSerialNumberInAll(R.band);
      if Assigned(Net) then Net^.SendSerNumData;
    end;
    if (Pos('<14A>', Msg) > 0) and (R.My_Serno = 0) then begin
      R.My_Serno := SerNum^.NextSerialNumberByBand(R.band);
      if Assigned(Net) then Net^.SendSerNumData;
    end;
    MakeSubstitutions(Msg, 150, R);
    if not RttyTerm1^.KeyBoardLive then RttyTerm1^.Transmit(True);
    if Pos('<BT>', Msg) = 0 then
      RttyTerm1^.SendString(Msg)
    else begin
      while Pos('<BT>', Msg) > 0 do begin
        RttyTerm1^.SendString(Copy(Msg, 1, Pos('<BT>', Msg)-1));
        Delete(Msg, 1, Pos('<BT>', Msg)+3);
        BreakText := '';
        if GetBreakText(BreakText) then RttyTerm1^.SendString(BreakText);
      end;
      RttyTerm1^.SendString(Msg); {remainder}
    end;
    if not RttyTerm1^.KeyBoardLive then RttyTerm1^.Transmit(False);
  end;

  procedure TRttyWin.ChangeBand(WhichWay : Char; var R : log_rec);
  begin
    case WhichWay of
      'U' : if R.band = B10 then R.band := B80
            else Inc(R.band);
      'D' : if R.band = B80 then R.band := B10
            else Dec(R.band);
    end;
    if Assigned(Radio) then begin
      Radio^.SetDisplayFrequency(Radio^.FavFreq[R.band]);
      with RadioTimer do
        ExpireTics := StartTics;  {v2.5a force redisplay}
    end;
    RttyIni^.SetOrAddProfileString('Band', 'CURRENT', BandS[r.band]);
  end;





end.
