{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$F+,O+}
unit cSection;
interface

  {* FUNCTIONS AVAILABLE FROM THIS UNIT *}

uses
  TpRarray,
  Rectypes;

const
  Num_Sections = 80;

type

  PSectionList = ^TSectionList;
  TSectionList = object
                 private
                   Section_Tbl    : array[1..Num_Sections] of Mult_Rec;
                 public
                   constructor Init;
                   Destructor  Done;
                   function    GetSection(const Sc : Byte; const Band : BandType) : boolean;
                   procedure   SetSection(const Sc : Byte; const Band : BandType);
                   procedure   ReSetSection(const Sc : Byte; const Band : BandType);
                   function    HowMany(const Band : BandType) : longint;
                   procedure   Display;
                   procedure   Print;
                   function    BandString(const Sc : Byte) : string;
                   function    GetSectionNo(const Pr : String) : Byte;
                   function    Load(const Qth : string; const Band : BandType) : Boolean;
                   function    GetSectionByCall(const Call : String) : string;
                 end;

  {* NOW DEFINE THE FUNCTIONS *}

var
  SectionList : PSectionList;

procedure MakeSectionList;
procedure KillSectionList;

implementation

uses
  DebugRtns,
  TpString,
  LowLvl,
  TpCrt,
  cColors,
  cDist_W,
  c_Contest,
  IniFile,
  TpMouse,
  TpSort,
  TpWindow,
  Keys,
  cEditWin,
  cMsgWin,
  Routines,
  OneTimers;





  const
  Sections  : array[1..Num_Sections] of String[3] =
  ('AB','AK','AL','AR','AZ','BC','CO','CT','DE','EB','EMA','ENY'
  ,'EPA','EWA','GA','IA','ID','IL','IN','KS','KY','LA','LAX','MAR'
  ,'MB','MDC','ME','MI','MN','MO','MS','MT','NC','ND','NE','NFL'
  ,'NH','NL','NLI','NM','NNJ','NNY','NTX','NV','NWT','OH','OK'
  ,'ON','OR','ORG','PAC','PR','QC','RI','SB','SC','SCV','SD','SDG'
  ,'SF','SFL','SJV','SK','SNJ','STX','SV','TN','UT','VA','VI','VT'
  ,'WCF','WI','WMA','WNY','WPA','WTX','WV','WWA','WY');


  constructor TSectionList.Init;
  var
    I : Integer;
    Mult : Mult_Rec;
    Band : BandType;
  begin
    for Band := B80 to TOT do
      Mult.Band[Band] := False;
    for I := 1 to Num_Sections do
      Section_Tbl[I] := Mult;
    {$IFDEF DEBUG}
    AppendDebugMessage('SectionTbl created');
    {$ENDIF}
  end;

  Destructor  TSectionList.Done;
  begin
    {$IFDEF DEBUG}
    AppendDebugMessage('SectionTbl destroyed');
    {$ENDIF}
  end;

  function    TSectionList.GetSection(const Sc : Byte; const Band : BandType) : boolean;
  begin
    {$IFDEF DEBUG}
    if (Sc = 0) or (Sc > Num_Sections) then
      AppendDebugMessage('CTSectionList.GetSection, invalid section# :'+long2str(Sc));
    {$ENDIF}
    if (Sc = 0) or (Sc > Num_Sections) then begin
      GetSection := False;
      Exit;
    end;
    GetSection := Section_Tbl[Sc].Band[Band]
  end;

  procedure   TSectionList.SetSection(const Sc : Byte; const Band : BandType);
  begin
    {$IFDEF DEBUG}
    if (Sc = 0) or (Sc > Num_Sections) then
      AppendDebugMessage('CTSectionList.SetSection, invalid section# :'+long2str(Sc));
    {$ENDIF}
    if (Sc = 0) or (Sc > Num_Sections) then Exit;
    Section_Tbl[Sc].Band[Band] := True;
    Section_Tbl[Sc].Band[TOT]  := True;
  end;

  procedure   TSectionList.ReSetSection(const Sc : Byte; const Band : BandType);
  var
    b : bandType;
  begin
    {$IFDEF DEBUG}
    if (Sc = 0) or (Sc > Num_Sections) then
      AppendDebugMessage('CTSectionList.ReSetSection, invalid section# :'+long2str(Sc));
    {$ENDIF}
    if (Sc = 0) or (Sc > Num_Sections) then Exit;
    Section_Tbl[Sc].Band[Band] := False;
    Section_Tbl[Sc].Band[TOT] := False;
    for B := B80 to B10 do begin
      if GetSection(Sc,B) then
        Section_Tbl[Sc].Band[TOT]  := True;
    end;
  end;

  function    TSectionList.HowMany(const Band : BandType) : longint;
  var
    L       : longint;
    Sc      : Word;
  begin
    L := 0;
    for Sc := 1 to Num_Sections do
      if GetSection(Sc,Band) then Inc(L);
    HowMany := L;
    {$IFDEF DEBUG}
    AppendDebugMessage('TSectionList.HowMany = '+Bands[Band]+'M '+long2str(l));
    {$ENDIF}
  end;

  procedure TSectionList.Display;
  var
    W              : Word;
    col, Row, ix   : Byte;
    Win            : WindowPtr;
    S              : String[5];
    Band           : BandType;
  begin
    with Colors^ do begin
     if not MakeWindow(Win, 1, 2, 80, 18, True, True, True,
                       WindowAttr, FrameAttr, HeaderAttr,
                       'Sections Multiplier Check Sheet') then Exit;
     HideMouse;
     DisplayWindow(Win);
     for col := 0 to 5 do
       for Row := 1 to 14 do
       begin
         ix := col*14+row;
         if ix <= Num_Sections then
           FastWriteWindow(Pad(Trim(Sections[ix]), 3)+' |'+BandString(ix)+'|', Row+1, col*13+2, WindowAttr);
       end;
     FastWrite('Press Any Key to continue', 18, 2, WindowAttr);
    end;
    ShowMouse;
    ReadKeyorButton;
    KillWindow(Win);
  end;

  procedure TSectionList.Print;
  var
    Str  : String[80];
    ix   : Byte;
    PrintFile : Text;
  begin
    Assign(PrintFile, FileName('SEC'));
    {$I-} Rewrite(PrintFile); {$I+}
    MsgWin^.Display('Preparing: '+FileName('SEC'));
    Writeln(PrintFile,'Section Multiplier sheet for '+RttyIni^.GetProfileString('Call','Profile',''));
    Writeln(PrintFile,'');
    for ix := 1 to Num_Sections do begin
      Str := ' '+Pad(Sections[ix], 4)+' |'+BandString(ix)+'|';
      Writeln(PrintFile,Str);
    end;
    Write(PrintFile,^L);
    Close(PrintFile);
  end;

  function  TSectionList.BandString(const Sc : Byte) : string;
  var
    s : string[5];
    Band : BandType;
  begin
    S := '.....';
    for Band := B80 to B10 do
      if GetSection(Sc,Band) then S[Ord(Band)+1] := '*';
    BandString := S;
  end;

  function TSectionList.GetSectionNo(const Pr : String) : Byte;
  var
    P : String[3];
    ix : Byte;
  begin
    P := Trim(Copy(Pr, 1, 3));
    for ix := 1 to Num_Sections do
      if P = Sections[ix] then begin
        GetSectionNo := ix;
        Exit;
      end;
    GetSectionNo := 0;
  end;

  function TSectionList.Load(const Qth : string; const Band : BandType) : Boolean;
  var
    T  : Boolean;
    ix : Byte;
  begin
    T := False;
    ix := GetSectionNo(Qth);
    if ix > 0 then T := not GetSection(ix,Band);
    SetSection(ix,Band);
    SetSection(ix,TOT);
    Load := T;
  end;

  function    TSectionList.GetSectionByCall(const Call : String) : string;
  var
    f3 : string[3];
    Temp_qth : string[3];
    Dist : byte;
  begin
    Temp_Qth := '';
    F3 := Copy(Call, 1, 3);
    if F3 = 'VE8' then Temp_Qth := 'NT' else
    if F3 = 'VY1' then Temp_Qth := 'YT' else
    if F3 = 'VY2' then Temp_Qth := 'PE' else
    if F3 = 'VE7' then Temp_Qth := 'BC' else
    if F3 = 'VE6' then Temp_Qth := 'AB' else
    if F3 = 'VE5' then Temp_Qth := 'SK' else
    if F3 = 'VE4' then Temp_Qth := 'MB' else
    if F3 = 'VE3' then Temp_Qth := 'ON' else
    if F3 = 'VE2' then Temp_Qth := 'QC' else
    if F3 = 'VE1' then Temp_Qth := 'NS' else
    if F3 = 'VO2' then Temp_Qth := 'NF' else
    if F3 = 'VO1' then Temp_Qth := 'NF';
    if Temp_Qth = '' then begin { if none above, go by the district }
      Dist := 0;
      if Assigned(WDistList) then Dist := WDistList^.GetDistNo(Call);
      case Dist of
        1 : Temp_Qth := ''; { can't tell easily }
        2 : Temp_Qth := 'QC'; { mni more qc than pe, so default to qc }
        3 : Temp_Qth := 'ON';
        4 : Temp_Qth := 'MB';
        5 : Temp_Qth := 'SK';
        6 : Temp_Qth := 'AB';
        7 : Temp_Qth := 'BC';
        8 : Temp_Qth := 'NT';
        9 : Temp_Qth := ''; { can't tell easily }
      end;
    end;
    GetSectionByCall := Temp_Qth;
  end;





  procedure MakeSectionList;
  begin
    SectionList := New(PSectionList, Init);
    if not assigned(SectionList) then begin
      MessageBox('Severe Error!',
                 'Unable to create Section list',
                 'Contact Wyvern Technology',
                 'Consider reinstalling from diskette');
      CloseDownScreen;
      Halt;
    end;
  end;

  procedure KillSectionList;
  begin
    if Assigned(SectionList) then begin
      Dispose(SectionList, Done);
      SectionList := NIL;
    end;
  end;



end.
