{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$F+,O+}
unit CSerNum;

interface

uses Rectypes;

type

  PSerNum = ^TSerNum;
  TSerNum = object
              private
                SerialNumber : Array[BandType] of word;
                SerialNumberAll : word;
              public
              constructor Init;
              destructor Done; virtual;
              function NextSerialNumberByBand(const Band : BandType) : word;
              function NextSerialNumberInAll(const Band : BandType) : word;
              function CurrentSerialNumberByBand(const Band : BandType) : word;
              function CurrentSerialNumberInAll : word;
              procedure SetSerialNumber(const Band : bandtype; const SN : word);
              procedure SetSerialNumberInAll(const SN : word);
              procedure Modify;
            end;

var
  SerNum : PSerNum;

implementation

uses
  Dos,
  Keys,
  Diskio,
  cMsgWin,
  OneTimers,
  LowLvl,
  TpDos,
  TpCrt,
  TpEntry,
  TpMouse,
  TpString,
  TpWindow,
  cColors,
  DebugRtns;


  constructor TSerNum.Init;
  var
    Band : BandType;
  begin
    for Band := B80 to B10 do
      SerialNumber[Band] := 0;
    SerialNumberAll := 0;
    {$IFDEF DEBUG}
    AppendDebugMessage('SerNum created');
    {$ENDIF}

  end;

  destructor TSerNum.Done;
  begin
    {$IFDEF DEBUG}
    AppendDebugMessage('SerNum destroyed');
    {$ENDIF}
  end;

  function TSerNum.NextSerialNumberByBand(const Band : BandType) : word;
  begin
    {$IFDEF DEBUG}
    AppendDebugMessage('TSerNum.NextSerialNumberByBand');
    {$ENDIF}
    if SerialNumberAll > LogFile^.NumRecs then begin
      SerialNumberAll := LogFile^.NumRecs;
      MsgWin^.Display('SERIAL NUMBER ADJUSTMENT MADE!');
    end;
    if SerialNumber[band] > LogFile^.NumRecs then begin
      SerialNumber[band] := LogFile^.NumRecs;
      MsgWin^.Display('SERIAL NUMBER ADJUSTMENT MADE!');
    end;
    inc(SerialNumberAll);
    inc(SerialNumber[Band]);
    NextSerialNumberByBand := SerialNumber[Band];
  end;

  function TSerNum.NextSerialNumberInAll(const Band : BandType) : word;
  begin
    {$IFDEF DEBUG}
    AppendDebugMessage('TSerNum.NextSerialNumberInAll');
    {$ENDIF}
    inc(SerialNumberAll);
    inc(SerialNumber[Band]);
    NextSerialNumberInAll := SerialNumberAll;
  end;

  function TSerNum.CurrentSerialNumberByBand(const Band : BandType) : word;
  begin
    {$IFDEF DEBUG}
    AppendDebugMessage('TSerNum.CurrentSerialNumberByBand');
    {$ENDIF}
    CurrentSerialNumberByBand := SerialNumber[Band];
  end;

  function TSerNum.CurrentSerialNumberInAll : word;
  begin
    {$IFDEF DEBUG}
    AppendDebugMessage('TSerNum.CurrentSerialNumberInAll');
    {$ENDIF}
    CurrentSerialNumberInAll := SerialNumberAll;
  end;

  procedure TSerNum.SetSerialNumber(const Band : bandtype; const SN : word);
  begin
    if SN > SerialNumber[Band] then SerialNumber[Band] := SN;
    if SN > SerialNumberAll    then SerialNumberAll := SN;
  end;

  procedure TSerNum.SetSerialNumberInAll(const SN : word);
  begin
    if SN > SerialNumberAll then SerialNumberAll := SN;
  end;

  procedure TSerNum.Modify;
  var
    Esr         : EsRecord;
    ExitCommand : EStype;
    ModPtr      : WindowPtr;
    Hold        : array[BandType] of word;
    HoldAll     : word;
    band        : bandtype;
  begin
    with Colors^ do
      MakeWindow(ModPtr, 3, 3, 26, 13, True, True, False, WindowAttr,
        FrameAttr, HeaderAttr, 'Serial Numbers');
    HideMouse;
    DisplayWindow(ModPtr);
    InitEsRecord(Esr);
    for Band := B80 to B10 do
      Hold[Band] := SerialNumber[Band];
    HoldAll := SerialNumberAll;
    AddWordField(Esr, '80 M', 5,  6, '9999', 5,  12, 0, 0, 9999, Hold[B80]);
    AddWordField(Esr, '40 M', 6,  6, '9999', 6,  12, 0, 0, 9999, Hold[B40]);
    AddWordField(Esr, '20 M', 7,  6, '9999', 7,  12, 0, 0, 9999, Hold[B20]);
    AddWordField(Esr, '15 M', 8,  6, '9999', 8,  12, 0, 0, 9999, Hold[B15]);
    AddWordField(Esr, '10 M', 9,  6, '9999', 9,  12, 0, 0, 9999, Hold[B10]);
    AddWordField(Esr, 'ALL' , 11, 6, '9999', 11, 12, 0, 0, 9999, HoldAll);
    CtrlEnter(13, 4);
    ShowMouse;
    AddCmd(ESEnd, kbEnd) ;
    AddCmd(ESHome, kbHome) ;
    AddCmd(EsQuit, kbEsc);
    repeat
      ExitCommand := EditScreen(Esr, 0, False);
    until ExitCommand in [EsQuit, EsDone];
    if ExitCommand = EsDone then begin
      for Band := B80 to B10 do
        SerialNumber[Band] := Hold[Band];
      SerialNumberAll := HoldAll;
    end;
    AddCmd(ESmy25, kbEnd);
    AddCmd(ESmy23, kbHome);
    AddCmd(Esmy49, kbEsc);
    KillWindow(ModPtr);
    DisposeEditScreen(Esr);
  end;

end.
