{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$F+,O+}
unit cSpTbl;
interface

  {* FUNCTIONS AVAILABLE FROM THIS UNIT *}

uses
  TpRarray,
  Rectypes;

const
  Num_SP_Provs = 16;

type

  PSpList = ^TSpList;
  TSpList = object
                 private
                   SP_Tbl    : array[1..Num_SP_Provs] of Mult_Rec;
                 public
                   constructor Init;
                   Destructor  Done;
                   function    GetProv(const Pv : Byte; const Band : BandType) : boolean;
                   procedure   SetProv(const Pv : Byte; const Band : BandType);
                   procedure   ReSetProv(const Pv : Byte; const Band : BandType);
                   function    HowManyProvs(const Band : BandType) : longint;
                   procedure   Display;
                   procedure   Print;
                   function    BandString(const Pv : Byte) : string;
                   function    GetProvNo(const Pr : String) : Byte;
                   function    Load(const Qth : string; const Band : BandType) : Boolean;
                 end;

  {* NOW DEFINE THE FUNCTIONS *}

var
  SpList : PSpList;

procedure MakeSpList;
procedure KillSpList;

implementation

uses
  DebugRtns,
  TpString,
  IniFile,
  TpCrt,
  TpMouse,
  c_Contest,
  LowLvl,
  TpSort,
  TpWindow,
  Keys,
  cEditWin,
  Routines,
  cMsgWin,
  cColors,
  OneTimers;

  const
  SP_Provs  : array[1..Num_SP_Provs] of String[1] = (
  'B','C','D','F','G','J','K','L','M','O','P','R','S','U','W','Z');

  constructor TSpList.Init;
  var
    I : Integer;
    Mult : Mult_Rec;
    Band : BandType;
  begin
    for Band := B80 to TOT do
      Mult.Band[Band] := False;
    for I := 1 to Num_SP_Provs do
      SP_Tbl[I] := Mult;
    {$IFDEF DEBUG}
    AppendDebugMessage('SpList created');
    {$ENDIF}
  end;

  Destructor  TSpList.Done;
  begin
    {$IFDEF DEBUG}
    AppendDebugMessage('SpList destroyed');
    {$ENDIF}
  end;

  function    TSpList.GetProv(const Pv : Byte; const Band : BandType) : boolean;
  begin
    {$IFDEF DEBUG}
    if (Pv = 0) or (Pv > Num_SP_Provs) then
      AppendDebugMessage('CSPProv.GetProv, invalid SP Prov# :'+long2str(Pv));
    {$ENDIF}
    if (Pv = 0) or (Pv > Num_SP_Provs) then begin
      GetProv := False;
      Exit;
    end;
    GetProv := SP_Tbl[Pv].Band[Band]
  end;

  procedure   TSpList.SetProv(const Pv : Byte; const Band : BandType);
  begin
    {$IFDEF DEBUG}
    if (Pv = 0) or (Pv > Num_SP_Provs) then
      AppendDebugMessage('CSpProv.SetProv, invalid SP Prov# :'+long2str(Pv));
    {$ENDIF}
    if (Pv = 0) or (Pv > Num_SP_Provs) then Exit;
    SP_Tbl[Pv].Band[Band] := True;
    SP_Tbl[Pv].Band[TOT]  := True;
  end;

  procedure   TSpList.ReSetProv(const Pv : Byte; const Band : BandType);
  var
    b : bandType;
  begin
    {$IFDEF DEBUG}
    if (Pv = 0) or (Pv > Num_SP_Provs) then
      AppendDebugMessage('CSpProv.ReSetProv, invalid SP Prov# :'+long2str(Pv));
    {$ENDIF}
    if (Pv = 0) or (Pv > Num_SP_Provs) then Exit;
    SP_Tbl[Pv].Band[Band] := False;
    SP_Tbl[Pv].Band[TOT] := False;
    for B := B80 to B10 do begin
      if GetProv(Pv,B) then
        SP_Tbl[Pv].Band[TOT]  := True;
    end;
  end;

  function    TSpList.HowManyProvs(const Band : BandType) : longint;
  var
    L       : longint;
    Pv      : Word;
  begin
    L := 0;
    for Pv := 1 to Num_SP_Provs do
      if GetProv(Pv,Band) then Inc(L);
    HowManyProvs := L;
    {$IFDEF DEBUG}
    AppendDebugMessage('TSpList.HowManyProvs = '+Bands[Band]+'M '+long2str(l));
    {$ENDIF}
  end;

  procedure TSpList.Display;
  var
    W              : Word;
    col, Row, ix   : Byte;
    Win            : WindowPtr;
    S              : String[5];
    Band           : BandType;
  begin
    With colors^ do begin
      if not MakeWindow(Win, 1, 2, 34, 14, True, True, True,
                        WindowAttr, FrameAttr, HeaderAttr,
                        'SP Provinces Multipliers') then Exit;
      HideMouse;
      DisplayWindow(Win);
      for col := 0 to 1 do
        for Row := 1 to 8 do begin
          ix := col*8+row;
          if ix <= Num_SP_Provs then
            FastWriteWindow(Pad(Trim(SP_Provs[ix]), 3)+' |'+BandString(ix)+'|', Row+1, col*13+2, WindowAttr);
        end;
      FastWrite('Press Any Key to continue', 14, 2, WindowAttr);
    end;
    ShowMouse;
    ReadKeyorButton;
    KillWindow(Win);
  end;

  procedure TSpList.Print;
  var
    Str  : String[80];
    ix   : Byte;
    PrintFile : Text;
  begin
    Assign(PrintFile, FileName('SP'));
    {$I-} Rewrite(PrintFile); {$I+}
    MsgWin^.Display('Preparing: '+FileName('SP'));
    Writeln(PrintFile,'SP Province Multiplier sheet for '+RttyIni^.GetProfileString('Call','Profile',''));
    Writeln(PrintFile,'');
    for ix := 1 to Num_SP_Provs do begin
      Str := ' '+Pad(SP_Provs[ix], 4)+' |'+BandString(ix)+'|';
      Writeln(PrintFile,Str);
    end;
    Write(PrintFile,^L);
    Close(PrintFile);
  end;

  function  TSpList.BandString(const Pv : Byte) : string;
  var
    s : string[5];
    Band : BandType;
  begin
    S := '.....';
    for Band := B80 to B10 do
      if GetProv(Pv,Band) then S[Ord(Band)+1] := '*';
    BandString := S;
  end;

  function TSpList.GetProvNo(const Pr : String) : Byte;
  var
    P : String[2];
    ix : Byte;
  begin
    P := Pr[1];
    for ix := 1 to Num_SP_Provs do
      if P = SP_Provs[ix] then begin
        GetProvNo := ix;
        Exit;
      end;
    GetProvNo := 0;
  end;

  function TSPList.Load(const Qth : string; const Band : BandType) : Boolean;
  var
    T  : Boolean;
    ix : Byte;
  begin
    T := False;
    ix := GetProvNo(Qth);
    if ix > 0 then T := not GetProv(ix,Band);
    SetProv(ix,Band);
    SetProv(ix,TOT);
    Load := T;
  end;


  procedure MakeSpList;
  begin
    SpList := New(PSpList, Init);
    if not assigned(SpList) then begin
      MessageBox('Severe Error!',
                 'Unable to create SP Prov list',
                 'Contact Wyvern Technology',
                 'Consider reinstalling from diskette');
      CloseDownScreen;
      Halt;
    end;
  end;

  procedure KillSpList;
  begin
    if Assigned(SpList) then Dispose(SpList, Done);
    SpList := NIL;
  end;



end.
