{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$F+,O+}
unit cState;
interface

  {* FUNCTIONS AVAILABLE FROM THIS UNIT *}

uses
  TpRarray,
  Rectypes;

const
  Num_States = 48;

type

  PStateList = ^TStateList;
  TStateList = object
                 private
                   St_Tbl    : array[1..Num_States] of Mult_Rec;
                 public
                   constructor Init;
                   Destructor  Done;
                   function    GetState(const St : Byte; const Band : BandType) : boolean;
                   function    GetStateByName(const St : String; const Band : BandType) : boolean;
                   procedure   SetState(const St : Byte; const Band : BandType);
                   procedure   SetStateByName(const St : String; const Band : BandType);
                   procedure   ReSetState(const St : Byte; const Band : BandType);
                   function    HowMany(const Band : BandType) : longint;
                   procedure   Display;
                   procedure   Print;
                   procedure   PrintStatesAndVE;
                   function    BandString(const St : Byte) : string;
                   function    GetStateNo(const St : String) : Byte;
                   function    GetZoneByName(const St : String) : Byte;
                   function    Name(const St : Byte) : string;
                   function    AdjustQth(const S : String) : string;
                   function    Load(const Qth : string; const Band : BandType) : Boolean;
                 end;

  {* NOW DEFINE THE FUNCTIONS *}

var
  StateList : PStateList;

procedure MakeStateList;
procedure KillStateList;

implementation

uses
  DebugRtns,
  TpString,
  TpCrt,
  TpMouse,
  IniFile,
  TpSort,
  c_Contest,
  TpWindow,
  Keys,
  cEditWin,
  LowLvl,
  cProvince,
  cMsgWin,
  cColors,
  Routines,
  OneTimers;

  const

  Names : array[1..Num_States] of String[2] = (
{1}    'CT', 'MA', 'ME', 'NH', 'RI', 'VT',             {1-6}
{2}    'NJ', 'NY',                                     {7-8}
{3}    'DE', 'MD', 'PA',                               {9-11}
{4}    'AL', 'FL', 'GA', 'KY', 'NC', 'SC', 'TN', 'VA', {12-19}
{5}    'AR', 'LA', 'MS', 'NM', 'OK', 'TX',             {20-25}
{6}    'CA',                                           {26}
{7}    'AZ', 'ID', 'MT', 'NV', 'OR', 'UT', 'WA', 'WY', {27-34}
{8}    'MI', 'OH', 'WV',                               {35-37}
{9}    'IL', 'IN', 'WI',                               {38-40}
{0}    'CO', 'IA', 'KS', 'MN', 'MO', 'NE', 'ND', 'SD');{41-48}

  CQZones  : array[1..Num_States] of Byte = (
{1}         5, 5, 5, 5, 5, 5,
{2}         5, 5,
{3}         5, 5, 5,
{4}         4, 5, 5, 4, 5, 5, 4, 5,
{5}         4, 4, 4, 4, 4, 4,
{6}         3,
{7}         3, 3, 4, 3, 3, 3, 3, 4,
{8}         4, 4, 5,
{9}         4, 4, 4,
{0}         4, 4, 4, 4, 4, 4, 4, 4);

  function TStateList.AdjustQth(const S : String) : string;
  begin
    if S = 'MAS' then AdjustQth := 'MA'
    else
      if S = 'CON' then AdjustQth := 'CT'
    else
      if S = 'CAL' then AdjustQth := 'CA'
    else
      if S = 'ILL' then AdjustQth := 'IL'
    else
      if S = 'ALA' then AdjustQth := 'AL'
    else
      AdjustQth := Trim(S);
  end;

  constructor TStateList.Init;
  var
    I : Integer;
    Mult : Mult_Rec;
    Band : BandType;
  begin
    for Band := B80 to TOT do
      Mult.Band[Band] := False;
    for I := 1 to Num_States do
      St_Tbl[I] := Mult;
    {$IFDEF DEBUG}
    AppendDebugMessage('StateList created');
    {$ENDIF}
  end;

  Destructor  TStateList.Done;
  begin
    {$IFDEF DEBUG}
    AppendDebugMessage('StateList destroyed');
    {$ENDIF}
  end;

  function    TStateList.GetState(const St : Byte; const Band : BandType) : boolean;
  begin
    {$IFDEF DEBUG}
    if (St = 0) or (St > Num_States) then
      AppendDebugMessage('CState.GetState, invalid State# :'+long2str(St));
    {$ENDIF}
    if (St = 0) or (St > Num_States) then begin
      GetState := False;
      Exit;
    end;
    GetState := St_Tbl[St].Band[Band]
  end;

  function    TStateList.GetStateByName(const St : string; const Band : BandType) : boolean;
  begin
    GetStateByName := GetState(GetStateNo(AdjustQth(St)),Band);
  end;

  procedure   TStateList.SetState(const St : Byte; const Band : BandType);
  begin
    {$IFDEF DEBUG}
    if (St = 0) or (St > Num_States) then
      AppendDebugMessage('CState.SetState, invalid State# :'+long2str(St));
    {$ENDIF}
    if (St = 0) or (St > Num_States) then Exit;
    St_Tbl[St].Band[Band] := True;
    St_Tbl[St].Band[TOT]  := True;
  end;

  procedure   TStateList.SetStateByName(const St : String; const Band : BandType);
  begin
    SetState(GetStateNo(AdjustQth(St)),Band);
  end;

  procedure   TStateList.ReSetState(const St : Byte; const Band : BandType);
  var
    b : bandType;
  begin
    {$IFDEF DEBUG}
    if (St = 0) or (St > Num_States) then
      AppendDebugMessage('CState.ReSetState, invalid State# :'+long2str(St));
    {$ENDIF}
    if (St = 0) or (St > Num_States) then Exit;
    St_Tbl[St].Band[Band] := False;
    St_Tbl[St].Band[TOT] := False;
    for B := B80 to B10 do begin
      if GetState(St,B) then
        St_Tbl[St].Band[TOT]  := True;
    end;
  end;

  function    TStateList.HowMany(const Band : BandType) : longint;
  var
    L       : longint;
    St      : Word;
  begin
    L := 0;
    for St := 1 to Num_States do
      if GetState(St,Band) then Inc(L);
    HowMany := L;
    {$IFDEF DEBUG}
    AppendDebugMessage('TStateList.HowMany = '+Bands[Band]+'M '+long2str(l));
    {$ENDIF}
  end;

  procedure TStateList.Print;
  var
    Str  : String[80];
    ix   : Byte;
    PrintFile : Text;
  begin
    Assign(PrintFile, FileName('USA'));
    {$I-} Rewrite(PrintFile); {$I+}
    MsgWin^.Display('Preparing: '+FileName('USA'));
    Writeln(PrintFile,'US States Multiplier sheet for '+RttyIni^.GetProfileString('Call','Profile',''));
    Writeln(PrintFile,'');
    for ix := 1 to Num_States do begin
      Str := ' '+Pad(Names[ix], 4)+' |'+BandString(ix)+'|';
      Writeln(PrintFile,Str);
    end;
    Write(PrintFile,^L);
    Close(PrintFile);
  end;

  procedure TStateList.PrintStatesAndVE;
  var
    Str  : String[80];
    ix   : Byte;
    PrintFile : Text;
    Band : BandType;
  begin
    Assign(PrintFile, FileName('USA'));
    {$I-} Rewrite(PrintFile); {$I+}
    MsgWin^.Display('Preparing: '+FileName('USA'));
    Str := 'US/VE Multiplier sheet for '+RttyIni^.GetProfileString('Call','Profile','');
    Writeln(PrintFile,Str);
    Writeln(PrintFile,'');
    for ix := 1 to Num_States do begin
      Str := ' '+Pad(Name(ix), 4)+' |'+BandString(ix)+'|  ';
      case Contest of
        ARRL,
        TARA,
        NAQP,
        DXPD,
        CQWW : if ix <= ProvList^.NumProvs then
                 Str := Str + ' ' +
                 Pad(ProvList^.Name(ix), 4)+' |'+
                 ProvList^.BandString(ix)+'|';
      end;
      Writeln(PrintFile,Str);
    end;
    Writeln(PrintFile,^L);
    Close(PrintFile);
  end;


  function  TStateList.BandString(const St : Byte) : string;
  var
    s : string[5];
    Band : BandType;
  begin
    S := '.....';
    for Band := B80 to B10 do
      if GetState(St,Band) then S[Ord(Band)+1] := '*';
    BandString := S;
  end;

  function TStateList.GetStateNo(const St : String) : Byte;
  var
    ix : Byte;
  begin
    for ix := 1 to Num_States do
      if St = Names[ix] then begin
        GetStateNo := ix;
        Exit;
      end;
    GetStateNo := 0;
  end;

  function    TStateList.GetZoneByName(const St : String) : Byte;
  var
    ix : byte;
  begin
    ix := GetStateNo(AdjustQth(St));
    if ix > 0 then GetZoneByName := CqZones[Ix]
              else GetZoneByName := 0;
  end;

  function    TStateList.Name(const St : Byte) : string;
  begin
    if St > 0 then Name := Names[St]
              else Name := '';
  end;

  procedure TStateList.Display;
  var
    W              : Word;
    col, Row, ix   : Byte;
    ScreenRow      : byte;
    Win            : WindowPtr;
    S              : String[5];
    Band : BandType;

    procedure ShowState(const Row, Col : byte);
    begin
      FastWriteWindow(Pad(Name(ix), 3)+' |'+BandString(ix)+'|', Row, Col, Colors^.WindowAttr);
    end;

    procedure ShowProvince(const Row, Col : byte);
    begin
      FastWriteWindow(Pad(ProvList^.Name(ix), 3)+' |'+ProvList^.BandString(ix)+'|', Row, Col, Colors^.WindowAttr);
    end;

  begin
    if not (Contest in [NAQP,ARRL,TARA,CQWW,DXPD]) then Exit;
    with Colors^ do
      if not MakeWindow(Win, 1, 1, 70, 20, True, True, True,
                        WindowAttr, FrameAttr, HeaderAttr,
                        'US/VE Multiplier Check Sheet') then Exit;
    HideMouse;
    DisplayWindow(Win);
    {column 1}
    for ix := 1 to 6 do {1}
      ShowState(ix+1, 2);
    for ix := 7 to 8 do {2}
      ShowState(ix+2, 2);
    for ix := 9 to 11 do {3}
      ShowState(ix+3, 2);
    {column 2}
    for ix := 12 to 19 do {4}
      ShowState(ix-11+1, 15);
    for ix := 20 to 25 do   {5}
      ShowState(ix-11+2, 15);
    {column 3}
    for ix := 26 to 26 do   {6}
      ShowState(ix-25+1, 28);
    for ix := 27 to 34 do    {7}
      ShowState(ix-25+2, 28);
    for ix := 35 to 37 do     {8}
      ShowState(ix-25+3, 28);
    {column 4}
    for ix := 38 to 40 do    {9}
      ShowState(ix-37+1, 41);
    for ix := 41 to 48 do    {0}
      ShowState(ix-37+2, 41);
    case Contest of
      NAQP,
      CQWW,
      ARRL,
      TARA,
      DXPD : for ix := 1 to ProvList^.NumProvs do ShowProvince(ix+1, 57);
    end;
    FastWrite('Press Any Key to continue', 20,2 ,Colors^.WindowAttr);
    ShowMouse;
    ReadKeyorButton;
    KillWindow(Win);
  end;

  function TStateList.Load(const Qth : string; const Band : BandType) : Boolean;
  var
    T  : Boolean;
    ix : Byte;
  begin
    T := False;
    ix := GetStateNo(Qth);
    if ix > 0 then begin
      T := not GetState(ix,Band);
      SetState(ix,Band);
      SetState(ix,TOT);
    end;
    Load := T;
  end;

  procedure MakeStateList;
  begin
    StateList := New(PStateList, Init);
    if not assigned(StateList) then begin
      MessageBox('Severe Error!',
                 'Unable to create US States list',
                 'Contact Wyvern Technology',
                 'Consider reinstalling from diskette');
      CloseDownScreen;
      Halt;
    end;
    {$IFDEF DEBUG}
    AppendDebugMessage('StateList created');
    {$ENDIF}
  end;

  procedure KillStateList;
  begin
    if Assigned(StateList) then begin
      Dispose(StateList, Done);
      StateList := NIL;
      {$IFDEF DEBUG}
      AppendDebugMessage('StateList destroyed');
      {$ENDIF}
    end;
  end;



end.
