{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$F+,O+}
unit cYearTbl;
interface

  {* FUNCTIONS AVAILABLE FROM THIS UNIT *}

uses
  TpRarray,
  Rectypes;

type

  PYearList = ^TYearList;
  TYearList = object
                 private
                   BaseYear    : Word;
                   Year_Tbl    : array[0..255] of Mult_Rec; {1900 thru 1900+255}
                 public
                   constructor Init;
                   Destructor  Done;
                   function    GetYear(const Yr : Word; const Band : BandType) : boolean;
                   procedure   SetYear(const Yr : Word; const Band : BandType);
                   procedure   ReSetYear(const Yr : Word; const Band : BandType);
                   function    HowManyYears(const Band : BandType) : longint;
                   procedure   Display;
                   procedure   Print;
                   function    BandString(const Yr : Word) : string;
                   function    Load(const Yr : word; const Band : BandType) : Boolean;
                 end;

  {* NOW DEFINE THE FUNCTIONS *}

var
  YearList : PYearList;

procedure MakeYearList;
procedure KillYearList;

implementation

uses
  DebugRtns,
  TpString,
  TpCrt,
  cColors,
  TpMouse,
  TpSort,
  c_Contest,
  LowLvl,
  TpWindow,
  Keys,
  IniFile,
  cEditWin,
  cMsgWin,
  Routines,
  OneTimers;


  constructor TYearList.Init;
  var
    Ix : Word;
    Mult : Mult_Rec;
    Band : BandType;
  begin
    BaseYear := RttyIni^.GetProfileWord('BaseYearLic','PROFILE',1900);
    for Band := B80 to TOT do
      Mult.Band[Band] := False;
    for Ix := 0 to 255 do
      Year_Tbl[Ix] := Mult;
    {$IFDEF DEBUG}
    AppendDebugMessage('YearList created');
    {$ENDIF}
  end;


  Destructor  TYearList.Done;
  begin
    {$IFDEF DEBUG}
    AppendDebugMessage('YearList destroyed');
    {$ENDIF}
  end;


  function    TYearList.GetYear(const Yr : Word; const Band : BandType) : boolean;
  {$ifdef debug}
  var
    s : string;
  {$endif}
  begin
    if (Yr < BaseYear) or (Yr > BaseYear+255) then begin
      GetYear := False; { outside range }
      Exit;
    end;
    GetYear := Year_Tbl[yr-BaseYear].Band[Band];
      {$IFDEF DEBUG}
      s := 'GetYear Yr='+long2str(yr)+' Band='+long2str(ord(band))++' Result=';
      if Year_Tbl[yr-BaseYear].Band[Band] then s := s + 'TRUE'
      else s := s + 'FALSE';
      AppendDebugMessage(s);
      {$ENDIF}
  end;

  procedure   TYearList.SetYear(const Yr : Word; const Band : BandType);
  {$ifdef debug}
  var
    s : string;
  {$endif}
  begin
    if (Yr < BaseYear) or (Yr > BaseYear+255) then Exit;
      {$IFDEF DEBUG}
      s := 'SetYear Yr='+long2str(yr)+' Band='+long2str(ord(band))++' Before=';
      if Year_Tbl[yr-BaseYear].Band[Band] then s := s + 'TRUE, After='
      else s := s + 'FALSE, After=';
      {$ENDIf}
    Year_Tbl[Yr-BaseYear].Band[Band] := True;
    Year_Tbl[Yr-BaseYear].Band[TOT]  := True;
  end;

  procedure   TYearList.ReSetYear(const Yr : Word; const Band : BandType);
  var
    b : bandType;
  begin
    if (Yr < BaseYear) or (Yr > BaseYear+255) then Exit;
    Year_Tbl[Yr-BaseYear].Band[Band] := False;
    Year_Tbl[Yr-BaseYear].Band[TOT] := False;
    for B := B80 to B10 do begin
      if Year_Tbl[Yr-BaseYear].Band[B] then
        Year_Tbl[Yr-BaseYear].Band[TOT]  := True;
    end;
  end;

  function    TYearList.HowManyYears(const Band : BandType) : longint;
  var
    L       : longint;
    Yr      : Word;
  begin
    L := 0;
    for Yr := BaseYear to BaseYear+255 do
      if Year_Tbl[Yr-BaseYear].Band[Band] then Inc(L);
    HowManyYears := L;
  end;

  procedure TYearList.Display;
  var
    W              : Word;
    ix             : Word;
    col, Row       : Byte;
    Win            : WindowPtr;
    S              : String[5];
    Band           : BandType;
  begin
    with Colors^ do begin
     if not MakeWindow(Win, 1, 2, 80, 25, True, True, True,
                       WindowAttr, FrameAttr, HeaderAttr,
                       'Year Licensed Multiplier Check Sheet') then Exit;
     HideMouse;
     DisplayWindow(Win);
     for col := 0 to 5 do
       for Row := 0 to 19 do
       begin
         ix := col*20+row;
         FastWriteWindow(Long2str(BaseYear+ix)+' |'+BandString(BaseYear+ix)+'|', Row+2, col*13+2, WindowAttr);
       end;
     FastWrite('Press Any Key to continue', 25, 2, WindowAttr);
    end;
    ShowMouse;
    ReadKeyorButton;
    KillWindow(Win);
  end;

  procedure TYearList.Print;
  var
    Str  : String[80];
    ix   : Byte;
    PrintFile : Text;
    Band : BandType;
  begin
    Assign(PrintFile, FileName('YRS'));
    {$I-} Rewrite(PrintFile); {$I+}
    MsgWin^.Display('Preparing: '+FileName('YRS')); {v3.0}
    Writeln(PrintFile,'Year Licensed Multiplier sheet for '+RttyIni^.GetProfileString('Call','Profile',''));
    Writeln(PrintFile,'');
    for ix := 1 to 60 do begin
      Str := ' '+long2str(BaseYear+ix)+' |';
      for Band := B80 to B10 do
        if Year_Tbl[ix].Band[Band] then Str := Str+'*'
          else Str := Str+'.';
      Str := Str+'|   ';
      Str := Str + ' '+long2str(BaseYear+60+ix)+' |';
      for Band := B80 to B10 do
        if Year_Tbl[60+ix].Band[Band] then Str := Str+'*'
          else Str := Str+'.';
      Str := Str+'|';
      Str := Str + '   '+long2str(BaseYear+120+ix)+' |';
      for Band := B80 to B10 do
        if Year_Tbl[120+ix].Band[Band] then Str := Str+'*'
          else Str := Str+'.';
      Str := Str+'|';
      Str := Str + '   '+long2str(BaseYear+180+ix)+' |';
      for Band := B80 to B10 do
        if Year_Tbl[180+ix].Band[Band] then Str := Str+'*'
          else Str := Str+'.';
      Str := Str+'|';
      Writeln(PrintFile,Str);
    end;
    Write(PrintFile,^L);
    Close(PrintFile);
  end;

  function  TYearList.BandString(const Yr : Word) : string;
  var
    s : string[5];
    Band : BandType;
  begin
    if (Yr < BaseYear) or (Yr > BaseYear+255) then begin
      BandString := '.....';
      Exit;
    end;
    S := '.....';
    for Band := B80 to B10 do
      if Year_Tbl[Yr-BaseYear].Band[Band] then S[Ord(Band)+1] := '*';
    BandString := S;
  end;

  function TYearList.Load(const Yr : word; const Band : BandType) : Boolean;
  begin
    if (Yr < BaseYear) or (Yr > BaseYear+255) then begin
      Load := True;
      Exit;
    end;
    Load := not Year_Tbl[Yr-BaseYear].Band[Band];
    SetYear(Yr,Band);
  end;



  procedure MakeYearList;
  begin
    YearList := New(PYearList, Init);
    if not assigned(YearList) then begin
      MessageBox('Severe Error!',
                 'Unable to create Year list',
                 'Contact Wyvern Technology',
                 'Consider reinstalling from diskette');
      AppendDebugMessage('FAILED: create YearList');
      CloseDownScreen;
      Halt;
    end;
    {$IFDEF DEBUG}
    AppendDebugMessage('YearList created');
    {$ENDIF}
  end;

  procedure KillYearList;
  begin
    if Assigned(YearList) then begin
      Dispose(YearList, Done);
      YearList := NIL;
      {$IFDEF DEBUG}
      AppendDebugMessage('YearList destroyed');
      {$ENDIF}
    end;
  end;



end.
