{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$F+,O+}
unit CZone;
interface

  {* FUNCTIONS AVAILABLE FROM THIS UNIT *}

uses
  Diskio,
  TpRarray,
  Rectypes;

type

  PZoneList = ^TZoneList;
  TZoneList = object
                 private
                   Zone_Tbl       : array[1..40] of Mult_Rec;
                 public
                   constructor Init;
                   Destructor  Done;
                   function    GetZone(const Zn : Byte; const Band : BandType) : boolean;
                   procedure   SetZone(const Zn : Byte; const Band : BandType);
                   procedure   ReSetZone(const Zn : Byte; const Band : BandType);
                   function    HowManyZones(const Band : BandType) : longint;
                   procedure   DisplayMap;
                   procedure   Print;
                   function    BandString(const Zn : Byte) : string;
                   function    GetZoneNo(const R : log_rec) : Byte;
                 end;

  {* NOW DEFINE THE FUNCTIONS *}

var
  ZoneList : PZoneList;

procedure MakeZoneList;
procedure KillZoneList;

implementation

uses
  DebugRtns,
  TpString,
  TpCrt,
  TpMouse,
  TpSort,
  TpWindow,
  Keys,
  cMsgWin,
  cEditWin,
  cCountry,
  cDist_w,
  cDist_ve,
  cProvince,
  cState,
  cColors,
  c_Contest,
  IniFile,
  LowLvl,
  Routines,
  OneTimers;

  constructor TZoneList.Init;
  var
    I : Integer;
    Mult : Mult_Rec;
    Band : BandType;
  begin
    for Band := B80 to TOT do
      Mult.Band[Band] := False;
    for I := 1 to 40 do
      Zone_Tbl[I] := Mult;
    {$IFDEF DEBUG}
    AppendDebugMessage('ZoneList created');
    {$ENDIF}
  end;

  Destructor  TZoneList.Done;
  begin
    {$IFDEF DEBUG}
    AppendDebugMessage('ZoneList destroyed');
    {$ENDIF}
  end;

  function    TZoneList.GetZone(const Zn : Byte; const Band : BandType) : boolean;
  begin
    {$IFDEF DEBUG}
    if (Zn = 0) or (Zn > 40) then
      AppendDebugMessage('CZone.SetZone, invalid zone# :'+long2str(Zn));
    {$ENDIF}
    if (Zn = 0) or (Zn > 40) then begin
      GetZone := False;
      Exit;
    end;
    GetZone := Zone_Tbl[Zn].Band[Band]
  end;

  procedure   TZoneList.SetZone(const Zn : Byte; const Band : BandType);
  begin
    {$IFDEF DEBUG}
    if (Zn = 0) or (Zn > 40) then
      AppendDebugMessage('CZone.SetZone, invalid zone# :'+long2str(Zn));
    {$ENDIF}
    if (Zn = 0) or (Zn > 40) then Exit;
    Zone_Tbl[Zn].Band[Band] := True;
    Zone_Tbl[Zn].Band[TOT]  := True;
  end;

  procedure   TZoneList.ReSetZone(const Zn : Byte; const Band : BandType);
  var
    b : bandType;
  begin
    {$IFDEF DEBUG}
    if (Zn = 0) or (Zn > 40) then
      AppendDebugMessage('CZone.ResetZone, invalid zone# :'+long2str(Zn));
    {$ENDIF}
    if (Zn = 0) or (Zn > 40) then Exit;
    Zone_Tbl[Zn].Band[Band] := False;
    Zone_Tbl[Zn].Band[TOT] := False;
    for B := B80 to B10 do begin
      if GetZone(Zn,B) then
        Zone_Tbl[Zn].Band[TOT]  := True;
    end;
  end;

  function    TZoneList.HowManyZones(const Band : BandType) : longint;
  var
    L       : longint;
    Zn      : Word;
  begin
    L := 0;
    for Zn := 1 to 40 do
      if GetZone(Zn,Band) then Inc(L);
    HowManyZones := L;
    {$IFDEF DEBUG}
    AppendDebugMessage('TZoneList.HowManyZones = '+Bands[Band]+'M '+long2str(l));
    {$ENDIF}
  end;

  procedure TZoneList.DisplayMap;
  var
    ZoneWin        : WindowPtr;
    Zone           : Byte;
    Band           : BandType;
  begin
    with Colors^ do begin
      if not MakeWindow(ZoneWin, 3, 3, 49, 11, True, True, True, WindowAttr,
                        FrameAttr, HeaderAttr, 'Zone Map') then Exit;
     HideMouse;
     DisplayWindow(ZoneWin) ;
      FastWriteWindow('1111111111222222222233333333334',1, 14, WindowAttr);
      FastWriteWindow('1234567890123456789012345678901234567890',2, 5, WindowAttr);
      FastWriteWindow('80',3, 2, WindowAttr);
      FastWriteWindow('40',4, 2, WindowAttr);
      FastWriteWindow('20',5, 2, WindowAttr);
      FastWriteWindow('15',6, 2, WindowAttr);
      FastWriteWindow('10',7, 2, WindowAttr);
      for Band := B80 to B10 do
        for Zone := 1 to 40 do
          if GetZone(Zone,Band) then FastWriteWindow('*', 3+Ord(Band), 4+Zone, WindowAttr)
          else FastWriteWindow('.', 3+Ord(Band), 4+Zone, WindowAttr);
      ChangeAttributeWindow(44, 3+Ord(EditW^.EditRecs[1].Band), 1, HiLiteAttr);
      FastWrite('Press Any Key to continue', 11, 4, WindowAttr);
    end;
    ShowMouse;
    ReadKeyorButton;
    KillWindow(ZoneWin);
  end;

  procedure TZoneList.Print;
  var
    Str            : String[80];
    Zone           : Byte;
    PrintFile      : Text;
    Band : BandType;
  begin
    Assign(PrintFile, FileName('ZON'));
    {$I-} Rewrite(PrintFile); {$I+}
    MsgWin^.Display('Preparing: '+FileName('ZON'));
    Str := 'Zone Multiplier sheet for '+RttyIni^.GetProfileString('Call','Profile','');
    Str := Pad(Str, 70)+'Page 1';
    Writeln(PrintFile,Str);
    Writeln(PrintFile,'');
    for Zone := 1 to 40 do begin
      Str := ' '+
             LeftPadCh(Long2Str(Zone), '0', 2)+
             ' |'+
             BandString(Zone)+
             '|';
        Writeln(PrintFile,Str);
      end;
    Write(PrintFile,^L);
    Close(PrintFile);
  end;

  function  TZoneList.BandString(const Zn : Byte) : string;
  var
    s : string[5];
    Band : BandType;
  begin
    S := '.....';
    for Band := B80 to B10 do
      if GetZone(Zn,Band) then S[Ord(Band)+1] := '*';
    BandString := S;
  end;

  function TZoneList.GetZoneNo(const R : log_rec) : Byte;
  var
    Zone : Byte;
  begin
    GetZoneNo := 0;
    if Trim(R.Call) = '' then Exit;
    Zone := 0;
    if CountryList^.Identify(R) = NoCountry then Exit;
    Zone := CountryList^.Ct.CQzone;
    if (CountryList^.Ct.Code = 'W') and Assigned(WDistList) then begin
      case WDistList^.GetDistNo(R.Call) of
        1, 2, 3, 4, 8 : Zone := 5;
        0, 5, 9 : Zone := 4;
        6, 7 : Zone := 3;
      end;
      if R.His_Qth <> '' then Zone := StateList^.GetZoneByName(R.His_Qth);
    end;
    if CountryList^.Ct.Code = 'VE' then
      if R.His_Qth <> '' then Zone := ProvList^.GetZoneByName(R.His_Qth)
      else Zone := ProvList^.GetZoneByCall(R.Call);
    if Zone < 41 then GetZoneNo := Zone
                 else GetZoneNo := 0;
  end;

{-------------------------------------------------------------}

  procedure MakeZoneList;
  begin
    ZoneList := New(PZoneList, Init);
    if not assigned(ZoneList) then begin
      MessageBox('Severe Error!',
                 'Unable to create zone list',
                 'Contact Wyvern Technology',
                 'Consider reinstalling from diskette');
      CloseDownScreen;
      Halt;
    end;
    {$IFDEF DEBUG}
    AppendDebugMessage('ZoneList created');
    {$ENDIF}
  end;

  procedure KillZoneList;
  begin
    if Assigned(ZoneList) then begin
      Dispose(ZoneList, Done);
      ZoneList := NIL;
      {$IFDEF DEBUG}
      AppendDebugMessage('ZoneList destroyed');
      {$ENDIF}
    end;
  end;



end.
