{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit c_anar;

interface

uses
  c_contest,
  Diskio,
  Rectypes;
type
  PANAR = ^TANAR;
  TANAR = object(TContest)
            constructor init;
            procedure Header; virtual;
            procedure Footer; virtual;
            procedure Detail; virtual;
            function LogLine(const R : Log_Rec) : String; virtual;
            destructor done; virtual;
            procedure PrintSummary; virtual;
            function  PointValue(const SomeQso : log_rec) : byte; virtual;
            procedure RemoveOldMults(var SomeQso : log_rec); virtual;
            procedure AddNewMults(var SomeQso : log_rec); virtual;
            procedure IdAnyMults(const SomeQso : log_rec); virtual;
            function ContestId : string; virtual;
            procedure HandleAltU; virtual;
            procedure DisplayScoreWin; virtual;
            function  ScoreContest : LongInt; virtual;
            procedure ScoreCountPoints; virtual;
            function  FieldsPerRecord : byte; virtual;
            function MultsCol : byte; virtual;
            procedure BuildEditWindow; virtual;
            function  MakeMultStr(const R : log_rec) : String; virtual;
            procedure LogContact(var ThisContact : log_rec); virtual;
            function  AdvisorSaysOK(const R : log_rec) : Boolean; virtual;
            procedure Writelog; virtual;
            procedure FillRec(const Call : String; var R : log_rec); virtual;
          end;

implementation

uses
  tpEntry,
  cEditWin,
  cClock,
  IniFile,
  cColors,
  cCountry,
  cDist_w,
  cDist_ve,
  cDist_vk,
  cDist_ja,
  cContinent,
  cDupeTbl,
  TpDate,
  TpString,
  TpWindow,
  TpCrt,
  TpMouse,
  TAHigh,
  TAccess,
  DebugRtns,
  cRttyWin,
  cZone,
  cProvince,
  LowLvl,
  cLogAnar,
  Routines;


  { TANAR.........................................................TANAR}
  { TANAR.........................................................TANAR}
  { TANAR.........................................................TANAR}
  { TANAR.........................................................TANAR}
  { TANAR.........................................................TANAR}

  constructor TANAR.init;
  begin
    inherited init;
    LogFile := New(PANARLogFile, Init);
    MakeZoneList;
    MakeProvList('ARRL'); {uses ARRL list}
    MakeContList;
    MakeWDistList;
    MakeVEDistList;
    MakeVKDistList;
    MakeJADistList;
    MakeCountryList('ARRL.CTY','ARRL.PFX');
    {$IFDEF DEBUG}
    AppendDebugMessage('ANAR created');
    {$ENDIF}
  end;

  destructor TANAR.done;
  begin
    KillZoneList;
    KillProvList;
    KillContList;
    KillWDistList;
    KillVEDistList;
    KillVKDistList;
    KillJADistList;
    KillCountryList;
    inherited done;
    {$IFDEF DEBUG}
    AppendDebugMessage('ANAR destroyed');
    {$ENDIF}
  end;

  function TANAR.LogLine(const R : Log_Rec) : String;
  var
    VkBonusPoints : String[3];
    t : String[127];
  begin
    t := BandS[R.Band]+' '+
      Clock^.Date2DateString(R.Date)+
      TimeToTimeString(' hhmm ', R.Ts)+
      Pad(R.Call, 12)+' '+
      Long2Str(R.My_Rst)+'9 '+
      TimeToTimeString('hhmm ', R.My_Time)+
      LeftPadCh(Long2Str(R.My_Zone), '0', 2)+' '+
      Long2Str(R.His_RST)+'9 '+
      TimeToTimeString('hhmm ', R.His_Time)+
      LeftPadCh(Long2Str(R.His_Zone), '0', 2)+' '+
      Pad(R.His_Country, 5)+' ';
    t := t+LeftPad(Long2Str(R.Point), 3)+' ';
    if R.New_Country then t := t+'C ';
    if R.New_Qth then t := t+'D ';
    if R.New_Continent then t := t+'N';
    if R.Dupe then t := t+'-DUPE';
    VkBonusPoints := '';
    if (not R.Dupe) and
    (MyCountry <> 'VK') and
    (R.His_Country = 'VK') then
      case R.Band of
        B10 : VkBonusPoints := '300';
        B15 : VkBonusPoints := '200';
        B20 : VkBonusPoints := '100';
        B40 : VkBonusPoints := '400';
        B80 : VkBonusPoints := '500';
      end;
    t := Pad(t, 68)+VkBonusPoints;
    LogLine := t;
  end;

  procedure TANAR.Header;
  begin
    inherited Header;
    PrintF('Band Date Time Call         RST Time Zn RST Time Zn Cntry Pts Mults/Bonus');
  end;

  procedure TANAR.Footer;
  begin
    PrintF('Page '+Pad(Long2Str(PageCT), 2)+
      '  Pts(excl. bonus) = '+Long2Str(sub_pts)+
      '  Dist = '+Long2Str(sub_Dist)+
      '  Cty = '+Long2Str(Sub_Cty)+
      '  Cont = '+Long2Str(sub_Cont));
  end;

  procedure TANAR.Detail;
  begin
    if Linect = 0 then Header;
    Inc(sub_pts, a.Point);
    if a.New_Qth then Inc(sub_Dist);
    if a.New_Country then Inc(Sub_Cty);
    if a.New_Continent then Inc(sub_Cont);
    PrintF(LogLine(a));
  end;

  procedure TANAR.PrintSummary;   {v2.5a}
  var
    Bonus : array[BandType] of LongInt;
    Band : BandType;
  begin
    inherited PrintSummary;
    PrintSummaryTop;
    PrintF('Band    QSOs   Pts  Dist    DX  Cont');
    for Band := B80 to B10 do
      PrintF(Pad(BandS[Band], 6)+
        LeftPad(Long2Str(Num_Qso[Band]), 6)+
        LeftPad(Long2Str(Num_Points[Band]), 6)+
        LeftPad(Long2Str(Num_Districts[Band]), 6)+
        LeftPad(Long2Str(Num_Country[Band]), 6)+
        LeftPad(Long2Str(Num_Continents[Band]), 6));
    PrintF('-----   ----  ----  ----  ----  ----');
    PrintF('Total '+
      LeftPad(Long2Str(Num_Qso[TOT]), 6)+
      LeftPad(Long2Str(Num_Points[TOT]), 6)+
      LeftPad(Long2Str(Num_Districts[TOT]), 6)+
      LeftPad(Long2Str(Num_Country[TOT]), 6)+
      LeftPad(Long2Str(Num_Continents[TOT]), 6));
    Bonus[B80] := 0;
    Bonus[B40] := 0;
    Bonus[B20] := 0;
    Bonus[B15] := 0;
    Bonus[B10] := 0;
    LogFile^.ResetLog;
    LogFile^.Get_Next(a);
    while OK do begin
      if (a.His_Country = 'VK') and (MyCountry <> 'VK') then
        case a.Band of
          B10 : Inc(Bonus[B10], 300);
          B15 : Inc(Bonus[B15], 200);
          B20 : Inc(Bonus[B20], 100);
          B40 : Inc(Bonus[B40], 400);
          B80 : Inc(Bonus[B80], 500);
        end;
      LogFile^.Get_Next(a);
    end;
    if MyCountry <> 'VK' then begin
      PrintF('');
      PrintF(CharStr('-', 38));
      PrintF('');
      PrintF('VK Bonus Points');
      PrintF('80   '+LeftPad(Long2Str(Bonus[B80]), 6));
      PrintF('40   '+LeftPad(Long2Str(Bonus[B40]), 6));
      PrintF('20   '+LeftPad(Long2Str(Bonus[B20]), 6));
      PrintF('15   '+LeftPad(Long2Str(Bonus[B15]), 6));
      PrintF('10   '+LeftPad(Long2Str(Bonus[B10]), 6));
      PrintF('-----------');
      PrintF('Total'+LeftPad(Long2Str(Bonus[B10]+Bonus[B15]+Bonus[B20]+Bonus[B40]+Bonus[B80]), 6));
      PrintF('');
    end;
    PrintSummaryBottom;
    LogFile^.BlankLogRec(a);
  end;

  function  TANAR.PointValue(const SomeQso : log_rec) : byte;
  var
    CPV : Byte;
  begin
    CountryList^.Identify(SomeQso);
    CPV := 0;
    if (SomeQso.His_Zone in [1..40]) and (SomeQso.My_Zone in [1..40])
      then CPV := ExchangePointTable[SomeQso.My_Zone, SomeQso.His_Zone];
    if SomeQso.Call = 'VK2SG'
      then PointValue := CPV*2
      else PointValue := CPV;
  end;

  procedure TANAR.RemoveOldMults(var SomeQso : log_rec);
  begin
    RemoveCountryMult(SomeQso);
    RemoveDistMult(SomeQso);
    RemoveContinentMult(SomeQso);
  end;

  procedure TANAR.AddNewMults(var SomeQso : log_rec);
  begin
    with SomeQSo do begin
      { according to the rules, you have to track them for the 4 band mults,
        but you cannot count them as a regular country mult.  So the code
        below loads it into the county table, but marks the QSO as not a
        multiplier.
      }
      New_Country := CountryList^.Load(SomeQso,Band);
      if (CountryList^.Ct.Code = 'W') or
         (CountryList^.Ct.Code = 'VE') or
         (CountryList^.Ct.Code = 'VK') or
         (CountryList^.Ct.Code = 'JA') or
         (CountryList^.Ct.Code = MyCountry)
        then New_Country := False;
      if (WDistList^.GetDistNo(Call) = MyDistrict) and
         (His_Country = MyCountry) then {dont do anything else...}
      else begin
        if His_Country = 'W' then
          New_Qth := WDistList^.Load(Call,Band);
        if His_Country = 'VE' then
          New_Qth := VEDistList^.Load(Call,Band);
        if His_Country = 'VK' then
          New_Qth := VKDistList^.Load(Call,Band);
        if His_Country = 'JA' then
          New_Qth := JADistList^.Load(Call,Band);
      end;
      if ((MyCountry = 'W') or (MyCountry = 'VE') or (MyCountry = 'VK') or (MyCountry = 'JA')) and {v2.5c}
      (His_Country = MyCountry) and
      (WDistList^.GetDistNo(Call) = WDistList^.GetDistNo(RttyIni^.GetProfileString('Call','Profile','')))
      then
      New_Continent := False
      else
        if CountryList^.CtyNum <> NoCountry then begin
          New_Continent := ContList^.Load(CountryList^.Ct.Continent,Band);
        end;
    end;
  end;

  procedure TANAR.IdAnyMults(const SomeQso : log_rec);
  var
    ix : Byte;
    Dist : Byte;
    tc : string[5]; { temp ct.code, it is easier to type in }
    tcn : word;     { temp country num, it is easier to type}
  begin
    if RttyIni^.GetProfileBool('PutMsgs','Profile',True) then
    with SomeQso do begin
      if Dupe then Exit;
      if Pos('/MM', Call) > 0 then Exit;
      tcn := CountryList^.Identify(SomeQso);
      if tcn = NoCountry then Exit;
      tc := CountryList^.Ct.Code;
      if (not CountryList^.GetCountry(tcn,Band)) and
         (tc <> 'W') and (tc <> 'VE') and
         (tc <> 'VK') and (tc <> 'JA')
        then RttyWin^.PutString(^Y'-DX '^Z);
      Dist := WDistList^.GetDistNo(Call);
      if (Dist < 10) and (Dist <> MyDistrict) then begin
        if ((tc = 'W') and (not WDistList^.GetDist(Dist,Band))) then RttyWin^.PutString(^Y'-DIST '^Z);
        if ((tc = 'VE') and (not VEDistList^.GetDist(Dist,Band))) then RttyWin^.PutString(^Y'-DIST '^Z);
        if ((tc = 'VK') and (not VKDistList^.GetDist(Dist,Band))) then RttyWin^.PutString(^Y'-DIST '^Z);
        if ((tc = 'JA') and (not JADistList^.GetDist(Dist,Band))) then RttyWin^.PutString(^Y'-DIST '^Z);
      end;
    end;
  end;

  function TANAR.ContestId : string;
  begin
    ContestId := 'ANAR';
  end;

  procedure TANAR.HandleAltU;
  begin
    WDistList^.Display;
  end;

  procedure TANAR.DisplayScoreWin;
  var
    PointsPerQso   : Real;
    PointsPerMult  : Real;
    TotMults       : LongInt;
    Band           : BandType;
    ScoreWin       : WindowPtr;
  begin
    ScoreCountPoints;
    Score := ScoreContest;
    if Score > 0 then PointsPerQso := Score / Num_Qso[Tot]
    else PointsPerQso := 0;
    TotMults := Num_Country[TOT]+Num_Qth[TOT];
    If TotMults > 0 then PointsPerMult := Score / TotMults
    else PointsPerMult := 0;
    with Colors^ do begin
    MakeWindow(ScoreWin, MW_xhi-40, MW_Ylo+1, MW_xhi-2, MW_Ylo+11,
                  True, True, True, WindowAttr, FrameAttr, HeaderAttr, 'ANARTS Score Summary') ;
    HideMouse;
    DisplayWindow(ScoreWin) ;
    FastWriteWindow('Band    QSOs   Pts  Dist    DX  Cont', 1, 1, WindowAttr);
    for Band := B80 to B10 do
      FastWriteWindow(Pad(BandS[Band], 6)+
                                 LeftPad(Long2Str(Num_Qso[Band]), 6)+
                                 LeftPad(Long2Str(Num_Points[Band]), 6)+
                                 LeftPad(Long2Str(Num_Districts[Band]), 6)+
                                 LeftPad(Long2Str(Num_Country[Band]), 6) +
                                 LeftPad(Long2Str(Num_Continents[Band]), 6),
                                 2+Ord(Band), 1, WindowAttr);
    FastWriteWindow('-----   ----  ----  ----  ----  ----', 7, 1, WindowAttr);
    FastWriteWindow('Total '+
                               LeftPad(Long2Str(Num_Qso[TOT]), 6)+
                               LeftPad(Long2Str(Num_Points[TOT]), 6)+
                               LeftPad(Long2Str(Num_Districts[TOT]), 6)+
                               LeftPad(Long2Str(Num_Country[TOT]), 6) +
                               LeftPad(Long2Str(Num_Continents[TOT]), 6),
                               8, 1, WindowAttr);
    FastWriteWindow('Tot: '+Pad(Long2Str(Score),8)+
                      ' P/Q:'+Real2Str(PointsPerQSO, 4, 1)+
                      ' P/M:'+Real2Str(PointsPerMult, 4, 1),
                      9, 1, WindowAttr);
    FastWrite('Press Any Key to continue', 12, 42, WindowAttr);
    end;
    ShowMouse;
    ReadKeyorButton;
    KillWindow(ScoreWin);
  end;

  function TANAR.ScoreContest : LongInt;
  begin
    if MyCountry = 'VK' then
               ScoreContest := (Num_Points[TOT]*(Num_Districts[TOT]+Num_Country[TOT])
                 *Num_Continents[TOT])
             else
               ScoreContest := (Num_Points[TOT]*(Num_Districts[TOT]+Num_Country[TOT])
                 *Num_Continents[TOT])+VK_Bonus; {v2.5a}
  end;

  procedure TANAR.ScoreCountPoints;
  var
    Band : BandType;
    Cont : ContinentType;
    Country, QSO : Word;
  begin
    SetCountersToZero;
    for Band := B80 to B10 do begin
      for Country := 0 to CountryList^.NumCountries-1 do begin
        with CountryList^ do begin
          GetCtRec(Country);
          if (Ct.Code = 'W') or
             (Ct.Code = 'VE') or
             (Ct.Code = 'VK') or
             (Ct.Code = 'JA') or
             (Ct.Code = MyCountry)
           then       {do nothing}
           else
             if GetCountry(Country,Band) then Inc(Num_Country[Band]);
        end;
      end;
      Num_Districts[Band] := WDistList^.HowMany(Band)+
                             VEDistList^.HowMany(Band)+
                             VKDistList^.HowMany(Band)+
                             JADistList^.HowMany(Band);
      Num_Continents[Band] := ContList^.HowMany(Band);
    end;
    LogFile^.ResetLog;
    LogFile^.Get_Next(a);
    while OK do begin
      Inc(Num_Points[a.Band], a.Point);
      Inc(Num_Qso[a.Band]);
      if a.Dupe then Inc(Num_Dupes[a.Band]);
      if a.His_Country = 'VK' then
        case a.Band of
          B10 : Inc(VK_Bonus, 300);
          B15 : Inc(VK_Bonus, 200);
          B20 : Inc(VK_Bonus, 100);
          B40 : Inc(VK_Bonus, 400);
          B80 : Inc(VK_Bonus, 500);
        end;
      LogFile^.Get_Next(a);
    end;
    SumCounters;
    Num_Continents[TOT] := ContList^.HowMany(TOT); { special logic }
    if Num_Continents[TOT] > 6 then Num_Continents[TOT] := 6;
  end;

  function TANAR.FieldsPerRecord : byte;
  { returns the number of fields (across) on the screen }
  begin
    FieldsPerRecord := 10;
  end;

  function TANAR.MultsCol : byte;
  { returns the number of "mults" column on the screen }
  begin
    MultsCol := 7;
  end;

  procedure TANAR.BuildEditWindow;
  var
    I : Integer;
  begin
    for I := 1 to 5 do begin
      AddChoiceField(EntryEsr, '', I, 2, 'XX', I, 1, 0, 1,
        EditIncChoicePtr, EditW^.EditRecs[I].Band);
      AddTimeField(EntryEsr, '', I, 2, 'hhmm', I, 4, 0,
        MinTime, MaxTime, EditW^.EditRecs[I].Ts);
      AddStringField(EntryEsr, '', I, 2, '!!!!!!!!!!!!', I, 9, 12, 0,
        ValidateCallSignPtr, EditW^.EditRecs[I].Call); { they ALL have callsign first! }
      SetForceMode(True, True);
      AddByteField(EntryEsr, '', I, 2, '99', I, 22, 0,
        0, 40, EditW^.EditRecs[I].His_Zone);
      SetForceMode(False, False);
      AddTimeField(EntryEsr, '', I, 2, 'hhmm', I, 25, 0,
        MinTime, MaxTime, EditW^.EditRecs[I].His_Time);
      AddByteField(EntryEsr, '', I, 2, '99', I, 30, 0,
        0, 59, EditW^.EditRecs[I].His_Rst);
      SetForceMode(False, False);
      AddTimeField(EntryEsr, '', I, 2, 'hhmm',          I, 33, 0,
        MinTime, MaxTime, EditW^.EditRecs[I].My_Time);
      SetProtection(True);
      AddStringField(EntryEsr, '',  I, 2, '!!!!!',      I, 38, 5,
        0, nil, EditW^.Mults[I]);
      SetProtection(False);
      AddByteField(EntryEsr, '',    I, 2, '999',        I, 44, 0,
        0, 255, EditW^.EditRecs[I].Point);
      AddYesNoField(EntryEsr, '', I, 2,   '',           I, 48, 0,
        EditW^.EditRecs[I].Dupe);
    end;
    SetFieldLinks(EntryESR, 5,  2,  4);
    ActivateWindow(EditWin);
    FastWriteWindow('Mt Time Callsign     Zn Time RS Time Mults Pts Dupe', 1, 1, Colors^.HeaderAttr);
  end;

  function TANAR.MakeMultStr(const R : log_rec) : String;
  var
    T : String[5];
  begin
    T := '';
    if R.New_Country then T := T+'C'
    else T := T+' ';
    if R.New_Qth then T := T+'D'
    else T := T+' ';
    if R.New_Continent then T := T+'N'
    else T := T+' ';
    MakeMultStr := T;
  end;

  procedure TANAR.LogContact(var ThisContact : log_rec);
  var
    I : Integer;
  begin
    PreLogContact(ThisContact);
    with ThisContact do begin
      My_Zone := RttyIni^.GetProfileByte('CQZone','Profile',0);
      if CountryList^.Identify(ThisContact) <> NoCountry then
        His_Country := CountryList^.Ct.Code;
      if not AdvisorSaysOK(ThisContact) then Exit;
      Dupe := DupeTbl^.Load(ThisContact);
      case Dupe of
        True  : Point := 0;
        False : Point := PointValue(ThisContact);
      end;
      AddNewMults(ThisContact);
      if Pos('/MM', Call) > 0 then begin
        New_Country := False;
        New_Qth := False;
        New_Continent := False;
      end;
    end;
    inherited LogContact(ThisContact);
  end;


  function TANAR.AdvisorSaysOK(const R : log_rec) : Boolean;
  { t = ok to log this qso, false = dont log, they want to edit }
  var
    Missing : String[40];
  begin
    if NOT inherited AdvisorSaysOK(R) then begin { hard edits, no bypass allowed }
      AdvisorSaysOK := False;
      Exit; { hard edits, no bypass allowed }
    end;
    if NOT RttyIni^.GetProfileBool('Advice','PROFILE',True) then begin                     {dont want advisor window }
      AdvisorSaysOK := True;
      Exit;
    end;
    Missing := '';
    if R.His_Zone = 0 then begin
      if Missing <> '' then Missing := Missing+',';
      Missing := Missing + 'CQ Zone';
    end;
    if R.His_Time = 0 then begin
      if Missing <> '' then Missing := Missing+',';
      Missing := Missing + 'Time';
    end;
    AdvisorSaysOK := AdvisorWindow(Missing)
  end;

  procedure TANAR.Writelog;
  begin
    inherited Writelog;
    WDistList^.Print;
    ContList^.Print;
  end;

  procedure TANAR.FillRec(const Call : String; var R : log_rec);
    { returns info to R if we has QSO before with CALL         }
    { I also try to fill as much of the QSO record as possible }
  begin
    R.Call := Trim(Call);
    if CountryList^.Identify(R) <> NoCountry then R.His_Country := CountryList^.Ct.Code;
    if R.His_Country = 'VE' then R.His_Zone := ProvList^.GetZoneByCall(R.Call);
    if R.His_Zone = 0 then R.His_Zone := ZoneList^.GetZoneNo(R);
  end;

end.
