{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit c_arfd;

interface

uses
  Diskio,
  c_contest,
  Rectypes;
type

  Parfd = ^Tarfd;
  Tarfd = object(TContest)
            constructor init;
            procedure Header; virtual;
            procedure Footer; virtual;
            procedure Detail; virtual;
            function LogLine(const R : Log_Rec) : String; virtual;
            procedure PrintLog; virtual;
            destructor done; virtual;
            procedure PrintSummary; virtual;
            procedure RemoveOldMults(var SomeQso : log_rec); virtual;
            procedure AddNewMults(var SomeQso : log_rec); virtual;
            function ContestId : string; virtual;
            procedure HandleAltU; virtual;
            procedure DisplayScoreWin; virtual;
            procedure ScoreCountPoints; virtual;
            function  FieldsPerRecord : byte; VIRTUAL;
            function MultsCol : byte; virtual;
            procedure BuildEditWindow; virtual;
            function  MakeMultStr(const R : log_rec) : String; virtual;
            procedure LogContact(var ThisContact : log_rec); virtual;
            procedure Writelog; virtual;
            procedure FillRec(const Call : String; var R : log_rec); virtual;
            function  AdvisorSaysOK(const R : log_rec) : Boolean; virtual;
          end;



implementation

uses
  TpEntry,
  cEditWin,
  cClock,
  cDist_W,
  cCountry,
  cDupeTbl,
  TpDate,
  TpString,
  TAHigh,
  TpWindow,
  TpMouse,
  TpCrt,
  TAccess,
  DebugRtns,
  ArfdPrec,
  LowLvl,
  IniFile,
  cLogarfd,
  cCOlors,
  cBotWin,
  cSection,
  Routines;

  { Tarfd.........................................................Tarfd}
  { Tarfd.........................................................Tarfd}
  { Tarfd.........................................................Tarfd}
  { Tarfd.........................................................Tarfd}
  { Tarfd.........................................................Tarfd}

  constructor Tarfd.init;
  begin
    PrecList := NIL;
    inherited init;
    LogFile := New(ParfdLogFile, Init);
    PrecList := New(PPrecList, Init);
    if not Assigned(PrecList) then
      MessageBox('SEVERE ERROR',
                 'Unable to create precident list',
                 'Please re-install, or contact WF1B',
                 '');
    MakeWDistList;
    MakeSectionList;
    MakeCountryList('ARRL.CTY','ARRL.PFX');
    {$IFDEF DEBUG}
    AppendDebugMessage('ARFD created');
    {$ENDIF}
  end;

  destructor Tarfd.done;
  begin
    Dispose(PrecList, Done);
    PrecList := NIL;
    inherited done;
    KillSectionList;
    KillCountryList;
    KillWDistList;
    {$IFDEF DEBUG}
    AppendDebugMessage('ARFD destroyed');
    {$ENDIF}
  end;

  function Tarfd.LogLine(const R : Log_Rec) : String;
  var
    t : String[127];
  begin
    t := BandS[R.Band]+' '+
      Clock^.Date2DateString(R.Date)+
      TimeToTimeString(' hhmm ', R.Ts)+
      Pad(R.Call, 12)+' '+
      Pad(Copy(R.Comment,1,3),4)+
      Pad(R.His_Qth, 3)+' '+
      Long2Str(R.His_RST)+'9 '+
      Pad(R.His_Country, 5)+' ';
    if R.New_Country then t := t+' C '
    else t := t+'   ';
    if R.New_Qth then t := t+' Q '
    else t := t+'   ';
    if R.Dupe then t := t+'-DUPE-';
    if Length(t) > 79 then t := Copy(t, 1, 79);
    LogLine := t;
  end;

  procedure Tarfd.Header;
  begin
    inherited Header;
    PrintF('Band Date Time Call         Prc Sec Rst Cntry Mults');
  end;

  procedure Tarfd.Footer;
  begin
    PrintF('Page '+Pad(Long2Str(PageCT), 2));
  end;

  procedure Tarfd.Detail;
  begin
    if Linect = 0 then Header;
    PrintF(LogLine(a));
  end;

  procedure Tarfd.PrintLog;
  begin
    AllBands;
  end;

  procedure Tarfd.PrintSummary;
  var
    Band : BandType;
  begin
    inherited PrintSummary;
    PrintSummaryTop;
    PrintF('Band    QSOs   Sec    DX');
    for Band := B80 to B10 do
      PrintF(Pad(BandS[Band], 6)+
        LeftPad(Long2Str(Num_Qso[Band]), 6)+
        LeftPad(Long2Str(Num_Qth[Band]), 6)+
        LeftPad(Long2Str(Num_Country[Band]), 6));
    PrintF(CharStr('-', 32));
    PrintF('Total '+
      LeftPad(Long2Str(Num_Qso[TOT]), 6)+
      LeftPad(Long2Str(Num_Qth[TOT]), 6)+
      LeftPad(Long2Str(Num_Country[TOT]), 6));
    PrintSummaryBottom;
  end;

  procedure Tarfd.RemoveOldMults(var SomeQso : log_rec);
  begin
    RemoveCountryMult(SomeQso);
    RemoveSectionMult(SomeQso);
  end;

  procedure Tarfd.AddNewMults(var SomeQso : log_rec);
  begin
    with SomeQso do begin
      if (His_Country = 'W') or
         (His_Country = 'VE') then begin
           New_Country := False;
           New_Qth := SectionList^.Load(His_Qth, Band);
         end
      else
        New_Country := CountryList^.Load(SomeQso,Band);
      PrecList^.SetPrecidence(Call, Trim(Copy(Comment,1,3)));
    end;
  end;

  function Tarfd.ContestId : string;
  begin
    ContestId := 'ARFD';
  end;

  procedure Tarfd.HandleAltU;
  begin
    SectionList^.Display;
  end;

  procedure Tarfd.DisplayScoreWin;
  var
    PointsPerQso   : Real;
    PointsPerMult  : Real;
    TotMults       : LongInt;
    Band           : BandType;
    ScoreWin       : WindowPtr;
  begin
    ScoreCountPoints;
    Score := ScoreContest;
    if Score > 0 then PointsPerQso := Score / Num_Qso[Tot]
    else PointsPerQso := 0;
    TotMults := Num_Country[TOT]+Num_Qth[TOT];
    If TotMults > 0 then PointsPerMult := Score / TotMults
    else PointsPerMult := 0;
    with Colors^ do begin
    MakeWindow(ScoreWin, MW_xhi-40, MW_Ylo+1, MW_xhi-2, MW_Ylo+11,
                  True, True, True, WindowAttr, FrameAttr, HeaderAttr, 'ARRL Field Day Summary') ;
    HideMouse;
    DisplayWindow(ScoreWin) ;
    FastWriteWindow('Band    QSOs   Sec    DX', 1, 1, WindowAttr);
    for Band := B80 to B10 do
      FastWriteWindow(Pad(BandS[Band], 6)+
                                 LeftPad(Long2Str(Num_Qso[Band]), 6)+
                                 LeftPad(Long2Str(Num_Qth[Band]), 6)+
                                 LeftPad(Long2Str(Num_Country[Band]), 6),
                                 2+Ord(Band), 1, WindowAttr);
    FastWriteWindow('----    ----  ----  ----', 7, 1, WindowAttr);
    FastWriteWindow('Total '+
                               LeftPad(Long2Str(Num_Qso[TOT]), 6)+
                               LeftPad(Long2Str(Num_Qth[TOT]), 6)+
                               LeftPad(Long2Str(Num_Country[TOT]), 6),
                               8, 1, WindowAttr);
    FastWrite('Press Any Key to continue', 12, 42, WindowAttr);
    end;
    ShowMouse;
    ReadKeyorButton;
    KillWindow(ScoreWin);
  end;

  procedure Tarfd.ScoreCountPoints;
  var
    Band : BandType;
  begin
    SetCountersToZero;
    for Band := B80 to B10 do begin
      Num_Qth[Band]     := SectionList^.HowMany(Band);
      Num_Country[Band] := CountryList^.HowMany(Band);
    end;
    LogFile^.ResetLog;
    LogFile^.Get_Next(a);
    while OK do begin
      Inc(Num_Points[a.Band], a.Point);
      Inc(Num_Qso[a.Band]);
      if a.Dupe then Inc(Num_Dupes[a.Band]);
      LogFile^.Get_Next(a);
    end;
    SumCounters;
  end;

  function Tarfd.FieldsPerRecord : byte;
  { returns the number of fields (across) on the screen }
  begin
    FieldsPerRecord := 7;
  end;

  function Tarfd.MultsCol : byte;
  { returns the number of "mults" column on the screen }
  begin
    MultsCol := 5;
  end;

  procedure Tarfd.BuildEditWindow;
  var
    I : Integer;
  begin
    for I := 1 to 5 do begin
      AddChoiceField(EntryEsr, '', I, 2, 'XX', I, 1, 0, 1,
        EditIncChoicePtr, EditW^.EditRecs[I].Band);
      AddTimeField(EntryEsr, '', I, 2, 'hhmm', I, 4, 0,
        MinTime, MaxTime, EditW^.EditRecs[I].Ts);
      AddStringField(EntryEsr, '', I, 2, '!!!!!!!!!!!!', I, 9, 12, 0,
        ValidateCallSignPtr, EditW^.EditRecs[I].Call); { they ALL have callsign first! }
      AddStringField(EntryEsr, '', 1, 2, '!!!', I, 22,  3,
        0, nil, EditW^.EditRecs[I].Comment);
      AddStringField(EntryEsr, '', I, 2, '!!!', I, 26, 3, 0,
        ValidateQTHPtr, EditW^.EditRecs[I].His_Qth);
      SetProtection(True);
      AddStringField(EntryEsr, '',  I, 2, '!!!!!',  I, 30, 5,
        0, nil, EditW^.Mults[I]);
      SetProtection(False);
      AddYesNoField(EntryEsr, '', I, 2,   '',       I, 36, 0,
        EditW^.EditRecs[I].Dupe);
    end;
    SetFieldLinks(EntryESR, 4,  2,  3);
    ActivateWindow(EditWin);
    FastWriteWindow('Mt Time Callsign     Prc Sec Mults Dupe', 1, 1, Colors^.HeaderAttr);
  end;

  function Tarfd.MakeMultStr(const R : log_rec) : String;
  var
    T : String[5];
  begin
    T := '';
    if R.New_Country then T := T+'C'
    else T := T+' ';
    if R.New_Qth then T := T+'Q'
    else T := T+' ';
    MakeMultStr := T;
  end;

  procedure Tarfd.LogContact(var ThisContact : log_rec);
  var
    ix : byte;
  begin
    PreLogContact(ThisContact);
    with ThisContact do begin
      if CountryList^.Identify(ThisContact) <> NoCountry then
        His_Country := CountryList^.Ct.Code;
      if not AdvisorSaysOK(ThisContact) then Exit;
      AddNewMults(ThisContact);
      Dupe := DupeTbl^.Load(ThisContact);
      case Dupe of
        True  : Point := 0;
        False : Point := PointValue(ThisContact);
      end;
      if Pos('/MM', Call) > 0 then begin
        New_Country := False;
        New_Qth := False;
        New_Continent := False;
      end;
    end;
    inherited LogContact(ThisContact);
  end;

  procedure Tarfd.Writelog;
  begin
    inherited Writelog;
    SectionList^.Print;
  end;

  procedure Tarfd.FillRec(const Call : String; var R : log_rec);
    { returns info to R if we has QSO before with CALL         }
    { I also try to fill as much of the QSO record as possible }
  var
    Slot : Word;
    F3 : String[3];
  begin
    R.Call := Trim(Call);
    if DupeTbl^.GetDupeByCall(Call, Slot) then
      R.His_Qth := DupeTbl^.GetQth;
    if CountryList^.Identify(R) = NoCountry then Exit;
    R.His_Country := CountryList^.Ct.Code;
    if R.His_Country = 'VE' then R.His_Qth := SectionList^.GetSectionByCall(R.Call);
    F3 := PrecList^.GetPrecidence(R.Call);
    if Trim(f3) <> '' then begin
      R.Comment := F3;
    end;
  end;

  function TARFD.AdvisorSaysOK(const R : log_rec) : Boolean;
  { t = ok to log this qso, false = dont log, they want to edit }
  var
    Missing : String[40];
  begin
    if NOT inherited AdvisorSaysOK(R) then begin { hard edits, no bypass allowed }
      AdvisorSaysOK := False;
      Exit; { hard edits, no bypass allowed }
    end;
    if NOT RttyIni^.GetProfileBool('Advice','PROFILE',True) then begin                     {dont want advisor window }
      AdvisorSaysOK := True;
      Exit;
    end;
    Missing := '';
    if (R.Comment = '') then begin
      if Missing <> '' then Missing := Missing+',';
      Missing := Missing+'Precident';
    end;
    if CountryList^.IsUSorCanada(R) and (R.His_Qth = '') then begin
      OK := False;
      if Missing <> '' then Missing := Missing+',';
      Missing := Missing + 'Section';
    end;
    AdvisorSaysOK := AdvisorWindow(Missing)
  end;


end.
