{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit c_arrl;

interface

uses
  c_contest,
  DiskIo,
  Rectypes;

type
  PARRL = ^TARRL;
  TARRL = object(TContest)
            constructor init;
            procedure Header; virtual;
            procedure Footer; virtual;
            procedure Detail; virtual;
            function LogLine(const R : Log_Rec) : String; virtual;
            procedure PrintLog; virtual;
            destructor done; virtual;
            procedure PrintSummary; virtual;
            procedure RemoveOldMults(var SomeQso : log_rec); virtual;
            procedure AddNewMults(var SomeQso : log_rec); virtual;
            procedure IdAnyMults(const SomeQso : log_rec); virtual;
            function  ContestId : string; virtual;
            procedure HandleAltU; virtual;
            procedure DisplayScoreWin; virtual;
            function  ScoreContest : LongInt; virtual;
            procedure ScoreCountPoints; virtual;
            function  FieldsPerRecord : byte; virtual;
            function  MultsCol : byte; virtual;
            procedure BuildEditWindow; virtual;
            function  MakeMultStr(const R : log_rec) : String; virtual;
            procedure LogContact(var ThisContact : log_rec); virtual;
            procedure RemoveStateMult(var R : log_rec); virtual;
            procedure RemoveVeProvinceMult(var R : log_rec); virtual;
            function  AdvisorSaysOK(const R : log_rec) : Boolean; virtual;
            procedure Writelog; virtual;
            procedure FillRec(const Call : String; var R : log_rec); virtual;
          end;


implementation

uses
  cRttyWin,
  cClock,
  cEditwin,
  cCountry,
  cState,
  cProvince,
  cDupeTbl,
  cDist_W,
  TpDate,
  TpString,
  cColors,
  TpWindow,
  TpCrt,
  TpEntry,
  TpMouse,
  TAHigh,
  TAccess,
  DebugRtns,
  IniFile,
  LowLvl,
  cLogArrl,
  Routines;


  { TARRL.........................................................TARRL}
  { TARRL.........................................................TARRL}
  { TARRL.........................................................TARRL}
  { TARRL.........................................................TARRL}
  { TARRL.........................................................TARRL}
  constructor TARRL.init;
  begin
    inherited init;
    LogFile := New(PARRLLogFile, Init);
    MakeStateList;
    MakeProvList(ContestId);
    MakeWDistList;
    MakeCountryList('ARRL.CTY','ARRL.PFX');
    {$IFDEF DEBUG}
    AppendDebugMessage('ARRL created');
    {$ENDIF}
  end;

  destructor TARRL.done;
  begin
    KillStateList;
    KillProvList;
    KillWDistList;
    KillCountryList;
    inherited done;
    {$IFDEF DEBUG}
    AppendDebugMessage('ARRL destroyed');
    {$ENDIF}
  end;

  function TARRL.LogLine(const R : Log_Rec) : String;
  var
    t : String[127];
  begin
    t := BandS[R.Band]+' '+
      Clock^.Date2DateString(R.Date)+
      TimeToTimeString(' hhmm ', R.Ts)+
      Pad(R.Call, 12)+' '+
      Long2Str(R.My_Rst)+'9 '+
      Pad(R.My_Qth, 4)+
      LeftPadCh(Long2Str(R.My_Serno), '0', 4)+' '+
      Long2Str(R.His_RST)+'9 '+
      Pad(R.His_Qth, 3)+' '+
      LeftPadCh(Long2Str(R.His_Serno), '0', 4)+' '+
      Pad(R.His_Country, 5)+' ';
    if R.New_Country then t := t+' C  '
    else t := t+'    ';
    if R.New_Qth then t := t+' Q  '
    else t := t+'    ';
    t := t+Long2Str(R.Point);
    if R.Dupe then t := t+'-DUPE';
    LogLine := t;
  end;

  procedure TARRL.Header;
  begin
    inherited Header;
    PrintF('Band Date Time Call         RST QTH Ser# RST QTH Ser# Cntry Cty QTH Pt');
  end;

  procedure TARRL.Footer;
  begin
    PrintF(CharStr(' ', 60)+'--- --- ---');
    PrintF('Page '+
      Pad(Long2Str(PageCT), 2)+
      CharStr(' ', 53)+
      Pad(Long2Str(Sub_Cty), 3)+
      ' '+
      Pad(Long2Str(Sub_Qth), 3)+
      ' '+
      Pad(Long2Str(sub_pts), 3));
  end;

  procedure TARRL.Detail;
  begin
    if Linect = 0 then Header;
    if a.New_Country then Inc(Sub_Cty);
    if a.New_Qth then Inc(Sub_Qth);
    sub_pts := sub_pts+a.Point;
    PrintF(LogLine(a));
  end;

  procedure TARRL.PrintLog;
  begin
    AllBands;
  end;

  procedure TARRL.PrintSummary;
  var
    Band : BandType;
  begin
    inherited PrintSummary;
    PrintSummaryTop;
    PrintF('Band    QSOs   Pts   QTH    DX');
    for Band := B80 to B10 do
      PrintF(Pad(BandS[Band], 6)+
        LeftPad(Long2Str(Num_Qso[Band]), 6)+
        LeftPad(Long2Str(Num_Points[Band]), 6)+
        LeftPad(Long2Str(Num_Qth[Band]), 6)+
        LeftPad(Long2Str(Num_Country[Band]), 6));
    PrintF(CharStr('-', 38));
    PrintF('Total '+
      LeftPad(Long2Str(Num_Qso[TOT]), 6)+
      LeftPad(Long2Str(Num_Points[TOT]), 6)+
      LeftPad(Long2Str(Num_Qth[TOT]), 6)+'*'+
      LeftPad(Long2Str(Num_Country[TOT]), 5)+'*');
    PrintF(CharStr('-', 38));
    PrintF('* - Counted only once (not once per band)');
    PrintF(CharStr('-', 38));
    PrintSummaryBottom;
  end;

  procedure TARRL.RemoveOldMults(var SomeQso : log_rec);
  begin
    RemoveCountryMult(SomeQso);
    RemoveStateMult(SomeQso);
    RemoveVeProvinceMult(SomeQso);
  end;

  procedure TARRL.AddNewMults(var SomeQso : log_rec);
  begin
    with SomeQso do begin
      if CountryList^.IsUSorCanada(SomeQso) then New_Country := False
        else begin
          New_Country := CountryList^.Load(SomeQso,TOT);
          CountryList^.SetCountry(CountryList^.CtyNum,Band);
         end;
      if His_Country = 'W' then begin
        New_Qth := StateList^.Load(His_Qth, TOT);
        StateList^.SetStateByName(His_Qth,SomeQso.Band);
      end;
      if His_Country = 'VE' then begin
        New_Qth := ProvList^.Load(His_Qth, TOT);
        ProvList^.SetProvByName(His_Qth,SomeQso.Band);
      end;
    end;
  end;

  procedure TARRL.IdAnyMults(const SomeQso : log_rec);
  var
    ix : Byte;
    tc : string[5]; { temp ct.code, it is easier to type in }
    tcn : word;     { temp country num, it is easier to type}
  begin
    if RttyIni^.GetProfileBool('PutMsgs','Profile',True) then
    with SomeQso do begin
      if Dupe then Exit;
      if Pos('/MM', Call) > 0 then Exit;
      tcn := CountryList^.Identify(SomeQso);
      if tcn = NoCountry then Exit;
      tc := CountryList^.Ct.Code;
      if (not CountryList^.GetCountry(tcn,TOT)) and
         (tc <> 'W') and
         (tc <> 'VE') then RttyWin^.PutString(^Y'-DX '^Z);
      if tc = 'W' then begin
        ix := StateList^.GetStateNo(His_Qth);
        if (ix > 0) and (not StateList^.GetState(ix,TOT)) then RttyWin^.PutString(^Y'-US '^Z);
      end;
      if tc = 'VE' then begin
        ix := ProvList^.GetProvNo(His_Qth);
        if (ix > 0) and (not ProvList^.GetProv(ix,TOT)) then RttyWin^.PutString(^Y'-VE '^Z);
      end;
    end;
  end;

  function TARRL.ContestId : string;
  begin
    ContestId := 'ARRL';
  end;

  procedure TARRL.HandleAltU;
  begin
    StateList^.Display;
  end;

  procedure TARRL.DisplayScoreWin;
  var
    PointsPerQso   : Real;
    PointsPerMult  : Real;
    TotMults       : LongInt;
    Band           : BandType;
    ScoreWin       : WindowPtr;
  begin
    ScoreCountPoints;
    Score := ScoreContest;
    if Score > 0 then PointsPerQso := Score / Num_Qso[Tot]
    else PointsPerQso := 0;
    TotMults := Num_Country[TOT] + Num_Qth[TOT];
    If TotMults > 0 then PointsPerMult := Score / TotMults
    else PointsPerMult := 0;
    with Colors^ do begin
    MakeWindow(ScoreWin, MW_xhi-40, MW_Ylo+1, MW_xhi-2, MW_Ylo+11,
                  True, True, True, WindowAttr, FrameAttr, HeaderAttr, 'ARRL Score Summary') ;
    HideMouse;
    DisplayWindow(ScoreWin) ;
    FastWriteWindow('Band    QSOs   Pts   QTH    DX', 1, 1, WindowAttr);
    for Band := B80 to B10 do
      FastWriteWindow(Pad(BandS[Band], 6)+
                                 LeftPad(Long2Str(Num_Qso[Band]), 6)+
                                 LeftPad(Long2Str(Num_Points[Band]), 6)+
                                 LeftPad(Long2Str(Num_Qth[Band]), 6)+
                                 LeftPad(Long2Str(Num_Country[Band]), 6),
                                 2+Ord(Band), 1, WindowAttr);
    FastWriteWindow('----    ----  ----  ----  ----', 7, 1, WindowAttr);
    FastWriteWindow('Total '+
                               LeftPad(Long2Str(Num_Qso[TOT]), 6)+
                               LeftPad(Long2Str(Num_Points[TOT]), 6)+
                               LeftPad(Long2Str(Num_Qth[TOT]), 6)+
                               LeftPad(Long2Str(Num_Country[TOT]), 6),
                               8, 1, WindowAttr);
    FastWriteWindow('Tot: '+Pad(Long2Str(Score),8)+
                      ' P/Q:'+Real2Str(PointsPerQSO, 4, 1)+
                      ' P/M:'+Real2Str(PointsPerMult, 4, 1),
                      9, 1, WindowAttr);
    FastWrite('Press Any Key to continue', 12, 42, WindowAttr);
    end;
    ShowMouse;
    ReadKeyorButton;
    KillWindow(ScoreWin);
  end;

  function TARRL.ScoreContest : LongInt;
  begin
    ScoreContest := Num_Points[TOT]*(Num_Qth[TOT]+Num_Country[TOT]);
  end;

  procedure TARRL.ScoreCountPoints;
  var
    Band : BandType;
  begin
    SetCountersToZero;
    for Band := B80 to B10 do begin
      Num_Qth[Band]     := ProvList^.HowMany(Band)+
                           StateList^.HowMany(Band);
      Num_Country[Band] := CountryList^.HowMany(Band);
    end;
    LogFile^.ResetLog;
    LogFile^.Get_Next(a);
    while OK do begin
      Inc(Num_Points[a.Band], a.Point);
      Inc(Num_Qso[a.Band]);
      if a.Dupe then Inc(Num_Dupes[a.Band]);
      LogFile^.Get_Next(a);
    end;
    Num_Qth[TOT]     := ProvList^.HowMany(TOT)+
                        StateList^.HowMany(TOT);
    Num_Country[TOT] := CountryList^.HowMany(TOT);
    for Band := B80 to B10 do begin
      Num_Qso[TOT] := Num_Qso[TOT]+Num_Qso[Band];
      Num_Points[TOT] := Num_Points[TOT]+Num_Points[Band];
    end;
  end;

  function TARRL.FieldsPerRecord : byte;
  { returns the number of fields (across) on the screen }
  begin
    FieldsPerRecord := 10;
  end;

  function TARRL.MultsCol : byte;
  { returns the number of "mults" column on the screen }
  begin
    MultsCol := 7;
  end;

  procedure TARRL.BuildEditWindow;
  var
    I : Integer;
  begin
    for I := 1 to 5 do begin
      AddChoiceField(EntryEsr, '', I, 2, 'XX', I, 1, 0, 1,
        EditIncChoicePtr, EditW^.EditRecs[I].Band);
      AddTimeField(EntryEsr, '', I, 2, 'hhmm', I, 4, 0,
        MinTime, MaxTime, EditW^.EditRecs[I].Ts);
      AddStringField(EntryEsr, '', I, 2, '!!!!!!!!!!!!', I, 9, 12, 0,
        ValidateCallSignPtr, EditW^.EditRecs[I].Call); { they ALL have callsign first! }
      AddStringField(EntryEsr, '', I, 2, '!!!', I, 22, 3, 0,
        ValidateQTHptr, EditW^.EditRecs[I].His_Qth);
      SetForceMode(True, True);
      AddWordField(EntryEsr, '', I, 2, '9999', I, 26, 0,
        0, 9999, EditW^.EditRecs[I].His_Serno);
      SetForceMode(False, False);
      AddByteField(EntryEsr, '', I, 2, '99', I, 31, 0,
        0, 59, EditW^.EditRecs[I].His_Rst);
      SetForceMode(True, True);
      AddWordField(EntryEsr, '', I, 2, '9999', I, 34, 0,
        0, 9999, EditW^.EditRecs[I].My_Serno);
      SetForceMode(False, False);
      SetProtection(True);
      AddStringField(EntryEsr, '',  I, 2, '!!!!!',      I, 39, 5,
        0, nil, EditW^.Mults[I]);
      SetProtection(False);
      AddByteField(EntryEsr, '',    I, 2, '999',        I, 45, 0,
        0, 255, EditW^.EditRecs[I].Point);
      AddYesNoField(EntryEsr, '', I, 2,   '',           I, 49, 0,
        EditW^.EditRecs[I].Dupe);
    end;
    SetFieldLinks(EntryESR, 5,  2,  4);
    ActivateWindow(EditWin);
    FastWriteWindow('Mt Time Callsign     Qth Ser# RS Ser# Mults Pts Dupe', 1, 1, Colors^.HeaderAttr);
  end;

  function TARRL.MakeMultStr(const R : log_rec) : String;
  var
    T : String[5];
  begin
    T := '';
    if R.New_Country then T := T+'C'
    else T := T+' ';
    if R.New_Qth then T := T+'Q'
    else T := T+' ';
    MakeMultStr := T;
  end;

  procedure TARRL.LogContact(var ThisContact : log_rec);
  var
    ix : byte;
  begin
    PreLogContact(ThisContact); { parent }
    With ThisContact do begin
      My_Qth := RttyIni^.GetProfileString('Qth','Profile','');
      if CountryList^.Identify(ThisContact) <> NoCountry then
        His_Country := CountryList^.Ct.Code;
      if not AdvisorSaysOK(ThisContact) then Exit;
      Dupe := DupeTbl^.Load(ThisContact);
      if His_Qth = '' then His_Qth := DupeTbl^.GetQth;
      AddNewMults(ThisContact);
      case Dupe of
         True  : Point := 0;
         False : Point := PointValue(ThisContact);
      end;
      if Pos('/MM', Call) > 0 then begin
        New_Country := False;
        New_Qth := False;
        New_Continent := False;
      end;
    end;
    inherited LogContact(ThisContact);
  end;

  procedure TARRL.RemoveStateMult(var R : log_rec);
  var
    ix : Byte;
  begin
    With R do begin
      if (not New_Qth) or (His_Country <> 'W') then Exit;
      New_Qth := False;
      ix := StateList^.GetStateNo(His_Qth);
      if ix = 0 then Exit;
      StateList^.ResetState(ix,TOT);
      StateList^.ResetState(ix,R.Band);
    end;
  end;

  procedure TARRL.RemoveVeProvinceMult(var R : log_rec);
  var
    ix : Byte;
  begin
    with R do begin
      if (not New_Qth) or (His_Country <> 'VE') then Exit;
      New_Qth := False;
      ix := ProvList^.GetProvNo(His_Qth);
      if ix = 0 then Exit;
      ProvList^.ResetProv(ix,TOT);
      ProvList^.ResetProv(ix,Band);
    end;
  end;

  function TARRL.AdvisorSaysOK(const R : log_rec) : Boolean;
  { t = ok to log this qso, false = dont log, they want to edit }
  var
    Missing : String[40];
  begin
    if NOT inherited AdvisorSaysOK(R) then begin { hard edits, no bypass allowed }
      AdvisorSaysOK := False;
      Exit; { hard edits, no bypass allowed }
    end;
    if NOT RttyIni^.GetProfileBool('Advice','PROFILE',True) then begin                     {dont want advisor window }
      AdvisorSaysOK := True;
      Exit;
    end;
    Missing := '';
    if CountryList^.IsUSorCanada(R) and (R.His_Qth = '') then begin
      OK := False;
      if Missing <> '' then Missing := Missing+',';
      Missing := Missing + 'QTH';
    end;
    if (NOT CountryList^.IsUSorCanada(R)) and (R.His_Serno = 0) then begin
      if Missing <> '' then Missing := Missing+',';
      Missing := Missing+'Serial #';
    end;
    AdvisorSaysOK := AdvisorWindow(Missing)
  end;

  procedure TARRL.Writelog;
  begin
    inherited Writelog;
    StateList^.PrintStatesAndVE;
  end;

  procedure TARRL.FillRec(const Call : String; var R : log_rec);
    { returns info to R if we has QSO before with CALL         }
    { I also try to fill as much of the QSO record as possible }
  var
    Slot : Word;
    F3 : String[3];
  begin
    R.Call := Trim(Call);
    if DupeTbl^.GetDupeByCall(Call, Slot) then
      R.His_Qth := DupeTbl^.GetQth;
    if CountryList^.Identify(R) = NoCountry then Exit;
    R.His_Country := CountryList^.Ct.Code;
    if R.His_Country = 'VE' then R.His_Qth := ProvList^.GetProvByCall(R.Call);
  end;



end.
