{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit c_bart;

interface

uses
  Diskio,
  c_contest,
  Rectypes;

type

  PBART = ^TBART;
  TBART = object(TContest)
            constructor init;
            procedure Header; virtual;
            procedure Footer; virtual;
            procedure Detail; virtual;
            function LogLine(const R : Log_Rec) : String; virtual;
            destructor done; virtual;
            procedure PrintSummary; virtual;
            procedure RemoveOldMults(var SomeQso : log_rec); virtual;
            procedure AddNewMults(var SomeQso : log_rec); virtual;
            procedure IdAnyMults(const SomeQso : log_rec); virtual;
            function ContestId : string; virtual;
            procedure HandleAltU; virtual;
            procedure DisplayScoreWin; virtual;
            function  ScoreContest : LongInt; virtual;
            procedure ScoreCountPoints; virtual;
            function  FieldsPerRecord : byte; virtual;
            function MultsCol : byte; virtual;
            procedure BuildEditWindow; virtual;
            function  MakeMultStr(const R : log_rec) : String; virtual;
            procedure LogContact(var ThisContact : log_rec); virtual;
            function  AdvisorSaysOK(const R : log_rec) : Boolean; virtual;
            procedure Writelog; virtual;
            procedure FillRec(const Call : String; var R : log_rec); virtual;
          end;

implementation

uses
  TpEntry,
  cEditWin,
  cClock,
  cContinent,
  cCountry,
  cDupeTbl,
  cDist_W,
  cDist_VE,
  cDist_VK,
  cDist_JA,
  TpDate,
  TpString,
  TpWindow,
  IniFile,
  TpMouse,
  cRttyWin,
  TpCrt,
  TAHigh,
  TAccess,
  LowLvl,
  cColors,
  cLogBart,
  DebugRtns,
  Routines;
  { TBART.........................................................TBART}
  { TBART.........................................................TBART}
  { TBART.........................................................TBART}
  { TBART.........................................................TBART}
  { TBART.........................................................TBART}

  constructor TBART.init;
  begin
    inherited init;
    LogFile := New(PBARTLogFile, Init);
    MakeContList;
    MakeWDistList;
    MakeVEDistList;
    MakeVKDistList;
    MakeJADistList;
    MakeCountryList('ARRL.CTY','ARRL.PFX');
    {$IFDEF DEBUG}
    AppendDebugMessage('BART created');
    {$ENDIF}
  end;

  destructor TBART.done;
  begin
    inherited done;
    KillWDistList;
    KillVEDistList;
    KillVKDistList;
    KillJADistList;
    KillCountryList;
    {$IFDEF DEBUG}
    AppendDebugMessage('BART destroyed');
    {$ENDIF}
  end;

  function TBART.LogLine(const R : Log_Rec) : String;
  var
    t : String[127];
  begin
    t := BandS[R.Band]+' '+
      Clock^.Date2DateString(R.Date)+
      TimeToTimeString(' hhmm ', R.Ts)+
      Pad(R.Call, 12)+' '+
      Long2Str(R.My_Rst)+'9 '+
      LeftPadCh(Long2Str(R.My_Serno), '0', 4)+' '+
      TimeToTimeString('hhmm ', R.My_Time)+
      Long2Str(R.His_RST)+'9 '+
      LeftPadCh(Long2Str(R.His_Serno), '0', 4)+' '+
      TimeToTimeString('hhmm ', R.His_Time)+
      Pad(R.His_Country, 5)+' ';
    if R.New_Continent then t := t+' N  '
    else t := t+'    ';
    if R.New_Country then t := t+' C '
    else t := t+'   ';
    if R.New_Qth then t := t+'  D  '
    else t := t+'     ';
    t := t+Long2Str(R.Point);
    if R.Dupe then t := t+'-DUPE';
    LogLine := t;
  end;

  procedure TBART.Header;
  begin
    inherited Header;
    PrintF('Band Date Time Call         RST Ser# Time RST Ser# Time Cntry Cnt Cty Dst Pt');
  end;

  procedure TBART.Footer;
  begin
    PrintF(CharStr(' ', 62)+'--- --- --- ---');
    PrintF('Page '+
      Pad(Long2Str(PageCT), 2)+
      CharStr(' ', 55)+
      Pad(Long2Str(sub_Cont), 4)+
      Pad(Long2Str(Sub_Cty), 4)+
      Pad(Long2Str(sub_Dist), 4)+
      Long2Str(sub_pts));
  end;

  procedure TBART.Detail;
  begin
    if Linect = 0 then Header;
    if a.New_Continent then Inc(sub_Cont);
    if a.New_Country then Inc(Sub_Cty);
    if a.New_Qth then Inc(sub_Dist);
    sub_pts := sub_pts+a.Point;
    PrintF(LogLine(a));
  end;

  procedure TBART.PrintSummary;
  var
    Band : BandType;
  begin
    inherited PrintSummary;
    PrintSummaryTop;
    PrintF('Band    QSOs   Pts  Dist    DX  Cont');
    for Band := B80 to B10 do
      PrintF(Pad(BandS[Band], 6)+
        LeftPad(Long2Str(Num_Qso[Band]), 6)+
        LeftPad(Long2Str(Num_Points[Band]), 6)+
        LeftPad(Long2Str(Num_Districts[Band]), 6)+
        LeftPad(Long2Str(Num_Country[Band]), 6)+
        LeftPad(Long2Str(Num_Continents[Band]), 6));
    PrintF('-----   ----  ----  ----  ----  ----');
    PrintF('Total '+
      LeftPad(Long2Str(Num_Qso[TOT]), 6)+
      LeftPad(Long2Str(Num_Points[TOT]), 6)+
      LeftPad(Long2Str(Num_Districts[TOT]), 6)+
      LeftPad(Long2Str(Num_Country[TOT]), 6)+
      LeftPad(Long2Str(Num_Continents[TOT]), 6));
    PrintSummaryBottom;
  end;

  procedure TBART.RemoveOldMults(var SomeQso : log_rec);
  begin
    RemoveCountryMult(SomeQso);
    RemoveDistMult(SomeQso);
    RemoveContinentMult(SomeQso);
  end;

  procedure TBART.AddNewMults(var SomeQso : log_rec);
  begin
    with SomeQSo do begin
      New_Country := CountryList^.Load(SomeQso,Band);
      if His_Country = 'W' then
        New_Qth := WDistList^.Load(Call,Band);
      if His_Country = 'VE' then
        New_Qth := VEDistList^.Load(Call,Band);
      if His_Country = 'VK' then
        New_Qth := VKDistList^.Load(Call,Band);
      if His_Country = 'JA' then
        New_Qth := JADistList^.Load(Call,Band);
      if CountryList^.CtyNum <> NoCountry then begin
        New_Continent := ContList^.Load(CountryList^.Ct.Continent,Band);
      end;
    end;
  end;

  procedure TBART.IdAnyMults(const SomeQso : log_rec);
  var
    ix : Byte;
    Dist : Byte;
    tc : string[5]; { temp ct.code, it is easier to type in }
    tcn : word;     { temp country num, it is easier to type}
  begin
    if RttyIni^.GetProfileBool('PutMsgs','Profile',True) then
      with SomeQso do begin
        if Dupe then Exit;
        if Pos('/MM', Call) > 0 then Exit;
        tcn := CountryList^.Identify(SomeQso);
        if tcn = NoCountry then Exit;
        tc := CountryList^.Ct.Code;
        if (not CountryList^.GetCountry(tcn,Band)) and
           (tc <> 'W') and (tc <> 'VE') then RttyWin^.PutString(^Y'-DX '^Z);
        Dist := WDistList^.GetDistNo(Call);
        if Dist in [0..9] then begin
          if ((tc = 'W')  and (not WDistList^.GetDist(Dist,Band))) then RttyWin^.PutString(^Y' -DIST '^Z);
          if ((tc = 'VE') and (not VEDistList^.GetDist(Dist,Band))) then RttyWin^.PutString(^Y' -DIST '^Z);
          if ((tc = 'JA') and (not JADistList^.GetDist(Dist,Band))) then RttyWin^.PutString(^Y'-DIST '^Z);
          if ((tc = 'VK') and (not VKDistList^.GetDist(Dist,Band))) then RttyWin^.PutString(^Y' -DIST '^Z);
        end;
      end;
  end;

  function TBART.ContestId : string;
  begin
    ContestId := 'BART';
  end;

  procedure TBART.HandleAltU;
  begin
    WDistList^.Display;
  end;

  procedure TBART.DisplayScoreWin;
  var
    PointsPerQso   : Real;
    PointsPerMult  : Real;
    TotMults       : LongInt;
    Band           : BandType;
    ScoreWin       : WindowPtr;
  begin
    ScoreCountPoints;
    Score := ScoreContest;
    if Score > 0 then PointsPerQso := Score / Num_Qso[Tot]
    else PointsPerQso := 0;
    TotMults := Num_Country[TOT]+Num_Districts[TOT]+Num_Continents[TOT];
    If TotMults > 0 then PointsPerMult := Score / TotMults
    else PointsPerMult := 0;
    with Colors^ do begin
    MakeWindow(ScoreWin, MW_xhi-40, MW_Ylo+1, MW_xhi-2, MW_Ylo+11,
                  True, True, True, WindowAttr, FrameAttr, HeaderAttr, 'BARTG Score Summary') ;
    HideMouse;
    DisplayWindow(ScoreWin) ;
    FastWriteWindow('Band    QSOs   Pts  Dist    DX  Cont', 1, 1, WindowAttr);
    for Band := B80 to B10 do
      FastWriteWindow(Pad(BandS[Band], 6)+
                                 LeftPad(Long2Str(Num_Qso[Band]), 6)+
                                 LeftPad(Long2Str(Num_Points[Band]), 6)+
                                 LeftPad(Long2Str(Num_Districts[Band]), 6)+
                                 LeftPad(Long2Str(Num_Country[Band]), 6)+
                                 LeftPad(Long2Str(Num_Continents[Band]), 6),
                                 2+Ord(Band), 1, WindowAttr);
    FastWriteWindow('-----   ----  ----  ----  ----  ----', 7, 1, WindowAttr);
    FastWriteWindow('Total '+
                               LeftPad(Long2Str(Num_Qso[TOT]), 6)+
                               LeftPad(Long2Str(Num_Points[TOT]), 6)+
                               LeftPad(Long2Str(Num_Districts[TOT]), 6)+
                               LeftPad(Long2Str(Num_Country[TOT]), 6)+
                               LeftPad(Long2Str(Num_Continents[TOT]), 6),
                               8, 1, WindowAttr);
    FastWriteWindow('Tot: '+Pad(Long2Str(Score),8)+
                      ' P/Q:'+Real2Str(PointsPerQSO, 4, 1)+
                      ' P/M:'+Real2Str(PointsPerMult, 4, 1),
                      9, 1, WindowAttr);
    FastWrite('Press Any Key to continue', 12, 42, WindowAttr);
    end;
    ShowMouse;
    ReadKeyorButton;
    KillWindow(ScoreWin);
  end;

  function TBART.ScoreContest : LongInt;
  begin
    ScoreContest := Num_Points[TOT]*(Num_Districts[TOT]+Num_Country[TOT])*Num_Continents[TOT];
  end;

  procedure TBART.ScoreCountPoints;
  var
    Band : BandType;
  begin
    SetCountersToZero;
    for Band := B80 to B10 do begin
      Num_Country[Band]   := CountryList^.HowMany(Band);
      Num_Continents[Band]:= ContList^.HowMany(Band);
      Num_Districts[Band] := WDistList^.HowMany(Band)+
                             VEDistList^.HowMany(Band)+
                             VKDistList^.HowMany(Band)+
                             JADistList^.HowMany(Band);
    end;
    LogFile^.ResetLog;
    LogFile^.Get_Next(a);
    while OK do begin
      Inc(Num_Points[a.Band], a.Point);
      Inc(Num_Qso[a.Band]);
      if a.Dupe then Inc(Num_Dupes[a.Band]);
      LogFile^.Get_Next(a);
    end;
    SumCounters;
    Num_Continents[TOT] := ContList^.HowMany(TOT); { special logic }
    if Num_Continents[TOT] > 6 then Num_Continents[TOT] := 6;
  end;

  function TBART.FieldsPerRecord : byte;
  { returns the number of fields (across) on the screen }
  begin
    FieldsPerRecord := 11;
  end;


  function TBART.MultsCol : byte;
  { returns the number of "mults" column on the screen }
  begin
    MultsCol := 8;
  end;

  procedure TBART.BuildEditWindow;
  var
    I : Integer;
  begin
    for I := 1 to 5 do begin
      AddChoiceField(EntryEsr, '', I, 2, 'XX', I, 1, 0, 1,
        EditIncChoicePtr, EditW^.EditRecs[I].Band);
      AddTimeField(EntryEsr, '', I, 2, 'hhmm', I, 4, 0,
        MinTime, MaxTime, EditW^.EditRecs[I].Ts);
      AddStringField(EntryEsr, '', I, 2, '!!!!!!!!!!!!', I, 9, 12, 0,
        ValidateCallSignPtr, EditW^.EditRecs[I].Call); { they ALL have callsign first! }
      SetForceMode(True, True);
      AddWordField(EntryEsr, '', I, 2, '9999',          I, 22, 0,
        0, 9999, EditW^.EditRecs[I].His_Serno);
      SetForceMode(False, False);
      AddTimeField(EntryEsr, '', I, 2, 'hhmm',          I, 27, 0,
        MinTime, MaxTime, EditW^.EditRecs[I].His_Time);
      AddByteField(EntryEsr, '', I, 2, '99',            I, 32, 0,
        0, 59, EditW^.EditRecs[I].His_Rst);
      SetForceMode(True, True);
      AddWordField(EntryEsr, '', I, 2, '9999',          I, 35, 0,
        0, 9999, EditW^.EditRecs[I].My_Serno);
      SetForceMode(False, False);
      AddTimeField(EntryEsr, '', I, 2, 'hhmm',          I, 40, 0,
        MinTime, MaxTime, EditW^.EditRecs[I].My_Time);
      SetProtection(True);
      AddStringField(EntryEsr, '',  I, 2, '!!!!!',      I, 45, 5,
        0, nil, EditW^.Mults[I]);
      SetProtection(False);
      AddByteField(EntryEsr, '',    I, 2, '999',        I, 51, 0,
        0, 255, EditW^.EditRecs[I].Point);
      AddYesNoField(EntryEsr, '', I, 2,   '',           I, 55, 0,
        EditW^.EditRecs[I].Dupe);
    end;
    SetFieldLinks(EntryESR, 5, 2,  4);
    ActivateWindow(EditWin);
    FastWriteWindow('Mt Time Callsign     Ser# Time RS Ser# Time Mults Pts Dupe', 1, 1, Colors^.HeaderAttr);
  end;

  function TBART.MakeMultStr(const R : log_rec) : String;
  var
    T : String[5];
  begin
    T := '';
    if R.New_Country then T := T+'C'
    else T := T+' ';
    if R.New_Qth then T := T+'D'
    else T := T+' ';
    if R.New_Continent then T := T+'N'
    else T := T+' ';
    MakeMultStr := T;
  end;


  procedure TBART.LogContact(var ThisContact : log_rec);
  begin
    PreLogContact(ThisContact);
    with ThisContact do begin
      if CountryList^.Identify(ThisContact) <> NoCountry then
        His_Country := CountryList^.Ct.Code;
      if not AdvisorSaysOK(ThisContact) then Exit;
      Dupe := DupeTbl^.Load(ThisContact);
      AddNewMults(ThisContact);
      case Dupe of
        True  : Point := 0;
        False : Point := PointValue(ThisContact);
      end;
      if Pos('/MM', Call) > 0 then begin
        New_Country := False;
        New_Qth := False;
        New_Continent := False;
      end;
    end;
    inherited LogContact(ThisContact);
  end;

  function TBART.AdvisorSaysOK(const R : log_rec) : Boolean;
  { t = ok to log this qso, false = dont log, they want to edit }
  var
    Missing : String[40];
  begin
    if NOT inherited AdvisorSaysOK(R) then begin { hard edits, no bypass allowed }
      AdvisorSaysOK := False;
      Exit; { hard edits, no bypass allowed }
    end;
    if NOT RttyIni^.GetProfileBool('Advice','PROFILE',True) then begin                     {dont want advisor window }
      AdvisorSaysOK := True;
      Exit;
    end;
    Missing := '';
    if R.His_Serno = 0 then begin
      if Missing <> '' then Missing := Missing+',';
      Missing := Missing + 'Serial #';
    end;
    if R.His_Time = 0 then begin
      if Missing <> '' then Missing := Missing+',';
      Missing := Missing + 'Time';
    end;
    AdvisorSaysOK := AdvisorWindow(Missing)
  end;

  procedure TBART.Writelog;
  begin
    inherited Writelog;
    ContList^.Print;
    WDistList^.Print;
  end;

  procedure TBART.FillRec(const Call : String; var R : log_rec);
    { returns info to R if we has QSO before with CALL         }
    { I also try to fill as much of the QSO record as possible }
  begin
    R.Call := Trim(Call);
    if CountryList^.Identify(R) <> NoCountry then
      R.His_Country := CountryList^.Ct.Code;
  end;





end.
