{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit c_contest;

interface

uses
  DiskIo,
  Rectypes;

type
  PContest = ^TContest;
  TContest = object
               PrintFile : Text;
               Linect,
               PageCT,
               Sub_QSO,
               Sub_Qth,
               Sub_Cty,
               sub_Cont,
               sub_Dist,
               sub_pts,
               sub_wpx,
               sub_zon,
               qtcs_s,
               qtcs_r,
               sub_MULTS : Integer;
               a : Log_Rec;       { used to process files }
               Loading : Boolean; {true during load, false for normal ops}
               Score : LongInt;
               {scoring variables}
               Num_Country    : array[BandType] of LongInt;
               Num_Qth        : array[BandType] of LongInt;
               Num_Qtcs       : array[BandType] of LongInt;
               Num_Zone       : array[BandType] of LongInt;
               Num_Qso        : array[BandType] of LongInt;
               Num_Points     : array[BandType] of LongInt;
               Num_Districts  : array[BandType] of LongInt;
               Num_Continents : array[BandType] of LongInt;
               Num_Dupes      : array[BandType] of LongInt;
               VK_Bonus       : LongInt;
               F : array[1..21] of String[100];
             public
               constructor init;
               destructor done; virtual;
               procedure PrintLogs; virtual;
               procedure PrintSummary; virtual;
               function  LogLine(const R : Log_Rec) : String; virtual;
               function  QsoIsGood(const ThisQso : log_rec) : Boolean; virtual;
               procedure ClearSubTotals;
               procedure PrintF(Text : String);
               procedure Header; virtual;
               procedure Footer; virtual;
               procedure Detail; virtual;
               procedure EachBand;
               procedure AllBands;
               procedure PrintDetail;
               procedure PrintSummaryTop;
               procedure PrintSummaryBottom;
               procedure PrintLogLine(const SomeQso : log_rec);
               function  PointValue(const SomeQso : log_rec) : byte; virtual;
               procedure RemoveOldMults(var SomeQso : log_rec); virtual;
               procedure AddNewMults(var SomeQso : log_rec); virtual;
               procedure IdAnyMults(const SomeQso : log_rec); virtual;
               function  ContestID : string; virtual;
               procedure HandleAltU; virtual;
               procedure DisplayScoreWin; virtual;
               function  ScoreContest : LongInt; virtual;
               procedure ScoreCountPoints; virtual;
               procedure SetCountersToZero; virtual;
               procedure SumCounters; virtual;
               function  FieldsPerRecord : byte; virtual;
               function  MultsCol : byte; virtual;
               procedure EditRQTc; virtual;
               procedure EditSQTc; virtual;
               procedure BuildEditWindow; virtual;
               function  MakeMultStr(const R : log_rec) : String; virtual;
               procedure PreLogContact(var ThisContact : log_rec);
               procedure LogContact(var ThisContact : log_rec); virtual;
               procedure RemoveCountryMult(var R : log_rec); virtual;
               procedure RemoveStateMult(var R : log_rec); virtual;
               procedure RemoveVeProvinceMult(var R : log_rec); virtual;
               procedure RemoveZoneMult(var R : log_rec); virtual;
               procedure RemoveContinentMult(var R : log_rec);
               procedure RemoveDistMult(var R : log_rec);
               procedure RemoveSectionMult(var R : log_rec);
               procedure RemoveYearMult(var R : log_rec);
               function  AdvisorSaysOK(Const R : log_rec) : boolean; virtual;
               function  AdvisorWindow(const Missing : String) : Boolean;
               procedure AdvisorWindowNoCall;
               procedure AdvisorWindowSameCountry;
               procedure Writelog; virtual;
               procedure FillRec(const Call : String; var R : log_rec); virtual;
               procedure ReadDefaultFKeys;
               procedure ProgramFKeys;
               procedure WriteFKeys;
               procedure ReadFKeys;
               procedure ReScore;
             end;


  procedure MakeContest;
  procedure KillContest;

var
  ThisContest : PContest;

implementation

uses
  Dos,
  Printer,
  ApPort,
  Keys,
  c_cqww,
  c_arrl,
  c_sart,
  c_hssc,
  c_bart,
  c_bspr,
  c_dxpd,
  c_jart,
  c_idra,
  c_volt,
  c_waed,
  c_naqp,
  c_anar,
  c_eaww,
  c_spdx,
  c_arii,
  c_rrww,
  c_spri,
  c_tara,
  c_arfd,
  c_sccc,
  cDist_w,
  cDist_ve,
  cDist_vk,
  cDist_ja,
  cDist_zl,
  cYearTbl,
  cSerNum,
  cDupeTbl,
  cAutoCq,
  cCountry,
  cState,
  cSection,
  cScoreWin,
  cProvince,
  cZone,
  cContinent,
  cCallQue,
  ClusterMon,
  cEditWin,
  cMaster,
  cClock,
  c_Radio,
  cMsgWin,
  cTopWin,
  cBotWin,
  Network,
  Bearings,
  IniTools,
  TpEntry,
  TpDate,
  TpCrt,
  TpMouse,
  TpDos,
  TpString,
  TpSort,
  TAHigh,
  TAccess,
  TpRArray,
  TpWindow,
  LowLvl,
  OneTimers,
  DebugRtns,
  IniFile,
  cRateWin,
  cColors,
  cRttyWin,
  Routines;


  { TContest.........................................................TContest}
  { TContest.........................................................TContest}
  { TContest.........................................................TContest}
  { TContest.........................................................TContest}
  { TContest.........................................................TContest}

  constructor TContest.init;
  begin
    Bearing := NIL;
    if (ExistFile('BEARINGS.DAT')) AND (RttyIni^.GetProfileBool('UseBearingsFile','PERFORMANCE',True))
      then begin
        Bearing := New(PBearing, init);
      end;
    FriendList := NIL;
    if (ExistFile('FRIEND.INI')) AND (RttyIni^.GetProfileBool('UseFriendFile','PERFORMANCE',True))
     then begin
      FriendList := New(PIni, Init('FRIEND.INI', True));
      if not assigned(FriendList) then begin
        RttyWin^.PutString(^M+'Unable to initialize FRIEND file. Is it too large?');
        AppendDebugMessage('Couldn''t create FRIEND.INI object.');
      end;
    end;
    MakeDupeTbl;
    autocq := NIL;
    autocq := New(PAutoCq, Init);
    RateWin := NIL;
    RateWin := New(PRateWin, Init);
    ScoreWin := NIL;
    ScoreWin := New(PScoreWin, Init);
    TopWin := NIL;
    TopWin := New(PTopWin, Init);
    BotWin := NIL;
    BotWin := New(PBotWin, Init);
    MsgWin := NIL;
    MsgWin  := New(PMsgWin, Init);
    MastList := NIL;
    Mastlist := New(PMastList, Init('MASTER.CAL'));
    if (AutoCq = NIL) or
       (RateWin = NIL) or
       (ScoreWin = NIL) or
       (TopWin = NIL) or
       (BotWin = NIL) or
       (MsgWin = NIL) or
       (Mastlist = NIL) then NotEnoughMemory(True);
    RttyWin^.PutString(^M+'Loading country and prefix files.');
    if RttyIni^.GetProfileBool('AutoReindex','PROFILE',True) then
      RebuildIndex;
    MakeCallqueue;
    {$IFDEF DEBUG}
    AppendDebugMessage('Contest created');
    {$ENDIF}
  end;

  destructor TContest.done;
  begin
    if Assigned(Bearing) then Dispose(Bearing, Done);
    Bearing := NIL;
    if Assigned(FriendList) then Dispose(FriendList, Done);
    FriendList := NIL;
    KillDupeTbl;
    Dispose(AutoCq, Done);
    AutoCq := NIL;
    Dispose(RateWin, Done);
    RateWin := NIL;
    Dispose(ScoreWin, Done);
    ScoreWin := NIL;
    Dispose(TopWin, Done);
    TopWin := NIL;
    Dispose(BotWin, Done);
    BotWin := NIL;
    Dispose(MsgWin, Done);
    MsgWin := NIL;
    Dispose(MastList, Done);
    MastList := NIL;
    KillCallQueue;
    Dispose(LogFile, Done);
    LogFile := NIL;
    {$IFDEF DEBUG}
    AppendDebugMessage('Contest destroyed');
    {$ENDIF}
  end;

  procedure TContest.EachBand;
  var
    Band : BandType;
  begin
    for Band := B80 to B10 do begin
      Linect := 0;
      PageCT := 0;
      LogFile^.Index := '';
      LogFile^.ResetLog;
      Assign(PrintFile, FileName(BandS[Band]));
      Rewrite(PrintFile);
      MsgWin^.Display('Preparing: '+FileName(BandS[Band]));
      LogFile^.Get_Next(a);
      while OK do
        begin
          if Band = a.Band then PrintDetail;
          LogFile^.Get_Next(a);
        end;
      if Linect > 0 then begin
        Footer;
        PrintF(^L);
      end;
      Close(PrintFile);
    end;
  end;

  procedure TContest.AllBands;
  begin
    Linect := 0;
    PageCT := 0;
    LogFile^.Index := '';
    LogFile^.ResetLog;
    Assign(PrintFile, FileName('ALL'));
    Rewrite(PrintFile);
    MsgWin^.Display('Preparing: '+FileName('ALL'));
    LogFile^.Get_Next(a);
    while OK do begin
      PrintDetail;
      LogFile^.Get_Next(a);
    end;
    if Linect > 0 then begin
      Footer;
      PrintF(^L);
    end;
    Close(PrintFile);
  end;

  procedure TContest.Detail;
  begin
    if Linect = 0 then Header;
    if a.New_Country then Inc(sub_MULTS);
    if a.New_Qth then Inc(sub_MULTS);
    sub_pts := sub_pts+a.Point;
    if not a.Dupe then Inc(Sub_QSO);
    PrintF(LogLine(a));
  end;


  procedure TContest.PrintF(Text : String);
  begin
    {$I-}
    if Text = ^L then Write(PrintFile, Text)
    else WriteLn(PrintFile, Text);
    {$I+}
  end;

  procedure TContest.ClearSubTotals;
  begin
    Sub_QSO := 0;
    Sub_Qth := 0;
    Sub_Cty := 0;
    sub_Cont := 0;
    sub_Dist := 0;
    sub_pts := 0;
    sub_wpx := 0;
    sub_zon := 0;
    qtcs_s := 0;
    qtcs_r := 0;
    sub_MULTS := 0;
  end;

  procedure TContest.PrintLogs;
  begin
    EachBand;
    AllBands;
  end;

  procedure TContest.Header;
  begin
    Inc(PageCT);
    ClearSubTotals;
    PrintF(' Call: '+
      RttyIni^.GetProfileString('Call','Profile','')+
      ' Class: '+RttyIni^.GetProfileString('Class', 'Profile', 'Single Op, All Band')+
      ' '+ ContestName[Contest]);
    PrintF('');
  end;

  procedure TContest.Footer;
  begin
  end;

  function TContest.LogLine(const R : Log_Rec) : String;
  begin
    LogLine := 'Error';
  end;

  procedure TContest.PrintDetail;
  begin
    Detail;
    Inc(Linect);
    if Linect in [10, 20, 30, 40] then PrintF('');
    if Linect = 50 then
      begin
        Footer;
        PrintF(^L);
        Linect := 0;
      end;
  end;

  procedure TContest.PrintSummaryTop;
  begin
    Assign(PrintFile, FileName('SUM')); {v2.5a}
    {$I-} Rewrite(PrintFile); {$I+}
    PrintF(ContestName[Contest]);
    PrintF('');
    PrintF('');
    PrintF('Call used: '+RttyIni^.GetProfileString('Call','Profile',''));
    if Contest = JART then PrintF('Age Sent:  '+RttyIni^.GetProfileString('Age','Profile',''))
    else PrintF('Location:  '+RttyIni^.GetProfileString('Qth','Profile',''));
    PrintF('');
    PrintF('Entry Class: '+RttyIni^.GetProfileString('Class','Profile','Single Op'));
    PrintF('');
  end;

  procedure TContest.PrintSummaryBottom;
  var
    I : Integer;
  begin
    PrintF('');
    PrintF(CharStr('-', 38));
    PrintF('');
    if (Contest <> DXPD) and (Contest <> ARFD)
     then PrintF('Claimed Score: '+Long2Str(ScoreContest));
    PrintF('');
    PrintF('Software: '+RTTYbyWF1B+' '+Version);
    PrintF('');
    PrintF('Power Output: '+RttyIni^.GetProfileString('Power', 'Profile', '0'));
    PrintF('');
    PrintF('Club Participation: '+RttyIni^.GetProfileString('Club', 'Profile', ''));
    PrintF('');
    PrintF('"I have observed all competition rules as well as all regulations');
    PrintF('established for amateur radio in my country. My report is correct');
    PrintF('and true to the best of my knowledge. I agree to be bound by the');
    PrintF('decisions of the Awards Committee."');
    PrintF('');
    PrintF('Date ___/___/___  Signature '+CharStr('_', 28)+'Call '
      +CharStr('_', 10));
    for I := 1 to 9 do PrintF('');
    PrintF(RttyIni^.GetProfileString('Name', 'Profile', ''));
    PrintF('');
    PrintF(RttyIni^.GetProfileString('Street', 'Profile', ''));
    PrintF('');
    PrintF(Trim(RttyIni^.GetProfileString('City', 'Profile', ''))+' '+
           Trim(RttyIni^.GetProfileString('State', 'Profile', ''))+' '+
           Trim(RttyIni^.GetProfileString('ZipCode', 'Profile', '')));
    Close(PrintFile);
  end;

  procedure TContest.PrintSummary;
  begin
    MsgWin^.Display('Preparing: '+FileName('SUM'));
    ScoreCountPoints;
  end;

  function  TContest.QsoIsGood(const ThisQso : log_rec) : Boolean;
  begin
    QsoIsGood := NOT DupeTbl^.IsDupe(ThisQso.Call, ThisQso.Band);
  end;

  procedure TContest.PrintLogLine(const SomeQso : log_rec);
  var
    Regs : Registers;
    function CheckPrinter : Boolean;
      function PrinterReady(const Number : Byte) : Boolean;
      begin
        Regs.AH := $02;
        Regs.DX := Number;
        Intr($17, Regs);
        PrinterReady := (Regs.AH and $80 = $80) and {test if ready}
                        (Regs.AH and $10 = $10) and {test if selected}
                        (Regs.AH and $08 = $00); {test if i/o error}
      end;

      function PrinterOutOfPaper(const Number : Byte) : Boolean;
      begin
        Regs.AH := $02;
        Regs.DX := Number;
        Intr($17, Regs);
        PrinterOutOfPaper := Regs.AH and $20 = $20;
      end;

    begin
      if (not PrinterReady(0)) or (PrinterOutOfPaper(0)) then MsgWin^.AskString('Printer Error! Press <enter> ','');
      if (not PrinterReady(0)) or (PrinterOutOfPaper(0)) then CheckPrinter := False
        else CheckPrinter := True;
    end;

  begin
    if not CheckPrinter then
      MsgWin^.Display('Printer error, unable to print QSO')
    else
      WriteLn(Lst,LogLine(SomeQso));
  end;


  function  TContest.PointValue(const SomeQso : log_rec) : byte;
  begin
    PointValue := 1;
  end;

  procedure TContest.RemoveOldMults(var SomeQso : log_rec);
  begin
    {$IFDEF DEBUG}
    AppendDebugMessage('TContest.RemoveOldMults invoked, should not have been');
    {$ENDIF}
  end;

  procedure TContest.AddNewMults(var SomeQso : log_rec);
  begin
    {$IFDEF DEBUG}
    AppendDebugMessage('TContest.AddNewMults invoked, should not have been');
    {$ENDIF}
  end;

  procedure TContest.IdAnyMults(const SomeQso : log_rec);
  begin
    {$IFDEF DEBUG}
    AppendDebugMessage('TContest.IsAnyMults invoked, should not have been');
    {$ENDIF}
  end;

  function TContest.ContestId : string;
  begin
    {$IFDEF DEBUG}
    AppendDebugMessage('TContest.ContestId invoked, should not have been');
    {$ENDIF}
  end;

procedure TContest.HandleAltU;
begin
    {$IFDEF DEBUG}
    AppendDebugMessage('TContest.HandleAltU invoked, should not have been');
    {$ENDIF}
end;

procedure TContest.DisplayScoreWin;
begin
    {$IFDEF DEBUG}
    AppendDebugMessage('TContest.DisplayScoreWin invoked, should not have been');
    {$ENDIF}
end;

  function TContest.ScoreContest : LongInt;
  begin
    ScoreContest := 0;
  end;

  procedure TContest.ScoreCountPoints;
  begin
    {$IFDEF DEBUG}
    AppendDebugMessage('TContest.ScoreCountPoints invoked, should not have been');
    {$ENDIF}
  end;

  procedure TContest.SetCountersToZero;
  var
    Band : BandType;
  begin
    for Band := B80 to TOT do begin
      Num_Country[Band] := 0;
      Num_Qth[Band] := 0;
      Num_Qtcs[Band] := 0;
      Num_Zone[Band] := 0;
      Num_Qso[Band] := 0;
      Num_Points[Band] := 0;
      Num_Districts[Band] := 0;
      Num_Continents[Band] := 0;
      Num_Dupes[Band] := 0;
    end;
    VK_Bonus := 0;
  end;

  procedure TContest.SumCounters;
  var
    Band : BandType;
  begin
    for Band := B80 to B10 do begin
      Inc(Num_Country[TOT],Num_Country[Band]);
      Inc(Num_Qth[TOT],Num_Qth[Band]);
      Inc(Num_Qtcs[TOT],Num_Qtcs[Band]);
      Inc(Num_Zone[TOT],Num_Zone[Band]);
      Inc(Num_Qso[TOT],Num_Qso[Band]);
      Inc(Num_Points[TOT],Num_Points[Band]);
      Inc(Num_Districts[TOT],Num_Districts[Band]);
      Inc(Num_Continents[TOT],Num_Continents[Band]);
      Inc(Num_Dupes[TOT],Num_Dupes[Band]);
    end;
  end;

  function TContest.FieldsPerRecord : byte;
  { returns the number of fields (across) on the screen }
  begin
    {$ifdef debug}
    AppendDebugMessage('TContest.FieldsPerRecord invoked, should not have been');
    {$ENDIF}
  end;

  function TContest.MultsCol : byte;
  { returns the number of "mults" column on the screen }
  begin
    MultsCol := 0;
  end;

  procedure TContest.EditRQTc;
  begin
    { only waed overrides this procedure }
  end;

  procedure TContest.EditSQTc;
  begin
    { only waed overrides this procedure }
  end;

  procedure TContest.BuildEditWindow;
  begin
    {$ifdef debug}
    AppendDebugMessage('TContest.BuildEditWindow invoked, should not have been');
    {$ENDIF}
  end;

  function  TContest.MakeMultStr(const R : log_rec) : String;
  begin
    {$ifdef debug}
    AppendDebugMessage('TContest.MakeMultStr invoked, should not have been');
    {$ENDIF}
  end;

  procedure TContest.PreLogContact(var ThisContact : log_rec);
  var
    i : integer;
  begin { pre log functions common to all contests }
    if HandlingNetQso then Exit;
    { if the qso came from the network, then dont do any of this stuff }
    with ThisContact do begin
      Call := Trim(Call);
      Contest := Contest;
      Clock^.DateAndTimeUtc(Date, ts);
      LogFile^.LastQSOTime := ts;
      if not QSOTimeSet then My_Time := ts;
      if My_Serno = 0 then begin
        case RttyIni^.GetProfileBool('Ser#ByBand', 'CONTEST', False) of
          True : My_Serno := SerNum^.NextSerialNumberByBand(Band);
          False: My_Serno := SerNum^.NextSerialNumberInAll(Band);
        end;
        if assigned(net) then net^.SendSerNumData;
      end;
      I := RttyIni^.GetProfileInt('UsualRST','Profile',59);
      if I > 59 then I := 59;
      if I < 11 then I := 59;
      My_Rst := I;
      if Assigned(Radio) then Band := Radio^.GetDisplayBand;
    end;
  end;


  procedure TContest.LogContact(var ThisContact : log_rec);
  begin
    { descendents should call this parent AFTER they have made whatever adjustments}
    { are need to the Log_rec }
    with ThisContact do begin
      LogFile^.SaveToDisk(ThisContact);
      if RttyIni^.GetProfileBool('PrintEachContact', 'Profile', False) then PrintLogLine(ThisContact);
      if SerNum^.CurrentSerialNumberInAll >= DupeTbl^.GetMax then Terminate(106);
      case HandlingNetQso of
        False: begin
                 TopWin^.Update(Call);
                 BotWin^.Update(Call,His_Qth);
                 RateWin^.NewEvent(Date, Ts);
                 CallQueue^.Drop(Call);
                 if Assigned(Net) then
                   Net^.SendLogData(True, ThisContact, LogFile^.Index);
                 Prev := ThisContact;
                 CatchCall := False;
                 QSOTimeSet := False;
                 Ccall := '';
                 EntryActiveField := 2;
               end;
        True: {};
      end;
      ScoreWin^.Update;
      RateWin^.UpdateNow;
      if Assigned(ClusterMonitor) then ClusterMonitor^.RemoveFromSpots(Call,Band);
      CountryList^.BlankCTRec;
      if NOT HandlingNetQso then begin
        Editw^.NewQso(ThisContact, False);
        LogFile^.BlankLogRec(ThisContact);
      end
      else
        Editw^.NewQsoFromNet(ThisContact);
    end;
  end;

  procedure TContest.RemoveCountryMult(var R : log_rec);
  var
    Num : word;
  begin
    with CountryList^ do begin
      if not R.New_Country then Exit;
      R.New_Country := False;
      Num := Identify(R);
      if Num <> NoCountry then ResetCountry(Num,R.Band);
    end;
  end;

  procedure TContest.RemoveStateMult(var R : log_rec);
  var
    ix : Byte;
  begin
    With R do begin
      if (not New_Qth) or (His_Country <> 'W') then Exit;
      New_Qth := False;
      ix := StateList^.GetStateNo(His_Qth);
      if ix = 0 then Exit;
      StateList^.ResetState(ix,Band);
    end;
  end;

  procedure TContest.RemoveSectionMult(var R : log_rec);
  var
    ix : Byte;
  begin
    With R do begin
      if (not New_Qth) then Exit;
      if (His_Country <> 'W') or (His_Country <> 'VE') then Exit;
      New_Qth := False;
      ix := SectionList^.GetSectionNo(His_Qth);
      if ix = 0 then Exit;
      SectionList^.ResetSection(ix,Band);
    end;
  end;

  procedure TContest.RemoveYearMult(var R : log_rec);
  begin
    with R do begin
      if not New_Continent then Exit;
      New_Continent := False;
      YearList^.ResetYear(His_Serno,Band);
    end;
  end;

  procedure TContest.RemoveVeProvinceMult(var R : log_rec);
  var
    ix : Byte;
  begin
    with R do begin
      if (not New_Qth) or (His_Country <> 'VE') then Exit;
      New_Qth := False;
      ix := ProvList^.GetProvNo(His_Qth);
      if ix = 0 then Exit;
      ProvList^.ResetProv(ix,Band);
    end;
  end;

  procedure TContest.RemoveZoneMult(var R : log_rec);
  begin
    with R do begin
      if not New_Zone then Exit;
      ZoneList^.ResetZone(His_Zone,Band);
      New_Zone := False;
    end;
  end;

  procedure TContest.RemoveContinentMult(var R : log_rec);
  var
    C : ContinentType;
    Result : Boolean;
  begin
    with R do begin
      if not New_Continent then Exit;
      New_Continent := False;
      with CountryList^ do begin
        if Identify(R) = NoCountry then Exit;
        C := ContList^.GetContNo(Ct.Continent);
        ContList^.ResetContinent(C,Band);
      end;
    end;
  end;

  procedure TContest.RemoveDistMult(var R : log_rec);
  var
    District : Byte;
  begin
    with R do begin
      if not New_Qth then Exit;
      New_Qth := False;
      if His_Country = 'W' then begin
        District := WDistList^.GetDistNo(Call);
        if District > 9 then Exit;
        WDistList^.ResetDist(District,Band);
      end
      else
      if R.His_Country = 'VE' then begin
        District := VEDistList^.GetDistNo(Call);
        if District > 9 then Exit;
        VEDistList^.ResetDist(District,Band);
      end
      else
      if R.His_Country = 'VK' then begin
        District := VKDistList^.GetDistNo(Call);
        if District > 9 then Exit;
        VKDistList^.ResetDist(District,Band);
      end
      else
      if R.His_Country = 'JA' then begin
        District := JADistList^.GetDistNo(Call);
        if District > 9 then Exit;
        JADistList^.ResetDist(District,Band);
      end
      else
      if His_Country = 'ZL' then begin
        District := ZLDistList^.GetDistNo(Call);
        if District > 9 then Exit;
        ZLDistList^.ResetDist(District,Band);
      end;
    end;
  end;

  function TContest.AdvisorSaysOK(Const R : log_rec) : boolean;
  begin
    if R.Call = '' then begin
      AdvisorWindowNoCall;
      AdvisorSaysOK := False;
    end
    else
      AdvisorSaysOK := True;
  end;

  function TContest.AdvisorWindow(const Missing : String) : Boolean;
  var
    WinPtr         : WindowPtr;
  begin
    if Missing = '' then begin
      AdvisorWindow := True;
      Exit;
    end;
    with Colors^ do begin
      MakeWindow(WinPtr, 20, 10, 60, 15, True, True, False, WindowAttr, FrameAttr, HeaderAttr, 'Advisor') ;
      HideMouse;
      DisplayWindow(WinPtr);
      FastWriteWindow('It appears you are missing(or invalid)', 1, 2, WindowAttr);
      FastWriteWindow(Missing, 2, 2, HiLiteAttr);
      FastWriteWindow('Press <ENTER> to log QSO anyway,', 3, 2, WindowAttr);
      FastWriteWindow('Press anykey to return to edit.', 4, 2, WindowAttr);
      ShowMouse;
      end;
    AdvisorWindow := Lo(ReadKeyWord) = 13;
    KillWindow(WinPtr);
  end;

  procedure TContest.AdvisorWindowNoCall;
  var
    WinPtr         : WindowPtr;
  begin
    with Colors^ do begin
    MakeWindow(WinPtr, 20, 10, 60, 15, True, True, False, WindowAttr, FrameAttr, HeaderAttr, 'Advisor') ;
    HideMouse;
    DisplayWindow(WinPtr);
    FastWriteWindow('You MUST have a callsign to', 1, 2, WindowAttr);
    FastWriteWindow('log any contact, other entry', 2, 2, WindowAttr);
    FastWriteWindow('fields may be optional', 3, 2, WindowAttr);
    FastWriteWindow('Press anykey to return to edit.', 4, 2, WindowAttr);
    ShowMouse;
    ReadKeyWord;
    KillWindow(WinPtr);
    end;
  end;

  procedure TContest.AdvisorWindowSameCountry;
  var
    WinPtr         : WindowPtr;
  begin
    with Colors^ do begin
    MakeWindow(WinPtr, 20, 10, 60, 15, True, True, False, WindowAttr, FrameAttr, HeaderAttr, 'Advisor');
    HideMouse;
    DisplayWindow(WinPtr);
    FastWriteWindow('Contacts between stations within',1, 2, WindowAttr);
    FastWriteWindow('the same country & district are not',     2, 2, WindowAttr);
    FastWriteWindow('allowed in this contest!',     3, 2, WindowAttr);
    FastWriteWindow('Press anykey to return to edit.',     4, 2, WindowAttr);
    ShowMouse;
    ReadKeyWord;
    KillWindow(WinPtr);
    end;
  end;

  procedure TContest.Writelog;
  begin
    PrintLogs;
    PrintSummary;
    DupeTbl^.PrintDupeReport;
    CountryList^.Print;
  end;

  procedure TContest.FillRec(const Call : String; var R : log_rec);
  begin
    {$ifdef debug}
    AppendDebugMessage('TContest.FillRec invoked, should not have been');
    {$ENDIF}
  end;

  procedure TContest.ReadFKeys;
  begin
    F[01] := RttyIni^.GetProfileString('F1', 'FKEYS', '');
    F[02] := RttyIni^.GetProfileString('F2', 'FKEYS', '');
    F[03] := RttyIni^.GetProfileString('F3', 'FKEYS', '');
    F[04] := RttyIni^.GetProfileString('F4', 'FKEYS', '');
    F[05] := RttyIni^.GetProfileString('F5', 'FKEYS', '');
    F[06] := RttyIni^.GetProfileString('F6', 'FKEYS', '');
    F[07] := RttyIni^.GetProfileString('F7', 'FKEYS', '');
    F[08] := RttyIni^.GetProfileString('F8', 'FKEYS', '');
    F[09] := RttyIni^.GetProfileString('F9', 'FKEYS', '');
    F[10] := RttyIni^.GetProfileString('F10', 'FKEYS', '');
    F[11] := RttyIni^.GetProfileString('SF1', 'FKEYS', '');
    F[12] := RttyIni^.GetProfileString('SF2', 'FKEYS', '');
    F[13] := RttyIni^.GetProfileString('SF3', 'FKEYS', '');
    F[14] := RttyIni^.GetProfileString('SF4', 'FKEYS', '');
    F[15] := RttyIni^.GetProfileString('SF5', 'FKEYS', '');
    F[16] := RttyIni^.GetProfileString('SF6', 'FKEYS', '');
    F[17] := RttyIni^.GetProfileString('SF7', 'FKEYS', '');
    F[18] := RttyIni^.GetProfileString('SF8', 'FKEYS', '');
    F[19] := RttyIni^.GetProfileString('SF9', 'FKEYS', '');
    F[20] := RttyIni^.GetProfileString('SF10', 'FKEYS', '');
    F[21] := RttyIni^.GetProfileString('B4', 'FKEYS', '');
  end;

  procedure TContest.WriteFKeys;
  begin
    RttyIni^.SetOrAddProfileString('F1', 'FKEYS', F[01]);
    RttyIni^.SetOrAddProfileString('F2', 'FKEYS', F[02]);
    RttyIni^.SetOrAddProfileString('F3', 'FKEYS', F[03]);
    RttyIni^.SetOrAddProfileString('F4', 'FKEYS', F[04]);
    RttyIni^.SetOrAddProfileString('F5', 'FKEYS', F[05]);
    RttyIni^.SetOrAddProfileString('F6', 'FKEYS', F[06]);
    RttyIni^.SetOrAddProfileString('F7', 'FKEYS', F[07]);
    RttyIni^.SetOrAddProfileString('F8', 'FKEYS', F[08]);
    RttyIni^.SetOrAddProfileString('F9', 'FKEYS', F[09]);
    RttyIni^.SetOrAddProfileString('F10', 'FKEYS', F[10]);
    RttyIni^.SetOrAddProfileString('SF1', 'FKEYS', F[11]);
    RttyIni^.SetOrAddProfileString('SF2', 'FKEYS', F[12]);
    RttyIni^.SetOrAddProfileString('SF3', 'FKEYS', F[13]);
    RttyIni^.SetOrAddProfileString('SF4', 'FKEYS', F[14]);
    RttyIni^.SetOrAddProfileString('SF5', 'FKEYS', F[15]);
    RttyIni^.SetOrAddProfileString('SF6', 'FKEYS', F[16]);
    RttyIni^.SetOrAddProfileString('SF7', 'FKEYS', F[17]);
    RttyIni^.SetOrAddProfileString('SF8', 'FKEYS', F[18]);
    RttyIni^.SetOrAddProfileString('SF9', 'FKEYS', F[19]);
    RttyIni^.SetOrAddProfileString('SF10', 'FKEYS', F[20]);
    RttyIni^.SetOrAddProfileString('B4', 'FKEYS', F[21]);
  end;



  procedure TContest.ReadDefaultFKeys;
  var
    b : Byte;
    DefIni : PIni;
    Contest : String[4];
  begin
    { this routine should be invoked only 1 time, thats when a new
      contest is starting for the very first time }
    if Not ExistFile('DEFAULT.INI') then begin
      MessageBox(
        'File not found!',
        'In attempting to read the DEFAULT.INI' ,
        'file, it was unable to be located.',
        '');
      Exit;
    end;
    DefIni := New(PIni, Init('DEFAULT.INI', True));
    Contest := ThisContest^.ContestId;
    F[1] := DefIni^.GetProfileString('F1', Contest, '');
    F[2] := DefIni^.GetProfileString('F2', Contest, '');
    F[3] := DefIni^.GetProfileString('F3', Contest, '');
    F[4] := DefIni^.GetProfileString('F4', Contest, '');
    F[5] := DefIni^.GetProfileString('F5', Contest, '');
    F[6] := DefIni^.GetProfileString('F6', Contest, '');
    F[7] := DefIni^.GetProfileString('F7', Contest, '');
    F[8] := DefIni^.GetProfileString('F8', Contest, '');
    F[9] := DefIni^.GetProfileString('F9', Contest, '');
    F[10] := DefIni^.GetProfileString('F10', Contest, '');
    F[11] := DefIni^.GetProfileString('SF1', Contest, '');
    F[12] := DefIni^.GetProfileString('SF2', Contest, '');
    F[13] := DefIni^.GetProfileString('SF3', Contest, '');
    F[14] := DefIni^.GetProfileString('SF4', Contest, '');
    F[15] := DefIni^.GetProfileString('SF5', Contest, '');
    F[16] := DefIni^.GetProfileString('SF6', Contest, '');
    F[17] := DefIni^.GetProfileString('SF7', Contest, '');
    F[18] := DefIni^.GetProfileString('SF8', Contest, '');
    F[19] := DefIni^.GetProfileString('SF9', Contest, '');
    F[20] := DefIni^.GetProfileString('SF10', Contest, '');
    F[21] := DefIni^.GetProfileString('B4', Contest, '');
    if F[1] = '' then
      F[1] := DefIni^.GetProfileString('F1', 'ELSE', 'CQ CQ CONTEST DE <11> <11> K');
    if F[2] = '' then
      F[2] := DefIni^.GetProfileString('F2', 'ELSE', '');
    if F[3] = '' then
      F[3] := DefIni^.GetProfileString('F3', 'ELSE', '<1> QSL TNX QRZ DE <11> <11> K');
    if F[4] = '' then
      F[4] := DefIni^.GetProfileString('F4', 'ELSE', '<1> DE <11> <11> K');
    if F[5] = '' then
      F[5] := DefIni^.GetProfileString('F5', 'ELSE', '');
    if F[6] = '' then
      F[6] := DefIni^.GetProfileString('F6', 'ELSE', '<1>? AGAIN? DE <11> K');
    if F[7] = '' then
      F[7] := DefIni^.GetProfileString('F7', 'ELSE', '');
    if F[8] = '' then
      F[8] := DefIni^.GetProfileString('F8', 'ELSE', 'CQ CQ CQ CONTEST DE <11> <11><CR>CQ CQ CQ CONTEST DE <11> <11> K');
    if F[9] = '' then
      F[9] := DefIni^.GetProfileString('F9', 'ELSE', '');
    if F[10] = '' then
      F[10] := DefIni^.GetProfileString('F10', 'ELSE', '');
    if F[11] = '' then
      F[11] := DefIni^.GetProfileString('SF1', 'ELSE', '');
    if F[12] = '' then
      F[12] := DefIni^.GetProfileString('SF2', 'ELSE', '');
    if F[13] = '' then
      F[13] := DefIni^.GetProfileString('SF3', 'ELSE', '');
    if F[14] = '' then
      F[14] := DefIni^.GetProfileString('SF4', 'ELSE', '');
    if F[15] = '' then
      F[15] := DefIni^.GetProfileString('SF5', 'ELSE', '');
    if F[16] = '' then
      F[16] := DefIni^.GetProfileString('SF6', 'ELSE', '');
    if F[17] = '' then
      F[17] := DefIni^.GetProfileString('SF7', 'ELSE', '');
    if F[18] = '' then
      F[18] := DefIni^.GetProfileString('SF8', 'ELSE', '');
    if F[19] = '' then
      F[19] := DefIni^.GetProfileString('SF9', 'ELSE', '');
    if F[20] = '' then
      F[20] := DefIni^.GetProfileString('SF10', 'ELSE', '');
    if F[21] = '' then
      F[21] := DefIni^.GetProfileString('B4', 'ELSE', '<1> WRKD B4 ON <22A>M CUL QRZ DE <11> K');
    Dispose(DefIni, Done);
    WriteFKeys;
  end;

  procedure TContest.ProgramFKeys;
  var
    Esr : EsRecord;
    ExitCommand : EStype;
    FKeyPtr : WindowPtr;
    b : Byte;
  begin
    with Colors^ do
    if not MakeWindow(FKeyPtr, 1, 1, 80, 25, True, True, False, WindowAttr,
      FrameAttr, HeaderAttr, 'Program Function Keys') then Exit;
    HideMouse;
    DisplayWindow(FKeyPtr);
    ReadFKeys;
    InitEsRecord(Esr);
    Colors^.SetEntryAttributes;
    SetTrimBlanks(Off);
    SetWrapMode(Esr, WrapAtEdges);
    AddStringField(Esr, 'F1 '+cGS, 2, 2, CharStr('!', 100),
      2, 9, 70, 0, nil, F[1]);
    AddStringField(Esr, 'F2 '+cGS, 3, 2, CharStr('!', 100),
      3, 9, 70, 0, nil, F[2]);
    AddStringField(Esr, 'F3 '+cGS, 4, 2, CharStr('!', 100),
      4, 9, 70, 0, nil, F[3]);
    AddStringField(Esr, 'F4 '+cGS, 5, 2, CharStr('!', 100),
      5, 9, 70, 0, nil, F[4]);
    AddStringField(Esr, 'F5 '+cGS, 6, 2, CharStr('!', 100),
      6, 9, 70, 0, nil, F[5]);
    AddStringField(Esr, 'F6 '+cGS, 7, 2, CharStr('!', 100),
      7, 9, 70, 0, nil, F[6]);
    AddStringField(Esr, 'F7 '+cGS, 8, 2, CharStr('!', 100),
      8, 9, 70, 0, nil, F[7]);
    AddStringField(Esr, 'F8 '+cGS, 9, 2, CharStr('!', 100),
      9, 9, 70, 0, nil, F[8]);
    AddStringField(Esr, 'F9 '+cGS, 10, 2, CharStr('!', 100),
      10, 9, 70, 0, nil, F[9]);
    AddStringField(Esr, 'F10'+cGS, 11, 2, CharStr('!', 100),
      11, 9, 70, 0, nil, F[10]);
    AddStringField(Esr, 'SF1 '+cGS, 13, 2, CharStr('!', 100),
      13, 9, 70, 0, nil, F[11]);
    AddStringField(Esr, 'SF2 '+cGS, 14, 2, CharStr('!', 100),
      14, 9, 70, 0, nil, F[12]);
    AddStringField(Esr, 'SF3 '+cGS, 15, 2, CharStr('!', 100),
      15, 9, 70, 0, nil, F[13]);
    AddStringField(Esr, 'SF4 '+cGS, 16, 2, CharStr('!', 100),
      16, 9, 70, 0, nil, F[14]);
    AddStringField(Esr, 'SF5 '+cGS, 17, 2, CharStr('!', 100),
      17, 9, 70, 0, nil, F[15]);
    AddStringField(Esr, 'SF6 '+cGS, 18, 2, CharStr('!', 100),
      18, 9, 70, 0, nil, F[16]);
    AddStringField(Esr, 'SF7 '+cGS, 19, 2, CharStr('!', 100),
      19, 9, 70, 0, nil, F[17]);
    AddStringField(Esr, 'SF8 '+cGS, 20, 2, CharStr('!', 100),
      20, 9, 70, 0, nil, F[18]);
    AddStringField(Esr, 'SF9 '+cGS, 21, 2, CharStr('!', 100),
      21, 9, 70, 0, nil, F[19]);
    AddStringField(Esr, 'SF10'+cGS, 22, 2, CharStr('!', 100),
      22, 9, 70, 0, nil, F[20]);
    AddStringField(Esr, 'B4 '+cGS, 24, 2, CharStr('!', 100),
      24, 9, 70, 0, nil, F[21]);
    CtrlEnter(25, 4);
    ShowMouse;
    AddCmd(ESEnd,  kbEnd);
    AddCmd(ESHome, kbHome);
    AddCmd(EsQuit, kbEsc);
    repeat
      ExitCommand := EditScreen(Esr, 0, False);
    until ExitCommand in [EsQuit, EsDone];
    if ExitCommand = EsDone then begin
      for b := 1 to 21 do
        F[b] := TrimTrailSpaces(F[b]);
      WriteFKeys;
    end;
    AddCmd(ESmy23, kbHome);
    AddCmd(ESmy25, kbEnd);
    AddCmd(Esmy49, kbEsc);
    KillWindow(FKeyPtr);
    DisposeEditScreen(Esr);
  end;

  procedure TContest.Rescore;
  begin
    if RttyIni^.GetProfileBool('AutoRescore','PROFILE',True) then
      ScoreCountPoints;
    Score := ScoreContest;
  end;









  procedure MakeContest;
  begin
    ThisContest := nil;
    case Contest of
      CQWW : ThisContest := Pointer(New(PCQWW, init));
      ARRL : ThisContest := Pointer(New(PARRL, init));
      SART : ThisContest := Pointer(New(PSART, init));
      BART : ThisContest := Pointer(New(PBART, init));
      DXPD : ThisContest := Pointer(New(PDXPD, init));
      JART : ThisContest := Pointer(New(PJART, init));
      WPXC : ThisContest := Pointer(New(PIDRA, init));
      VOLT : ThisContest := Pointer(New(PVOLT, init));
      WAED : ThisContest := Pointer(New(PWAED, init));
      NAQP : ThisContest := Pointer(New(PNAQP, init));
      ANAR : ThisContest := Pointer(New(PANAR, init));
      EAWW : ThisContest := Pointer(New(PEAWW, init));
      SPDX : ThisContest := Pointer(New(PSPDX, init));
      ARII : ThisContest := Pointer(New(PARII, init));
      RRWW : ThisContest := Pointer(New(PRRWW, init));
      TARA : ThisContest := Pointer(New(PTARA, init));
      SPRI : ThisContest := Pointer(New(PSPRI, init));
      ARFD : ThisContest := Pointer(New(PARFD, init));
      HSSC : ThisContest := Pointer(New(PHSSC, Init));
      SCCC : ThisContest := Pointer(New(PSCCC, Init));
      BSPR : ThisContest := Pointer(New(PBSPR, Init));
    else
      AppendDebugMessage('Contest is INVALID, Contact WF1B as soon as possible!');
      {$ifdef debug}
      halt;
      {$endif}
    end;
  end;

  procedure KillContest;
  begin
    if Assigned(ThisContest) then begin
      Dispose(ThisContest, done);
    end;
  end;


end.
