{****************************************************}
{*    Copyright (c) Ray Ortgiesen 1990,1991,1992    *}
{*                All rights reserved.              *}
{****************************************************}
{$O+,F+}
unit c_cqww;

interface

uses
  c_contest,
  DiskIo,
  Rectypes;

type
  PCQWW = ^TCQWW;
  TCQWW = object(TContest)
            constructor init;
            procedure Header; virtual;
            procedure Footer; virtual;
            procedure Detail; virtual;
            function LogLine(const R : Log_Rec) : String; virtual;
            destructor done; virtual;
            procedure PrintSummary; virtual;
            function  PointValue(const SomeQso : log_rec) : byte; virtual;
            procedure RemoveOldMults(var SomeQso : log_rec); virtual;
            procedure AddNewMults(var SomeQso : log_rec); virtual;
            procedure IdAnyMults(const SomeQso : log_rec); virtual;
            function  ContestId : string; virtual;
            procedure HandleAltU; virtual;
            procedure DisplayScoreWin; virtual;
            function  ScoreContest : LongInt; virtual;
            procedure ScoreCountPoints; virtual;
            function  FieldsPerRecord : byte; virtual;
            function  MultsCol : byte; virtual;
            procedure BuildEditWindow; virtual;
            function  MakeMultStr(const R : log_rec) : String; virtual;
            procedure LogContact(var ThisContact : log_rec); virtual;
            function  AdvisorSaysOK(const R : log_rec) : Boolean; virtual;
            procedure Writelog; virtual;
            procedure FillRec(const Call : String; var R : log_rec); virtual;
          end;

implementation

uses
  cClock,
  IniFile,
  cCountry,
  cContinent,
  cDist_W,
  cDist_VE,
  cDist_ZL,
  cDist_VK,
  cDist_JA,
  cState,
  cDupeTbl,
  cProvince,
  cZone,
  TpDate,
  TpWindow,
  TpMouse,
  TpCrt,
  cColors,
  TpEntry,
  cEditWin,
  TpString,
  TAHigh,
  TAccess,
  LowLvl,
  DebugRtns,
  cRttyWin,
  cLogCqww,
  Routines;


  { TCQWW.........................................................TCQWW}
  { TCQWW.........................................................TCQWW}
  { TCQWW.........................................................TCQWW}
  { TCQWW.........................................................TCQWW}
  { TCQWW.........................................................TCQWW}

  constructor TCQWW.init;
  begin
    inherited init;
    LogFile := New(PCQWWLogFile, Init);
    MakeZoneList;
    MakeStateList;
    MakeProvList('CQWW');
    MakeContList;
    MakeWDistList;
    MakeVEDistList;
    MakeVKDistList;
    MakeJADistList;
    MakeZLDistList;
    MakeCountryList('CQWW.CTY','CQWW.PFX');
    {$IFDEF DEBUG}
    AppendDebugMessage('CQWW created');
    {$ENDIF}
  end;

  destructor TCQWW.done;
  begin
    inherited done;
    KillZoneList;
    KillStateList;
    KillProvList;
    KillContList;
    KillWDistList;
    KillVEDistList;
    KillVKDistList;
    KillJADistList;
    KillZLDistList;
    KillCountryList;
    {$IFDEF DEBUG}
    AppendDebugMessage('CQWW destroyed');
    {$ENDIF}
  end;

  function TCQWW.LogLine(const R : Log_Rec) : String;
  var
    t : String[127];
  begin
    t := BandS[R.Band]+' '+
      Clock^.Date2DateString(R.Date)+
      TimeToTimeString(' hhmm ', R.Ts)+
      Pad(R.Call, 12)+' '+
      Long2Str(R.My_Rst)+'9 '+
      Pad(R.My_Qth, 4)+
      LeftPadCh(Long2Str(R.My_Zone), '0', 2)+' '+
      Long2Str(R.His_RST)+'9 '+
      Pad(R.His_Qth, 4)+
      LeftPadCh(Long2Str(R.His_Zone), '0', 2)+' '+
      Pad(R.His_Country, 5)+' ';
    if R.New_Zone then t := t+' Z '
    else t := t+'   ';
    if R.New_Country then t := t+' C '
    else t := t+'   ';
    if R.New_Qth then t := t+'  Q  '
    else t := t+'     ';
    t := t+Long2Str(R.Point);
    if R.Dupe then t := t+'-DUPE';
    LogLine := t;
  end;

  procedure TCQWW.Header;
  begin
    inherited Header;
    PrintF('Band Date Time Call         RST QTH Zn RST QTH Zn Cntry Zn Cty QTH Pt');
  end;

  procedure TCQWW.Footer;
  begin
    PrintF(CharStr(' ', 56)+'-- --- --- ---');
    PrintF('Page '+
      Pad(Long2Str(PageCT), 2)+
      CharStr(' ', 50)+
      Pad(Long2Str(sub_zon), 3)+
      Pad(Long2Str(Sub_Cty), 4)+
      Pad(Long2Str(Sub_Qth), 3)+
      Long2Str(sub_pts));
  end;

  procedure TCQWW.Detail;
  begin
    if Linect = 0 then Header;
    if a.New_Zone then Inc(sub_zon);
    if a.New_Country then Inc(Sub_Cty);
    if a.New_Qth then Inc(Sub_Qth);
    sub_pts := sub_pts+a.Point;
    PrintF(LogLine(a));
  end;

  procedure TCQWW.PrintSummary;
  var
    Band : BandType;
  begin
    inherited PrintSummary;
    PrintSummaryTop;
    PrintF('Band    QSOs   Pts   QTH    DX  Zones');
    for Band := B80 to B10 do
      PrintF(Pad(BandS[Band], 6)+
        LeftPad(Long2Str(Num_Qso[Band]), 6)+
        LeftPad(Long2Str(Num_Points[Band]), 6)+
        LeftPad(Long2Str(Num_Qth[Band]), 6)+
        LeftPad(Long2Str(Num_Country[Band]), 6)+
        LeftPad(Long2Str(Num_Zone[Band]), 6));
    PrintF(CharStr('-', 38));
    PrintF('Total '+
      LeftPad(Long2Str(Num_Qso[TOT]), 6)+
      LeftPad(Long2Str(Num_Points[TOT]), 6)+
      LeftPad(Long2Str(Num_Qth[TOT]), 6)+
      LeftPad(Long2Str(Num_Country[TOT]), 6)+
      LeftPad(Long2Str(Num_Zone[TOT]), 6));
    PrintSummaryBottom;
  end;

  function  TCQWW.PointValue(const SomeQso : log_rec) : byte;
  begin
    CountryList^.Identify(SomeQso);
    if (CountryList^.CtyNum = NoCountry) then PointValue := 0
      else if Trim(SomeQso.His_Country) = MyCountry then PointValue := 1
      else if MyContinent = CountryList^.Ct.Continent then PointValue := 2
      else PointValue := 3;
  end;

  procedure TCQWW.RemoveOldMults(var SomeQso : log_rec);
  begin
    RemoveCountryMult(SomeQso);
    RemoveStateMult(SomeQso);
    RemoveVeProvinceMult(SomeQso);
    RemoveZoneMult(SomeQso);
  end;

  procedure TCQWW.AddNewMults(var SomeQso : log_rec);
  begin
    with SomeQso do begin
      New_Country := CountryList^.Load(SomeQso,Band);
      if His_Country = 'W' then New_Qth := StateList^.Load(His_Qth, Band);
      if His_Country = 'VE' then New_Qth := ProvList^.Load(His_Qth, Band);
      New_Zone := not ZoneList^.GetZone(His_Zone,Band);
      ZoneList^.SetZone(His_Zone,Band);
    end;
  end;

  procedure TCQWW.IdAnyMults(const SomeQso : log_rec);
  var
    ix : Byte;
    tc : string[5]; { temp ct.code, it is easier to type in "this country"}
    tcn : word;     { temp country num, it is easier to type "this country num"}
  begin
    if RttyIni^.GetProfileBool('PutMsgs','Profile',True) then
      with SomeQso do begin
        if Dupe then Exit;
        if Pos('/MM', Call) > 0 then Exit;
        tcn := CountryList^.Identify(SomeQso);
        if tcn = NoCountry then Exit;
        tc := CountryList^.Ct.Code;
        if (tc <> 'W') and (tc <> 'VE') and
           (not CountryList^.GetCountry(tcn,Band)) then RttyWin^.PutString(^Y'-DX '^Z);
        if not ZoneList^.GetZone(CountryList^.Ct.CQzone,Band) then RttyWin^.PutString(^Y'-ZN '^Z);
        if tc = 'W' then begin
          ix := StateList^.GetStateNo(His_Qth);
          if (ix > 0) and (not StateList^.GetState(ix,Band)) then RttyWin^.PutString(^Y'-US '^Z);
        end;
        if tc = 'VE' then begin
          ix := ProvList^.GetProvNo(His_Qth);
          if (ix > 0) and (not ProvList^.GetProv(ix,Band)) then RttyWin^.PutString(^Y'-VE '^Z);
        end;
      end;
  end;

  function TCQWW.ContestId : string;
  begin
    ContestId := 'CQWW';
  end;

  procedure TCQWW.HandleAltU;
  begin
    StateList^.Display;
  end;

  procedure TCQWW.DisplayScoreWin;
  var
    PointsPerQso   : Real;
    PointsPerMult  : Real;
    TotMults       : LongInt;
    Band           : BandType;
    ScoreWin       : WindowPtr;
  begin
    ScoreCountPoints;
    Score := ScoreContest;
    if Score > 0 then PointsPerQso := Score / Num_Qso[Tot]
    else PointsPerQso := 0;
    TotMults := Num_Country[TOT]+Num_Qth[TOT]+Num_Zone[TOT];
    If TotMults > 0 then PointsPerMult := Score / TotMults
    else PointsPerMult := 0;
    with Colors^ do begin
    MakeWindow(ScoreWin, MW_xhi-40, MW_Ylo+1, MW_xhi-2, MW_Ylo+11,
                  True, True, True, WindowAttr, FrameAttr, HeaderAttr, 'CQWW Score Summary') ;
    HideMouse;
    DisplayWindow(ScoreWin) ;
    FastWriteWindow('Band    QSOs   Pts   QTH    DX  Zones', 1, 1, WindowAttr);
    for Band := B80 to B10 do
      FastWriteWindow(Pad(BandS[Band], 6)+
                                 LeftPad(Long2Str(Num_Qso[Band]), 6)+
                                 LeftPad(Long2Str(Num_Points[Band]), 6)+
                                 LeftPad(Long2Str(Num_Qth[Band]), 6)+
                                 LeftPad(Long2Str(Num_Country[Band]), 6)+
                                 LeftPad(Long2Str(Num_Zone[Band]), 6),
                                 2+Ord(Band), 1, WindowAttr);
    FastWriteWindow('----    ----  ----  ----  ----   ----', 7, 1, WindowAttr);
    FastWriteWindow('Total '+
                               LeftPad(Long2Str(Num_Qso[TOT]), 6)+
                               LeftPad(Long2Str(Num_Points[TOT]), 6)+
                               LeftPad(Long2Str(Num_Qth[TOT]), 6)+
                               LeftPad(Long2Str(Num_Country[TOT]), 6)+
                               LeftPad(Long2Str(Num_Zone[TOT]), 6),
                               8, 1, WindowAttr);
    FastWriteWindow('Tot: '+Pad(Long2Str(Score),8)+
                      ' P/Q:'+Real2Str(PointsPerQSO, 4, 1)+
                      ' P/M:'+Real2Str(PointsPerMult, 4, 1),
                      9, 1, WindowAttr);
    FastWrite('Press Any Key to continue', 12, 42, WindowAttr);
    end;
    ShowMouse;
    ReadKeyorButton;
    KillWindow(ScoreWin);
  end;

  function TCQWW.ScoreContest : LongInt;
  begin
    ScoreContest := Num_Points[TOT]*(Num_Qth[TOT]+Num_Country[TOT]+Num_Zone[TOT]);
  end;

  procedure TCQWW.ScoreCountPoints;
  var
    Band : BandType;
  begin
    SetCountersToZero;
    for Band := B80 to B10 do begin
      Num_Zone[Band]    := ZoneList^.HowManyZones(Band);
      Num_Qth[Band]     := ProvList^.HowMany(Band)+
                           StateList^.HowMany(Band);
      Num_Country[Band] := CountryList^.HowMany(Band);
    end;
    LogFile^.ResetLog;
    LogFile^.Get_Next(a);
    while OK do begin
      Inc(Num_Points[a.Band], a.Point);
      Inc(Num_Qso[a.Band]);
      if a.Dupe then Inc(Num_Dupes[a.Band]);
      LogFile^.Get_Next(a);
    end;
    SumCounters;
  end;

  function TCQWW.FieldsPerRecord : byte;
  { returns the number of fields (across) on the screen }
  begin
    FieldsPerRecord := 9;
  end;

  function TCQWW.MultsCol : byte;
  { returns the number of "mults" column on the screen }
  begin
    MultsCol := 6;
  end;


  procedure TCQWW.BuildEditWindow;
  var
    I : Integer;
  begin
    for I := 1 to 5 do begin
      AddChoiceField(EntryEsr, '', I, 2, 'XX', I, 1, 0, 1,
        EditIncChoicePtr, EditW^.EditRecs[I].Band);
      AddTimeField(EntryEsr, '', I, 2, 'hhmm', I, 4, 0,
        MinTime, MaxTime, EditW^.EditRecs[I].Ts);
      AddStringField(EntryEsr, '', I, 2, '!!!!!!!!!!!!', I, 9, 12, 0,
        ValidateCallSignPtr, EditW^.EditRecs[I].Call); { they ALL have callsign first! }
      AddStringField(EntryEsr, '', I, 2, '!!!',         I, 22, 3, 0,
        ValidateQTHPtr, EditW^.EditRecs[I].His_Qth);
      SetForceMode(True, True);
      AddByteField(EntryEsr, '', I, 2, '99',            I, 26, 0,
        0, 40, EditW^.EditRecs[I].His_Zone);
      SetForceMode(False, False);
      AddByteField(EntryEsr, '', I, 2, '99',            I, 29, 0,
        0, 59, EditW^.EditRecs[I].His_Rst);
      SetProtection(True);
      AddStringField(EntryEsr, '',  I, 2, '!!!!!',      I, 32, 5,
        0, nil, EditW^.Mults[I]);
      SetProtection(False);
      AddByteField(EntryEsr, '',    I, 2, '999',        I, 38, 0,
        0, 255, EditW^.EditRecs[I].Point);
      AddYesNoField(EntryEsr, '', I, 2,   '',           I, 42, 0,
        EditW^.EditRecs[I].Dupe);
    end;
    SetFieldLinks(EntryESR, 5,  2,  4);
    ActivateWindow(EditWin);
    FastWriteWindow('Mt Time Callsign     Qth Zn RS Mults Pts Dupe', 1, 1, Colors^.HeaderAttr);
  end;

  function TCQWW.MakeMultStr(const R : log_rec) : String;
  var
    T : String[5];
  begin
    T := '';
    if R.New_Country then T := T+'C'
    else T := T+' ';
    if R.New_Qth then T := T+'Q'
    else T := T+' ';
    if R.New_Zone then T := T+'Z'
    else T := T+' ';
    MakeMultStr := T;
  end;

  procedure TCQWW.LogContact(var ThisContact : log_rec);
  var
    ix : byte;
  begin
    PreLogContact(ThisContact); { parent }
    with ThisContact do begin
      if His_Zone > 40 then His_Zone := 0;
      My_Qth := RttyIni^.GetProfileString('Qth','Profile','');
      My_Zone := RttyIni^.GetProfileByte('CQZone','Profile',0);
      if CountryList^.Identify(ThisContact) <> NoCountry then
        His_Country := CountryList^.Ct.Code;
      if not HandlingNetQso then {ignore net qsos for advisor}
        if not AdvisorSaysOK(ThisContact) then Exit;
      AddNewMults(ThisContact);
      Dupe := DupeTbl^.Load(ThisContact);
      case Dupe of
        True  : Point := 0;
        False : Point := PointValue(ThisContact);
      end;
      if Pos('/MM', Call) > 0 then begin
        New_Country := False;
        New_Qth := False;
        New_Continent := False;
      end;
    end;
    inherited LogContact(ThisContact);
  end;                            {LogContact}


  function TCQWW.AdvisorSaysOK(const R : log_rec) : Boolean;
  { t = ok to log this qso, false = dont log, they want to edit }
  var
    Missing : String[40];
  begin
    if NOT inherited AdvisorSaysOK(R) then begin { hard edits, no bypass allowed }
      AdvisorSaysOK := False;
      Exit; { hard edits, no bypass allowed }
    end;
    if NOT RttyIni^.GetProfileBool('Advice','PROFILE',True) then begin                     {dont want advisor window }
      AdvisorSaysOK := True;
      Exit;
    end;
    Missing := '';
    if CountryList^.IsUSorCanada(R) and (R.His_Qth = '') then begin
      if Missing <> '' then Missing := Missing+',';
      Missing := Missing + 'QTH';
    end;
    if R.His_Zone = 0 then begin
      if Missing <> '' then Missing := Missing+',';
      Missing := Missing + 'CQ Zone';
    end;
    AdvisorSaysOK := AdvisorWindow(Missing)
  end;

  procedure TCQWW.Writelog;
  begin
    inherited Writelog;
    StateList^.PrintStatesAndVE;
    ZoneList^.Print;
  end;

  procedure TCQWW.FillRec(const Call : String; var R : log_rec);
    { returns info to R if we has QSO before with CALL         }
    { I also try to fill as much of the QSO record as possible }
  var
    Slot : Word;
    F3 : String[3];
  begin
    R.Call := Trim(Call);
    if DupeTbl^.GetDupeByCall(Call, Slot) then begin
      R.His_Qth := DupeTbl^.GetQth;
      if R.His_Zone = 0 then R.His_Zone := ZoneList^.GetZoneNo(R);
    end;
    if CountryList^.Identify(R) = NoCountry then Exit;
    R.His_Country := CountryList^.Ct.Code;
    if R.His_Zone = 0 then R.His_Zone := ZoneList^.GetZoneNo(R);
    if R.His_Country = 'VE' then begin
      R.His_Qth := ProvList^.GetProvByCall(R.Call);
      R.His_Zone := ProvList^.GetZoneByCall(R.Call);
    end;
  end;




end.
