{****************************************************}
{*    Copyright (c) Ray Ortgiesen 1990,1991,1992    *}
{*                All rights reserved.              *}
{****************************************************}
{$O+,F+}
unit c_dxpd;

interface

uses
  Diskio,
  c_contest,
  Rectypes;
type

  PDXPD = ^TDXPD;
  TDXPD = object(TContest)
            constructor init;
            procedure Header; virtual;
            procedure Footer; virtual;
            procedure Detail; virtual;
            function LogLine(const R : Log_Rec) : String; virtual;
            procedure PrintLog; virtual;
            destructor done; virtual;
            procedure PrintSummary; virtual;
            procedure RemoveOldMults(var SomeQso : log_rec); virtual;
            procedure AddNewMults(var SomeQso : log_rec); virtual;
            function ContestId : string; virtual;
            procedure HandleAltU; virtual;
            procedure DisplayScoreWin; virtual;
            procedure ScoreCountPoints; virtual;
            function  FieldsPerRecord : byte; VIRTUAL;
            function MultsCol : byte; virtual;
            procedure BuildEditWindow; virtual;
            function  MakeMultStr(const R : log_rec) : String; virtual;
            procedure LogContact(var ThisContact : log_rec); virtual;
            procedure Writelog; virtual;
            procedure FillRec(const Call : String; var R : log_rec); virtual;
          end;



implementation

uses
  TpEntry,
  cEditWin,
  cClock,
  cState,
  cProvince,
  cCountry,
  cDupeTbl,
  TpDate,
  TpString,
  TAHigh,
  TpWindow,
  TpMouse,
  TpCrt,
  TAccess,
  DebugRtns,
  LowLvl,
  cLogDxpd,
  cCOlors,
  Routines;

  { TDXPD.........................................................TDXPD}
  { TDXPD.........................................................TDXPD}
  { TDXPD.........................................................TDXPD}
  { TDXPD.........................................................TDXPD}
  { TDXPD.........................................................TDXPD}

  constructor TDXPD.init;
  begin
    inherited init;
    LogFile := New(PDXPDLogFile, Init);
    MakeStateList;
    MakeProvList(ContestId);
    MakeCountryList('ARRL.CTY','ARRL.PFX');
    {$IFDEF DEBUG}
    AppendDebugMessage('DXPD created');
    {$ENDIF}
  end;

  destructor TDXPD.done;
  begin
    inherited done;
    KillStateList;
    KillProvList;
    KillCountryList;
    {$IFDEF DEBUG}
    AppendDebugMessage('DXPD destroyed');
    {$ENDIF}
  end;

  function TDXPD.LogLine(const R : Log_Rec) : String;
  var
    t : String[127];
  begin
    t := BandS[R.Band]+' '+
      Clock^.Date2DateString(R.Date)+
      TimeToTimeString(' hhmm ', R.Ts)+
      Pad(R.Call, 12)+' '+
      Long2Str(R.His_RST)+'9 '+
      Pad(R.His_Qth, 3)+' '+
      Pad(R.His_Country, 5)+' ';
    if R.New_Country then t := t+' C '
    else t := t+'   ';
    if R.New_Qth then t := t+' Q '
    else t := t+'   ';
    if R.Dupe then t := t+'-DUPE-';
    t := t+R.Comment;
    if Length(t) > 79 then t := Copy(t, 1, 79);
    LogLine := t;
  end;

  procedure TDXPD.Header;
  begin
    inherited Header;
    PrintF('Band Date Time Call         RST QTH Cntry Cty QTH Comments');
  end;

  procedure TDXPD.Footer;
  begin
    PrintF('Page '+Pad(Long2Str(PageCT), 2));
  end;

  procedure TDXPD.Detail;
  begin
    if Linect = 0 then Header;
    PrintF(LogLine(a));
  end;

  procedure TDXPD.PrintLog;
  begin
    AllBands;
  end;

  procedure TDXPD.PrintSummary;
  var
    Band : BandType;
  begin
    inherited PrintSummary;
    PrintSummaryTop;
    PrintF('Band    QSOs   QTH    DX');
    for Band := B80 to B10 do
      PrintF(Pad(BandS[Band], 6)+
        LeftPad(Long2Str(Num_Qso[Band]), 6)+
        LeftPad(Long2Str(Num_Qth[Band]), 6)+
        LeftPad(Long2Str(Num_Country[Band]), 6));
    PrintF(CharStr('-', 32));
    PrintF('Total '+
      LeftPad(Long2Str(Num_Qso[TOT]), 6)+
      LeftPad(Long2Str(Num_Qth[TOT]), 6)+
      LeftPad(Long2Str(Num_Country[TOT]), 6));
    PrintSummaryBottom;
  end;

  procedure TDXPD.RemoveOldMults(var SomeQso : log_rec);
  begin
    RemoveCountryMult(SomeQso);
    RemoveStateMult(SomeQso);
    RemoveVeProvinceMult(SomeQso);
  end;

  procedure TDXPD.AddNewMults(var SomeQso : log_rec);
  begin
    with SomeQso do begin
      New_Country := CountryList^.Load(SomeQso,Band);
      if His_Country = 'W' then New_Qth := StateList^.Load(His_Qth, Band);
      if His_Country = 'VE' then New_Qth := ProvList^.Load(His_Qth, Band);
    end;
  end;

  function TDXPD.ContestId : string;
  begin
    ContestId := 'DXPD';
  end;

  procedure TDXPD.HandleAltU;
  begin
    StateList^.Display;
  end;

  procedure TDXPD.DisplayScoreWin;
  var
    PointsPerQso   : Real;
    PointsPerMult  : Real;
    TotMults       : LongInt;
    Band           : BandType;
    ScoreWin       : WindowPtr;
  begin
    ScoreCountPoints;
    Score := ScoreContest;
    if Score > 0 then PointsPerQso := Score / Num_Qso[Tot]
    else PointsPerQso := 0;
    TotMults := Num_Country[TOT]+Num_Qth[TOT];
    If TotMults > 0 then PointsPerMult := Score / TotMults
    else PointsPerMult := 0;
    with Colors^ do begin
    MakeWindow(ScoreWin, MW_xhi-40, MW_Ylo+1, MW_xhi-2, MW_Ylo+11,
                  True, True, True, WindowAttr, FrameAttr, HeaderAttr, 'DxPedition Summary') ;
    HideMouse;
    DisplayWindow(ScoreWin) ;
    FastWriteWindow('Band    QSOs   QTH    DX', 1, 1, WindowAttr);
    for Band := B80 to B10 do
      FastWriteWindow(Pad(BandS[Band], 6)+
                                 LeftPad(Long2Str(Num_Qso[Band]), 6)+
                                 LeftPad(Long2Str(Num_Qth[Band]), 6)+
                                 LeftPad(Long2Str(Num_Country[Band]), 6),
                                 2+Ord(Band), 1, WindowAttr);
    FastWriteWindow('----    ----  ----  ----', 7, 1, WindowAttr);
    FastWriteWindow('Total '+
                               LeftPad(Long2Str(Num_Qso[TOT]), 6)+
                               LeftPad(Long2Str(Num_Qth[TOT]), 6)+
                               LeftPad(Long2Str(Num_Country[TOT]), 6),
                               8, 1, WindowAttr);
    FastWrite('Press Any Key to continue', 12, 42, WindowAttr);
    end;
    ShowMouse;
    ReadKeyorButton;
    KillWindow(ScoreWin);
  end;

  procedure TDXPD.ScoreCountPoints;
  var
    Band : BandType;
  begin
    SetCountersToZero;
    for Band := B80 to B10 do begin
      Num_Qth[Band]     := ProvList^.HowMany(Band)+
                           StateList^.HowMany(Band);
      Num_Country[Band] := CountryList^.HowMany(Band);
    end;
    LogFile^.ResetLog;
    LogFile^.Get_Next(a);
    while OK do begin
      Inc(Num_Points[a.Band], a.Point);
      Inc(Num_Qso[a.Band]);
      if a.Dupe then Inc(Num_Dupes[a.Band]);
      LogFile^.Get_Next(a);
    end;
    SumCounters;
  end;

  function TDXPD.FieldsPerRecord : byte;
  { returns the number of fields (across) on the screen }
  begin
    FieldsPerRecord := 8;
  end;

  function TDXPD.MultsCol : byte;
  { returns the number of "mults" column on the screen }
  begin
    MultsCol := 6;
  end;

  procedure TDXPD.BuildEditWindow;
  var
    I : Integer;
  begin
    for I := 1 to 5 do begin
      AddChoiceField(EntryEsr, '', I, 2, 'XX', I, 1, 0, 1,
        EditIncChoicePtr, EditW^.EditRecs[I].Band);
      AddTimeField(EntryEsr, '', I, 2, 'hhmm', I, 4, 0,
        MinTime, MaxTime, EditW^.EditRecs[I].Ts);
      AddStringField(EntryEsr, '', I, 2, '!!!!!!!!!!!!', I, 9, 12, 0,
        ValidateCallSignPtr, EditW^.EditRecs[I].Call); { they ALL have callsign first! }
      AddStringField(EntryEsr, '', I, 2, '!!!', I, 22, 3, 0,
        ValidateQTHPtr, EditW^.EditRecs[I].His_Qth);
      AddByteField(EntryEsr, '', I, 2, '99', I, 26, 0,
        0, 59, EditW^.EditRecs[I].His_Rst);
      AddStringField(EntryEsr, '', 1, 2, Charstr('X', 15), I, 29, 15,
        0, nil, EditW^.EditRecs[I].Comment);
      SetProtection(True);
      AddStringField(EntryEsr, '',  I, 2, '!!!!!',         I, 45, 5,
        0, nil, EditW^.Mults[I]);
      SetProtection(False);
      AddYesNoField(EntryEsr, '', I, 2,   '',              I, 51, 0,
        EditW^.EditRecs[I].Dupe);
    end;
    SetFieldLinks(EntryESR, 5,  2,  4);
    ActivateWindow(EditWin);
    FastWriteWindow('Mt Time Callsign     Qth RS Comment         Mults Dupe', 1, 1, Colors^.HeaderAttr);
  end;

  function TDXPD.MakeMultStr(const R : log_rec) : String;
  var
    T : String[5];
  begin
    T := '';
    if R.New_Country then T := T+'C'
    else T := T+' ';
    if R.New_Qth then T := T+'Q'
    else T := T+' ';
    MakeMultStr := T;
  end;

  procedure TDXPD.LogContact(var ThisContact : log_rec);
  var
    ix : byte;
  begin
    PreLogContact(ThisContact);
    with ThisContact do begin
      if CountryList^.Identify(ThisContact) <> NoCountry then
        His_Country := CountryList^.Ct.Code;
      if not AdvisorSaysOK(ThisContact) then Exit;
      AddNewMults(ThisContact);
      Dupe := DupeTbl^.Load(ThisContact);
      case Dupe of
        True  : Point := 0;
        False : Point := PointValue(ThisContact);
      end;
      if Pos('/MM', Call) > 0 then begin
        New_Country := False;
        New_Qth := False;
        New_Continent := False;
      end;
    end;
    inherited LogContact(ThisContact);
  end;

  procedure TDXPD.Writelog;
  begin
    inherited Writelog;
    StateList^.PrintStatesAndVe;
  end;

  procedure TDXPD.FillRec(const Call : String; var R : log_rec);
    { returns info to R if we has QSO before with CALL         }
    { I also try to fill as much of the QSO record as possible }
  var
    Slot : Word;
    F3 : String[3];
  begin
    R.Call := Trim(Call);
    if DupeTbl^.GetDupeByCall(Call, Slot) then
      R.His_Qth := DupeTbl^.GetQth;
    if CountryList^.Identify(R) = NoCountry then Exit;
    R.His_Country := CountryList^.Ct.Code;
    if R.His_Country = 'VE' then R.His_Qth := ProvList^.GetProvByCall(R.Call);
  end;



end.
