{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit C_FT1000;

interface

uses
  c_ftcmmn,
  apport;

type
  pFT1000 = ^TFT1000;
  TFT1000 = object(tFTCommon)
              constructor Init(const ComName : ComNameType);
              destructor  Done; virtual;
              procedure   GetCurrentVFO; virtual;
              procedure   GetStatusFlags;
              function    GetDisplayFrequency : LongInt; virtual;
            end;

implementation

uses
  ApTimer,
  debugrtns,
  tpstring;

  constructor TFT1000.Init(const ComName : ComNameType);
  begin
    inherited Init(ComName);
    Name := 'FT-1000';
    {$IFDEF DEBUG}
    AppendDebugMessage('FT1000 created');
    {$ENDIF}
  end;

  destructor TFT1000.Done;
  begin
    inherited Done;
    {$IFDEF DEBUG}
    AppendDebugMessage('FT1000 destroyed');
    {$ENDIF}
  end;

  procedure TFT1000.GetCurrentVFO;
  begin
    if ((StatusFlags[1] and $01) = $01) then CurrentVFO := 'B'
                                        else CurrentVFO := 'A';
  end;

  procedure TFT1000.GetStatusFlags;
    procedure TryIt;
    begin
      NewTimer(ET, 5);
      repeat until TimerExpired(ET) or (UP^.TransReady);
      ClearCommand;
      ClearStatusFlags;
      Command[5] := $FA;
      SendCommand;
      GetCommandData(StatusFlags, 5);
    end;
  begin
    TryIt;
    MemFunc := m_vfo;
    if ((StatusFlags[2] and $40) = $40) then MemFunc := m_memory;
    if ((StatusFlags[2] and $20) = $20) then MemFunc := m_vfo;
    if ((StatusFlags[2] and $10) = $10) then MemFunc := m_memory_tune;
  end;

  function TFT1000.GetDisplayFrequency : LongInt;
  var
    O1 : Byte;                    {vfo dependent offset}
    L : LongInt;

    procedure HandleVFO;
      procedure TryIt;
      var
        ET : EventTimer;
      begin
        NewTimer(ET, 5);
        repeat until TimerExpired(ET) or (UP^.TransReady);
        ClearCommand;
        Command[5] := $10;
        Command[4] := $03;
        SendCommand;
        ClearInfoBlk;
        GetCommandData(InfoBlk, 32);
      end;
    begin
      GetCurrentVFO;
      Case CurrentVFO of
        'A' : O1 := 1;
        'B' : O1 := 17;
      end;
      TryIt;
      GetDisplayFrequency :=
        ((longint(InfoBlk[O1]) * 65536) +
         (longint(InfoBlk[O1+1]) * 256) +
          longint(InfoBlk[O1+2])) * 10;
      end;

    procedure HandleMemory;
      procedure GetChFreq;
      var
        ET : EventTimer;
      begin
        NewTimer(ET, 5);
        repeat until TimerExpired(ET) or (UP^.TransReady);
        ClearCommand;
        Command[5] := $10;
        Command[4] := $02;
        SendCommand;
        ClearInfoBlk;
        GetCommandData(InfoBlk, 16);
      end;
    begin
      GetChFreq;
      GetDisplayFrequency :=
        ((longint(InfoBlk[1]) * 65536) +
         (longint(InfoBlk[2]) * 256) +
          longint(InfoBlk[3])) * 10;
    end;

  begin
    if not Assigned(UP) then begin
      GetDisplayFrequency := 0;
      Exit;
    end;
    GetStatusFlags;
    case MemFunc of
      m_vfo : HandleVFO;
      m_memory : HandleMemory;
      m_memory_tune : HandleMemory;
      m_quick_memory : HandleMemory;
    end;
  end;



end.