{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit C_FT10MP;                    { the Yaesu FT-1000MP }

interface
uses
  c_ftcmmn,
  apport;

const
  m_vfo = $01;
  m_memory = $02;
  m_memory_tune = $03;
  m_quick_memory = $04;

type
  pFT1000MP = ^TFT1000MP;
  TFT1000MP = object(tFTCommon)
                constructor Init(const ComName : ComNameType);
                destructor Done; virtual;
                function GetDisplayFrequency : LongInt; virtual;
                procedure GetCurrentVFO; virtual;
                procedure GetStatusFlags;
              end;


implementation

uses
  lowlvl,
  TpString,
  DebugRtns,
  ApTimer;

  constructor TFT1000MP.Init(const ComName : ComNameType);
  begin
    inherited Init(ComName);
    Name := 'FT-1000MP';
    {$IFDEF DEBUG}
    AppendDebugMessage('FT1000MP created');
    {$ENDIF}
  end;

  destructor TFT1000MP.Done;
  begin
    inherited Done;
    {$IFDEF DEBUG}
    AppendDebugMessage('FT1000MP destroyed');
    {$ENDIF}
  end;

  function TFT1000MP.GetDisplayFrequency : LongInt;
  var
    O1 : Byte;                    {vfo dependent offset}
    L : LongInt;

    procedure HandleVFO;
      procedure TryIt;
      begin
        NewTimer(ET, 5);
        repeat until TimerExpired(ET) or (UP^.TransReady);
        ClearCommand;
        Command[5] := $10;
        Command[4] := $03;
        SendCommand;
        ClearInfoBlk;
        GetCommandData(InfoBlk, 32);
      end;
    begin
      GetCurrentVFO;
      case CurrentVFO of
        'A' : begin
                O1 := 1;
              end;
        'B' : begin
                O1 := 17;
              end;
      end;
      TryIt;
      L := (LongInt(InfoBlk[O1+0])*$1000000)+
        (LongInt(InfoBlk[O1+1])*$10000)+
        (LongInt(InfoBlk[O1+2])*$100)+
        LongInt(InfoBlk[O1+3]);
      if L = 0 then GetDisplayFrequency := 0
      else GetDisplayFrequency := (L shr 4)*10;
    end;

    procedure HandleMemory;
      procedure GetChFreq;
      begin
        NewTimer(ET, 5);
        repeat until TimerExpired(ET) or (UP^.TransReady);
        ClearCommand;
        Command[5] := $10;
        Command[4] := $02;
        SendCommand;
        ClearInfoBlk;
        GetCommandData(InfoBlk, 16);
      end;
    begin
      GetChFreq;
      L := (LongInt(InfoBlk[1])*$1000000)+
        (LongInt(InfoBlk[2])*$10000)+
        (LongInt(InfoBlk[3])*$100)+
        LongInt(InfoBlk[4]);
      if L = 0 then GetDisplayFrequency := 0
      else GetDisplayFrequency := (L shr 4)*10;
    end;


  begin
    if not Assigned(UP) then begin
      GetDisplayFrequency := 0;
      Exit;
    end;
    GetStatusFlags;
    case MemFunc of
      m_vfo : HandleVFO;
      m_memory : HandleMemory;
      m_memory_tune : HandleMemory;
      m_quick_memory : HandleMemory;
    end;
  end;

  procedure TFT1000MP.GetCurrentVFO;
  begin
    if ((StatusFlags[1] and $01) = $01) then CurrentVFO := 'B'
    else CurrentVFO := 'A';
  end;

  procedure TFT1000MP.GetStatusFlags;
    procedure TryIt;
    begin
      NewTimer(ET, 5);
      repeat until TimerExpired(ET) or (UP^.TransReady);
      ClearCommand;
      Command[5] := $FA;
      SendCommand;
      ClearStatusFlags;
      GetCommandData(StatusFlags, 6);
    end;
  begin
    TryIt;
    MemFunc := m_vfo;
    if ((StatusFlags[2] and $40) = $40) then MemFunc := m_memory;
    if ((StatusFlags[2] and $20) = $20) then MemFunc := m_vfo;
    if ((StatusFlags[2] and $10) = $10) then MemFunc := m_memory_tune;
    if ((StatusFlags[2] and $08) = $08) then MemFunc := m_quick_memory;
  end;


end.
