{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit C_ft900;

interface

uses
  c_ftcmmn,
  apport;

type
  pft900 = ^Tft900;
  Tft900 = object(tFTCommon)
             constructor Init(const ComName : ComNameType);
             destructor  Done; virtual;
             function    GetDisplayFrequency : LongInt; virtual;
             procedure   GetCurrentVfo; virtual;
             procedure   GetStatusFlags; Virtual;
             procedure   SetDisplayFrequency(const FreqInHertz : LongInt); virtual;
           end;

implementation

uses
  opstring,
  ApUart,
  ApTimer,
  DebugRtns,
  c_radio;

  constructor Tft900.Init(const ComName : ComNameType);
  begin
    inherited Init(ComName);
    Name := 'FT-900';
    {$IFDEF DEBUG}
    AppendDebugMessage('FT900 created');
    {$ENDIF}
  end;

  destructor Tft900.Done;
  begin
    inherited Done;
    {$IFDEF DEBUG}
    AppendDebugMessage('FT900 destroyed');
    {$ENDIF}
  end;

  function TFT900.GetDisplayFrequency : LongInt;
  var
    O1 : Byte;                    {vfo dependent offset}
    L : LongInt;

    procedure HandleVFO;
      procedure TryIt;
      begin
        NewTimer(ET, 5);
        repeat until TimerExpired(ET) or (UP^.TransReady);
        ClearCommand;
        Command[5] := $10;
        Command[4] := $03;
        Delay(500);
        SendCommand;
        Delay(500);
        ClearInfoBlk;
        GetCommandData(InfoBlk, 18);
      end;
    begin
      TryIt;
      GetCurrentVFO;
      Case CurrentVFO of
      'A' : GetDisplayFrequency :=
                             ((longint(InfoBlk[1]) * 65536) +
                              (longint(InfoBlk[2]) * 256) +
                               longint(InfoBlk[3])) * 10;
      'B' : GetDisplayFrequency :=
                             ((longint(InfoBlk[10]) * 65536) +
                              (longint(InfoBlk[11]) * 256) +
                               longint(InfoBlk[12])) * 10;
      end;
    end;

    procedure HandleMemory;
      procedure GetChFreq;
      begin
        NewTimer(ET, 5);
        repeat until TimerExpired(ET) or (UP^.TransReady);
        ClearCommand;
        Command[5] := $10;
        Command[4] := $02;
        SendCommand;
        ClearInfoBlk;
        GetCommandData(InfoBlk, 19);
      end;
    begin
      GetChFreq;
      GetDisplayFrequency := ((longint(InfoBlk[2]) * 65536) +
                              (longint(InfoBlk[3]) * 256) +
                               longint(InfoBlk[4])) * 10;
    end;


  begin
    if not Assigned(UP) then begin
      GetDisplayFrequency := 0;
      Exit;
    end;
    GetStatusFlags;
    HandleVfo;
{    case MemFunc of
      m_vfo : HandleVFO;
      m_memory : HandleMemory;
      m_memory_tune : HandleMemory;
      m_quick_memory : HandleMemory;
    end; }
  end;


  procedure TFT900.GetCurrentVFO;
  begin
    if ((StatusFlags[1] and $40) = $40) then CurrentVfo := 'B'
      else CurrentVFO := 'A';
  end;


  procedure TFT900.GetStatusFlags;
    procedure TryIt;
    begin
      NewTimer(ET, 5);
      repeat until TimerExpired(ET) or (UP^.TransReady);
      ClearCommand;
      Command[5] := $FA;
      SendCommand;
      ClearStatusFlags;
      GetCommandData(StatusFlags, 5);
    end;
  begin
    TryIt;
    if ((StatusFlags[1] and $80) = $80) then MemFunc := m_vfo;
    if ((StatusFlags[1] and $20) = $20) then MemFunc := m_memory;
    if ((StatusFlags[1] and $10) = $10) then MemFunc := m_memory_tune;
  end;

  procedure TFt900.SetDisplayFrequency(const FreqInHertz : LongInt);
  var
    Freq : LongInt;

    procedure TryIt;
    begin
      ClearUP;
      Command[5] := $0A;
      SendCommand;
    end;

    function ToBcd(B : Byte) : Byte;
    begin
      ToBcd := ((B div 10) shl 4) + (b mod 10);
    end;


  begin

    Freq := FreqInHertz;
    {100's and 10's of mhz}
    Command[4] := ToBcd(Freq div 10000000);
    Freq := Freq mod 10000000;

    {1's of mhz and 100's of khz}
    Command[3] := ToBcd(Freq div 100000);
    Freq := Freq mod 100000;

    {10's of khz and 1's of khz}
    Command[2] := ToBcd(Freq div 1000);
    Freq := Freq mod 1000;

    {100's of hz and 10's of hz}
    Command[1] := ToBcd(Freq div 10);
    TryIt;
    {$IFDEF RADTEST}
    AppendRadTestDebugMessage('c_ftCommon.SetDisplayFrequency complete.');
    {$ENDIF}
  end;


end.
