{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit C_FT990;

interface
uses
  c_ftcmmn,
  apport;


type
  pFT990 = ^TFT990;
  TFT990 = object(tFTCommon)
             constructor Init(const ComName : ComNameType);
             destructor  Done; virtual;
             function    GetDisplayFrequency : LongInt; virtual;
             procedure   GetCurrentVFO; virtual;
             procedure   GetStatusFlags;
           end;


implementation

uses
  opstring,
  ApTimer,
  DebugRtns,
  c_radio;

  constructor TFT990.Init(const ComName : ComNameType);
  begin
    inherited Init(ComName);
    Name := 'FT-990';
    {$IFDEF DEBUG}
    AppendDebugMessage('FT990 created');
    {$ENDIF}
  end;

  destructor TFT990.Done;
  begin
    inherited Done;
    {$IFDEF DEBUG}
    AppendDebugMessage('FT990 destroyed');
    {$ENDIF}
  end;

  function TFT990.GetDisplayFrequency : LongInt;
  var
    O1 : Byte;                    {vfo dependent offset}
    L : LongInt;

    procedure HandleVFO;
      procedure TryIt;
      begin
        NewTimer(ET, 5);
        repeat until TimerExpired(ET) or (UP^.TransReady);
        ClearCommand;
        Command[5] := $10;
        Command[4] := $03;
        SendCommand;
        ClearInfoBlk;
        GetCommandData(InfoBlk, 32);
      end;
    begin
      GetCurrentVFO;
      case CurrentVFO of
        'A' : O1 := 1;
        'B' : O1 := 17;
      end;
      TryIt;
      GetDisplayFrequency :=
       ((longint(InfoBlk[O1]) * 65536) +
        (longint(InfoBlk[O1+1]) * 256) +
         longint(InfoBlk[O1+2])) * 10;
    end;

    procedure HandleMemory;
      procedure GetChFreq;
      begin
        NewTimer(ET, 5);
        repeat until TimerExpired(ET) or (UP^.TransReady);
        ClearCommand;
        Command[5] := $10;
        Command[4] := $02;
        SendCommand;
        ClearInfoBlk;
        GetCommandData(InfoBlk, 16);
      end;
    begin
      GetChFreq;
      GetDisplayFrequency :=
       ((longint(InfoBlk[1]) * 65536) +
        (longint(InfoBlk[2]) * 256) +
         longint(InfoBlk[3])) * 10;
    end;


  begin
    if not Assigned(UP) then begin
      GetDisplayFrequency := 0;
      Exit;
    end;
    GetStatusFlags;
    case MemFunc of
      m_vfo : HandleVFO;
      m_memory : HandleMemory;
      m_memory_tune : HandleMemory;
      m_quick_memory : HandleMemory;
    end;
  end;

  procedure TFT990.GetCurrentVFO;
  begin
    if ((StatusFlags[1] and $01) = $01) then CurrentVFO := 'B'
    else CurrentVFO := 'A';  { I don't think this logic is correct }
  end;

  procedure TFT990.GetStatusFlags;
    procedure TryIt;
    begin
      NewTimer(ET, 5);
      repeat until TimerExpired(ET) or (UP^.TransReady);
      ClearCommand;
      Command[5] := $FA;
      SendCommand;
      ClearStatusFlags;
      GetCommandData(StatusFlags, 5);
    end;
  begin
    TryIt;
    MemFunc := m_vfo;
    if ((StatusFlags[2] and $40) = $40) then MemFunc := m_memory;
    if ((StatusFlags[2] and $20) = $20) then MemFunc := m_vfo;
    if ((StatusFlags[2] and $10) = $10) then MemFunc := m_memory_tune;
    if ((StatusFlags[2] and $08) = $08) then MemFunc := m_quick_memory;
  end;


end.
