{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit C_FTCMMN;

interface

uses
  ApMisc,
  ApPort,
  ApUart,
  OoCom,
  TpString,
  C_Radio,
  ApTimer;

const
  m_vfo = $01;
  m_memory = $02;
  m_memory_tune = $03;
  m_quick_memory = $04;

type
  pFTCommon = ^TFTCommon;
  TFTCommon = object(tRadio)
              CurrentVFO  : Char;
              StatusFlags : array[1..8] of byte;
              InfoBlk     : array[0..31] of Byte;
              Command     : array[1..5] of Byte;
              Name        : string[10];
              MemFunc     : byte;
              ET          : EventTimer;
              constructor Init(const ComName : ComNameType);
              destructor  Done; virtual;
              function    IsOnline : Boolean; virtual;
              function    GetDisplayFrequency : LongInt; virtual;
              procedure   SetDisplayFrequency(const FreqInHertz : LongInt); virtual;
              procedure   RitClear; virtual;
              procedure   ClearCommand;
              procedure   ClearStatusFlags;
              procedure   ClearInfoBlk;
              procedure   SendCommand;
              procedure   GetCommandData(var Block; ExpectedSize : Word);
              procedure   GetCurrentVFO; virtual;
              procedure   ShowStatusFlags;
              procedure   ShowInfoBlk;
            end;


implementation

  uses
    DebugRtns,
    Lowlvl;


  constructor TFTCommon.Init(const ComName : ComNameType);
  begin
    inherited Init(ComName);
    Name := 'FTCommon';
    CurrentVFO := 'A';
    ClearCommand;
    ClearInfoBlk;
    {$IFDEF DEBUG}
    AppendDebugMessage('FTCommon created');
    {$ENDIF}

  end;

  destructor TFTCommon.Done;
  begin
    inherited Done;
    {$IFDEF DEBUG}
    AppendDebugMessage('FTCommon destroyed');
    {$ENDIF}
  end;

  procedure TFTCommon.ClearCommand;
  var
    B : Byte;
  begin
    for B := 5 downto 1 do
      Command[B] := 0;
  end;

  procedure TFTCommon.SendCommand;
  var
    BytesWritten : Word;
  begin
    UP^.PutBlock(Command[1], 5, BytesWritten);
    {$IFDEF DEBUG}
    if AsyncStatus <> ecOk then AppendDebugMessage(Name+'.SendCommand.PutBlock:'+
      StatusStr(AsyncStatus)+'-'+long2str(AsyncStatus));
    {$ENDIF}
  end;

  procedure TFTCommon.GetCommandData(var Block; ExpectedSize : Word);
  var
    BytesRead : Word;
  begin
    UP^.GetBlockTimeOut(Block, ExpectedSize, BytesRead, [], 4);
    {$IFDEF DEBUG}
    if AsyncStatus <> ecOk then AppendDebugMessage(Name+'.GetCommandData.GetBlockTimeout:'+
    StatusStr(AsyncStatus)+'-'+long2str(AsyncStatus));
    {$ENDIF}
  end;


  procedure TFTCommon.RitClear;
    procedure TryIt;
    begin
      ClearUP;
      ClearCommand;
      Command[5] := $09;          {clarifier command}
      Command[4] := $FF;          {zero it out}
      SendCommand;
    end;
  begin
    TryIt;
  end;


  function TFTCommon.IsOnline : Boolean;
  var
    Result : Boolean;

    procedure Tryit;
    begin
      ClearUP;
      Result := GetDisplayFrequency > 0;
    end;

  begin
    { try to get a valid freq up to 3 times to see if radio is there }
    Tryit;
    if Result then begin
      IsOnline := True;
      Exit
    end;
    Delay(100);
    Tryit;
    if Result then begin
      IsOnline := True;
      Exit
    end;
    Delay(100);
    Tryit;
    IsOnline := Result;
  end;

  procedure TFTCommon.GetCurrentVFO;
    procedure TryIt;
    begin
      ClearUP;
      ClearCommand;
      Command[1] := $01;
      Command[2] := $01;
      Command[3] := $01;
      Command[4] := $01;
      Command[5] := $FA;
      SendCommand;
      ClearStatusFlags;
      GetCommandData(StatusFlags, 5);
      if (StatusFlags[1] and $80) = $80 then CurrentVFO := 'B'
                                        else CurrentVFO := 'A';
    end;
  begin
    TryIt;
  end;

  function TFTCommon.GetDisplayFrequency : LongInt;
  begin
    GetDisplayFrequency := 0;
  end;


  procedure TFTCommon.SetDisplayFrequency(const FreqInHertz : LongInt);
  var
    Freq : LongInt;

    procedure TryIt;
    begin
      ClearUP;
      case CurrentVFO of
        'A' : Command[5] := $0A;
        'B' : Command[5] := $8A;
      end;
      SendCommand;
    end;

    function ToBcd(B : Byte) : Byte;
    begin
      ToBcd := ((B div 10) shl 4) + (b mod 10);
    end;


  begin

    Freq := FreqInHertz;
    {100's and 10's of mhz}
    Command[4] := ToBcd(Freq div 10000000);
    Freq := Freq mod 10000000;

    {1's of mhz and 100's of khz}
    Command[3] := ToBcd(Freq div 100000);
    Freq := Freq mod 100000;

    {10's of khz and 1's of khz}
    Command[2] := ToBcd(Freq div 1000);
    Freq := Freq mod 1000;

    {100's of hz and 10's of hz}
    Command[1] := ToBcd(Freq div 10);
    TryIt;
    {$IFDEF RADTEST}
    AppendRadTestDebugMessage('c_ftCommon.SetDisplayFrequency complete.');
    {$ENDIF}
  end;

  procedure   TFTCommon.ShowStatusFlags;
  var
    i : integer;
    s : string[79];
  begin
    s := '';
    for i := 1 to 8 do
      s := s + binaryb(StatusFlags[i])+' ';
    AppendDebugMessage('Status Flags= '+s);
  end;

  procedure   TFTCommon.ShowInfoBlk;
  var
    i : integer;
    s : string[79];
  begin
    s := '';
    for i := 0 to 31 do
      s := s + hexb(InfoBlk[i])+' ';
    AppendDebugMessage('Info Block= '+s);
  end;

  procedure   TFTCOmmon.ClearStatusFlags;
  var
    i : integer;
  begin
    for i := 1 to 8 do
      StatusFlags[i] := 0;
  end;

  procedure   TFTCOmmon.ClearInfoBlk;
  var
    i : integer;
  begin
    for i := 0 to 31 do
      InfoBlk[i] := 0;
  end;


end.
