{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit c_idra;

interface

uses
  Diskio,
  c_contest,
  Rectypes;

type

  PIDRA = ^TIDRA;
  TIDRA = object(TContest)
            constructor init;
            procedure Header; virtual;
            procedure Footer; virtual;
            procedure Detail; virtual;
            function LogLine(const R : Log_Rec) : String; virtual;
            destructor done; virtual;
            procedure PrintSummary; virtual;
            function  PointValue(const SomeQso : log_rec) : byte; virtual;
            procedure RemoveOldMults(var SomeQso : log_rec); virtual;
            procedure AddNewMults(var SomeQso : log_rec); virtual;
            procedure IdAnyMults(const SomeQso : log_rec); virtual;
            function ContestId : string; virtual;
            procedure HandleAltU; virtual;
            procedure DisplayScoreWin; virtual;
            function  ScoreContest : LongInt; virtual;
            procedure ScoreCountPoints; virtual;
            function  FieldsPerRecord : byte; virtual;
            function MultsCol : byte; virtual;
            procedure BuildEditWindow; virtual;
            function  MakeMultStr(const R : log_rec) : String; virtual;
            procedure LogContact(var ThisContact : log_rec); virtual;
            procedure RemovePrefixMult(var R : log_rec);
            function  AdvisorSaysOK(const R : log_rec) : Boolean; virtual;
            procedure AllBandsPerStation;
            procedure Writelog; virtual;
            procedure FillRec(const Call : String; var R : log_rec); virtual;
          end;


implementation

uses
  TpEntry,
  cEditWin,
  cClock,
  cCountry,
  TpDate,
  TpString,
  TpWindow,
  TpMouse,
  TpCrt,
  TAHigh,
  TAccess,
  cDupeTbl,
  LowLvl,
  DebugRtns,
  IniFile,
  Network,
  cColors,
  cMsgWin,
  cRttyWin,
  cLogIdra,
  Routines;

  { TIDRA.........................................................TIDRA}
  { TIDRA.........................................................TIDRA}
  { TIDRA.........................................................TIDRA}
  { TIDRA.........................................................TIDRA}
  { TIDRA.........................................................TIDRA}

  constructor TIDRA.init;
  begin
    PrefixList := NIL;
    inherited init;
    LogFile := New(PIDRALogFile, Init);
    New(PrefixList, Init);
    if PrefixList <> nil then begin
      RttyWin^.PutString(^M+'WPX List allocated.');
    end;
    MakeCountryList('ARRL.CTY','ARRL.PFX');
    {$IFDEF DEBUG}
    AppendDebugMessage('WPXC created');
    {$ENDIF}
 end;

  destructor TIDRA.done;
  begin
    KillCountryList;
    if Assigned(PrefixList) then Dispose(PrefixList, Done);
    PrefixList := NIL;
    inherited done;
    {$IFDEF DEBUG}
    AppendDebugMessage('WPXC destroyed');
    {$ENDIF}
  end;

  function TIDRA.LogLine(const R : Log_Rec) : String;
  var
    t : String[127];
  begin
    t := BandS[R.Band]+' '+
      Clock^.Date2DateString(R.Date)+
      TimeToTimeString(' hhmm ', R.Ts)+
      Pad(R.Call, 12)+' '+
      Long2Str(R.My_Rst)+'9 '+
      LeftPadCh(Long2Str(R.My_Serno), '0', 4)+' '+
      Long2Str(R.His_RST)+'9 '+
      LeftPadCh(Long2Str(R.His_Serno), '0', 4)+' ';
    if R.New_Qth then
      t := t+Pad(PrefixList^.PrefixOf(R.Call), 7)
    else
      t := t+'       ';
    t := t+Long2Str(R.Point);
    if R.Dupe then t := t+'-DUPE'
              else t := t+'     ';
    if Assigned(Net) then
      t := t + '  S#'+long2str(R.His_Time);
    LogLine := t;
  end;

  procedure TIDRA.Header;
  var
    H : string[79];
  begin
    inherited Header;
    H := 'Band Date Time Call         RST Ser# RST Ser# Prefix Pt';
    if Assigned(Net) then H := H + '      Rig';
    PrintF(H);
  end;

  procedure TIDRA.Footer;
  begin
    PrintF('Page '+Pad(Long2Str(PageCT), 2)+
      '  SubTotal-Pts = '+Long2Str(sub_pts)+ {v2.5a}
      '  SubTotal-Wpx = '+Long2Str(sub_wpx)); {v2.5a}
  end;

  procedure TIDRA.Detail;
  begin
    if Linect = 0 then Header;
    if not a.Dupe then Inc(sub_pts, a.Point); {v2.5a}
    if a.New_Qth then Inc(sub_wpx); {v2.5a}
    PrintF(LogLine(a));
  end;

  procedure TIDRA.PrintSummary;
  var
    Pfx : String[5];
    PfxCnt : Byte;
    Band : BandType;
  begin
    inherited PrintSummary;
    PrintSummaryTop;
    PrintF('Band    QSOs   Pts  Mults');
    for Band := B80 to B10 do
      PrintF(Pad(BandS[Band], 6)+
        LeftPad(Long2Str(Num_Qso[Band]), 6)+
        LeftPad(Long2Str(Num_Points[Band]), 6)+'      -');
    PrintF('-----   ----  ----  ----');
    PrintF('Total '+
      LeftPad(Long2Str(Num_Qso[TOT]), 6)+
      LeftPad(Long2Str(Num_Points[TOT]), 6)+
      LeftPad(Long2Str(PrefixList^.Size), 6));
    PrintF('Prefixes counted only once (not once per band)');
    PrintF('');
    PrefixList^.MostCommonPrefix(Pfx, PfxCnt);
    PrintF('Most frequently worked prefix: '+Pfx+' - '+Long2Str(PfxCnt)+' QSO''s');
    PrintSummaryBottom;
  end;


  function  TIDRA.PointValue(const SomeQso : log_rec) : byte;
  var
    cpv : byte;
  begin
    Cpv := 1;
    CountryList^.Identify(SomeQso);
    if MyContinent <> CountryList^.Ct.Continent then CPV := 3;
    if (MyCountry <> SomeQSo.His_Country) and (MyContinent = CountryList^.Ct.Continent) then CPV := 2;
    if (Pos('/MM', SomeQso.Call) > 0) then CPV := 2; {2.5d}
    if SomeQso.Band in [B80, B40]
      then PointValue := CPV*2
      else PointValue := CPV;
  end;

  procedure TIDRA.RemoveOldMults(var SomeQso : log_rec);
  begin
    RemoveCountryMult(SomeQso);
    RemovePrefixMult(SomeQso);
  end;

  procedure TIDRA.AddNewMults(var SomeQso : log_rec);
  begin
    with SomeQso do begin
      New_Qth := PrefixList^.Add(PrefixList^.PrefixOf(Call));
      New_Country := CountryList^.Load(SomeQso,Band);
    end;
  end;

  procedure TIDRA.IdAnyMults(const SomeQso : log_rec);
  begin
    with SomeQso do begin
      if Dupe then Exit;
      if NOT RttyIni^.GetProfileBool('PutMsgs','Profile',True) then Exit;
      if not PrefixList^.Find(PrefixList^.PrefixOf(Call)) then RttyWin^.PutString(^Y'-WPX '^Z);
    end;
  end;

  function TIDRA.ContestId : string;
  begin
    ContestId := 'WPXC';
  end;

  procedure TIDRA.HandleAltU;
  begin
    PrefixList^.Display;
  end;

  procedure TIDRA.DisplayScoreWin;
  var
    PointsPerQso   : Real;
    PointsPerMult  : Real;
    TotMults       : LongInt;
    Band           : BandType;
    ScoreWin       : WindowPtr;
  begin
    ScoreCountPoints;
    Score := ScoreContest;
    if Score > 0 then PointsPerQso := Score / Num_Qso[Tot]
    else PointsPerQso := 0;
    TotMults := PrefixList^.Size;
    If TotMults > 0 then PointsPerMult := Score / TotMults
    else PointsPerMult := 0;
    with Colors^ do begin
    MakeWindow(ScoreWin, MW_xhi-40, MW_Ylo+1, MW_xhi-2, MW_Ylo+11,
                  True, True, True, WindowAttr, FrameAttr, HeaderAttr, 'WPX Score Summary') ;
    HideMouse;
    DisplayWindow(ScoreWin) ;
    FastWriteWindow('Band    QSOs   Pts  Mults', 1, 1, WindowAttr);
    for Band := B80 to B10 do
      FastWriteWindow(Pad(BandS[Band], 6)+
                                 LeftPad(Long2Str(Num_Qso[Band]), 6)+
                                 LeftPad(Long2Str(Num_Points[Band]), 6)+'      -',
                                 2+Ord(Band), 1, WindowAttr);
    FastWriteWindow('-----   ----  ----  -----', 7, 1, WindowAttr);
    FastWriteWindow('Total '+
                               LeftPad(Long2Str(Num_Qso[TOT]), 6)+
                               LeftPad(Long2Str(Num_Points[TOT]), 6)+
                               LeftPad(Long2Str(PrefixList^.Size), 7),
                               8, 1, WindowAttr);
    FastWriteWindow('Tot: '+Pad(Long2Str(Score),8)+
                      ' P/Q:'+Real2Str(PointsPerQSO, 4, 1)+
                      ' P/M:'+Real2Str(PointsPerMult, 4, 1),
                      9, 1, WindowAttr);
    FastWrite('Press Any Key to continue', 12, 42, WindowAttr);
    end;
    ShowMouse;
    ReadKeyorButton;
    KillWindow(ScoreWin);
  end;


  function TIDRA.ScoreContest : LongInt;
  begin
    ScoreContest := Num_Points[TOT]*PrefixList^.Size;
  end;

  procedure TIDRA.ScoreCountPoints;
  var
    Band : BandType;
  begin
    SetCountersToZero;
    LogFile^.ResetLog;
    LogFile^.Get_Next(a);
    while OK do begin
      Inc(Num_Points[a.Band], a.Point);
      Inc(Num_Qso[a.Band]);
      if a.Dupe then Inc(Num_Dupes[a.Band]);
      LogFile^.Get_Next(a);
    end;
    SumCounters;
  end;

  function TIDRA.FieldsPerRecord : byte;
  { returns the number of fields (across) on the screen }
  begin
    FieldsPerRecord := 9;
  end;


  function TIDRA.MultsCol : byte;
  { returns the number of "mults" column on the screen }
  begin
    MultsCol := 6;
  end;

  procedure TIDRA.BuildEditWindow;
  var
    I : Integer;
  begin
    for I := 1 to 5 do begin
      AddChoiceField(EntryEsr, '', I, 2, 'XX', I, 1, 0, 1,
        EditIncChoicePtr, EditW^.EditRecs[I].Band);
      AddTimeField(EntryEsr, '', I, 2, 'hhmm', I, 4, 0,
        MinTime, MaxTime, EditW^.EditRecs[I].Ts);
      AddStringField(EntryEsr, '', I, 2, '!!!!!!!!!!!!', I, 9, 12, 0,
        ValidateCallSignPtr, EditW^.EditRecs[I].Call); { they ALL have callsign first! }
      SetForceMode(True, True);
      AddWordField(EntryEsr, '', I, 2, '9999', I, 22, 0,
        0, 9999, EditW^.EditRecs[I].His_Serno);
      SetForceMode(False, False);
      AddByteField(EntryEsr, '', I, 2, '99', I, 27, 0,
        0, 59, EditW^.EditRecs[I].His_Rst);
      SetProtection(True);
      AddWordField(EntryEsr, '', I, 2, '9999', I, 31, 0,
        0, 9999, EditW^.EditRecs[I].My_Serno);
      AddStringField(EntryEsr, '',  I, 2, '!!!!!', I, 37, 5,
        0, nil, EditW^.Mults[I]);
      AddByteField(EntryEsr, '',    I, 2, '999',   I, 43, 0,
        0, 255, EditW^.EditRecs[I].Point);
      AddYesNoField(EntryEsr, '', I, 2,   '',      I, 47, 0,
        EditW^.EditRecs[I].Dupe);
      SetProtection(False);
    end;
    SetFieldLinks(EntryESR, 4,  2,  3);
    ActivateWindow(EditWin);
    FastWriteWindow('Mt Time Callsign     Ser# RS  Ser#  Mults Pts Dupe', 1, 1, Colors^.HeaderAttr);
  end;

  function TIDRA.MakeMultStr(const R : log_rec) : String;
  var
    T : String[5];
  begin
    T := '';
    if R.New_Qth then
      T := PrefixList^.PrefixOf(R.Call);
    MakeMultStr := T;
  end;

  procedure TIDRA.LogContact(var ThisContact : log_rec);
    { procedure to update the mult tables, rescore, and save record }
  begin
    PreLogContact(ThisContact);
    with ThisContact do begin
      if (NOT HandlingNetQso) then
        if not AdvisorSaysOK(ThisContact) then begin
          if Assigned(Net) then Net^.OkToReceive(True);
          Exit;
        end;
      AddNewMults(ThisContact);
      Dupe := DupeTbl^.Load(ThisContact);
      case Dupe of
        True  : Point := 0;
        False : Point := PointValue(ThisContact);
      end;
      if Pos('/MM', Call) > 0 then begin
        New_Country := False;
        New_Qth := False;
        New_Continent := False;
      end;
    end;
    if (NOT HandlingNetQso) and
       (ClassStr2Class(RttyIni^.GetProfileString('Class', 'Profile', 'Single Op, All Band')) in [Multi2,MultiMulti]) then
      ThisContact.His_Time := RttyIni^.GetProfileByte('StationNum','NetWork',0);  { store station# in time }
    inherited LogContact(ThisContact);
  end;

  procedure TIDRA.RemovePrefixMult(var R : log_rec);
  begin
    with R do begin
      if not New_Qth then Exit;
      PrefixList^.Remove(PrefixList^.PrefixOf(Call));
      New_Qth := False;
    end;
  end;

  function TIDRA.AdvisorSaysOK(const R : log_rec) : Boolean;
  { t = ok to log this qso, false = dont log, they want to edit }
  var
    Missing : String[40];
  begin
    if NOT inherited AdvisorSaysOK(R) then begin { hard edits, no bypass allowed }
      AdvisorSaysOK := False;
      Exit; { hard edits, no bypass allowed }
    end;
    if NOT RttyIni^.GetProfileBool('Advice','PROFILE',True) then begin                     {dont want advisor window }
      AdvisorSaysOK := True;
      Exit;
    end;
    Missing := '';
    if R.His_Serno = 0 then begin
      if Missing <> '' then Missing := Missing+',';
      Missing := Missing + 'Serial #';
    end;
    AdvisorSaysOK := AdvisorWindow(Missing)
  end;

  procedure TIDRA.AllBandsPerStation;
  var
    B : Byte;
  procedure LogForStation(StationNum : byte);
  var
    FileIsOpen : Boolean;
  begin
    Linect := 0;
    PageCT := 0;
    FileIsOpen := False;
    LogFile^.Index := '';
    LogFile^.ResetLog;
    MsgWin^.Display('Preparing: '+FileName('ST'+long2str(StationNum)));
    LogFile^.Get_Next(a);
    while OK do begin
      if a.His_Time = StationNum then begin
        if NOT FileIsOpen then begin
          Assign(PrintFile, FileName('ST'+long2str(StationNum)));
          Rewrite(PrintFile);
          FileIsOpen := True;
        end;
        PrintDetail;
      end;
      LogFile^.Get_Next(a);
    end;
    if FileIsOpen then begin
      if Linect > 0 then begin
        Footer;
        PrintF(^L);
      end;
      Close(PrintFile);
    end;
  end;

  begin
    with RttyIni^ do
    if NOT (ClassStr2Class(GetProfileString('Class', 'Profile', '')) in [Multi2,MultiMulti])
      then Exit;
    for B := 1 to 8 do
      LogForStation(B);
  end;

  procedure TIDRA.Writelog;
  begin
    inherited Writelog;
    PrefixList^.PrintReport;
    AllBandsPerStation;
  end;

  procedure TIDRA.FillRec(const Call : String; var R : log_rec);
    { returns info to R if we has QSO before with CALL         }
    { I also try to fill as much of the QSO record as possible }
  begin
    R.Call := Trim(Call);
    if CountryList^.Identify(R) <> NoCountry then
      R.His_Country := CountryList^.Ct.Code;
  end;



end.
