{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit c_jart;

interface

uses
  c_contest,
  Rectypes,
  Diskio;
type

  PJART = ^TJART;
  TJART = object(TContest)
            constructor init;
            procedure Header; virtual;
            procedure Footer; virtual;
            procedure Detail; virtual;
            function LogLine(const R : Log_Rec) : String; virtual;
            destructor done; virtual;
            procedure PrintSummary; virtual;
            function  PointValue(const SomeQso : log_rec) : byte; virtual;
            procedure RemoveOldMults(var SomeQso : log_rec); virtual;
            procedure AddNewMults(var SomeQso : log_rec); virtual;
            procedure IdAnyMults(const SomeQso : log_rec); virtual;
            function ContestId : string; virtual;
            procedure HandleAltU; virtual;
            procedure DisplayScoreWin; virtual;
            function  ScoreContest : LongInt; virtual;
            procedure ScoreCountPoints; virtual;
            function  FieldsPerRecord : byte; virtual;
            function MultsCol : byte; virtual;
            procedure BuildEditWindow; virtual;
            function  MakeMultStr(const R : log_rec) : String; virtual;
            procedure LogContact(var ThisContact : log_rec); virtual;
            function  AdvisorSaysOK(const R : log_rec) : Boolean; virtual;
            procedure Writelog; virtual;
            procedure FillRec(const Call : String; var R : log_rec); virtual;
          end;

implementation

uses
  tpEntry,
  cEditWin,
  cClock,
  cCountry,
  cDist_w,
  cDist_ve,
  cDist_vk,
  cDist_ja,
  cDupeTbl,
  TpDate,
  TpString,
  TpWindow,
  TpMouse,
  IniFile,
  TpCrt,
  TAHigh,
  cColors,
  TAccess,
  LowLvl,
  cRttyWin,
  cLogJart,
  DebugRtns,
  Routines;


  { TJART.........................................................TJART}
  { TJART.........................................................TJART}
  { TJART.........................................................TJART}
  { TJART.........................................................TJART}
  { TJART.........................................................TJART}

  constructor TJART.init;
  begin
    inherited init;
    LogFile := New(PJARTLogFile, Init);
    MakeWDistList;
    MakeVEDistList;
    MakeVKDistList;
    MakeJADistList;
    MakeCountryList('ARRL.CTY','ARRL.PFX');
    {$IFDEF DEBUG}
    AppendDebugMessage('JART created');
    {$ENDIF}
  end;

  destructor TJART.done;
  begin
    KillWDistList;
    KillVEDistList;
    KillVKDistList;
    KillJADistList;
    KillCountryList;
    inherited done;
    {$IFDEF DEBUG}
    AppendDebugMessage('JART destroyed');
    {$ENDIF}
  end;

  function TJART.LogLine(const R : Log_Rec) : String;
  var
    t : String[127];
  begin
    t := BandS[R.Band]+' '+
      Clock^.Date2DateString(R.Date)+
      TimeToTimeString(' hhmm ', R.Ts)+
      Pad(R.Call, 12)+' '+
      Long2Str(R.My_Rst)+'9 '+
      Pad(RttyIni^.GetProfileString('Qth','Profile',''),3)+' '+
      Long2Str(R.His_RST)+'9 '+
      Pad(R.His_Qth, 3)+' '+
      Pad(R.His_Country, 5)+' ';
    if R.New_Country then t := t+' C '
    else t := t+'   ';
    if R.New_Qth then t := t+'  D  '
    else t := t+'     ';
    t := t+LeftPad(Long2Str(R.Point), 2);
    if R.Dupe then t := t+'-DUPE';
    LogLine := t;
  end;

  procedure TJART.Header;
  begin
    inherited Header;
    PrintF('Band Date Time Call         RST Age RST Age Cntry Cty Dis Pt');
  end;

  procedure TJART.Footer;
  begin
    PrintF('Page '+
      Pad(Long2Str(PageCT), 2)+
      CharStr(' ', 30)+
      'QSOs:'+Long2Str(Sub_QSO)+
      ' Mults:'+Long2Str(sub_MULTS)+
      ' Points:'+Long2Str(sub_pts));
  end;

  procedure TJART.Detail;
  begin
    if Linect = 0 then Header;
    if a.New_Country then Inc(sub_MULTS);
    if a.New_Qth then Inc(sub_MULTS);
    sub_pts := sub_pts+a.Point;
    if not a.Dupe then Inc(Sub_QSO);
    PrintF(LogLine(a));
  end;

  procedure TJART.PrintSummary;
  var
    Band : BandType;
  begin
    inherited PrintSummary;
    PrintSummaryTop;
    PrintF('Band    QSOs   Pts  Dist    DX');
    for Band := B80 to B10 do
      PrintF(Pad(BandS[Band], 6)+
        LeftPad(Long2Str(Num_Qso[Band]), 6)+
        LeftPad(Long2Str(Num_Points[Band]), 6)+
        LeftPad(Long2Str(Num_Districts[Band]), 6)+
        LeftPad(Long2Str(Num_Country[Band]), 6));
    PrintF('-----   ----  ----  ----  ----');
    PrintF('Total '+
      LeftPad(Long2Str(Num_Qso[TOT]), 6)+
      LeftPad(Long2Str(Num_Points[TOT]), 6)+
      LeftPad(Long2Str(Num_Districts[TOT]), 6)+
      LeftPad(Long2Str(Num_Country[TOT]), 6));
    PrintSummaryBottom;
  end;

  function  TJART.PointValue(const SomeQso : log_rec) : byte;
  begin
    CountryList^.Identify(SomeQso);
    if MyContinent = CountryList^.Ct.Continent
      then PointValue := 2
      else PointValue := 3;
  end;

  procedure TJART.RemoveOldMults(var SomeQso : log_rec);
  begin
    RemoveCountryMult(SomeQso);
    RemoveDistMult(SomeQso);
  end;

  procedure TJART.AddNewMults(var SomeQso : log_rec);
  begin
    with SomeQso do begin
      if NOT ((His_Country = 'W') or (His_Country = 'VE') or
         (His_Country = 'VK') or (His_Country = 'JA'))
        then New_Country := CountryList^.Load(SomeQso,Band);
      if His_Country = 'W' then
        New_Qth := WDistList^.Load(Call,Band);
      if His_Country = 'VE' then
        New_Qth := VEDistList^.Load(Call,Band);
      if His_Country = 'VK' then
        New_Qth := VKDistList^.Load(Call,Band);
      if His_Country = 'JA' then
        New_Qth := JADistList^.Load(Call,Band);
    end;
  end;

  procedure TJART.IdAnyMults(const SomeQso : log_rec);
  var
    ix : Byte;
    Dist : Byte;
    tc : string[5]; { temp ct.code, it is easier to type in }
    tcn : word;     { temp country num, it is easier to type}
  begin
    if RttyIni^.GetProfileBool('PutMsgs','Profile',True) then
    with SomeQso do begin
      if Dupe then Exit;
      if Pos('/MM', Call) > 0 then Exit;
      tcn := CountryList^.Identify(SomeQso);
      if tcn = NoCountry then Exit;
      tc := CountryList^.Ct.Code;
      if (tc = 'W') or (tc = 'VE') or (tc = 'VK') or (tc = 'JA') then begin
        Dist := WDistList^.GetDistNo(Call);
        if Dist < 10 then begin
          if (tc = 'W')  and (not WDistList^.GetDist(Dist,Band))  then RttyWin^.PutString(^Y'-DIST '^Z);
          if (tc = 'VE') and (not VEDistList^.GetDist(Dist,Band)) then RttyWin^.PutString(^Y'-DIST '^Z);
          if (tc = 'VK') and (not VKDistList^.GetDist(Dist,Band)) then RttyWin^.PutString(^Y'-DIST '^Z);
          if (tc = 'JA') and (not JADistList^.GetDist(Dist,Band)) then RttyWin^.PutString(^Y'-DIST '^Z);
        end;
      end else begin
        if not CountryList^.GetCountry(tcn,Band) then RttyWin^.PutString(^Y'-DX '^Z);
      end;
    end;
  end;

  function TJART.ContestId : string;
  begin
    ContestId := 'JART';
  end;

  procedure TJART.HandleAltU;
  begin
    WDistList^.Display;
  end;

  procedure TJART.DisplayScoreWin;
  var
    PointsPerQso   : Real;
    PointsPerMult  : Real;
    TotMults       : LongInt;
    Band           : BandType;
    ScoreWin       : WindowPtr;
  begin
    ScoreCountPoints;
    Score := ScoreContest;
    if Score > 0 then PointsPerQso := Score / Num_Qso[Tot]
    else PointsPerQso := 0;
    TotMults := Num_Country[TOT]+Num_Qth[TOT];
    If TotMults > 0 then PointsPerMult := Score / TotMults
    else PointsPerMult := 0;
    with Colors^ do begin
    MakeWindow(ScoreWin, MW_xhi-40, MW_Ylo+1, MW_xhi-2, MW_Ylo+11,
                  True, True, True, WindowAttr, FrameAttr, HeaderAttr, 'JARTS Score Summary') ;
    HideMouse;
    DisplayWindow(ScoreWin) ;
    FastWriteWindow('Band    QSOs   Pts  Dist    DX', 1, 1, WindowAttr);
    for Band := B80 to B10 do
      FastWriteWindow(Pad(BandS[Band], 6)+
                                 LeftPad(Long2Str(Num_Qso[Band]), 6)+
                                 LeftPad(Long2Str(Num_Points[Band]), 6)+
                                 LeftPad(Long2Str(Num_Districts[Band]), 6)+
                                 LeftPad(Long2Str(Num_Country[Band]), 6),
                                 2+Ord(Band), 1, WindowAttr);
    FastWriteWindow('-----   ----  ----  ----  ----', 7, 1, WindowAttr);
    FastWriteWindow('Total '+
                               LeftPad(Long2Str(Num_Qso[TOT]), 6)+
                               LeftPad(Long2Str(Num_Points[TOT]), 6)+
                               LeftPad(Long2Str(Num_Districts[TOT]), 6)+
                               LeftPad(Long2Str(Num_Country[TOT]), 6),
                               8, 1, WindowAttr);
    FastWriteWindow('Tot: '+Pad(Long2Str(Score),8)+
                      ' P/Q:'+Real2Str(PointsPerQSO, 4, 1)+
                      ' P/M:'+Real2Str(PointsPerMult, 4, 1),
                      9, 1, WindowAttr);
    FastWrite('Press Any Key to continue', 12, 42, WindowAttr);
    end;
    ShowMouse;
    ReadKeyorButton;
    KillWindow(ScoreWin);
  end;

  function TJART.ScoreContest : LongInt;
  begin
    ScoreContest := Num_Points[TOT]*(Num_Districts[TOT]+Num_Country[TOT]);
  end;

  procedure TJART.ScoreCountPoints;
  var
    Band : BandType;
  begin
    SetCountersToZero;
    for Band := B80 to B10 do begin
      Num_Country[Band]   := CountryList^.HowMany(Band);
      Num_Districts[Band] := WDistList^.HowMany(Band)+
                             VEDistList^.HowMany(Band)+
                             VKDistList^.HowMany(Band)+
                             JADistList^.HowMany(Band);
    end;
    LogFile^.ResetLog;
    LogFile^.Get_Next(a);
    while OK do begin
      Inc(Num_Points[a.Band], a.Point);
      Inc(Num_Qso[a.Band]);
      if a.Dupe then Inc(Num_Dupes[a.Band]);
      LogFile^.Get_Next(a);
    end;
    SumCounters;
  end;

  function TJART.FieldsPerRecord : byte;
  { returns the number of fields (across) on the screen }
  begin
    FieldsPerRecord := 8;
  end;

  function TJART.MultsCol : byte;
  { returns the number of "mults" column on the screen }
  begin
    MultsCol := 5;
  end;

  procedure TJART.BuildEditWindow;
  var
    I : Integer;
  begin
    for I := 1 to 5 do begin
      AddChoiceField(EntryEsr, '', I, 2, 'XX', I, 1, 0, 1,
        EditIncChoicePtr, EditW^.EditRecs[I].Band);
      AddTimeField(EntryEsr, '', I, 2, 'hhmm', I, 4, 0,
        MinTime, MaxTime, EditW^.EditRecs[I].Ts);
      AddStringField(EntryEsr, '', I, 2, '!!!!!!!!!!!!', I, 9, 12, 0,
        ValidateCallSignPtr, EditW^.EditRecs[I].Call); { they ALL have callsign first! }
      AddStringField(EntryEsr, '', I, 2, '!!!', I, 22, 3, 0,
        ValidateQTHPtr, EditW^.EditRecs[I].His_Qth); {  for operator age  }
      AddByteField(EntryEsr, '', I, 2, '99',            I, 26, 0,
        0, 59, EditW^.EditRecs[I].His_Rst);
      SetForceMode(False, False);
      SetProtection(True);
      AddStringField(EntryEsr, '',  I, 2, '!!!!!',      I, 29, 5,
        0, nil, EditW^.Mults[I]);
      SetProtection(False);
      AddByteField(EntryEsr, '',    I, 2, '999',        I, 35, 0,
        0, 255, EditW^.EditRecs[I].Point);
      AddYesNoField(EntryEsr, '', I, 2,   '',           I, 39, 0,
        EditW^.EditRecs[I].Dupe);
    end;
    SetFieldLinks(EntryESR, 4,  2,  3);
    ActivateWindow(EditWin);
    FastWriteWindow('Mt Time Callsign     Age RS Mults Pts Dupe', 1, 1, Colors^.HeaderAttr);
  end;

  function TJART.MakeMultStr(const R : log_rec) : String;
  var
    T : String[5];
  begin
    T := '';
    if R.New_Country then T := T+'C'
                     else T := T+' ';
    if R.New_Qth then T := T+'D'
                 else T := T+' ';
    MakeMultStr := T;
  end;

  procedure TJART.LogContact(var ThisContact : log_rec);
  begin
    PreLogContact(ThisContact);
    with ThisContact do begin
      if CountryList^.Identify(ThisContact) <> NoCountry then
        His_Country := CountryList^.Ct.Code;
      if not AdvisorSaysOK(ThisContact) then Exit;
      AddNewMults(ThisContact);
      Dupe := DupeTbl^.Load(ThisContact);
      case Dupe of
        True  : Point := 0;
        False : Point := PointValue(ThisContact);
      end;
      if Pos('/MM', Call) > 0 then begin
        New_Country := False;
        New_Qth := False;
        New_Continent := False;
      end;
    end;
    inherited LogContact(ThisContact);
  end;                            {LogContact}


  function TJART.AdvisorSaysOK(const R : log_rec) : Boolean;
  { t = ok to log this qso, false = dont log, they want to edit }
  var
    Missing : String[40];
    W : Word;
  begin
    if NOT inherited AdvisorSaysOK(R) then begin { hard edits, no bypass allowed }
      AdvisorSaysOK := False;
      Exit; { hard edits, no bypass allowed }
    end;
    if NOT RttyIni^.GetProfileBool('Advice','PROFILE',True) then begin                     {dont want advisor window }
      AdvisorSaysOK := True;
      Exit;
    end;
    Missing := '';
    if (not Str2Word(R.His_Qth, W)) or
               (R.His_Qth[1] = '$') or
               (R.His_Qth = '') then begin
      if Missing <> '' then Missing := Missing+',';
      Missing := Missing + 'Age';
    end;
    AdvisorSaysOK := AdvisorWindow(Missing)
  end;

  procedure TJART.Writelog;
  begin
    inherited Writelog;
    WDistList^.Print;
  end;

  procedure TJART.FillRec(const Call : String; var R : log_rec);
    { returns info to R if we has QSO before with CALL         }
    { I also try to fill as much of the QSO record as possible }
  var
    Slot : Word;
  begin
    R.Call := Trim(Call);
    if DupeTbl^.GetDupeByCall(Call, Slot) then
      R.His_Qth := DupeTbl^.GetQth;
    if CountryList^.Identify(R) <> NoCountry then
      R.His_Country := CountryList^.Ct.Code;
  end;



end.
