{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit C_Kenwood;

interface
uses
  ApPort,
  ApTimer,
  C_Radio;
type
  PKenwood = ^TKenwood;
  TKenwood = object(tRadio)
             InfoStr : String[39];
             CurrentVFO : char;
             MemCh : string[2];
             Split : Boolean;
             ET : EventTimer;
             constructor Init(const ComName : ComNameType);
             destructor Done; virtual;
             function IsOnline : boolean; virtual;
             function GetDisplayFrequency : LongInt; virtual;
             procedure SetDisplayFrequency(const FreqInHertz : LongInt); virtual;
             procedure SetRITOffset(const RITOffset : ShortInt); virtual;
             procedure RitClear; virtual;
           end;

implementation
  Uses
    ApMisc,
    ApUart,
    OoCom,
    DebugRtns,
    TncControl,
    TpString;

  constructor TKenwood.Init(const ComName : ComNameType);
  begin
    inherited Init(ComName);
    CurrentVFO := 'A';
    {$IFDEF DEBUG}
    AppendDebugMessage('KENWOOD created');
    {$ENDIF}
  end;

  destructor TKenwood.Done;
  begin
    inherited Done;
    {$IFDEF DEBUG}
    AppendDebugMessage('KENWOOD destroyed');
    {$ENDIF}
  end;

  function TKenwood.GetDisplayFrequency : LongInt;
  var
    FreqLI : LongInt;

    procedure TryIt;
    begin
      NewTimer(ET, 5);
      repeat until TimerExpired(ET) or (UP^.TransReady);
      UP^.PutString('IF;');
      UP^.GetStringTimeout(InfoStr, 39, [';'], 18);
    end;

  begin
    if not Assigned(UP) then begin
      GetDisplayFrequency := 0;
      Exit;
    end;
    TryIt;
    if Length(InfoStr) <> 38 then begin
      UP^.FlushInBuffer; { flush the input buffer }
      UP^.FlushOutBuffer; { flush the output buffer }
      GetDisplayFrequency := 0;
      Exit;
    end;
    MemCh := '';
    case infostr[31] of
      '0' : CurrentVFO := 'A';
      '1' : CurrentVFO := 'B';
      '2' : begin
              MemCh := Copy(InfoStr,27,2);
              CurrentVFO := ' ';
            end;
    end;
    Split := INfoStr[33] = '1';
    Str2Long(Copy(InfoStr, 3, 11), FreqLI);
    GetDisplayFrequency := FreqLI;
  end;

  function TKenwood.IsOnline : Boolean;
  var
    Result : Boolean;
    count : byte;

    procedure TryIt;
    begin
      NewTimer(ET, 5);
      repeat until TimerExpired(ET) or (UP^.TransReady);
      UP^.PutString('ID;');
      UP^.GetStringTimeout(InfoStr, 6, [';'], 4);
    end;

  begin
    if not Assigned(UP) then begin
      IsOnline := False;
      Exit;
    end;
    Count := 0;
    Result := False; {assume failure}
    repeat
      UP^.FlushInBuffer; { flush the input buffer }
      UP^.FlushOutBuffer; { flush the output buffer }
      TryIt;
      if Length(InfoStr) = 6 then Result := True;
      if pos('?;',InfoStr) > 0 then Result := True;
      inc(count);
    until (Count > 5) or (Result);
    UP^.FlushInBuffer; { flush the input buffer }
    UP^.FlushOutBuffer; { flush the output buffer }
    IsOnline := Result;
  end;

  procedure TKenwood.SetDisplayFrequency(const FreqInHertz : LongInt);
  var
    FreqStr : String[14];

    procedure SetVFOA;
    begin
      NewTimer(ET, 5);
      repeat until TimerExpired(ET) or (UP^.TransReady);
      UP^.PutString('FA'+FreqStr+';');
    end;

    procedure SetVFOB;
    begin
      NewTimer(ET, 5);
      repeat until TimerExpired(ET) or (UP^.TransReady);
      UP^.PutString('FB'+FreqStr+';');
    end;

    procedure SetMemToVFOA;
    begin
      NewTimer(ET, 5);
      repeat until TimerExpired(ET) or (UP^.TransReady);
      UP^.PutString('FT0;');
      UP^.PutString('FR0;');
      SetVFOA;
      CurrentVFO := 'A';
    end;

  begin
    FreqStr := Long2str(FreqInHertz);
    FreqStr := LeftPadCh(FreqStr, '0', 11);
    Case CurrentVfo of
      'A' : SetVFOA;
      'B' : SetVFOB;
      ' ' : SetMemToVFOA;
    end;
  end;

  procedure TKenwood.SetRITOffset(const RITOffset : ShortInt);
  var
    ct : integer;
    updown : boolean;

    procedure RitUp;
    begin
      NewTimer(ET, 5);
      repeat until TimerExpired(ET) or (UP^.TransReady);
      UP^.PutString('RU;');
    end;

    procedure RitDown;
    begin
      NewTimer(ET, 5);
      repeat until TimerExpired(ET) or (UP^.TransReady);
      UP^.PutString('RD;');
    end;

  begin
    if RITOffset = 0 then Exit;
    RitClear;
    updown := (RitOffset > 0);
    For ct := 1 to abs(RitOffset) do begin
      case updown of
        TRUE : RitDown;
        FALSE: RitUp;
      end;
    end;
  end;

  procedure TKenwood.RitClear;
    procedure TryIt;
    begin
      NewTimer(ET, 5);
      repeat until TimerExpired(ET) or (UP^.TransReady);
      UP^.PutString('RC;');
    end;

  begin
    TryIt;
  end;


end.
