{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit c_naqp;

interface

uses
  c_contest,
  Diskio,
  Rectypes;
type
  PNAQP = ^TNAQP;
  TNAQP = object(TContest)
            constructor init;
            procedure Header; virtual;
            procedure Footer; virtual;
            procedure Detail; virtual;
            function LogLine(const R : Log_Rec) : String; virtual;
            destructor done; virtual;
            procedure PrintSummary; virtual;
            procedure RemoveOldMults(var SomeQso : log_rec); virtual;
            procedure AddNewMults(var SomeQso : log_rec); virtual;
            procedure IdAnyMults(const SomeQso : log_rec); virtual;
            function  ContestId : string; virtual;
            procedure HandleAltU; virtual;
            procedure DisplayScoreWin; virtual;
            function  ScoreContest : LongInt; virtual;
            procedure ScoreCountPoints; virtual;
            function  FieldsPerRecord : byte; virtual;
            function MultsCol : byte; virtual;
            procedure BuildEditWindow; virtual;
            function  MakeMultStr(const R : log_rec) : String; virtual;
            procedure LogContact(var ThisContact : log_rec); virtual;
            function  AdvisorSaysOK(const R : log_rec) : Boolean; virtual;
            procedure Writelog; virtual;
            procedure FillRec(const Call : String; var R : log_rec); virtual;
          end;

implementation

uses
  IniFile,
  cClock,
  cCountry,
  cState,
  cProvince,
  cDist_Ve,
  NaqpName,
  TpDate,
  TpString,
  TAHigh,
  TAccess,
  LowLvl,
  cDupeTbl,
  cColors,
  TpWindow,
  TpCrt,
  TpMouse,
  TpEntry,
  DebugRtns,
  cRttyWin,
  cLogNaqp,
  cEditWin,
  Routines;

  { TNAQP.........................................................TNAQP}
  { TNAQP.........................................................TNAQP}
  { TNAQP.........................................................TNAQP}
  { TNAQP.........................................................TNAQP}
  { TNAQP.........................................................TNAQP}

  constructor TNAQP.init;
  begin
    NameList := NIL;
    inherited init;
    LogFile := New(PNAQPLogFile, Init);
    NameList := New(PNameList, Init);
    if not Assigned(NameList) then
      MessageBox('SEVERE ERROR',
                 'Unable to create name list',
                 'Please re-install, or contact WF1B',
                 '');
    MakeStateList;
    MakeProvList(ContestId);
    MakeVEDistList;
    MakeCountryList('ARRL.CTY','ARRL.PFX');
    {$IFDEF DEBUG}
    AppendDebugMessage('NAQP created');
    {$ENDIF}
  end;

  destructor TNAQP.done;
  begin
    Dispose(NameList, Done);
    NameList := NIL;
    KillStateList;
    KillProvList;
    KillVEDistList;
    KillCountryList;
    inherited done;
    {$IFDEF DEBUG}
    AppendDebugMessage('NAQP destroyed');
    {$ENDIF}
  end;

  function TNAQP.LogLine(const R : Log_Rec) : String;
  var
    t : String[127];
  begin
    t := BandS[R.Band]+' '+
      Clock^.Date2DateString(R.Date)+
      TimeToTimeString(' hhmm ', R.Ts)+
      Pad(R.Call, 12)+' '+
      Pad(Copy(R.Comment, 1, 10), 11)+
      Pad(R.His_Qth, 3)+' '+
      Pad(R.His_Country, 5)+' ';
    if R.New_Country then t := t+'C '
    else t := t+'  ';
    if R.New_Qth then t := t+'Q  '
    else t := t+'   ';
    t := t+Long2Str(R.Point);
    if R.Dupe then t := t+'-DUPE';
    LogLine := t;
  end;

  procedure TNAQP.Header;
  begin
    inherited Header;
    PrintF('Band Date Time Call         Name       QTH Cntry Mult Pt');
  end;

  procedure TNAQP.Footer;
  begin
    PrintF('Page '+Pad(Long2Str(PageCT), 2)+
      '  SubTotal-QSO = '+Long2Str(Sub_QSO)+
      '  SubTotal-Qth = '+Long2Str(Sub_Qth)+
      '  SubTotal-Cty = '+Long2Str(Sub_Cty));
  end;

  procedure TNAQP.Detail;
  begin
    if Linect = 0 then Header;
    if a.New_Country then Inc(Sub_Cty);
    if a.New_Qth then Inc(Sub_Qth);
    if not a.Dupe then Inc(Sub_QSO);
    PrintF(LogLine(a));
  end;

  procedure TNAQP.PrintSummary;
  var
    Band : BandType;
  begin
    inherited PrintSummary;
    PrintSummaryTop;
    PrintF('Band    QSOs   Pts   QTH    DX');
    for Band := B80 to B10 do
      PrintF(Pad(BandS[Band], 6)+
        LeftPad(Long2Str(Num_Qso[Band]), 6)+
        LeftPad(Long2Str(Num_Points[Band]), 6)+
        LeftPad(Long2Str(Num_Qth[Band]), 6)+
        LeftPad(Long2Str(Num_Country[Band]), 6));
    PrintF(CharStr('-', 32));
    PrintF('Total '+
      LeftPad(Long2Str(Num_Qso[TOT]), 6)+
      LeftPad(Long2Str(Num_Points[TOT]), 6)+
      LeftPad(Long2Str(Num_Qth[TOT]), 6)+
      LeftPad(Long2Str(Num_Country[TOT]), 6));
    PrintSummaryBottom;
  end;

  procedure TNAQP.RemoveOldMults(var SomeQso : log_rec);
  begin
    RemoveCountryMult(SomeQso);
    RemoveStateMult(SomeQso);
    RemoveVeProvinceMult(SomeQso);
  end;

  procedure TNAQP.AddNewMults(var SomeQso : log_rec);
  begin
    with SomeQso do begin
      if (CountryList^.Ct.Continent = 'NA') and
         not (
              (CountryList^.Ct.Code = 'W') or
              (CountryList^.Ct.Code = 'VE')
             ) then New_Country := CountryList^.Load(SomeQso,Band); {NA countries, not US or VE }
      if CountryList^.Ct.Code = 'KH6' then { kh6 counts as NA country in this contest}
        New_Country := CountryList^.Load(SomeQso,Band);
      if His_Country = 'W' then New_Qth := StateList^.Load(His_Qth, Band);
      if His_Country = 'VE' then New_Qth := ProvList^.Load(His_Qth, Band);
      NameList^.SetName(Call, Comment);
    end;
  end;

  procedure TNAQP.IdAnyMults(const SomeQso : log_rec);
  var
    ix : Byte;
    tc : string[5]; { temp ct.code, it is easier to type in }
    tcn : word;     { temp country num, it is easier to type}
  begin
    if RttyIni^.GetProfileBool('PutMsgs','Profile',True) then
    with SomeQso do begin
      if Dupe then Exit;
      if Pos('/MM', Call) > 0 then Exit;
      tcn := CountryList^.Identify(SomeQso);
      if tcn = NoCountry then Exit;
      tc := CountryList^.Ct.Code;
      if (not CountryList^.GetCountry(tcn,Band)) and
         (tc <> 'W') and
         (tc <> 'VE') then RttyWin^.PutString(^Y'-DX '^Z);
      if tc = 'W' then begin
        ix := StateList^.GetStateNo(His_Qth);
        if (ix > 0) and (not StateList^.GetState(ix,Band)) then RttyWin^.PutString(^Y'-US '^Z);
      end;
      if tc = 'VE' then begin
        ix := ProvList^.GetProvNo(His_Qth);
        if (ix > 0) and (not ProvList^.GetProv(ix,Band)) then RttyWin^.PutString(^Y'-VE '^Z);
      end;
    end;
  end;

  function TNAQP.ContestId : string;
  begin
    ContestId := 'NAQP';
  end;

  procedure TNAQP.HandleAltU;
  begin
    StateList^.Display;
  end;

  procedure TNAQP.DisplayScoreWin;
  var
    PointsPerQso   : Real;
    PointsPerMult  : Real;
    TotMults       : LongInt;
    Band           : BandType;
    ScoreWin       : WindowPtr;
  begin
    ScoreCountPoints;
    Score := ScoreContest;
    if Score > 0 then PointsPerQso := Score / Num_Qso[Tot]
    else PointsPerQso := 0;
    TotMults := Num_Country[TOT]+Num_Qth[TOT];
    If TotMults > 0 then PointsPerMult := Score / TotMults
    else PointsPerMult := 0;
    with Colors^ do begin
    MakeWindow(ScoreWin, MW_xhi-40, MW_Ylo+1, MW_xhi-2, MW_Ylo+11,
                  True, True, True, WindowAttr, FrameAttr, HeaderAttr, 'NAQP Score Summary') ;
    HideMouse;
    DisplayWindow(ScoreWin) ;
    FastWriteWindow('Band    QSOs   Pts   QTH    DX', 1, 1, WindowAttr);
    for Band := B80 to B10 do
      FastWriteWindow(Pad(BandS[Band], 6)+
                                 LeftPad(Long2Str(Num_Qso[Band]), 6)+
                                 LeftPad(Long2Str(Num_Points[Band]), 6)+
                                 LeftPad(Long2Str(Num_Qth[Band]), 6)+
                                 LeftPad(Long2Str(Num_Country[Band]), 6),
                                 2+Ord(Band), 1, WindowAttr);
    FastWriteWindow('----    ----  ----  ----  ----', 7, 1, WindowAttr);
    FastWriteWindow('Total '+
                               LeftPad(Long2Str(Num_Qso[TOT]), 6)+
                               LeftPad(Long2Str(Num_Points[TOT]), 6)+
                               LeftPad(Long2Str(Num_Qth[TOT]), 6)+
                               LeftPad(Long2Str(Num_Country[TOT]), 6),
                               8, 1, WindowAttr);
    FastWriteWindow('Tot: '+Pad(Long2Str(Score),8)+
                      ' P/Q:'+Real2Str(PointsPerQSO, 4, 1)+
                      ' P/M:'+Real2Str(PointsPerMult, 4, 1),
                      9, 1, WindowAttr);
    FastWrite('Press Any Key to continue', 12, 42, WindowAttr);
    end;
    ShowMouse;
    ReadKeyorButton;
    KillWindow(ScoreWin);
  end;

  function TNAQP.ScoreContest : LongInt;
  begin
    ScoreContest := Num_Points[TOT]*(Num_Qth[TOT]+Num_Country[TOT]); {v2.5a}
  end;

  procedure TNAQP.ScoreCountPoints;
  var
    Band : BandType;
  begin
    SetCountersToZero;
    for Band := B80 to B10 do begin
      Num_Qth[Band]     := ProvList^.HowMany(Band)+
                           StateList^.HowMany(Band);
      Num_Country[Band] := CountryList^.HowMany(Band);
    end;
    LogFile^.ResetLog;
    LogFile^.Get_Next(a);
    while OK do begin
      Inc(Num_Points[a.Band], a.Point);
      Inc(Num_Qso[a.Band]);
      if a.Dupe then Inc(Num_Dupes[a.Band]);
      LogFile^.Get_Next(a);
    end;
    SumCounters;
  end;

  function TNAQP.FieldsPerRecord : byte;
  { returns the number of fields (across) on the screen }
  begin
    FieldsPerRecord := 9;
  end;

  function TNAQP.MultsCol : byte;
  { returns the number of "mults" column on the screen }
  begin
    MultsCol := 6;
  end;

  procedure TNAQP.BuildEditWindow;
  var
    I : Integer;
  begin
    for I := 1 to 5 do begin
      AddChoiceField(EntryEsr, '', I, 2, 'XX', I, 1, 0, 1,
        EditIncChoicePtr, EditW^.EditRecs[I].Band);
      AddTimeField(EntryEsr, '', I, 2, 'hhmm', I, 4, 0,
        MinTime, MaxTime, EditW^.EditRecs[I].Ts);
      AddStringField(EntryEsr, '', I, 2, '!!!!!!!!!!!!', I, 9, 12, 0,
        ValidateCallSignPtr, EditW^.EditRecs[I].Call); { they ALL have callsign first! }
      AddStringField(EntryEsr, '',  I, 2, '!!!!!!!!!!', I, 22, 10, 0,
        nil, EditW^.EditRecs[I].Comment);
      AddStringField(EntryEsr, '',  I, 2, '!!!',        I, 33, 3, 0,
        ValidateQTHptr, EditW^.EditRecs[I].His_Qth);
      AddStringField(EntryEsr, '',  I, 2, 'XXXXX',      I, 37, 5, 0,
        nil, EditW^.EditRecs[I].His_Country);
      SetProtection(True);
      AddStringField(EntryEsr, '',  I, 2, '!!!!!',      I, 43, 5, 0,
        nil, EditW^.Mults[I]);
      SetProtection(False);
      AddByteField(EntryEsr, '',    I, 2, '999',        I, 49, 0,
        0, 255, EditW^.EditRecs[I].Point);
      AddYesNoField(EntryEsr, '', I, 2,   '',           I, 53, 0,
        EditW^.EditRecs[I].Dupe);
    end;
    SetFieldLinks(EntryESR, 5,  2,  4);
    ActivateWindow(EditWin);
    FastWriteWindow('Mt Time Callsign     Name       Qth Cntry Mults Pts Dupe', 1, 1, Colors^.HeaderAttr);
  end;

  function TNAQP.MakeMultStr(const R : log_rec) : String;
  var
    T : String[5];
  begin
    T := '';
    if R.New_Country then T := T+'C'
    else T := T+' ';
    if R.New_Qth then T := T+'Q'
    else T := T+' ';
    MakeMultStr := T;
  end;

  procedure TNAQP.LogContact(var ThisContact : log_rec);
    { procedure to update the mult tables, rescore, and save record }
  var
    ix : byte;
  begin
    PreLogContact(ThisContact);
    with ThisContact do begin
      My_Qth := RttyIni^.GetProfileString('Qth','Profile','');
      if CountryList^.Identify(ThisContact) <> NoCountry then
        His_Country := CountryList^.Ct.Code;
      if not AdvisorSaysOK(ThisContact) then Exit;
      AddNewMults(ThisContact);
      Dupe := DupeTbl^.Load(ThisContact);
      case Dupe of
        True  : Point := 0;
        False : Point := PointValue(ThisContact);
      end;
      if Pos('/MM', Call) > 0 then begin
        New_Country := False;
        New_Qth := False;
        New_Continent := False;
      end;
    end;
    inherited LogContact(ThisContact);
  end;


  function TNAQP.AdvisorSaysOK(const R : log_rec) : Boolean;
  { t = ok to log this qso, false = dont log, they want to edit }
  var
    Missing : String[40];
  begin
    if NOT inherited AdvisorSaysOK(R) then begin { hard edits, no bypass allowed }
      AdvisorSaysOK := False;
      Exit; { hard edits, no bypass allowed }
    end;
    if NOT RttyIni^.GetProfileBool('Advice','PROFILE',True) then begin                     {dont want advisor window }
      AdvisorSaysOK := True;
      Exit;
    end;
    Missing := '';
    if Trim(R.Comment) = '' then begin
      if Missing <> '' then Missing := Missing+',';
      Missing := Missing + 'Name';
    end;
    if (R.His_Qth = '') and
       ((R.His_Country = 'W') or (R.His_Country = 'VE')) then begin
      if Missing <> '' then Missing := Missing+',';
      Missing := Missing + 'QTH';
    end;
    AdvisorSaysOK := AdvisorWindow(Missing)
  end;

  procedure TNAQP.Writelog;
  begin
    inherited Writelog;
    StateList^.PrintStatesAndVE;
  end;


  procedure TNAQP.FillRec(const Call : String; var R : log_rec);
    { returns info to R if we has QSO before with CALL         }
    { I also try to fill as much of the QSO record as possible }
  var
    Slot : Word;
    F3 : String[3];
  begin
    R.Call := Trim(Call);
    if DupeTbl^.GetDupeByCall(Call, Slot) then begin
      R.His_Qth := DupeTbl^.GetQth;
      R.Comment := NameList^.GetName(R.Call);
    end;
    if CountryList^.Identify(R) = NoCountry then Exit;
    R.His_Country := CountryList^.Ct.Code;
    if R.His_Country = 'VE' then
      R.His_Qth := ProvList^.GetProvByCall(R.Call);
  end;



end.
