{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit C_Radio;

interface
uses
  ApMisc,
  ApPort,
  ApUart,
  OoCom,
  TpString,
  RecTypes;
type
  PRadio = ^TRadio;
  TRadio = object
             LastFreq : LongInt;  { last known, valid, frequency }
             UP : UartPortPtr;
             PollFreqInTics : Byte; { # of tics between calls }
             FavFreq : array[B80..B10] of LongInt;
             constructor Init(const ComPort : ComNameType);
             destructor Done; virtual;
             function IsOnline : Boolean; virtual;
             function GetDisplayFrequency : LongInt; virtual;
             function GetDisplayBand : BandType; virtual;
             function GetDisplayBandStr : String;
             procedure SetDisplayFrequency(const Freq : LongInt); virtual;
             procedure SetRITOffset(const RITOffset : ShortInt); virtual;
             procedure RitClear; virtual;
             procedure ZeroBeatTnc; virtual;
             procedure ChangeFrequency(const Str : String; const Offset : Integer);
             procedure ChangeFrequencyHz(const Str : String; const Offset : Integer);
             procedure ClearUP;
           end;

procedure MakeRadio;
procedure KillRadio;

var
  Radio : PRadio;

implementation

uses
  c_ft1000,
  c_ft10mp,
  c_ft990,
  c_ft920,
  c_ft900,
  c_ft890,
  c_Kenwood,
  cRttyWin,
  DebugRtns,
  Icomciv,
  LowLvl,
  IniFile,
  TncObject,
  TncControl,
  TpDate;


  constructor TRadio.Init(const ComPort : ComNameType);
  const
    RadioPortOptions = ptReturnPartialGets+ptReturnDelimiter+
    ptRaiseModemOnOpen+ptExecutePartialPuts+
    ptRestoreOnClose;
  var
    iBaudRate : LongInt;
    iDataBits : LongInt;
    iStopBits : LongInt;
    iHandShake : HandShakeType;
    iParity : ParityType;
    S : String;
    PollFreqInMs : integer;
  begin
    UP := New(UartPortPtr, InitCustom(ComPort, 4800, NoParity, 8, 1, 1024, 256, RadioPortOptions));
    {$IFDEF DEBUG}
    if AsyncStatus <> ecOk then begin
      WriteLn('Failed to open port: ', AsyncStatus);
      Halt;
    end;
    {$ELSE}
    if AsyncStatus <> ecOk then begin
      RttyWin^.PutString(cCR+'Com Port for Radio could not be opened!');
      AppendDebugMessage('Radio.Init Error, AsyncStatus='+long2str(AsyncStatus));
    end;
    {$ENDIF}
    iBaudRate := RttyIni^.GetProfileLong('BaudRate', ComNameString(ComPort), 4800);
    S := RttyIni^.GetProfileString('Parity', ComNameString(ComPort), 'N');
    iParity := ParityStringToParity(S);
    iDataBits := RttyIni^.GetProfileLong('DataBits', ComNameString(ComPort), 8);
    iStopBits := RttyIni^.GetProfileLong('StopBits', ComNameString(ComPort), 2);
    S := RttyIni^.GetProfileString('HandShake', ComNameString(ComPort), 'None');
    iHandShake := HandShakeStringToHandShake(S);
    with UP^ do begin
      ChangeBaud(iBaudRate);
      if AsyncStatus <> ecOk then AppendDebugMessage('Radio.Init.ChangeBaud, AsyncStatus='+long2str(AsyncStatus));
      ChangeDataBits(iDataBits);
      if AsyncStatus <> ecOk then AppendDebugMessage('Radio.Init.ChangeDataBits, AsyncStatus='+long2str(AsyncStatus));
      ChangeStopBits(iStopBits);
      if AsyncStatus <> ecOk then AppendDebugMessage('Radio.Init.ChangeStopBits, AsyncStatus='+long2str(AsyncStatus));
      ChangeParity(iParity);
      if AsyncStatus <> ecOk then AppendDebugMessage('Radio.Init.ChangeParity, AsyncStatus='+long2str(AsyncStatus));
    end;
    ChangeHandShake(UP, iHandShake);
    if AsyncStatus <> ecOk then AppendDebugMessage('Radio.Init.ChangeHandShake, AsyncStatus='+long2str(AsyncStatus));
    PollFreqInMs := RttyIni^.GetProfileInt('Poll', 'RADIO', 500); { every 1/2 second }
    if PollFreqInMs < 55 then PollFreqInMs := 55;
    if PollFreqInMs > 5000 then PollFreqInMs := 5000;
    PollFreqInTics := PollFreqInMs div 55;
    FavFreq[B80] := RttyIni^.GetProfileLong('80', 'RADIO', 3600)*1000;
    FavFreq[B40] := RttyIni^.GetProfileLong('40', 'RADIO', 7080)*1000;
    FavFreq[B20] := RttyIni^.GetProfileLong('20', 'RADIO', 14080)*1000;
    FavFreq[B15] := RttyIni^.GetProfileLong('15', 'RADIO', 21080)*1000;
    FavFreq[B10] := RttyIni^.GetProfileLong('10', 'RADIO', 28080)*1000;
    ClearUP;
    {$IFDEF DEBUG}
    AppendDebugMessage('RADIO created');
    {$ENDIF}
  end;

  destructor TRadio.Done;
  begin
    if Assigned(UP) then UP^.Done;
    {$IFDEF DEBUG}
    AppendDebugMessage('RADIO destroyed');
    {$ENDIF}
  end;

  function TRadio.GetDisplayFrequency : LongInt;
  begin
    RunError(211);
  end;

  function TRadio.IsOnline : Boolean;
  begin
    IsOnline := False;
    AppendDebugMessage('TRadio.IsOnline=False');
  end;

  procedure TRadio.ClearUP;
  begin
    UP^.FlushInBuffer; { flush the input buffer }
    UP^.FlushOutBuffer; { flush the input buffer }
    if UP^.CheckFifoError then begin
      SetFifoBuffering(UP^.PR^.BaseAddr,False,0); { flush the fifo buffer }
      SetFifoBuffering(UP^.PR^.BaseAddr,True,14);
    end;
    UP^.GetLineError;  { resets error indicators }
  end;

  procedure TRadio.SetDisplayFrequency(const Freq : LongInt);
  begin
    RunError(211);
  end;

  procedure TRadio.SetRITOffset(const RITOffset : ShortInt);
  begin
  end;


  procedure TRadio.RitClear;
  begin
  end;

  procedure TRadio.ZeroBeatTnc;
  begin
    if not (RttyTerm1^.TncNum in [bP38,bPci4000,bDSP4100,bDxp38]) then Exit;
    SetRitOffset(RttyTerm1^.FreqDelta);
  end;

  procedure TRadio.ChangeFrequency(const Str : String; const Offset : Integer);
  var
    I : Integer;
    valid : Boolean;
    S : String[11];
    FreqLI : LongInt;
  begin
    valid := True;
    S := Trim(Str);
    while Pos('.', S) > 0 do
      Delete(S, Pos('.', S), 1);
    S := Trim(S);
    for I := 1 to Length(S) do
      if not(S[I] in ['0'..'9']) then valid := False;
    if not valid then Exit;
    Str2Long(S, FreqLI);
    if ((FreqLI >= 3500) and (FreqLI <= 4000)) or
       ((FreqLI >= 7000) and (FreqLI <= 7300)) or
       ((FreqLI >= 14000) and (FreqLI <= 14350)) or
       ((FreqLI >= 21000) and (FreqLI <= 21450)) or
       ((FreqLI >= 28000) and (FreqLI <= 29000)) then begin
      FreqLI := FreqLI * 1000; { make is hurtz :) }
      SetDisplayFrequency(FreqLI-Offset);
    end;
  end;

  procedure TRadio.ChangeFrequencyHz(const Str : String; const Offset : Integer);
  var
    I : Integer;
    S : String[11];
    FreqLI : LongInt;
  begin
    S := Trim(Str);
    for I := 1 to Length(S) do
      if not(S[I] in ['0'..'9']) then Exit; { not a valid number }
    Str2Long(S, FreqLI);
    if ((FreqLI >= 3500000) and (FreqLI <= 4000000)) or
       ((FreqLI >= 7000000) and (FreqLI <= 7300000)) or
       ((FreqLI >= 14000000) and (FreqLI <= 14350000)) or
       ((FreqLI >= 21000000) and (FreqLI <= 21450000)) or
       ((FreqLI >= 28000000) and (FreqLI <= 29000000)) then begin
      SetDisplayFrequency(FreqLI-Offset);
    end;
  end;


  function TRadio.GetDisplayBand : BandType;
  var
    Freq : LongInt;
  begin
    Freq := Radio^.GetDisplayFrequency;
    if Freq = 0 then Freq := LastFreq
    else LastFreq := Freq;        { adjust last good freq }
    case Freq div 1000000 of
      3 : GetDisplayBand := B80;
      7 : GetDisplayBand := B40;
      14 : GetDisplayBand := B20;
      21 : GetDisplayBand := B15;
      28 : GetDisplayBand := B10;
    else
{      GetDisplayBand := B20; }
       GetDisplayBand := CurrentBand;
    end;
  end;

  function TRadio.GetDisplayBandStr : String;
  begin
    GetDisplayBandStr := BandS[GetDisplayBand];
  end;

  {----------------------------------------------------------------------}

  procedure MakeRadio;
  var
    RadioStr : String[10];
    ComPort : Byte;
    ComName : ComNameType;
  const
    Msg2 = 'Radio: [Radio] section of RTTY.INI is incorrect!';

    procedure TestOnline;
    const
      Msg1 = 'Radio: Unable to initialize, will not be used.';
    begin
      if not Radio^.IsOnline then begin
        RttyWin^.PutString(cCr+Msg1);
        AppendDebugMessage(Msg1);
        Dispose(Radio, Done);
        Radio := nil;
        Exit;
      end;
    end;

  begin
    ComPort := RttyIni^.GetProfileByte('ComPort', 'RADIO', 0);
    RadioStr := StUpCase(RttyIni^.GetProfileString('Type', 'RADIO', 'NONE'));
    if (RadioStr = 'NONE') or
    (ComPort = 0) or
    (ComPort > 8) then Exit;
    ComName := ComNameType(Ord(ComPort)-1);
    RttyWin^.PutString(cCR+'Radio "'+RadioStr+'" is on '+ComNameString(ComName));
    AppendDebugMessage('Radio "'+RadioStr+'" is on '+ComNameString(ComName));

    if RadioStr = 'KENWOOD' then Radio := Pointer(New(pKenwood, Init(ComName)));
    if RadioStr = 'FT-1000' then Radio := Pointer(New(pFT1000, Init(ComName)));
    if RadioStr = 'FT-1000MP' then Radio := Pointer(New(pFT1000MP, Init(ComName)));
    if RadioStr = 'FT-990' then Radio := Pointer(New(pFT990, Init(ComName)));
    if RadioStr = 'FT-920' then Radio := Pointer(New(pFT920, Init(ComName)));
    if RadioStr = 'FT-900' then Radio := Pointer(New(pFT900, Init(ComName)));
    if RadioStr = 'FT-890' then Radio := Pointer(New(pFT890, Init(ComName)));
    if Copy(RadioStr, 1, 3) = 'IC-' then Radio := Pointer(New(pIcomCIV, Init(ComName, RadioStr)));

    if not Assigned(Radio) then begin
      RttyWin^.PutString(cCr+Msg2);
      AppendDebugMessage(Msg2);
      Exit;
    end;

    TestOnline;

  end;

  procedure KillRadio;
  begin
    if Assigned(Radio) then Dispose(Radio, Done);
    Radio := nil;
  end;


end.
