{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit c_rrww;

interface

uses
  c_contest,
  Diskio,
  Rectypes;
type

  PRRWW = ^TRRWW;
  TRRWW = object(TContest)
            constructor init;
            procedure Header; virtual;
            procedure Footer; virtual;
            procedure Detail; virtual;
            function LogLine(const R : Log_Rec) : String; virtual;
            destructor done; virtual;
            procedure PrintSummary; virtual;
            function  PointValue(const SomeQso : log_rec) : byte; virtual;
            procedure RemoveOldMults(var SomeQso : log_rec); virtual;
            procedure AddNewMults(var SomeQso : log_rec); virtual;
            procedure IdAnyMults(const SomeQso : log_rec); virtual;
            function ContestId : string; virtual;
            procedure HandleAltU; virtual;
            procedure DisplayScoreWin; virtual;
            function  ScoreContest : LongInt; virtual;
            procedure ScoreCountPoints; virtual;
            function  FieldsPerRecord : byte; virtual;
            function MultsCol : byte; virtual;
            procedure BuildEditWindow; virtual;
            function  MakeMultStr(const R : log_rec) : String; virtual;
            procedure LogContact(var ThisContact : log_rec); virtual;
            procedure RemoveOblastMult(var R : log_rec);
            function  AdvisorSaysOK(const R : log_rec) : Boolean; virtual;
            procedure Writelog; virtual;
            procedure FillRec(const Call : String; var R : log_rec); virtual;
          end;

implementation

uses
  TpEntry,
  cEditWin,
  cClock,
  cColors,
  cCountry,
  cDupeTbl,
  cZone,
  cDist_W,
  cRTbl,
  cProvince,
  TpDate,
  IniFile,
  TpString,
  TpWindow,
  TpCrt,
  TpMouse,
  cRttyWin,
  cLogRrww,
  TAHigh,
  TAccess,
  LowLvl,
  DebugRtns,
  Routines;

  { TRRWW.........................................................TRRWW}
  { TRRWW.........................................................TRRWW}
  { TRRWW.........................................................TRRWW}
  { TRRWW.........................................................TRRWW}
  { TRRWW.........................................................TRRWW}

  constructor TRRWW.init;
  begin
    inherited init;
    LogFile := New(PRRWWLogFile, Init);
    MakeRList;
    MakeZoneList;
    MakeCountryList('ARRL.CTY','ARRL.PFX');
    MakeProvList('ARRL');
    MakeWDistList;
    {$IFDEF DEBUG}
    AppendDebugMessage('RRWW created');
    {$ENDIF}
  end;

  destructor TRRWW.done;
  begin
    KillRList;
    KillZoneList;
    KillCountryList;
    KillProvList;
    KillWDistList;
    inherited done;
    {$IFDEF DEBUG}
    AppendDebugMessage('RRWW destroyed');
    {$ENDIF}
  end;

  function TRRWW.LogLine(const R : Log_Rec) : String;
  var
    t : String[127];
  begin
    t := BandS[R.Band]+' '+
      Clock^.Date2DateString(R.Date)+
      TimeToTimeString(' hhmm ', R.Ts)+
      Pad(R.Call, 12)+' '+
      Long2Str(R.My_Rst)+'9 '+
      Pad(R.My_Qth, 4)+
      LeftPadCh(Long2Str(R.My_Zone), '0', 2)+' '+
      Long2Str(R.His_RST)+'9 '+
      Pad(R.His_Qth, 4)+
      LeftPadCh(Long2Str(R.His_Zone), '0', 2)+' '+
      Pad(R.His_Country, 5)+' ';
    if R.New_Country then t := t+' C '
    else t := t+'   ';
    if R.New_Qth then t := t+'  Q  '
    else t := t+'     ';
    t := t+LeftPad(Long2Str(R.Point), 2);
    if R.Dupe then t := t+'-DUPE';
    LogLine := t;
  end;

  procedure TRRWW.Header;
  begin
    inherited Header;
    PrintF('Band Date Time Call         RST QTH Zn RST QTH Zn Cntry Cty QTH Pt');
  end;

  procedure TRRWW.Footer;
  begin
    PrintF(CharStr(' ', 56)+'-- --- ---');
    PrintF('Page '+
      Pad(Long2Str(PageCT), 2)+
      CharStr(' ', 50)+
      Pad(Long2Str(Sub_Cty), 4)+
      Pad(Long2Str(Sub_Qth), 3)+
      Long2Str(sub_pts));
  end;

  procedure TRRWW.Detail;
  begin
    if Linect = 0 then Header;
    if a.New_Country then Inc(Sub_Cty);
    if a.New_Qth then Inc(Sub_Qth);
    sub_pts := sub_pts+a.Point;
    PrintF(LogLine(a));
  end;

  procedure TRRWW.PrintSummary;
  var
    Band : BandType;
  begin
    inherited PrintSummary;
    PrintSummaryTop;
    PrintF('Band    QSOs   Pts   QTH    DX');
    for Band := B80 to B10 do
      PrintF(Pad(BandS[Band], 6)+
        LeftPad(Long2Str(Num_Qso[Band]), 6)+
        LeftPad(Long2Str(Num_Points[Band]), 6)+
        LeftPad(Long2Str(Num_Qth[Band]), 6)+
        LeftPad(Long2Str(Num_Country[Band]), 6));
    PrintF(CharStr('-', 32));
    PrintF('Total '+
      LeftPad(Long2Str(Num_Qso[TOT]), 6)+
      LeftPad(Long2Str(Num_Points[TOT]), 6)+
      LeftPad(Long2Str(Num_Qth[TOT]), 6)+
      LeftPad(Long2Str(Num_Country[TOT]), 6));
    PrintSummaryBottom;
  end;

  function  TRRWW.PointValue(const SomeQso : log_rec) : byte;
  begin
    CountryList^.Identify(SomeQso);
    if MyContinent = CountryList^.Ct.Continent
      then PointValue := 5
      else PointValue := 10;
  end;

  procedure TRRWW.RemoveOldMults(var SomeQso : log_rec);
  begin
    RemoveCountryMult(SomeQso);
    RemoveOblastMult(SomeQso);
    RemoveZoneMult(SomeQso);
  end;

  procedure TRRWW.AddNewMults(var SomeQso : log_rec);
  begin
    with SomeQso do begin
      New_Country := CountryList^.Load(SomeQso,Band);
      if (His_Country = 'UA') or
         (His_Country = 'UA2') or
         (His_Country = 'UA9') then New_Qth := RList^.Load(His_Qth, Band);
      ZoneList^.SetZone(His_Zone,Band);
    end;
  end;

  procedure TRRWW.IdAnyMults(const SomeQso : log_rec);
  var
    ix : Byte;
    tc : string[5]; { temp ct.code, it is easier to type in }
    tcn : word;     { temp country num, it is easier to type}
  begin
    if RttyIni^.GetProfileBool('PutMsgs','Profile',True) then
    with SomeQso do begin
      if Dupe then Exit;
      if Pos('/MM', Call) > 0 then Exit;
      tcn := CountryList^.Identify(SomeQso);
      if tcn = NoCountry then Exit;
      tc := CountryList^.Ct.Code;
      if not CountryList^.GetCountry(tcn,Band) then RttyWin^.PutString(^Y'-DX '^Z);
      if (tc = 'UA') or
         (tc = 'UA2') or
         (tc = 'UA9') then begin
        ix := RList^.GetProvNo(His_Qth);
        if (ix > 0) and (not RList^.GetProv(ix,Band)) then RttyWin^.PutString(^Y'-Obl '^Z);
      end;
    end;
  end;

  function TRRWW.ContestId : string;
  begin
    ContestId := 'RRWW';
  end;

  procedure TRRWW.HandleAltU;
  begin
    RList^.Display;
  end;

  procedure TRRWW.DisplayScoreWin;
  var
    PointsPerQso   : Real;
    PointsPerMult  : Real;
    TotMults       : LongInt;
    Band           : BandType;
    ScoreWin       : WindowPtr;
  begin
    ScoreCountPoints;
    Score := ScoreContest;
    if Score > 0 then PointsPerQso := Score / Num_Qso[Tot]
    else PointsPerQso := 0;
    TotMults := Num_Country[TOT]+Num_Qth[TOT];
    If TotMults > 0 then PointsPerMult := Score / TotMults
    else PointsPerMult := 0;
    with Colors^ do begin
    MakeWindow(ScoreWin, MW_xhi-40, MW_Ylo+1, MW_xhi-2, MW_Ylo+11,
                  True, True, True, WindowAttr, FrameAttr, HeaderAttr, 'RR WW Score Summary') ;
    HideMouse;
    DisplayWindow(ScoreWin) ;
    FastWriteWindow('Band    QSOs   Pts   Obl    DX', 1, 1, WindowAttr);
    for Band := B80 to B10 do
      FastWriteWindow(Pad(BandS[Band], 6)+
                                 LeftPad(Long2Str(Num_Qso[Band]), 6)+
                                 LeftPad(Long2Str(Num_Points[Band]), 6)+
                                 LeftPad(Long2Str(Num_Qth[Band]), 6)+
                                 LeftPad(Long2Str(Num_Country[Band]), 6),
                                 2+Ord(Band), 1, WindowAttr);
    FastWriteWindow('----    ----  ----  ----  ----', 7, 1, WindowAttr);
    FastWriteWindow('Total '+
                               LeftPad(Long2Str(Num_Qso[TOT]), 6)+
                               LeftPad(Long2Str(Num_Points[TOT]), 6)+
                               LeftPad(Long2Str(Num_Qth[TOT]), 6)+
                               LeftPad(Long2Str(Num_Country[TOT]), 6),
                               8, 1, WindowAttr);
    FastWriteWindow('Tot: '+Pad(Long2Str(Score),8)+
                      ' P/Q:'+Real2Str(PointsPerQSO, 4, 1)+
                      ' P/M:'+Real2Str(PointsPerMult, 4, 1),
                      9, 1, WindowAttr);
    FastWrite('Press Any Key to continue', 12, 42, WindowAttr);
    end;
    ShowMouse;
    ReadKeyorButton;
    KillWindow(ScoreWin);
  end;

  function TRRWW.ScoreContest : LongInt;
  begin
    ScoreContest := Num_Points[TOT]*(Num_Qth[TOT]+Num_Country[TOT]);
  end;

  procedure TRRWW.ScoreCountPoints;
  var
    Band : BandType;
  begin
    SetCountersToZero;
    for Band := B80 to B10 do begin
      Num_Qth[Band]     := RList^.HowManyProvs(Band);
      Num_Country[Band] := CountryList^.HowMany(Band);
    end;
    LogFile^.ResetLog;
    LogFile^.Get_Next(a);
    while OK do begin
      Inc(Num_Points[a.Band], a.Point);
      Inc(Num_Qso[a.Band]);
      if a.Dupe then Inc(Num_Dupes[a.Band]);
      LogFile^.Get_Next(a);
    end;
    SumCounters;
  end;


  function TRRWW.FieldsPerRecord : byte;
  { returns the number of fields (across) on the screen }
  begin
    FieldsPerRecord := 9;
  end;

  function TRRWW.MultsCol : byte;
  { returns the number of "mults" column on the screen }
  begin
    MultsCol := 6;
  end;

  procedure TRRWW.BuildEditWindow;
  var
    I : Integer;
  begin
    for I := 1 to 5 do begin
      AddChoiceField(EntryEsr, '', I, 2, 'XX', I, 1, 0, 1,
        EditIncChoicePtr, EditW^.EditRecs[I].Band);
      AddTimeField(EntryEsr, '', I, 2, 'hhmm', I, 4, 0,
        MinTime, MaxTime, EditW^.EditRecs[I].Ts);
      AddStringField(EntryEsr, '', I, 2, '!!!!!!!!!!!!', I, 9, 12, 0,
        ValidateCallSignPtr, EditW^.EditRecs[I].Call); { they ALL have callsign first! }
      AddStringField(EntryEsr, '', I, 2, '!!!', I, 22, 3, 0,
        ValidateQTHPtr, EditW^.EditRecs[I].His_Qth);
      SetForceMode(True, True);
      AddByteField(EntryEsr, '', I, 2, '99', I, 26, 0,
        0, 40, EditW^.EditRecs[I].His_Zone);
      SetForceMode(False, False);
      AddByteField(EntryEsr, '', I, 2, '99', I, 29, 0,
        0, 59, EditW^.EditRecs[I].His_Rst);
      SetProtection(True);
      AddStringField(EntryEsr, '',  I, 2, '!!!!!',      I, 32, 5,
        0, nil, EditW^.Mults[I]);
      SetProtection(False);
      AddByteField(EntryEsr, '',    I, 2, '999',        I, 38, 0,
        0, 255, EditW^.EditRecs[I].Point);
      AddYesNoField(EntryEsr, '', I, 2,   '',           I, 42, 0,
        EditW^.EditRecs[I].Dupe);
    end;
    SetFieldLinks(EntryESR, 5,  2,  4);
    ActivateWindow(EditWin);
    FastWriteWindow('Mt Time Callsign     Qth Zn RS Mults Pts Dupe', 1, 1, Colors^.HeaderAttr);
  end;

  function TRRWW.MakeMultStr(const R : log_rec) : String;
  var
    T : String[5];
  begin
    T := '';
    if R.New_Country then T := T+'C'
    else T := T+' ';
    if R.New_Qth then T := T+'Q'
    else T := T+' ';
    MakeMultStr := T;
  end;

  procedure TRRWW.LogContact(var ThisContact : log_rec);
  var
    I : Integer;
  begin
    PreLogContact(ThisContact);
    with ThisContact do begin
      if His_Zone > 40 then His_Zone := 0;
      My_Qth := RttyIni^.GetProfileString('Qth','Profile','');
      My_Zone := RttyIni^.GetProfileByte('CqZone','Profile',0);
      if CountryList^.Identify(ThisContact) <> NoCountry then
        His_Country := CountryList^.Ct.Code;
      if not AdvisorSaysOK(ThisContact) then Exit;
      ZoneList^.SetZone(His_Zone,Band);
      AddNewMults(ThisContact);
      Dupe := DupeTbl^.Load(ThisContact);
      case Dupe of
        True  : Point := 0;
        False : Point := PointValue(ThisContact);
      end;
      if Pos('/MM', Call) > 0 then begin
        New_Country := False;
        New_Qth := False;
        New_Continent := False;
      end;
    end;
    inherited LogContact(ThisContact);
  end;

  procedure TRRWW.RemoveOblastMult(var R : log_rec);
  var
    ix : Byte;
  begin
    with R do begin
      if not New_Qth then Exit;
      if not((His_Country = 'UA') or
             (His_Country = 'UA2') or
             (His_Country = 'UA9')) then Exit;
      New_Qth := False;
      ix := RList^.GetProvNo(His_Qth);
      if ix = 0 then Exit;
      RList^.ResetProv(ix,Band);
    end;
  end;


  function TRRWW.AdvisorSaysOK(const R : log_rec) : Boolean;
  { t = ok to log this qso, false = dont log, they want to edit }
  var
    Missing : String[40];
  begin
    if NOT inherited AdvisorSaysOK(R) then begin { hard edits, no bypass allowed }
      AdvisorSaysOK := False;
      Exit; { hard edits, no bypass allowed }
    end;
    if NOT RttyIni^.GetProfileBool('Advice','PROFILE',True) then begin                     {dont want advisor window }
      AdvisorSaysOK := True;
      Exit;
    end;
    Missing := '';
    if ((Trim(R.His_Country) = 'UA') or
                 (Trim(R.His_Country) = 'UA2') or
                 (Trim(R.His_Country) = 'UA9')) and
               (R.His_Qth = '') then begin
      if Missing <> '' then Missing := Missing+',';
      Missing := Missing + 'Oblast';
    end;
    if R.His_Zone = 0 then begin
      if Missing <> '' then Missing := Missing+',';
      Missing := Missing + 'CQ Zone';
    end;
    AdvisorSaysOK := AdvisorWindow(Missing)
  end;

  procedure TRRWW.Writelog;
  begin
    inherited Writelog;
    RList^.Print;
  end;

  procedure TRRWW.FillRec(const Call : String; var R : log_rec);
    { returns info to R if we has QSO before with CALL         }
    { I also try to fill as much of the QSO record as possible }
  var
    Slot : Word;
  begin
    R.Call := Trim(Call);
    if DupeTbl^.GetDupeByCall(Call, Slot) then begin
      R.His_Qth := DupeTbl^.GetQth;
      if R.His_Zone = 0 then R.His_Zone := ZoneList^.GetZoneNo(R);
    end;
    if CountryList^.Identify(R) = NoCountry then Exit;
    if R.His_Country = 'VE' then R.His_Zone := ProvList^.GetZoneByCall(R.Call);
    if R.His_Zone = 0 then R.His_Zone := ZoneList^.GetZoneNo(R);
  end;



end.
