{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit c_SCCC;

{
  his_serno = year first licensed
  continent = year multipliers
}

interface

uses
  Diskio,
  c_contest,
  Rectypes;

type

  PSCCC = ^TSCCC;
  TSCCC = object(TContest)
            constructor init;
            procedure Header; virtual;
            procedure Footer; virtual;
            procedure Detail; virtual;
            function LogLine(const R : Log_Rec) : String; virtual;
            destructor done; virtual;
            procedure PrintSummary; virtual;
            procedure RemoveOldMults(var SomeQso : log_rec); virtual;
            procedure AddNewMults(var SomeQso : log_rec); virtual;
            procedure IdAnyMults(const SomeQso : log_rec); virtual;
            function ContestId : string; virtual;
            procedure HandleAltU; virtual;
            procedure DisplayScoreWin; virtual;
            function  PointValue(const SomeQso : log_rec) : byte; virtual;
            function  ScoreContest : LongInt; virtual;
            procedure ScoreCountPoints; virtual;
            function  FieldsPerRecord : byte; virtual;
            function MultsCol : byte; virtual;
            procedure BuildEditWindow; virtual;
            function  MakeMultStr(const R : log_rec) : String; virtual;
            procedure LogContact(var ThisContact : log_rec); virtual;
            function  AdvisorSaysOK(const R : log_rec) : Boolean; virtual;
            procedure Writelog; virtual;
            procedure FillRec(const Call : String; var R : log_rec); virtual;
          end;

implementation

uses
  TpEntry,
  cEditWin,
  cClock,
  cContinent,
  cCountry,
  cDupeTbl,
  cYearTbl,
  cDist_W,
  TpDate,
  TpString,
  TpWindow,
  IniFile,
  TpMouse,
  cRttyWin,
  TpCrt,
  TAHigh,
  TAccess,
  LowLvl,
  cColors,
  cLogSCCC,
  DebugRtns,
  Routines;
  { TSCCC.........................................................TSCCC}
  { TSCCC.........................................................TSCCC}
  { TSCCC.........................................................TSCCC}
  { TSCCC.........................................................TSCCC}
  { TSCCC.........................................................TSCCC}

  constructor TSCCC.init;
  begin
    inherited init;
    LogFile := New(PSCCCLogFile, Init);
    MakeCountryList('CQWW.CTY','CQWW.PFX');
    MakeYearList;
    MakeWDistList;
    {$IFDEF DEBUG}
    AppendDebugMessage('SCCC created');
    {$ENDIF}
  end;

  destructor TSCCC.done;
  begin
    inherited done;
    KillWDistList;
    KillYearList;
    KillCountryList;
    {$IFDEF DEBUG}
    AppendDebugMessage('SCCC destroyed');
    {$ENDIF}
  end;

  function TSCCC.LogLine(const R : Log_Rec) : String;
  var
    t : String[127];
  begin
    t := BandS[R.Band]+' '+
      Clock^.Date2DateString(R.Date)+
      TimeToTimeString(' hhmm ', R.Ts)+
      Pad(R.Call, 12)+' '+
      LeftPadCh(Long2Str(R.His_Serno), '0', 4)+' '+
      Long2Str(R.His_RST)+'9 '+
      Pad(R.His_Country, 5)+' ';
    if R.New_Continent then t := t+'  Y  '
    else t := t+'     ';
    t := t+Long2Str(R.Point);
    if R.Dupe then t := t+'-DUPE';
    LogLine := t;
  end;

  procedure TSCCC.Header;
  begin
    inherited Header;
    PrintF('Band Date Time Call         Year RST Cntry Mult Pt');
  end;

  procedure TSCCC.Footer;
  begin
    PrintF(CharStr(' ', 40)+'Page '+Long2Str(PageCT));

  end;

  procedure TSCCC.Detail;
  begin
    if Linect = 0 then Header;
    PrintF(LogLine(a));
  end;

  procedure TSCCC.PrintSummary;
  var
    Band : BandType;
  begin
    inherited PrintSummary;
    PrintSummaryTop;
    PrintF('Band    QSOs   Pts  Years');
    for Band := B80 to B10 do
      PrintF(Pad(BandS[Band], 6)+
        LeftPad(Long2Str(Num_Qso[Band]), 6)+
        LeftPad(Long2Str(Num_Points[Band]), 6)+
        LeftPad(Long2Str(Num_Continents[Band]), 6));
    PrintF('-----   ----  ----  -----');
    PrintF('Total '+
      LeftPad(Long2Str(Num_Qso[TOT]), 6)+
      LeftPad(Long2Str(Num_Points[TOT]), 6)+
      LeftPad(Long2Str(Num_Continents[TOT]), 6));
    PrintSummaryBottom;
  end;

  procedure TSCCC.RemoveOldMults(var SomeQso : log_rec);
  begin
      RemoveYearMult(SomeQSO); {serno = his yr licensed!}
  end;

  procedure TSCCC.AddNewMults(var SomeQso : log_rec);
  begin
    with SomeQso do begin
      New_Continent := YearList^.Load(His_Serno,Band); {serno = his yr licensed!}
    end;
  end;

  procedure TSCCC.IdAnyMults(const SomeQso : log_rec);
  begin
    if RttyIni^.GetProfileBool('PutMsgs','Profile',True) then begin
      if SomeQso.His_Serno < 1900 then Exit;
      if not YearList^.GetYear(SomeQso.His_Serno,SomeQso.Band) then RttyWin^.PutString(^Y'-YR '^Z);
    end;
  end;

  function TSCCC.ContestId : string;
  begin
    ContestId := 'SCCC';
  end;

  procedure TSCCC.HandleAltU;
  begin
    YearList^.Display;
  end;

  procedure TSCCC.DisplayScoreWin;
  var
    PointsPerQso   : Real;
    PointsPerMult  : Real;
    TotMults       : LongInt;
    Band           : BandType;
    ScoreWin       : WindowPtr;
  begin
    ScoreCountPoints;
    Score := ScoreContest;
    if Score > 0 then PointsPerQso := Score / Num_Qso[Tot]
    else PointsPerQso := 0;
    TotMults := Num_Continents[TOT];
    If TotMults > 0 then PointsPerMult := Score / TotMults
    else PointsPerMult := 0;
    with Colors^ do begin
    MakeWindow(ScoreWin, MW_xhi-40, MW_Ylo+1, MW_xhi-2, MW_Ylo+11,
                  True, True, True, WindowAttr, FrameAttr, HeaderAttr, 'SCC RTTY Championship Score Summary') ;
    HideMouse;
    DisplayWindow(ScoreWin) ;
    FastWriteWindow('Band    QSOs   Pts  Mults', 1, 1, WindowAttr);
    for Band := B80 to B10 do
      FastWriteWindow(Pad(BandS[Band], 6)+
                                 LeftPad(Long2Str(Num_Qso[Band]), 6)+
                                 LeftPad(Long2Str(Num_Points[Band]), 6)+
                                 LeftPad(Long2Str(Num_Continents[Band]), 6),
                                 2+Ord(Band), 1, WindowAttr);
    FastWriteWindow('-----   ----  ----  -----', 7, 1, WindowAttr);
    FastWriteWindow('Total '+
                               LeftPad(Long2Str(Num_Qso[TOT]), 6)+
                               LeftPad(Long2Str(Num_Points[TOT]), 6)+
                               LeftPad(Long2Str(Num_Continents[TOT]), 6),
                               8, 1, WindowAttr);
    FastWriteWindow('Tot: '+Pad(Long2Str(Score),8)+
                      ' P/Q:'+Real2Str(PointsPerQSO, 4, 1)+
                      ' P/M:'+Real2Str(PointsPerMult, 4, 1),
                      9, 1, WindowAttr);
    FastWrite('Press Any Key to continue', 12, 42, WindowAttr);
    end;
    ShowMouse;
    ReadKeyorButton;
    KillWindow(ScoreWin);
  end;

  function  TSCCC.PointValue(const SomeQso : log_rec) : byte;

  function LU_Score : byte;
  var
    MyPrefix  : string[4];
    HisPrefix : string[4];
    HisSuffix : string[2];
    MySuffix  : string[2];
  begin
    LU_Score := 2; {assume all are 2 points unless they are in the same region as below}
    CountryList^.ParseCall(RttyIni^.GetProfileString('Call','Profile',''),MyPrefix,MySuffix);
    CountryList^.ParseCall(SomeQso.Call,HisPrefix,HisSuffix);
    if (MySuffix[1] in ['A'..'C']) and (HisSuffix[1] in ['A'..'C']) then LU_Score := 1;
    if (MySuffix[1] in ['D'..'E']) and (HisSuffix[1] in ['D'..'E']) then LU_Score := 1;
    if (MySuffix[1] = 'G') and (HisSuffix[1]='G') then
      if (MySuffix[2] in ['A'..'O']) and (HisSuffix[2] in ['A'..'O']) then LU_Score := 1;
    if (MySuffix[1] = 'G') and (HisSuffix[1]='G') then
      if (MySuffix[2] in ['P'..'Z']) and (HisSuffix[2] in ['P'..'Z']) then LU_Score := 1;

    if (MySuffix[1] = 'X') and (HisSuffix[1]='X') then
      if (MySuffix[2] in ['A'..'O']) and (HisSuffix[2] in ['A'..'O']) then LU_Score := 1;
    if (MySuffix[1] = 'X') and (HisSuffix[1]='X') then
      if (MySuffix[2] in ['P'..'Z']) and (HisSuffix[2] in ['P'..'Z']) then LU_Score := 1;

    if (MySuffix[1] = HisSuffix[1]) and
      (MySuffix[1] in ['F','H'..'W','Y','Z']) then LU_Score := 1;


  end;

  function UA9_Score : byte;
  var
    HisDistrict : byte;
    MyPrefix  : string[4];
    HisPrefix : string[4];
    HisSuffix : string[2];
    MySuffix  : string[2];
  begin
    UA9_Score := 2;
    HisDistrict := WDistList^.GetDistNo(SomeQso.Call);
    CountryList^.ParseCall(RttyIni^.GetProfileString('Call','Profile',''),MyPrefix,MySuffix);
    CountryList^.ParseCall(SomeQso.Call,HisPrefix,HisSuffix);

    { if different districts then 2 points & get out}
    if Mydistrict <> HisDistrict then Exit;

    { ok, same district, check to see if different oblasts}
    case MyDistrict of
      8 : begin
            if (MySuffix[1] = HisSuffix[1]) and
              (MySuffix[1] in ['A'..'D','H'..'K','W'..'Z']) then UA9_Score := 1;
            if (MySuffix[1] in ['E'..'G']) and (HisSuffix[1] in ['E'..'G']) then UA9_Score := 1;
            if (MySuffix[1] in ['L'..'N']) and (HisSuffix[1] in ['L'..'N']) then UA9_Score := 1;
            if (MySuffix[1] in ['O'..'P']) and (HisSuffix[1] in ['O'..'P']) then UA9_Score := 1;
            if (MySuffix[1] in ['Q'..'R']) and (HisSuffix[1] in ['Q'..'R']) then UA9_Score := 1;
            if (MySuffix[1] in ['S'..'T']) and (HisSuffix[1] in ['S'..'T']) then UA9_Score := 1;
            if (MySuffix[1] in ['U'..'V']) and (HisSuffix[1] in ['U'..'V']) then UA9_Score := 1;
          end;
      9 : begin
            if (MySuffix[1] = HisSuffix[1]) and
              (MySuffix[1] in ['A'..'G','J'..'N','W'..'Z']) then UA9_Score := 1;
            if (MySuffix[1] in ['H'..'I']) and (HisSuffix[1] in ['H'..'I']) then UA9_Score := 1;
            if (MySuffix[1] in ['O'..'P']) and (HisSuffix[1] in ['O'..'P']) then UA9_Score := 1;
            if (MySuffix[1] in ['Q'..'R']) and (HisSuffix[1] in ['Q'..'R']) then UA9_Score := 1;
            if (MySuffix[1] in ['S'..'T']) and (HisSuffix[1] in ['S'..'T']) then UA9_Score := 1;
            if (MySuffix[1] in ['U'..'V']) and (HisSuffix[1] in ['U'..'V']) then UA9_Score := 1;
          end;
      0 : begin
            if (MySuffix[1] = HisSuffix[1]) and
              (MySuffix[1] in ['A'..'D','H'..'K','W'..'Z']) then UA9_Score := 1;
            if (MySuffix[1] in ['E'..'G']) and (HisSuffix[1] in ['E'..'G']) then UA9_Score := 1;
            if (MySuffix[1] in ['L'..'N']) and (HisSuffix[1] in ['L'..'N']) then UA9_Score := 1;
            if (MySuffix[1] in ['O'..'P']) and (HisSuffix[1] in ['O'..'P']) then UA9_Score := 1;
            if (MySuffix[1] in ['Q'..'R']) and (HisSuffix[1] in ['Q'..'R']) then UA9_Score := 1;
            if (MySuffix[1] in ['S'..'T']) and (HisSuffix[1] in ['S'..'T']) then UA9_Score := 1;
            if (MySuffix[1] in ['U'..'V']) and (HisSuffix[1] in ['U'..'V']) then UA9_Score := 1;
          end;
    end;


  end;

  begin
    PointValue := LU_Score;
    CountryList^.Identify(SomeQso);
    if MyContinent <> CountryList^.Ct.Continent then begin
      PointValue := 3;
      Exit;
    end;
    if (MyContinent = CountryList^.Ct.Continent) AND
       (MyCountry <> CountryList^.Ct.Code)
    then begin
      PointValue := 2;
      Exit;
    end;


    if MyCountry = CountryList^.Ct.Code then begin
     {check for rule 8.3}
      if (mycountry = 'W') or
         (mycountry = 'VE') or
         (mycountry = 'VK') or
         (mycountry = 'ZL') or
         (mycountry = 'ZS') or
         (mycountry = 'JA') or
         (mycountry = 'PY') then
        if mydistrict = WDistList^.GetDistNo(SomeQso.Call) then begin
          pointValue := 1;
          Exit;
        end
        else begin
          pointValue := 2;
          Exit;
        end
      else
       if (mycountry = 'LU') then PointValue := LU_Score else
       if (mycountry = 'UA9') then PointValue := UA9_Score else
         pointValue := 1;
    end;
  end;

  function TSCCC.ScoreContest : LongInt;
  begin
    ScoreContest := Num_Points[TOT]*Num_Continents[TOT];
  end;

  procedure TSCCC.ScoreCountPoints;
  var
    Band : BandType;
    T : longint;
  begin
    SetCountersToZero;
    LogFile^.ResetLog;
    LogFile^.Get_Next(a);
    while OK do begin
      Inc(Num_Points[a.Band], a.Point);
      Inc(Num_Qso[a.Band]);
      if a.Dupe then Inc(Num_Dupes[a.Band]);
      LogFile^.Get_Next(a);
    end;
    for Band := B80 to B10 do begin { cant use SUmCounters, use this instead (below)}
      Inc(Num_Qso[TOT],Num_Qso[Band]);
      Inc(Num_Points[TOT],Num_Points[Band]);
      Inc(Num_Dupes[TOT],Num_Dupes[Band]);
      Inc(Num_Country[Tot],CountryList^.HowMany(Band));
      T := YearList^.HowManyYears(Band);
      Inc(Num_Continents[Tot],T);
      Num_Continents[Band] := T;
    end;
  end;

  function TSCCC.FieldsPerRecord : byte;
  { returns the number of fields (across) on the screen }
  begin
    FieldsPerRecord := 8;
  end;


  function TSCCC.MultsCol : byte;
  { returns the number of "mults" column on the screen }
  begin
    MultsCol := 5;
  end;

  procedure TSCCC.BuildEditWindow;
  var
    I : Integer;
  begin
    for I := 1 to 5 do begin
      AddChoiceField(EntryEsr, '', I, 2, 'XX', I, 1, 0, 1,
        EditIncChoicePtr, EditW^.EditRecs[I].Band);
      AddTimeField(EntryEsr, '', I, 2, 'hhmm', I, 4, 0,
        MinTime, MaxTime, EditW^.EditRecs[I].Ts);
      AddStringField(EntryEsr, '', I, 2, '!!!!!!!!!!!!', I, 9, 12, 0,
        ValidateCallSignPtr, EditW^.EditRecs[I].Call); { they ALL have callsign first! }
      SetForceMode(True, True);
      AddWordField(EntryEsr, '', I, 2, '9999',          I, 22, 0,
        0, 0, EditW^.EditRecs[I].His_Serno); {year licensed!}
      AddByteField(EntryEsr, '', I, 2, '99',            I, 27, 0,
        0, 59, EditW^.EditRecs[I].His_Rst);
      SetForceMode(False, False);
      SetProtection(True);
      AddStringField(EntryEsr, '',  I, 2, '!!!!!',      I, 30, 5,
        0, nil, EditW^.Mults[I]);
      SetProtection(False);
      AddByteField(EntryEsr, '',    I, 2, '999',        I, 36, 0,
        0, 255, EditW^.EditRecs[I].Point);
      AddYesNoField(EntryEsr, '', I, 2,   '',           I, 40, 0,
        EditW^.EditRecs[I].Dupe);
    end;
    SetFieldLinks(EntryESR, 4, 2,  3);
    ActivateWindow(EditWin);
    FastWriteWindow('Mt Time Callsign     Year RS Mults Pts Dupe', 1, 1, Colors^.HeaderAttr);
  end;

  function TSCCC.MakeMultStr(const R : log_rec) : String;
  begin
    if R.New_Continent then MakeMultStr := 'Y'
                       else MakeMultStr := '';
  end;


  procedure TSCCC.LogContact(var ThisContact : log_rec);
  begin
    PreLogContact(ThisContact);
    with ThisContact do begin
      if CountryList^.Identify(ThisContact) <> NoCountry then
        His_Country := CountryList^.Ct.Code;
      if not AdvisorSaysOK(ThisContact) then Exit;
      Dupe := DupeTbl^.Load(ThisContact);
      AddNewMults(ThisContact);
      case Dupe of
        True  : Point := 0;
        False : Point := PointValue(ThisContact);
      end;
      if Pos('/MM', Call) > 0 then begin
        New_Country := False;
        New_Qth := False;
        New_Continent := False;
      end;
    end;
    inherited LogContact(ThisContact);
  end;

  function TSCCC.AdvisorSaysOK(const R : log_rec) : Boolean;
  { t = ok to log this qso, false = dont log, they want to edit }
  var
    Missing : String[40];
  begin
    if NOT inherited AdvisorSaysOK(R) then begin { hard edits, no bypass allowed }
      AdvisorSaysOK := False;
      Exit; { hard edits, no bypass allowed }
    end;
    if NOT RttyIni^.GetProfileBool('Advice','PROFILE',True) then begin                     {dont want advisor window }
      AdvisorSaysOK := True;
      Exit;
    end;
    Missing := '';
    if (R.His_Serno < 1900) or (R.His_Serno > 2155) then Missing := 'Year Licensed';
    AdvisorSaysOK := AdvisorWindow(Missing)
  end;

  procedure TSCCC.Writelog;
  begin
    inherited Writelog;
    YearList^.Print;
  end;

  procedure TSCCC.FillRec(const Call : String; var R : log_rec);
    { returns info to R if we has QSO before with CALL         }
    { I also try to fill as much of the QSO record as possible }
  var
    Slot : Word;
  begin
    R.Call := Trim(Call);
    if DupeTbl^.GetDupeByCall(Call, Slot) then begin
      R.His_Serno := DupeTbl^.GetYear;
    end;
    if CountryList^.Identify(R) <> NoCountry then
      R.His_Country := CountryList^.Ct.Code;
  end;





end.
