{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit c_spri;

interface

uses
  c_contest,
  Diskio,
  Rectypes;
type
  PSPRI = ^TSPRI;
  TSPRI = object(TContest)
            constructor init;
            procedure Header; virtual;
            procedure Detail; virtual;
            function LogLine(const R : Log_Rec) : String; virtual;
            destructor done; virtual;
            procedure PrintSummary; virtual;
            procedure PrintLogs; virtual;
            function  QsoIsGood(const ThisQso : log_rec) : Boolean; virtual;
            function  PointValue(const SomeQso : log_rec) : byte; virtual;
            procedure RemoveOldMults(var SomeQso : log_rec); virtual;
            procedure AddNewMults(var SomeQso : log_rec); virtual;
            procedure IdAnyMults(const SomeQso : log_rec); virtual;
            function  ContestId : string; virtual;
            procedure DisplayScoreWin; virtual;
            function  ScoreContest : LongInt; virtual;
            procedure ScoreCountPoints; virtual;
            function  FieldsPerRecord : byte; virtual;
            function  MultsCol : byte; virtual;
            procedure BuildEditWindow; virtual;
            procedure LogContact(var ThisContact : log_rec); virtual;
            function  AdvisorSaysOK(const R : log_rec) : Boolean; virtual;
            procedure FillRec(const Call : String; var R : log_rec); virtual;
          end;

implementation

uses
  TpEntry,
  cEditWin,
  IniFile,
  TpDate,
  cCOlors,
  TpString,
  TAHigh,
  TAccess,
  LowLvl,
  NaqpName,
  TpWindow,
  TpCrt,
  cDupeTbl,
  cCountry,
  cLogSpri,
  TpMouse,
  DebugRtns,
  Routines;


  { TSPRI.........................................................TSPRI}
  { TSPRI.........................................................TSPRI}
  { TSPRI.........................................................TSPRI}
  { TSPRI.........................................................TSPRI}
  { TSPRI.........................................................TSPRI}

  constructor TSPRI.init;
  begin
    NameList := NIL;
    inherited init;
    LogFile := New(PSPRILogFile, Init);
    NameList := New(PNameList, Init);
    if not Assigned(NameList) then
      MessageBox('SEVERE ERROR',
                 'Unable to create name list',
                 'Please re-install, or contact WF1B',
                 '');
    MakeCountryList('ARRL.CTY','ARRL.PFX');
    {$IFDEF DEBUG}
    AppendDebugMessage('SPRI created');
    {$ENDIF}

 end;

  destructor TSPRI.done;
  begin
    if Assigned(NameList) then Dispose(NameList, Done);
    NameList := NIL;
    KillCountryList;
    inherited done;
    {$IFDEF DEBUG}
    AppendDebugMessage('SPRI destroyed');
    {$ENDIF}
  end;

  function TSPRI.LogLine(const R : Log_Rec) : String;
  var
    t : String[127];
  begin
    t := BandS[R.Band]+' '+
      TimeToTimeString(' hhmm ', R.Ts)+
      Pad(R.Call, 12)+' '+
      LeftPadCh(Long2Str(R.My_Serno), '0', 4)+' '+
      LeftPadCh(Long2Str(R.His_Serno), '0', 4)+' '+
      Pad(Copy(R.Comment, 1, 10), 11)+
      Pad(R.His_Qth, 4)+
      Long2Str(R.Point);
    if R.Dupe then t := t+' INVALID';
    LogLine := t;
  end;

  procedure TSPRI.PrintLogs;
  begin
    AllBands;
  end;

  procedure TSPRI.Header;
  begin
    inherited Header;
    PrintF('Bd Time Call         Ser# Ser# Name      Qth Pt');
  end;

  procedure TSPRI.Detail;
  begin
    if Linect = 0 then Header;
    PrintF(LogLine(a));
  end;

  procedure TSPRI.PrintSummary;
  var
    Band : BandType;
  begin
    inherited PrintSummary;
    PrintSummaryTop;
    PrintF('Band    QSOs   Pts');
    for Band := B80 to B10 do
      PrintF(Pad(BandS[Band], 6)+
        LeftPad(Long2Str(Num_Qso[Band]), 6)+
        LeftPad(Long2Str(Num_Points[Band]), 6));
    PrintF(CharStr('-', 32));
    PrintF('Total '+
      LeftPad(Long2Str(Num_Qso[TOT]), 6)+
      LeftPad(Long2Str(Num_Points[TOT]), 6));
    PrintSummaryBottom;
  end;


  function TSPRI.QsoIsGood(const ThisQso : Log_Rec) : Boolean;
  { loading = true if this is during startup procedures }

  function IsGoodForLoading : Boolean;
  var
    StopLooking : boolean;
    IsGood : boolean;
    R : log_rec;
    Ct : byte;
    ValidQsos : byte;
    StopCt : byte;
    Holdrecnum : longint;
  begin
    if TARecNum <= 1 then begin { first record is always good }
      IsGoodForLoading := True;
      Exit;
    End;
    HoldRecNum := TaRecNum;
    StopLooking := False;
    IsGood := True;
    Ct := 0;
    ValidQsos := 0;
    if HoldRecNum > 10 then StopCt := 10
                       else StopCt := HoldRecNum-1;
    while (not StopLooking) and (Ct <= StopCt) do begin
      LogFile^.Get_Prev(R); { get previous record }
      inc(ct);
      if (not R.Dupe) and (R.Call <> ThisQso.Call) then inc(ValidQsos);
      if (R.Call = ThisQso.Call) and (not R.Dupe) then StopLooking := True;
      if ValidQsos >= 3 then StopLooking := True;
    end;
    IsGood := ValidQsos >= 3;
    {if i am still checking the first 4 records, special logic is needed
     since it is not possible to have 3 valid qsos for qso #2!!}
    if (not IsGood) and { only override if false }
       (R.Call <> ThisQso.Call) and { and the last record checked was not a "hit"}
       (HoldRecNum <= 4) then { and if the current record number is 4 or less }
       IsGood := True; {THEN we can say it is ok}
    GetRec(LogFile^.TpDataSet.Data, HoldrecNum, R);
    LogFile^.Index := LogFile^.MakeIx(R);
    TARead(LogFile^.TpDataSet,R,LogFile^.Index,True);
    IsGoodForLoading := IsGood;
  end;

  function IsGoodForRunTime : Boolean;
  var
    StopLooking : boolean;
    IsGood : boolean;
    R : log_rec;
    Ct : byte;
    ValidQsos : byte;
    StopCt : byte;
    Holdrecnum : longint;
  begin
    StopLooking := False;
    IsGood := True;
    Ct := 0;
    ValidQsos := 0;
    LogFile^.ResetLog; { bottom of log }
    if LogFile^.NumRecs > 10 then StopCt := 10
                    else StopCt := LogFile^.NumRecs-1;
    if LogFile^.NumRecs <= 1 then begin
      IsGoodForRunTime := True;
      Exit
    end;
    while (not StopLooking) and (Ct <= StopCt) do begin
      LogFile^.Get_Prev(R); { get previous record }
      inc(ct);
      if (not R.Dupe) and (R.Call <> ThisQso.Call) then inc(ValidQsos);
      if (R.Call = ThisQso.Call) and (not R.Dupe) then StopLooking := True;
      if ValidQsos >= 3 then StopLooking := True;
    end;
    IsGood := ValidQsos >= 3;
    {so few records, if they werent a hit then its ok}
    if (not IsGood) and (R.Call <> ThisQso.Call) and (LogFile^.NumRecs <= 4) then IsGood := True;
    LogFile^.ResetLog;
    IsGoodForRunTime := IsGood;
  end;

  begin
    Case Loading of
      True : QsoIsGood := IsGoodForLoading;
      False: QsoIsGood := IsGoodForRunTime;
    end;
  end;

  function  TSPRI.PointValue(const SomeQso : log_rec) : byte;
  begin
    if QsoIsGood(SomeQso)
      then PointValue := 1
      else PointValue := 0;
  end;

  procedure TSPRI.RemoveOldMults(var SomeQso : log_rec);
  begin
    RemoveCountryMult(SomeQso);
    RemoveStateMult(SomeQso);
    RemoveVeProvinceMult(SomeQso);
    RemoveZoneMult(SomeQso);
  end;

  procedure TSPRI.AddNewMults(var SomeQso : log_rec);
  begin
    with SomeQso do begin
      { no multipliers in this contest}
      NameList^.SetName(Call, Comment);
    end;
  end;

  procedure TSPRI.IdAnyMults(const SomeQso : log_rec);
  begin
     { no multipliers in this contest}
  end;

  function TSPRI.ContestId : string;
  begin
    ContestId := 'SPRI';
  end;

  procedure TSPRI.DisplayScoreWin;
  var
    PointsPerQso   : Real;
    PointsPerMult  : Real;
    TotMults       : LongInt;
    Band           : BandType;
    ScoreWin       : WindowPtr;
  begin
    ScoreCountPoints;
    Score := ScoreContest;
    if Score > 0 then PointsPerQso := Score / Num_Qso[Tot]
    else PointsPerQso := 0;
    TotMults := Num_Country[TOT]+Num_Qth[TOT];
    If TotMults > 0 then PointsPerMult := Score / TotMults
    else PointsPerMult := 0;
    with Colors^ do begin
    MakeWindow(ScoreWin, MW_xhi-40, MW_Ylo+1, MW_xhi-2, MW_Ylo+11,
                  True, True, True, WindowAttr, FrameAttr, HeaderAttr, 'SPRINT Score Summary') ;
    HideMouse;
    DisplayWindow(ScoreWin) ;
    {}
    FastWriteWindow('Band    QSOs   Pts', 1, 1, WindowAttr);
    for Band := B80 to B10 do
      FastWriteWindow(Pad(BandS[Band], 6)+
                                 LeftPad(Long2Str(Num_Qso[Band]), 6)+
                                 LeftPad(Long2Str(Num_Points[Band]), 6),
                                 2+Ord(Band), 1, WindowAttr);
    FastWriteWindow('----    ----  ----', 7, 1, WindowAttr);
    FastWriteWindow('Total '+
                               LeftPad(Long2Str(Num_Qso[TOT]), 6)+
                               LeftPad(Long2Str(Num_Points[TOT]), 6),
                               8, 1, WindowAttr);
    {}
    FastWriteWindow('Tot: '+Long2Str(Score), 9, 1, WindowAttr);
    FastWrite('Press Any Key to continue', 12, 42, WindowAttr);
    end;
    ShowMouse;
    ReadKeyorButton;
    KillWindow(ScoreWin);
  end;

  function TSPRI.ScoreContest : LongInt;
  begin
    ScoreContest := Num_Points[TOT];
  end;

  procedure TSPRI.ScoreCountPoints;
  var
    Band : BandType;
  begin
    SetCountersToZero;
    LogFile^.ResetLog;
    LogFile^.Get_Next(a);
    while OK do begin
      Inc(Num_Points[a.Band], a.Point);
      Inc(Num_Qso[a.Band]);
      if a.Dupe then Inc(Num_Dupes[a.Band]);
      LogFile^.Get_Next(a);
    end;
    SumCounters;
  end;


  function TSPRI.FieldsPerRecord : byte;
  { returns the number of fields (across) on the screen }
  begin
    FieldsPerRecord := 9;
  end;

  function TSPRI.MultsCol : byte;
  { returns the number of "mults" column on the screen }
  begin
    MultsCol := 0;
  end;

  procedure TSPRI.BuildEditWindow;
  var
    I : Integer;
  begin
    for I := 1 to 5 do begin
      AddChoiceField(EntryEsr, '', I, 2, 'XX', I, 1, 0, 1,
        EditIncChoicePtr, EditW^.EditRecs[I].Band);
      AddTimeField(EntryEsr, '', I, 2, 'hhmm', I, 4, 0,
        MinTime, MaxTime, EditW^.EditRecs[I].Ts);
      AddStringField(EntryEsr, '', I, 2, '!!!!!!!!!!!!', I, 9, 12, 0,
        ValidateCallSignPtr, EditW^.EditRecs[I].Call); { they ALL have callsign first! }
      SetForceMode(True, True);
      AddWordField(EntryEsr, '',    I, 2, '9999',       I, 22, 0,
        0, 9999, EditW^.EditRecs[I].His_Serno);
      SetForceMode(False, False);
      AddStringField(EntryEsr, '',  I, 2, '!!!!!!!!!!', I, 27, 10,
        0, nil, EditW^.EditRecs[I].Comment);
      AddStringField(EntryEsr, '',  I, 2, '!!!',        I, 38, 3,
        0, ValidateQTHptr, EditW^.EditRecs[I].His_Qth);
      SetForceMode(True, True);
      AddWordField(EntryEsr, '',    I, 2, '9999',       I, 42, 0,
        0, 9999, EditW^.EditRecs[I].My_Serno);
      SetForceMode(False, False);
      AddByteField(EntryEsr, '',    I, 2, '999',        I, 47, 0,
        0, 255, EditW^.EditRecs[I].Point);
      AddYesNoField(EntryEsr, '',   I, 2, '',           I, 51, 0,
        EditW^.EditRecs[I].Dupe);
    end;
    SetFieldLinks(EntryESR, 5,  2,  4);
    ActivateWindow(EditWin);
    FastWriteWindow('Mt Time Callsign     Ser# Name       Qth Ser# Pts Invalid', 1, 1, Colors^.HeaderAttr);
  end;

  procedure TSPRI.LogContact(var ThisContact : log_rec);
  begin
    PreLogContact(ThisContact);
    with ThisContact do begin
      Call := Trim(Call);
      My_Qth := RttyIni^.GetProfileString('Qth','Profile','');
      if not AdvisorSaysOK(ThisContact) then Exit;
      DupeTbl^.Load(ThisContact); { just put the data in the table, ignore results}
      Dupe := NOT QsoIsGood(ThisContact); { whether it is good or not}
      if Assigned(NameList) then NameList^.SetName(Call, Comment);
      case Dupe of
        True  : Point := 0;
        False : Point := PointValue(ThisContact);
      end;
      if Pos('/MM', Call) > 0 then begin
        New_Country := False;
        New_Qth := False;
        New_Continent := False;
      end;
    end;
    inherited LogContact(ThisContact);
  end;                            {LogContact}

  function TSPRI.AdvisorSaysOK(const R : log_rec) : Boolean;
  { t = ok to log this qso, false = dont log, they want to edit }
  var
    Missing : String[40];
  begin
    if NOT inherited AdvisorSaysOK(R) then begin { hard edits, no bypass allowed }
      AdvisorSaysOK := False;
      Exit; { hard edits, no bypass allowed }
    end;
    if NOT RttyIni^.GetProfileBool('Advice','PROFILE',True) then begin                     {dont want advisor window }
      AdvisorSaysOK := True;
      Exit;
    end;
    Missing := '';
    if R.His_Qth = '' then begin
      if Missing <> '' then Missing := Missing+',';
      Missing := Missing + 'QTH';
    end;
    if Trim(R.Comment) = '' then begin
      if Missing <> '' then Missing := Missing+',';
      Missing := Missing + 'Name';
    end;
    if R.His_Serno = 0 then begin
      if Missing <> '' then Missing := Missing+',';
      Missing := Missing + 'Serial #';
    end;
    AdvisorSaysOK := AdvisorWindow(Missing)
  end;

  procedure TSPRI.FillRec(const Call : String; var R : log_rec);
    { returns info to R if we has QSO before with CALL         }
    { I also try to fill as much of the QSO record as possible }
  var
    Slot : Word;
  begin
    R.Call := Trim(Call);
    if DupeTbl^.GetDupeByCall(Call, Slot) then begin
      R.His_Qth := DupeTbl^.GetQth;
      R.Comment := NameList^.GetName(R.Call);
    end;
    if CountryList^.Identify(R) <> NoCountry then
      R.His_Country := CountryList^.Ct.Code;
  end;


end.
