{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit c_volt;

interface

uses
  c_contest,
  Diskio,
  Rectypes;
type

  PVOLT = ^TVOLT;
  TVOLT = object(TContest)
            constructor init;
            procedure Header; virtual;
            procedure Footer; virtual;
            procedure Detail; virtual;
            function LogLine(const R : Log_Rec) : String; virtual;
            destructor done; virtual;
            procedure PrintSummary; virtual;
            function  PointValue(const SomeQso : log_rec) : byte; virtual;
            procedure RemoveOldMults(var SomeQso : log_rec); virtual;
            procedure AddNewMults(var SomeQso : log_rec); virtual;
            procedure IdAnyMults(const SomeQso : log_rec); virtual;
            function ContestId : string; virtual;
            procedure HandleAltU; virtual;
            procedure DisplayScoreWin; virtual;
            function  ScoreContest : LongInt; virtual;
            procedure ScoreCountPoints; virtual;
            function  FieldsPerRecord : byte; virtual;
            function MultsCol : byte; virtual;
            procedure BuildEditWindow; virtual;
            function  MakeMultStr(const R : log_rec) : String; virtual;
            procedure LogContact(var ThisContact : log_rec); virtual;
            function  AdvisorSaysOK(const R : log_rec) : Boolean; virtual;
            procedure Writelog; virtual;
            procedure FillRec(const Call : String; var R : log_rec); virtual;
          end;

implementation

uses
  TpEntry,
  cEditWin,
  cClock,
  cRttyWin,
  cDupeTbl,
  cProvince,
  TpDate,
  TpString,
  TpWindow,
  TpMouse,
  TpCrt,
  cColors,
  TAHigh,
  IniFile,
  TAccess,
  LowLvl,
  cDist_W,
  cDist_VE,
  DebugRtns,
  cDist_VK,
  cDist_JA,
  cDist_ZL,
  cCountry,
  cZone,
  cLogVolt,
  Routines;

  { TVOLT.........................................................TVOLT}
  { TVOLT.........................................................TVOLT}
  { TVOLT.........................................................TVOLT}
  { TVOLT.........................................................TVOLT}
  { TVOLT.........................................................TVOLT}

  constructor TVOLT.init;
  begin
    inherited init;
    LogFile := New(PVOLTLogFile, Init);
    MakeZoneList;
    MakeProvList('ARRL');
    MakeWDistList;
    MakeVEDistList;
    MakeVKDistList;
    MakeJADistList;
    MakeZLDistList;
    MakeCountryList('ARRL.CTY','ARRL.PFX');
    {$IFDEF DEBUG}
    AppendDebugMessage('VOLT created');
    {$ENDIF}
  end;

  destructor TVOLT.done;
  begin
    inherited done;
    KillZoneList;
    KillPRovList;
    KillWDistList;
    KillVEDistList;
    KillVKDistList;
    KillJADistList;
    KillZLDistList;
    KillCountryList;
    {$IFDEF DEBUG}
    AppendDebugMessage('VOLT destroyed');
    {$ENDIF}
  end;

  function TVOLT.LogLine(const R : Log_Rec) : String;
  var
    t : String[127];
  begin
    t := BandS[R.Band]+'  '+
      Clock^.Date2DateString(R.Date)+
      TimeToTimeString(' hhmm ', R.Ts)+
      Pad(R.Call, 12)+' '+
      LeftPadCh(Long2Str(R.My_Serno), '0', 4)+' '+
      Long2Str(R.My_Rst)+'9 '+
      LeftPadCh(Long2Str(R.My_Zone), '0', 2)+' '+
      LeftPadCh(Long2Str(R.His_Serno), '0', 4)+' '+
      Long2Str(R.His_RST)+'9 '+
      LeftPadCh(Long2Str(R.His_Zone), '0', 2)+' '+
      Pad(R.His_Country, 5)+' ';
    if R.New_Country or R.New_Qth then begin
      if R.His_Country = 'W' then t := t+'W'+Long2Str(WDistList^.GetDistNo(R.Call))+'    ' else
        if R.His_Country = 'VE' then t := t+'VE'+Long2Str(VEDistList^.GetDistNo(R.Call))+'   ' else
          if R.His_Country = 'VK' then t := t+'VK'+Long2Str(VKDistList^.GetDistNo(R.Call))+'   ' else
            if R.His_Country = 'JA' then t := t+'JA'+Long2Str(JADistList^.GetDistNo(R.Call))+'   ' else
              if R.His_Country = 'ZL' then t := t+'ZL'+Long2Str(ZLDistList^.GetDistNo(R.Call))+'   ' else
                t := t+Pad(R.His_Country, 5)+' ';
    end
    else t := t+'      ';
    t := t+LeftPad(Long2Str(R.Point), 3);
    if R.Dupe then t := t+'-DUPE';
    LogLine := t;
  end;

  procedure TVOLT.Header;
  begin
    inherited Header;
    PrintF('Band Date  Time Call         QSO# RST Zn QSO# RST Zn Cntry Mults  Pt');
  end;

  procedure TVOLT.Footer;
  begin
    PrintF('Page '+Pad(Long2Str(PageCT), 2)+
      '  SubTotal-QSO = '+Long2Str(Sub_QSO)+
      '  SubTotal-Dist = '+Long2Str(Sub_Qth)+
      '  SubTotal-Cty = '+Long2Str(Sub_Cty));
  end;

  procedure TVOLT.Detail;
  begin
    if Linect = 0 then Header;
    if a.New_Qth then Inc(Sub_Qth);
    if a.New_Country then Inc(Sub_Cty);
    if not a.Dupe then Inc(Sub_QSO);
    PrintF(LogLine(a));           {v2.5a}
  end;

  procedure TVOLT.PrintSummary;
  var
    Band : BandType;
  begin
    inherited PrintSummary;
    PrintSummaryTop;
    PrintF('Band    QSOs      Pts Mults');
    for Band := B80 to B10 do
      PrintF(Pad(BandS[Band], 6)+
        LeftPad(Long2Str(Num_Qso[Band]), 6)+
        LeftPad(Long2Str(Num_Points[Band]), 9)+
        LeftPad(Long2Str(Num_Districts[Band]+Num_Country[Band]), 6));
    PrintF('___   ______ ________ _____');
    PrintF('Total '+
      LeftPad(Long2Str(Num_Qso[TOT]), 6)+
      LeftPad(Long2Str(Num_Points[TOT]), 9)+
      LeftPad(Long2Str(Num_Districts[TOT]+Num_Country[TOT]), 6));
    PrintF('');
    PrintF('* - Total QSOs does NOT include dupes');
    PrintF('');
    PrintF('Intercontinent DX on 4 bands: '+Long2Str(CountryList^.NumberOfQuadMults));
    PrintSummaryBottom;
  end;

  function  TVOLT.PointValue(const SomeQso : log_rec) : byte;
  var
    CPV : Byte;
  begin
    CountryList^.Identify(SomeQso);
    CPV := 0;
    if (SomeQso.His_Zone in [1..40]) and (SomeQso.My_Zone in [1..40]) then
      CPV := ExchangePointTable[SomeQso.My_Zone, SomeQso.His_Zone];
    if ((MyCountry = 'W') or
      (MyCountry = 'VE') or
      (MyCountry = 'VK') or
      (MyCountry = 'JA') or
      (MyCountry = 'ZL')
      ) and
      (SomeQso.His_Country = MyCountry) and
      (WDistList^.GetDistNo(SomeQso.Call) = WDistList^.GetDistNo(RttyIni^.GetProfileString('Call','Profile','')))
      then CPV := 0;
    if (SomeQso.Band in [B80, B10]) and
      (CountryList^.CtyNum <> NoCountry) and
      (MyContinent <> CountryList^.Ct.Continent) then CPV := CPV*2;
    PointValue := CPV;
  end;

  procedure TVOLT.RemoveOldMults(var SomeQso : log_rec);
  begin
    RemoveCountryMult(SomeQso);
    RemoveDistMult(SomeQso);
  end;

  procedure TVOLT.AddNewMults(var SomeQso : log_rec);
  begin
    with SomeQso do begin
      { according to the rules, you have to track them for the ICD mults,
        but you cannot count them as a regular country mult.  So the code
        below loads it into the county table, but marks the QSO as not a
        multiplier.
      }
      New_Country := CountryList^.Load(SomeQso,Band);
      if (CountryList^.Ct.Code = 'W') or
         (CountryList^.Ct.Code = 'VE') or
         (CountryList^.Ct.Code = 'ZL') or
         (CountryList^.Ct.Code = 'VK') or
         (CountryList^.Ct.Code = 'JA') or
         (CountryList^.Ct.Code = MyCountry)
        then New_Country := False;
      if His_Country = 'W' then
        New_Qth := WDistList^.Load(Call,Band);
      if His_Country = 'VE' then
        New_Qth := VEDistList^.Load(Call,Band);
      if His_Country = 'VK' then
        New_Qth := VKDistList^.Load(Call,Band);
      if His_Country = 'JA' then
        New_Qth := JADistList^.Load(Call,Band);
      if His_Country = 'ZL' then
        New_Qth := ZLDistList^.Load(Call,Band);
    end;
  end;

  procedure TVOLT.IdAnyMults(const SomeQso : log_rec);
  var
    ix : Byte;
    Dist : Byte;
    tc : string[5]; { temp ct.code, it is easier to type in }
    tcn : word;     { temp country num, it is easier to type}
  begin
    if RttyIni^.GetProfileBool('PutMsgs','Profile',True) then
    with SomeQso do begin
      if Dupe then Exit;
      if Pos('/MM', Call) > 0 then Exit;
      tcn := CountryList^.Identify(SomeQso);
      if tcn = NoCountry then Exit;
      tc := CountryList^.Ct.Code;
      if (not CountryList^.GetCountry(tcn,Band)) and (tc <> 'W') and
         (tc <> 'VE') and (tc <> 'VK')
         then RttyWin^.PutString(^Y'-DX '^Z);
      if CountryList^.CompletesQuadMult(tcn, Band) then RttyWin^.PutString(^Y'-IC'^Z);
      Dist := WDistList^.GetDistNo(Call);
      if Dist < 10 then begin
        if (tc = 'W') and (not WDistList^.GetDist(Dist,Band)) then RttyWin^.PutString(^Y'-DIST '^Z);
        if (tc = 'VE') and (not VEDistList^.GetDist(Dist,Band)) then RttyWin^.PutString(^Y'-DIST '^Z);
        if (tc = 'VK') and (not VKDistList^.GetDist(Dist,Band)) then RttyWin^.PutString(^Y'-DIST '^Z);
        if (tc = 'JA') and (not JADistList^.GetDist(Dist,Band)) then RttyWin^.PutString(^Y'-DIST '^Z);
        if (tc = 'ZL') and (Dist in [1..4]) and (not ZLDistList^.GetDist(Dist,Band)) then
          RttyWin^.PutString(^Y'-DIST '^Z);
      end;
    end;
  end;

  function TVOLT.ContestId : string;
  begin
    ContestId := 'VOLT';
  end;

  procedure TVOLT.HandleAltU;
  begin
    WDistList^.Display;
  end;

  procedure TVOLT.DisplayScoreWin;
  var
    PointsPerQso   : Real;
    PointsPerMult  : Real;
    TotMults       : LongInt;
    Band           : BandType;
    ScoreWin       : WindowPtr;
  begin
    ScoreCountPoints;
    Score := ScoreContest;
    if Score > 0 then PointsPerQso := Score / Num_Qso[Tot]
    else PointsPerQso := 0;
    TotMults := Num_Country[TOT]+Num_Qth[TOT];
    If TotMults > 0 then PointsPerMult := Score / TotMults
    else PointsPerMult := 0;
    with Colors^ do begin
    MakeWindow(ScoreWin, MW_xhi-40, MW_Ylo+1, MW_xhi-2, MW_Ylo+11,
                  True, True, True, WindowAttr, FrameAttr, HeaderAttr, 'VOLTA Score Summary') ;
    HideMouse;
    DisplayWindow(ScoreWin) ;
    FastWriteWindow('Band    QSOs   Pts  Dist    DX', 1, 1, WindowAttr);
    for Band := B80 to B10 do
      FastWriteWindow(pad(BandS[Band], 6)+
                                 LeftPad(Long2Str(Num_Qso[Band]), 6)+
                                 LeftPad(Long2Str(Num_Points[Band]), 6)+
                                 LeftPad(Long2Str(Num_Districts[Band]), 6)+
                                 LeftPad(Long2Str(Num_Country[Band]), 6),
                                 2+Ord(Band), 1, WindowAttr);
    FastWriteWindow('-----   ----  ----  ----  ----', 7, 1, WindowAttr);
    FastWriteWindow('Total '+
                               LeftPad(Long2Str(Num_Qso[TOT]), 6)+
                               LeftPad(Long2Str(Num_Points[TOT]), 6)+
                               LeftPad(Long2Str(Num_Districts[TOT]), 6)+
                               LeftPad(Long2Str(Num_Country[TOT]), 6),
                               8, 1, WindowAttr);
    FastWriteWindow('Tot: '+Pad(Long2Str(Score),8)+
                      ' P/Q:'+Real2Str(PointsPerQSO, 4, 1)+
                      ' P/M:'+Real2Str(PointsPerMult, 4, 1),
                      9, 1, WindowAttr);
    FastWrite('Press Any Key to continue', 12, 42, WindowAttr);
    end;
    ShowMouse;
    ReadKeyorButton;
    KillWindow(ScoreWin);
  end;

  function TVOLT.ScoreContest : LongInt;
  begin
    ScoreContest := Num_Points[TOT]*(Num_Districts[TOT]+Num_Country[TOT]+CountryList^.NumberOfQuadMults)*Num_Qso[TOT];
  end;

  procedure TVOLT.ScoreCountPoints;
  var
    Band : BandType;
    Country : Word;
  begin
    SetCountersToZero;
    for Band := B80 to B10 do begin
      for Country := 0 to CountryList^.NumCountries-1 do begin
        with CountryList^ do begin
          GetCtRec(Country);
          if (Ct.Code = 'W') or
             (Ct.Code = 'VE') or
             (Ct.Code = 'JA') or
             (Ct.Code = 'ZL') or
             (Ct.Code = 'VK')
          then         {}
          else
            if GetCountry(Country,Band) then Inc(Num_Country[Band]);
        end;
      end;
      Num_Districts[Band] := WDistList^.HowMany(Band)+
                             VEDistList^.HowMany(Band)+
                             VKDistList^.HowMany(Band)+
                             JADistList^.HowMany(Band)+
                             ZLDistList^.HowMany(Band);

    end;
    LogFile^.ResetLog;
    LogFile^.Get_Next(a);
    while OK do begin
      Inc(Num_Points[a.Band], a.Point);
      Inc(Num_Qso[a.Band]);
      if a.Dupe then Inc(Num_Dupes[a.Band]);
      LogFile^.Get_Next(a);
    end;
    SumCounters;
  end;


  function TVOLT.FieldsPerRecord : byte;
  { returns the number of fields (across) on the screen }
  begin
    FieldsPerRecord := 10;
  end;


  function TVOLT.MultsCol : byte;
  { returns the number of "mults" column on the screen }
  begin
    MultsCol := 7;
  end;

  procedure TVOLT.BuildEditWindow;
  var
    I : Integer;
  begin
    for I := 1 to 5 do begin
      AddChoiceField(EntryEsr, '', I, 2, 'XX', I, 1, 0, 1,
        EditIncChoicePtr, EditW^.EditRecs[I].Band);
      AddTimeField(EntryEsr, '', I, 2, 'hhmm', I, 4, 0,
        MinTime, MaxTime, EditW^.EditRecs[I].Ts);
      AddStringField(EntryEsr, '', I, 2, '!!!!!!!!!!!!', I, 9, 12, 0,
        ValidateCallSignPtr, EditW^.EditRecs[I].Call); { they ALL have callsign first! }
      SetForceMode(True, True);
      AddByteField(EntryEsr, '', I, 2, '99', I, 22, 0,
        0, 40, EditW^.EditRecs[I].His_Zone);
      AddWordField(EntryEsr, '', I, 2, '9999', I, 25, 0,
        0, 9999, EditW^.EditRecs[I].His_Serno);
      SetForceMode(False, False);
      AddByteField(EntryEsr, '', I, 2, '99', I, 30, 0,
        0, 59, EditW^.EditRecs[I].His_Rst);
      SetForceMode(True, True);
      AddWordField(EntryEsr, '', I, 2, '9999', I, 33, 0,
        0, 9999, EditW^.EditRecs[I].My_Serno);
      SetForceMode(False, False);
      SetProtection(True);
      AddStringField(EntryEsr, '',  I, 2, '!!!!!',      I, 38, 5,
        0, nil, EditW^.Mults[I]);
      SetProtection(False);
      AddByteField(EntryEsr, '',    I, 2, '999',        I, 44, 0,
        0, 255, EditW^.EditRecs[I].Point);
      AddYesNoField(EntryEsr, '', I, 2,   '',           I, 48, 0,
        EditW^.EditRecs[I].Dupe);
    end;
    SetFieldLinks(EntryESR, 5,  2,  4);
    ActivateWindow(EditWin);
    FastWriteWindow('Mt Time Callsign     Zn Ser# RS Ser# Mults Pts Dupe', 1, 1, Colors^.HeaderAttr);
  end;

  function TVOLT.MakeMultStr(const R : log_rec) : String;
  var
    T : String[5];
  begin
    T := '';
    if R.New_Country then T := T+'C'
      else T := T+' ';
        if R.New_Qth then
          if R.His_Country = 'W' then T := T+'W'+long2str(WDistList^.GetDistNo(R.Call)) else
            if R.His_Country = 'VE' then T := T+'VE'+long2str(VEDistList^.GetDistNo(R.Call)) else
              if R.His_Country = 'VK' then T := T+'VK'+long2str(VKDistList^.GetDistNo(R.Call)) else
                if R.His_Country = 'JA' then T := T+'JA'+long2str(JADistList^.GetDistNo(R.Call)) else
                  if R.His_Country = 'ZL' then T := T+'ZL'+long2str(ZLDistList^.GetDistNo(R.Call));
    MakeMultStr := T;
  end;

  procedure TVOLT.LogContact(var ThisContact : log_rec);
    { procedure to update the mult tables, rescore, and save record }
  var
    I : Integer;
  begin
    PreLogContact(ThisContact);
    with ThisContact do begin
      My_Zone := RttyIni^.GetProfileByte('CQZone','Profile',0);
      if CountryList^.Identify(ThisContact) <> NoCountry then
        His_Country := CountryList^.Ct.Code;
      if not AdvisorSaysOK(ThisContact) then Exit;
      AddNewMults(ThisContact);
      Dupe := DupeTbl^.Load(ThisContact);
      case Dupe of
        True  : Point := 0;
        False : Point := PointValue(ThisContact);
      end;
      if Pos('/MM', Call) > 0 then begin
        New_Country := False;
        New_Qth := False;
        New_Continent := False;
      end;
    end;
    inherited LogContact(ThisContact);
  end;                            {LogContact}



  function TVOLT.AdvisorSaysOK(const R : log_rec) : Boolean;
  { t = ok to log this qso, false = dont log, they want to edit }
  var
    Missing : String[40];
  begin
    if NOT inherited AdvisorSaysOK(R) then begin { hard edits, no bypass allowed }
      AdvisorSaysOK := False;
      Exit; { hard edits, no bypass allowed }
    end;
    if NOT RttyIni^.GetProfileBool('Advice','PROFILE',True) then begin                     {dont want advisor window }
      AdvisorSaysOK := True;
      Exit;
    end;
    if ((MyCountry = 'W') or
        (MyCountry = 'VE') or
        (MyCountry = 'VK') or
        (MyCountry = 'JA') or
        (MyCountry = 'ZL')) and
        (R.His_Country = MyCountry) and
        (WDistList^.GetDistNo(R.Call) = WDistList^.GetDistNo(RttyIni^.GetProfileString('Call','Profile',''))) then begin
      ThisContest^.AdvisorWindowSameCountry;
      AdvisorSaysOK := False; { this is not allowed }
      Exit;
    end;
    Missing := '';
    if R.His_Serno = 0 then begin
      if Missing <> '' then Missing := Missing+',';
      Missing := Missing + 'Serial#';
    end;
    if R.His_Zone = 0 then begin
      if Missing <> '' then Missing := Missing+',';
      Missing := Missing + 'CQ Zone';
    end;
    AdvisorSaysOK := AdvisorWindow(Missing)
  end;

  procedure TVOLT.Writelog;
  begin
    inherited Writelog;
    WDistList^.Print;
    CountryList^.PrintInterContinentMults;
  end;

  procedure TVOLT.FillRec(const Call : String; var R : log_rec);
    { returns info to R if we has QSO before with CALL         }
    { I also try to fill as much of the QSO record as possible }
  var
    F3 : string[3];
  begin
    R.Call := Trim(Call);
    if CountryList^.Identify(R) <> NoCountry then
      R.His_Country := CountryList^.Ct.Code;
    if R.His_Country = 'VE' then R.His_Zone := ProvList^.GetZoneByCall(R.Call);
    if R.His_Zone = 0 then R.His_Zone := ZoneList^.GetZoneNo(R);
  end;

end.
