{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit c_waed;

interface

uses
  c_contest,
  DiskIo;
type
  PWAED = ^TWAED;
  TWAED = object(TContest)
            constructor init;
            procedure Header; virtual;
            procedure Footer; virtual;
            procedure Detail; virtual;
            function  LogLine(const R : Log_Rec) : String; virtual;
            procedure PrintLog; virtual;
            destructor done; virtual;
            procedure PrintSummary; virtual;
            procedure RemoveOldMults(var SomeQso : log_rec); virtual;
            procedure AddNewMults(var SomeQso : log_rec); virtual;
            procedure IdAnyMults(const SomeQso : log_rec); virtual;
            function  ContestId : string; virtual;
            procedure DisplayScoreWin; virtual;
            function  ScoreContest : LongInt; virtual;
            procedure ScoreCountPoints; virtual;
            function  FieldsPerRecord : byte; virtual;
            function  MultsCol : byte; virtual;
            procedure EditRQTc; virtual;
            procedure EditSQTc; virtual;
            procedure BuildEditWindow; virtual;
            function  MakeMultStr(const R : log_rec) : String; virtual;
            procedure LogContact(var ThisContact : log_rec); virtual;
            function  AdvisorSaysOK(const R : log_rec) : Boolean; virtual;
            procedure Writelog; virtual;
            procedure FillRec(const Call : String; var R : log_rec); virtual;
          end;

implementation

uses
  TpEntry,
  cEditWin,
  cColors,
  cClock,
  cDupeTbl,
  IniFile,
  cCountry,
  TpDate,
  TpString,
  TpWindow,
  TpCrt,
  TpMouse,
  TAHigh,
  TAccess,
  LowLvl,
  Filer,
  IsamTool,
  DebugRtns,
  cRttyWin,
  Routines,
  RecTypes,
  QtcSobj,
  QtcRObj,
  cLogWaed,
  cMsgWin;


  { TWAED.........................................................TWAED}
  { TWAED.........................................................TWAED}
  { TWAED.........................................................TWAED}
  { TWAED.........................................................TWAED}
  { TWAED.........................................................TWAED}

  constructor TWAED.init;
  var
    BTPages : LongInt;
    FNam    : string[8];
  begin
    inherited init;
    LogFile := New(PWAEDLogFile, Init);
    FNam := ForceExtension(JustFileName(InputFileName),'');
    if FNam[Length(Fnam)] = '.' then Dec(FNam[0]);
    FNam := Copy(FNam + '________',1,7);
    BTPages := BTInitIsam(NoNet, MinimizeUseOfNormalHeap, 0);
    if not IsamOK then
      MessageBox('SEVERE ERROR',
                 'Unable to init B-Tree Filer (InitIsam)',
                 'Please re-install, or contact WF1B',
                 'QTC exchange will not be available.');
    QtcSFile := New(PQtcSFile, Init(FNam+'S'));
    if not Assigned(QtcSFile) then
      MessageBox('SEVERE ERROR',
                 'Unable to create QTC send file.',
                 'Please re-install, or contact WF1B',
                 'QTC sending will not be available.');
    QtcRFile := New(PQtcRFile, Init(FNam+'R'));
    if not Assigned(QtcRFile) then
      MessageBox('SEVERE ERROR',
                 'Unable to create QTC receive file.',
                 'Please re-install, or contact WF1B',
                 'QTC receiving will not be available.');
    MakeCountryList('CQWW.CTY','CQWW.PFX');
    {$IFDEF DEBUG}
    AppendDebugMessage('WAED created');
    {$ENDIF}
  end;

  destructor TWAED.done;
  begin
    KillCountryList;
    if Assigned(QtcSFile) then Dispose(QtcSFile, Done);
    QtcSFile := NIL;
    if Assigned(QtcRFile) then Dispose(QtcRFile, Done);
    QtcRFile := NIL;
    BTExitIsam;
    inherited done;
    {$IFDEF DEBUG}
    AppendDebugMessage('WAED destroyed');
    {$ENDIF}
  end;

  function TWAED.LogLine(const R : Log_Rec) : String;
  var
    t : String[127];
  begin
    t := BandS[R.Band]+' '+
      Clock^.Date2DateString(R.Date)+
      TimeToTimeString(' hhmm ', R.Ts)+
      Pad(R.Call, 12)+' '+
      Long2Str(R.My_Rst)+'9 '+
      LeftPadCh(Long2Str(R.My_Serno), '0', 4)+' '+
      Long2Str(R.His_RST)+'9 '+
      LeftPadCh(Long2Str(R.His_Serno), '0', 4)+' '+
      Pad(R.His_Country, 5)+' ';
    if R.New_Country then t := t+' C '
    else t := t+'   ';
    t := t+Long2Str(R.Point);
    if R.Dupe then t := t+'-DUPE';
    LogLine := t;
  end;

  procedure TWAED.Header;
  begin
    inherited Header;
    PrintF('Band Date Time Call         RST Ser# RST Ser# Ctry  Mt Pt Qtc');
  end;

  procedure TWAED.Footer;
  begin
    PrintF('');
    PrintF('Page '+
      Pad(Long2Str(PageCT), 2)+
      CharStr(' ', 8)+
      'QSO''s='+Long2Str(Sub_QSO)+
      '   Mult''s='+Long2Str(sub_MULTS)+
      '   QTC''s(tx)='+Long2Str(qtcs_s)+
      '   QTC''s(rx)='+Long2Str(qtcs_r));
  end;

  procedure TWAED.Detail;
  var
    I, J : Integer;
    Line : String;
  begin
    if Linect = 0 then Header;
    if a.New_Country then Inc(sub_MULTS);
    if not a.Dupe then Inc(Sub_QSO);
    Inc(sub_pts, a.Point);
    Line := LogLine(a);
    I := QtcSFile^.HowManySentToAtTime(a.Call, a.Ts, a.Date);
    if I > 0 then
      Line := Line+'  '+Long2Str(I)+'S';
    J := QtcRFile^.HowManyRecdFromAtTime(a.Call, a.Ts, a.Date);
    if J > 0 then
      Line := Line+'  '+Long2Str(J)+'R';
    Inc(qtcs_s, Lo(I));
    Inc(qtcs_r, Lo(J));
    PrintF(Line);
  end;

  procedure TWAED.PrintLog;
  begin
    AllBands;
  end;

  procedure TWAED.PrintSummary;
  var
    Band : BandType;
  begin
    inherited PrintSummary;
    PrintSummaryTop;
    PrintF('Band    QSOs   Pts  QTCs Mults');
    for Band := B80 to B10 do
      PrintF(Pad(BandS[Band], 6)+
        LeftPad(Long2Str(Num_Qso[Band]), 6)+
        LeftPad(Long2Str(Num_Points[Band]), 6)+
        LeftPad(Long2Str(Num_Qtcs[Band]), 6)+
        LeftPad(Long2Str(Num_Country[Band]), 6));
    PrintF('-----   ----  ----  ----  ----');
    PrintF('Total '+
      LeftPad(Long2Str(Num_Qso[TOT]), 6)+
      LeftPad(Long2Str(Num_Points[TOT]), 6)+
      LeftPad(Long2Str(Num_Qtcs[TOT]), 6)+
      LeftPad(Long2Str(Num_Country[TOT]), 6));
    PrintSummaryBottom;
  end;


  procedure TWAED.RemoveOldMults(var SomeQso : log_rec);
  begin
    RemoveCountryMult(SomeQso);
    SomeQso.New_Qth := False;
    SomeQso.His_Qth := '';
  end;

  procedure TWAED.AddNewMults(var SomeQso : log_rec);
  begin
    with SomeQso do begin
      New_Country := CountryList^.Load(SomeQso,Band);
      New_Qth := False;
      His_Qth := '';
    end;
  end;

  procedure TWAED.IdAnyMults(const SomeQso : log_rec);
  var
    ix : Byte;
    Dist : Byte;
    tc : string[5]; { temp ct.code, it is easier to type in }
    tcn : word;     { temp country num, it is easier to type}
  begin
    if RttyIni^.GetProfileBool('PutMsgs','Profile',True) then
    with SomeQso do begin
      if Dupe then Exit;
      if Pos('/MM', Call) > 0 then Exit;
      tcn := CountryList^.Identify(SomeQso);
      if tcn = NoCountry then Exit;
      tc := CountryList^.Ct.Code;
      if (not CountryList^.GetCountry(tcn,Band)) and
         (tc <> 'W') and (tc <> 'VE') then RttyWin^.PutString(^Y'-DX '^Z);
    end;
  end;

  function TWAED.ContestId : string;
  begin
    ContestId := 'WAED';
  end;

  procedure TWAED.DisplayScoreWin;
  var
    PointsPerQso   : Real;
    PointsPerMult  : Real;
    TotMults       : LongInt;
    Band           : BandType;
    ScoreWin       : WindowPtr;
  begin
    ScoreCountPoints;
    Score := ScoreContest;
    if Score > 0 then PointsPerQso := Score / Num_Qso[Tot]
    else PointsPerQso := 0;
    TotMults := Num_Country[TOT];
    If TotMults > 0 then PointsPerMult := Score / TotMults
    else PointsPerMult := 0;
    with Colors^ do begin
    MakeWindow(ScoreWin, MW_xhi-40, MW_Ylo+1, MW_xhi-2, MW_Ylo+11,
                  True, True, True, WindowAttr, FrameAttr, HeaderAttr, 'WAEDC Score Summary') ;
    HideMouse;
    DisplayWindow(ScoreWin) ;
    FastWriteWindow('Band    QSOs   Pts  QTCs Mults', 1, 1, WindowAttr);
    for Band := B80 to B10 do
      FastWriteWindow(Pad(BandS[Band], 6)+
                                 LeftPad(Long2Str(Num_Qso[Band]), 6)+
                                 LeftPad(Long2Str(Num_Points[Band]), 6)+
                                 LeftPad(Long2Str(Num_QTCs[Band]), 6)+
                                 LeftPad(Long2Str(Num_Country[Band]), 6),
                                 2+Ord(Band), 1, WindowAttr);
    FastWriteWindow('-----   ----  ----  ----  ----', 7, 1, WindowAttr);
    FastWriteWindow('Total '+
                               LeftPad(Long2Str(Num_Qso[TOT]), 6)+
                               LeftPad(Long2Str(Num_points[TOT]), 6)+
                               LeftPad(Long2Str(Num_QTCs[TOT]), 6)+
                               LeftPad(Long2Str(Num_Country[TOT]), 7),
                               8, 1, WindowAttr);
    FastWriteWindow('Tot: '+Pad(Long2Str(Score),8)+
                      ' P/Q:'+Real2Str(PointsPerQSO, 4, 1)+
                      ' P/M:'+Real2Str(PointsPerMult, 4, 1),
                      9, 1, WindowAttr);
    FastWrite('Press Any Key to continue', 12, 42, WindowAttr);
    end;
    ShowMouse;
    ReadKeyorButton;
    KillWindow(ScoreWin);
  end;

  function TWAED.ScoreContest : LongInt;
  begin
    ScoreContest := (Num_Points[TOT]+Num_Qtcs[TOT])*
        ((Num_Country[B80]*4)+
          (Num_Country[B40]*3)+
          (Num_Country[B20]*2)+
          (Num_Country[B15]*2)+
          (Num_Country[B10]*2));
  end;

  procedure TWAED.ScoreCountPoints;
  var
    Band : BandType;
    b1, b2, b3, b4, b5 : LongInt;
  begin
    SetCountersToZero;
    for Band := B80 to B10 do Num_Country[Band] := CountryList^.HowMany(Band);
    LogFile^.ResetLog;
    LogFile^.Get_Next(a);
    while OK do begin
      Inc(Num_Points[a.Band], a.Point);
      Inc(Num_Qso[a.Band]);
      if a.Dupe then Inc(Num_Dupes[a.Band]);
      LogFile^.Get_Next(a);
    end;
    QtcSFile^.CountByBand(b1, b2, b3, b4, b5);
    Inc(Num_Qtcs[B80], b1);
    Inc(Num_Qtcs[B40], b2);
    Inc(Num_Qtcs[B20], b3);
    Inc(Num_Qtcs[B15], b4);
    Inc(Num_Qtcs[B10], b5);
    QtcRFile^.CountByBand(b1, b2, b3, b4, b5);
    Inc(Num_Qtcs[B80], b1);
    Inc(Num_Qtcs[B40], b2);
    Inc(Num_Qtcs[B20], b3);
    Inc(Num_Qtcs[B15], b4);
    Inc(Num_Qtcs[B10], b5);
    SumCounters;
  end;

  function TWAED.FieldsPerRecord : byte;
  { returns the number of fields (across) on the screen }
  begin
    FieldsPerRecord := 9;
  end;


  function TWAED.MultsCol : byte;
  { returns the number of "mults" column on the screen }
  begin
    MultsCol := 6;
  end;

  procedure TWAED.EditRQTc;
  begin
    if Assigned(QtcRFile) then QtcRFile^.EditQtcLog;
  end;

  procedure TWAED.EditSQTc;
  begin
    if Assigned(QtcSFile) then QtcSFile^.EditQtcLog;
  end;

  procedure TWAED.BuildEditWindow;
  var
    I : Integer;
  begin
    for I := 1 to 5 do begin
      AddChoiceField(EntryEsr, '', I, 2, 'XX', I, 1, 0, 1,
        EditIncChoicePtr, EditW^.EditRecs[I].Band);
      AddTimeField(EntryEsr, '', I, 2, 'hhmm', I, 4, 0,
        MinTime, MaxTime, EditW^.EditRecs[I].Ts);
      AddStringField(EntryEsr, '', I, 2, '!!!!!!!!!!!!', I, 9, 12, 0,
        ValidateCallSignPtr, EditW^.EditRecs[I].Call); { they ALL have callsign first! }
      SetForceMode(True, True);
      AddWordField(EntryEsr, '', I, 2, '9999', I, 22, 0,
        0, 9999, EditW^.EditRecs[I].His_Serno);
      SetForceMode(False, False);
      AddByteField(EntryEsr, '', I, 2, '99', I, 27, 0,
        0, 59, EditW^.EditRecs[I].His_Rst);
      SetForceMode(True, True);
      AddWordField(EntryEsr, '', I, 2, '9999', I, 30, 0,
        0, 9999, EditW^.EditRecs[I].My_Serno);
      SetForceMode(False, False);
      SetProtection(True);
      AddStringField(EntryEsr, '',  I, 2, '!!!!!',      I, 35, 5,
        0, nil, EditW^.Mults[I]);
      SetProtection(False);
      AddByteField(EntryEsr, '',    I, 2, '999',        I, 41, 0,
        0, 255, EditW^.EditRecs[I].Point);
      AddYesNoField(EntryEsr, '', I, 2,   '',           I, 45, 0,
        EditW^.EditRecs[I].Dupe);
    end;
    SetFieldLinks(EntryESR, 4,  2,  3);
    ActivateWindow(EditWin);
    FastWriteWindow('Mt Time Callsign     Ser# RS Ser# Mults Pts Dupe', 1, 1, Colors^.HeaderAttr);
  end;

  function TWAED.MakeMultStr(const R : log_rec) : String;
  var
    T : String[5];
  begin
    T := '';
    if R.New_Country then T := T+'C'
    else T := T+' ';
    MakeMultStr := T;
  end;

  procedure TWAED.LogContact(var ThisContact : log_rec);
    { procedure to update the mult tables, rescore, and save record }
  begin
    PreLogContact(ThisContact); { a parent function }
    with ThisContact do begin
      if CountryList^.Identify(ThisContact) <> NoCountry then
        His_Country := CountryList^.Ct.Code;
      if not AdvisorSaysOK(ThisContact) then Exit;
      AddNewMults(ThisContact);
      Dupe := DupeTbl^.Load(ThisContact);
      case Dupe of
        True  : Point := 0;
        False : Point := PointValue(ThisContact);
      end;
      if Pos('/MM', Call) > 0 then begin
        New_Country := False;
        New_Qth := False;
        New_Continent := False;
      end;
    end;
    inherited LogContact(ThisContact);
  end;                            {LogContact}


  function TWAED.AdvisorSaysOK(const R : log_rec) : Boolean;
  { t = ok to log this qso, false = dont log, they want to edit }
  var
    Missing : String[40];
  begin
    if NOT inherited AdvisorSaysOK(R) then begin { hard edits, no bypass allowed }
      AdvisorSaysOK := False;
      Exit; { hard edits, no bypass allowed }
    end;
    if NOT RttyIni^.GetProfileBool('Advice','PROFILE',True) then begin                     {dont want advisor window }
      AdvisorSaysOK := True;
      Exit;
    end;
    Missing := '';
    if R.His_Serno = 0 then begin
      if Missing <> '' then Missing := Missing+',';
      Missing := Missing + 'Serial #';
    end;
    AdvisorSaysOK := AdvisorWindow(Missing)
  end;

  procedure TWAED.Writelog;
  begin
    inherited Writelog;
    MsgWin^.Display('Preparing: WAE QTC Reports');
    QtcRFile^.PrintReport;
    QtcRFile^.PrintSummaryReport;
    QtcSFile^.PrintReport;
    QtcSFile^.PrintSummaryReport;
  end;

  procedure TWAED.FillRec(const Call : String; var R : log_rec);
    { returns info to R if we has QSO before with CALL         }
    { I also try to fill as much of the QSO record as possible }
  var
    Slot : Word;
  begin
    R.Call := Trim(Call);
    if CountryList^.Identify(R) <> NoCountry then
      R.His_Country := CountryList^.Ct.Code;
  end;




end.
