{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit Diskio;

interface

uses
  TAHigh,
  TpDate,
  TpEntry,
  TpWindow,
  RecTypes;


type
  Log_Rec        = record
                     ToolBoxNumber  : Integer;
                     VersionControl : VersionType;
                     Contest        : ContestType;
                     Date           : Date;
                     Ts             : Time;
                     Band           : BandType;
                     Mode           : ModeType;
                     Call           : String[12];
                     His_Country    : String[5];
                     His_RST        : Byte;
                     His_Qth        : String[3]; {grp# in waedc}
                     His_Zone       : Byte; {age in jarts, qtcs in waedc}
                     His_Serno      : Word;
                     His_Time       : Time;
                     My_Rst         : Byte;
                     My_Qth         : String[3];
                     My_Zone        : Byte;
                     My_Serno       : Word;
                     My_Time        : Time;
                     Point          : Byte;
                     New_Zone       : Boolean;
                     New_Country    : Boolean;
                     New_Qth        : Boolean; {in wae, F=not sent, T=sent}
                     New_Continent  : Boolean;
                     Dupe           : Boolean;
                     Comment        : String[25];
                   end;

PLogFile = ^TLogFile;
TLogFile = object
             Index        : String[8];
             TpDataset    : DataSet;
             BaseFName    : string[8];
             SaveOnMinute : byte;
             SaveLastHour : Byte;
             LastQSOTime  : Time;
             SearchStr    : String[12];
             EditEsr      : EsRecord;
             ExitCommand  : EStype;
             EditPtr      : WindowPtr;
             DoneEditting : Boolean;
             Work         : Log_Rec;
             Updated      : Boolean;
             UserSavePath : String[79];

             constructor Init;
             destructor  Done; virtual;
             procedure AutoSave;
             procedure SaveLog(Drive : Char);
             procedure DeleteRecord(var R : log_rec);
             procedure DeleteThisRecord(var R : log_rec);
             procedure FindThenUpdate(var ThisRecNo : longint; var R : log_rec);
             procedure Flush;
             procedure Get_Next(var R : log_rec);
             procedure Get_Prev(var R : log_rec);
             function  GetNext(var R : log_rec; var I : String) : Boolean;
             function  GetPrev(var R : log_rec; var I : String) : Boolean;
             function  MakeIx(var R : log_rec) : String;
             function  NumRecs : LongInt;
             procedure ResetLog;
             procedure SaveToDisk(var R : log_rec);
             procedure ResaveScrap(var R : log_rec);
             procedure UpDate(var R : log_rec);
             function  CurrentIndex : String;
             function  GetFileSize(S : String) : LongInt;
             function  SaveData(Path : String) : Boolean;
             procedure BlankLogRec(var R : log_rec);
             procedure AutoSaveOn;
             procedure AutoSaveOff;
             function  LoadData : Word;
             function  CabrilloDetail(var R : log_rec) : string; virtual;
             { file edit functions }
             procedure Edit;
             procedure Edit_GetSearchStr;
             procedure Edit_Search;
             procedure Edit_SearchContinued;
             procedure Edit_UpdateIfNeeded(TR : Log_Rec);
             procedure Edit_SaveAndExit;
             procedure Edit_JustExit;
             procedure Edit_SaveAndScrollBack;
             procedure Edit_SaveAndScrollForward;
             procedure Edit_DeleteContact;
             procedure Edit_CancelChanges;
             procedure BuildEditScreen; virtual;
           end;

var
  LogFile : PLogFile;
  Prev    : Log_Rec; { previously logged QSO }
  Hold    : Log_Rec; { previously logged QSO }

function  PeekAtBinContest : ContestType;
procedure RebuildIndex;
procedure SAFECOPY (fromFile, toFile : string);

implementation

uses
  ApTimer,
  TAccess,
  TpCmd,
  TpString,
  TpRarray,
  TpCrt,
  TpDos,
  Dos,
  Keys,
  TpEdit,
  TpMouse,
  LowLvl,
  c_Contest,
  cYearTbl,
  cMsgWin,
  cDupeTbl,
  cSerNum,
  cCountry,
  cEditWin,
  DebugRtns,
  OneTimers,
  cRateWin,
  cRttyWin,
  Network,
  ApPort,
  TncControl,
  Routines,
  cColors,
  IniFile;

  procedure DiskError; far;
  var
    T : string[50];
  begin
    if TAStatus < 1000 then T := TurboError(TAStatus)
                       else T := TAError(TAStatus);
    MessageBox('DISK IO ERROR',T,'Unable to continue!','RTTY will now terminate.');
    TAClose(LogFile^.TpDataSet);
    Halt;
  end;

  constructor TLogFile.Init;
  begin
    TAErrorProc := @DiskError;
    TAOpen(TpDataSet, FileName('BIN'), SizeOf(log_rec), FileName('NDX'), SizeOf(Index)-1);
    if not OK then
      TACreate(TpDataSet, fileName('BIN'), SizeOf(log_rec), FileName('NDX'), SizeOf(Index)-1);
    if not OK then begin
      MessageBox('SEVERE ERROR','Unable to create the log file',
                 'Be sure there is room on the disk, or the',
                 'floppy is not write protected.');
      Halt;
    end;
    TAReset(TpDataSet);
    TAFlush(TpDataSet);
    SaveOnMinute := RttyIni^.GetProfileByte('Minute', 'Autosave', 0);
    if SaveOnMinute > 59 then SaveOnMinute := 0;
    SaveLastHour := 99;
    UserSavePath := RttyIni^.GetProfileString('Path', 'Autosave', 'A:\');
    UserSavePath := Trim(UserSavePath);
    if UserSavePath[Length(UserSavePath)] = '\' then Dec(UserSavePath[0]);
    if NOT ExistDir(UserSavePath) then begin
      MessageBox('AUTOSAVE ERROR','Invalid autosave path specified in RTTY.INI,',
                 'Path='+UserSavePath,
                 'must exist on the specified drive.');
      AppendDebugMessage('Invalid autosave path specified in RTTY.INI='+UserSavePath)
    end;
    if UserSavePath[Length(UserSavePath)] <> '\' then UserSavePath := UserSavePath + '\';
    AppendDebugMessage('Autosave Path: "'+UserSavePath+'"');
    {$IFDEF DEBUG}
    AppendDebugMessage('DiskIO created');
    {$ENDIF}
  end;

  destructor TLogFile.Done;
  begin
    TAFlush(TpDataSet);
    TAClose(TpDataSet);
    {$IFDEF DEBUG}
    AppendDebugMessage('DiskIO destroyed');
    {$ENDIF}
  end;


  function TLogFile.MakeIx(var R : log_rec) : String;
  begin
    MakeIx := DateToSortString(R.Date)+TimeToSortString(R.Ts);
  end;


  function TLogFile.CurrentIndex : String;
  var
    Junk : log_rec;
  begin
    if (TARecNum < 4000) and (TARecNum > 0) then begin
      GetRec(TpDataSet.Data, TARecNum, Junk);
      CurrentIndex := MakeIx(Junk);
    end
    else CurrentIndex := '';
  end;

  function TLogFile.NumRecs : LongInt;
  begin
    NumRecs := UsedRecs(TpDataSet.Data);
  end;

  procedure TLogFile.Get_Next(var R : log_rec);
  begin
    TANext(TpDataSet, R, Index);
    while OK and (R.ToolBoxNumber <> 0) do TANext(TpDataSet, R, Index);
  end;

  procedure TLogFile.Get_Prev(var R : log_rec);
  begin
    TAPrev(TpDataSet, R, Index);
    while OK and (R.ToolBoxNumber <> 0) do TAPrev(TpDataSet, R, Index);
  end;

  function TLogFile.GetNext(var R : log_rec; var I : String) : Boolean;
    { return record & true if all is OK else return false & assume EOF }
  begin
    TANext(TpDataSet, R, I);
    while OK and (R.ToolBoxNumber <> 0) do TANext(TpDataSet, R, I);
    GetNext := OK;
  end;

  function TLogFile.GetPrev(var R : log_rec; var I : String) : Boolean;
    { return record & true if all is OK else return false & assume EOF }
  begin
    TAPrev(TpDataSet, R, I);
    while OK and (R.ToolBoxNumber <> 0) do TAPrev(TpDataSet, R, I);
    GetPrev := OK;
  end;

  procedure TLogFile.ResetLog;
  begin
    TAReset(TpDataSet);
  end;

  procedure TLogFile.Flush;
  begin
    TAFlush(TpDataSet);
  end;

  procedure TLogFile.UpDate(var R : log_rec);
  var
    NewIndex : String[8];
    OldIndex : String[8];
    RecNo : LongInt;
    bailout : Integer;
  begin
    OldIndex := CurrentIndex;
    if OldIndex = '' then Exit;
    NewIndex := MakeIx(R);
    if OldIndex <> NewIndex then begin {update indexes here}
      DeleteKey(TpDataSet.Index, RecNo, OldIndex);
      bailout := 0;
      repeat
        AddKey(TpDataSet.Index, RecNo, NewIndex);
        if not OK then begin
          Inc(R.Ts, 3);
          NewIndex := MakeIx(R);
          Inc(bailout);
        end;
      until OK or (bailout = 19);
    end;
    Index := NewIndex;
    TAUpdate(TpDataSet, R, Index);
  end;

  procedure TLogFile.FindThenUpdate(var ThisRecNo : longint; var R : log_rec);
  var
    a : log_rec;
  begin
    {$IFDEF DEBUG}
    if ThisRecNo = 0 then begin
      AppendDebugMessage('FindThenUpdate, ThisRecNo = 0');
      halt;
    end;
    {$ENDIF}
    GetRec(TpDataSet.Data, ThisRecNo, a);
    {$IFDEF DEBUG}
    if TAStatus <> 0 then begin
      AppendDebugMessage('FindThenUpdate, GetRec failed, ThisRecNo:'+long2str(ThisRecNo)+' TAStatus='+long2str(tastatus));
      halt;
    end;
    {$ENDIF}
    PutRec(TpDataSet.Data, ThisRecNo, r);
    {$IFDEF DEBUG}
    if TAStatus <> 0  then begin
      AppendDebugMessage('FindThenUpdate, PutRec failed, ThisRecNo:'+long2str(ThisRecNo)+' TAStatus='+long2str(tastatus));
      halt;
    end;
    {$ENDIF}
  end;

  procedure TLogFile.ResaveScrap(var R : log_rec);
  begin
    TAUpdate(TpDataSet, R, Index);
  end;

  procedure TLogFile.DeleteRecord(var R : log_rec);
  begin
    Index := MakeIx(R);
    TADelete(TpDataSet, Index);
    if OK then begin
      TAPrev(TpDataSet, R, Index);
      while OK and (R.ToolBoxNumber <> 0) do
        TAPrev(TpDataSet, R, Index);
    end;
  end;

  procedure TLogFile.DeleteThisRecord(var R : log_rec);
  var
    DelIndex : String[8];
  begin
    DelIndex := MakeIx(R);
    if DelIndex <> '' then TADelete(TpDataSet, DelIndex);
  end;

  procedure TLogFile.SaveToDisk(var R : log_rec);
  var
    hh, mm, ss : Byte;
  begin
    TimeToHMS(R.Ts, hh, mm, ss);
    R.Ts := HMSToTime(hh, mm, 0); { start all qso's as the '0' second }
    Index := MakeIx(R);
    TAInsert(TpDataSet, R, Index);
    while not OK do begin         { if insert not ok then it was }
      Inc(R.Ts, 3);               { a duplicate key, bounce ts  }
      Index := MakeIx(R);
      TAInsert(TpDataSet, R, Index); { and try again }
    end;
    TAFlush(TpDataSet);
  end;

  function TLogFile.GetFileSize(S : String) : LongInt;
  var
    f : file of Byte;
  begin
    Assign(f, S);
    Reset(f);
    GetFileSize := FileSize(f);
    Close(f);
  end;

  function TLogFile.SaveData(Path : String) : Boolean;

  function SamePath : boolean;
  var
    RttyPath : String[79];
  begin
    RttyPath := JustPathName(FullPathName(ParamStr(0))); { where are we running from ?}
    if RttyPath[Length(RttyPath)] = '.' then Dec(RttyPath[0]); { remove '.' the last char  }
    if RttyPath[Length(RttyPath)] <> '\' then RttyPath := RttyPath+'\';
    if RttyPath[1] <> Path[1] then begin
      SamePath := False;
      Exit;
    end;
    if Path[Length(Path)] <> '\' then Path := Path+'\';
    if StUpCase(RttyPath) = StUpCase(Path) then begin
      RttyWin^.PutString(^Y^G^G'Cannot save into directory you are executing from!'^Z^M);
      RttyWin^.PutString(^Y^G^G'Use a different drive or directory.'^Z^M);
      SamePath := True;
      Exit;
    end;
  end;

  function EnoughSpace : boolean;
  var
    SpaceNeeded : LongInt;
    SpaceAvail : LongInt;
  begin
    SpaceAvail := DiskFree(Ord(Upcase(Path[1]))-64);
    { if the files are there, i can "count" the space used when they aer deleted}
    if ExistFile(Path+JustFileName(FileName('BIN'))) then SpaceAvail := SpaceAvail +
                  GetFileSize(Path+JustFileName(FileName('BIN')));
    if ExistFile(Path+JustFileName(FileName('NDX'))) then SpaceAvail := SpaceAvail +
                  GetFileSize(Path+JustFileName(FileName('NDX')));
    SpaceNeeded := GetFileSize(FileName('BIN'))
                  + GetFileSize(FileName('NDX'));
    if SpaceNeeded > SpaceAvail then begin
      MsgWin^.Display('Not enough free space on '+Path+'  Need '+
        +long2str(SpaceNeeded)+' bytes free. ');
      EnoughSpace := False;
    end
    else
      EnoughSpace := True;
  end;

  begin
    SaveData := False; { assume failure }
    if SamePath then Exit;
    if NOT EnoughSpace then Exit;
    DeleteFile(Path+JustFileName(FileName('BIN')));
    DeleteFile(Path+JustFileName(FileName('NDX')));
    SafeCopy(FileName('BIN'),Path+JustFileName(FileName('BIN')));
    SafeCopy(FileName('NDX'),Path+JustFileName(FileName('NDX')));
    SaveData := True;
  end;

  procedure TLogFile.Savelog(Drive : Char);
  begin
    Drive := UpCase(Drive);
    if (Drive < 'A') or (Drive > 'Z') then Drive := ' ';
    if not(Drive in [' ', 'A', 'B']) then begin
      MsgWin^.Display('SAVELOG permitted only to drive A,B, and '' ''');
      Exit;
    end;
    if Drive in ['A','B'] then begin
      if not DriveReady(Drive) then begin
        MsgWin^.Display('SAVELOG Floppy Drive '+Drive+':\ not ready!');
        Exit;
      end;
    end;
    case Drive of
      'A','B' : if SaveData(Drive+':\') then MsgWin^.Display('SAVELOG complete to '+Drive+':\')
                                        else MsgWin^.Display('SAVELOG failed!');
      ' '     : if SaveData(UserSavePath) then MsgWin^.Display('SAVELOG complete to '+UserSavePath)
                                        else MsgWin^.Display('SAVELOG failed!');
    end;
  end;

  procedure TLogFile.AutoSave;
  var
    Hr, MN, SC : Byte;
    WinPtr : WindowPtr;
    et : EventTimer;
    c : byte;
  begin
    if RttyTerm1^.KeyBoardLive then Exit;
    if NOT RttyIni^.GetProfileBool('Enabled', 'AutoSave', False) then Exit;
    TimeToHMS(CurrentTime, Hr, MN, SC);
    if SaveLastHour = Hr then Exit;
    if SaveOnMinute <> MN then Exit;
    SaveLastHour := Hr;
    with Colors^ do begin
      MakeWindow(WinPtr, 20, 10, 60, 14, True, True, False, WindowAttr, FrameAttr, HeaderAttr, 'Autosave');
      HideMouse;
      DisplayWindow(WinPtr);
      FastWriteWindow('Press <ENTER> to backup your', 1, 6, WindowAttr);
      FastWriteWindow('QSO data.', 2, 8, WindowAttr);
      FastWriteWindow('Press anykey to SKIP autosave', 3, 6, WindowAttr);
      NewTimer(et,18*30); { 30 seconds }
      repeat
      until KeyPressed or TimerExpired(et);
    end;
    KillWindow(WinPtr);
    ShowMouse;
    case KeyPressed of
      true: c := lo(ReadKeyWord);
      false: c := 13;  { if they dont respond, assume <enter> }
    end;
    if c = 13 then
      if SaveData(UserSavePath) then begin
        MsgWin^.Display('Autosave complete to '+UserSavePath);
        end
        else begin
        MsgWin^.Display('Unable to AUTOSAVE to '+UserSavePath);
        end;
  end;

  procedure TLogFile.BlankLogRec(var R : log_rec);
  begin
    with R do begin
      ToolBoxNumber := 0;
      VersionControl := V2_0_0;
      if Assigned(RttyIni) then begin
        case RttyIni^.GetProfileInt('Band', 'CURRENT', 20) of
          80 : Band := b80;
          40 : Band := b40;
          20 : Band := b20;
          15 : Band := b15;
          10 : Band := b10;
        end;
      end;
      Call := '';
      His_RST := 59;
      His_Zone := 0;
      His_Qth := '';
      His_Serno := 0;
      His_Time := 0;
      My_Time := 0;
      My_Serno := 0;
      ts := 0;
      Date := 0;
      Point := 0;
      New_Country := False;
      New_Zone := False;
      New_Qth := False;
      New_Continent := False;
      Dupe := False;
      His_Country := '';
      Comment := '';
    end;
  end;

  procedure TLogFile.AutoSaveOn;
  begin
    SaveLastHour := 99;
    RttyIni^.SetOrAddProfileString('Enabled', 'AutoSave', 'True');
  end;

  procedure TLogFile.AutoSaveOff;
  begin
    SaveLastHour := 99;
    RttyIni^.SetOrAddProfileString('Enabled', 'AutoSave', 'False');
  end;

  function TLogFile.LoadData : Word;
  var
    Count : Word;
    DLogRec : Diskio.log_rec;                  { DLogRec is read from disk file }
    a : log_rec;                  { a is a copy of DLogRec }
    st_t : Time;
    hh, mm, ss : Byte;
    cty_num : word;
    cqzone : integer;
  begin
    ThisContest^.Loading := True;
    st_t := CurrentTime;
    LastQSOTime := CurrentTime;
    Count := 0;
    ResetLog;
    while GetNext(DLogRec, Index) do begin
      if RttyIni^.GetProfileBool('DISKIO.LOADDATA', 'DEBUG', False) then
        AppendDebugMessage('LoadData: Callsign '+DlogRec.call+' Ser#'+Long2str(DLogRec.MY_Serno));
      if (DLogRec.Call = 'DELETE') or (DLogRec.Call = '')
      then DeleteRecord(DLogRec)
      else begin
        Inc(Count);
        if Count >= DupeTbl^.GetMax then Terminate(106);
        SerNum^.SetSerialNumber(DLogRec.Band, DLogRec.My_Serno);
        a := DLogRec;
        DLogRec.My_Qth := RttyIni^.GetProfileString('Qth','Profile','');
        if Str2Int(RttyIni^.GetProfileString('CqZone', 'Profile', '')  , cqzone) then
          DLogRec.My_Zone := CqZone;
        DLogRec.Contest := Contest;
        DLogRec.Call := Trim(DLogRec.Call);
        Cty_Num := CountryList^.Identify(DLogRec);
        if Cty_Num <> NoCountry then begin
          DLogRec.His_Country := CountryList^.Ct.Code;
          DLogRec.Point := ThisContest^.PointValue(DLogRec);
        end
        else with DLogRec do begin
          His_Country := '';
          New_Country := False;
          Point := 0;
        end;
        if RttyIni^.GetProfileBool('ReCalcMults', 'Profile', True) then begin
          with DLogRec do begin
            New_Zone       := False;
            New_Country    := False;
            New_Qth        := False;
            New_Continent  := False;
          end;
        end;
        ThisContest^.RemoveOldMults(DLogRec);
        case Contest of
          SPRI : DLogRec.dupe := NOT ThisContest^.QsoIsGood(DLogRec);
        else
          DLogRec.Dupe := DupeTbl^.Load(DLogRec);
        end;
        if DLogRec.Dupe then DLogRec.Point := 0
          else
            ThisContest^.AddNewMults(DLogRec);
        if Pos('/MM', DLogRec.Call) > 0 then begin
          DLogRec.New_Country := False;
          DLogRec.New_Qth := False;
          if Contest <> SCCC then DLogRec.New_Continent := False;
          { SCCC uses continent mult for YEAR mult }
        end;
        if (a.Contest <> DLogRec.Contest) or
           (a.Call <> DLogRec.Call) or
           (a.New_Country <> DLogRec.New_Country) or
           (a.New_Qth <> DLogRec.New_Qth) or
           (a.New_Zone <> DLogRec.New_Zone) or
           (a.Point <> DLogRec.Point) or
           (a.New_Continent <> DLogRec.New_Continent) or
           (a.My_Qth <> DLogRec.My_Qth) or
           (a.My_Zone <> DLogRec.My_Zone) or
           (a.His_Country <> DLogRec.His_Country) or
           (a.His_Zone <> DLogRec.His_Zone) then
          TAUpdate(TpDataSet, DLogRec, Index);
        LastQSOTime := DLogRec.ts;
        RateWin^.NewEvent(DLogRec.Date, DLogRec.Ts);
      end;
    end;
    CountryList^.BlankCTRec;
    BlankLogRec(DLogRec);
    TimeDiff(st_t, CurrentTime, hh, mm, ss);
    AppendDebugMessage('LoadData: '+long2str(mm)+' minutes, '+long2str(ss)+
      ' seconds to load '+long2str(Count)+' records.');
    Editw^.RefreshFromDisk;
    ThisContest^.Loading := False;
    LoadData := Count;
  end;

procedure RebuildIndex;
var
  RecordNumber   : LongInt;
  DatF           : DataFile;
  IndexF         : IndexFile;
  TimeOut          : byte;
  Num            : Word;
  a              : log_rec;
  Index     : String[8];

begin
  if not ExistFile(FileName('BIN')) then Exit;
  DeleteFile(FileName('BI2'));
  SafeCopy(FileName('BIN'),FileName('BI2'));
  OpenFile(DatF, FileName('BIN'), SizeOf(a));
  if not OK then begin
    AppendDebugMessage('Diskio: Open BIN file failed. Error='+long2str(TaStatus));
    MessageBox('Auto Re-Index Error',
               'Open BIN file failed. Error='+long2str(TaStatus),
               'This is a SERIOUS error, please exit and',
               'perhaps use a backup file');
    Exit;
  end;
  MakeIndex(IndexF, FileName('NDX'), SizeOf(Index)-1, NoDuplicates);
  if not OK then begin
    AppendDebugMessage('Diskio: Make new index failed. Error='+long2str(TaStatus));
    MessageBox('Auto Re-Index Error',
               'Make new index failed. Error='+long2str(TaStatus),
               'This is a SERIOUS error, please exit and',
               'perhaps make sure there is disk space');
    Exit;
  end;
  for RecordNumber := 1 to FileLen(DatF)-1 do begin
    GetRec(DatF, RecordNumber, a);
    if not OK then begin
      AppendDebugMessage('Diskio: Get data failed at rec:'+long2str(RecordNumber));
      Exit;
    end;
    if (a.Call = 'DELETE') and (a.ToolBoxNumber = 0) then DeleteRec(DatF, RecordNUmber);
    if a.ToolBoxNumber = 0 then begin
      TimeOut := 0;
      Index := DateToSortString(a.Date)+TimeToSortString(a.Ts);
      AddKey(IndexF, RecordNumber, Index);
      While not OK do begin
        Inc(a.Ts,3);
        Index := DateToSortString(a.Date)+TimeToSortString(a.Ts);
        AddKey(IndexF, RecordNumber, Index);
        inc(TimeOut);
        if TimeOut = 99 then begin
          AppendDebugMessage('ReIndex: Create index failed at rec:'+long2str(RecordNumber));
          Exit;
        end;
      end;
    end;
  end;
  CloseFile(DatF);
  CloseIndex(IndexF);
  AppendDebugMessage('Diskio: Auto Re-Index complete');
end;

function PeekAtBinContest : ContestType;
var
  FromF : File;
  Buf: array[1..87*2] of Byte;
  NumRead : Word;
  b : byte;
begin
  { default to [CURRENT], Contest value }
  PeekatBinContest := ContestStr2Contest(RttyIni^.GetProfileString('Contest','CURRENT',''));
  if not ExistFile(FileName('BIN')) then Exit;
  if GetFileSize(FileName('BIN')) < 87*2 then Exit;
  { try to read value from the disk file }
  Assign(FromF, FileName('BIN'));
  Reset(FromF,1);
  BlockRead(FromF, Buf, SizeOf(Buf), NumRead);
  Close(FromF);
  b := ord(buf[91]);
  PeekAtBinContest := ContestType(b);
end;

{ local functions for the file editor }
  procedure NotFoundMessage(Str : String);
  var
    MessageWin     : WindowPtr;
    KeyStroke      : Word;
  begin
    with Colors^ do begin
      if not MakeWindow(MessageWin, 5, 5, 40, 7, True, True, True,
                        WindowAttr, FrameAttr, HeaderAttr, '') then Exit;
      HideMouse;
      DisplayWindow(MessageWin) ;
      FastWriteWindow(Str+' - record not found!', 1, 1, WindowAttr);
    end;
    ShowMouse;
    KeyStroke := ReadKeyorButton;
    KillWindow(MessageWin);
  end;

  function AreYouSure : boolean;
  var
    MessageWin     : WindowPtr;
    KeyStroke      : Word;
  begin
    with Colors^ do begin
    if not MakeWindow(MessageWin, 5, 7, 40, 9, True, True, True,
                      WindowAttr, FrameAttr, HeaderAttr, 'Delete Record') then Exit;
    HideMouse;
    DisplayWindow(MessageWin) ;
    FastWriteWindow('Press Enter to delete this record', 1, 1, WindowAttr);
    end;
    ShowMouse;
    AreYouSure := ReadKeyWord = kbEnter;
    KillWindow(MessageWin);
  end;

  procedure NoMoreFoundMessage(Str : String);
  var
    MessageWin     : WindowPtr;
    KeyStroke      : Word;
  begin
    with Colors^ do begin
    if not MakeWindow(MessageWin, 5, 5, 40, 7, True, True, True,
                      WindowAttr, FrameAttr, HeaderAttr, '') then Exit;
    HideMouse;
    DisplayWindow(MessageWin);
    FastWriteWindow(Str+' - no more found!', 1, 1, WindowAttr);
    end;
    ShowMouse;
    KeyStroke := ReadKeyorButton;
    KillWindow(MessageWin);
  end;

  procedure TLogFile.Edit_GetSearchStr;
  var
    WinPtr         : WindowPtr;
    Escaped        : Boolean;
  begin
    SearchStr := '';
    with Colors^ do begin
    MakeWindow(WinPtr, 5, 5, 20, 7, True, True, False, WindowAttr,
                  FrameAttr, HeaderAttr, 'Search') ;
    DisplayWindow(WinPtr) ;
    ReadString('', 1, 1, 12, PromptAttr, StringAttr, CtrlAttr, Escaped, SearchStr);
    SearchStr := StUpCase(Trim(SearchStr));
    KillWindow(WinPtr);
    end;
  end;


  {* This routine handles the choice type variables. If a + or ' ' is entered *}
  {* then the item is incremented. If a - is entered, then the choice is      *}
  {* decremented.                                                             *}

  procedure EditIncChoice(var Value; FieldID : Byte;
                                 Factor         : Integer;
                             var St             : String); far;
  var
    Mode           : ModeType absolute Value;
    Band           : BandType absolute Value;
  begin
    case FieldID of
      002 : begin
              case Factor of
                1 : if Band = Band_Last then Band := Band_First
                    else Inc(Band);
                -1 : if Band = Band_First then Band := Band_Last
                     else Dec(Band);
              end;
              St := BandS[Band];
            end;
      003 : begin
              case Factor of
                1 : if Mode = Mode_Last then Mode := Mode_First
                    else Inc(Mode);
                -1 : if Mode = Mode_First then Mode := Mode_Last
                     else Dec(Mode);
              end;
              St := Modes[Mode];
            end;
    end;
  end;


  procedure TLogFile.Edit_Search;
  var
    Found,
    EndOfFile      : Boolean;
  begin
    Found := False;
    EndOfFile := False;
    Edit_GetSearchStr;
    ResetLog;
    while (not Found) and (not EndOfFile) do begin
      EndOfFile := not GetNext(Work, Index);
      if Pos(SearchStr, Work.Call) > 0 then Found := True;
    end;
    if (not Found) or (EndOfFile) then NotFoundMessage(SearchStr);
  end;

  procedure TLogFile.Edit_SearchContinued;
  var
    Found,
    EndOfFile      : Boolean;
  begin
    Found := False;
    EndOfFile := False;
    while (not Found) and (not EndOfFile) do begin
      EndOfFile := not GetNext(Work, Index);
      if Pos(SearchStr, Work.Call) > 0 then Found := True;
    end;
    if (not Found) or (EndOfFile) then NoMoreFoundMessage(SearchStr);
  end;

  procedure TLogFile.Edit_UpdateIfNeeded(TR : Log_Rec);
  begin
    if Updated then begin
      { get rid of old mults first }
      ThisContest^.RemoveOldMults(Hold);
      { if it was a good QSO, remove from dupe_tbl }
      if not Hold.Dupe then
        DupeTbl^.ResetDupeByCall(Hold.Call, Hold.Band);
      { now add in new mults, if any }
      ThisContest^.AddNewMults(TR);
      { now update dupe table }
      TR.Dupe := DupeTbl^.Load(TR);
      { update points }
      if TR.Dupe then Tr.Point := 0
        else TR.Point := ThisContest^.PointValue(TR);
      { finally, save it to disk }
      Update(TR);
      if Assigned(Net) then Net^.SendLogData(False, TR,Index);
      Updated := False;
    end;
  end;

  procedure TLogFile.Edit_SaveAndExit;
  begin
    Edit_UpdateIfNeeded(Work);
    DoneEditting := True;
  end;

  procedure TLogFile.Edit_JustExit;
  begin
    Work := Hold;
    DoneEditting := True;
  end;

  procedure TLogFile.Edit_SaveAndScrollBack;
  begin
    Edit_UpdateIfNeeded(Work);
    if GetPrev(Work, Index) then
      Hold := Work  {set hold to new if OK }
    else
      if GetNext(Work, Index) then
        Hold := Work;
  end;

  procedure TLogFile.Edit_SaveAndScrollForward;
  begin
    Edit_UpDateIfNeeded(Work);
    if GetNext(Work, Index) then
      Hold := Work
    else
      if GetPrev(Work, Index) then
        Hold := Work;  {set hold to new if OK }
  end;

  procedure CantDeleteBlankCall;
  var
    MessageWin     : WindowPtr;
  begin
    with Colors^ do begin
      if not MakeWindow(MessageWin, 5, 7, 40, 12, True, True, True,
                        WindowAttr, FrameAttr, HeaderAttr, 'Delete Error!') then Exit;
      HideMouse;
      DisplayWindow(MessageWin) ;
      FastWriteWindow('This record has a blank callsign!', 1, 1, WindowAttr);
      FastWriteWindow('It will be deleted automatically.', 2, 1, WindowAttr);
      FastWriteWindow('If you wish, add a callsign, then', 3, 1, WindowAttr);
      FastWriteWindow('you may delete this record.',       4, 1, WindowAttr);
      ShowMouse;
      ReadKeyWord;
      KillWindow(MessageWin);
    end;
  end;

  procedure TLogFile.Edit_DeleteContact;
  begin
    if Trim(Work.Call) = '' then begin
      CantDeleteBlankCall;
      Exit;
    end;
    if AreYouSure then begin
      if Assigned(Net) then Net^.SendDeleteLogData(Work);
      if not Work.Dupe then DupeTbl^.ResetDupeByCall(Work.Call, Work.Band);
      ThisContest^.RemoveOldMults(Work);
      DeleteThisRecord(Work);
      GetPrev(Work, Index);
      Hold := Work;
    end;
    DoneEditting := NumRecs = 1;
  end;

  procedure TLogFile.Edit_CancelChanges;
  begin
    Work := Hold;
  end;

  procedure TLogFile.BuildEditScreen;
  begin
    with Colors^ do begin
    if not MakeWindow(EditPtr, 1, 1, 50, 23, True, True, False, WindowAttr,
                      FrameAttr, HeaderAttr, 'Edit QSO Information') then Exit;
    DoneEditting := False;
    HideMouse;
    DisplayWindow(EditPtr) ;
    InitEsRecord(EditEsr);
    SetEntryAttributes;
    SetWrapMode(EditEsr, WrapAtEdges);
    FastWriteWindow('F1 Search         F7  Save & Scroll Back',    1, 3, WindowAttr);
    FastWriteWindow('F2 Search Next    F8  Save & Scroll Forward', 2, 3, WindowAttr);
    FastWriteWindow('F3 Save & Exit    F10 Delete Contact',        3, 3, WindowAttr);
    FastWriteWindow('F4 Exit           Esc Cancel Change',         4, 3, WindowAttr);
    FastWriteWindow(''+CharStr('', 48)+'', 5, 0, FrameAttr);
    end;
    AddDateField(EditEsr,   'Date',      07, 3, 'yy/mm/dd',
                                         07, 15, 0, MinDate, MaxDate, Work.Date);
    AddTimeField(EditEsr,   'Time',      08, 3, 'hh:mm',
                                         08, 15, 0, 0, 0, Work.Ts);
    AddChoiceField(EditEsr, 'Band',     09, 3, 'XX',
                                         09, 15, 0, 1, @EditIncChoice, Work.Band);
    AddChoiceField(EditEsr, 'Mode',     10, 3, 'XXXXXX',
                                         10, 15, 0, 1, @EditIncChoice, Work.Mode);
    AddStringField(EditEsr, 'Callsign',  11, 3, '!!!!!!!!!!!!',
                                         11, 15, 12, 0, nil, Work.Call);
    AddByteField(EditEsr,   'Sent RS(T)',12, 3, '99',
                                         12, 15, 0, 11, 59, Work.My_RST);
    AddWordField(EditEsr,   'QSO Number',13, 3, '9999',
                                         13, 15, 0, 0, 9999, Work.My_Serno);
    AddStringField(EditEsr, 'Country',   14, 3, '',
                                         14, 15, 5, 0, nil, Work.His_Country);
    AddByteField(EditEsr,   'Recd RS(T)',15, 3, '99',
                                         15, 15, 0, 11, 59, Work.His_RST);
  end;

  procedure TLogFile.Edit;
  var
    FieldName : string[3];
    CurBand   : BandType;
  begin
    if NumRecs = 0 then Exit;
    CurBand := EditW^.EditRecs[1].Band;
    Updated := False;
    ResetLog;
    GetPrev(Work, Index);
    Hold := Work;
    BuildEditScreen;
    AddCmd(ESEnd,  kbEnd);
    AddCmd(ESHome, kbHome);
    AddCmd(EsQuit, kbEsc);
    AddCmd(ESIncChoice, kbGrayPlus);
    AddCmd(ESIncChoice, kbSpace);
    repeat
      ExitCommand := EditScreen(EditEsr, 1, False);
      if TimeToTimeString('hhmm',hold.ts) = TimeToTimeString('hhmm',Work.ts) then
        Work.ts := Hold.ts;
      Updated := CompStruct(Hold,Work,sizeof(Work)) <> Equal;
      case ExitCommand of
        Esmy13 {F1}  : Edit_Search;
        Esmy14 {F2}  : Edit_SearchContinued;
        Esmy15 {F3}  : Edit_SaveAndExit;
        Esmy16 {F4}  : Edit_JustExit;
        Esmy19 {F7}  : Edit_SaveAndScrollBack;
        Esmy50 {F8}  : Edit_SaveAndScrollForward;
        Esmy52 {F10} : Edit_DeleteContact;
        EsQuit {ESC} : Edit_CancelChanges;
      end;
    until DoneEditting;
    AddCmd(ESmy25, kbEnd);
    AddCmd(ESmy23, kbHome);
    AddCmd(Esmy49, kbEsc);
    AddCmd(ESmy28, kbGrayPlus);
    if RttyIni^.GetProfileBool('SpaceTab','PROFILE',False) then AddCmd(EsNextField,kbSpace);
    Flush;
    ResetLog;
    KillWindow(EditPtr);
    DisposeEditScreen(EditEsr);
    Editw^.RefreshFromDisk;
    EditW^.EditRecs[1].Band := CurBand;
  end;

  procedure TellError(s : string; i: integer);
  begin
  case I of
   0 : AppendDebugMessage(s+'no error');
   1 : AppendDebugMessage(s+'invalid function');
   2 : AppendDebugMessage(s+'file not found');
   3 : AppendDebugMessage(s+'path not found');
   4 : AppendDebugMessage(s+'too many files open');
   5 : AppendDebugMessage(s+'access denied');
   6 : AppendDebugMessage(s+'invalid file handle');
   12 : AppendDebugMessage(s+'invalid file access code');
   15 : AppendDebugMessage(s+'invalid drive');
  else
    AppendDebugMessage(s+'unknown error='+long2str(I));
  end;
  end;

procedure SAFECOPY (fromFile, toFile : string);
type
  bufferType = array [1..16383] of char;
  bufferTypePtr = ^bufferType;  { Use the heap }
var
  bufferPtr : bufferTypePtr;     { for the buffer }
  f1, f2 : file;
  bufferSize, readCount, writeCount : word;
  fmSave : byte;              { To store the filemode }
  i : integer;
begin
  bufferSize := SizeOf(bufferType);
  if MaxAvail < bufferSize then begin { Assure there is enough memory }
    AppendDebugMessage('SafeCopy, not enough memory');
    exit;
  end;
  New (bufferPtr);              { Create the buffer, on the heap }
  fmSave := FileMode;           { Store the filemode }
  FileMode := 0;                { To read also read-only files }
  Assign (f1, fromFile);
  {$I-} Reset (f1, 1); {$I+}    { Note the record size 1, important! }
  i := IoResult;
  if I <> 0 then begin
    TellError('SafeCopy, Reset f1',i);
    exit;   { Does the file exist? }
  end;
  Assign (f2, toFile);
  {$I-} Reset (f2, 1); {$I+}    { Don't copy on an existing file }
  if IOResult = 0 then begin
    close (f2);
    Dispose (bufferPtr);          { Release the buffer from the heap}
    TellError('SafeCopy, Reset f2, file exists',0);
    exit;
  end;
  {$I-} Rewrite (f2, 1); {$I+}  { Open the target }
  i := IoResult;
  if I <> 0 then begin
    TellError('SafeCopy, Rewrite f2 ',i);
    exit;
  end;
  repeat                        { Do the copying }
    BlockRead (f1, bufferPtr^, bufferSize, readCount);
    {$I-} BlockWrite (f2, bufferPtr^, readCount, writeCount);
    {$I+}
    i := ioresult;
    if I <> 0 then begin
      close (f1);
      Dispose (bufferPtr);          { Release the buffer from the heap}
      TellError('SafeCopy, Blockwrite failed ',i);
      exit;
    end;
  until (readCount = 0) or (writeCount <> readCount);
  AppendDebugMessage('Safecopy copied '+ fromFile+ ' to '+ toFile+ ' '+ long2str(FileSize(f2))+ ' bytes');
  close (f1); close (f2);
  FileMode := fmSave;           { Restore the original filemode }
  Dispose (bufferPtr);          { Release the buffer from the heap}
end;  (* safecopy *)

 function TLogFile.CabrilloDetail(var R : log_rec) : string;
 var
   s : string;
 begin
{                              --------info sent------- -------info rcvd--------
 QSO: freq  mo date       time call          rst exch   call          rst exch   t
 QSO: ***** ** yyyy-mm-dd nnnn ************* nnn ****** ************* nnn ****** n
 QSO:  3799 PH 1999-03-06 0711 HC8N          59  700    W1AW          59  CT     0
 000000000111111111122222222223333333333444444444455555555556666666666777777777788
 123456789012345678901234567890123456789012345678901234567890123456789012345678901
}

   case r.band of
     B80: S := ' 3580';
     B40: S := ' 7080';
     B20: S := '14080';
     B15: S := '21080';
     B10: S := '28080';
   else
     S := '14000';
   end;
   S := S + ' RY ';
   S := S + DateToDateString('yyyy-mm-dd ', r.Date);
   S := S + TimeToTimeString('hhmm ', r.Ts);
   S := S + Copy(Pad(RttyIni^.GetProfileString('CALLSIGN','Cabrillo',''), 13), 1, 13) + ' ';
   CabrilloDetail := S;
  end;


end.
