{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit EditText;

interface


procedure FileEdit(InFName : String);

implementation

uses
  TpCrt,
  TpWindow,
  TpMemo,
  TpMouse,
  TpString,
  TpPick,
  Keys,
  Rectypes,
  DebugRtns,
  cColors,
  IniFile;

const
  StatusA  : array[Boolean] of Byte = ($2F, $70);
  ErrorA   : array[Boolean] of Byte = ($1F, $0F);
  TextA    : array[Boolean] of Byte = ($1B, $07);
  CtrlA    : array[Boolean] of Byte = ($1C, $0F);
  MouseA   : array[Boolean] of Byte = ($4E, $70);
  UserCmds : array[1..1] of EMtype = (EMnone);

var
  WP2 : WindowPtr;
  EMCB : EMcontrolBlock;
  Buffer : Pointer;
  BufSize : Word;
  FSize : LongInt;
  BandW : Boolean;
  FName : String[79];
  AbortNow : Boolean;

  function EditHelpStr(Choice : Word) : String; far;
  begin
    case Choice of
      1 : EditHelpStr := '<LEFT>      Cursor left 1 character';
      2 : EditHelpStr := '<RIGHT>     Cursor right 1 character';
      3 : EditHelpStr := '<CTL-LEFT>  Cursor left 1 word';
      4 : EditHelpStr := '<CTL-RIGHT> Cursor right 1 word';
      5 : EditHelpStr := '<HOME>      Cursor to beginning of line';
      6 : EditHelpStr := '<END>       Cursor to end of the line';
      7 : EditHelpStr := '<UP>        Cursor up 1 line';
      8 : EditHelpStr := '<DOWN>      Cursor down 1 line';
      9 : EditHelpStr := '<PGUP>      Scroll the display up one page';
      10 : EditHelpStr := '<PGDN>      Scroll the display down one page';
      11 : EditHelpStr := '<DEL>       Delete the character under the cursor';
      12 : EditHelpStr := '<BKSP>      Delete the character to the left of the cursor';
      13 : EditHelpStr := '<CTL-Y>     Delete the current line';
      14 : EditHelpStr := '<CTL-T>     Delete the word to the right of the cursor';
      15 : EditHelpStr := '<ENTER>     Start a new line';
      16 : EditHelpStr := '<INS>       Toggle insert mode on or off';
      17 : EditHelpStr := '<CTL-O><W>  Toggle word wrap on or off';
      18 : EditHelpStr := '<CTL-Q><L>  Restore original contents of the current line';
      19 : EditHelpStr := '<ESC>       Quit editing';
      20 : EditHelpStr := '<CTL-K><S>  Save the file and continue editing';
      21 : EditHelpStr := '<CTL-K><X>  Save the file and exit';
      22 : EditHelpStr := '<CTL-K><Q>  Abandon the file';
    end;
  end;


  procedure ClearMessageLine;
    {-Clear the message line}
  begin
    FastWrite(CharStr(' ', ScreenWidth), ErrorRow, 1, ErrorAttr);
  end;

  procedure DisplayMessage(Msg : String);
    {-Display a message at the top of the screen}
  begin
    ClearMessageLine;
    FastWrite(Msg, ErrorRow, 1, ErrorAttr);
    GotoXYabs(Length(Msg)+2, ErrorRow);
  end;

  procedure EditHelp;
  var
    Choice : word;
  begin
    Choice := 1;
    PickMatrix := 1;
    if not PickWindow(@EditHelpStr, 22, 18, 1, ScreenWidth,ScreenHeight-1, True, Colors^.PickColors,
                  'EDIT COMMANDS', Choice) then
      DisplayMessage('Sorry, not enough free RAM to display window! <CTL-K><X>=SAVE/EXIT, <ESC>=QUIT');
  end;


  procedure Abort(Msg : String);
  begin
    DisplayMessage(Msg);
    if ReadKeyWord = 0 then {} ;
    AppendDebugMessage(Msg);
    AbortNow := True;
  end;

  function YesNo(Msg : String) : Boolean;
    {-Get a response to a yes/no question. Return True for Y, False for N}
  var
    ChWord : Word;
    Ch : Char absolute ChWord;
  begin
    DisplayMessage(Msg);
    repeat
      ChWord := ReadKeyWord;
      Ch := Upcase(Ch);
    until (Ch = 'Y') or (Ch = 'N');
    YesNo := (Ch = 'Y');
    ClearMessageLine;
  end;
  procedure SaveFile;
    {-Save the file in the edit buffer}
  var
    MakeBackUp : boolean;
  begin
    MakeBackup := RttyIni^.GetProfileBool('MakeBackup','EDITOR', True);
    DisplayMessage('Saving file...');
    case SaveMemoFile(EMCB, FName, MakeBackUp) of
      mstOK :
        {file was saved} ;
      mstCreationError :
        Abort('Error creating '+FName);
      mstWriteError :
        Abort('Error writing to '+FName);
      mstCloseError :
        Abort('Error closing '+FName);
    end;
    ClearMessageLine;
  end;


  procedure ReadFile;
  begin
    case ReadMemoFile(Buffer^, BufSize, FName, FSize) of
      mstOK :
        {file read in OK} ;
      mstInvalidName :
        Abort(FName+' is an invalid pathname');
      mstNotFound :
        {file not found, we'll create it later} ;
      mstReadError :
        Abort('Error reading '+FName);
      mstTooLarge :
        Abort(FName+' is too large to edit');
      mstCloseError :
        Abort('Error closing '+FName);
    end;
  end;


  procedure FileEdit(InFName : String);
  var
    ExitCommand : EMtype;
    InsertOn, IndentOn, WordWrapOn : Boolean;
    TabSize, RightMargin           : byte;
    MaxFileSize : word;
    I : Integer;
  begin
    AbortNow := False;
    FName := Trim(InFName);

    if Length(FName) = 0 then Exit;

    {don't allow reading of partial files}
    AllowTruncation := False;
    HideMouse;
    with Colors^ do
      MakeWindow(WP2, 1, 1, ScreenWidth, ScreenHeight, False, True, True,
        WindowAttr, FrameAttr, HeaderAttr, '');
    DisplayWindow(WP2);
    ShowMouse;
    InsertOn := RttyIni^.GetProfileBool('Insert','EDITOR', True);
    IndentOn := RttyIni^.GetProfileBool('Indent','EDITOR', True);
    WordWrapOn := RttyIni^.GetProfileBool('WordWrap','EDITOR', True);
    TabSize  := RttyIni^.GetProfileByte('TabSize','EDITOR', 8);
    RightMargin := RttyIni^.GetProfileByte('RightMargin','EDITOR', 77);
    MaxFileSize := RttyIni^.GetProfileWord('MaxFileSize','EDITOR', 32);
    if MaxFileSize > 64 then begin
      MaxFileSize := 64;
      RttyIni^.SetOrAddProfileString('MaxFileSize','EDITOR', long2str(MaxFileSize));
    end;
    if MaxFileSize < 4 then begin
      MaxFileSize := 4;
      RttyIni^.SetOrAddProfileString('MaxFileSize','EDITOR', long2str(MaxFileSize));
    end;
    {allocate edit buffer}
    I := MaxAvail;
    if I > $7FFF then { 32K max buffer }
      BufSize := $7FFF
    else
      BufSize := I;
    GetMem(Buffer, BufSize);

    ReadFile;
    if AbortNow then begin
      HideMouse;
      KillWindow(WP2);
      ShowMouse;
      FreeMem(Buffer, BufSize);
      Exit;
    end;
    {use default status and error handlers}
    MemoStatusPtr := @MemoStatus;
    MemoErrorPtr := @MemoError;

    {set attribute for status and error lines}
    BandW := (CurrentMode = 7) or (CurrentMode = 2);
    StatusAttr := StatusA[BandW];
    ErrorAttr := ErrorA[BandW];
    ErrorRow := 1;
    StatusRow := ScreenHeight;

    if MouseInstalled then begin
      ShowMouse;
      EnableMemoMouse;
    end;

    {EMuser0 = save file and continue: ^KS, F2}
    if not AddMemoCommand(EMuser0, 2, Ord(^K), Ord(^S)) then {} ;

    {EMuser1 = save file and exit: ^KX, ^F2}
    if not AddMemoCommand(EMuser1, 2, Ord(^K), Ord(^X)) then {} ;

    {EMuser2 = abandon file: ^KQ, AltF2}
    if not AddMemoCommand(EMuser2, 2, Ord(^K), Ord(^Q)) then {} ;

    {EMuser3 = my help: F1}
    if not AddMemoCommand(EMuser3, 1, kbF1, 0 ) then {} ;

    {initialize the edit control block}
    InitControlBlock(
      EMCB,                       {control block}
      1,                          {left column of edit window}
      2,                          {top row of edit window}
      ScreenWidth,                {right column of edit window}
      ScreenHeight-1,             {bottom row of edit window}
      Colors^.WindowAttr,         {attribute for normal text}
      Colors^.HiLiteAttr,         {attribute for control characters}
      InsertOn,                   {insert mode on?}
      IndentOn,                   {auto-indent on?}
      WordWrapOn,                 {word wrap on?}
      TabSize,                    {distance between tab stops}
      0,                          {help index}
      RightMargin,                {right margin}
      MaxInt,                     {maximum number of lines}
      BufSize,                    {size of edit buffer}
      Buffer^);                   {edit buffer}

    repeat
      ExitCode := EditMemo(EMCB, False, UserCmds);

      {process exit command}
      case ExitCode of
        EMuser0,                  {save and continue}
        EMuser1 :                 {save and quit}
          SaveFile;
        EMquit,                   {quit}
        EMuser2 :                 {abandon file}
          if not EMCB.Modified then
            ExitCode := EMquit
              {file was modified--verify that user wants to quit}
          else if YesNo('File modified. Quit anyway?') then
            ExitCode := EMquit
          else
            ExitCode := EMnone;
        EMuser3 :                 {show help}
          EditHelp;
      end;
    until (ExitCode = EMquit) or (ExitCode = EMuser1) or (AbortNow);
    HideMouse;
    KillWindow(WP2);
    ShowMouse;
    FreeMem(Buffer, BufSize);
  end;
end.
