{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
unit EventRate;

interface

uses TpDAte;

const MaxEvents = 100;

type

  PEventRate = ^TEventRate;
  TEventRate = object
     Event : array[1..MaxEvents] of LongInt;
     Start_dt  : date;
     Events_Recorded : integer;
     Tm_Last_Updated : time;
     constructor Init;
     destructor  Done; virtual;
     procedure   NewEvent(Event_Dt : date ; Event_Tm : time); virtual;
     procedure   EventNow; virtual;
     function    DateTimeToSeconds(Event_Dt : date ; Event_Tm : time) : longint; virtual;
     function    DiffInSeconds(slot : byte) : longint; virtual;
     function    RatePerHourLast(Count : byte) : integer; virtual;
     procedure   Update; virtual;
  end;

implementation

uses
  DebugRtns,
  cClock,
  IniFile,
  TpString;

constructor TEventRate.Init;
var
  i : integer;
begin
  for i := 1 to MaxEvents do Event[i] := 0;
  Start_Dt := Clock^.DateUtc;
  Events_Recorded := 0;
  Tm_Last_Updated := 0;
  {$IFDEF DEBUG}
  AppendDebugMessage('EventRate created');
  {$ENDIF}
end;

destructor TEventRate.Done;
var
  i : integer;
begin
  {$IFDEF DEBUG}
  for i := 1 to MaxEvents do
    AppendDebugMessage('Event#'+leftpad(long2str(i),3)+' '++long2str(Event[i])+
    ' DiffInSeconds '+long2str(DiffInSeconds(i)) );
  AppendDebugMessage('EventRate destroyed');
  {$ENDIF}
end;

function  TEventRate.DateTimeToSeconds(Event_Dt : date ; Event_Tm : time) : longint;
const
  SecsInAMinute = 60;
  SecsInAHour   = 60 * SecsInAMinute;
  SecsInADay    = 24 * SecsInAHour;
var
  ye,mo,da : integer; {year month day}
  hh,mm,ss : byte; {hours minutes seconds}
  Days_to_secs   : longint;
  Hours_to_secs   : longint;
  Minutes_to_secs : longint;
begin
  DateDiff(Start_Dt, Event_dt, da, mo, ye);
  if (da > 1) or (mo > 1) or (ye > 1) then begin
    if RttyIni^.GetProfileBool('EVENTRAT.DATETIMETOSECONDS', 'DEBUG', False) then
      AppendDebugMessage('EVENTRAT: DateDiff Warning StrtTm:' +
                          DateToDateString('yyyy-mm-dd ', Start_Dt) + 'EvtDt:' +
                          DateToDateString('yyyy-mm-dd ', Event_Dt) + 'EvtTm:' +
                          TimeToTimeString('hh:mm:ss ', Event_Tm) );
  end;
  TimeToHMS(Event_Tm,hh,mm,ss);
  Days_to_secs      := longint(da) * longint(SecsInADay);
  Hours_to_secs     := longint(hh) * longint(SecsInAHour);
  Minutes_to_secs   := longint(mm) * longint(SecsInAMinute);
  DateTimeToSeconds := Hours_to_secs + Minutes_to_secs + longint(ss);
end;

function  TEventRate.RatePerHourLast(Count : byte) : integer;
var
  amt_time_for_count : longint;
  temp_rate          : longint;
begin
  if Count > Events_Recorded then Count := Events_Recorded;
  amt_time_for_count := DiffInSeconds(Count);
  if amt_time_for_count = 0 then
    temp_rate := 0
  else
    temp_Rate := longint(3600) * longint(count) div amt_time_for_count;
  if temp_Rate > 999 then temp_rate := 999; {max rate = 999 per hour}
  {$IFDEF DEBUG}
  AppendDebugMessage('3600 * '+long2str(count)+' div '+long2str(amt_time_for_count)+' = '+long2str(temp_rate));
  {$ENDIF}
  RatePerHourLast := temp_rate;
end;

procedure TEventRate.EventNow;
begin
  NewEvent(Today,Clock^.TimeUtc);
  {$IFDEF DEBUG}
  AppendDebugMessage('TEventRate.NewEventNow');
  {$ENDIF}
end;

procedure TEventRate.NewEvent(Event_Dt : date ; Event_Tm : time);
var
  i : integer;
  l : longint;
begin
  if Events_Recorded < MaxEvents then inc(Events_Recorded);
  if Events_Recorded = 1 then begin { first event !}
    Start_Dt := Event_Dt;
    l := DateTimeToSeconds(Event_dt, Event_Tm);
    for i := 1 to MaxEvents do Event[i] := l;
  end
  else begin
    for i := MaxEvents downto 2 do Event[i] := Event[i-1];
    Event[1] := DateTimeToSeconds(Event_Dt, Event_Tm);
  end;
  {$IFDEF DEBUG}
  AppendDebugMessage('EVENTRAT: New Event @ StrtTm:' +
                      DateToDateString('yyyymmdd ', Start_Dt) + 'EvtDt:' +
                      DateToDateString('yyyymmdd ', Event_Dt) + 'EvtTm:' +
                      TimeToTimeString('hhmmss ', Event_Tm));

  {$ENDIF}
end;

function TEventRate.DiffInSeconds(slot : byte) : LongInt;
{between slot1 and the parameter }
begin
  if Slot > Events_Recorded then Slot := Events_Recorded;
  if Slot = 0 then Slot := 1;
  DiffInSeconds := Abs(DateTimeToSeconds(Clock^.DateUtc,Clock^.TimeUtc) - Event[Slot] + 1);
end;

  procedure TEventRate.Update;
  begin
    { should be overridden as needed }
  end;



end.