{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
program Extract;

uses
  TpString,
  TpDate,
  TpDos,
  TAHigh,
  TAccess,
  Diskio,
  RecTypes;

var
  BF1,BF3,IF1,IF3 : String[79];
  DB1,DB3             : DataSet;
  R1, R3              : Log_Rec;
  I1, I3              : String[8];
  EOF1                : Boolean;
  C1,C3                : word;
  Band                 : BandType;
  OverWrite            : boolean;
  OverStr              : string[2];

const
  HighIndex : string[8] = #255#255#255#255#255#255#255#255;
  LowIndex  : string[8] = #00#00#00#00#00#00#00#00;

procedure NotFound(S : string);
begin
  writeln('Error! Cannot find '+S);
  halt;
end;

procedure InsertFromR1;
begin
  Move(R1, R3, SizeOf(R1));
  Move(I1, I3, SizeOf(I1));
  TAInsert(DB3, R3, I3); {put R1 into DB3}
  inc(c3);
  TANext(DB1, R1, I1);
  if not OK then begin
    EOF1 := True;
    Move(HighIndex, I1, SizeOf(HighIndex));
  end
  else inc(c1);
end;

procedure EraseFile(S : String);
var
 F:  file;
 Ch: Char;
begin
 Assign(F, S);
 {$I-}
 Reset(F);
 {$I+}
 if IOResult <> 0 then Exit;
 Close(F);
 Erase(F);
end;

begin
  writeln;
  WriteLn('BIN File Extractor Copyright(c) 1997 by Wyvern Technology, Inc.');
  writeln;

  if (ParamCount < 3) or (ParamStr(1) = '/?') or (ParamStr(1) = '?') then begin
    WriteLn('Usage:  EXTRACT BinFile1 Band OutFile [/O[VERWRITE]]');
    writeln;
    Writeln('   eg:  extract cqww96.bin 20 A:cqww20m /O');
    writeln('        This example will copy all the 20 meter contacts out');
    writeln('        of the CQWW96.BIN file and put them into a file called');
    writeln('        cqww20m on floppy drive A:  This will overwrite the');
    writeln('        existing floppy file.');
    writeln;
    writeln('* * * WARNING! * * * Please practice using this program!  It works');
    writeln('as promised.  Improper use of this program could lead to data loss.');
    Halt(1);
  end;

  Band := Tot;
  if (ParamStr(2) = '80') then Band := B80;
  if (ParamStr(2) = '40') then Band := B40;
  if (ParamStr(2) = '20') then Band := B20;
  if (ParamStr(2) = '15') then Band := B15;
  if (ParamStr(2) = '10') then Band := B10;
  if Band = Tot then begin
    writeln('Error! Enter a valid band such as 80, 40, 20, 15, or 10');
    halt;
  end;

  BF1 := ForceExtension(ParamStr(1),'BIN');
  BF3 := ForceExtension(ParamStr(3),'BIN');

  IF1 := ForceExtension(ParamStr(1),'NDX');
  IF3 := ForceExtension(ParamStr(3),'NDX');

  if not ExistFile(BF1) then NotFound(BF1);
  if not ExistFile(IF1) then NotFound(IF1);

  TAOpen(DB3, BF3, SizeOf(Log_Rec), IF3, 8-1);

  OverStr := (Copy(ParamStr(4),1,2));
  OverWrite := OverStr[2] in ['o','O'];

  If (OK) and (not OverWrite) then begin
    WriteLn('Error! Delete the output bin file before running merge');
    halt;
    end;

  If OverWrite then begin
    EraseFile(BF3);
    EraseFile(IF3);
  end;

  TACreate(DB3, BF3, SizeOf(Log_Rec), IF3, 8-1);

  TAOpen(DB1, BF1, SizeOf(Log_Rec), IF1, 8-1);

  TAReset(DB1);
  TAReset(DB3);

  FillChar(R1, SizeOf(R1), #00);
  FillChar(R3, SizeOf(R3), #00);

  I1 := LowIndex;
  I3 := LowIndex;

  C1 := 0;
  C3 := 0;

  TANext(DB1, R1, I1);
  if not OK then begin
    EOF1 := True;
    Move(HighIndex, I1, SizeOf(HighIndex));
    end else
  inc(c1);

  while not EOF1 do
    if R1.Band = Band then InsertFromR1
                      else begin
                        TANext(DB1, R1, I1);
                        if not OK then begin
                          EOF1 := True;
                          Move(HighIndex, I1, SizeOf(HighIndex));
                        end
                        else inc(c1);
                      end;

  TAClose(DB1);
  TAClose(DB3);
  writeln;
  WriteLn('Extraction Summary:');
  writeln;
  writeln('All ', C3, ' of the ',ParamStr(2),' meter records were copied');
  writeln('from the ',StUpCase(BF1), ' file to the ', StUpCase(BF3), ' file.');
end.



