{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit LowLvl;

interface

uses RecTypes,
     ApPort,
     ApMisc,
     ApUart,
     DiskIo,
     OoCom;

procedure InitLowLvl;

procedure Alarm;
procedure AnyKey(Row, col : Byte);
procedure Beep;
function ClassStr2Class(const Cls : string) : ClassType;
procedure CtrlEnter(Row, col : Byte);
function  CarotToCtl(const S : string) : string;
procedure ChangeHandShake(const SomeUP : AbstractPortPtr; const NewHandShake : HandShakeType);
procedure CtlToCarot(var S : string);
function ContestStr2Contest(const Cts : string) : ContestType;
procedure DeleteFile(FileN : String);
function DriveReady(Drive : Char) : boolean;
function ExistDir(const Path : string) : boolean;
function FreqToBand(freq : string) : bandtype;
function GetFileSize(const FileName : string) : longint;
function GoodFileName(FileN : String) : Boolean;
function HandShakeStringToHandShake(const str : string) : HandShakeType;
procedure MyDelayTicks(const Ticks : longint);
procedure MyDelaySecs(const Secs : longint);
function NotSameCall(a, B : String) : Boolean;
function NeedStr(flag : boolean) : string;
function NeedAttr(flag : boolean) : byte;
function ParityStringToParity(const S : string) : ParityType;
procedure RenameFile(OldName, NewName : string);
function RomanMonth(Month : Integer) : String;
function SameCall(a, B : String) : Boolean;
procedure SuckSpaces(var a : String);
function TrimTrailSpaces(S : string) : string;
procedure MoveLogRec(const FromRec : log_rec; var ToRec : log_rec);
procedure ShellToDos;
function  Filename(const Extension : string) : string;
function  SpeedToStr(const Speed : byte) : string;
function  SpeedToByte(const Speed : string) : byte;
function  CurrentBand : BandType;
procedure LoadClickDelims;

implementation
uses
  ColorDef,
  TpDate,
  TpCrt,
  TpDos,
  TpMouse,
  TpWindow,
  TpString,
  ApTimer,
  Dos,
  cCountry,
  cMsgWin,
  cRttyWin,
  cColors,
  TpEntry,
  keys,
  Routines,
  TpEdit,
  TncControl,
  DebugRtns,
  IniFile;

var
  TextAttr       : Byte;
  B              : Byte;
  RxCol, RxRow   : Byte;

function FreqToBand(freq : string) : bandtype;
begin
  FreqToBand := B20;
  if Trim(copy(freq,1,3)) = '3' then FreqToBand := B80;
  if Trim(copy(freq,1,3)) = '7' then FreqToBand := B40;
  if Trim(copy(freq,1,3)) = '14' then FreqToBand := B20;
  if Trim(copy(freq,1,3)) = '21' then FreqToBand := B15;
  if Trim(copy(freq,1,3)) = '28' then FreqToBand := B10;
end;

procedure RenameFile(OldName, NewName : string);
var
  f : file;
begin
  Assign(f,OldName);
  Rename(f,NewName);
end;

  function SameCall(a, B : String) : Boolean;
  begin
    SameCall := (Pad(a, 12) = Pad(B, 12));
  end;

  function NotSameCall(a, B : String) : Boolean;
  begin
    NotSameCall := (Pad(a, 12) <> Pad(B, 12));
  end;

  procedure CtrlEnter(Row, col : Byte);
  begin
    FastWrite('Ctrl-Enter to continue', Row, col, Colors^.WindowAttr);
  end;

  procedure AnyKey(Row, col : Byte);
  begin
    FastWrite('Press Any Key to continue', Row, col, Colors^.WindowAttr);
  end;

  procedure Beep;
  var
    BeepFreq : word;
    BeepLen  : word;
  begin
    if RttyIni^.GetProfileBool('OK', 'BEEP', True) then begin
      BeepFreq := RttyIni^.GetProfileWord('Frequency', 'BEEP', 800);
      BeepLen  := RttyIni^.GetProfileWord('Duration', 'BEEP', 250);
      Sound(BeepFreq);
      Delay(BeepLen);
      NoSound;
    end;
  end;

  procedure Alarm;
  var
    BeepLen  : word;
  begin
    BeepLen  := RttyIni^.GetProfileWord('Duration', 'BEEP', 250);
    Beep;
    Delay(BeepLen);
    Beep;
    Delay(BeepLen);
    Beep;
  end;

  function RomanMonth(Month : Integer) : String;
  const
    RMonths        : array[1..12] of String[4] =
    ('I', 'II', 'III', 'IV', 'V', 'VI', 'VII', 'VIII', 'IX', 'X', 'XI', 'XII');
  begin
    RomanMonth := RMonths[Month];
  end;

  procedure SuckSpaces(var a : String);
  begin
    a := Trim(a);
    while (Length(a) > 0) and (Pos(' ', a) > 0) do
      Delete(a, Pos(' ', a), 1);
  end;

  function TrimTrailSpaces(S : string) : string;
    {-Return a string with trailing white space removed}
  var
    SLen : Byte absolute S;
  begin
    while (SLen > 0) and (S[SLen] = ' ') do
      Dec(SLen);
    TrimTrailSpaces := S;
  end;

  procedure DeleteFile(FileN : String);
  var
    FileReference  : file;
  begin
    if Trim(FileN) <> '' then
      if ExistFile(FileN) then begin
        {$I-}
        Assign(FileReference, FileN);
        Erase(FileReference);
        {$I-}
      end;
  end;


  function GoodFileName(FileN : String) : Boolean;
  var
    BadChars       : set of Char;
    FileS          : String[12];
    Idx            : Byte;
    Result         : Boolean;
  begin
    BadChars := [#00..#21, ',', '"', '/', '\', '[', ']', ':', '|', '<', '>', '+', '=', ';', ',',' '];
    FileS := Trim(JustFileName(FileN));
    Result := True;
    for Idx := 1 to Length(FileS) do
      if FileS[Idx] in BadChars then Result := False;
    GoodFileName := Result;
  end;

function DriveReady(Drive : Char) : boolean;
begin
  DriveReady := (DiskFree(ord(UpCase(Drive))-64) <> -1)
end;

  procedure CtlToCarot(var S : string);
  var
    I : integer;
  begin
    for i := 1 to length(S) do
      if s[i] in [^A..^[] then begin
        s[i] := chr(ord(s[i])+64);
        insert('^',s,i);
      end;
  end;

  function  CarotToCtl(const S : string) : string;
  var
    I : integer;
    r : string[10];
  begin
    r := s;
    for i := 1 to length(r) do
      if (r[i] = '^') and (r[i+1] in ['A'..']']) then begin
        delete(r,i,1);
        r[i] := chr(ord(r[i])-64);
      end;
    CarotToCtl := r;
  end;

procedure MyDelayTicks(const Ticks : longint);
var
  et : eventtimer;
begin
  if Ticks = 0 then Exit;
  newtimer(et,Ticks);
  repeat until timerexpired(et);
end;

procedure MyDelaySecs(const Secs : longint);
var
  et : eventtimer;
begin
  if Secs = 0 then Exit;
  newtimerSecs(et,Secs);
  repeat until timerexpired(et);
end;

function GetFileSize(const FileName : string) : longint;
var
  F : text;
begin
  GetFileSize := -1;
  if not ExistFile(FileName) then Exit;
  Assign(F,FileName);
  if IOResult > 0 then Exit;
  Reset(F);
  if IOResult > 0 then Exit;
  GetFileSize := TextFileSize(F);
  Close(F);
end;

procedure MoveLogRec(const FromRec : log_rec; var ToRec : log_rec);
begin
  ToRec.ToolBoxNumber  := FromRec.ToolBoxNumber ;
  ToRec.VersionControl := FromRec.VersionControl ;
  ToRec.Contest        := FromRec.Contest  ;
  ToRec.Date           := FromRec.Date  ;
  ToRec.Ts             := FromRec.Ts ;
  ToRec.Band           := FromRec.Band ;
  ToRec.Mode           := FromRec.Mode ;
  ToRec.Call           := FromRec.Call ;
  ToRec.His_Country    := FromRec.His_Country;
  ToRec.His_RST        := FromRec.His_RST ;
  ToRec.His_Qth        := FromRec.His_Qth ;
  ToRec.His_Zone       := FromRec.His_Zone ;
  ToRec.His_Serno      := FromRec.His_Serno ;
  ToRec.His_Time       := FromRec.His_Time ;
  ToRec.My_Rst         := FromRec.My_Rst ;
  ToRec.My_Qth         := FromRec.My_Qth ;
  ToRec.My_Zone        := FromRec.My_Zone ;
  ToRec.My_Serno       := FromRec.My_Serno ;
  ToRec.My_Time        := FromRec.My_Time ;
  ToRec.Point          := FromRec.Point ;
  ToRec.New_Zone       := FromRec.New_Zone ;
  ToRec.New_Country    := FromRec.New_Country  ;
  ToRec.New_Qth        := FromRec.New_Qth ;
  ToRec.New_Continent  := FromRec.New_Continent ;
  ToRec.Dupe           := FromRec.Dupe   ;
  ToRec.Comment        := FromRec.Comment  ;
end;

  var
    ScreenBuffPtr : Pointer;

  procedure DOSUserExit(ActionCode : ActionCodeType; Param : Word); far;
  begin
    Case ActionCode of
      ExecSaveScreen :
        begin
          SaveWindow(1,1,ScreenWidth,ScreenHeight,False,ScreenBuffPtr);
          window(1,1,ScreenWidth,ScreenHeight);
          ClrScr;
          Writeln('Type EXIT to return to RTTY');
        end;
      ExecShowMemory :
        Writeln('Approximately ',Param,' K bytes available.');
      ExecPauseAfterRun :
        begin
          Write('Press any key to continue...');
          ReadKeyWord;
        end;
      ExecRestoreScreen :
        RestoreWindow(1,1,ScreenWidth,ScreenHeight,False,ScreenBuffPtr);
    end;
  end;

  procedure ShellToDos;
  const
    UnMsg = 'DOS Shell Error: ';
  begin
    DisableEventHandling;
    GetMem(ScreenBuffPtr,2 * Screenwidth * ScreenHeight);
    case ExecDos('',True,@DOSUserExit) of
      0  : {success, get out};
      -1,-4 : MsgWin^.Display(UnMsg+'Insufficient memory');
      -2 : MsgWin^.Display(UnMsg+'Setblock error before call');
      -3 : MsgWin^.Display(UnMsg+'Setblock error after call');
    end;
    FreeMem(ScreenBuffPtr,2 * Screenwidth * ScreenHeight);
    ScreenBuffPtr := NIL;
    RttyWin^.ReInitAfterShell;
    EnableEventHandling;
  end;

procedure InitLowLvl;
begin
  if RttyIni^.GetProfileByte('AutoCr', 'PROFILE', Lo(ScreenWidth)-17) < 20 then
    RttyIni^.SetOrAddProfileString('AutoCr','PROFILE','20');
  if RttyIni^.GetProfileByte('AutoCr', 'PROFILE', Lo(ScreenWidth)-17) > Lo(ScreenWidth)-13 then
    RttyIni^.SetOrAddProfileString('AutoCr','PROFILE',long2str(ScreenWidth-13));
  RxCol := 1;
  RxRow := 1;
end;

  function ParityStringToParity(const S : string) : ParityType;
  begin
    if Trim(S) = '' then ParityStringToParity := NoParity
    else
    case Upcase(S[1]) of
      'N' : ParityStringToParity := NoParity;
      'E' : ParityStringToParity := EvenParity;
      'O' : ParityStringToParity := OddParity;
      'M' : ParityStringToParity := MarkParity;
      'S' : ParityStringToParity := SpaceParity;
    else
      ParityStringToParity := NoParity;
    end;
  end;

  function ClassStr2Class(const Cls : string) : ClassType;
  var
    S : string[22];
    CL : ClassType;
  begin
    ClassStr2Class := SingleAll;
    s := Trim(StUpCase(Cls));
    if Trim(S) = '' then Exit;
    for CL := Class_First to Class_Last do
      if S = StUpCase(Trim(Classes[CL])) then ClassStr2Class := CL;
  end;

  function ContestStr2Contest(const Cts : string) : ContestType;
  var
    Ct : ContestType;
  begin
    ContestStr2Contest := Contest_First;
    if Trim(Cts) = '' then Exit;
    for Ct := Contest_First to Contest_Last do
      if Cts = Trim(ContestName[Ct]) then ContestStr2Contest := Ct;
  end;

  function HandShakeStringToHandShake(const str : string) : HandShakeType;
  var
    S : string;
  begin
    HandShakeStringToHandShake:= NoHandShake;
    s := Trim(Str);
    if Trim(S) = '' then begin
      Exit;
    end;
    s := StUpCase(S);
    if (Pos('DTR',S) > 0) or
      (Pos('DSR',S) > 0) then HandShakeStringToHandShake := DtrDsr;
    if (Pos('CTS',S) > 0) or
      (Pos('RTS',S) > 0) then HandShakeStringToHandShake:= RtsCts;
    if (Pos('SOFT',S) > 0) or
      (Pos('XON',S) > 0) or
      (Pos('XOFF',S) > 0) then HandShakeStringToHandShake:= SoftHandShake;
  end;


  procedure ChangeHandShake(const SomeUP : AbstractPortPtr; const NewHandShake : HandShakeType);
  var
    ComPort : ComNameType;
  begin
    ComPort := SomeUP^.ComPortName;
    SomeUP^.HWFlowDisable;
    {$IFDEF DEBUG}
    if AsyncStatus <> ecOk then AppendDebugMessage('LowLvl.ChangeHandShake.HWFlowDisable, AsyncStatus='+long2str(AsyncStatus));
    {$ENDIF}
    SomeUP^.SWFlowDisable;
    {$IFDEF DEBUG}
    if AsyncStatus <> ecOk then AppendDebugMessage('LowLvl.ChangeHandShake.SWFlowDisable, AsyncStatus='+long2str(AsyncStatus));
    {$ENDIF}
    case NewHandShake of
      NoHandShake   : { dont do anything! };
      SoftHandShake : begin
                        SomeUP^.SWFlowEnable(1024, 256);
                        {$IFDEF DEBUG}
                        if AsyncStatus <> ecOk then
                          AppendDebugMessage('LowLvl.ChangeHandShake.SWFlowEnable, AsyncStatus='+long2str(AsyncStatus));
                        {$ENDIF}
                      end;
      RtsCts        : begin
                        SomeUP^.HWFlowEnable(1024, 128, hfUseRts+hfRequireCts);
                        {$IFDEF DEBUG}
                        if AsyncStatus <> ecOk then
                          AppendDebugMessage('LowLvl.ChangeHandShake.HWFlowEnable1, AsyncStatus='+long2str(AsyncStatus));
                        {$ENDIF}
                      end;
      DtrDsr        : begin
                        SomeUP^.HWFlowEnable(1024, 128, hfUseDtr+hfRequireDsr);
                        {$IFDEF DEBUG}
                        if AsyncStatus <> ecOk then
                          AppendDebugMessage('LowLvl.ChangeHandShake.HWFlowEnable2, AsyncStatus='+long2str(AsyncStatus));
                        {$ENDIF}
                      end;
    end;
  end;

function  Filename(const Extension : string) : string;
begin
  FileName := ForceExtension(InputFileName,Extension);
end;

  function NeedStr(flag : boolean) : string;
  begin
    case Flag of
      true : NeedStr := 'WORKED';
      false: NeedStr := 'NEEDED';
    end;
  end;

  function NeedAttr(flag : boolean) : byte;
  begin
    case Flag of
      true : NeedAttr := Colors^.WindowAttr;
      false: NeedAttr := Colors^.HiLiteAttr;
    end;
  end;

  function ExistDir(const Path : string) : boolean;
  var
    CurDir : string[79];
  begin
    GetDir(0, CurDir);
    ChDir(Path);
    ExistDir := IoResult = 0;
    ChDir(CurDir);
  end;

function  SpeedToStr(const Speed : byte) : string;
begin
  Case Speed of
    0 : SpeedToStr := '45';
    1 : SpeedToStr := '50';
    2 : SpeedToStr := '57';
    3 : SpeedToStr := '74';
    4 : SpeedToStr := '100';
  else
    SpeedToStr := '45';
  end;
end;

function  SpeedToByte(const Speed : string) : byte;
begin
  if Speed = '45'  then SpeedToByte := 0 else
  if Speed = '50'  then SpeedToByte := 1 else
  if Speed = '57'  then SpeedToByte := 2 else
  if Speed = '74'  then SpeedToByte := 3 else
  if Speed = '100' then SpeedToByte := 4 else
  SpeedToByte := 0;
end;


function  CurrentBand : BandType;
begin
  if not Assigned(RttyIni) then
    CurrentBand := b20
  else
    case RttyIni^.GetProfileInt('Band', 'CURRENT', 20) of
      80 : CurrentBand := b80;
      40 : CurrentBand := b40;
      20 : CurrentBand := b20;
      15 : CurrentBand := b15;
      10 : CurrentBand := b10;
    end;
end;

procedure LoadClickDelims;
var
  Delims   : string[29];
  OneDelim : string[4];
  b : byte;
  w : word;
  NumEntries : byte;
begin
  Delims := RttyIni^.GetProfileString('DelimSet','MOUSE','$20,$2C,$2D,$2E');
  NumEntries := WordCount(Delims,[',']);
  if (NumEntries = 0) or (NumEntries > 9) then Exit;
  for b := 1 to NumEntries do begin
    OneDelim := ExtractWord(b,Delims,[',']);
    if Str2Word(OneDelim, w) then
      if w < 127 then ClickDelims[b] := chr(lo(w))
                 else begin
                   ClickDelims[b] := #$00;
                   AppendDebugMessage('Error loading [MOUSE] DelimSet :'+Delims);
                 end;
  end;
end;

end.
