{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
program Merge;

uses
  TpString,TpDate,TpDos,TAHigh,TAccess;

type
  ContestType    = (ARRL, CQWW, BART, DXPD, SART, JART, WPXC, VOLT,
                    WAED, NAQP, ANAR, EAWW, SPDX, ARII, RRWW, TARA,
                    SPRI, ARFD, HSSC, SCCC, BSPR);

    BandType       = (B80, B40, B20, B15, B10, TOT);


  VersionType    = (V1_1_6, V2_0_0);

  ModeType       = (RTTYMODE, ASCII, AMTOR, Packet, SSB, CW, Other);

  Log_Rec        = record
                     ToolBoxNumber  : Integer;
                     VersionControl : VersionType;
                     Contest        : ContestType;
                     Date           : Date;
                     Ts             : Time;
                     Band           : BandType;
                     Mode           : ModeType;
                     Call           : String[12];
                     His_Country    : String[5];
                     His_RST        : Byte;
                     His_Qth        : String[3]; {grp# in waedc}
                     His_Zone       : Byte; {age in jarts, qtcs in waedc}
                     His_Serno      : Word;
                     His_Time       : Time;
                     My_Rst         : Byte;
                     My_Qth         : String[3];
                     My_Zone        : Byte;
                     My_Serno       : Word;
                     My_Time        : Time;
                     Point          : Byte;
                     New_Zone       : Boolean;
                     New_Country    : Boolean;
                     New_Qth        : Boolean; {in wae, F=not sent, T=sent}
                     New_Continent  : Boolean;
                     Dupe           : Boolean;
                     Comment        : String[25];
                   end;
var
  BF1      : String[79]; {binfile 1 name}
  BF2      : String[79]; {binfile 2 name}
  BF3      : String[79]; {newbinfile name}
  IF1      : String[79]; {indexfile1 name}
  IF2      : String[79]; {indexfile2 name}
  IF3      : String[79]; {newindexfile name}
  DB1      : DataSet;    {input database handle 1}
  DB2      : DataSet;    {input database handle 2}
  DB3      : DataSet;    {output database handle 3}
  R1       : Log_Rec;
  R2       : Log_Rec;
  R3       : Log_Rec;
  I1, I2, I3              : String[8];
  EOF1,EOF2               : Boolean;
  C1,C2,C3                : word;
  OverWrite,SomeDupes     : Boolean;
  OverStr                 : string[2];

const
  HighIndex : string[8] = #255#255#255#255#255#255#255#255;
  LowIndex  : string[8] = #00#00#00#00#00#00#00#00;

procedure NotFound(S : string);
begin
  writeln('Error! Cannot find '+S);
  halt;
end;

procedure InsertFromR1;
begin
  Move(R1, R3, SizeOf(R1));
  Move(I1, I3, SizeOf(I1));
  TAInsert(DB3, R3, I3); {put R1 into DB3}
  inc(c3);
  TANext(DB1, R1, I1);
  if not OK then begin
    EOF1 := True;
    Move(HighIndex, I1, SizeOf(HighIndex));
  end
  else inc(c1);
end;

procedure InsertFromR2;
begin
  Move(R2, R3, SizeOf(R2));
  Move(I2, I3, SizeOf(I2));
  TAInsert(DB3, R3, I3); {put R2 into DB3}
  inc(c3);
  TANext(DB2, R2, I2);
  if not OK then begin
    EOF2 := True;
    Move(HighIndex, I2, SizeOf(HighIndex));
  end
  else inc(c2);
end;

procedure EraseFile(S : String);
var
 F:  file;
 Ch: Char;
begin
 Assign(F, S);
 {$I-}
 Reset(F);
 {$I+}
 if IOResult <> 0 then Exit;
 Close(F);
 Erase(F);
end;

function CompareIndexes : CompareType;
var
  T1, T2 : Time;
  TS1, TS2 : string[10];
begin
  T1 := RoundToNearestMinute(R1.Ts,True);
  T2 := RoundToNearestMinute(R2.ts,True);
  TS1 := TimeToTimeString('hh:mm:ss',T1);
  TS2 := TimeToTimeString('hh:mm:ss',T2);
  if R1.Date < R2.Date then begin
    CompareIndexes := Less;
    Exit;
    end;
  if R1.Date > R2.Date then begin
    CompareIndexes := Greater;
    Exit;
    end;
  if T1 < T2 then begin
    CompareIndexes := Less;
    Exit;
    end;
  if T1 > T2 then begin
    CompareIndexes := Greater;
    Exit;
    end;
  { equal dates & times so far}
  CompareIndexes := Equal;
end;



begin
  writeln;
  WriteLn('BIN File Merger Copyright(c) 1997 by Wyvern Technology, Inc.');
  writeln;

  if (ParamCount < 3) or (ParamStr(1) = '/?') or (ParamStr(1) = '?') then begin
    WriteLn('Usage:  MERGE BinFile1 BinFile2 NewBinFile /O');
    Writeln;
    Writeln('Merges two .BIN files together to create a new .BIN file.');
    Writeln('The /O parameter allows overwriting of the output .BIN file.');
    writeln;
    writeln('* * * WARNING! * * * Please practice using this program!  It works');
    writeln('as promised.  Improper use of this program could lead to data loss.');
    Halt(1);
  end;

  BF1 := StUpCase(ForceExtension(ParamStr(1),'BIN'));
  BF2 := StUpCase(ForceExtension(ParamStr(2),'BIN'));
  BF3 := StUpCase(ForceExtension(ParamStr(3),'BIN'));

  IF1 := StUpCase(ForceExtension(ParamStr(1),'NDX'));
  IF2 := StUpCase(ForceExtension(ParamStr(2),'NDX'));
  IF3 := StUpCase(ForceExtension(ParamStr(3),'NDX'));

  if not ExistFile(BF1) then NotFound(BF1);
  if not ExistFile(BF2) then NotFound(BF2);
  if not ExistFile(IF1) then NotFound(IF1);
  if not ExistFile(IF2) then NotFound(IF2);

  if (BF1 = BF3) or (IF1 = IF3) then begin
    writeln('Error!  Don''t try to overwrite the files you are merging!');
    writeln('If you need to do this, then write a small batch file to');
    writeln('perform the merge then delete/rename the new file name.');
    writeln;
    writeln('* * * WARNING! * * * Please practice using this program!  It works');
    writeln('as promised.  Improper use of this program could lead to data loss.');
    Halt(1);
  end;


  OverStr := Copy(PAramStr(4),1,2);
  OverWrite := OverStr[2] in ['o','O'];

  If (ExistFile(BF3) or ExistFile(IF3) ) and (not OverWrite) then begin
    WriteLn('Error! Delete the output bin file before running merge');
    WriteLn('or use the /O parameter.');
    halt;
    end;
  If OverWrite then begin
    EraseFile(BF3);
    EraseFile(IF3);
  end;

  writeln('Creating DB3');
  TACreate(DB3, BF3, SizeOf(Log_Rec), IF3, SizeOf(I1)-1);

  writeln('Opening DB1');
  TAOpen(DB1, BF1, SizeOf(Log_Rec), IF1, SizeOf(I1)-1);
  writeln('Opening DB2');
  TAOpen(DB2, BF2, SizeOf(Log_Rec), IF2, SizeOf(I1)-1);

  writeln('Resetting DB1');
  TAReset(DB1);
  writeln('Resetting DB2');
  TAReset(DB2);
  writeln('Resetting DB3');
  TAReset(DB3);

  FillChar(R1, SizeOf(R1), #00);
  FillChar(R2, SizeOf(R2), #00);
  FillChar(R3, SizeOf(R3), #00);

  I1 := LowIndex;
  I2 := LowIndex;
  I3 := LowIndex;

  C1 := 0;
  C2 := 0;
  C3 := 0;

  SomeDupes := False;

  writeln('Nexting DB1');
  writeln;
  writeln;
  writeln;
  writeln;
  writeln;
  writeln;
  TANext(DB1, R1, I1);
  if not OK then begin
    EOF1 := True;
    Move(HighIndex, I1, SizeOf(HighIndex));
    end else
  inc(c1);

  writeln('Nexting DB2');
  writeln;
  writeln;
  writeln;
  writeln;
  writeln;
  writeln;
  TANext(DB2, R2, I2);
  if not OK then begin
    EOF2 := True;
    Move(HighIndex, I2, SizeOf(HighIndex));
    end else
  inc(c2);

  while not (EOF1 and EOF2) do begin
{    case CompString(I1,I2) of }
     Case CompareIndexes of
      Less: begin
              InsertFromR1;
            end;
      Greater: begin
              InsertFromR2;
            end;
      Equal: begin
        if (Trim(R1.Call) <> Trim(R2.Call)) or (R1.Band <> R2.Band)
         then begin
          InsertFromR1;
          Inc(R2.Ts);
          I2 := DateToSortString(R2.Date)+TimeToSortString(R2.Ts);
          InsertFromR2;
        end
        else begin
          {writeln(pad(R1.Call,12)+' same time in both files: one kept, one dropped.'); }
          SomeDupes := True;
          InsertFromR1;
          TANext(DB2, R2, I2);
          if not OK then begin
            EOF2 := True;
            Move(HighIndex, I2, SizeOf(HighIndex));
          end
          else inc(c2);
        end;
      end;
    end;
  end;

  TAClose(DB1);
  TAClose(DB2);
  TAClose(DB3);
  writeln;
  WriteLn('Merge Summary.');
  writeln;
  writeln(BF1,' has been combined with ',BF2,' to create ',BF3);
  writeln(C1,' records were combined with ',C2,' records giving ',C3,' total records.');
  if SomeDupes then begin
    writeln('Only one copy of the matching records was kept which accounts for');
    writeln('the discrepancy.');
  end;
end.



