{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
program Merge2;

uses
  TpString,TpDate,TpDos,TAHigh,TAccess,TpCrt ,rectypes;

type
  ContestType    = (ARRL, CQWW, BART, DXPD, SART, JART, WPXC, VOLT,
                    WAED, NAQP, ANAR, EAWW, SPDX, ARII, RRWW, TARA,
                    SPRI, ARFD, HSSC, SCCC, BSPR);

    BandType       = (B80, B40, B20, B15, B10, TOT);


  VersionType    = (V1_1_6, V2_0_0);

  ModeType       = (RTTYMODE, ASCII, AMTOR, Packet, SSB, CW, Other);

  Log_Rec        = record
                     ToolBoxNumber  : Integer;
                     VersionControl : VersionType;
                     Contest        : ContestType;
                     Date           : Date;
                     Ts             : Time;
                     Band           : BandType;
                     Mode           : ModeType;
                     Call           : String[12];
                     His_Country    : String[5];
                     His_RST        : Byte;
                     His_Qth        : String[3]; {grp# in waedc}
                     His_Zone       : Byte; {age in jarts, qtcs in waedc}
                     His_Serno      : Word;
                     His_Time       : Time;
                     My_Rst         : Byte;
                     My_Qth         : String[3];
                     My_Zone        : Byte;
                     My_Serno       : Word;
                     My_Time        : Time;
                     Point          : Byte;
                     New_Zone       : Boolean;
                     New_Country    : Boolean;
                     New_Qth        : Boolean; {in wae, F=not sent, T=sent}
                     New_Continent  : Boolean;
                     Dupe           : Boolean;
                     Comment        : String[25];
                   end;
var

  DatF1    : DataFile;
  DatF2    : DataFile;
  DatF3    : DataFile;

  DB3      : DataSet;    {output database handle 3}
  I3       : String[8];

  BF1      : String[79]; {binfile 1 name}
  BF2      : String[79]; {binfile 2 name}
  BF3      : String[79]; {newbinfile name}
  IF3      : String[79]; {newindexfile name}

  R1       : Log_Rec;
  R2       : Log_Rec;
  R3       : Log_Rec;

  EOF1,EOF2               : Boolean;

  n1, n2                  : word;
  ReadFrom1,
  ReadFrom2,
  WriteTo3,
  Duplicates              : word;
  NumRecs1, NumRecs2      : word; { number of records in each file}

  OverWrite,SomeDupes     : Boolean;

  OverStr                 : string[2];


procedure NotFound(S : string);
begin
  writeln('Error! Cannot find '+S);
  halt;
end;


procedure  GetFrom1;
begin
  if EOF1 then Exit;
  if n1 = NumRecs1 then begin
    EOF1 := True;
    Exit;
  end;
  inc(n1);
{  Writeln('Reading '+Bf1); }
  GetRec(DatF1, n1, R1);
  if TAStatus = 0 then begin
    inc(ReadFrom1);
  end
  else begin
    writeln('Merge: Error reading '+BF1+' Status='+long2str(TAStatus));
    EOF1 := True;
  end;
end;

procedure  GetFrom2;
begin
  if EOF2 then Exit;
  if n2 = NumRecs2 then begin
    EOF2 := True;
    Exit;
  end;
  inc(n2);
{  Writeln('Reading '+bf2); }
  GetRec(DatF2, n2, R2);
  if TAStatus = 0 then begin
    inc(ReadFrom2);
  end
  else begin
    writeln('Merge: Error reading '+BF2+' Status='+long2str(TAStatus));
    EOF2 := True;
  end;
end;

procedure AddTo3;
begin
  I3 := DateToSortString(r3.Date)+TimeToSortString(r3.Ts);
{  Writeln('Writing '+bf3); }
  TAInsert(DB3, R3, I3); {put R1 into DB3}
  inc(WriteTo3);
end;

procedure InsertFromR1;
begin
  Move(R1, R3, SizeOf(R1));
  AddTo3;
end;

procedure InsertFromR2;
begin
  Move(R2, R3, SizeOf(R2));
  AddTo3;
end;

procedure EraseFile(S : String);
var
 F:  file;
 Ch: Char;
begin
 Assign(F, S);
 {$I-}
 Reset(F);
 {$I+}
 if IOResult <> 0 then Exit;
 Close(F);
 Erase(F);
end;

function CompareIndexes : CompareType;
var
  T1, T2 : Time;
  TS1, TS2 : string[10];
begin
  if EOF1 then begin
    CompareIndexes := Greater;
    Exit;
  end;
  if EOF2 then begin
    CompareIndexes := Less;
    Exit;
  end;
  T1 := RoundToNearestMinute(R1.Ts,True);
  T2 := RoundToNearestMinute(R2.ts,True);
  TS1 := TimeToTimeString('hh:mm:ss',T1);
  TS2 := TimeToTimeString('hh:mm:ss',T2);
  if R1.Date < R2.Date then begin
    CompareIndexes := Less;
    Exit;
    end;
  if R1.Date > R2.Date then begin
    CompareIndexes := Greater;
    Exit;
    end;
  if T1 < T2 then begin
    CompareIndexes := Less;
    Exit;
    end;
  if T1 > T2 then begin
    CompareIndexes := Greater;
    Exit;
    end;
  { equal dates & times so far}
  CompareIndexes := Equal;
end;



begin
  clrscr;
  writeln;
  WriteLn('BIN File Merger Copyright(c) 1997-1999 by Wyvern Technology, Inc.');
  writeln;

  if (ParamCount < 3) or (ParamStr(1) = '/?') or (ParamStr(1) = '?') then begin
    WriteLn('Usage:  MERGE BinFile1 BinFile2 NewBinFile /O');
    Writeln;
    Writeln('Merges two .BIN files together to create a new .BIN file.');
    Writeln('The /O parameter allows overwriting of the output .BIN file.');
    writeln;
    writeln('* * * WARNING! * * * Please practice using this program!  It works');
    writeln('as promised.  Improper use of this program could lead to data loss.');
    Halt(1);
  end;

  BF1 := StUpCase(ForceExtension(ParamStr(1),'BIN'));
  BF2 := StUpCase(ForceExtension(ParamStr(2),'BIN'));
  BF3 := StUpCase(ForceExtension(ParamStr(3),'BIN'));

  IF3 := StUpCase(ForceExtension(ParamStr(3),'NDX'));

  if not ExistFile(BF1) then NotFound(BF1);
  if not ExistFile(BF2) then NotFound(BF2);

  if (BF1 = BF3) then begin
    writeln('Error!  Don''t try to overwrite the files you are merging!');
    writeln('If you need to do this, then write a small batch file to');
    writeln('perform the merge then delete/rename the new file name.');
    writeln;
    writeln('* * * WARNING! * * * Please practice using this program!  It works');
    writeln('as promised.  Improper use of this program could lead to data loss.');
    Halt(1);
  end;


  OverStr := Copy(PAramStr(4),1,2);
  OverWrite := OverStr[2] in ['o','O'];

  If (ExistFile(BF3) or ExistFile(IF3) ) and (not OverWrite) then begin
    WriteLn('Error! Delete the output bin file before running merge');
    WriteLn('or use the /O parameter.');
    halt;
    end;
  If OverWrite then begin
    EraseFile(BF3);
    EraseFile(IF3);
  end;

  OpenFile(DatF1, BF1, SizeOf(Log_Rec));
  if not OK then begin
    Writeln('Merge: Open '+BF1+' file failed. Error='+long2str(TaStatus));
    Halt;
  end
  else
    writeln('Opened '+ BF1);

  OpenFile(DatF2, BF2, SizeOf(Log_Rec));
  if not OK then begin
    Writeln('Merge: Open '+BF2+' file failed. Error='+long2str(TaStatus));
    Halt;
  end
  else
    writeln('Opened '+ BF2);

  TACreate(DB3, BF3, SizeOf(Log_Rec), IF3, SizeOf(I3)-1);
  if not OK then begin
    Writeln('Merge: Create '+BF3+' file failed. Error='+long2str(TaStatus));
    Halt;
  end
  else
    writeln('Created '+ BF3);

  FillChar(R1, SizeOf(R1), #00);
  FillChar(R2, SizeOf(R2), #00);
  FillChar(R3, SizeOf(R3), #00);

  ReadFrom1 := 1;
  ReadFrom2 := 1;
  WriteTo3 := 0;

  SomeDupes := False;

  NumRecs1 := FileLen(DatF1)-1;
  NumRecs2 := FileLen(DatF2)-1;

  n1 := 0;
  n2 := 0;

  GetFrom1;
  GetFrom2;

  while not (EOF1 and EOF2) do begin

{    write('Comparing: ');
    write('File1 '+ pad(R1.Call,13)+TimeToTimeString('hh:mm:ss',r1.ts)+ ' VS. ');
    writeln('File2 '+ pad(R2.Call,13)+TimeToTimeString('hh:mm:ss',r2.ts)); }

     Case CompareIndexes of
      Less: begin
              InsertFromR1;
              GetFrom1;
            end;
      Greater: begin
              InsertFromR2;
              GetFrom2;
            end;
      Equal: begin
        if (Trim(R1.Call) <> Trim(R2.Call)) or (R1.Band <> R2.Band)
         then begin
          InsertFromR1;
          GetFrom1;
        end
        else begin
          SomeDupes := True;
          InsertFromR1;  {arbitrarily keep #1, since they are the same }
          GetFrom1;
          GetFrom2;
        end;
      end;
    end;
  end;

  CloseFile(DatF1);
  CloseFile(DatF2);
  TAClose(DB3);
  writeln;
  WriteLn('Merge Summary.');
  writeln;
  writeln(BF1,' has been combined with ',BF2,' to create ',BF3);
  writeln;
  writeln(BF1+' Record Count '+long2str(ReadFrom1));
  writeln(BF2+' Record Count '+long2str(ReadFrom2));
  writeln(BF3+' Record Count '+long2str(WriteTo3));
  writeln;
  writeln(ReadFrom1,' records were combined with ',ReadFrom2,' records giving ',WriteTo3,' total records.');
  if SomeDupes then begin
    writeln('Only one copy of the matching records was kept which accounts for');
    writeln('the discrepancy.');
  end;
end.



