{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit NetNode;

interface

uses
  ApPort,
  ApUart,
  OoCom;

type

PNetNode = ^TNetNode;
TNetNode = object
             ThisStationNum : byte;
             LoopBack       : Boolean;
             UP : UartPortPtr;
             LastAsyncStatus : Word;
             Timeout : word;
             Constructor Init(ComPort : ComNameType; Loop : Boolean; StationNum : byte);
             Destructor  Done;virtual;
             function    LastError : string; virtual;
             function    TransmitBlock(var Block : String) : boolean; virtual;
             function    ReceiveBlock(var Block : String) : boolean; virtual;
             procedure   OkToReceive(const oktr : boolean);
           end;

implementation

uses
  Network,
  cMsgWin,
  ApMisc,
  OpString,
  IniTools,
  LowLvl,
  RecTypes,
  ApTimer,
  TpCrt,
  cRttyWin,
  DebugRtns,
  IniFile;

procedure ShowBlock(Block, SMsg : String);
var
  st : string[10];
begin
  Case Block[1] of
   #$00 : Exit;
   #$01 : st := 'Ping';
   #$02 : st := 'Pong';
   #$03 : st := 'Note';
   #$04 : st := 'Date';
   #$05 : st := 'Log Rec';
   #$06 : st := 'Qrg Data';
   #$07 : st := 'Qrg Req';
   #$08 : st := 'Del Rec';
   #$09 : st := 'Ser# Req';
   #$0A : st := 'Ser# Data';
   #$0B : st := 'Clst TX';
   #$0C : st := 'Clst RX';
   #$0D : st := 'CWS Data';
   #$0E : st := 'TEST Data';
  else
    begin
      st := 'INVALID';
    end;
  end;
  AppendDebugMessage(Copy(Pad(st,10),1,10)+
                       'Id=$'+hexb(ord(block[1]))+
                     ', Sz=$'+hexb(ord(block[0]))+
                     ', Orig=$'+hexb(ord(block[2]))+
                     ', Dest=$'+hexb(ord(block[3]))+
                     ', CheckDig=$'+hexb(ord(block[4])));
end;



Constructor TNetNode.Init(ComPort : ComNameType; Loop : Boolean; StationNum : byte);
var
  Speed : string[4];
  BaudRate : LongInt;
  T : string;
  HS : HandShakeType;
  Options : word;
begin
  ThisStationNum := StationNum;
  Loopback := Loop;
  BaudRate := RttyIni^.GetProfileLong('BaudRate', ComNameString(ComPort), 2400);
  TimeOut := RttyIni^.GetProfileWord('Timeout','Network',18);
  UP := New(UartPortPtr, InitFast(ComPort, BaudRate));
  if not Assigned(UP) then begin
    MessageBox('SEVERE ERROR','Couldn''t allocate the network','comport.  Will not be used','');
    Fail;
  end;
  Case Loop of
    FALSE: begin
             UP^.HWFlowEnable(1024,128,hfUseDTR+hfRequireDSR);
             UP^.SWFlowDisable;
           end;
    TRUE: begin
            UP^.HWFlowDisable;
            UP^.SWFlowDisable;
          end;
  end;
  UP^.ptOptionsOff(ptReturnPartialGets);
  UP^.ptOptionsOff(ptExecutePartialPuts);
  if UP^.CheckLineError then begin
    UP^.FlushInBuffer; { flush the input buffer }
    if UP^.CheckFifoError then begin
      SetFifoBuffering(UP^.PR^.BaseAddr,False,0); { flush the fifo buffer }
      SetFifoBuffering(UP^.PR^.BaseAddr,True,14);
    end;
    UP^.GetLineError;  { resets error indicators }
  end;
  SetFifoBuffering(UP^.PR^.BaseAddr,True,14);
  if not Loop then UP^.SetDtr(True);
  AppendDebugMessage('NetWork: '+ComNameString(ComPort)+' '+long2str(BaudRate)+' baud');
  {$IFDEF DEBUG}
  AppendDebugMessage('NetNode created');
  {$ENDIF}
end;

Destructor TNetNode.Done;
begin
  if Assigned(UP) then Dispose(UP, Done);
  UP := NIL;
  {$IFDEF DEBUG}
  AppendDebugMessage('NetNode destroyed');
  {$ENDIF}
end;

function TNetNode.LastError : string;
begin
  LastError := ComNameString(UP^.ComPortName)+'-'+StatusStr(LastAsyncStatus);
end;

function GetCheckDigit(Data : string) : byte;
var
  b : byte;
  i : integer;
begin
  Data[4] := cNUL;
  i := 0;
  for b := 0 to ord(Data[0])-1 do
    inc(i, ord(Data[b]));
  GetCheckDigit := i mod 255;
  {$IFDEF DEBUG}
  AppendDebugMessage('NETWORK: CheckDigit=$'+hexb(i mod 255)+' data='+Data);
  {$ENDIF}
end;

function TNetNode.TransmitBlock(var Block : String) : boolean;
var
  BytesWritten : word;
  BlockLen : word;
  cd : byte;
  et : eventtimer;
begin
  if ord(Block[1]) > LastMsgId then Exit; { kill invalid messages immediately!}
  Block[4] := chr(GetCheckDigit(Block)); { generate a check digit }
  BlockLen := word(byte(Block[0])); {get the length into word data type }
  if not  Loopback  then begin
    if not up^.checkdsr then begin
      NewTimer(et,TimeOut);
      while (not up^.checkdsr) and (not Timerexpired(et)) do
        MsgWin^.Display('WARNING! Other station not ready, time left='+long2str(RemainingTimeInMSecs(et))+' msecs');
      MsgWin^.Clear;
    end;
  end;
  { if there is no room in buffer, wait a short while, then try again }
  NewTimer(et,TimeOut);
  While (UP^.OutBuffFree < BlockLen) and (not Timerexpired(et)) do {};
  {if still no room, time to kill this message }
  if UP^.OutBuffFree < BlockLen then begin
    Msgwin^.Display('Unable to send network data, buffer full');
    AppendDebugMessage('NETNODE: BufferFull Unable to send network data.  Not sent.');
    Exit;
  end;
  {ShowBlock(Block,'Sending'); }
  UP^.PutBlockDirect(Block[0], BlockLen, BytesWritten);
  LastAsyncStatus := AsyncStatus;
  if LastAsyncStatus <> 0 then
    AppendDebugMessage('NETNODE: TransmitBlock, Error='+StatusStr(LastAsyncStatus));
  if BytesWritten <> BlockLen then
    AppendDebugMessage('NETNODE: TransmitBlock, Error=BytesWritten<>BlockLen');
  while not UP^.CheckTE do; { wait for 'T'ransmitter to 'E'mpty }
end;

function TNetNode.ReceiveBlock(var Block : String) : boolean;
var
  c       : char;
  RestOfBlock : word;
  ActualGetSize : word;
  et : EventTimer;
  Br : boolean;
begin
  FillChar(Block, SizeOf(Block), cNUL);
  ReceiveBlock := False;
  if not UP^.CharReady then Exit;
  UP^.GetChar(C);
  LastAsyncStatus := AsyncStatus;
  if (C = #$FF) or
     (C = cNUL) or
     (LastAsyncStatus <> ecOK) then Exit;
  Block[0] := c;
  RestOfBlock := word(ord(c)-1);
  NewTimer(Et, 27); {wait up to 1.5 seconds}
  repeat
    br := UP^.BlockReady(RestOfBlock,[]);
  until TimerExpired(Et) or Br;
  if not Br then begin
    AppendDebugMessage('NETNODE: Incomplete block received');
    FillChar(Block, SizeOf(Block), cNUL);
    Exit;
  end;
  UP^.GetBlockTimeOut(Block[1],RestOfBlock,ActualGetSize,[],27);
  LastAsyncStatus := AsyncStatus;
  if Ord(Block[4]) = GetCheckDigit(Block) then
    ReceiveBlock := True
  else begin
    AppendDebugMessage('NETNODE: Check digit is invalid');
    FillChar(Block, SizeOf(Block), cNUL);
  end;
end;

  procedure   TNetNode.OkToReceive(const oktr : boolean);
  begin
    Case Loopback of
      False : UP^.SetDtr(oktr);
      True  : { nothing special for loop };
    end;
  end;


end.
