{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit Network;

interface

uses
  NetNode,
  DiskIo,
  RecTypes;

const
  PingDataId      = $01;
  PongDataId      = $02;
  NoteDataId      = $03;
  DateDataId      = $04;
  LogDataId       = $05;
  QrgDataId       = $06;
  QrgReqID        = $07;
  DeleteLogDataId = $08;
  SerNumReqId     = $09;
  SerNumDataId    = $0A;
  ClusterTxDataId = $0B;
  ClusterRxDataId = $0C;
  CallSentDataId  = $0D;
  TestMsgId	  = $0E;
  LastMsgId       = $0E;

type
  MastData = record
               Size     : byte;
               DataID   : byte;
               OrigNode : byte;
               DestNode : byte;
               CheckDig : byte;
               EndOfBuf : byte;
             end;

  PingData = record
               Size     : byte;
               DataID   : byte;
               OrigNode : byte;
               DestNode : byte;
               CheckDig : byte;
               Msg      : string[4];
               EndOfBuf : byte;
             end;

  NoteData = record
               Size     : byte;
               DataID   : byte;
               OrigNode : byte;
               DestNode : byte;
               CheckDig : byte;
               Note     : string[59];
               EndOfBuf : byte;
             end;

  DateData = record
               Size     : byte;
               DataID   : byte;
               OrigNode : byte;
               DestNode : byte;
               CheckDig : byte;
               Year,
               Month,
               Day,
               Hour,
               Minute,
               Second   : word;
               HrsDiff  : integer;
               EndOfBuf : byte;
             end;

  LogData = record
               Size     : byte;
               DataID   : byte;
               OrigNode : byte;
               DestNode : byte;
               CheckDig : byte;
               NewRec   : boolean;
               LogRec   : Log_Rec;
               Index    : String[8];
               EndOfBuf : byte;
             end;

  DeleteLogData = record
               Size     : byte;
               DataID   : byte;
               OrigNode : byte;
               DestNode : byte;
               CheckDig : byte;
               LogRec   : Log_Rec;
               EndOfBuf : byte;
             end;

  QrgData = record
               Size     : byte;
               DataID   : byte;
               OrigNode : byte;
               DestNode : byte;
               CheckDig : byte;
               Frequency: longint;
               EndOfBuf : byte;
             end;

  SerNumData = record
               Size      : byte;
               DataID    : byte;
               OrigNode  : byte;
               DestNode  : byte;
               CheckDig  : byte;
               Band      : BandType;
               SNum      : word;
               SNumAll   : word;
               EndOfBuf : byte;
             end;

  ClusterData = record
               Size     : byte;
               DataID   : byte;
               OrigNode : byte;
               DestNode : byte;
               CheckDig : byte;
               Data     : string[78+78]; { 2 lines of data }
               EndOfBuf : byte;
             end;

  TestMsgData = record
               Size     : byte;
               DataID   : byte;
               OrigNode : byte;
               DestNode : byte;
               CheckDig : byte;
               SendTime : string[11]; {hh.mm.ss.hh}
               EndOfBuf : byte;
             end;


const
  FirstNode = True;
  SecondNode = False;

var
  Block : string[255];
  PD  :  PingData      absolute Block;
  MD  :  MastData      absolute Block;
  N   :  NoteData      absolute Block;
  DR  :  DateData      absolute Block;
  LD  :  LogData       absolute Block;
  DLD :  DeleteLogData absolute Block;
  QRGD:  QrgData       absolute Block;
  SND :  SerNumData    absolute Block;
  PCD :  ClusterData   absolute Block;
  TMD :  TestMsgData   absolute Block;
  StationId : Byte;

type
  NodeSwitch = boolean;

  PNetwork = ^TNetwork;
  TNetwork = object
    ThisNode : byte;
    Node : array[NodeSwitch] of PNetNode;
    Msg : string[59];
    MidDelay : integer;
    SetLastHr : byte;
    NetRxOK   : boolean;
    LoopBack  : boolean;
    QRGS      : array[1..8] of string[7];
    Constructor Init(StationNum : byte);
    Destructor  Done; virtual;
    Procedure   PassItOn(ToNode : NodeSwitch); virtual;
    Procedure   Broadcast; virtual;
    procedure   Idle; virtual;
    function    LastMessage : string;
    procedure   AppendMessage(const aMsg : String);
    procedure   EditMessages;
    procedure   SendAMessage;
    procedure   SendAnnouncement;
    procedure   SendPass(const Who : string);
    procedure   AutoSyncTime;
    procedure   OkToReceive(const oktr : boolean);
    procedure   SendPing(WhoToPing : byte);
    procedure   SendPong(WhoToPong : byte);
    procedure   SendNote(Dest : byte; ANote : string);
    procedure   SendClusterTxData(aData : string); { for the com to get }
    procedure   SendClusterRxData(aData : string); { from the com to others }
    procedure   SendDateTime;
    procedure   SendLogData(ANewRec : boolean; A : log_rec; Ind : string);
    procedure   SendDeleteLogData(A : log_rec);
    procedure   SendQrgReq(Dest : byte);
    procedure   SendQrgData(Dest : byte);
    procedure   SendSerNumReq;
    procedure   SendSerNumData;
    procedure   SendCallSentData;
    procedure   SendTest(WhoToTest : byte);
    procedure   HandlePing;
    procedure   HandlePong;
    procedure   HandleNote;
    procedure   HandleDateTime;
    procedure   HandleLogData;
    procedure   HandleDeleteLogData;
    procedure   HandleQrgData;
    procedure   HandleQrgReq;
    procedure   HandleSerNumReq;
    procedure   HandleSerNumData;
    procedure   HandleClusterRxData;
    procedure   HandleClusterTxData;
    procedure   HandleTestMsg;
    function    GetNodeBlock(WhichNode : NodeSwitch) : byte;
    procedure   ProcessDataBlock(WhichNode : NodeSwitch);
  end;

var
  Net : PNetWork;
  procedure InitNetwork;

implementation

uses
  cRttyWin,  cClock,  EditText,  LowLvl,  Routines,  IniFile,  IniTools,
  OneTimers,  Keys,  ApPort,  cMsgWin,  TpDate,  TpDos,  TpString,  Dos,
  TpCrt,  DebugRtns,  CSerNum,  cEditWin,  TpEntry,  TpWindow,  TpMouse,
  ClusterTerm,  ClstrCom,  ClstrNet,  cColors,  c_Contest,  c_Radio,
  cDupeTbl, NetworkD, NetworkL;

procedure ShowBlock(SMsg : String);
begin
  if MD.DataId = $00 then Exit;
  AppendDebugMessage(Copy(Pad(SMsg,15),1,15)+
                     'DataId=$'+hexb(md.dataid)+
                     ', Size=$'+hexb(md.size)+
                     ', OrigNode=$'+hexb(md.orignode)+
                     ', DestNode=$'+hexb(md.destnode)+
                     ', CheckDig=$'+hexb(md.Checkdig));
end;

Constructor TNetwork.Init(StationNum : Byte);
  { responsible for creating itself , then using INI settings, create
    any NetNodes needed }
var
  N0, N1 : Byte;
  s : string;
begin
  ThisNode :=  StationNum;
  SetLastHr := 0;
  Node[FirstNode] := nil;
  Node[SecondNode] := nil;
  Msg := '';
  NetRxOk := True;
  with RttyIni^ do begin
    Loopback := GetProfileBool('Loop','NetWork',False);
    N0 := GetProfileByte('ComportA','NetWork',0);
    if Loopback then N1 := 0 { no second port needed or allowed }
            else N1 := GetProfileByte('ComPortB','NetWork',0);
  end;
  if N0 > 0 then Node[FirstNode] := New(PNetNode, Init(ComNameType(N0-1),Loopback,ThisNode));
  if N1 > 0 then Node[SecondNode] := New(PNetNode, Init(ComNameType(N1-1),Loopback,ThisNode));
  if Loopback then
    AppendDebugMessage('NetWork: Loopback mode selected');
  if Assigned(Node[FirstNode]) then begin
    AppendDebugMessage('NetWork: NetNode created on ComPortA='+long2str(n0));
  end;
  if Assigned(Node[SecondNode]) then begin
    AppendDebugMessage('NetWork: NetNode created on ComPortB='+long2str(n1));
  end;
  for n0 := 1 to 8 do
    QRGS[n0] := '';
  {$IFDEF DEBUG}
  AppendDebugMessage('Network created');
  {$ENDIF}
end;

Destructor TNetwork.Done;
begin
  if Assigned(Node[FirstNode]) then Dispose(Node[FirstNode], Done);
  if Assigned(Node[SecondNode]) then Dispose(Node[SecondNode], Done);
  {$IFDEF DEBUG}
  AppendDebugMessage('Network destroyed');
  {$ENDIF}
end;

procedure TNetWork.AppendMessage(const aMsg : String);
const
  FileOK = $0;
  FileNotFound = $2;
var
  DebugFile : Text;
  IOStatus : Integer;
begin
  Assign(DebugFile, FileName('MSG'));
  {$I-} Append(DebugFile); {$I+}
  IOStatus := IoResult;
  if (IOStatus <> FileOK) and (IOStatus <> FileNotFound) then
    Exit;                       {FATAL ERROR-Get the hell out!}
  if IOStatus = FileNotFound then
    begin
      Rewrite(DebugFile);
      WriteLn(DebugFile, 'Date     Time  Message');
    end;
  WriteLn(DebugFile, TodayString('yy-mm-dd ')+CurrentTimeString('hh:mm ')+aMsg);
  Close(DebugFile);
end;

function TNetWork.LastMessage : string;
begin
  LastMessage := Msg;
  Msg := '';
end;

Procedure TNetWork.PassItOn(ToNode : NodeSwitch);
begin
  AppendDebugMessage('NETWORK: PassItOn invoked');
end;

Procedure TNetwork.Broadcast;
begin
  if Assigned(Node[FirstNode]) then begin
    Node[FirstNode]^.TransmitBlock(Block);
  end;
  if Assigned(Node[SecondNode]) then begin
    Node[SecondNode]^.TransmitBlock(Block);
  end;
end;

procedure TNetWork.SendPing(WhoToPing : byte);
begin
  FillChar(Block, SizeOf(Block), cNUL);
  with PD do begin
    Size := SizeOf(PD);
    DataId := PingDataId;
    OrigNode := ThisNode;
    DestNode := WhoToPing;
    Msg := 'Ping';
    EndOfBuf := $FF;
  end;
  Broadcast;
end;

procedure TNetWork.SendTest(WhoToTest : byte);
var
  Hour, Minute, Second, Sec100 : word;
begin
  FillChar(Block, SizeOf(Block), cNUL);
  with TMD do begin
    Size := SizeOf(TMD);
    DataId := TestMsgId;
    OrigNode := ThisNode;
    DestNode := WhoToTest;
    GetTime(Hour, Minute, Second, Sec100);
    SendTime := LeftPadCh(long2str(Hour),'0',2)+'.'+
                LeftPadCh(long2str(Minute),'0',2)+'.'+
                LeftPadCh(long2str(Second),'0',2)+'.'+
                LeftPadCh(long2str(Sec100),'0',2);
    EndOfBuf := $FF;
  end;
  Broadcast;
end;

procedure TNetWork.SendNote(Dest : Byte; ANote : string);
begin
  FillChar(Block, SizeOf(Block), cNUL);
  with N do begin
    Size := SizeOf(N);
    DataId := NoteDataId;
    OrigNode := ThisNode;
    DestNode := Dest;
    Note := Pad(ANote,SizeOf(Note));
    EndOfBuf := $FF;
  end;
  Broadcast;
end;

procedure   TNetWork.SendClusterTxData(aData : string);
begin
  FillChar(Block, SizeOf(Block), cNUL);
  with PCD do begin
    Size := SizeOf(PCD);
    DataId := ClusterTxDataId;
    OrigNode := ThisNode;
    DestNode := 0;
    FillChar(Data, SizeOf(Data), ' ');
    Data := aData;
    EndOfBuf := $FF;
  end;
  Broadcast;
end;

procedure   TNetWork.SendClusterRxData(aData : string);
begin
  FillChar(Block, SizeOf(Block), cNUL);
  with PCD do begin
    Size := SizeOf(PCD);
    DataId := ClusterRxDataId;
    OrigNode := ThisNode;
    DestNode := 0;
    FillChar(Data, SizeOf(Data), ' ');
    Data := aData;
    EndOfBuf := $FF;
    {$IFDEF DEBUG}
    AppendDebugMessage('Network.SendRxData.Data='+Data);
    {$ENDIF}
  end;
  Broadcast;
end;

procedure   TNetWork.SendLogData(ANewRec : boolean; A : log_rec; Ind : String);
begin
  FillChar(Block, SizeOf(Block), cNUL);
  with LD do begin
    Size := SizeOf(LD);
    DataId := LogDataId;
    OrigNode := ThisNode;
    DestNode := 0;
    NewRec := ANewRec;
    MoveLogRec(A,LogRec);
    Index := Ind;
    EndOfBuf := $FF;
  end;
  Broadcast;
end;

procedure   TNetWork.SendDeleteLogData(A : log_rec);
begin
  FillChar(Block, SizeOf(Block), cNUL);
  with DLD do begin
    Size := SizeOf(DLD);
    DataId := DeleteLogDataId;
    OrigNode := ThisNode;
    DestNode := 0;
    MoveLogRec(A,LogRec);
    EndOfBuf := $FF;
  end;
  Broadcast;
end;

procedure   TNetWork.SendQrgReq(Dest : byte);
begin
  FillChar(Block, SizeOf(Block), cNUL);
  with MD do begin
    Size := SizeOf(MD);
    DataId := QrgReqId;
    OrigNode := ThisNode;
    DestNode := Dest;
    EndOfBuf := $FF;
  end;
  Broadcast;
end;

procedure   TNetWork.SendQrgData(Dest : byte);
begin
  FillChar(Block, SizeOf(Block), cNUL);
  with QRGD do begin
    Size := SizeOf(QRGD);
    DataId := QrgDataId;
    OrigNode := ThisNode;
    DestNode := Dest;
    if Assigned(Radio) then
      Frequency := Radio^.GetDisplayFrequency
    else
      Frequency := 0;
    EndOfBuf := $FF;
  end;
  Broadcast;
end;

procedure   TNetWork.SendSerNumReq;
begin
  FillChar(Block, SizeOf(Block), cNUL);
  with SND do begin
    Size := SizeOf(SND);
    DataId := SerNumReqId;
    OrigNode := ThisNode;
    DestNode := 1;
    EndOfBuf := $FF;
  end;
  Broadcast;
end;

procedure   TNetWork.SendSerNumData;
begin
  FillChar(Block, SizeOf(Block), cNUL);
  if Not Assigned(SerNum) then Exit;
  with SND do begin
    Size := SizeOf(SND);
    DataId := SerNumDataId;
    OrigNode := ThisNode;
    DestNode := 0; {it's a broadcast}
    SNum := SerNum^.CurrentSerialNumberByBand(Editw^.EditRecs[1].Band);
    SNumAll := SerNum^.CurrentSerialNumberInAll;
    Band := Editw^.EditRecs[1].Band;
    EndOfBuf := $FF;
  end;
{  Broadcast;   HC8N }
end;

procedure   TNetWork.SendCallSentData;
begin
  FillChar(Block, SizeOf(Block), cNUL);
  with MD do begin
    Size := SizeOf(MD);
    DataId := CallSentDataId;
    OrigNode := ThisNode;
    DestNode := 0;
    EndOfBuf := $FF;
  end;
  Broadcast;
end;

procedure   TNetWork.SendDateTime;
var
  Dow : word;
  Sec100 : word;
begin
  FillChar(Block, SizeOf(Block), cNUL);
{  if ThisNode <> 1 then begin
    Msg := 'Only station 1 can broadcast a date/time update!';
    AppendMessage(Msg);
    MsgWin^.Display(Msg);
    exit;
  end; }
  with DR do begin
    Size := SizeOf(DR);
    DataID := DateDataId;
    OrigNode := ThisNode;
    DestNode := 0;
    GetDate(Year, Month, Day, Dow);
    GetTime(Hour, Minute, Second, Sec100);
    HrsDiff := Clock^.HoursFromUtc;
    EndOfBuf := $FF;
  end;
  Broadcast;
end;

procedure TNetWork.AutoSyncTime;
var
  Hr, MN, SC : Byte;
begin
  if not RttyIni^.GetProfileBool('AutoSync', 'Network', False) then Exit;
  TimeToHMS(CurrentTime, Hr, MN, SC);
  if SetLastHr = Hr then Exit;
  if RttyIni^.GetProfileByte('Minute', 'Network', 0) <> MN then Exit;
  SetLastHr := Hr;
  SendDateTime;
End;

procedure TNetWork.SendPong(WhoToPong : byte);
begin
  FillChar(Block, SizeOf(Block), cNUL);
  with PD do begin
    Size := SizeOf(PD);
    DataId := PongDataId;
    OrigNode := ThisNode;
    DestNode := WhoToPong;
    Msg := 'Pong';
    EndOfBuf := $FF;
  end;
  Broadcast;
end;

procedure   TNetWork.HandlePing;
begin { HandlePing }
  SendPong(MD.OrigNode);
  Msg := 'Ping from S# '+long2str(MD.DestNode);
  AppendMessage(Msg);
  {$IFNDEF NETTEST}
  MsgWin^.Display(Msg);
  {$ELSE}
  Writeln(Msg);
  sound(1000);delay(400);nosound;
  {$ENDIF}
end; { HandlePing }

procedure   TNetWork.HandlePong;
begin { HandlePong }
  Msg := 'Propagation is good to S# ' + long2str(MD.OrigNode);
  AppendMessage(Msg);
  {$IFNDEF NETTEST}
  MsgWin^.Display(Msg);
  {$ELSE}
  Writeln(Msg);
  sound(500);delay(300);nosound;
  {$ENDIF}
end; { HandlePong }

procedure   TNetWork.HandleTestMsg;
begin
  if TMD.OrigNode = ThisNode then
    AppendDebugMessage('Made it around the loop');
  Msg := 'TestMsg from S# '+long2str(TMD.DestNode)+' '+TMD.SendTime;
  AppendMessage(Msg);
  MsgWin^.Display(Msg);
end;

procedure   TNetWork.HandleNote;
var
  ShowIt : Boolean;
begin { HandleNote }
  ShowIt := False;
  with N do begin
    Msg := 'From # '+long2str(OrigNode)+' : ' + Note;
  end;
  AppendMessage(Msg);
  if Copy(Msg,1,8)='PASS >> ' then begin
    Case Editw^.EditRecs[1].Band of
      B80: ShowIt := pos('3.',Msg) > 0;
      B40: ShowIt := pos('7.',Msg) > 0;
      B20: ShowIt := pos('4.',Msg) > 0;
      B15: ShowIt := pos('1.',Msg) > 0;
      B10: ShowIt := pos('8.',Msg) > 0;
    end;
  end
  else begin
    Showit := True;
  end;
  if ShowIt then MsgWin^.Display(Msg);
end; { HandleNote }

procedure   TNetWork.HandleClusterRxData;
begin
  if not Assigned(ClusterTerminal) then Exit;
  {$IFDEF DEBUG}
  AppendDebugMessage('TNetwork.HandleRxData.pcd.data='+Pcd.Data);
  {$ENDIF}
  with PCD do begin
    { only accept this data if this is a NETWORK terminal }
    if TypeOf(ClusterTerminal^) = TypeOf(TNetClusterTerminal) then begin
      ClusterTerminal^.AcceptDataFromNetCluster(Data);
      {$IFDEF DEBUG}
      AppendDebugMessage('TNetwork.HandleClusterRxData ClusterTerminal IS TNetClusterTerminal');
      {$ENDIF}
    end
    else begin
      {$IFDEF DEBUG}
      AppendDebugMessage('TNetwork.HandleClusterRxData ClusterTerminal IS NOT TNetClusterTerminal');
      {$ENDIF}
    end;
  end;
end;

procedure   TNetWork.HandleClusterTxData;
begin
  {$IFDEF DEBUG}
  AppendDebugMessage('TNetwork.HandleClusterTxData');
  {$ENDIF}
  if not Assigned(ClusterTerminal) then Exit;
  with PCD do begin
    { only accept this data if this is a COMPORT terminal }
    if TypeOf(ClusterTerminal^) = TypeOf(TComClusterTerminal) then begin
      ClusterTerminal^.SendMessageToCluster(Data);
      {$IFDEF DEBUG}
      AppendDebugMessage('TNetwork.HandleClusterTxData ClusterTerminal IS TComClusterTerminal');
      {$ENDIF}
    end
    else begin
      {$IFDEF DEBUG}
      AppendDebugMessage('TNetwork.HandleClusterTxData ClusterTerminal IS NOT TComClusterTerminal');
      {$ENDIF}
    end;
  end;
end;

procedure   TNetWork.HandleQrgData;
var
  T : string[7];
  s : string[39];
  i : integer;
begin
  with QRGD do begin
    if Frequency = 0 then QRGS[MD.OrigNode] := ''
    else begin
      T := LeftPad(long2str(Frequency div 100),6);
      Insert('.', T, 6);
      QRGS[MD.OrigNode] := T;
    end;
  end;
  s := 'QRGS: ';
  for i := 1 to 8 do
    if QRGS[i] <> '' then S := S + Trim(QRGS[i])+' ';
  MsgWin^.Display(S);
end;

procedure   TNetWork.HandleQrgReq;
begin
  SendQrgData(MD.OrigNode);
end;

procedure   TNetWork.HandleSerNumReq;
begin
  SendSerNumData;
end;

procedure   TNetWork.HandleLogData;
begin
  with LD do begin
    Msg := 'Record Arrived from # '+long2str(OrigNode);
    HandleContact(NewRec, LogRec,Index); {Routines.HandleContact}
    {$IFDEF DEBUG}
    AppendDebugMessage('NETWORK: '+msg);
    AppendDebugMessage('NETWORK: Call='+LogRec.Call);
    {$ENDIF}
  end;
end;

procedure   TNetWork.HandleSerNumData;
begin
  with SND do begin
    if SNum    > SerNum^.CurrentSerialNumberByBand(Band) then
      SerNum^.SetSerialNumber(Band,SNum);
    if SNumAll > SerNum^.CurrentSerialNumberInAll then
      SerNum^.SetSerialNumberInAll(SNumAll);
    {$IFDEF DEBUG}
      AppendDebugMessage('NETWORK:HandleSerNumData,SNum='+long2str(snum)+
                         ' SNumAll='+long2str(snumall));
    {$ENDIF}
  end;
end;

procedure   TNetWork.HandleDeleteLogData;
var
  Slot : Word;
begin
  with DLD do begin
    Msg := 'Delete record arrived from # '+long2str(OrigNode);
    if not LogRec.Dupe then
      if DupeTbl^.GetDupeByCall(LogRec.Call,Slot) then
        DupeTbl^.ResetDupeByCall(LogRec.Call, LogRec.Band);
    ThisContest^.RemoveOldMults(LogRec);
    LogFile^.DeleteThisRecord(LogRec);
    Editw^.RefreshFromDisk;
    if gInMainLoop then DrawEditScreen(EntryEsr);
  end;
end;

procedure   TNetWork.HandleDateTime;
begin { HandleDateTime }
  if ThisNode > 1 then with DR do begin
    Clock^.SetHoursFromUtc(HrsDiff);
    SetDate(Year,Month,Day);
    SetTime(Hour, Minute, Second, 0);
    Msg := 'Time synchronization message received & processed.';
  AppendMessage(Msg);
  {$IFNDEF NETTEST}
  MsgWin^.Display(Msg);
  {$ELSE}
  Writeln(Msg);
  {$ENDIF}
  end;
end; { HandleDateTime }

function TNetwork.GetNodeBlock(WhichNode : NodeSwitch) : byte;
begin
  FillChar(Block, Sizeof(Block), cNUL);
  if Node[WhichNode]^.ReceiveBlock(Block) then begin
    GetNodeBlock := ord(Block[1]);
    {$ifdef debug}
    ShowBlock('RECEIVED');
    {$endif}
    end
  else begin
    GetNodeBlock := 0;
  end;
end;


procedure TNetWork.EditMessages;
begin
  if ExistFile(FileName('MSG')) then FileEdit(FileName('MSG'));
end;

procedure TNetWork.SendAMessage;
var
  Esr : EsRecord;
  ExitCommand : esType;
  WinPtr : WindowPtr;
  Dest : Byte;
  Note : String[59];
begin
  with Colors^ do
    MakeWindow(WinPtr, 3, 3, 75, 8, True, True, False, WindowAttr,
      FrameAttr, HeaderAttr, 'Send Note');
  HideMouse;
  DisplayWindow(WinPtr);
  InitEsRecord(Esr);
  Colors^.SetEntryAttributes;
  SetWrapMode(Esr, WrapAtEdges);
  Dest := 0;
  Note := '';
  AddByteField(Esr, 'Station    (1-8)', 5, 5, '9', 5, 14, 0, 0, 8, Dest);
  AddStringField(Esr, 'Message', 6, 5, '', 6, 14, 59, 0, nil, Note);
  FastWriteStack(WinPtr,'<ESC> to cancel, <ENTER> to send',
                 5,3,Colors^.HeaderAttr);
  ShowMouse;
  AddCmd(EsMy98,kbEnter);
  repeat
    ExitCommand := EditScreen(Esr, 1, False);
  until ExitCommand in [EsMy49, EsQuit, EsDone, EsMy98];
  AddCmd(EsMy98,$0000);
  KillWindow(WinPtr);
  DisposeEditScreen(Esr);
  if (ExitCommand = EsMy49) then Exit;
  if (ExitCommand in [EsDone,EsMy98]) and (Trim(Note) <> '') then
    SendNote(Dest, Note);
end;

procedure TNetWork.SendAnnouncement;
var
  Esr : EsRecord;
  ExitCommand : esType;
  WinPtr : WindowPtr;
  Dest : Byte;
  Note : String[59];
begin
  with Colors^ do
    MakeWindow(WinPtr, 3, 3, 75, 7, True, True, False, WindowAttr,
      FrameAttr, HeaderAttr, 'Send Announcement to ALL Stations');
  HideMouse;
  DisplayWindow(WinPtr);
  InitEsRecord(Esr);
  Colors^.SetEntryAttributes;
  SetWrapMode(Esr, WrapAtEdges);
  Dest := 0;
  Note := '';
  AddStringField(Esr, 'Message', 5, 5, '', 5, 14, 59, 0, nil, Note);
  ShowMouse;
  repeat
    ExitCommand := EditScreen(Esr, 1, False);
  until ExitCommand in [EsQuit, EsDone];
  KillWindow(WinPtr);
  DisposeEditScreen(Esr);
  if (ExitCommand = EsDone) and (Trim(Note) <> '') then SendNote(Dest, Note);
end;

procedure TNetWork.SendPass(const Who : string);
var
  Esr : EsRecord;
  ExitCommand : esType;
  WinPtr : WindowPtr;
  Dest : Byte;
  Note : String[19];
begin
  with Colors^ do
    MakeWindow(WinPtr, 3, 3, 45, 7, True, True, False, WindowAttr,
      FrameAttr, HeaderAttr, 'Passing > '+who);
  HideMouse;
  DisplayWindow(WinPtr);
  InitEsRecord(Esr);
  Colors^.SetEntryAttributes;
  SetWrapMode(Esr, WrapAtEdges);
  Dest := 0;
  Note := '';
  AddStringField(Esr, 'Freq / Time', 5, 5, '', 5, 20, 19, 0, nil, Note);
  AddCmd(EsMy98,kbEnter);
  FastWriteStack(WinPtr,'<ESC> to cancel, <ENTER> to send',
                 4,3,Colors^.HeaderAttr);
  ShowMouse;
  repeat
    ExitCommand := EditScreen(Esr, 1, False);
  until ExitCommand in [EsMy49, EsMy98, EsQuit, EsDone];
  AddCmd(EsMy98,$0000);
  KillWindow(WinPtr);
  DisposeEditScreen(Esr);
  if ExitCommand = EsMy49 then Exit;
  if (ExitCommand in [EsDone,EsMy98]) and (Trim(Note) <> '') then
    SendNote(Dest, 'PASS >> '+ Who + Note);
end;

procedure   TNetWork.OkToReceive(const oktr : boolean);
begin
  if Assigned(Node[FirstNode]) then Node[FirstNode]^.OkToReceive(oktr);
  if Assigned(Node[SecondNode]) then Node[SecondNode]^.OkToReceive(oktr);
  NetRxOk := oktr;
  {$IFDEF DEBUG}
  if oktr then AppendDebugMessage('OK_TO_RECEIVE=TRUE')
          else AppendDebugMessage('OK_TO_RECEIVE=FALSE')
  {$ENDIF}
end;

procedure TNetwork.ProcessDataBlock(WhichNode : NodeSwitch);
begin
  if (not Assigned(Node[WhichNode])) or
     (GetNodeBlock(WhichNode) = 0) or
     (MD.OrigNode = ThisNode) then Exit; {then end of the line}
  { if it is for someother station, send it on its way & get out }
  if (MD.DestNode > 0) and (MD.DestNode <> ThisNode) then begin
    PassItOn(NOT WhichNode); {always pass it along if not for me}
    Exit;                    {then get out, dont handle it}
  end;

  { if a broadcast, send it on it's way now }
  if (MD.DestNode = 0) then PassItOn(NOT WhichNode);

  { now handle the messages locally }
  case MD.DataId of
    PingDataId       : HandlePing;
    PongDataId       : HandlePong;
    NoteDataId       : HandleNote;
    DateDataId       : HandleDateTime;
    LogDataId        : HandleLogData;
    DeleteLogDataId  : HandleDeleteLogData;
    QrgReqId         : HandleQrgReq;
    QrgDataId        : HandleQrgData;
    SerNumReqId      : HandleSerNumReq;
    SerNumDataId     : HandleSerNumData;
    ClusterRxDataId  : HandleClusterRxData;
    ClusterTxDataId  : HandleClusterTxData;
    TestMsgId        : HandleTestMsg;
  else
    Msg := 'TNetWork.Idle: Unknown data block! DataId = '
            +long2str(ord(block[1]));
  end;
end; { ProcessDataBlock }


procedure TNetWork.Idle;
begin
  if NOT NetRxOk then Exit;
  case Loopback of
    False: begin
             ProcessDataBlock(FirstNode);
             ProcessDataBlock(SecondNode);
             AutoSyncTime;
           end;
    True:  begin
             ProcessDataBlock(FirstNode);
             AutoSyncTime;
           end;
   end;
end;



procedure InitNetwork;
begin
  Net := nil;
  with RttyIni^ do begin
    StationId := GetProfileByte('StationNum','NetWork',0);
    if StationId = 0 then GetProfileByte('Station','NetWork',0);
  end;
  if StationID > 0 then
    Case RttyIni^.GetProfileBool('Loop','NetWork',False) of
      False : Net := New(PNetWorkD, Init(StationId));
      True  : Net := New(PNetWorkL, Init(StationId));
    end;
  if Assigned(Net) then
    AppendDebugMessage('NetWork: Net created on StationNum='
                       +long2str(StationId));
end;

end.
