{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit QtcRObj;                     { the QTC receive Object }


interface


uses TpDate,
  OpString,
  Filer,
  OoFiler,
  TpEntry,
  TpWindow,
  IniTools;

type
  QtcType = record
              QtcTime : Time;
              QtcCall : String[12];
              QtcSerNum : Word;
            end;

  QtcRecordType = record
                    Del : LongInt; { a delete flag }
                    RecordNumber : Integer;
                    QtcGroup : String[3];
                    Number : String[2];
                    GroupTime : Time;
                    GroupDate : Date;
                    Band : Byte;
                    RecdFrom : String[12];
                    QtcsRecd : array[1..10] of QtcType;
                    Filler : array[1..35] of Byte;
                  end;


  PQtcRFile = ^TQtcRFile;
  TQtcRFile = object(FileBlock)
                QtcWin   : WindowPtr;
                CurRef : LongInt;
                CurRec : QtcRecordType;
                BaseFName : String[11];
                constructor Init(QtcFName : IsamFileBlockName);
                function BuildKey(var Rec; KeyNr : Word) : IsamKeyStr; virtual;
                function EqualRec(var Rec1, Rec2) : Boolean; virtual;
                procedure CheckOK;
                procedure GotoTop;
                procedure ReportError;
                procedure HardError; virtual;
                function GetByCallsign(const Callsign : String) : Boolean;
                function HowManyGroups : LongInt;
                procedure RecdQtcData(const Call : String);
                function HowManyRecdFrom(const Call : String) : Integer;
                function HowManyRecdFromAtTime(const Call : String; const AtTime : Time; const OnDate : Date) : Integer;
                function NextRecordNumber : Integer;
                procedure CountByBand(var B0, B1, B2, B3, B4 : LongInt);
                function HowManyMoreFor(const Call : String) : Integer;
                function HowManyInGroup : Integer;
                procedure IAmQrv;
                procedure BlankRec(var C : QtcRecordType);
                procedure AgnAllQtc;
                function EditQtcData : Boolean;
                procedure EditQtcLog;
                procedure AgnQtc(const QtcNum : Byte);
                procedure SaveNewQtc;
                destructor Done; virtual;
                procedure PrintReport;
                procedure PrintSummaryReport;
                procedure Fix;
              end;

var
  QtcRFile : PQtcRFile;


implementation

uses
  ApPort,
  IniFile,
  TpMouse,
  TpCrt,
  DebugRtns,
  TpEdit,
  cEditWin,
  cCountry,
  OneTimers,
  Routines,
  Notes,
  Keys,
  LowLvl,
  cClock,
  TncControl,
  c_Contest,
  c_waed,
  cMsgWin,
  cRttyWin,
  cColors,
  qtcsobj,
  RecTypes;

type
  Fields = set of 0..32;

const
  TimeFields : Fields = [3, 6, 9, 12, 15, 18, 21, 24, 27, 30];
  CallFields : Fields = [4, 7, 10, 13, 16, 19, 22, 25, 28, 31];
  SerNFields : Fields = [5, 8, 11, 14, 17, 20, 23, 26, 29, 32];
  RecordNumberIndex = 1;
  CallSignIndex = 2;
  NumberOfIndexes = 2;
  QtcGroup = 0;
  QtcNumber = 1;
  QtcCall = 2;
  SaveMode = True;
  NetMode = False;


  constructor TQtcRFile.Init(QtcFName : IsamFileBlockName);
  var
    IID : IsamIndDescr;

    procedure InitIID;
    begin
      IID[RecordNumberIndex].KeyL := 3;
      IID[RecordNumberIndex].AllowDupK := False;
      IID[CallSignIndex].KeyL := 12;
      IID[CallSignIndex].AllowDupK := True;
    end;

  begin
    CurRef := 0;
    BaseFName := ForceExtension(JustName(QtcFName),'');
    if not inherited Init(QtcFName, False, False, SaveMode, NetMode) then begin
      if IsamError = 9903 then begin     {data file not found}
        InitIID;
        Create(QtcFName, SizeOf(QtcRecordType), NumberOfIndexes, IID,
                         False, False, SaveMode, NetMode);
        AppendDebugMessage('WAE QTC Receive File created: '+QtcFName);
        end
      else if IsamError = 10010 then begin { fileblock corrupt }
        InitIID;
        Recover(QtcFName, SizeOf(QtcRecordType), NumberOfIndexes, IID, True,
                          False, False, SaveMode, NetMode);
        AppendDebugMessage('WAE QTC Receive File recovered: '+QtcFName);
        end
      else MessageBox('SEVERE ERROR',
                 'Unable to create QTC rcv file:'+QtcFName,
                 'Please re-install, or contact WF1B',
                 'QTC receiving will not be available.');
    end else begin
      AppendDebugMessage('Qtc Rcv File, '+QtcFName+' initialized.');
    {$IFDEF DEBUG}
      AppendDebugMessage('TotalRecs  ='+long2str(TotalRecs));
      AppendDebugMessage('FreeRecs   ='+long2str(FreeRecs));
      AppendDebugMessage('UsedKeys(1)='+long2str(UsedKeys(1)));
      AppendDebugMessage('UsedKeys(2)='+long2str(UsedKeys(2)));
      AppendDebugMessage('UsedRecs   ='+long2str(UsedRecs));
    {$ENDIF}
    end;
  end;

  procedure TQtcRFile.AgnAllQtc;
  var
    Buffer : string[100];
  begin
    Buffer := RttyIni^.GetProfileString('Alt-A','Recd QTC', 'ALL AGN? BK'+^M );
    MakeSubstitutions(Buffer,100, EditW^.EditRecs[1]);
    if not RttyTerm1^.KeyBoardLive then RttyTerm1^.Transmit(True);
    RttyTerm1^.SendString(Buffer);
    if not RttyTerm1^.KeyBoardLive then RttyTerm1^.Transmit(False);
  end;

  function TQtcRFile.BuildKey(var Rec; KeyNr : Word) : IsamKeyStr;
  begin
    with QtcRecordType(Rec) do
      case KeyNr of
        RecordNumberIndex : BuildKey := LeftPadCh(long2str(RecordNumber), '0', 3);
        CallSignIndex     : BuildKey := RecdFrom;
      end;
  end;

  function TQtcRFile.HowManyRecdFrom(const Call : String) : Integer;
  var
    ThisRef : LongInt;
    ThisRec : QtcRecordType;
    FirstRef : LongInt;
    KeyStr : String[12];
    NumRecd : Integer;
    NumThisTime : Integer;
  begin
    KeyStr := Call;
    FindKey(CallSignIndex, ThisRef, KeyStr);
    if not IsamOK then
      HowManyRecdFrom := 0        {none recd from him so far...}
    else begin
      NumRecd := 0;
      GetRec(ThisRef, ThisRec);
      FirstRef := ThisRef;
      while (IsamOK) and (ThisRec.RecdFrom = Call) do begin
        Str2Int(ThisRec.Number, NumThisTime);
        Inc(NumRecd, NumThisTime);
        NextKey(CallSignIndex, ThisRef, KeyStr);
        GetRec(ThisRef, ThisRec);
        if ThisRef = FirstRef then IsamOK := False; { same record we started with}
      end;
      HowManyRecdFrom := NumRecd;
    end;
  end;

  function TQtcRFile.HowManyRecdFromAtTime(const Call : String; const AtTime : Time; const OnDate : Date) : Integer;
  var
    ThisRef : LongInt;
    ThisRec : QtcRecordType;
    FirstRef : LongInt;
    KeyStr : String[12];
    NumRecd : Integer;
    Hours, Mins, Secs : Byte;
    Found : Boolean;
  begin
    KeyStr := Call;
    FindKey(CallSignIndex, ThisRef, KeyStr);
    if not IsamOK then
      HowManyRecdFromAtTime := 0        {none recd from him so far...}
    else begin
      NumRecd := 0;
      Found := False;
      GetRec(ThisRef, ThisRec);
      FirstRef := ThisRef;
      while (not Found) and (IsamOK) and (ThisRec.RecdFrom = Call) do begin
        TimeDiff(AtTime,ThisRec.GroupTime,Hours,Mins,Secs);
        if (ThisRec.GroupDate = OnDate) and
           (Hours = 0) and (Mins < RttyIni^.GetProfileByte('Range','Recd QTC', 5)) then begin
          {within 10 mintes, must be the one I want to get}
          Str2Int(ThisRec.Number, NumRecd);
          Found := True;
        end;
        NextKey(CallSignIndex, ThisRef, KeyStr);
        GetRec(ThisRef, ThisRec);
        if ThisRef = FirstRef then IsamOK := False; { same record we started with}
      end;
      HowManyRecdFromAtTime := NumRecd;
    end;
  end;

  procedure TQtcRFile.BlankRec(var C : QtcRecordType);
  var
    B : Byte;
  begin
    with C do begin
      Del := 0;
      QtcGroup := '';
      Number := '';
      GroupTime := 0;
      GroupDate := 0;
      RecdFrom := '';
      Band := 0;
      for B := 1 to 10 do begin
        QtcsRecd[B].QtcTime := 0;
        QtcsRecd[B].QtcCall := '';
        QtcsRecd[B].QtcSerNum := 0;
      end;
    end;
  end;

  function TQtcRFile.HowManyInGroup : Integer;
  var
    i : Integer;
    Num : Integer;
  begin
    Num := 0;
    for i := 1 to 10 do
      if CurRec.QtcsRecd[i].QtcCall <> '' then Inc(Num);
    HowManyInGroup := Num;
  end;

  function TQtcRFile.HowManyMoreFor(const Call : String) : Integer;
  begin
    HowManyMoreFor := 10-
                      QtcSFile^.HowManySentTo(Call)-
                      HowManyRecdFrom(Call);
  end;

  procedure TQtcRFile.Fix;
  var
    ThisRef : LongInt;
    ThisRec : QtcRecordType;
    NewRec  : QtcRecordType;
    Num : Integer;
    KeyStr : String[3];
  begin
    KeyStr := '001';
    FindKey(RecordNumberIndex, ThisRef, KeyStr);
    while IsamOK do begin
      GetRec(ThisRef, ThisRec);
      Str2Int(ThisRec.Number, Num);
      AppendDebugMessage(' Rec#='+long2str(ThisRec.RecordNumber)+
                         ' Grp#='+ThisRec.QtcGroup+
                         ' Num#='+ThisRec.Number+
                         ' Time='+timetotimestring('hhmmss',thisrec.grouptime)+
                         ' Date='+DatetoDateString('yymmdd',thisrec.groupdate)+
                         ' Band='+long2str(ThisRec.band)+
                         ' From='+ThisRec.RecdFrom);

      NewRec := ThisRec;
      with NewRec do begin
        if Trim(QtcGroup) = '' then QtcGroup := '000';
        if Trim(Number) = '' then Number := '00';
        if Band > 5 then Band := 2; { default to 20M }
        RecdFrom := Pad(Trim(Copy(RecdFrom,1,12)),12);
      end;
      ModifyRecord(ThisRef, ThisRec, NewRec);
      NextDiffKey(RecordNumberIndex, ThisRef, KeyStr);
    end;
  end;



  procedure TQtcRFile.CountByBand(var B0, B1, B2, B3, B4 : LongInt);
  var
    ThisRef : LongInt;
    ThisRec : QtcRecordType;
    T : array[0..4] of LongInt;
    Num : Integer;
    KeyStr : String[3];
  begin
    T[0] := 0;
    T[1] := 0;
    T[2] := 0;
    T[3] := 0;
    T[4] := 0;
    KeyStr := '001';
    FindKey(RecordNumberIndex, ThisRef, KeyStr);
    while IsamOK do begin
      GetRec(ThisRef, ThisRec);
      Str2Int(ThisRec.Number, Num);
      Inc(T[ThisRec.Band], Num);
      NextDiffKey(RecordNumberIndex, ThisRef, KeyStr);
    end;
    B0 := T[0];
    B1 := T[1];
    B2 := T[2];
    B3 := T[3];
    B4 := T[4];
    {$IFDEF DEBUG}
    AppendDebugMessage('QtcRObj.CountByBand:'+
              long2str(b0)+','+
              long2str(b1)+','+
              long2str(b2)+','+
              long2str(b3)+','+
              long2str(b4)+' ');
    {$ENDIF}
  end;

  procedure TQtcRFile.PrintReport;
  var
    F       : Text;
    ThisRef : LongInt;
    ThisRec : QtcRecordType;
    Num : Integer;
    KeyStr : String[3];
    QtcNum : integer;
  begin
    Assign(F, ForceExtension(BaseFName,'RPT'));
    {$I-} Rewrite(F); {$I+}
    Writeln(F,'QTC Report - Received QTC data');
    writeln(f);
    KeyStr := '001';
    FindKey(RecordNumberIndex, ThisRef, KeyStr);
    while IsamOK do begin
      GetRec(ThisRef, ThisRec);
      with ThisRec do begin
        Writeln(F,'QTC ',QtcGroup, '/', Number,  ' @ ',
                  TimeToTimeString('hh:mmZ on ',GroupTime),
                  Copy('8040201510',Band*2+1,2), 'Mtrs from ',
                  RecdFrom);
        writeln(F);
        for QtcNum := 1 to 10 do
          if Trim(QtcsRecd[QtcNum].QtcCall) <> '' then
            writeln(F,'      '+TimeToTimeString('hhmm/',QtcsRecd[QtcNum].QtcTime)+
                    Trim(QtcsRecd[QtcNum].QtcCall)+'/'+
                    LeftPadCh(Long2str(QtcsRecd[QtcNum].QtcSernum),'0',3));
      end;
      Writeln(F);
      Writeln(F);
      NextDiffKey(RecordNumberIndex, ThisRef, KeyStr);
    end;
    Close(F);
  end;

  procedure TQtcRFile.PrintSummaryReport;
  var
    F       : Text;
    ThisRef : LongInt;
    ThisRec : QtcRecordType;
    Num : Integer;
    KeyStr : String[3];
    QtcNum : integer;
    TotQtc : integer;
  begin
    Assign(F, ForceExtension(BaseFName,'SUM'));
    {$I-} Rewrite(F); {$I+}
    Writeln(F,'QTC''s Received Summary');
    Writeln(F,'----------------------');
    writeln(f);
    writeln(f,'Time      Freq      Station      Grp/Cnt');
    writeln(f,'----      ----      -------      --- ---');
    TotQtc := 0;
    KeyStr := '001';
    FindKey(RecordNumberIndex, ThisRef, KeyStr);
    while IsamOK do begin
      GetRec(ThisRef, ThisRec);
      with ThisRec do begin
        Writeln(F,TimeToTimeString('hh:mmZ    ',GroupTime),
                  Copy('8040201510',Band*2+1,2), 'Mtrs    ',
                  pad(RecdFrom,13),
                  leftpadch(QtcGroup,'0',3),'/',Number);
        Str2Int(Number, Num);
        Inc(TotQtc,Num);
      end;
      NextDiffKey(RecordNumberIndex, ThisRef, KeyStr);
    end;
    Writeln(F);
    Writeln(F,'Total QTC''s Received = ',TotQtc);
    Close(F);
  end;

  procedure TQtcRFile.GotoTop;
  var
    KeyStr : String[3];
  begin                           { TQtcRFile.GotoTop }
    KeyStr := '001';
    FindKey(RecordNumberIndex, CurRef, KeyStr);
    if IsamOK then GetRec(CurRef, CurRec);
  end;                            { TQtcRFile.GotoTop }

  function TQtcRFile.EqualRec(var Rec1, Rec2) : Boolean;
  var
    Q1 : QtcRecordType absolute Rec1;
    Q2 : QtcRecordType absolute Rec2;
    X : Byte;
  begin
    EqualRec := False;
    if Q1.Del <> Q2.Del then Exit;
    if Q1.RecordNumber <> Q2.RecordNumber then Exit;
    if Q1.QtcGroup <> Q2.QtcGroup then Exit;
    if Q1.Number <> Q2.Number then Exit;
    if Q1.GroupTime <> Q2.GroupTime then Exit;
    if Q1.GroupDate <> Q2.GroupDate then Exit;
    if Q1.Band <> Q2.Band then Exit;
    if Q1.RecdFrom <> Q2.RecdFrom then Exit;
    for X := 1 to 10 do begin
      if Q1.QtcsRecd[X].QtcTime <> Q2.QtcsRecd[X].QtcTime then Exit;
      if Q1.QtcsRecd[X].QtcCall <> Q2.QtcsRecd[X].QtcCall then Exit;
      if Q1.QtcsRecd[X].QtcSerNum <> Q2.QtcsRecd[X].QtcSerNum then Exit;
    end;
    EqualRec := True;
  end;

  procedure TQtcRFile.ReportError;
  begin
    AppendDebugMessage('TQtcRObj.ReportError: Isam Error: '+long2str(IsamError));
  end;

  procedure TQtcRFile.HardError;
  begin
    ReportError;
    Done;
    BTExitIsam;
    Halt;
  end;

  procedure TQtcRFile.CheckOK;
  begin
    if not IsamOK then begin
      ReportError;
      Halt;
    end;
  end;

  function TQtcRFile.GetByCallsign(const Callsign : String) : Boolean;
  begin
    FindKey(CallSignIndex, CurRef, Callsign);
    if not IsamOK then
      GetByCallsign := False
    else begin
      GetRec(CurRef, CurRec);
      GetByCallsign := True;
    end;
  end;

  function TQtcRFile.HowManyGroups : LongInt;
  begin
    HowManyGroups := UsedRecs;
  end;

  procedure TQtcRFile.IAmQrv;
  var
    Buffer : string[100];
  begin
    Buffer := RttyIni^.GetProfileString('Alt-Q','Recd QTC', '<CR><1> QRV DE <11> KN' );
    MakeSubstitutions(Buffer,100,EditW^.EditRecs[1]);
    if not RttyTerm1^.KeyBoardLive then RttyTerm1^.Transmit(True);
    RttyTerm1^.SendString(Buffer);
    if not RttyTerm1^.KeyBoardLive then RttyTerm1^.Transmit(False);
  end;

  procedure TQtcRFile.AgnQtc(const QtcNum : Byte);
  begin
    if not RttyTerm1^.KeyBoardLive then RttyTerm1^.Transmit(True);
    RttyTerm1^.SendString('AGN? '+long2str(QtcNum)+'-'+long2str(QtcNum)+'  BK'+^M);
    if not RttyTerm1^.KeyBoardLive then RttyTerm1^.Transmit(False);
  end;

  function TQtcRFile.EditQtcData : Boolean;
  var
    Esr : EsRecord;
    ExitCommand : EStype;
    Num : Byte;
    HoldPutMsgs : Boolean;

    function IsItSerno(const TestS : String) : Boolean;
    begin
      IsItSerno := False;
      if (TestS[3] = ':') or (TestS[5] = 'Z') or (TestS[6] = 'Z') then Exit;
      if not(TestS[1] in ['0'..'9']) then Exit;
      if not(TestS[2] in ['0'..'9']) then Exit;
      if not(TestS[3] in ['0'..'9']) then Exit;
      if length(TestS) >= 4 then
        if not(TestS[4] in ['0'..'9', ' ']) then Exit;
      IsItSerno := True;
    end;

    function IsItTime(const TestS : String) : Boolean;
    var
      T : Time;
    begin
      IsItTime := False;
      if not(TestS[1] in ['0'..'2']) then Exit;
      if not(TestS[2] in ['0'..'9']) then Exit;
      if not(TestS[3] in ['0'..'9']) then Exit;
      if not(TestS[4] in ['0'..'9']) then Exit;
      T := TimeStringToTime('hhmm', Copy(TestS, 1, 4));
      if T = BadTime then Exit;
      IsItTime := True;
    end;

    function IsItCall(const TestS : String) : Boolean;
    var
      i, L : Integer;
      HasNums,
      HasLtrs : Boolean;
    begin
      IsItCall := False;
      L := Length(Trim(TestS));
      HasNums := False;
      HasLtrs := False;
      for i := 1 to L do begin
        if not(TestS[i] in ['A'..'Z', '0'..'9', '/']) then Exit;
        if TestS[i] in ['0'..'9'] then HasNums := True;
        if TestS[i] in ['A'..'Z'] then HasLtrs := True;
      end;
      if HasNums and HasLtrs and (L > 3) then IsItCall := True;
      if (TestS[1] in ['0'..'9']) and
      (TestS[2] in ['0'..'9']) and
      (TestS[3] in ['0'..'9']) and
      (TestS[4] in ['0'..'9']) and
      (TestS[5] in ['Z', ' ']) then IsItCall := False; {* 2.12e *}
    end;

    function ESRCurrentFieldId(const Row, Col : Byte) : Word;
    begin
      if Row = ScreenHeight-6 then begin
        if Col in [4..10] then ESRCurrentFieldId := QtcGroup;
        if Col in [11..13] then ESRCurrentFieldId := QtcNumber;
        Exit;
      end;
      if (Row > ScreenHeight-6) and (Row < ScreenHeight-6+6) then
        case Col of
          05..08 : ESRCurrentFieldId := ((Row-ScreenHeight-6)*3)+0;
          10..21 : ESRCurrentFieldId := ((Row-ScreenHeight-6)*3)+1;
          23..26 : ESRCurrentFieldId := ((Row-ScreenHeight-6)*3)+2;
          32..35 : ESRCurrentFieldId := ((Row-ScreenHeight-6-1)*3)+18;
          37..48 : ESRCurrentFieldId := ((Row-ScreenHeight-6-1)*3)+19;
          50..53 : ESRCurrentFieldId := ((Row-ScreenHeight-6-1)*3)+20;
        end;
    end;

    procedure HandleRightClick;
    var
      EntireRow : String[80];
      Temp      : string[19];
      Row, Col  : Byte;
      S_time    : string[4];
      S_Call    : string[12];
      S_SerNo   : string[3];
      S_QtcGroup: string[3];
      S_Number  : string[2];
      Num       : integer;
      W         : word;

      procedure ProcessQTCHeader;
      begin
        S_QtcGroup := ExtractWord(2, EntireRow, ['-','/',' ']);
        if Str2Word(S_QtcGroup, W) then
          CurRec.QtcGroup := ExtractWord(2, EntireRow, ['-','/',' ']);
        S_Number := ExtractWord(3, EntireRow, ['-','/',' ']);
        if Str2Word(S_Number, W) then
          CurRec.Number := ExtractWord(3, EntireRow, ['-','/',' ']);
        Esr.CurrentID := 3;
      end;

      function IsItSerno(TestS : String) : Boolean;
      begin
        IsItSerno := False;
        if not(TestS[1] in ['0'..'9']) then Exit;
        if not(TestS[2] in ['0'..'9']) then Exit;
        if not(TestS[3] in ['0'..'9']) then Exit;
        if length(TestS) = 4 then
          if not(TestS[4] in ['0'..'9', ' ']) then Exit;
        IsItSerno := True;
      end;

      procedure ProcessQTCData;
      var
        t : array[1..4] of string[19];
        i : integer;
      begin
        if Esr.CurrentID < 3 then Esr.CurrentID := 3;
        for i := 1 to 4 do
          t[i] := ExtractWord(i, EntireRow, ['-','/',' ']);
        S_Time := t[1]; { first word is time }
        S_Call := t[2]; { second word is start of callsign}
        case IsItSerno(T[3]) of
          TRUE : S_Serno := T[3];
          FALSE: begin
                   if EntireRow[Pos(S_Call,EntireRow) + length(S_Call)] = '/' then
                     S_Call := S_Call + '/' + t[3];
                   S_Serno := T[4];
                 end;
        end;
        Num := Esr.CurrentId div 3;
        CurRec.QtcsRecd[Num].QtcTime := TimeStringToTime('hhmm', S_Time);
        CurRec.QtcsRecd[Num].QtcCall := Trim(S_Call);
        if Str2Word(S_Serno, W) then
          CurRec.QtcsRecd[Num].QtcSerNum := W;
        Esr.CurrentID := (Num*3) + 3;
        if Esr.CurrentID > 32 then Esr.CurrentID := 3;
      end;

    begin
      Col := MouseWhereX;
      Row := MouseWhereY;
      FastRead(78, Row, 2, EntireRow);
      EntireRow := Trim(EntireRow);
      Temp := ExtractWord(1, EntireRow, ['-','/',' ']);
      if Temp = 'QTC' then
        ProcessQTCHeader else
        ProcessQTCData;
    end;

    procedure AcceptTime(const FieldId : Word; const S : String);
    begin
      if IsItTime(S) then begin
        CurRec.QtcsRecd[FieldId div 3].QtcTime := TimeStringToTime('hhmm', S);
        Inc(Esr.CurrentId);
      end;
    end;

    procedure AcceptCall(const FieldId : Word; const S : String);
    begin
      if IsItCall(S) then begin
        CurRec.QtcsRecd[(FieldId-1) div 3].QtcCall := Trim(S);
        Inc(Esr.CurrentId);
      end;
    end;

    procedure AcceptSern(const FieldId : Word; const S : String);
    var
      W : Word;
    begin
      if IsItSerno(S) then
        if Str2Word(S, W) then begin
          CurRec.QtcsRecd[(FieldId-2) div 3].QtcSerNum := W;
          Inc(Esr.CurrentId);
        end;
    end;

    procedure HandleLeftClick;
    var
      Row, Col : Byte;
      FieldId : Word;
      EntireRow : String[80];
      S : String[12];
      StartPos : Integer;
      W : word;
    begin
      Col := MouseWhereX;
      Row := MouseWhereY;
      if Row >= ScreenHeight-6 then begin { we are in bottom of screen area}
        FieldId := Esr.CurrentId;
        Esr.CurrentId := ESRCurrentFieldId(Row, Col); {see if mouse changed it}
        if Col in [57..77] then begin
          if Row = ScreenHeight-6+1 then ExitCommand := ESmy92;
          if Row = ScreenHeight-6+2 then ExitCommand := ESmy80;
          if Row = ScreenHeight-6+4 then ExitCommand := ESdone;
          if Row = ScreenHeight-6+5 then ExitCommand := ESmy91;
        end;
        Exit;                     {blast out}
      end;
      if (Row > MW_Ylo) and (Row < MW_Yhi) and (Col > 1) and (Col < MW_xhi) then begin
        FastRead(78, Row, 2, EntireRow);
        Dec(Col);
        while (EntireRow[Col] in ['A'..'Z', '0'..'9']) and (Col > 0) do Dec(Col);
        Inc(Col);
        StartPos := Col;
        while (EntireRow[Col] in ['A'..'Z', '0'..'9']) and (Col < StartPos+12) do Inc(Col);
        S := Copy(EntireRow, StartPos, Col-StartPos);
        FieldId := Esr.CurrentId;
        if (FieldId in TimeFields) then AcceptTime(FieldId, S);
        if (FieldId in CallFields) then AcceptCall(FieldId, S);
        if (FieldId in SerNFields) then AcceptSern(FieldId, S);
        if (FieldId = 0) then
          if (S[1] in ['0'..'9',' ']) and
             (S[2] in ['0'..'9',' ']) and
             (S[3] in ['0'..'9',' ']) then begin
            CurRec.QtcGroup := S;
            Esr.CurrentId := 1;
          end;
        if (FieldId = 1) then
          if (S[1] in ['0'..'9',' ']) and
             (S[2] in ['0'..'9',' ']) then begin
            CurRec.Number := S;
            Esr.CurrentId := 3;
          end;
      end;
    end;

  begin
    EditQtcData := False;
    with Colors^ do begin
    if not MakeWindow(QtcWin, 1, ScreenHeight-6, ScreenWidth, ScreenHeight, True, True, True, WindowAttr,
                      FrameAttr, HeaderAttr, 'Receive QTC Data') then begin
      Msgwin^.Display('Not enough memory to make window');
      Exit;
    end;
    if not DisplayWindow(QtcWin) then begin
      Killwindow(QtcWin);
      Msgwin^.Display('Not enough memory to display window');
      Exit;
    end;
    HideMouse;
    InitEsRecord(Esr);
    EnableEntryMouse;
    SetWrapMode(Esr, WrapAtEdges);
    AddStringField(Esr, 'QTC ', ScreenHeight-6, 4, '', ScreenHeight-6, 8, 3, 0, nil, CurRec.QtcGroup);
    AddStringField(Esr, '/', ScreenHeight-6, 11, '', ScreenHeight-6, 12, 2, 0, nil, CurRec.Number);
    AddStringField(Esr, '', ScreenHeight-6, 14, '', ScreenHeight-6, 15, 12, 0, nil, CurRec.RecdFrom);
    FastWrite('', ScreenHeight-6, 3, FrameAttr);
    FastWrite('', ScreenHeight-6, 27, FrameAttr);
    FastWrite(' ', ScreenHeight-6, 14, FrameAttr);
    for Num := 1 to 5 do begin
      SetForceMode(True,True);
      AddTimeField(Esr, long2str(Num), ScreenHeight-6+Num, 3, 'hhmm', ScreenHeight-6+Num, 5, 0, 0, 0,
        CurRec.QtcsRecd[Num].QtcTime);
      SetForceMode(False,False);
      AddStringField(Esr, '/', ScreenHeight-6+Num, 9, '!!!!!!!!!!!!', ScreenHeight-6+Num, 10, 12, 0, nil,
        CurRec.QtcsRecd[Num].QtcCall);
      SetForceMode(True,True);
      AddWordField(Esr, '/', ScreenHeight-6+Num, 22, '9999', ScreenHeight-6+Num, 23, 0, 0, 9999,
        CurRec.QtcsRecd[Num].QtcSerNum);
      SetForceMode(False,False);
      FastWrite('', ScreenHeight-6+Num, 28, FrameAttr);
    end;
    FastWrite('', ScreenHeight-6, 28, FrameAttr);
    FastWrite('', ScreenHeight-6+6, 28, FrameAttr);
    for Num := 1 to 5 do begin
      SetForceMode(True,True);
      AddTimeField(Esr, LeftPad(long2str(Num+5), 2), ScreenHeight-6+Num, 29, 'hhmm', ScreenHeight-6+Num, 32, 0, 0, 0,
        CurRec.QtcsRecd[Num+5].QtcTime);
      SetForceMode(False,False);
      AddStringField(Esr, '/', ScreenHeight-6+Num, 36, '!!!!!!!!!!!!', ScreenHeight-6+Num, 37, 12, 0, nil,
        CurRec.QtcsRecd[Num+5].QtcCall);
      SetForceMode(True,True);
      AddWordField(Esr, '/', ScreenHeight-6+Num, 49, '9999', ScreenHeight-6+Num, 50, 0, 0, 9999,
        CurRec.QtcsRecd[Num+5].QtcSerNum);
      SetForceMode(False,False);
      FastWrite('', ScreenHeight-6+Num, 55, FrameAttr);
    end;
    FastWrite('', ScreenHeight-6, 55, FrameAttr);
    FastWrite('', ScreenHeight-6+6, 55, FrameAttr);
    FastWrite('Alt-Q   QRV', ScreenHeight-6+1, 57, WindowAttr);
    FastWrite('Alt-A   Agn All Qtc''s', ScreenHeight-6+2, 57, WindowAttr);
    FastWrite('F#      Agn Qtc #', ScreenHeight-6+3, 57, WindowAttr);
    FastWrite('^Enter  Done', ScreenHeight-6+4, 57, WindowAttr);
    FastWrite('Alt-C   Cancel', ScreenHeight-6+5, 57, WindowAttr);
    AddCmd(ESmy80, kbAltA);
    AddCmd(Esmy81, kbF1);
    AddCmd(Esmy82, kbF2);
    AddCmd(Esmy83, kbF3);
    AddCmd(Esmy84, kbF4);
    AddCmd(Esmy85, kbF5);
    AddCmd(Esmy86, kbF6);
    AddCmd(Esmy87, kbF7);
    AddCmd(Esmy88, kbF8);
    AddCmd(Esmy89, kbF9);
    AddCmd(Esmy90, kbF10);
    AddCmd(ESmy91, kbAltC);
    AddCmd(ESmy92, kbAltQ);
    gQtcExchange := True;         {signal to Rtty.MyGetKey}
    { out messages wont allow right click to work, so diable it temporarily}
    HoldPutMsgs := RttyIni^.GetProfileBool('PutMsgs', 'PROFILE', False);
    RttyIni^.SetOrAddProfileString('PutMsgs', 'PROFILE', 'False');
    repeat
      ExitCommand := EditScreen(Esr, Esr.CurrentId, False);
      if ExitCommand = ESmy20 then HandleLeftClick;
      if ExitCommand = ESmy21 then HandleRightClick;
      case ExitCommand of
        ESdone : {allow to fall through} ;
        ESmy80 : AgnAllQtc;
        Esmy81 : AgnQtc(1);
        Esmy82 : AgnQtc(2);
        Esmy83 : AgnQtc(3);
        Esmy84 : AgnQtc(4);
        Esmy85 : AgnQtc(5);
        Esmy86 : AgnQtc(6);
        Esmy87 : AgnQtc(7);
        Esmy88 : AgnQtc(8);
        Esmy89 : AgnQtc(9);
        Esmy90 : AgnQtc(10);
        ESmy91 : { alt-c -- cancel } ;
        ESmy92 : IAmQrv;
        ESmy7  : RttyTerm1^.ToggleChatMode;
        ESmy11 : AddNote;
        Esmy49 : RttyTerm1^.ResetBox; {esc - emergency stop}
        EsMy60..EsMy69 : RttyWin^.SendKeys(ExitCommand,EditW^.EditRecs[1]);
        ESmy10 : RttyTerm1^.Letters;
        ESmy31 : RttyTerm1^.Figures;
        Esmy53 : RttyTerm1^.ToggleNormRev;
      end;
    until ExitCommand in [ESmy91, ESdone];
    Case HoldPutMsgs of
      TRUE: RttyIni^.SetOrAddProfileString('PutMsgs', 'PROFILE', 'True');
      FALSE:RttyIni^.SetOrAddProfileString('PutMsgs', 'PROFILE', 'False');
    end;
    gQtcExchange := False;        {signal to Rtty.MyGetKey}
    if ExitCommand = ESdone then EditQtcData := True; {save our qtc data}
    AddMyCommands;                {reset all overridden commands back to default values}
    end;
    KillWindow(QtcWin);
    DisposeEditScreen(Esr);
  end;

  procedure TQtcRFile.SaveNewQtc;
  begin
    AddRecord(CurRef, CurRec);
  end;

  procedure TQtcRFile.RecdQtcData(const Call : String);
  var
    i : Integer;
    NumToRecd : Integer;
    NumRecdFrom : Integer;
    SaveQtcData : Boolean;
  begin
    if Call = '' then begin
      RttyWin^.PutString(^M^G+'Need to enter a callsign in order to receive QTC data'^M);
      Exit;
    end;
    if MyContinent = CountryList^.CT.Continent then begin
      RttyWin^.PutString(^M^G+'Cannot receive QTC''s from same continent ');
      Exit;
    end;
    BlankRec(CurRec);
    with CurRec do begin
      RecdFrom := Call;
      RecordNumber := NextRecordNumber;
      Band := Ord(EditW^.EditRecs[1].Band);
    end;
    NumToRecd := HowManyMoreFor(Call);
    if NumToRecd <= 0 then begin
      RttyWin^.PutString(^M^G+'Cannot receive anymore QTC''s from '+CurRec.RecdFrom);
      Exit;
    end;
    {$IFDEF DEBUG}
    if NumToRecd < 0 then begin
      AppendDebugMessage('QtcRObj.RecdQtcData: Already exchanged too many with '+Call);
      Exit;
    end;
    {$ENDIF}
    SaveQtcData := EditQtcData;
    if SaveQtcData then begin
      Clock^.DateAndTimeutc(CurRec.GroupDate, CurRec.GroupTime);
      SaveNewQtc;
      MsgWin^.Display('QTC received data saved to disk.');
    end;
  end;

  function TQtcRFile.NextRecordNumber : Integer;
  begin
    NextRecordNumber := HowManyGroups+1;
  end;

  destructor TQtcRFile.Done;
  begin
    inherited Done;
    {$IFDEF DEBUG}
    AppendDebugMessage('QtcRObj destroyed');
    {$ENDIF}
  end;


  procedure IncChoice(var Value; FieldID : Byte;
                      Factor         : Integer;
                      var St         : String); far;
  var
    Band           : Byte absolute Value;
  begin
    case FieldID of
      003 : begin
              if Band > 4 then begin
                Band := 0;
                Alarm;
              end;
              case Factor of
                1 : if Band = 4 then Band := 0
                    else Inc(Band);
                -1 : if Band = 0 then Band := 4
                     else Dec(Band);
              end;
              St := Copy('8040201510',Band*2+1,2);
            end;
    end;
  end;



  procedure TQtcRFile.EditQtcLog;
  var
    Esr : EsRecord;
    ExitCommand : EStype;
    Num : Byte;
    KeyStr : string[12];
    HowManyRecs : longint;
    OldRec : QtcRecordType;

    procedure NotFoundMessage;
    var
      WinPtr         : WindowPtr;
    begin
      with Colors^ do
      MakeWindow(WinPtr, 5, 15, 25, 17, True, True, False, WindowAttr,
                FrameAttr, HeaderAttr, 'Search Results') ;
      DisplayWindow(WinPtr) ;
      FastwriteWindow('Callsign not found!',1, 1, Colors^.WindowAttr);
      Delay(2000);
      KillWindow(WinPtr);
    end;

    procedure SearchForCallsign;

      procedure NotFoundMessage;
      var
        WinPtr         : WindowPtr;
      begin
        with Colors^ do
          MakeWindow(WinPtr, 5, 15, 25, 17, True, True, False, WindowAttr,
                    FrameAttr, HeaderAttr, 'Search Results') ;
        DisplayWindow(WinPtr) ;
        FastwriteWindow('Callsign not found!',1, 1, Colors^.WindowAttr);
        Delay(2000);
        KillWindow(WinPtr);
      end;

      function GetSearchStr : string;
      var
        WinPtr         : WindowPtr;
        Escaped        : Boolean;
        SearchStr      : string[12];
      begin
        SearchStr := '';
        with Colors^ do begin
          MakeWindow(WinPtr, 5, 15, 20, 17, True, True, False, WindowAttr,
                    FrameAttr, HeaderAttr, 'Search') ;
          DisplayWindow(WinPtr) ;
          ReadString('', 1, 1, 12, PromptAttr, StringAttr, CtrlAttr, Escaped, SearchStr);
          SearchStr := StUpCase(Trim(SearchStr));
          GetSearchStr := SearchStr;
        end;
        KillWindow(WinPtr);
      end;

    begin
      KeyStr := GetSearchStr;
      if KeyStr <> '' then begin
        GotoTop;
        FindKey(CallSignIndex, CurRef, KeyStr);
        if IsamOK then begin
          GetRec(CurRef, CurRec);
          OldRec := CurRec;
        end
          else
          NotFoundMessage;
      end;
    end;

    Procedure FindNextMatch;
    begin
      NextKey(CallSignIndex, CurRef, KeyStr);
      GetRec(CurRef, CurRec);
      OldRec := CurRec;
    end;

    procedure ScrollBack;
    var
      RecKey : string[3];
    begin
      if CurRec.RecordNumber > 1 then begin
        ModifyRecord(CurRef, OldRec, CurRec);
        RecKey := LeftPadCh(long2str(CurRec.RecordNumber), '0', 3);
        PrevDiffKey(RecordNumberIndex, CurRef, RecKey);
        GetRec(CurRef, CurRec);
        OldRec := CurRec;
      end;
    end;

    procedure ScrollForward;
    var
      RecKey : string[3];
    begin
      if CurRec.RecordNumber < HowManyRecs then begin
        ModifyRecord(CurRef, OldRec, CurRec);
        RecKey := LeftPadCh(long2str(CurRec.RecordNumber), '0', 3);
        NextDiffKey(RecordNumberIndex, CurRef, RecKey);
        GetRec(CurRef, CurRec);
        OldRec := CurRec;
      end;
    end;

    function ESRCurrentFieldId(const Row, Col : Byte) : Word;
    begin
      if Row = ScreenHeight-6 then begin
        if Col in [4..10] then ESRCurrentFieldId := QtcGroup;
        if Col in [11..13] then ESRCurrentFieldId := QtcNumber;
        Exit;
      end;
      if (Row > ScreenHeight-6) and (Row < ScreenHeight-6+6) then
        case Col of
          05..08 : ESRCurrentFieldId := ((Row-ScreenHeight-6)*3)+0;
          10..21 : ESRCurrentFieldId := ((Row-ScreenHeight-6)*3)+1;
          23..26 : ESRCurrentFieldId := ((Row-ScreenHeight-6)*3)+2;
          32..35 : ESRCurrentFieldId := ((Row-ScreenHeight-6-1)*3)+18;
          37..48 : ESRCurrentFieldId := ((Row-ScreenHeight-6-1)*3)+19;
          50..53 : ESRCurrentFieldId := ((Row-ScreenHeight-6-1)*3)+20;
        end;
    end;

    procedure HandleLeftClick;
    var
      Row, Col : Byte;
      FieldId : Word;
      EntireRow : String[80];
      S : String[12];
      StartPos : Integer;
      W : word;
    begin
      Col := MouseWhereX;
      Row := MouseWhereY;
      if Row >= ScreenHeight-6 then begin { we are in bottom of screen area}
        FieldId := Esr.CurrentId;
        Esr.CurrentId := ESRCurrentFieldId(Row, Col); {see if mouse changed it}
        if Col in [57..77] then begin
          if Row = ScreenHeight-6+1 then ExitCommand := ESmy81;
          if Row = ScreenHeight-6+2 then ExitCommand := ESmy82;
          if Row = ScreenHeight-6+3 then ExitCommand := ESmy87;
          if Row = ScreenHeight-6+4 then ExitCommand := ESmy88;
          if Row = ScreenHeight-6+5 then ExitCommand := ESdone;
        end;
      end;
    end;

  begin
    with Colors^ do begin
    if not MakeWindow(QtcWin, 1, ScreenHeight-6, ScreenWidth, ScreenHeight, True, True, True, WindowAttr,
                      FrameAttr, HeaderAttr, 'Edit QTC Log Data') then begin
      Msgwin^.Display('Not enough memory to make QTC Edit window');
      Exit;
    end;
    if not DisplayWindow(QtcWin) then begin
      Killwindow(QtcWin);
      Msgwin^.Display('Not enough memory to display QTC Edit window');
      Exit;
    end;
    HideMouse;
    InitEsRecord(Esr);
    EnableEntryMouse;
    SetWrapMode(Esr, WrapAtEdges);
    AddStringField(Esr, 'QTC ', ScreenHeight-6, 4, '', ScreenHeight-6, 8, 3, 0, nil, CurRec.QtcGroup);
    AddStringField(Esr, '/', ScreenHeight-6, 11, '', ScreenHeight-6, 12, 2, 0, nil, CurRec.Number);
    AddStringField(Esr, '', ScreenHeight-6, 14, '!!!!!!!!!!!!', ScreenHeight-6, 15, 12, 0, nil, CurRec.RecdFrom);
    AddChoiceField(Esr, '', ScreenHeight-6, 28, 'XX',ScreenHeight-6, 28, 0, 1, @IncChoice, CurRec.Band);
    FastWrite('', ScreenHeight-6, 3, FrameAttr);
    FastWrite('', ScreenHeight-6, 30, FrameAttr);
    FastWrite(' ', ScreenHeight-6, 14, FrameAttr);
    FastWrite(' ', ScreenHeight-6, 27, FrameAttr);
    for Num := 1 to 5 do begin
      SetForceMode(True,True);
      AddTimeField(Esr, long2str(Num), ScreenHeight-6+Num, 3, 'hhmm', ScreenHeight-6+Num, 5, 0, 0, 0,
        CurRec.QtcsRecd[Num].QtcTime);
      SetForceMode(False,False);
      AddStringField(Esr, '/', ScreenHeight-6+Num, 9, '!!!!!!!!!!!!', ScreenHeight-6+Num, 10, 12, 0, nil,
        CurRec.QtcsRecd[Num].QtcCall);
      SetForceMode(True,True);
      AddWordField(Esr, '/', ScreenHeight-6+Num, 22, '9999', ScreenHeight-6+Num, 23, 0, 0, 9999,
        CurRec.QtcsRecd[Num].QtcSerNum);
      SetForceMode(False,False);
      FastWrite('', ScreenHeight-6+Num, 28, FrameAttr);
    end;
    FastWrite('', ScreenHeight-6, 28, FrameAttr);
    FastWrite('', ScreenHeight-6+6, 28, FrameAttr);
    for Num := 1 to 5 do begin
      SetForceMode(True,True);
      AddTimeField(Esr, LeftPad(long2str(Num+5), 2), ScreenHeight-6+Num, 29, 'hhmm', ScreenHeight-6+Num, 32, 0, 0, 0,
        CurRec.QtcsRecd[Num+5].QtcTime);
      SetForceMode(False,False);
      AddStringField(Esr, '/', ScreenHeight-6+Num, 36, '!!!!!!!!!!!!', ScreenHeight-6+Num, 37, 12, 0, nil,
        CurRec.QtcsRecd[Num+5].QtcCall);
      SetForceMode(True,True);
      AddWordField(Esr, '/', ScreenHeight-6+Num, 49, '9999', ScreenHeight-6+Num, 50, 0, 0, 9999,
        CurRec.QtcsRecd[Num+5].QtcSerNum);
      SetForceMode(False,False);
      FastWrite('', ScreenHeight-6+Num, 55, FrameAttr);
    end;
    FastWrite('', ScreenHeight-6, 55, FrameAttr);
    FastWrite('', ScreenHeight-6+6, 55, FrameAttr);
    FastWrite('F1      Search Call', ScreenHeight-6+1, 57, WindowAttr);
    FastWrite('F2      Search Next', ScreenHeight-6+2, 57, WindowAttr);
    FastWrite('F7      Prev QTC', ScreenHeight-6+3, 57, WindowAttr);
    FastWrite('F8      Next QTC', ScreenHeight-6+4, 57, WindowAttr);
    FastWrite('^Enter  Done', ScreenHeight-6+5, 57, WindowAttr);
    end;
    AddCmd(Esmy81, kbF1);
    AddCmd(Esmy82, kbF2);
    AddCmd(Esmy87, kbF7);
    AddCmd(Esmy88, kbF8);
    HowManyRecs := HowManyGroups;
    GotoTop;
    OldRec := CurRec;
    gQtcExchange := False; {signal to Rtty.MyGetKey}
    gInMainLoop := False; {signal to Rtty.MyGetKey}
    repeat
      ExitCommand := EditScreen(Esr, Esr.CurrentId, False);
      if ExitCommand = ESmy20 then HandleLeftClick;
      case ExitCommand of
        ESdone : {allow to fall through} ;
        Esmy81 : SearchForCallsign;
        Esmy82 : FindNextMatch;
        Esmy87 : ScrollBack;
        Esmy88 : ScrollForward;
        EsMy49 : begin
                   CurRec := OldRec;
                   DrawEditScreen(Esr);
                 end;
      end;
    until ExitCommand in [ESdone, EsMy91];
    gQtcExchange := False;  {signal to Rtty.MyGetKey}
    gInMainLoop := True;    {signal to Rtty.MyGetKey}
    ModifyRecord(CurRef, OldRec, CurRec);
    AddMyCommands;                {reset all overridden commands back to default values}
    KillWindow(QtcWin);
    DisposeEditScreen(Esr);
  end;



end.
