{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit QtcSObj; { the QTC send Object }

interface

uses
  TpDate,
  TpWindow,
  OpString,
  Filer,
  OoFiler,
  IniTools;

const

  GroupIndex = 1;
  CallSignIndex = 2;
  NumberOfIndexes = 2;

type
  QtcType        = record
                     QtcTime        : Time;
                     QtcCall        : string[12];
                     QtcSerNum      : word;
                     TARecNum       : LongInt; { Taccess Record Number}
                   end;

  QtcRecordType  = record
                     Del            : LongInt;   { a delete flag }
                     QtcGroup       : string[3];
                     Number         : string[2];
                     GroupTime      : Time;
                     GroupDate      : Date;
                     SentTo         : String[12];
                     Band           : byte;
                     QtcsExchanged  : array[1..10] of QtcType;
                   end;

  PQtcSFile = ^TQtcSFile;
  TQtcSFile = object(FileBlock)
              QtcWin   : WindowPtr;
              CurRef    : LongInt;
              CurRec    : QtcRecordType;
              BaseFName : string[11];
              constructor Init(QtcFName : IsamFileBlockName);
              function    BuildKey(var Rec; KeyNr : Word) : IsamKeyStr; virtual;
              function    EqualRec(var Rec1, Rec2) : Boolean; virtual;
              procedure   CheckOK;
              procedure   ReportError;
              procedure   HardError; virtual;
              procedure   GotoTop;
              function    GetByNumber(const GroupNum : string) : boolean;
              function    GetByCallsign(const Callsign : string) : boolean;
              function    HowManyGroups : LongInt;
              function    NextGroupNumber : string;
              procedure   SendQtcData(const Call : string);
              function    EditQtcData : boolean;
              procedure   EditQtcLog;
              procedure   BlankRec(var C : QtcRecordType);
              procedure   SaveNewQTC;
              procedure   SendAllQtcs;
              procedure   SendQtc(const QtcNum : byte);
              procedure   RetrieveQtcData(const HowMany : integer);
              procedure   FlagQtcData;
              function    HowManyMoreFor(const Call : string) : integer;
              function    HowManySentTo(const Call : string) : integer;
              function    HowManySentToAtTime(const Call : String; const AtTime : Time; const OnDate : Date) : Integer;
              function    HowManyInGroup : integer;
              procedure   AreYouQrv;
              procedure   PrintReport;
              procedure   PrintSummaryReport;
              procedure   QueryStatus(const ACall : string);
              procedure   CountByBand(var B0,B1,B2,B3,B4 : longint);
              destructor  Done; virtual;
            end;

var
  QtcSFile : PQtcSFile;

implementation

uses
    ApPort,
    IniFile,
    OneTimers, {for AddMyCommands routine}
    LowLvl,    {for PutString routine}
    TncControl,{for SendString routine}
    Routines,  {chat mode}
    Notes,     {adding to note file}
    Diskio,    {get qtc data}
    cEditWin,
    cCountry,
    cColors,
    cRttyWin,
    RecTypes,
    DebugRtns,
    TAccess,
    TpMouse,
    TpEntry,
    TpEdit,
    TpCrt,
    cClock,
    Keys,
    c_Contest,
    c_waed,
    cMsgWin,
    qtcRobj,
    TAHigh;

const

  SaveMode = True;
  NetMode  = False;

  constructor TQtcSFile.Init(QtcFName : IsamFileBlockName);
  var
    IID     : IsamIndDescr;

    procedure InitIID;
    begin
      IID[GroupIndex].KeyL := 3; { sizeof(group), is PK }
      IID[GroupIndex].AllowDupK := False;
      IID[CallsignIndex].KeyL := 12;
      IID[CallsignIndex].AllowDupK := True;
    end;

  begin
    CurRef := 0;
    BaseFName := ForceExtension(JustName(QtcFName),'');
    if not inherited Init(QtcFName, False, False, SaveMode, NetMode) then begin
      if IsamError = 9903 then begin {data file not found}
        InitIID;
        Create(QtcFName, SizeOf(QtcRecordType), NumberOfIndexes, IID,
               False, False, SaveMode, NetMode);
      end
      else if IsamError = 10010 then begin { fileblock corrupt }
        InitIID;
        Recover(QtcFName, SizeOf(QtcRecordType), NumberOfIndexes, IID, True,
                False, False, SaveMode, NetMode);
      end
      else
        MessageBox('SEVERE ERROR',
                 'Unable to create QTC send file.',
                 'Please re-install, or contact WF1B',
                 'QTC sending will not be available.');
    end else begin
      AppendDebugMessage('Qtc Send File, '+QtcFName+' initialized.');
    {$IFDEF DEBUG}
      AppendDebugMessage('TotalRecs  ='+long2str(TotalRecs));
      AppendDebugMessage('FreeRecs   ='+long2str(FreeRecs));
      AppendDebugMessage('UsedKeys(1)='+long2str(UsedKeys(1)));
      AppendDebugMessage('UsedKeys(2)='+long2str(UsedKeys(2)));
      AppendDebugMessage('UsedRecs   ='+long2str(UsedRecs));
    {$ENDIF}
    end;
  end;

  procedure TQtcSFile.BlankRec(var C : QtcRecordType);
  var
    B : byte;
  begin
    with C do begin
      Del            := 0;
      QtcGroup       := '';
      Number         := '';
      GroupTime      := 0;
      GroupDate      := 0;
      SentTo         := '';
      Band           := 0;
      for b := 1 to 10 do begin
        QtcsExchanged[b].QtcTime   := 0;
        QtcsExchanged[b].QtcCall   := '';
        QtcsExchanged[b].QtcSerNum := 0;
        QtcsExchanged[b].TARecNum  := 0;
      end;
    end;
  end;

  function TQtcSFile.BuildKey(var Rec; KeyNr : word) : IsamKeyStr;
  begin
    with QtcRecordType(Rec) do
      case KeyNr of
        GroupIndex    : BuildKey := QtcGroup;
        CallSignIndex : BuildKey := SentTo;
      end;
  end;

  function TQtcSFile.EqualRec(var Rec1, Rec2) : Boolean;
  var
    Q1 : QtcRecordType absolute Rec1;
    Q2 : QtcRecordType absolute Rec2;
    X  : Byte;
  begin
    EqualRec := False;
    if Q1.Del <> Q2.Del then Exit;
    if Q1.QtcGroup <> Q2.QtcGroup then Exit;
    if Q1.Number <> Q2.Number then Exit;
    if Q1.GroupTime <> Q2.GroupTime then Exit;
    if Q1.GroupDate <> Q2.GroupDate then Exit;
    if Q1.SentTo <> Q2.SentTo then Exit;
    if Q1.Band <> Q2.Band then Exit;
    for X := 1 to 10 do begin
      if Q1.QtcsExchanged[X].QtcTime <> Q2.QtcsExchanged[X].QtcTime then Exit;
      if Q1.QtcsExchanged[X].QtcCall <> Q2.QtcsExchanged[X].QtcCall then Exit;
      if Q1.QtcsExchanged[X].QtcSerNum <> Q2.QtcsExchanged[X].QtcSerNum then Exit;
      if Q1.QtcsExchanged[X].TARecNum <> Q2.QtcsExchanged[X].TARecNum then Exit;
    end;
    EqualRec := True;
  end;

  procedure TQtcSFile.ReportError;
  begin
    AppendDebugMessage('TQtcSFile.ReportError.IsamError: '+long2str(IsamError));
  end;

  procedure TQtcSFile.HardError;
  begin
    ReportError;
    BTExitIsam;
    Halt;
  end;

  procedure TQtcSFile.CheckOK;
  begin
    if not IsamOK then begin
      ReportError;
      Halt;
    end;
  end;

  procedure TQtcSFile.QueryStatus(const ACall : string);
  var
    a : log_rec;
    num : integer;
    ThisCall : string[12]; {need to do this... why?? dunno}
  begin
    Num := 0;
    ThisCall := ACall;
    LogFile^.ResetLog;
    LogFile^.Get_Prev(a);
    while ((OK) and (Num < 10)) do begin
      case ThisCall = '' of
        False : if (not a.Dupe) and
                   (a.His_Qth = '') and
                   (a.Call <> ThisCall) then inc(Num);
        True  : if (not a.Dupe) and
                   (a.His_Qth = '') then inc(Num);
      end;
      LogFile^.Get_Prev(a);
      if a.His_Qth <> '' then OK := False;
    end;
    RttyWin^.PutString(^M+'QTC''s available: ');
    if Num >= 10 then
      RttyWin^.PutString(^Y+'10'+^Z+' or more')
    else
      RttyWin^.PutString(^Y+long2str(Num)+^Z);
    RttyWin^.PutString('  '); {just a spacer}
    if EditW^.EditRecs[1].Call <> '' then
      RttyWin^.PutString(^M+ThisCall+' can take: '+
                ^Y+long2str(HowManyMoreFor(ThisCall))+^Z+^M);
  end;

  procedure TQtcSFile.SaveNewQtc;
  begin
    AddRecord(CurRef, CurRec);
  end;

  procedure TQtcSFile.GotoTop;
  begin
    GetByNumber('001');
  end;

  function TQtcSFile.GetByNumber(const GroupNum : string) : boolean;
  begin
    FindKey(GroupIndex, CurRef, GroupNum);
    if not IsamOK then
      GetByNumber := False
    else begin
      GetRec(CurRef, CurRec);
      GetByNumber := True;
    end;
  end;

  function TQtcSFile.GetByCallsign(const Callsign : string) : boolean;
  begin
    FindKey(CallsignIndex, CurRef, Callsign);
    if not IsamOK then
      GetByCallsign := False
    else begin
      GetRec(CurRef, CurRec);
      GetByCallsign := True;
    end;
  end;

  function TQtcSFile.HowManyGroups : LongInt;
  begin
    HowManyGroups := UsedRecs;
  end;

  function TQtcSFile.NextGroupNumber : string;
  begin
    NextGroupNumber := LeftPadCh(long2str(HowmanyGroups + 1), '0', 3);
  end;

  function TQtcSFile.HowManySentTo(const Call : string) : integer;
  var
    ThisRef : LongInt;
    ThisRec : QtcRecordType;
    FirstRef: LongInt;
    KeyStr  : String[12];
    NumSent : integer;
    NumThisTime : integer;
  begin
    KeyStr := Call;
    FindKey(CallsignIndex, ThisRef, KeyStr);
    if not IsamOK then
      HowManySentTo := 0 {none sent to him so far...}
    else begin
      NumSent := 0;
      GetRec(ThisRef, ThisRec);
      FirstRef := ThisRef;
      while (IsamOK) and (ThisRec.SentTo = Call) do begin
        Str2Int(ThisRec.Number, NumThisTime);
        inc(NumSent,NumThisTime);
        NextKey(CallsignIndex,ThisRef,KeyStr);   { use whatever function gives me the next record for this key!}
        GetRec(ThisRef, ThisRec);
        if ThisRef = FirstRef then IsamOK := False; { same record we started with}
      end;
      HowManySentTo := NumSent;
    end;
  end;

  function TQtcSFile.HowManySentToAtTime(const Call : String; const AtTime : Time; const OnDate : Date) : Integer;
  var
    ThisRef : LongInt;
    ThisRec : QtcRecordType;
    FirstRef : LongInt;
    KeyStr : String[12];
    NumSent : Integer;
    Hours, Mins, Secs : Byte;
    Found : Boolean;
  begin
    KeyStr := Call;
    FindKey(CallSignIndex, ThisRef, KeyStr);
    if not IsamOK then
      HowManySentToAtTime := 0        {none sent to him so far...}
    else begin
      NumSent := 0;
      Found := False;
      GetRec(ThisRef, ThisRec);
      FirstRef := ThisRef;
      while (not Found) and (IsamOK) and (ThisRec.SentTo = Call) do begin
        TimeDiff(AtTime,ThisRec.GroupTime,Hours,Mins,Secs);
        if (ThisRec.GroupDate = OnDate) and
           (Hours = 0) and (Mins < RttyIni^.GetProfileByte('Range','Send QTC', 5)) then begin
          {within 10 minutes, must be the one I want to get}
          Str2Int(ThisRec.Number, NumSent);
          Found := True;
        end;
        NextKey(CallSignIndex, ThisRef, KeyStr);
        GetRec(ThisRef, ThisRec);
        if ThisRef = FirstRef then IsamOK := False; { same record we started with}
      end;
      HowManySentToAtTime := NumSent;
    end;
  end;

  procedure TQtcSFile.CountByBand(var B0,B1,B2,B3,B4 : longint);
  var
    ThisRef : LongInt;
    ThisRec : QtcRecordType;
    T       : array[0..4] of longint;
    Num     : integer;
    KeyStr  : string[3];
  begin
    T[0] := 0;
    T[1] := 0;
    T[2] := 0;
    T[3] := 0;
    T[4] := 0;
    KeyStr := '001';
    FindKey(GroupIndex, ThisRef, KeyStr);
    while IsamOK do begin
      GetRec(ThisRef, ThisRec);
      Str2Int(ThisRec.Number, Num);
      inc(T[ThisRec.Band],Num);
      NextDiffKey(GroupIndex,ThisRef,KeyStr);
    end;
    B0 := T[0];
    B1 := T[1];
    B2 := T[2];
    B3 := T[3];
    B4 := T[4];
    {$IFDEF DEBUG}
    AppendDebugMessage('QtcSObj.CountByBand:'+
              long2str(b0)+','+
              long2str(b1)+','+
              long2str(b2)+','+
              long2str(b3)+','+
              long2str(b4)+' ');
    {$ENDIF}
  end;

  function TQtcSFile.HowManyMoreFor(const Call : string) : integer;
  begin
    HowManyMoreFor := 10 -
                      HowManySentTo(Call) -
                      QtcRFile^.HowManyRecdFrom(Call);
  end;

  procedure TQtcSFile.PrintReport;
  var
    F       : Text;
    ThisRef : LongInt;
    ThisRec : QtcRecordType;
    Num : Integer;
    KeyStr : String[3];
    QtcNum : integer;
  begin
    Assign(F, ForceExtension(BaseFName,'RPT'));
    {$I-} Rewrite(F); {$I+}
    Writeln(F,'QTC Report - Transmitted QTC data');
    writeln(f);
    KeyStr := '001';
    FindKey(GroupIndex, ThisRef, KeyStr);
    while IsamOK do begin
      GetRec(ThisRef, ThisRec);
      with ThisRec do begin
        Writeln(F,'QTC ',QtcGroup, '/', Number,  ' @ ',
                  TimeToTimeString('hh:mmZ on ',GroupTime),
                  Copy('8040201510',Band*2+1,2), 'Mtrs to ',
                  SentTo);
        writeln(F);
        for QtcNum := 1 to 10 do
          if Trim(QtcsExchanged[QtcNum].QtcCall) <> '' then
            writeln(F,'      '+TimeToTimeString('hhmm/',QtcsExchanged[QtcNum].QtcTime)+
                    Trim(QtcsExchanged[QtcNum].QtcCall)+'/'+
                    LeftPadCh(Long2str(QtcsExchanged[QtcNum].QtcSernum),'0',3));
      end;
      Writeln(F);
      Writeln(F);
      NextDiffKey(GroupIndex, ThisRef, KeyStr);
    end;
    Close(F);
  end;

  procedure TQtcSFile.PrintSummaryReport;
  var
    F       : Text;
    ThisRef : LongInt;
    ThisRec : QtcRecordType;
    Num : Integer;
    KeyStr : String[3];
    QtcNum : integer;
    TotQtc : Integer;
  begin
    Assign(F, ForceExtension(BaseFName,'SUM'));
    {$I-} Rewrite(F); {$I+}
    Writeln(F,'QTC''s Sent Summary');
    Writeln(F,'------------------');
    writeln(f);
    writeln(f,'Time      Freq      Station      Grp/Cnt');
    writeln(f,'----      ----      -------      --- ---');
    TotQtc := 0;
    KeyStr := '001';
    FindKey(GroupIndex, ThisRef, KeyStr);
    while IsamOK do begin
      GetRec(ThisRef, ThisRec);
      with ThisRec do begin
        Writeln(F,TimeToTimeString('hh:mmZ    ',GroupTime),
                  Copy('8040201510',Band*2+1,2), 'Mtrs    ',
                  pad(SentTo,13),
                  leftpadch(QtcGroup,'0',3),'/',Number);
        Str2Int(Number,Num);
        Inc(TotQtc,Num);
      end;
      NextDiffKey(GroupIndex, ThisRef, KeyStr);
    end;
    Writeln(F);
    Writeln(F,'Total QTC''s Sent = ',TotQtc);
    Close(F);
  end;

  function TQtcSFile.HowManyInGroup : integer;
  var
   i : integer;
   Num : integer;
  begin
    num := 0;
    for I := 1 to 10 do
      if CurRec.QtcsExchanged[I].QtcCall <> '' then inc(Num);
    HowManyInGroup := Num;
  end;

  procedure TQtcSFile.RetrieveQtcData(const HowMany : integer);
  var
    a : log_rec;
    num : integer;

    function InList(const aCall : string) : Boolean;
    var
      I : integer;
    begin
      InList := False;
      for I := 1 to 10 do
        if CurRec.QtcsExchanged[I].QtcCall = aCall then InList := True;
    end;

  begin
    Num := 0;
    LogFile^.ResetLog;
    LogFile^.Get_Next(a);
    while ((OK) and (Num < HowMany)) do begin
      if (not a.Dupe) and
         (a.His_Qth = '') and
         (a.Call <> CurRec.SentTo) and
         (not InList(a.Call)) then
        with CurRec do begin
          Inc(Num);
          QtcsExchanged[Num].QtcCall := Trim(a.Call);
          QtcsExchanged[Num].QtcTime := a.Ts;
          QtcsExchanged[Num].QtcSerNum := a.His_Serno;
          QtcsExchanged[Num].TARecNum := TARecNum;
        end;
      LogFile^.Get_Next(a);
    end;
    CurRec.Number := leftpadch(long2str(Num),'0',2);
  end;

  procedure TQtcSFile.FlagQtcData;
  var
    a : log_rec;
    NumActuallySent : integer;
    NumUpdated      : integer;
    LookingForNum   : integer;
    i : integer;
  begin
    NumActuallySent := 0;
    NumUpdated := 0;
    for i := 1 to 10 do
      if Trim(CurRec.QtcsExchanged[i].QtcCall) <> '' then inc(NumActuallySent);
    if NumActuallySent = 0 then Exit;
    LogFile^.ResetLog;
    LogFile^.Get_Next(a);
    LookingForNum := 1;
    while ((OK) and (NumUpdated < NumActuallySent)) do begin
      if TARecNum = CurRec.QtcsExchanged[LookingForNum].TARecNum then begin
        a.His_Qth := CurRec.QtcGroup;
        LogFile^.ResaveScrap(a);
        Inc(NumUpdated);
        Inc(LookingForNum);
      end;
      LogFile^.Get_Next(a);
    end;
    if NumActuallySent <> NumUpdated then
      AppendDebugMessage('QtcSObj.FlagQtcData: Sent <> Updated');
    CurRec.Number := leftpadch(long2str(NumActuallySent),'0',2);
  end;

  procedure TQtcSFile.SendAllQtcs;
  var
    QtcNum : Byte;
    Buffer : string[100];
  begin
    Buffer := RttyIni^.GetProfileString('Alt-A','Send QTC','<1> DE <11> QSL? BK' );
    MakeSubstitutions(Buffer,100, EditW^.EditRecs[1]);
    if not RttyTerm1^.KeyBoardLive then RttyTerm1^.Transmit(True);
    RttyTerm1^.SendString(^M);
    RttyTerm1^.SendString('QTC '+CurRec.QtcGroup+'/'+CurRec.Number+^M);
    for QtcNum := 1 to 10 do
      if CurRec.QtcsExchanged[QtcNum].QtcCall <> '' then RttyTerm1^.SendString(
        TimeToTimeString('hhmm-',CurRec.QtcsExchanged[QtcNum].QtcTime)+
        Trim(CurRec.QtcsExchanged[QtcNum].QtcCall)+'-'+
        LeftPadCh(Long2str(CurRec.QtcsExchanged[QtcNum].QtcSernum),'0',3)
        +'  '+^M
        );
    RttyTerm1^.SendString(Buffer);
    if not RttyTerm1^.KeyBoardLive then RttyTerm1^.Transmit(False);
  end;

  procedure TQtcSFile.SendQtc(const QtcNum : byte);
  begin
    if not RttyTerm1^.KeyBoardLive then RttyTerm1^.Transmit(True);
      RttyTerm1^.SendString(
        TimeToTimeString('hhmm-',CurRec.QtcsExchanged[QtcNum].QtcTime)+
        Trim(CurRec.QtcsExchanged[QtcNum].QtcCall)+'-'+
        LeftPadCh(Long2str(CurRec.QtcsExchanged[QtcNum].QtcSernum),'0',3)
        +'  BK'+^M
        );
    if not RttyTerm1^.KeyBoardLive then RttyTerm1^.Transmit(False);
  end;

  procedure TQtcSFile.AreYouQrv;
  var
    Buffer : string[100];
  begin
    Buffer := RttyIni^.GetProfileString('Alt-Q','Send QTC', '<CR><1> QRV? DE <11> KN');
    MakeSubstitutions(Buffer,100,EditW^.EditRecs[1]);
    if not RttyTerm1^.KeyBoardLive then RttyTerm1^.Transmit(True);
    RttyTerm1^.SendString(Buffer);
    if not RttyTerm1^.KeyBoardLive then RttyTerm1^.Transmit(False);
  end;

  procedure UpdateQtcCount(var ESR : ESrecord); far;
  var
    Num : integer;
    I   : integer;
    S   : string[2];
  begin
    Num := QtcSFile^.HowManyInGroup;
    S := leftpadch(long2str(Num),'0',2);
    QtcSFile^.CurRec.Number := S;
    DrawField(Esr, 1);
  end;

  function TQtcSFile.EditQtcData : boolean;
  var
    Esr : EsRecord;
    ExitCommand : EStype;
    Num : Byte;
    HoldPutMsgs : Boolean;

    procedure HandleLeftClick;
    var
      Row, Col : Byte;
    begin
      Col := MouseWhereX;
      Row := MouseWhereY;
      if Row >= ScreenHeight-6 then begin { we are in bottom of screen area}
        if Col in [57..77] then begin
          if Row = ScreenHeight-6+1 then ExitCommand := ESmy92; {alt-q}
          if Row = ScreenHeight-6+2 then ExitCommand := ESmy80; {alt-a}
          if Row = ScreenHeight-6+4 then ExitCommand := ESdone; {^enter}
          if Row = ScreenHeight-6+5 then ExitCommand := ESmy91; {alt-c}
        end;
      end;
    end;


  begin
    EditQtcData := False;
    with Colors^ do begin
    if not MakeWindow(QtcWin, 1, ScreenHeight-6, ScreenWidth, ScreenHeight, True, True, True, WindowAttr,
      FrameAttr, HeaderAttr, 'Send QTC Data') then begin
      Msgwin^.Display('Not enough memory to make window');
      Exit;
    end;
    if not DisplayWindow(QtcWin) then begin
      KillWindow(QtcWin);
      Msgwin^.Display('Not enough memory to display window');
      Exit;
    end;
    InitEsRecord(Esr);
    SetWrapMode(Esr, WrapAtEdges);
    SetProtection(True);
    AddStringField(Esr, 'QTC ',         ScreenHeight-6, 4,  '', ScreenHeight-6, 8,   3, 0, nil, CurRec.QtcGroup);
    AddStringField(Esr, '/',            ScreenHeight-6, 11, '', ScreenHeight-6, 12,  2, 0, nil, CurRec.Number);
    AddStringField(Esr, ' ', ScreenHeight-6, 14, '!!!!!!!!!!!!', ScreenHeight-6, 15, 12, 0, nil, CurRec.SentTo);
    SetProtection(False);
    FastWrite('', ScreenHeight-6, 3,  FrameAttr);
    FastWrite('', ScreenHeight-6, 27, FrameAttr);
    FastWrite(' ', ScreenHeight-6, 14, FrameAttr);
    for Num := 1 to 5 do begin
      SetProtection(True);
      AddTimeField(Esr, long2str(Num), ScreenHeight-6+Num, 3, 'hhmm', ScreenHeight-6+Num, 5, 0, 0, 0,
        CurRec.QtcsExchanged[Num].QtcTime);
      SetProtection(False);
      AddStringField(Esr, '/',ScreenHeight-6+Num, 9, '', ScreenHeight-6+Num, 10,12, 0, nil,
        CurRec.QtcsExchanged[Num].QtcCall);
      SetProtection(True);
      AddWordField(Esr,  '/', ScreenHeight-6+Num,22, '9999', ScreenHeight-6+Num, 23, 0, 0, 9999,
        CurRec.QtcsExchanged[Num].QtcSernum);
      SetProtection(False);
      FastWrite('', ScreenHeight-6+Num, 28, FrameAttr);
    end;
    FastWrite('', ScreenHeight-6, 28, FrameAttr);
    FastWrite('', ScreenHeight-6+6, 28, FrameAttr);
    for Num := 1 to 5 do begin
      SetProtection(True);
      AddTimeField(Esr, LeftPad(long2str(Num+5),2), ScreenHeight-6+Num, 29, 'hhmm', ScreenHeight-6+Num, 32, 0, 0, 0,
        CurRec.QtcsExchanged[Num+5].QtcTime);
      SetProtection(False);
      AddStringField(Esr, '/',ScreenHeight-6+Num, 36, '', ScreenHeight-6+Num, 37,12, 0, nil,
        CurRec.QtcsExchanged[Num+5].QtcCall);
      SetProtection(True);
      AddWordField(Esr,  '/', ScreenHeight-6+Num,49, '9999', ScreenHeight-6+Num, 50, 0, 0, 9999,
        CurRec.QtcsExchanged[Num+5].QtcSernum);
      SetProtection(False);
      FastWrite('', ScreenHeight-6+Num, 55, FrameAttr);
    end;
    FastWrite('', ScreenHeight-6, 55, FrameAttr);
    FastWrite('', ScreenHeight-6+6, 55, FrameAttr);
    FastWrite('Alt-Q   QRV?',            ScreenHeight-6+1, 57, WindowAttr);
    FastWrite('Alt-A   Send All Qtc''s', ScreenHeight-6+2, 57, WindowAttr);
    FastWrite('F#      Send Qtc #',      ScreenHeight-6+3, 57, WindowAttr);
    FastWrite('^Enter  Done',            ScreenHeight-6+4, 57, WindowAttr);
    FastWrite('Alt-C   Cancel',          ScreenHeight-6+5, 57, WindowAttr);
    end;
    AddCmd(ESmy80, kbAltA) ;
    AddCmd(Esmy81, kbF1) ;
    AddCmd(Esmy82, kbF2) ;
    AddCmd(Esmy83, kbF3) ;
    AddCmd(Esmy84, kbF4) ;
    AddCmd(Esmy85, kbF5) ;
    AddCmd(Esmy86, kbF6) ;
    AddCmd(Esmy87, kbF7) ;
    AddCmd(Esmy88, kbF8) ;
    AddCmd(Esmy89, kbF9) ;
    AddCmd(Esmy90, kbF10) ;
    AddCmd(ESmy91, kbAltC) ;
    AddCmd(ESmy92, kbAltQ) ;
    SetPostEditPtr(Esr, @UpdateQtcCount);
    gQtcExchange := True; {signal to Rtty.MyGetKey}
    gInMainLoop  := False;
    HoldPutMsgs := RttyIni^.GetProfileBool('PutMsgs', 'PROFILE', False);
    RttyIni^.SetOrAddProfileString('PutMsgs', 'PROFILE', 'False');
    repeat
      ExitCommand := EditScreen(Esr, Esr.CurrentId, False);
      if ExitCommand = ESmy20 then HandleLeftClick;
      case ExitCommand of
        EsDone : {allow to fall through};
        EsMy80 : SendAllQtcs;
        EsMy81 : SendQtc(1);
        EsMy82 : SendQtc(2);
        EsMy83 : SendQtc(3);
        EsMy84 : SendQtc(4);
        EsMy85 : SendQtc(5);
        EsMy86 : SendQtc(6);
        EsMy87 : SendQtc(7);
        EsMy88 : SendQtc(8);
        EsMy89 : SendQtc(9);
        EsMy90 : SendQtc(10);
        EsMy91 : { alt-c -- cancel };
        EsMy92 : AreYouQrv;
        ESmy7  : RttyTerm1^.ToggleChatMode;
        ESmy11 : AddNote;
        Esmy49 : RttyTerm1^.ResetBox; {esc - emergency stop}
        EsMy60..EsMy69 : RttyWin^.SendKeys(ExitCommand,EditW^.EditRecs[1]);
        ESmy10 : RttyTerm1^.Letters;
        ESmy31 : RttyTerm1^.Figures;
        Esmy53 : RttyTerm1^.ToggleNormRev;
      end;
    until ExitCommand in [EsMy91, EsDone];
    Case HoldPutMsgs of
      TRUE: RttyIni^.SetOrAddProfileString('PutMsgs', 'PROFILE', 'True');
      FALSE:RttyIni^.SetOrAddProfileString('PutMsgs', 'PROFILE', 'False');
    end;
    gQtcExchange := False; {signal to Rtty.MyGetKey}
    gInMainLoop  := True;
    if ExitCommand = EsDone then EditQtcData := True; {save our qtc data}
    AddMyCommands; {reset all overridden commands back to default values}
    KillWindow(QtcWin);
    DisposeEditScreen(Esr);
  end;

  procedure TQtcSFile.SendQtcData(const Call : string);
  var
    I : integer;
    NumToSend : integer;
    NumRecdFrom : integer;
    SaveQtcData : Boolean;
  begin
    if Call = '' then begin
      RttyWin^.PutString(^M^G+'Need to enter a callsign in order to send QTC data'^M);
      Exit;
    end;
    if MyContinent = CountryList^.CT.Continent then begin
      RttyWin^.PutString(^M^G+'Cannot send QTC''s to the same continent ');
      Exit;
    end;
    BlankRec(CurRec);
    with CurRec do begin
      SentTo := Call;
      QtcGroup := NextGroupNumber;
      Band := ord(EditW^.EditRecs[1].Band);
    end;
    NumToSend := HowManyMoreFor(Call);
    if NumToSend <= 0 then begin
      RttyWin^.PutString(^M^G+'Cannot give '+Call+' anymore QTC''s');
      Exit;
    end;
    {$IFDEF DEBUG}
    if NumToSend < 0 then begin
      AppendDebugMessage('QtcSObj.SendQtcData: Already exchanged too many with '+Call);
      Exit;
    end;
    {$ENDIF}
    RetrieveQtcData(NumToSend);
    if CurRec.Number = '00' then begin
      RttyWin^.PutString(^M^G+'No QTC''s available for sending!'^M);
      Exit;
    end;
    SaveQtcData := EditQtcData;
    if SaveQtcData then begin
      Clock^.DateAndTimeUtc(CurRec.GroupDate, CurRec.GroupTime);
      FlagQtcData; {out in the log file}
      SaveNewQtc;
      MsgWin^.Display('QTC sent data saved to disk.');
    end;
  end;

  destructor TQtcSFile.Done;
  begin
    inherited Done;
    {$IFDEF DEBUG}
    AppendDebugMessage('QtcSObj destroyed');
    {$ENDIF}
  end;

  procedure IncChoice(var Value; FieldID : Byte;
                      Factor         : Integer;
                      var St         : String); far;
  var
    Band           : Byte absolute Value;
  begin
    case FieldID of
      003 : begin
              if Band > 4 then begin
                Band := 0;
                Alarm;
              end;
              case Factor of
                1 : if Band = 4 then Band := 0
                    else Inc(Band);
                -1 : if Band = 0 then Band := 4
                     else Dec(Band);
              end;
              St := Copy('8040201510',Band*2+1,2);
            end;
    end;
  end;

  procedure TQtcSFile.EditQtcLog;
  var
    Esr : EsRecord;
    ExitCommand : EStype;
    Num : Byte;
    KeyStr : string[12];
    HowManyRecs : longint;
    OldRec : QtcRecordType;

    procedure NotFoundMessage;
    var
      WinPtr         : WindowPtr;
    begin
      with Colors^ do begin
        MakeWindow(WinPtr, 5, 15, 25, 17, True, True, False, WindowAttr,
                  FrameAttr, HeaderAttr, 'Search Results') ;
        DisplayWindow(WinPtr) ;
        FastwriteWindow('Callsign not found!',1, 1, WindowAttr);
      end;
      Delay(2000);
      KillWindow(WinPtr);
    end;

    procedure SearchForCallsign;

      procedure NotFoundMessage;
      var
        WinPtr         : WindowPtr;
      begin
        with Colors^ do begin
          MakeWindow(WinPtr, 5, 15, 25, 17, True, True, False, WindowAttr,
                    FrameAttr, HeaderAttr, 'Search Results') ;
          DisplayWindow(WinPtr) ;
          FastwriteWindow('Callsign not found!',1, 1, WindowAttr);
        end;
        Delay(2000);
        KillWindow(WinPtr);
      end;

      function GetSearchStr : string;
      var
        WinPtr         : WindowPtr;
        Escaped        : Boolean;
        SearchStr      : string[12];
      begin
        SearchStr := '';
        with Colors^ do begin
          MakeWindow(WinPtr, 5, 15, 20, 17, True, True, False, WindowAttr,
                    FrameAttr, HeaderAttr, 'Search') ;
          DisplayWindow(WinPtr) ;
          ReadString('', 1, 1, 12, PromptAttr, StringAttr, CtrlAttr, Escaped, SearchStr);
        end;
        SearchStr := StUpCase(Trim(SearchStr));
        GetSearchStr := SearchStr;
        KillWindow(WinPtr);
      end;

    begin
      KeyStr := GetSearchStr;
      if KeyStr <> '' then begin
        GotoTop;
        FindKey(CallSignIndex, CurRef, KeyStr);
        if IsamOK then begin
          GetRec(CurRef, CurRec);
          OldRec := CurRec;
        end
          else
          NotFoundMessage;
      end;
    end;

    Procedure FindNextMatch;
    begin
      NextKey(CallSignIndex, CurRef, KeyStr);
      GetRec(CurRef, CurRec);
      OldRec := CurRec;
    end;

    procedure ScrollBack;
    var
      RecKey : string[3];
    begin
      if CurRec.QtcGroup > '001' then begin
        ModifyRecord(CurRef, OldRec, CurRec);
        RecKey := CurRec.QtcGroup;
        PrevDiffKey(GroupIndex, CurRef, RecKey);
        GetRec(CurRef, CurRec);
        OldRec := CurRec;
      end;
    end;

    procedure ScrollForward;
    var
      RecKey : string[3];
      RecNum : longint;
      GoodNum : Boolean;
    begin
      GoodNum := Str2long(CurRec.QtcGroup, RecNum);
      if GoodNum and (RecNum < HowManyRecs) then begin
        ModifyRecord(CurRef, OldRec, CurRec);
        RecKey := CurRec.QtcGroup;
        NextDiffKey(GroupIndex, CurRef, RecKey);
        GetRec(CurRef, CurRec);
        OldRec := CurRec;
      end;
    end;

    procedure HandleLeftClick;
    var
      Row, Col : Byte;
      FieldId : Word;
      EntireRow : String[80];
      S : String[12];
      StartPos : Integer;
      W : word;
    begin
      Col := MouseWhereX;
      Row := MouseWhereY;
      if Row >= ScreenHeight-6 then begin { we are in bottom of screen area}
        if Col in [57..77] then begin
          if Row = ScreenHeight-6+1 then ExitCommand := ESmy81;
          if Row = ScreenHeight-6+2 then ExitCommand := ESmy82;
          if Row = ScreenHeight-6+3 then ExitCommand := ESmy87;
          if Row = ScreenHeight-6+4 then ExitCommand := ESmy88;
          if Row = ScreenHeight-6+5 then ExitCommand := ESdone;
        end;
      end;
    end;

  begin
    with Colors^ do begin
    if not MakeWindow(QtcWin, 1, ScreenHeight-6, ScreenWidth, ScreenHeight, True, True, True, WindowAttr,
                      FrameAttr, HeaderAttr, 'Edit QTC Log Data') then begin
      MsgWin^.Display('Not enough memory to make QTC Edit window');
      Exit;
    end;
    if not DisplayWindow(QtcWin) then begin
      KillWindow(QtcWin);
      Msgwin^.Display('Not enough memory to display QTC Edit window');
      Exit;
    end;
    InitEsRecord(Esr);
    SetWrapMode(Esr, WrapAtEdges);
    SetProtection(True);
    AddStringField(Esr, 'QTC ', ScreenHeight-6, 4, '', ScreenHeight-6, 8, 3, 0, nil, CurRec.QtcGroup);
    SetProtection(False);
    AddStringField(Esr, '/',    ScreenHeight-6, 11, '', ScreenHeight-6, 12, 2, 0, nil, CurRec.Number);
    AddStringField(Esr, ' ',    ScreenHeight-6, 14, '', ScreenHeight-6, 15, 12, 0, nil, CurRec.SentTo);
    AddChoiceField(Esr, ' ',    ScreenHeight-6, 28, 'XX',ScreenHeight-6, 28, 0, 1, @IncChoice, CurRec.Band);
    FastWrite('', ScreenHeight-6, 3,  FrameAttr);
    FastWrite('', ScreenHeight-6, 30, FrameAttr);
    FastWrite(' ', ScreenHeight-6, 14, FrameAttr);
    FastWrite(' ', ScreenHeight-6, 27, FrameAttr);
    for Num := 1 to 5 do begin
      SetForceMode(True,True);
      AddTimeField(Esr, long2str(Num), ScreenHeight-6+Num, 3, 'hhmm', ScreenHeight-6+Num, 5, 0, 0, 0,
        CurRec.QtcsExchanged[Num].QtcTime);
      SetForceMode(False,False);
      AddStringField(Esr, '/',ScreenHeight-6+Num, 9, '', ScreenHeight-6+Num, 10,12, 0, nil,
        CurRec.QtcsExchanged[Num].QtcCall);
      SetForceMode(True,True);
      AddWordField(Esr,  '/', ScreenHeight-6+Num,22, '9999', ScreenHeight-6+Num, 23, 0, 0, 9999,
        CurRec.QtcsExchanged[Num].QtcSernum);
      SetForceMode(False,False);
      FastWrite('', ScreenHeight-6+Num, 28, FrameAttr);
    end;
    FastWrite('', ScreenHeight-6, 28, FrameAttr);
    FastWrite('', ScreenHeight-6+6, 28, FrameAttr);
    for Num := 1 to 5 do begin
      SetForceMode(True,True);
      AddTimeField(Esr, LeftPad(long2str(Num+5),2), ScreenHeight-6+Num, 29, 'hhmm', ScreenHeight-6+Num, 32, 0, 0, 0,
        CurRec.QtcsExchanged[Num+5].QtcTime);
      SetForceMode(False,False);
      AddStringField(Esr, '/',ScreenHeight-6+Num, 36, '', ScreenHeight-6+Num, 37,12, 0, nil,
        CurRec.QtcsExchanged[Num+5].QtcCall);
      SetForceMode(True,True);
      AddWordField(Esr,  '/', ScreenHeight-6+Num,49, '9999', ScreenHeight-6+Num, 50, 0, 0, 9999,
        CurRec.QtcsExchanged[Num+5].QtcSernum);
      SetForceMode(False,False);
      FastWrite('', ScreenHeight-6+Num, 55, FrameAttr);
    end;
    FastWrite('', ScreenHeight-6, 55, FrameAttr);
    FastWrite('', ScreenHeight-6+6, 55, FrameAttr);


    FastWrite('F1      Search Call', ScreenHeight-6+1, 57, WindowAttr);
    FastWrite('F2      Search Next', ScreenHeight-6+2, 57, WindowAttr);
    FastWrite('F7      Prev QTC', ScreenHeight-6+3, 57, WindowAttr);
    FastWrite('F8      Next QTC', ScreenHeight-6+4, 57, WindowAttr);
    FastWrite('^Enter  Done', ScreenHeight-6+5, 57, WindowAttr);
    AddCmd(Esmy81, kbF1);
    AddCmd(Esmy82, kbF2);
    AddCmd(Esmy87, kbF7);
    AddCmd(Esmy88, kbF8);
    HowManyRecs := HowManyGroups;
    GotoTop;
    OldRec := CurRec;
    gQtcExchange := False; {signal to Rtty.MyGetKey}
    gInMainLoop := False; {signal to Rtty.MyGetKey}
    repeat
      ExitCommand := EditScreen(Esr, Esr.CurrentId, False);
      if ExitCommand = ESmy20 then HandleLeftClick;
      case ExitCommand of
        ESdone : {allow to fall through} ;
        Esmy81 : SearchForCallsign;
        Esmy82 : FindNextMatch;
        Esmy87 : ScrollBack;
        Esmy88 : ScrollForward;
        EsMy49 : begin
                   CurRec := OldRec;
                   DrawEditScreen(Esr);
                 end;
      end;
    until ExitCommand in [ESdone, EsMy91];
    gInMainLoop := True; {signal to Rtty.MyGetKey}
    ModifyRecord(CurRef, OldRec, CurRec);
    AddMyCommands;                {reset all overridden commands back to default values}
    end;
    KillWindow(QtcWin);
    DisposeEditScreen(Esr);
  end;



end.