{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
unit RecTypes;

interface

uses
  TpDate,
  TpWindow,
  TpCrt,
  TpPick,
  TpRarray,
  TAHigh,
  TpEntry,
  ApPort,
  ApTimer,
  Keys,
  WpxList,
  IniTools,
  Bearings,
  ColorDef;


type
  ContestType    = (ARRL, CQWW, BART, DXPD, SART, JART, WPXC, VOLT,
                    WAED, NAQP, ANAR, EAWW, SPDX, ARII, RRWW, TARA,
                    SPRI, ARFD, HSSC, SCCC, BSPR);
const
  Contest_First = ARRL;
  Contest_Last  = BSPR;
  ContestName : array[ContestType] of String[18] = (
          'ARRL RTTY Roundup ',
          'CQ WW RTTY Contest',
          'BARTG RTTY Contest',
          'DX-Pedition Mode  ',
          'SARTG Worldwide   ',
          'JARTS WW RTTY     ',
          'CQ/RJ WW RTTY WPX ',
          'VOLTA RTTY DX     ',
          'WAEDC RTTY Contest',
          'NA QSO Party      ',
          'ANARTS WW Digital ',
          'EAWW RTTY Contest ',
          'SP-DX RTTY Contest',
          'ARI International ',
          'Russian RTTY WW   ',
          'TARA Sprint       ',
          'Internet Sprint   ',
          'ARRL Field Day    ',
          'High Speed Sprint ',
          'SCC RTTY Champions',
          'BARTG RTTY Sprint ');

type
  VersionType    = (V1_1_6, V2_0_0);

  ModeType       = (RTTYMODE, ASCII, AMTOR, Packet, SSB, CW, Other);
const
  Mode_First     = RTTYMODE;
  Mode_Last      = Other;

type
  ClassType      = (SingleAll, SingleSingle, SingleAss, SWL, MultiSingle, Multi2, MultiMulti);
const
  Class_First    = SingleAll;
  Class_Last     = MultiMulti;


type
  BandType       = (B80, B40, B20, B15, B10, TOT);

const
  Band_First = B80;
  Band_Last  = B10;


const
  SpeedString : array[0..4] of String[3] = (' 45', ' 50', ' 57', ' 74', '100');




type
  JunkType        = (JUNKTYPEITEM);



  Mult_Rec       = record
                     Band           : array[BandType] of Boolean;
                   end;

  Cont_Rec_Type  = record
                     Code           : String[5];
                     Info           : String[5];
                   end;

  CanadaXRef = record
               Name : string[3];
               Prov : byte;
             end;

  CanadaCqXRef = record
               Name : string[3];
               Prov : byte;
             end;

const
  Modes          : array[ModeType] of String[6] = (
             'RTTY', 'ASCII', 'AMTOR', 'PACKET', 'SSB', 'CW', 'OTHER');

  Classes        : array[ClassType] of String[22] = (
           'Single Op, All Band   ',
           'Single Op, Single Band',
           'Single Op, Assisted   ',
           'S.W.L.                ',
           'Multi Op, Single Xmit ',
           'Multi Op, Two Xmit    ',
           'Multi Op, Multi Xmit  ');


  PolarityChoices : array[Boolean] of String[7] = ('Reverse', 'Normal');
  YesNo     : array[Boolean] of String[3] = ('No ','Yes');
  OnOff     : array[Boolean] of String[3] = ('Off','On ');
  TrueFalse : array[Boolean] of String[5] = ('False','True ');
  EndOfLineChoices : array[0..3] of String[10] =
                 ('CR LF LTRS','CR','CR CR LF','CR LF');

  BandS          : array[BandType] of String[3] = (
             '80', '40', '20', '15', '10', 'Tot');

  MHZs           : array[BandType] of String[3] = (
              '3.5', '7', '14', '21', '28', 'Tot');

  bAeaCommand = 0;
  bAmt        = 1;
  bBaudotDrv  = 2;
  bDsp232     = 3;
  bDsp4100    = 4;
  bDxp38      = 5;
  bK6Sti      = 6;
  bKam        = 7;
  bMfj1278    = 8;
  bPtc1       = 9;
  bPtc2       = 10;
  bP38        = 11;
  bPci3000    = 12;
  bPci4000    = 13;
  bPK232      = 14;
  bPK900      = 15;
  bUserDef    = 16;
  bUtu        = 17;
  bNone       = 18;
  Tnc_First   = bAeaCommand;
  Tnc_Last    = bNone;
  TncS           : array[Tnc_First..Tnc_Last] of String[10] = (
              'AEA Cmd',
              'AMT',
              'BAUDOT-DRV',
              'DSP-232',
              'DSP-4100',
              'DXP38',
              'K6STI',
              'KAM',
              'MFJ-1278',
              'PTC-I',
              'PTC-II',
              'P38',
              'PCI-3000',
              'PCI-4000',
              'PK-232',
              'PK-900',
              'USER-DEF',
              'UTU',
              'NONE');

Type
  HandShakeType = (NoHandShake, SoftHandShake, RtsCts, DtrDsr);

Const
  HandShakeString : array[HandShakeType] of string[8] =
  ('None', 'Xon/Xoff', 'Rts/Cts', 'Dtr/Dsr');

const
ExchangePointTable : array[1..40] of array[1..40] of Byte = (
(2,14,10,13,16,18,22,20,25,30,36,37,39,21,22,19,20,17,11,25,29,29,22,22,16,28,25,31,39,35,14,36,25,29,34,39,40,47,44,15),
(14,2,15,8,7,16,16,12,16,23,24,30,30,12,14,16,19,20,19,19,25,31,26,30,28,35,35,40,50,50,25,47,14,21,21,28,33,36,37,6   ),
(10,15,2,8,11,9,13,14,18,21,28,28,30,26,28,27,29,27,21,32,37,39,32,31,24,37,33,40,43,35,11,32,29,35,35,42,48,50,52,20  ),
(13,8,8,2,3,8,10,8,12,18,22,25,27,19,21,23,26,26,22,26,33,37,32,34,30,40,38,44,52,44,20,40,21,28,26,33,40,41,44,14     ),
(16,7,11,3,2,9,9,6,10,17,20,24,25,18,20,22,26,26,24,35,32,38,33,35,31,41,40,45,54,46,22,41,19,27,24,31,38,39,42,13     ),
(18,16,9,8,9,2,4,7,10,12,19,19,21,27,29,31,34,33,29,34,40,46,40,40,33,46,42,49,47,38,17,32,28,36,30,37,44,43,48,22     ),
(22,16,13,10,9,4,2,4,6,8,15,15,17,26,29,31,35,36,33,33,40,47,42,44,38,50,46,53,49,40,22,34,26,34,26,33,40,38,44,22     ),
(20,12,14,8,6,7,4,2,5,11,15,18,19,22,24,27,31,32,30,29,35,42,38,42,37,47,46,51,54,44,24,38,21,30,23,30,38,36,41,18     ),
(25,16,18,12,10,10,6,5,2,8,10,14,15,23,25,29,33,35,34,29,35,43,41,45,41,50,50,55,52,45,28,38,21,30,20,27,35,32,38,21   ),
(30,23,21,18,17,12,8,11,8,2,9,7,9,31,33,37,41,43,41,36,42,51,49,52,45,50,52,54,44,37,28,31,28,36,24,29,38,31,38,29     ),
(36,24,28,22,20,19,15,15,10,9,2,9,7,26,28,33,36,41,43,30,34,42,45,51,52,49,55,49,42,41,37,35,22,29,16,20,28,23,29,27   ),
(37,30,28,25,24,19,15,18,14,7,9,2,3,35,37,41,45,49,48,39,42,49,53,58,50,52,52,48,37,33,32,27,31,37,34,27,33,27,33,34   ),
(39,30,30,27,25,21,17,19,15,9,7,3,2,33,35,40,43,48,49,37,39,46,50,56,53,50,52,46,37,34,35,29,29,34,21,24,30,24,30,34   ),
(21,12,26,19,18,27,26,22,23,31,26,35,33,2,3,6,10,14,18,7,14,21,19,25,27,27,30,32,42,49,34,55,5,10,15,19,21,26,26,6     ),
(22,14,28,21,20,29,29,24,25,33,28,37,35,3,2,5,9,13,18,6,11,18,17,23,27,25,29,30,39,47,36,54,6,7,15,18,19,25,24,8       ),
(19,16,27,23,22,31,31,27,29,37,33,41,40,6,5,2,4,8,13,6,10,15,12,18,22,21,24,26,36,42,33,49,10,9,20,21,21,27,25,9       ),
(20,19,29,26,26,34,35,31,33,41,36,45,43,10,9,4,2,5,12,7,8,12,8,14,19,17,20,22,32,38,32,45,14,10,22,22,20,27,23,12      ),
(17,20,27,26,26,33,36,32,35,43,41,49,48,14,13,8,5,2,7,12,12,12,6,11,14,15,16,20,30,35,29,40,18,15,27,28,24,31,27,14    ),
(11,19,21,22,24,29,33,30,34,41,43,48,49,18,18,13,12,7,2,18,19,16,10,10,9,16,15,20,30,32,21,36,23,21,33,34,30,38,33,16  ),
(25,19,32,26,35,34,33,29,29,36,30,39,37,7,6,6,7,12,18,2,6,14,14,20,26,21,26,25,34,43,39,49,8,3,15,16,15,22,20,12       ),
(29,25,37,33,32,40,40,35,35,42,34,42,39,14,11,10,8,12,19,6,2,9,11,17,24,16,21,20,28,37,40,43,14,6,18,16,11,19,15,19    ),
(29,31,39,37,38,46,47,42,43,51,42,49,46,21,18,15,12,12,16,14,9,2,6,10,18,7,13,11,21,29,36,35,22,14,26,22,15,22,16,24   ),
(22,26,32,32,33,40,42,38,41,49,45,53,50,19,17,12,8,6,10,14,11,6,2,6,13,8,12,14,24,30,31,37,22,16,29,26,21,28,22,20     ),
(22,30,31,34,35,40,44,42,45,52,51,58,56,25,23,18,14,11,10,20,17,10,6,2,8,6,6,10,20,24,26,30,28,22,35,33,25,32,25,25    ),
(16,28,24,30,31,33,38,37,41,45,52,50,53,27,27,22,19,14,9,26,24,18,13,8,2,13,9,15,23,30,18,27,32,28,41,40,33,40,33,25   ),
(28,35,37,40,41,46,50,47,50,50,49,52,50,27,25,21,17,15,16,21,16,7,8,6,13,2,6,5,16,22,31,29,29,21,33,29,21,27,20,29     ),
(25,35,33,38,40,42,46,46,50,52,55,52,52,30,29,24,20,16,15,26,21,13,12,6,9,6,2,7,15,18,25,25,34,27,40,35,27,32,26,30    ),
(31,40,40,44,45,49,53,51,55,54,49,48,46,32,30,26,22,20,20,25,20,11,14,10,15,5,7,2,10,17,31,24,34,25,36,30,22,26,19,34  ),
(39,50,43,52,54,47,49,54,52,44,42,37,37,42,39,36,32,30,30,34,28,21,24,20,23,16,15,10,2,9,15,32,42,33,39,31,24,24,20,44 ),
(35,50,35,44,46,38,40,44,45,37,41,33,34,49,47,42,38,35,32,43,37,29,30,24,30,22,18,17,9,2,24,7,51,42,47,40,33,32,29,48  ),
(14,25,11,20,22,17,22,24,28,28,37,32,35,34,36,33,32,29,21,39,40,36,31,26,18,31,25,31,15,24,2,22,39,42,46,53,52,56,51,28),
(36,47,32,40,41,32,34,38,38,31,35,27,29,55,54,49,45,40,36,49,43,35,37,30,27,29,25,24,32,7,22,2,57,48,47,42,38,34,33,50 ),
(25,14,29,21,19,28,26,21,21,28,22,31,29,5,6,10,14,18,23,8,14,22,22,28,32,29,34,34,42,51,39,57,2,9,10,14,18,22,23,10    ),
(29,21,35,28,27,36,34,30,30,36,29,37,34,10,7,9,10,15,21,3,6,14,16,22,28,21,27,25,33,42,42,48,9,2,13,12,12,18,16,16     ),
(34,21,35,26,24,30,26,23,20,24,16,34,21,15,15,20,22,27,33,15,18,26,29,35,41,33,40,36,39,47,46,47,10,13,2,7,15,15,19,20 ),
(39,28,42,33,31,37,33,30,27,29,20,27,24,19,18,21,22,28,34,16,16,22,26,33,40,29,35,30,31,40,53,42,14,12,7,2,8,8,11,24   ),
(40,33,48,40,38,44,40,38,35,38,28,33,30,21,19,21,20,24,30,15,11,15,21,25,33,21,27,22,24,33,52,38,18,12,15,8,2,7,5,28   ),
(47,36,50,41,39,43,38,36,32,31,23,27,24,26,25,27,27,31,38,22,19,22,28,32,40,27,32,26,24,32,56,34,22,18,15,8,7,2,6,32   ),
(44,37,52,44,42,48,44,41,38,38,29,33,30,26,24,25,23,27,33,20,15,16,22,25,33,20,26,19,20,29,51,33,23,16,19,11,5,6,2,32  ),
(15,6,20,14,13,22,22,18,21,29,27,34,34,6,8,9,12,14,16,12,19,24,20,25,25,29,30,34,44,48,28,50,10,16,20,24,28,32,32,2    )
);


const
  No_ID          : Byte = 99;
  SpecialEnd     : String[5] = '';
  CRLf           : String[2] = Chr(13)+Chr(10);
  MyContinent    : String[2] = 'NA';
  MyCountry      : String[5] = 'W';
  MyDistrict     : Byte = 1;
  RttyMono       : Boolean = False;
  MonoColors     : Boolean = False;
  RTTYbyWF1B     : String[12] = 'RTTY by WF1B';
  Version        : String[12]  = ' v5 ';
  Copyright      : String[43] = 'Copyright 1990-2000 Wyvern Technology, Inc.';
var
  FriendList     : pIni;

  GetOut         : Boolean;
  Ccall,
  MouseCall      : String[12];
  TuneWin,
  EditWin        : WindowPtr;

  MW_Xlo         : Byte;
  MW_xhi         : Byte;
  MW_Ylo         : Byte;
  MW_Yhi         : Byte;

  EW_Xlo         : Byte;
  EW_Xhi         : Byte;
  EW_Ylo         : Byte;
  EW_Yhi         : Byte;

  ActionBarLine  : Byte;
  QSOinLine      : Byte;

  CatchCall      : Boolean;
  gQtcExchange,
  gInMainLoop,
  QSOTimeSet     : Boolean;
  EntryActiveField : Word;
  EntryEsr       : EsRecord;
  EntryExitCommand : EStype;

  PrefixList : PWpxList;

  { must be global, sorry :( }
  EditIncChoicePtr    : pointer;
  ValidateCallsignPtr : pointer;
  ValidateQthPtr      : pointer;

  ClockTimer : EventTimer;
  RadioTimer : EventTimer;

  {copy data from TNC to file}
  DataLine : string[80];
  DataCount : byte;
  RxBufferSize : byte;
  RxScrollIncr : byte;

  RttyFunction : string[8];

  ByPassWorkSheet : Boolean;  {the -now switch}
  FastStart : Boolean;  {the -now switch}

  Bearing : PBearing;

  InputFileName : string[79];
  NewContestStartup : Boolean; { t=brand new file }

  Contest        : ContestType;

  HandlingNetQso : boolean;

  ClickDelims : array[1..9] of char;


implementation

end.