{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit TncObject;

interface

uses
  RecTypes,
  ApPort,
  OoCom;

const
  DisplayS : array[0..5] of String[19] = (
    'No Display',
    'Zero Center',
    'One Bar',
    'Two Bars',
    'Two Touch Bars',
    'Four Bars');

type
  PTnc         = ^TTnc;
  TTnc         = object
                     UP : AbstractPortPtr;
                     Safety : Byte;
                     KeyBoardLive : Boolean;
                     TncTalk      : Boolean;
                     Name         : String[10];
                     TncNum       : Byte;
                     FreqDelta    : ShortInt;
                     constructor Init;
                     Constructor InitCom(const ComPort : ComNameType);
                     procedure WaitSafety;
                     procedure SendByte(const Data : Byte); virtual;
                     procedure SendChar(const Data : Char); virtual;
                     procedure SendString(const Data : String); virtual;
                     procedure Receive(var Ch : Char); virtual;
                     procedure WaitForString(    MaxLen       : Byte;
                                 var S            : String;
                                     TimeOut      : word;
                                 var EntireString : boolean);
                     procedure GetString(MaxLen : Byte;
                              var S : String);
                     procedure WaitForBlock(var S : String;
                                TimeOut : word;
                                var EntireBlock : boolean);
                     procedure WaitTransReady(Ticks : integer);
                     destructor Done; virtual;
                     procedure SendPortWord(const Data : Word);
                     function GetPortWord : word;
                     function  RXToneState : string;
                     procedure Transmit(const Xmit : Boolean); virtual;
                     procedure Letters; virtual;
                     procedure Figures; virtual;
                     procedure SetUsos(const USOS : Boolean); virtual;
                     function  GetUsos : boolean; virtual;
                     procedure SetDiddles(const Diddles : Boolean); virtual;
                     function  GetDiddles : boolean; virtual;
                     procedure SetEOL(const EOL : Byte); virtual;
                     function  GetEOL : byte; virtual;
                     procedure SetSquelchEnabled(const SquelchEnabled : Boolean); virtual;
                     function  GetSquelchEnabled : boolean; virtual;
                     procedure SetSquelchLevel(const SquelchLevel : Byte); virtual;
                     function  GetSquelchLevel : byte; virtual;
                     procedure SetFskPolarity(const FskPolarity : Boolean); virtual;
                     function  GetFskPolarity : boolean; virtual;
                     procedure SetFilterWidth(const FilterWidth : Byte); virtual;
                     function  GetFilterWidth : byte; virtual;
                     procedure SetDisplay(const Display : Byte); virtual;
                     function  GetDisplay : byte; virtual;
                     procedure SetDisplayScale(const DisplayScale : Byte); virtual;
                     function  GetDisplayScale : byte; virtual;
                     procedure SetMode(const Mode : String); virtual;
                     function  GetMode : string; virtual;
                     procedure StopEverything; virtual;
                     procedure SetModem(const Modem : Byte); virtual;
                     function  GetModem : byte; virtual;
                     procedure ClearXmitBuffer; virtual;
                     procedure ToggleNormRev; virtual;
                     procedure SetSense(const Sense : Boolean); virtual;
                     function  GetSense : Boolean; virtual;
                     procedure SetTxSense(const Sense : Boolean); virtual;
                     function  GetTxSense : Boolean; virtual;
                     procedure SetMarkFreq(const MarkFreq : word); virtual;
                     function  GetMarkFreq : word; virtual;
                     procedure SetSpaceFreq(const SpaceFreq : word); virtual;
                     function  GetSpaceFreq : word; virtual;
                     procedure ToggleChatMode;
                     procedure ResetBox;
                     procedure ToggleTncTalk; virtual;
                     procedure SetBaudRate(const Rate : Byte); virtual;
                     procedure SetWpmRate(const Rate : Byte); virtual;
                     function  GetBaudRate : byte; virtual;
                     procedure GetPip(const WhichOne : byte; var Display : string; var DispAttr : byte); virtual;
                     procedure GetZero(var NumForZero : shortint; var DispAttr : byte); virtual;
                     procedure UpdateFilterStrings(var S1, S2 : string; var A1, A2 : Byte); virtual;
                     procedure ClearUP;
                   end;
implementation

  uses
    DebugRtns,
    cAutoCq,
    cMsgWin,
    ApMisc,
    ApTimer,
    ApUArt,
    cRttyWin,
    IniFile,
    TpCrt,
    TpString,
    LowLvl;

  constructor TTnc.Init;
  begin
    KeyBoardLive := False;
    TncTalk := False;
    UP := NIL;
    Safety := 0;
    Name := 'NOT ASSIGN';
    TncNum := bNone;
    FreqDelta := 0;
    {$IFDEF DEBUG}
    AppendDebugMessage('Tnc(Init) created');
    {$ENDIF}
  end;

  constructor TTnc.InitCom(const ComPort : ComNameType);
  var
    iBaudRate  : Longint;
    iParity    : ParityType;
    iDataBits  : Longint;
    iStopBits  : Longint;
    iHandShake : HandShakeType;
    S      : string;
  begin
    UP := New(UartPortPtr, InitCustom(ComPort, 9600, NoParity, 8, 1, 2048, 2048, DefPortOptions));
    if not Assigned(UP) then begin
      {$IFDEF DEBUG}
      AppendDebugMessage('UP not assigned, uh oh.');
      Halt;
      {$ENDIF}
    end;
    iBaudRate := RttyIni^.GetProfileLong('BaudRate', ComNameString(ComPort), 9600);
    iParity   := ParityStringToParity(RttyIni^.GetProfileString('Parity', ComNameString(ComPort), 'NONE'));
    iDataBits := RttyIni^.GetProfileLong('DataBits', ComNameString(ComPort), 8);
    iStopBits := RttyIni^.GetProfileLong('StopBits', ComNameString(ComPort), 1);
    S         := RttyIni^.GetProfileString('HandShake', ComNameString(ComPort), 'None');
    iHandShake:= HandShakeStringToHandShake(S);
    with UP^ do begin
      ChangeBaud(iBaudRate);
      ChangeDataBits(iDataBits);
      ChangeStopBits(iStopBits);
      ChangeParity(iParity);
    end;
    ChangeHandShake(UP,iHandShake);
    Safety := 0;
    KeyBoardLive := False;
    TncTalk := False;
    Name := 'NOT ASSIGN';
    TncNum := bNone;
    if UP^.CheckLineError then begin
      UP^.FlushInBuffer; { flush the input buffer }
      if UP^.CheckFifoError then begin
        SetFifoBuffering(UP^.PR^.BaseAddr,False,0); { flush the fifo buffer }
        SetFifoBuffering(UP^.PR^.BaseAddr,True,14);
      end;
      UP^.GetLineError;  { resets error indicators }
    end;
    {$IFDEF DEBUG}
    AppendDebugMessage('Tnc(InitCom) created');
    {$ENDIF}
  end;

  destructor TTnc.Done;
  begin
    if Assigned(UP) then
      Dispose(UP, Done);
    UP := NIL;
    {$IFDEF DEBUG}
    AppendDebugMessage('Tnc destroyed');
    {$ENDIF}
  end;

  procedure TTnc.WaitSafety;
  {
  Routine will wait for the ComPort to be ready to accept a character
  It will wait up to 1/2 second ( 5ms * 200 tries )
  }
  begin
    if not Assigned(UP) then Exit;
    safety := 0;
    while (NOT UP^.TransReady) and (safety < 200) do begin
      Delay(5); { 5 ms }
      inc(safety);
    end;
  end;

  procedure TTnc.SendByte(const Data : Byte);
  begin
    if not Assigned(UP) then Exit;
    WaitSafety;
    UP^.PutChar(Chr(Data));
  end;

  procedure TTnc.SendChar(const Data : Char);
  begin
    if not Assigned(UP) then Exit;
    WaitSafety;
    UP^.PutChar(Data);
  end;

  procedure TTnc.SendString(const Data : String);
  var
    I : Integer;
  begin
 {   for I := 1 to length(Data) do
      TTnc.SendChar(Data[I]); }
    if not Assigned(UP) then Exit;
    UP^.PutString(Data);
  end;

  procedure TTnc.Receive(var Ch : Char);
  begin
    if not Assigned(UP) then Exit;
    if UP^.CharReady then UP^.GetChar(Ch)
    else Ch := cNUL;
  end;

  procedure TTnc.WaitForString(    MaxLen       : Byte;
                                 var S            : String;
                                     TimeOut      : word;
                                 var EntireString : boolean);
  begin
    if not Assigned(UP) then Exit;
    S := '';
    EntireString := True;
    if Timeout = 0 then Timeout := 18; {wait one second}
    UP^.GetStringTimeout(S, MaxLen, [cCr], TimeOut);
    if AsyncStatus = ecTimeout then EntireString := False;
  end;

  procedure TTnc.GetString(MaxLen : Byte;
                              var S : String);
  begin
    if not Assigned(UP) then Exit;
    S := '';
    UP^.ptOptionsOn(ptReturnDelimiter);
    UP^.GetString(S, MaxLen, [cCr]);
  end;

  procedure TTnc.WaitForBlock(var S : String;
                                TimeOut : word;
                                var EntireBlock : boolean);
  var
   BytesRead : Word;
  begin
    if not Assigned(UP) then Exit;
    S := '';
    EntireBlock := True;
    if Timeout = 0 then Timeout := 9; {wait one second}
    UP^.ptOptionsOff(ptReturnPartialGets);
    UP^.GetBlockTimeout(S[1], 0, BytesRead, [cETB], TimeOut);
    byte(s[0]) := lo(BytesRead);
    EntireBlock := AsyncStatus = ecOK;
  end;

  procedure TTnc.WaitTransReady(Ticks : integer);
  var
    ET : EventTimer;
  begin
    if not Assigned(UP) then Exit;
    if ticks = 0 then ticks := 9; {1/2 second wait by default}
    NewTimer(ET, Ticks);
    repeat
    until (UP^.TransReady) or (TimerExpired(ET));
  end;

  procedure TTnc.SendPortWord(const Data : Word);
  var
    W : Word;
    BytesWritten : word;
    B : Byte;
  begin
    W := Data;
    B := hi(w);
    UP^.PutBlock(b,SizeOf(b), BytesWritten);
    B := lo(w);
    UP^.PutBlock(b,SizeOf(b), BytesWritten);
  end;

  function TTnc.GetPortWord : word;
  var
    W : Word;
    b1, b2 : byte;
    c : char;
  begin
    W := 0;
    if UP^.BlockReady(2,[]) then begin
      UP^.GetChar(C);
      b1 := ord(c);
      UP^.GetChar(C);
      b2 := ord(c);
      w := (b1*256)+b2;
    end;
    GetPortWord := w;
  end;

  function  TTnc.RXToneState : string;
  const
    Str : array[boolean] of string[3]=('REV','NOR');
  begin
    RXToneState := Str[GetSense];
  end;

  procedure TTnc.Transmit(const Xmit : Boolean);
  begin
    { no default action in abstract object }
  end;

  procedure TTnc.Letters;
  begin
    { no default action in abstract object }
  end;

  procedure TTnc.Figures;
  begin
    { no default action in abstract object }
  end;

  procedure TTnc.SetUSOS(const USOS : Boolean);
  begin
    RttyIni^.SetOrAddProfileString('Usos', 'TNC1', YesNo[USOS]);
    { no default action in abstract object }
  end;

  function TTnc.GETUSOS : boolean;
  begin
    GetUsos := RttyIni^.GetProfileBool('Usos', 'TNC1', True);
  end;

  procedure TTnc.SetDiddles(const Diddles : Boolean);
  begin
    RttyIni^.SetOrAddProfileString('Diddles', 'TNC1', YesNo[Diddles]);
  end;

  function TTnc.GetDiddles : boolean;
  begin
    GetDiddles := RttyIni^.GetProfileBool('Diddles', 'TNC1', True);
  end;

  procedure TTnc.SetEol(const Eol : Byte);
  begin
    RttyIni^.SetOrAddProfileString('EndOfLine', 'TNC1', long2str(eol));
  end;

  function TTnc.GetEOL : byte;
  begin
    GetEOL := RttyIni^.GetProfileByte('EndOfLine', 'TNC1', 3); {CR/LF}
  end;

  procedure TTnc.SetSquelchEnabled(const SquelchEnabled : Boolean);
  begin
    RttyIni^.SetOrAddProfileString('SquelchEnabled', 'TNC1', YesNo[SquelchEnabled]);
  end;

  function TTnc.GetSquelchEnabled : boolean;
  begin
    GetSquelchEnabled := RttyIni^.GetProfileBool('SquelchEnabled', 'TNC1', True);
  end;

  procedure TTnc.SetSquelchLevel(const SquelchLevel : Byte);
  begin
    RttyIni^.SetOrAddProfileString('SquelchLevel', 'TNC1', long2str(SquelchLevel));
  end;

  function TTnc.GetSquelchLevel : byte;
  begin
    GetSquelchLevel := RttyIni^.GetProfileByte('SquelchLevel', 'TNC1', 68);
  end;

  procedure TTnc.SetFskPolarity(const FskPolarity : Boolean);
  begin
    RttyIni^.SetOrAddProfileString('FskPolarity', 'TNC1', YesNo[FskPolarity]);
  end;

  function TTnc.GetFskPolarity : boolean;
  begin
    GetFskPolarity := RttyIni^.GetProfileBool('FskPolarity', 'TNC1', True); {T=normal,f=reverse}
  end;

  procedure TTnc.SetFilterWidth(const FilterWidth : Byte);
  begin
    RttyIni^.SetOrAddProfileString('FilterWidth', 'TNC1', long2str(FilterWidth));
  end;

  function TTnc.GetFilterWidth : byte;
  begin
    GetFilterWidth := RttyIni^.GetProfileByte('FilterWidth', 'TNC1', 0);
  end;

  procedure TTnc.SetDisplay(const Display : Byte);
  begin
    RttyIni^.SetOrAddProfileString('Display', 'TNC1', long2str(Display));
  end;

  function TTnc.GetDisplay : byte;
  begin
    GetDisplay := RttyIni^.GetProfileByte('Display', 'TNC1', 1);
  end;

  procedure TTnc.SetDisplayScale(const DisplayScale : Byte);
  begin
    RttyIni^.SetOrAddProfileString('DisplayScale', 'TNC1', long2str(DisplayScale));
  end;

  function TTnc.GetDisplayScale : byte;
  begin
    GetDisplayScale := RttyIni^.GetProfileByte('DisplayScale', 'TNC1', 100);
  end;

  procedure TTnc.SetMode(const Mode : String);
  begin
    RttyIni^.SetOrAddProfileString('Mode', 'TNC1', Mode);
  end;

  function TTnc.getMode : string;
  begin
    GetMode := StUpCase(RttyIni^.GetProfileString('Mode', 'TNC1', 'Rtty'));
  end;

  procedure TTnc.StopEverything;
  begin
    { no default action in abstract object }
  end;

  procedure TTnc.SetModem(const Modem : Byte);
  begin
    RttyIni^.SetOrAddProfileString('Modem', 'TNC1', long2str(Modem));
  end;

  function TTnc.GetModem : byte;
  begin
    GetModem := RttyIni^.GetProfileByte('Modem', 'TNC1', 1);
  end;

  procedure TTnc.ClearXmitBuffer;
  begin
    { no default action in abstract object }
  end;

  procedure TTnc.ToggleNormRev;
  begin
    SetSense(NOT GetSense); { switch norm/rev }
  end;

  procedure TTnc.SetSense(const Sense : Boolean);
  begin
    RttyIni^.SetOrAddProfileString('Normal', 'TNC1', YesNo[Sense]);
    { no default action in abstract object }
  end;

  function TTnc.GetSense : boolean;
  begin
    GetSense := RttyIni^.GetProfileBool('Normal', 'TNC1', True);
  end;

  procedure TTnc.SetTxSense(const Sense : Boolean);
  begin
    RttyIni^.SetOrAddProfileString('TxNormal', 'TNC1', YesNo[Sense]);
    { no default action in abstract object }
  end;

  function TTnc.GetTxSense : boolean;
  begin
    GetTxSense := RttyIni^.GetProfileBool('TxNormal', 'TNC1', True);
  end;

  procedure TTnc.SetMarkFreq(const MarkFreq : word);
  begin
    RttyIni^.SetOrAddProfileString('MarkFreq', 'TNC1', long2str(MarkFreq));
  end;

  function  TTnc.GetMarkFreq : word;
  begin
    GetMarkFreq := RttyIni^.GetProfileWord('MarkFreq', 'TNC1', 2125);
  end;

  procedure TTnc.SetSpaceFreq(const SpaceFreq : word);
  begin
    RttyIni^.SetOrAddProfileString('SpaceFreq', 'TNC1', long2str(SpaceFreq));
  end;

  function  TTnc.GetSpaceFreq : word;
  begin
    GetSpaceFreq := RttyIni^.GetProfileWord('SpaceFreq', 'TNC1', 2295);
  end;




  procedure TTnc.ToggleChatMode;
  begin
    case KeyBoardLive of
      False : begin
                Transmit(True);
                KeyBoardLive := True;
                SendChar(^M);
                RttyWin^.HilightOn;
              end;
      True : begin
               Transmit(False);
               KeyBoardLive := False;
               RttyWin^.HilightOff;
             end;
    end;
  end;

  procedure TTnc.ResetBox;
  begin
    StopEverything;
    KeyBoardLive := False;
    AutoCq^.Stop;
    CatchCall := False;
  end;

  procedure TTnc.ToggleTncTalk;
  begin
    TncTalk := not TncTalk;
    if TncTalk then RttyWin^.HilightOff; {just to put cusror on screen}
  end;

  procedure TTnc.SetBaudRate(const Rate : Byte); { speed in baud }
  begin
    RttyIni^.SetOrAddProfileString('BaudRate', 'TNC1', long2str(Rate));
    { no default action in abstract object }
  end;

  procedure TTnc.SetWpmRate(const Rate : Byte); { speed in word per minute }
  begin
    Case Rate of
      60 : SetBaudRate(45);
      66 : SetBaudRate(50);
      75 : SetBaudRate(57);
      100: SetBaudRate(74);
    else
      SetBaudRate(Rate * 3 div 4); { kind of goofy, but it should work ... }
    end;
  end;

  function TTnc.GetBaudRate : byte;
  begin
    GetBaudRate := RttyIni^.GetProfileByte('BaudRate', 'TNC1', 45);
  end;

  procedure TTnc.GetPip(const WhichOne : byte; var Display : string; var DispAttr : byte);
  begin
   { no action }
  end;

  procedure TTnc.GetZero(var NumForZero : shortint; var DispAttr : byte);
  begin
   { no action }
  end;

  procedure TTnc.UpdateFilterStrings(var S1, S2 : string; var A1, A2 : Byte);
  begin
    S1 := '';
    S2 := '';
  end;

  procedure TTnc.ClearUP;
  begin
    if not Assigned(UP) then Exit;
    UP^.FlushInBuffer; { flush the input buffer }
    UP^.FlushOutBuffer; { flush the input buffer }
    if UP^.CheckFifoError then begin
      SetFifoBuffering(UP^.PR^.BaseAddr,False,0); { flush the fifo buffer }
      SetFifoBuffering(UP^.PR^.BaseAddr,True,14);
    end;
    UP^.GetLineError;  { resets error indicators }
  end;

end.
