{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$o+}
unit WpxList;
interface

uses OpRoot, WpxNode;
type
  {An object for manipulating the list of Prefixes}
  PWpxList = ^TWpxList;
  TWpxList =
    object(SingleList)
      PrefixPtr : PWpxNode;
      constructor Init;
      function Add(NewPrefix : String) : Boolean;
      function Find(ThisPrefix : String) : Boolean;
      function PrefixOf(ThisCall : String) : String;
      procedure MostCommonPrefix(var CommonPrefix : String;
        var CommonCount : Byte);
      procedure Remove(ThisPrefix : String);
      procedure SkipTo(NodeNum : LongInt);
      procedure Show;
      procedure Display;
      procedure PrintReport;
      destructor Done; virtual;
    end;

implementation

uses
  TpString,
  TpPick,
  TpCrt,
  DebugRtns,
  RecTypes,
  c_Contest,
  cMsgWin,
  cColors,
  LowLvl;

  {---------- PrefixList Methods --------------------------------------}
  constructor TWpxList.Init;
  begin
    inherited Init;
    {$IFDEF DEBUG}
    AppendDebugMessage('WpxList created');
    {$ENDIF}
  end;

  destructor TWpxList.Done;
  begin
    inherited Done;
    {$IFDEF DEBUG}
    AppendDebugMessage('WpxList destroyed');
    {$ENDIF}
  end;

  procedure TWpxList.Show;
  begin
    WriteLn('TWpxList');
    {Get the Head, then traverse the entire List}
    PrefixPtr := PWpxNode(Head);
    while PrefixPtr <> nil do begin
      PrefixPtr^.Show;
      PrefixPtr := PWpxNode(Next(PrefixPtr));
    end;
  end;

  function TWpxList.Find(ThisPrefix : String) : Boolean;
  begin
    Find := False;
    {Get the Head, then traverse the entire List}
    PrefixPtr := PWpxNode(Head);
    while PrefixPtr <> nil do begin
      if PrefixPtr^.GetPrefix = ThisPrefix then begin
        Find := True;
        Exit;
      end;
      if PrefixPtr^.GetPrefix > ThisPrefix then begin
        Find := False;
        Exit;
      end;
      PrefixPtr := PWpxNode(Next(PrefixPtr));
    end;
  end;

  procedure TWpxList.MostCommonPrefix(var CommonPrefix : String;
    var CommonCount : Byte);
  begin
    CommonPrefix := '';
    CommonCount := 0;
    {Get the Head, then traverse the entire List}
    PrefixPtr := PWpxNode(Head);
    while PrefixPtr <> nil do begin
      if PrefixPtr^.GetCount > CommonCount then begin
        CommonCount := PrefixPtr^.GetCount;
        CommonPrefix := PrefixPtr^.GetPrefix;
      end;
      PrefixPtr := PWpxNode(Next(PrefixPtr));
    end;
  end;


  procedure TWpxList.SkipTo(NodeNum : LongInt);
  begin
    PrefixPtr := PWpxNode(Nth(NodeNum));
  end;

  function TWpxList.Add(NewPrefix : String) : Boolean;
  var
    NewPrefixPtr : PWpxNode;
  begin
    if Trim(NewPrefix) = '' then begin
      Add := False;
      Exit;
    end;
    if not Find(NewPrefix) then begin
      {add node}
      Add := True;
      if PrefixPtr = nil then begin
        New(PrefixPtr, Init(NewPrefix)); {make a new list node}
        Append(PrefixPtr);        {put it at the end of the list}
      end
      else begin
        New(NewPrefixPtr, Init(NewPrefix)); {make a new list node}
        PlaceBefore(NewPrefixPtr, PrefixPtr); {put it at the end of the list}
      end;
    end
    else begin
      {node exists}
      Add := False;
      PrefixPtr^.IncCount;
    end;
  end;

  procedure TWpxList.Remove(ThisPrefix : String);
  begin
    if not Find(ThisPrefix) then Exit;
    PrefixPtr^.DecCount;
    if PrefixPtr^.GetCount = 0 then Delete(PrefixPtr)
  end;

  function TWpxList.PrefixOf(ThisCall : String) : String;
    function NoSlashes(CallToTest : String) : Boolean;
    begin
      NoSlashes := Pos('/', CallToTest) = 0;
    end;

    function PosOfSlash(CallToTest : String) : Byte;
    begin
      PosOfSlash := Pos('/', CallToTest);
    end;

    function HasNoNumbers(CallToTest : String) : Boolean;
    var
      CallPos : Byte;
    begin
      HasNoNumbers := True;
      if Length(CallToTest) < 1 then Exit;
      for CallPos := 1 to Length(CallToTest) do
        if CallToTest[CallPos] in ['0'..'9'] then HasNoNumbers := False;
    end;

    function PosOfEndPrefix(CallToTest : String) : Byte;
    var
      CallPos : Byte;
      EndOfCall : Byte;
    begin
      PosOfEndPrefix := 0;
      if PosOfSlash(CallToTest) = 0 then EndOfCall := Length(CallToTest)
      else EndOfCall := PosOfSlash(CallToTest);
      for CallPos := EndOfCall downto 1 do
        if CallToTest[CallPos] in ['0'..'9'] then Break;
      if CallPos > 1 then PosOfEndPrefix := CallPos
      else PosOfEndPrefix := EndOfCall;
    end;

    function AdjustForNumber(CallToTest : String) : String;
    var
      HasNumbers : Boolean;
      CallPos : Byte;
    begin
      HasNumbers := False;
      for CallPos := 1 to Length(CallToTest) do
        if CallToTest[CallPos] in ['0'..'9'] then HasNumbers := True;
      if HasNumbers then AdjustForNumber := CallToTest
      else AdjustForNumber := Copy(CallToTest, 1, 2)+'0';
    end;

    function PortIsJustNumber(CallToTest : String) : Boolean;
    begin
      PortIsJustNumber := False;
      if PosOfSlash(CallToTest) <> Length(CallToTest)-1 then Exit;
      if CallToTest[Length(CallToTest)] in ['0'..'9'] then PortIsJustNumber := True;
    end;

    function InvalidPortDesignator(CallToTest : String) : Boolean;
    var
      SlashPos : Byte;
      PortDesg : String[12];
    begin
      InvalidPortDesignator := False;
      SlashPos := PosOfSlash(CallToTest);
      if SlashPos = 0 then Exit;
      PortDesg := Copy(CallToTest, SlashPos+1, Length(CallToTest)-SlashPos+1);
      if (PortDesg = 'M') or (PortDesg = 'MM') or (PortDesg = 'AE') or
      (PortDesg = 'KT') or (PortDesg = 'AG') or (PortDesg = 'AA') or
      (PortDesg = 'A') or (PortDesg = 'E') or (PortDesg = 'J') or
      (PortDesg = 'P') or (PortDesg = 'QRP') then InvalidPortDesignator := True;
    end;

    function GetPrefix(CallToTest : String) : String;
    var
      PrefixReturned : String[12];
    begin
      PrefixReturned := '';
      if NoSlashes(CallToTest) then begin
        PrefixReturned := AdjustForNumber(CallToTest);
        PrefixReturned := Copy(PrefixReturned, 1, PosOfEndPrefix(PrefixReturned));
      end
      else begin
        if InvalidPortDesignator(CallToTest) then
          PrefixReturned := Copy(CallToTest, 1, PosOfEndPrefix(CallToTest))
        else
          if PortIsJustNumber(CallToTest) then begin
            PrefixReturned := Copy(CallToTest, 1, PosOfEndPrefix(CallToTest));
            PrefixReturned[Length(PrefixReturned)] := CallToTest[Length(CallToTest)];
          end
        else begin
          PrefixReturned := Copy(CallToTest, PosOfSlash(CallToTest)+1, Length(CallToTest)-PosOfSlash(CallToTest));
          if HasNoNumbers(PrefixReturned) then PrefixReturned := PrefixReturned+'0';
        end;
      end;
      GetPrefix := PrefixReturned;
    end;

    procedure AdjustForPrefix(var CallToTest : String);
    begin
      if NoSlashes(CallToTest) then Exit;
      if PosOfSlash(CallToTest) > (Length(CallToTest) div 2) then Exit;
      CallToTest := Copy(CallToTest, PosOfSlash(CallToTest)+1, Length(CallToTest)-PosOfSlash(CallToTest))+
        '/'+
        Copy(CallToTest, 1, PosOfSlash(CallToTest)-1);
    end;

  begin
    if Pos('/MM', ThisCall) > 0 then begin
      PrefixOf := '';
      Exit;
    end;
    if Length(ThisCall) < 3 then
      PrefixOf := ThisCall
    else begin
      AdjustForPrefix(ThisCall);
      PrefixOf := GetPrefix(GetPrefix(ThisCall));
    end;
  end;

  procedure TWpxList.PrintReport;
  var
    PageCt : Byte;
    ColPtr : array[0..9] of PWpxNode;
    Count : LongInt;
    T : String[80];
    PrintFile : Text;

    procedure PrintPage;
    var
      RowCt : LongInt;
      Count : LongInt;
    begin
      WriteLn(PrintFile, 'Prefix Multiplers');
      WriteLn(PrintFile, '');

      for RowCt := 1 to 50 do begin
        T := '';
        for Count := 0 to 9 do begin
          if ColPtr[Count] <> nil then begin
            T := T+pad(ColPtr[Count]^.GetPrefix, 7);
            PrefixPtr := ColPtr[Count];
            PrefixPtr := PWpxNode(Next(PrefixPtr));
            ColPtr[Count] := PrefixPtr;
          end;
        end;
        WriteLn(PrintFile, T);
      end;
      Write(PrintFile, ^L);
    end;

    procedure IncrementPointers;
    var
      Count : Integer;
    begin
      { I assume pagect is actually pagect -1, eg Page 1 = PageCt=0 }
      for Count := 0 to 9 do begin
        SkipTo((PageCt*500)+(Count*50)+1);
        ColPtr[Count] := PrefixPtr;
      end;
    end;

  begin
    Assign(PrintFile, FileName('WPX'));
    {$I-} Rewrite(PrintFile); {$I+}
    MsgWin^.Display('Preparing: '+FileName('WPX'));
    PageCt := 0;
    IncrementPointers;
    repeat
      Inc(PageCt);
      PrintPage;
      IncrementPointers;
    until ColPtr[0] = nil;
    Close(PrintFile);
  end;


  function PrefixAt(PrefixNo : Word) : String; far;
  begin
    If PrefixNo > PrefixList^.Size then
      PrefixAt := ''
    else begin
      PrefixList^.SkipTo(PrefixNo);
      PrefixAt := PrefixList^.PrefixPtr^.GetPrefix;
    end;
  end;

  procedure TWpxList.Display;
  var
    Choice : word;
    HiNum  : Word;
    HOrientType : PickOrientType;
  begin
    HOrientType := PickOrientation;
    Choice := 1;
    HiNum := PrefixList^.Size;
    Pickmatrix := 11;
    PickWindow(@PrefixAt, PrefixList^.Size, 1, 1, ScreenWidth, ScreenHeight,
               True, Colors^.PickColors, 'Prefixes Worked', Choice);

  end;




end.
