{* start-of-license

Copyright (C) 1990-2000 Raymond Ortgiesen

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Required Reading For Developers and Distributors:
Full Text of License   http://www.gnu.org/copyleft/gpl.html
What is Free Software? http://www.gnu.org/philosophy/free-sw.html
What is copyleft?      http://www.gnu.org/copyleft/copyleft.html

Raymond Ortgiesen
35 Colvintown Road
Coventry, RI 02816
USA

end-of-license *}
{$O+,F+}
unit ICOMCIV;

interface
uses
  C_Radio, ApPort;
type
  PIcomCIV = ^TIcomCIV;
  TIcomCIV = object(tRadio)
             Model : string[9];
             CurrentVFO : char;
             PreAmble : string[2];
             TransceiverAddress : Char;
             ControllerAddress : Char;
             Data : string;
             EndOfMessage : Char;
             constructor Init(const ComName : ComNameType; const ThisModel : string);
             destructor  Done; virtual;
             function    IsOnline : boolean; virtual;
             function    GetDisplayFrequency : LongInt; virtual;
             procedure   SetDisplayFrequency(const FreqInHertz : LongInt); virtual;
           end;

implementation

uses
  IniTools,
  IniFile,
  ApMisc,
  ApUart,
  DebugRtns,
  OoCom,
  TpString,
  ApTimer;

  constructor TIcomCIV.Init(const ComName : ComNameType; const ThisModel : String);
  var
    iAddress : string[3];
    i : integer;

  procedure SetDefaultAddress;
  begin
    {list in order by address number}
    TransceiverAddress := #$04; { default is IC-735 }
    if Model = 'IC-735'   then TransceiverAddress := #$04;
    if Model = 'IC-R7000' then TransceiverAddress := #$08;
    if Model = 'IC-275'   then TransceiverAddress := #$10;
    if Model = 'IC-475'   then TransceiverAddress := #$14;
    if Model = 'IC-375'   then TransceiverAddress := #$12;
    if Model = 'IC-575'   then TransceiverAddress := #$16;
    if Model = 'IC-R71'   then TransceiverAddress := #$1A;
    if Model = 'IC-751'   then TransceiverAddress := #$1C;
    if Model = 'IC-761'   then TransceiverAddress := #$1E;
    if Model = 'IC-271'   then TransceiverAddress := #$20;
    if Model = 'IC-471'   then TransceiverAddress := #$22;
    if Model = 'IC-1271'  then TransceiverAddress := #$24;
    if Model = 'IC-781'   then TransceiverAddress := #$26;
    if Model = 'IC-725'   then TransceiverAddress := #$28;
    if Model = 'IC-970'   then TransceiverAddress := #$2E;
    if Model = 'IC-R9000' then TransceiverAddress := #$30;
    if Model = 'IC-726'   then TransceiverAddress := #$30;
    if Model = 'IC-R72'   then TransceiverAddress := #$32;
    if Model = 'IC-R7100' then TransceiverAddress := #$34;
    if Model = 'IC-728'   then TransceiverAddress := #$38;
    if Model = 'IC-729'   then TransceiverAddress := #$3A;
    if Model = 'IC-765'   then TransceiverAddress := #$2C;
    if Model = 'IC-737'   then TransceiverAddress := #$3C;
    if Model = 'IC-736'   then TransceiverAddress := #$40;
    if Model = 'IC-820'   then TransceiverAddress := #$42;
    if Model = 'IC-738'   then TransceiverAddress := #$44;
    if Model = 'IC-775'   then TransceiverAddress := #$46;
    if Model = 'IC-706'   then TransceiverAddress := #$48;
    if Model = 'IC-707'   then TransceiverAddress := #$3E;
    if Model = 'IC-756'   then TransceiverAddress := #$50;
    if Model = 'IC-746'   then TransceiverAddress := #$56;
  end;

  begin
    inherited Init(ComName);
    Model := ThisModel;
    iAddress := RttyIni^.GetProfileString('Address', 'RADIO', '');
    if iAddress = '' then SetDefaultAddress
    else begin
      if (iAddress[3] = 'h') or (iAddress[3] = 'H') then iAddress[3] := ' ';
      if (iAddress[1] = '$') then iAddress[1] := ' ';
      iAddress := Trim(iAddress);
      if Str2Int('$'+iAddress,i) then TransceiverAddress := Char(i)
    end;
    PreAmble := #$FE+#$FE;
    ControllerAddress := #$E0;
    EndOfMessage := #$FD;
    FillChar(Data, SizeOf(Data), cNUL);
    Data[0] := #0;
    CurrentVFO := 'A';
    {$IFDEF DEBUG}
    AppendDebugMessage('IcomCIV created');
    {$ENDIF}
  end;

  destructor TIcomCIV.Done;
  begin
    inherited Done;
    {$IFDEF DEBUG}
    AppendDebugMessage('IcomCIV destroyed');
    {$ENDIF}
  end;

  function TIcomCIV.GetDisplayFrequency : LongInt;
  const
    GetFreqCmd : char = #$03;
  var
    Freq : LongInt;
    FS   : string[10];
    Command : array[1..5] of byte;
    DataPacket : string[11];
    Info : string[11];
    b : byte;
    Start : byte;

    procedure TryIt;
    var
      ET : EventTimer;
      Echo : String[11];
    begin
      NewTimer(ET, 5);
      repeat until TimerExpired(ET) or (UP^.TransReady);
      UP^.PutString(DataPacket);
      UP^.GetStringTimeout(Echo, SizeOf(Echo), [#$FD], 18);
      if Echo = DataPacket then begin
        UP^.GetStringTimeout(Info, SizeOf(Info), [#$FD], 18);
      end
      else
        UP^.FlushInBuffer;
      NewTimer(ET, 2);
      repeat until TimerExpired(ET);
    end;

  begin
    GetDisplayFrequency := 0;
    Freq := 0;
    FillChar(DataPacket, SizeOf(DataPacket), cNUL);
    FillChar(Info, SizeOf(Info), cNUL);
    Info[0] := #0;
    DataPacket := PreAmble+
                  TransceiverAddress+
                  ControllerAddress+
                  GetFreqCmd+
                  EndOfMessage;
    TryIt;
    if Info <> '' then begin
      FS := '';
      if Model = 'IC-735' then Start := 9
                          else Start := 10;
      for b := Start downto 6 do
        FS := FS + hexb(ord(Info[b]));
      if Str2Long(FS,Freq) then GetDisplayFrequency := Freq;
    end;
  end;


  function TIcomCIV.IsOnline : Boolean;
  var
    Online : Boolean;
    ET : EventTimer;
  begin
    {try to read freq 3x, if no freq, then assume not online!}
    Online := GetDisplayFrequency > 0;
    if not Online then begin
      NewTimer(ET, 5);
      repeat until TimerExpired(ET);
      Online := GetDisplayFrequency > 0;
    end;
    if not Online then begin
      NewTimer(ET, 5);
      repeat until TimerExpired(ET);
      Online := GetDisplayFrequency > 0;
    end;
    IsOnline := Online;
  end;

  procedure TIcomCIV.SetDisplayFrequency(const FreqInHertz : LongInt);
  const
    SetFreqCmd : char = #$05;
  var
    Freq : LongInt;
    Command : array[1..5] of byte;
    DataPacket : string[11];

    procedure TryIt;
    var
      ET : EventTimer;
      Echo : String[11];
    begin
      NewTimer(ET, 5);
      repeat until TimerExpired(ET) or (UP^.TransReady);
      UP^.PutString(DataPacket);
      UP^.GetStringTimeout(Echo, SizeOf(Echo), [#$FD], 18);
      if Echo <> DataPacket then UP^.FlushInBuffer;
      NewTimer(ET, 2);
      repeat until TimerExpired(ET);
    end;

    function ToBcd(B : Byte) : Byte;
    begin
      ToBcd := ((B div 10) shl 4) + (b mod 10);
    end;


  begin
    Freq := FreqInHertz;
    {1's of ghz and 100's of mhz}
    Command[5] := ToBcd(Freq div 100000000);
    Freq := Freq mod 100000000;

    {10's and 1's of mhz}
    Command[4] := ToBcd(Freq div 1000000);
    Freq := Freq mod 1000000;

    {100's and 10's of khz}
    Command[3] := ToBcd(Freq div 10000);
    Freq := Freq mod 10000;

    {1's of khz and 100's of khz}
    Command[2] := ToBcd(Freq div 100);
    Freq := Freq mod 100;

    {10's of hz and 1's of hz}
    Command[1] := ToBcd(Freq div 1);

    FillChar(DataPacket, SizeOf(DataPacket), cNUL);
    if Model = 'IC-735' then
       DataPacket := PreAmble+TransceiverAddress+ControllerAddress+SetFreqCmd+
         Char(Command[1])+Char(Command[2])+Char(Command[3])+Char(Command[4])+
         EndOfMessage
    else
       DataPacket := PreAmble+TransceiverAddress+ControllerAddress+SetFreqCmd+
         Char(Command[1])+Char(Command[2])+Char(Command[3])+Char(Command[4])+Char(Command[5])+
         EndOfMessage;

    TryIt;
  end;



  { the icoms do not have a rit clear function, this was determined after
    much programming effort and 2 emails to icom tech support }


end.
